/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
/*****************************************************************************/
/*  LibreDWG - free implementation of the DWG file format                    */
/*                                                                           */
/*  Copyright (C) 2019-2025 Free Software Foundation, Inc.                   */
/*                                                                           */
/*  This library is free software, licensed under the terms of the GNU       */
/*  General Public License as published by the Free Software Foundation,     */
/*  either version 3 of the License, or (at your option) any later version.  */
/*  You should have received a copy of the GNU General Public License        */
/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
/*****************************************************************************/

/*
 * objects.inc: define all object and entities
 * written by Reini Urban
 * generated by src/gen-dynapi.pl from include/dwg.h, do not modify.
 */

DWG_ENTITY (_3DFACE)
DWG_ENTITY (_3DLINE)
DWG_ENTITY (_3DSOLID)
DWG_ENTITY (ALIGNMENTPARAMETERENTITY)
DWG_ENTITY (ARC)
DWG_ENTITY (ARCALIGNEDTEXT)
DWG_ENTITY (ARC_DIMENSION)
DWG_ENTITY (ATTDEF)
DWG_ENTITY (ATTRIB)
DWG_ENTITY (BASEPOINTPARAMETERENTITY)
DWG_ENTITY (BLOCK)
DWG_ENTITY (BODY)
DWG_ENTITY (CAMERA)
DWG_ENTITY (CIRCLE)
DWG_ENTITY (DGNUNDERLAY)
DWG_ENTITY (DIMENSION_ALIGNED)
DWG_ENTITY (DIMENSION_ANG2LN)
DWG_ENTITY (DIMENSION_ANG3PT)
DWG_ENTITY (DIMENSION_DIAMETER)
DWG_ENTITY (DIMENSION_LINEAR)
DWG_ENTITY (DIMENSION_ORDINATE)
DWG_ENTITY (DIMENSION_RADIUS)
DWG_ENTITY (DWFUNDERLAY)
DWG_ENTITY (ELLIPSE)
DWG_ENTITY (ENDBLK)
DWG_ENTITY (ENDREP)
DWG_ENTITY (EXTRUDEDSURFACE)
DWG_ENTITY (FLIPGRIPENTITY)
DWG_ENTITY (FLIPPARAMETERENTITY)
DWG_ENTITY (GEOPOSITIONMARKER)
DWG_ENTITY (HATCH)
DWG_ENTITY (HELIX)
DWG_ENTITY (IMAGE)
DWG_ENTITY (INSERT)
DWG_ENTITY (JUMP)
DWG_ENTITY (LARGE_RADIAL_DIMENSION)
DWG_ENTITY (LAYOUTPRINTCONFIG)
DWG_ENTITY (LEADER)
DWG_ENTITY (LIGHT)
DWG_ENTITY (LINE)
DWG_ENTITY (LINEARGRIPENTITY)
DWG_ENTITY (LINEARPARAMETERENTITY)
DWG_ENTITY (LOAD)
DWG_ENTITY (LOFTEDSURFACE)
DWG_ENTITY (LWPOLYLINE)
DWG_ENTITY (MESH)
DWG_ENTITY (MINSERT)
DWG_ENTITY (MLINE)
DWG_ENTITY (MPOLYGON)
DWG_ENTITY (MTEXT)
DWG_ENTITY (MULTILEADER)
DWG_ENTITY (NAVISWORKSMODEL)
DWG_ENTITY (NURBSURFACE)
DWG_ENTITY (OLE2FRAME)
DWG_ENTITY (OLEFRAME)
DWG_ENTITY (PDFUNDERLAY)
DWG_ENTITY (PLANESURFACE)
DWG_ENTITY (POINT)
DWG_ENTITY (POINTCLOUD)
DWG_ENTITY (POINTCLOUDEX)
DWG_ENTITY (POINTPARAMETERENTITY)
DWG_ENTITY (POLARGRIPENTITY)
DWG_ENTITY (POLYLINE_2D)
DWG_ENTITY (POLYLINE_3D)
DWG_ENTITY (POLYLINE_MESH)
DWG_ENTITY (POLYLINE_PFACE)
DWG_ENTITY (PROXY_ENTITY)
DWG_ENTITY (RAY)
DWG_ENTITY (REGION)
DWG_ENTITY (REPEAT)
DWG_ENTITY (REVOLVEDSURFACE)
DWG_ENTITY (ROTATIONGRIPENTITY)
DWG_ENTITY (ROTATIONPARAMETERENTITY)
DWG_ENTITY (RTEXT)
DWG_ENTITY (SECTIONOBJECT)
DWG_ENTITY (SEQEND)
DWG_ENTITY (SHAPE)
DWG_ENTITY (SOLID)
DWG_ENTITY (SPLINE)
DWG_ENTITY (SWEPTSURFACE)
DWG_ENTITY (TABLE)
DWG_ENTITY (TEXT)
DWG_ENTITY (TOLERANCE)
DWG_ENTITY (TRACE)
DWG_ENTITY (UNKNOWN_ENT)
DWG_ENTITY (VERTEX_2D)
DWG_ENTITY (VERTEX_3D)
DWG_ENTITY (VERTEX_MESH)
DWG_ENTITY (VERTEX_PFACE)
DWG_ENTITY (VERTEX_PFACE_FACE)
DWG_ENTITY (VIEWPORT)
DWG_ENTITY (VISIBILITYGRIPENTITY)
DWG_ENTITY (VISIBILITYPARAMETERENTITY)
DWG_ENTITY (WIPEOUT)
DWG_ENTITY (XLINE)
DWG_ENTITY (XYGRIPENTITY)
DWG_ENTITY (XYPARAMETERENTITY)

DWG_OBJECT (ACMECOMMANDHISTORY)
DWG_OBJECT (ACMESCOPE)
DWG_OBJECT (ACMESTATEMGR)
DWG_OBJECT (ACSH_BOOLEAN_CLASS)
DWG_OBJECT (ACSH_BOX_CLASS)
DWG_OBJECT (ACSH_BREP_CLASS)
DWG_OBJECT (ACSH_CHAMFER_CLASS)
DWG_OBJECT (ACSH_CONE_CLASS)
DWG_OBJECT (ACSH_CYLINDER_CLASS)
DWG_OBJECT (ACSH_EXTRUSION_CLASS)
DWG_OBJECT (ACSH_FILLET_CLASS)
DWG_OBJECT (ACSH_HISTORY_CLASS)
DWG_OBJECT (ACSH_LOFT_CLASS)
DWG_OBJECT (ACSH_PYRAMID_CLASS)
DWG_OBJECT (ACSH_REVOLVE_CLASS)
DWG_OBJECT (ACSH_SPHERE_CLASS)
DWG_OBJECT (ACSH_SWEEP_CLASS)
DWG_OBJECT (ACSH_TORUS_CLASS)
DWG_OBJECT (ACSH_WEDGE_CLASS)
DWG_OBJECT (ALDIMOBJECTCONTEXTDATA)
DWG_OBJECT (ANGDIMOBJECTCONTEXTDATA)
DWG_OBJECT (ANNOTSCALEOBJECTCONTEXTDATA)
DWG_OBJECT (APPID)
DWG_OBJECT (APPID_CONTROL)
DWG_OBJECT (ASSOC2DCONSTRAINTGROUP)
DWG_OBJECT (ASSOC3POINTANGULARDIMACTIONBODY)
DWG_OBJECT (ASSOCACTION)
DWG_OBJECT (ASSOCACTIONPARAM)
DWG_OBJECT (ASSOCALIGNEDDIMACTIONBODY)
DWG_OBJECT (ASSOCARRAYACTIONBODY)
DWG_OBJECT (ASSOCARRAYMODIFYACTIONBODY)
DWG_OBJECT (ASSOCASMBODYACTIONPARAM)
DWG_OBJECT (ASSOCBLENDSURFACEACTIONBODY)
DWG_OBJECT (ASSOCCOMPOUNDACTIONPARAM)
DWG_OBJECT (ASSOCDEPENDENCY)
DWG_OBJECT (ASSOCDIMDEPENDENCYBODY)
DWG_OBJECT (ASSOCEDGEACTIONPARAM)
DWG_OBJECT (ASSOCEDGECHAMFERACTIONBODY)
DWG_OBJECT (ASSOCEDGEFILLETACTIONBODY)
DWG_OBJECT (ASSOCEXTENDSURFACEACTIONBODY)
DWG_OBJECT (ASSOCEXTRUDEDSURFACEACTIONBODY)
DWG_OBJECT (ASSOCFACEACTIONPARAM)
DWG_OBJECT (ASSOCFILLETSURFACEACTIONBODY)
DWG_OBJECT (ASSOCGEOMDEPENDENCY)
DWG_OBJECT (ASSOCLOFTEDSURFACEACTIONBODY)
DWG_OBJECT (ASSOCMLEADERACTIONBODY)
DWG_OBJECT (ASSOCNETWORK)
DWG_OBJECT (ASSOCNETWORKSURFACEACTIONBODY)
DWG_OBJECT (ASSOCOBJECTACTIONPARAM)
DWG_OBJECT (ASSOCOFFSETSURFACEACTIONBODY)
DWG_OBJECT (ASSOCORDINATEDIMACTIONBODY)
DWG_OBJECT (ASSOCOSNAPPOINTREFACTIONPARAM)
DWG_OBJECT (ASSOCPATCHSURFACEACTIONBODY)
DWG_OBJECT (ASSOCPATHACTIONPARAM)
DWG_OBJECT (ASSOCPERSSUBENTMANAGER)
DWG_OBJECT (ASSOCPLANESURFACEACTIONBODY)
DWG_OBJECT (ASSOCPOINTREFACTIONPARAM)
DWG_OBJECT (ASSOCRESTOREENTITYSTATEACTIONBODY)
DWG_OBJECT (ASSOCREVOLVEDSURFACEACTIONBODY)
DWG_OBJECT (ASSOCROTATEDDIMACTIONBODY)
DWG_OBJECT (ASSOCSWEPTSURFACEACTIONBODY)
DWG_OBJECT (ASSOCTRIMSURFACEACTIONBODY)
DWG_OBJECT (ASSOCVALUEDEPENDENCY)
DWG_OBJECT (ASSOCVARIABLE)
DWG_OBJECT (ASSOCVERTEXACTIONPARAM)
DWG_OBJECT (BLKREFOBJECTCONTEXTDATA)
DWG_OBJECT (BLOCKALIGNEDCONSTRAINTPARAMETER)
DWG_OBJECT (BLOCKALIGNMENTGRIP)
DWG_OBJECT (BLOCKALIGNMENTPARAMETER)
DWG_OBJECT (BLOCKANGULARCONSTRAINTPARAMETER)
DWG_OBJECT (BLOCKARRAYACTION)
DWG_OBJECT (BLOCKBASEPOINTPARAMETER)
DWG_OBJECT (BLOCKDIAMETRICCONSTRAINTPARAMETER)
DWG_OBJECT (BLOCKFLIPACTION)
DWG_OBJECT (BLOCKFLIPGRIP)
DWG_OBJECT (BLOCKFLIPPARAMETER)
DWG_OBJECT (BLOCKGRIPLOCATIONCOMPONENT)
DWG_OBJECT (BLOCKHORIZONTALCONSTRAINTPARAMETER)
DWG_OBJECT (BLOCKLINEARCONSTRAINTPARAMETER)
DWG_OBJECT (BLOCKLINEARGRIP)
DWG_OBJECT (BLOCKLINEARPARAMETER)
DWG_OBJECT (BLOCKLOOKUPACTION)
DWG_OBJECT (BLOCKLOOKUPGRIP)
DWG_OBJECT (BLOCKLOOKUPPARAMETER)
DWG_OBJECT (BLOCKMOVEACTION)
DWG_OBJECT (BLOCKPARAMDEPENDENCYBODY)
DWG_OBJECT (BLOCKPOINTPARAMETER)
DWG_OBJECT (BLOCKPOLARGRIP)
DWG_OBJECT (BLOCKPOLARPARAMETER)
DWG_OBJECT (BLOCKPOLARSTRETCHACTION)
DWG_OBJECT (BLOCKPROPERTIESTABLE)
DWG_OBJECT (BLOCKPROPERTIESTABLEGRIP)
DWG_OBJECT (BLOCKRADIALCONSTRAINTPARAMETER)
DWG_OBJECT (BLOCKREPRESENTATION)
DWG_OBJECT (BLOCKROTATEACTION)
DWG_OBJECT (BLOCKROTATIONGRIP)
DWG_OBJECT (BLOCKROTATIONPARAMETER)
DWG_OBJECT (BLOCKSCALEACTION)
DWG_OBJECT (BLOCKSTRETCHACTION)
DWG_OBJECT (BLOCKUSERPARAMETER)
DWG_OBJECT (BLOCKVERTICALCONSTRAINTPARAMETER)
DWG_OBJECT (BLOCKVISIBILITYGRIP)
DWG_OBJECT (BLOCKVISIBILITYPARAMETER)
DWG_OBJECT (BLOCKXYGRIP)
DWG_OBJECT (BLOCKXYPARAMETER)
DWG_OBJECT (BLOCK_CONTROL)
DWG_OBJECT (BLOCK_HEADER)
DWG_OBJECT (BREAKDATA)
DWG_OBJECT (BREAKPOINTREF)
DWG_OBJECT (CELLSTYLEMAP)
DWG_OBJECT (CONTEXTDATAMANAGER)
DWG_OBJECT (CSACDOCUMENTOPTIONS)
DWG_OBJECT (CURVEPATH)
DWG_OBJECT (DATALINK)
DWG_OBJECT (DATATABLE)
DWG_OBJECT (DBCOLOR)
DWG_OBJECT (DETAILVIEWSTYLE)
DWG_OBJECT (DICTIONARY)
DWG_OBJECT (DICTIONARYVAR)
DWG_OBJECT (DICTIONARYWDFLT)
DWG_OBJECT (DIMASSOC)
DWG_OBJECT (DIMSTYLE)
DWG_OBJECT (DIMSTYLE_CONTROL)
DWG_OBJECT (DMDIMOBJECTCONTEXTDATA)
DWG_OBJECT (DUMMY)
DWG_OBJECT (DYNAMICBLOCKPROXYNODE)
DWG_OBJECT (DYNAMICBLOCKPURGEPREVENTER)
DWG_OBJECT (EVALUATION_GRAPH)
DWG_OBJECT (FCFOBJECTCONTEXTDATA)
DWG_OBJECT (FIELD)
DWG_OBJECT (FIELDLIST)
DWG_OBJECT (GEODATA)
DWG_OBJECT (GEOMAPIMAGE)
DWG_OBJECT (GRADIENT_BACKGROUND)
DWG_OBJECT (GROUND_PLANE_BACKGROUND)
DWG_OBJECT (GROUP)
DWG_OBJECT (IBL_BACKGROUND)
DWG_OBJECT (IDBUFFER)
DWG_OBJECT (IMAGEDEF)
DWG_OBJECT (IMAGEDEF_REACTOR)
DWG_OBJECT (IMAGE_BACKGROUND)
DWG_OBJECT (INDEX)
DWG_OBJECT (LAYER)
DWG_OBJECT (LAYERFILTER)
DWG_OBJECT (LAYER_CONTROL)
DWG_OBJECT (LAYER_INDEX)
DWG_OBJECT (LAYOUT)
DWG_OBJECT (LEADEROBJECTCONTEXTDATA)
DWG_OBJECT (LIGHTLIST)
DWG_OBJECT (LONG_TRANSACTION)
DWG_OBJECT (LTYPE)
DWG_OBJECT (LTYPE_CONTROL)
DWG_OBJECT (MATERIAL)
DWG_OBJECT (MENTALRAYRENDERSETTINGS)
DWG_OBJECT (MLEADEROBJECTCONTEXTDATA)
DWG_OBJECT (MLEADERSTYLE)
DWG_OBJECT (MLINESTYLE)
DWG_OBJECT (MOTIONPATH)
DWG_OBJECT (MTEXTATTRIBUTEOBJECTCONTEXTDATA)
DWG_OBJECT (MTEXTOBJECTCONTEXTDATA)
DWG_OBJECT (NAVISWORKSMODELDEF)
DWG_OBJECT (OBJECT_PTR)
DWG_OBJECT (ORDDIMOBJECTCONTEXTDATA)
DWG_OBJECT (PARTIAL_VIEWING_INDEX)
DWG_OBJECT (PERSUBENTMGR)
DWG_OBJECT (PLACEHOLDER)
DWG_OBJECT (PLOTSETTINGS)
DWG_OBJECT (POINTCLOUDCOLORMAP)
DWG_OBJECT (POINTCLOUDDEF)
DWG_OBJECT (POINTCLOUDDEFEX)
DWG_OBJECT (POINTCLOUDDEF_REACTOR)
DWG_OBJECT (POINTCLOUDDEF_REACTOR_EX)
DWG_OBJECT (POINTPATH)
DWG_OBJECT (PROXY_OBJECT)
DWG_OBJECT (RADIMLGOBJECTCONTEXTDATA)
DWG_OBJECT (RADIMOBJECTCONTEXTDATA)
DWG_OBJECT (RAPIDRTRENDERSETTINGS)
DWG_OBJECT (RASTERVARIABLES)
DWG_OBJECT (RENDERENTRY)
DWG_OBJECT (RENDERENVIRONMENT)
DWG_OBJECT (RENDERGLOBAL)
DWG_OBJECT (RENDERSETTINGS)
DWG_OBJECT (SCALE)
DWG_OBJECT (SECTIONVIEWSTYLE)
DWG_OBJECT (SECTION_MANAGER)
DWG_OBJECT (SECTION_SETTINGS)
DWG_OBJECT (SKYLIGHT_BACKGROUND)
DWG_OBJECT (SOLID_BACKGROUND)
DWG_OBJECT (SORTENTSTABLE)
DWG_OBJECT (SPATIAL_FILTER)
DWG_OBJECT (SPATIAL_INDEX)
DWG_OBJECT (STYLE)
DWG_OBJECT (STYLE_CONTROL)
DWG_OBJECT (SUN)
DWG_OBJECT (SUNSTUDY)
DWG_OBJECT (TABLECONTENT)
DWG_OBJECT (TABLEGEOMETRY)
DWG_OBJECT (TABLESTYLE)
DWG_OBJECT (TEXTOBJECTCONTEXTDATA)
DWG_OBJECT (TVDEVICEPROPERTIES)
DWG_OBJECT (UCS)
DWG_OBJECT (UCS_CONTROL)
DWG_OBJECT (UNKNOWN_OBJ)
DWG_OBJECT (VBA_PROJECT)
DWG_OBJECT (VIEW)
DWG_OBJECT (VIEW_CONTROL)
DWG_OBJECT (VISUALSTYLE)
DWG_OBJECT (VPORT)
DWG_OBJECT (VPORT_CONTROL)
DWG_OBJECT (VX_CONTROL)
DWG_OBJECT (VX_TABLE_RECORD)
DWG_OBJECT (WIPEOUTVARIABLES)
DWG_OBJECT (XRECORD)
DWG_OBJECT (PDFDEFINITION)
DWG_OBJECT (DGNDEFINITION)
DWG_OBJECT (DWFDEFINITION)
DWG_OBJECT (ASSOCARRAYMODIFYPARAMETERS)
DWG_OBJECT (ASSOCARRAYPATHPARAMETERS)
DWG_OBJECT (ASSOCARRAYPOLARPARAMETERS)
DWG_OBJECT (ASSOCARRAYRECTANGULARPARAMETERS)
