/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
/* generated by make regen-unknown, do not modify */

unknown_dxf[0].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_0;
unknown_dxf[1].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_1;
unknown_dxf[2].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_2;
unknown_dxf[3].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_3;
unknown_dxf[4].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_4;
unknown_dxf[5].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_5;
unknown_dxf[6].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_6;
unknown_dxf[7].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_7;
unknown_dxf[8].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_8;
unknown_dxf[9].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_9;
unknown_dxf[10].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_10;
unknown_dxf[11].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_11;
unknown_dxf[12].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_12;
unknown_dxf[13].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_13;
unknown_dxf[14].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_14;
unknown_dxf[15].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_15;
unknown_dxf[16].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_16;
unknown_dxf[17].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_17;
unknown_dxf[18].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_18;
unknown_dxf[19].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_19;
unknown_dxf[20].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_20;
unknown_dxf[21].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_21;
unknown_dxf[22].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_22;
unknown_dxf[23].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_23;
unknown_dxf[24].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_24;
unknown_dxf[25].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_25;
unknown_dxf[26].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_26;
unknown_dxf[27].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_27;
unknown_dxf[28].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_28;
unknown_dxf[29].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_29;
unknown_dxf[30].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_30;
unknown_dxf[31].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_31;
unknown_dxf[32].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_32;
unknown_dxf[33].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_33;
unknown_dxf[34].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_34;
unknown_dxf[35].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_35;
unknown_dxf[36].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_36;
unknown_dxf[37].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_37;
unknown_dxf[38].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_38;
unknown_dxf[39].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_39;
unknown_dxf[40].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_40;
unknown_dxf[41].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_41;
unknown_dxf[42].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_42;
unknown_dxf[43].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_43;
unknown_dxf[44].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_44;
unknown_dxf[45].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_45;
unknown_dxf[46].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_46;
unknown_dxf[47].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_47;
unknown_dxf[48].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_48;
unknown_dxf[49].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_49;
unknown_dxf[50].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_50;
unknown_dxf[51].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_51;
unknown_dxf[52].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_52;
unknown_dxf[53].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_53;
unknown_dxf[54].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_54;
unknown_dxf[55].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_55;
unknown_dxf[56].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_56;
unknown_dxf[57].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_57;
unknown_dxf[58].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_58;
unknown_dxf[59].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_59;
unknown_dxf[60].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_60;
unknown_dxf[61].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_61;
unknown_dxf[62].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_62;
unknown_dxf[63].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_63;
unknown_dxf[64].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_64;
unknown_dxf[65].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_65;
unknown_dxf[66].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_66;
unknown_dxf[67].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_67;
unknown_dxf[68].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_68;
unknown_dxf[69].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_69;
unknown_dxf[70].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_70;
unknown_dxf[71].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_71;
unknown_dxf[72].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_72;
unknown_dxf[73].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_73;
unknown_dxf[74].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_74;
unknown_dxf[75].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_75;
unknown_dxf[76].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_76;
unknown_dxf[77].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_77;
unknown_dxf[78].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_78;
unknown_dxf[79].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_79;
unknown_dxf[80].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_80;
unknown_dxf[81].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_81;
unknown_dxf[82].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_82;
unknown_dxf[83].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_83;
unknown_dxf[84].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_84;
unknown_dxf[85].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_85;
unknown_dxf[86].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_86;
unknown_dxf[87].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_87;
unknown_dxf[88].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_88;
unknown_dxf[89].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_89;
unknown_dxf[90].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_90;
unknown_dxf[91].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_91;
unknown_dxf[92].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_92;
unknown_dxf[93].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_93;
unknown_dxf[94].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_94;
unknown_dxf[95].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_95;
unknown_dxf[96].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_96;
unknown_dxf[97].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_97;
unknown_dxf[98].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_98;
unknown_dxf[99].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_99;
unknown_dxf[100].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_100;
unknown_dxf[101].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_101;
unknown_dxf[102].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_102;
unknown_dxf[103].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_103;
unknown_dxf[104].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_104;
unknown_dxf[105].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_105;
unknown_dxf[106].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_106;
unknown_dxf[107].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_107;
unknown_dxf[108].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_108;
unknown_dxf[109].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_109;
unknown_dxf[110].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_110;
unknown_dxf[111].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_111;
unknown_dxf[112].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_112;
unknown_dxf[113].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_113;
unknown_dxf[114].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_114;
unknown_dxf[115].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_115;
unknown_dxf[116].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_116;
unknown_dxf[117].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_117;
unknown_dxf[118].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_118;
unknown_dxf[119].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_119;
unknown_dxf[120].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_120;
unknown_dxf[121].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_121;
unknown_dxf[122].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_122;
unknown_dxf[123].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_123;
unknown_dxf[124].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_124;
unknown_dxf[125].fields = unknown_dxf_ACAD_EVALUATION_GRAPH_125;
unknown_dxf[126].fields = unknown_dxf_ACAD_TABLE_126;
unknown_dxf[127].fields = unknown_dxf_ACAD_TABLE_127;
unknown_dxf[128].fields = unknown_dxf_ACAD_TABLE_128;
unknown_dxf[129].fields = unknown_dxf_ACAD_TABLE_129;
unknown_dxf[130].fields = unknown_dxf_ACAD_TABLE_130;
unknown_dxf[131].fields = unknown_dxf_ACAD_TABLE_131;
unknown_dxf[132].fields = unknown_dxf_ACAD_TABLE_132;
unknown_dxf[133].fields = unknown_dxf_ACAD_TABLE_133;
unknown_dxf[134].fields = unknown_dxf_ACAD_TABLE_134;
unknown_dxf[135].fields = unknown_dxf_ACAD_TABLE_135;
unknown_dxf[136].fields = unknown_dxf_ACAD_TABLE_136;
unknown_dxf[137].fields = unknown_dxf_ACAD_TABLE_137;
unknown_dxf[138].fields = unknown_dxf_ACAD_TABLE_138;
unknown_dxf[139].fields = unknown_dxf_ACAD_TABLE_139;
unknown_dxf[140].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_140;
unknown_dxf[141].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_141;
unknown_dxf[142].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_142;
unknown_dxf[143].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_143;
unknown_dxf[144].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_144;
unknown_dxf[145].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_145;
unknown_dxf[146].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_146;
unknown_dxf[147].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_147;
unknown_dxf[148].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_148;
unknown_dxf[149].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_149;
unknown_dxf[150].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_150;
unknown_dxf[151].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_151;
unknown_dxf[152].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_152;
unknown_dxf[153].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_153;
unknown_dxf[154].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_154;
unknown_dxf[155].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_155;
unknown_dxf[156].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_156;
unknown_dxf[157].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_157;
unknown_dxf[158].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_158;
unknown_dxf[159].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_159;
unknown_dxf[160].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_160;
unknown_dxf[161].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_161;
unknown_dxf[162].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_162;
unknown_dxf[163].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_163;
unknown_dxf[164].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_164;
unknown_dxf[165].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_165;
unknown_dxf[166].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_166;
unknown_dxf[167].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_167;
unknown_dxf[168].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_168;
unknown_dxf[169].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_169;
unknown_dxf[170].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_170;
unknown_dxf[171].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_171;
unknown_dxf[172].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_172;
unknown_dxf[173].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_173;
unknown_dxf[174].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_174;
unknown_dxf[175].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_175;
unknown_dxf[176].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_176;
unknown_dxf[177].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_177;
unknown_dxf[178].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_178;
unknown_dxf[179].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_179;
unknown_dxf[180].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_180;
unknown_dxf[181].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_181;
unknown_dxf[182].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_182;
unknown_dxf[183].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_183;
unknown_dxf[184].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_184;
unknown_dxf[185].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_185;
unknown_dxf[186].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_186;
unknown_dxf[187].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_187;
unknown_dxf[188].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_188;
unknown_dxf[189].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_189;
unknown_dxf[190].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_190;
unknown_dxf[191].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_191;
unknown_dxf[192].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_192;
unknown_dxf[193].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_193;
unknown_dxf[194].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_194;
unknown_dxf[195].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_195;
unknown_dxf[196].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_196;
unknown_dxf[197].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_197;
unknown_dxf[198].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_198;
unknown_dxf[199].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_199;
unknown_dxf[200].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_200;
unknown_dxf[201].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_201;
unknown_dxf[202].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_202;
unknown_dxf[203].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_203;
unknown_dxf[204].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_204;
unknown_dxf[205].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_205;
unknown_dxf[206].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_206;
unknown_dxf[207].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_207;
unknown_dxf[208].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_208;
unknown_dxf[209].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_209;
unknown_dxf[210].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_210;
unknown_dxf[211].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_211;
unknown_dxf[212].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_212;
unknown_dxf[213].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_213;
unknown_dxf[214].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_214;
unknown_dxf[215].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_215;
unknown_dxf[216].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_216;
unknown_dxf[217].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_217;
unknown_dxf[218].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_218;
unknown_dxf[219].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_219;
unknown_dxf[220].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_220;
unknown_dxf[221].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_221;
unknown_dxf[222].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_222;
unknown_dxf[223].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_223;
unknown_dxf[224].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_224;
unknown_dxf[225].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_225;
unknown_dxf[226].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_226;
unknown_dxf[227].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_227;
unknown_dxf[228].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_228;
unknown_dxf[229].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_229;
unknown_dxf[230].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_230;
unknown_dxf[231].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_231;
unknown_dxf[232].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_232;
unknown_dxf[233].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_233;
unknown_dxf[234].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_234;
unknown_dxf[235].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_235;
unknown_dxf[236].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_236;
unknown_dxf[237].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_237;
unknown_dxf[238].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_238;
unknown_dxf[239].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_239;
unknown_dxf[240].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_240;
unknown_dxf[241].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_241;
unknown_dxf[242].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_242;
unknown_dxf[243].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_243;
unknown_dxf[244].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_244;
unknown_dxf[245].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_245;
unknown_dxf[246].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_246;
unknown_dxf[247].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_247;
unknown_dxf[248].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_248;
unknown_dxf[249].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_249;
unknown_dxf[250].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_250;
unknown_dxf[251].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_251;
unknown_dxf[252].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_252;
unknown_dxf[253].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_253;
unknown_dxf[254].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_254;
unknown_dxf[255].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_255;
unknown_dxf[256].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_256;
unknown_dxf[257].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_257;
unknown_dxf[258].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_258;
unknown_dxf[259].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_259;
unknown_dxf[260].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_260;
unknown_dxf[261].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_261;
unknown_dxf[262].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_262;
unknown_dxf[263].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_263;
unknown_dxf[264].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_264;
unknown_dxf[265].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_265;
unknown_dxf[266].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_266;
unknown_dxf[267].fields = unknown_dxf_ACDB_ALDIMOBJECTCONTEXTDATA_CLASS_267;
unknown_dxf[268].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_268;
unknown_dxf[269].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_269;
unknown_dxf[270].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_270;
unknown_dxf[271].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_271;
unknown_dxf[272].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_272;
unknown_dxf[273].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_273;
unknown_dxf[274].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_274;
unknown_dxf[275].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_275;
unknown_dxf[276].fields = unknown_dxf_ACDBASSOC2DCONSTRAINTGROUP_276;
unknown_dxf[277].fields = unknown_dxf_ACDBASSOCALIGNEDDIMACTIONBODY_277;
unknown_dxf[278].fields = unknown_dxf_ACDBASSOCALIGNEDDIMACTIONBODY_278;
unknown_dxf[279].fields = unknown_dxf_ACDBASSOCALIGNEDDIMACTIONBODY_279;
unknown_dxf[280].fields = unknown_dxf_ACDBASSOCALIGNEDDIMACTIONBODY_280;
unknown_dxf[281].fields = unknown_dxf_ACDBASSOCARRAYACTIONBODY_281;
unknown_dxf[282].fields = unknown_dxf_ACDBASSOCARRAYACTIONBODY_282;
unknown_dxf[283].fields = unknown_dxf_ACDBASSOCDEPENDENCY_283;
unknown_dxf[284].fields = unknown_dxf_ACDBASSOCDEPENDENCY_284;
unknown_dxf[285].fields = unknown_dxf_ACDBASSOCDEPENDENCY_285;
unknown_dxf[286].fields = unknown_dxf_ACDBASSOCDEPENDENCY_286;
unknown_dxf[287].fields = unknown_dxf_ACDBASSOCDEPENDENCY_287;
unknown_dxf[288].fields = unknown_dxf_ACDBASSOCDEPENDENCY_288;
unknown_dxf[289].fields = unknown_dxf_ACDBASSOCDEPENDENCY_289;
unknown_dxf[290].fields = unknown_dxf_ACDBASSOCDEPENDENCY_290;
unknown_dxf[291].fields = unknown_dxf_ACDBASSOCDEPENDENCY_291;
unknown_dxf[292].fields = unknown_dxf_ACDBASSOCDEPENDENCY_292;
unknown_dxf[293].fields = unknown_dxf_ACDBASSOCDEPENDENCY_293;
unknown_dxf[294].fields = unknown_dxf_ACDBASSOCDEPENDENCY_294;
unknown_dxf[295].fields = unknown_dxf_ACDBASSOCDEPENDENCY_295;
unknown_dxf[296].fields = unknown_dxf_ACDBASSOCDEPENDENCY_296;
unknown_dxf[297].fields = unknown_dxf_ACDBASSOCDEPENDENCY_297;
unknown_dxf[298].fields = unknown_dxf_ACDBASSOCDEPENDENCY_298;
unknown_dxf[299].fields = unknown_dxf_ACDBASSOCDEPENDENCY_299;
unknown_dxf[300].fields = unknown_dxf_ACDBASSOCDEPENDENCY_300;
unknown_dxf[301].fields = unknown_dxf_ACDBASSOCDEPENDENCY_301;
unknown_dxf[302].fields = unknown_dxf_ACDBASSOCDEPENDENCY_302;
unknown_dxf[303].fields = unknown_dxf_ACDBASSOCDEPENDENCY_303;
unknown_dxf[304].fields = unknown_dxf_ACDBASSOCDEPENDENCY_304;
unknown_dxf[305].fields = unknown_dxf_ACDBASSOCDEPENDENCY_305;
unknown_dxf[306].fields = unknown_dxf_ACDBASSOCDEPENDENCY_306;
unknown_dxf[307].fields = unknown_dxf_ACDBASSOCDEPENDENCY_307;
unknown_dxf[308].fields = unknown_dxf_ACDBASSOCDEPENDENCY_308;
unknown_dxf[309].fields = unknown_dxf_ACDBASSOCDEPENDENCY_309;
unknown_dxf[310].fields = unknown_dxf_ACDBASSOCDEPENDENCY_310;
unknown_dxf[311].fields = unknown_dxf_ACDBASSOCDEPENDENCY_311;
unknown_dxf[312].fields = unknown_dxf_ACDBASSOCDEPENDENCY_312;
unknown_dxf[313].fields = unknown_dxf_ACDBASSOCDEPENDENCY_313;
unknown_dxf[314].fields = unknown_dxf_ACDBASSOCDEPENDENCY_314;
unknown_dxf[315].fields = unknown_dxf_ACDBASSOCDEPENDENCY_315;
unknown_dxf[316].fields = unknown_dxf_ACDBASSOCDEPENDENCY_316;
unknown_dxf[317].fields = unknown_dxf_ACDBASSOCDEPENDENCY_317;
unknown_dxf[318].fields = unknown_dxf_ACDBASSOCDEPENDENCY_318;
unknown_dxf[319].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_319;
unknown_dxf[320].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_320;
unknown_dxf[321].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_321;
unknown_dxf[322].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_322;
unknown_dxf[323].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_323;
unknown_dxf[324].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_324;
unknown_dxf[325].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_325;
unknown_dxf[326].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_326;
unknown_dxf[327].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_327;
unknown_dxf[328].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_328;
unknown_dxf[329].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_329;
unknown_dxf[330].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_330;
unknown_dxf[331].fields = unknown_dxf_ACDBASSOCEDGEACTIONPARAM_331;
unknown_dxf[332].fields = unknown_dxf_ACDBASSOCEXTRUDEDSURFACEACTIONBODY_332;
unknown_dxf[333].fields = unknown_dxf_ACDBASSOCEXTRUDEDSURFACEACTIONBODY_333;
unknown_dxf[334].fields = unknown_dxf_ACDBASSOCLOFTEDSURFACEACTIONBODY_334;
unknown_dxf[335].fields = unknown_dxf_ACDBASSOCLOFTEDSURFACEACTIONBODY_335;
unknown_dxf[336].fields = unknown_dxf_ACDBASSOCOSNAPPOINTREFACTIONPARAM_336;
unknown_dxf[337].fields = unknown_dxf_ACDBASSOCOSNAPPOINTREFACTIONPARAM_337;
unknown_dxf[338].fields = unknown_dxf_ACDBASSOCOSNAPPOINTREFACTIONPARAM_338;
unknown_dxf[339].fields = unknown_dxf_ACDBASSOCOSNAPPOINTREFACTIONPARAM_339;
unknown_dxf[340].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_340;
unknown_dxf[341].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_341;
unknown_dxf[342].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_342;
unknown_dxf[343].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_343;
unknown_dxf[344].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_344;
unknown_dxf[345].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_345;
unknown_dxf[346].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_346;
unknown_dxf[347].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_347;
unknown_dxf[348].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_348;
unknown_dxf[349].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_349;
unknown_dxf[350].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_350;
unknown_dxf[351].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_351;
unknown_dxf[352].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_352;
unknown_dxf[353].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_353;
unknown_dxf[354].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_354;
unknown_dxf[355].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_355;
unknown_dxf[356].fields = unknown_dxf_ACDBASSOCPATHACTIONPARAM_356;
unknown_dxf[357].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_357;
unknown_dxf[358].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_358;
unknown_dxf[359].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_359;
unknown_dxf[360].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_360;
unknown_dxf[361].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_361;
unknown_dxf[362].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_362;
unknown_dxf[363].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_363;
unknown_dxf[364].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_364;
unknown_dxf[365].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_365;
unknown_dxf[366].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_366;
unknown_dxf[367].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_367;
unknown_dxf[368].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_368;
unknown_dxf[369].fields = unknown_dxf_ACDBASSOCPERSSUBENTMANAGER_369;
unknown_dxf[370].fields = unknown_dxf_ACDBASSOCPLANESURFACEACTIONBODY_370;
unknown_dxf[371].fields = unknown_dxf_ACDBASSOCREVOLVEDSURFACEACTIONBODY_371;
unknown_dxf[372].fields = unknown_dxf_ACDBASSOCREVOLVEDSURFACEACTIONBODY_372;
unknown_dxf[373].fields = unknown_dxf_ACDBASSOCSWEPTSURFACEACTIONBODY_373;
unknown_dxf[374].fields = unknown_dxf_ACDBASSOCSWEPTSURFACEACTIONBODY_374;
unknown_dxf[375].fields = unknown_dxf_ACDBASSOCVALUEDEPENDENCY_375;
unknown_dxf[376].fields = unknown_dxf_ACDBASSOCVALUEDEPENDENCY_376;
unknown_dxf[377].fields = unknown_dxf_ACDBASSOCVALUEDEPENDENCY_377;
unknown_dxf[378].fields = unknown_dxf_ACDBASSOCVARIABLE_378;
unknown_dxf[379].fields = unknown_dxf_ACDBASSOCVARIABLE_379;
unknown_dxf[380].fields = unknown_dxf_ACDBASSOCVARIABLE_380;
unknown_dxf[381].fields = unknown_dxf_ACDBASSOCVARIABLE_381;
unknown_dxf[382].fields = unknown_dxf_ACDBASSOCVARIABLE_382;
unknown_dxf[383].fields = unknown_dxf_ACDBASSOCVARIABLE_383;
unknown_dxf[384].fields = unknown_dxf_ACDBASSOCVARIABLE_384;
unknown_dxf[385].fields = unknown_dxf_ACDBASSOCVARIABLE_385;
unknown_dxf[386].fields = unknown_dxf_ACDBASSOCVARIABLE_386;
unknown_dxf[387].fields = unknown_dxf_ACDBASSOCVARIABLE_387;
unknown_dxf[388].fields = unknown_dxf_ACDBASSOCVARIABLE_388;
unknown_dxf[389].fields = unknown_dxf_ACDBASSOCVARIABLE_389;
unknown_dxf[390].fields = unknown_dxf_ACDBASSOCVARIABLE_390;
unknown_dxf[391].fields = unknown_dxf_ACDBASSOCVARIABLE_391;
unknown_dxf[392].fields = unknown_dxf_ACDBASSOCVARIABLE_392;
unknown_dxf[393].fields = unknown_dxf_ACDBASSOCVARIABLE_393;
unknown_dxf[394].fields = unknown_dxf_ACDBASSOCVARIABLE_394;
unknown_dxf[395].fields = unknown_dxf_ACDBASSOCVARIABLE_395;
unknown_dxf[396].fields = unknown_dxf_ACDBASSOCVARIABLE_396;
unknown_dxf[397].fields = unknown_dxf_ACDBASSOCVARIABLE_397;
unknown_dxf[398].fields = unknown_dxf_ACDBASSOCVARIABLE_398;
unknown_dxf[399].fields = unknown_dxf_ACDBASSOCVARIABLE_399;
unknown_dxf[400].fields = unknown_dxf_ACDBASSOCVARIABLE_400;
unknown_dxf[401].fields = unknown_dxf_ACDBASSOCVARIABLE_401;
unknown_dxf[402].fields = unknown_dxf_ACDBASSOCVARIABLE_402;
unknown_dxf[403].fields = unknown_dxf_ACDBASSOCVARIABLE_403;
unknown_dxf[404].fields = unknown_dxf_ACDBASSOCVARIABLE_404;
unknown_dxf[405].fields = unknown_dxf_ACDBASSOCVARIABLE_405;
unknown_dxf[406].fields = unknown_dxf_ACDBASSOCVARIABLE_406;
unknown_dxf[407].fields = unknown_dxf_ACDBASSOCVARIABLE_407;
unknown_dxf[408].fields = unknown_dxf_ACDBASSOCVARIABLE_408;
unknown_dxf[409].fields = unknown_dxf_ACDBASSOCVARIABLE_409;
unknown_dxf[410].fields = unknown_dxf_ACDBASSOCVARIABLE_410;
unknown_dxf[411].fields = unknown_dxf_ACDBASSOCVARIABLE_411;
unknown_dxf[412].fields = unknown_dxf_ACDBASSOCVARIABLE_412;
unknown_dxf[413].fields = unknown_dxf_ACDBASSOCVARIABLE_413;
unknown_dxf[414].fields = unknown_dxf_ACDBASSOCVARIABLE_414;
unknown_dxf[415].fields = unknown_dxf_ACDBASSOCVARIABLE_415;
unknown_dxf[416].fields = unknown_dxf_ACDBASSOCVARIABLE_416;
unknown_dxf[417].fields = unknown_dxf_ACDBASSOCVARIABLE_417;
unknown_dxf[418].fields = unknown_dxf_ACDBASSOCVARIABLE_418;
unknown_dxf[419].fields = unknown_dxf_ACDBASSOCVARIABLE_419;
unknown_dxf[420].fields = unknown_dxf_ACDBASSOCVARIABLE_420;
unknown_dxf[421].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_421;
unknown_dxf[422].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_422;
unknown_dxf[423].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_423;
unknown_dxf[424].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_424;
unknown_dxf[425].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_425;
unknown_dxf[426].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_426;
unknown_dxf[427].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_427;
unknown_dxf[428].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_428;
unknown_dxf[429].fields = unknown_dxf_ACDBASSOCVERTEXACTIONPARAM_429;
unknown_dxf[430].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_430;
unknown_dxf[431].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_431;
unknown_dxf[432].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_432;
unknown_dxf[433].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_433;
unknown_dxf[434].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_434;
unknown_dxf[435].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_435;
unknown_dxf[436].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_436;
unknown_dxf[437].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_437;
unknown_dxf[438].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_438;
unknown_dxf[439].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_439;
unknown_dxf[440].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_440;
unknown_dxf[441].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_441;
unknown_dxf[442].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_442;
unknown_dxf[443].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_443;
unknown_dxf[444].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_444;
unknown_dxf[445].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_445;
unknown_dxf[446].fields = unknown_dxf_ACDB_BLKREFOBJECTCONTEXTDATA_CLASS_446;
unknown_dxf[447].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_447;
unknown_dxf[448].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_448;
unknown_dxf[449].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_449;
unknown_dxf[450].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_450;
unknown_dxf[451].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_451;
unknown_dxf[452].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_452;
unknown_dxf[453].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_453;
unknown_dxf[454].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_454;
unknown_dxf[455].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_455;
unknown_dxf[456].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_456;
unknown_dxf[457].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_457;
unknown_dxf[458].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_458;
unknown_dxf[459].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_459;
unknown_dxf[460].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_460;
unknown_dxf[461].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_461;
unknown_dxf[462].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_462;
unknown_dxf[463].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_463;
unknown_dxf[464].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_464;
unknown_dxf[465].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_465;
unknown_dxf[466].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_466;
unknown_dxf[467].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_467;
unknown_dxf[468].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_468;
unknown_dxf[469].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_469;
unknown_dxf[470].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_470;
unknown_dxf[471].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_471;
unknown_dxf[472].fields = unknown_dxf_ACDB_BLOCKREPRESENTATION_DATA_472;
unknown_dxf[473].fields = unknown_dxf_ACDB_DYNAMICBLOCKPROXYNODE_473;
unknown_dxf[474].fields = unknown_dxf_ACDB_DYNAMICBLOCKPROXYNODE_474;
unknown_dxf[475].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_475;
unknown_dxf[476].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_476;
unknown_dxf[477].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_477;
unknown_dxf[478].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_478;
unknown_dxf[479].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_479;
unknown_dxf[480].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_480;
unknown_dxf[481].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_481;
unknown_dxf[482].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_482;
unknown_dxf[483].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_483;
unknown_dxf[484].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_484;
unknown_dxf[485].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_485;
unknown_dxf[486].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_486;
unknown_dxf[487].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_487;
unknown_dxf[488].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_488;
unknown_dxf[489].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_489;
unknown_dxf[490].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_490;
unknown_dxf[491].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_491;
unknown_dxf[492].fields = unknown_dxf_ACDB_LEADEROBJECTCONTEXTDATA_CLASS_492;
unknown_dxf[493].fields = unknown_dxf_ACDB_MLEADEROBJECTCONTEXTDATA_CLASS_493;
unknown_dxf[494].fields = unknown_dxf_ACDB_MLEADEROBJECTCONTEXTDATA_CLASS_494;
unknown_dxf[495].fields = unknown_dxf_ACDB_MLEADEROBJECTCONTEXTDATA_CLASS_495;
unknown_dxf[496].fields = unknown_dxf_ACDB_MLEADEROBJECTCONTEXTDATA_CLASS_496;
unknown_dxf[497].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_497;
unknown_dxf[498].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_498;
unknown_dxf[499].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_499;
unknown_dxf[500].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_500;
unknown_dxf[501].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_501;
unknown_dxf[502].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_502;
unknown_dxf[503].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_503;
unknown_dxf[504].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_504;
unknown_dxf[505].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_505;
unknown_dxf[506].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_506;
unknown_dxf[507].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_507;
unknown_dxf[508].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_508;
unknown_dxf[509].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_509;
unknown_dxf[510].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_510;
unknown_dxf[511].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_511;
unknown_dxf[512].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_512;
unknown_dxf[513].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_513;
unknown_dxf[514].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_514;
unknown_dxf[515].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_515;
unknown_dxf[516].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_516;
unknown_dxf[517].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_517;
unknown_dxf[518].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_518;
unknown_dxf[519].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_519;
unknown_dxf[520].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_520;
unknown_dxf[521].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_521;
unknown_dxf[522].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_522;
unknown_dxf[523].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_523;
unknown_dxf[524].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_524;
unknown_dxf[525].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_525;
unknown_dxf[526].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_526;
unknown_dxf[527].fields = unknown_dxf_ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS_527;
unknown_dxf[528].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_528;
unknown_dxf[529].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_529;
unknown_dxf[530].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_530;
unknown_dxf[531].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_531;
unknown_dxf[532].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_532;
unknown_dxf[533].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_533;
unknown_dxf[534].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_534;
unknown_dxf[535].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_535;
unknown_dxf[536].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_536;
unknown_dxf[537].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_537;
unknown_dxf[538].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_538;
unknown_dxf[539].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_539;
unknown_dxf[540].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_540;
unknown_dxf[541].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_541;
unknown_dxf[542].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_542;
unknown_dxf[543].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_543;
unknown_dxf[544].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_544;
unknown_dxf[545].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_545;
unknown_dxf[546].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_546;
unknown_dxf[547].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_547;
unknown_dxf[548].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_548;
unknown_dxf[549].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_549;
unknown_dxf[550].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_550;
unknown_dxf[551].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_551;
unknown_dxf[552].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_552;
unknown_dxf[553].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_553;
unknown_dxf[554].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_554;
unknown_dxf[555].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_555;
unknown_dxf[556].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_556;
unknown_dxf[557].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_557;
unknown_dxf[558].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_558;
unknown_dxf[559].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_559;
unknown_dxf[560].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_560;
unknown_dxf[561].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_561;
unknown_dxf[562].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_562;
unknown_dxf[563].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_563;
unknown_dxf[564].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_564;
unknown_dxf[565].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_565;
unknown_dxf[566].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_566;
unknown_dxf[567].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_567;
unknown_dxf[568].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_568;
unknown_dxf[569].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_569;
unknown_dxf[570].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_570;
unknown_dxf[571].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_571;
unknown_dxf[572].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_572;
unknown_dxf[573].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_573;
unknown_dxf[574].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_574;
unknown_dxf[575].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_575;
unknown_dxf[576].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_576;
unknown_dxf[577].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_577;
unknown_dxf[578].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_578;
unknown_dxf[579].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_579;
unknown_dxf[580].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_580;
unknown_dxf[581].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_581;
unknown_dxf[582].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_582;
unknown_dxf[583].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_583;
unknown_dxf[584].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_584;
unknown_dxf[585].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_585;
unknown_dxf[586].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_586;
unknown_dxf[587].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_587;
unknown_dxf[588].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_588;
unknown_dxf[589].fields = unknown_dxf_ACDB_MTEXTOBJECTCONTEXTDATA_CLASS_589;
unknown_dxf[590].fields = unknown_dxf_ACDBPERSSUBENTMANAGER_590;
unknown_dxf[591].fields = unknown_dxf_ACDBPERSSUBENTMANAGER_591;
unknown_dxf[592].fields = unknown_dxf_ACDBPERSSUBENTMANAGER_592;
unknown_dxf[593].fields = unknown_dxf_ACDBPERSSUBENTMANAGER_593;
unknown_dxf[594].fields = unknown_dxf_ACDBPERSSUBENTMANAGER_594;
unknown_dxf[595].fields = unknown_dxf_ACDBPERSSUBENTMANAGER_595;
unknown_dxf[596].fields = unknown_dxf_ACDBPERSSUBENTMANAGER_596;
unknown_dxf[597].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_597;
unknown_dxf[598].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_598;
unknown_dxf[599].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_599;
unknown_dxf[600].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_600;
unknown_dxf[601].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_601;
unknown_dxf[602].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_602;
unknown_dxf[603].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_603;
unknown_dxf[604].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_604;
unknown_dxf[605].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_605;
unknown_dxf[606].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_606;
unknown_dxf[607].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_607;
unknown_dxf[608].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_608;
unknown_dxf[609].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_609;
unknown_dxf[610].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_610;
unknown_dxf[611].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_611;
unknown_dxf[612].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_612;
unknown_dxf[613].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_613;
unknown_dxf[614].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_614;
unknown_dxf[615].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_615;
unknown_dxf[616].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_616;
unknown_dxf[617].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_617;
unknown_dxf[618].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_618;
unknown_dxf[619].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_619;
unknown_dxf[620].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_620;
unknown_dxf[621].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_621;
unknown_dxf[622].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_622;
unknown_dxf[623].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_623;
unknown_dxf[624].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_624;
unknown_dxf[625].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_625;
unknown_dxf[626].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_626;
unknown_dxf[627].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_627;
unknown_dxf[628].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_628;
unknown_dxf[629].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_629;
unknown_dxf[630].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_630;
unknown_dxf[631].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_631;
unknown_dxf[632].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_632;
unknown_dxf[633].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_633;
unknown_dxf[634].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_634;
unknown_dxf[635].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_635;
unknown_dxf[636].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_636;
unknown_dxf[637].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_637;
unknown_dxf[638].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_638;
unknown_dxf[639].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_639;
unknown_dxf[640].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_640;
unknown_dxf[641].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_641;
unknown_dxf[642].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_642;
unknown_dxf[643].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_643;
unknown_dxf[644].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_644;
unknown_dxf[645].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_645;
unknown_dxf[646].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_646;
unknown_dxf[647].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_647;
unknown_dxf[648].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_648;
unknown_dxf[649].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_649;
unknown_dxf[650].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_650;
unknown_dxf[651].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_651;
unknown_dxf[652].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_652;
unknown_dxf[653].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_653;
unknown_dxf[654].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_654;
unknown_dxf[655].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_655;
unknown_dxf[656].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_656;
unknown_dxf[657].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_657;
unknown_dxf[658].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_658;
unknown_dxf[659].fields = unknown_dxf_ACDB_TEXTOBJECTCONTEXTDATA_CLASS_659;
unknown_dxf[660].fields = unknown_dxf_ACSH_CHAMFER_CLASS_660;
unknown_dxf[661].fields = unknown_dxf_ACSH_CHAMFER_CLASS_661;
unknown_dxf[662].fields = unknown_dxf_ACSH_CHAMFER_CLASS_662;
unknown_dxf[663].fields = unknown_dxf_ACSH_CHAMFER_CLASS_663;
unknown_dxf[664].fields = unknown_dxf_ACSH_CHAMFER_CLASS_664;
unknown_dxf[665].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_665;
unknown_dxf[666].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_666;
unknown_dxf[667].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_667;
unknown_dxf[668].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_668;
unknown_dxf[669].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_669;
unknown_dxf[670].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_670;
unknown_dxf[671].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_671;
unknown_dxf[672].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_672;
unknown_dxf[673].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_673;
unknown_dxf[674].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_674;
unknown_dxf[675].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_675;
unknown_dxf[676].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_676;
unknown_dxf[677].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_677;
unknown_dxf[678].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_678;
unknown_dxf[679].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_679;
unknown_dxf[680].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_680;
unknown_dxf[681].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_681;
unknown_dxf[682].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_682;
unknown_dxf[683].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_683;
unknown_dxf[684].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_684;
unknown_dxf[685].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_685;
unknown_dxf[686].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_686;
unknown_dxf[687].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_687;
unknown_dxf[688].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_688;
unknown_dxf[689].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_689;
unknown_dxf[690].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_690;
unknown_dxf[691].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_691;
unknown_dxf[692].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_692;
unknown_dxf[693].fields = unknown_dxf_ACSH_EXTRUSION_CLASS_693;
unknown_dxf[694].fields = unknown_dxf_ACSH_LOFT_CLASS_694;
unknown_dxf[695].fields = unknown_dxf_ACSH_LOFT_CLASS_695;
unknown_dxf[696].fields = unknown_dxf_ACSH_LOFT_CLASS_696;
unknown_dxf[697].fields = unknown_dxf_ACSH_REVOLVE_CLASS_697;
unknown_dxf[698].fields = unknown_dxf_ACSH_REVOLVE_CLASS_698;
unknown_dxf[699].fields = unknown_dxf_ACSH_REVOLVE_CLASS_699;
unknown_dxf[700].fields = unknown_dxf_ACSH_REVOLVE_CLASS_700;
unknown_dxf[701].fields = unknown_dxf_ACSH_REVOLVE_CLASS_701;
unknown_dxf[702].fields = unknown_dxf_ACSH_SWEEP_CLASS_702;
unknown_dxf[703].fields = unknown_dxf_ACSH_SWEEP_CLASS_703;
unknown_dxf[704].fields = unknown_dxf_ACSH_SWEEP_CLASS_704;
unknown_dxf[705].fields = unknown_dxf_ACSH_SWEEP_CLASS_705;
unknown_dxf[706].fields = unknown_dxf_ACSH_SWEEP_CLASS_706;
unknown_dxf[707].fields = unknown_dxf_ACSH_SWEEP_CLASS_707;
unknown_dxf[708].fields = unknown_dxf_ACSH_SWEEP_CLASS_708;
unknown_dxf[709].fields = unknown_dxf_ACSH_SWEEP_CLASS_709;
unknown_dxf[710].fields = unknown_dxf_ACSH_SWEEP_CLASS_710;
unknown_dxf[711].fields = unknown_dxf_ACSH_SWEEP_CLASS_711;
unknown_dxf[712].fields = unknown_dxf_ACSH_SWEEP_CLASS_712;
unknown_dxf[713].fields = unknown_dxf_ACSH_SWEEP_CLASS_713;
unknown_dxf[714].fields = unknown_dxf_ACSH_SWEEP_CLASS_714;
unknown_dxf[715].fields = unknown_dxf_ACSH_SWEEP_CLASS_715;
unknown_dxf[716].fields = unknown_dxf_ACSH_SWEEP_CLASS_716;
unknown_dxf[717].fields = unknown_dxf_ACSH_SWEEP_CLASS_717;
unknown_dxf[718].fields = unknown_dxf_ARC_DIMENSION_718;
unknown_dxf[719].fields = unknown_dxf_ARC_DIMENSION_719;
unknown_dxf[720].fields = unknown_dxf_ARC_DIMENSION_720;
unknown_dxf[721].fields = unknown_dxf_ARC_DIMENSION_721;
unknown_dxf[722].fields = unknown_dxf_ARC_DIMENSION_722;
unknown_dxf[723].fields = unknown_dxf_ASSOCDIMDEPENDENCYBODY_723;
unknown_dxf[724].fields = unknown_dxf_ASSOCDIMDEPENDENCYBODY_724;
unknown_dxf[725].fields = unknown_dxf_ASSOCDIMDEPENDENCYBODY_725;
unknown_dxf[726].fields = unknown_dxf_ASSOCDIMDEPENDENCYBODY_726;
unknown_dxf[727].fields = unknown_dxf_ASSOCDIMDEPENDENCYBODY_727;
unknown_dxf[728].fields = unknown_dxf_BLOCKLOOKUPACTION_728;
unknown_dxf[729].fields = unknown_dxf_BLOCKPARAMDEPENDENCYBODY_729;
unknown_dxf[730].fields = unknown_dxf_BLOCKPOLARSTRETCHACTION_730;
unknown_dxf[731].fields = unknown_dxf_BLOCKSTRETCHACTION_731;
unknown_dxf[732].fields = unknown_dxf_BLOCKSTRETCHACTION_732;
unknown_dxf[733].fields = unknown_dxf_BLOCKSTRETCHACTION_733;
unknown_dxf[734].fields = unknown_dxf_BLOCKSTRETCHACTION_734;
unknown_dxf[735].fields = unknown_dxf_BLOCKSTRETCHACTION_735;
unknown_dxf[736].fields = unknown_dxf_BLOCKSTRETCHACTION_736;
unknown_dxf[737].fields = unknown_dxf_BLOCKSTRETCHACTION_737;
unknown_dxf[738].fields = unknown_dxf_BLOCKSTRETCHACTION_738;
unknown_dxf[739].fields = unknown_dxf_BLOCKSTRETCHACTION_739;
unknown_dxf[740].fields = unknown_dxf_BLOCKSTRETCHACTION_740;
unknown_dxf[741].fields = unknown_dxf_BLOCKSTRETCHACTION_741;
unknown_dxf[742].fields = unknown_dxf_BLOCKSTRETCHACTION_742;
unknown_dxf[743].fields = unknown_dxf_BLOCKSTRETCHACTION_743;
unknown_dxf[744].fields = unknown_dxf_BLOCKSTRETCHACTION_744;
unknown_dxf[745].fields = unknown_dxf_BLOCKSTRETCHACTION_745;
unknown_dxf[746].fields = unknown_dxf_BLOCKSTRETCHACTION_746;
unknown_dxf[747].fields = unknown_dxf_BLOCKSTRETCHACTION_747;
unknown_dxf[748].fields = unknown_dxf_BLOCKSTRETCHACTION_748;
unknown_dxf[749].fields = unknown_dxf_BLOCKSTRETCHACTION_749;
unknown_dxf[750].fields = unknown_dxf_BLOCKSTRETCHACTION_750;
unknown_dxf[751].fields = unknown_dxf_BLOCKSTRETCHACTION_751;
unknown_dxf[752].fields = unknown_dxf_BLOCKSTRETCHACTION_752;
unknown_dxf[753].fields = unknown_dxf_BLOCKSTRETCHACTION_753;
unknown_dxf[754].fields = unknown_dxf_BLOCKSTRETCHACTION_754;
unknown_dxf[755].fields = unknown_dxf_BLOCKSTRETCHACTION_755;
unknown_dxf[756].fields = unknown_dxf_BLOCKSTRETCHACTION_756;
unknown_dxf[757].fields = unknown_dxf_BLOCKSTRETCHACTION_757;
unknown_dxf[758].fields = unknown_dxf_BLOCKSTRETCHACTION_758;
unknown_dxf[759].fields = unknown_dxf_BLOCKSTRETCHACTION_759;
unknown_dxf[760].fields = unknown_dxf_BLOCKSTRETCHACTION_760;
unknown_dxf[761].fields = unknown_dxf_BLOCKSTRETCHACTION_761;
unknown_dxf[762].fields = unknown_dxf_BLOCKSTRETCHACTION_762;
unknown_dxf[763].fields = unknown_dxf_BLOCKSTRETCHACTION_763;
unknown_dxf[764].fields = unknown_dxf_BLOCKSTRETCHACTION_764;
unknown_dxf[765].fields = unknown_dxf_BLOCKSTRETCHACTION_765;
unknown_dxf[766].fields = unknown_dxf_BLOCKXYGRIP_766;
unknown_dxf[767].fields = unknown_dxf_BLOCKXYGRIP_767;
unknown_dxf[768].fields = unknown_dxf_CSACDOCUMENTOPTIONS_768;
unknown_dxf[769].fields = unknown_dxf_DATALINK_769;
unknown_dxf[770].fields = unknown_dxf_DATATABLE_770;
unknown_dxf[771].fields = unknown_dxf_DATATABLE_771;
unknown_dxf[772].fields = unknown_dxf_DATATABLE_772;
unknown_dxf[773].fields = unknown_dxf_DATATABLE_773;
unknown_dxf[774].fields = unknown_dxf_DATATABLE_774;
unknown_dxf[775].fields = unknown_dxf_DATATABLE_775;
unknown_dxf[776].fields = unknown_dxf_DATATABLE_776;
unknown_dxf[777].fields = unknown_dxf_DATATABLE_777;
unknown_dxf[778].fields = unknown_dxf_DIMASSOC_778;
unknown_dxf[779].fields = unknown_dxf_DIMASSOC_779;
unknown_dxf[780].fields = unknown_dxf_DIMASSOC_780;
unknown_dxf[781].fields = unknown_dxf_DIMASSOC_781;
unknown_dxf[782].fields = unknown_dxf_DIMASSOC_782;
unknown_dxf[783].fields = unknown_dxf_DIMASSOC_783;
unknown_dxf[784].fields = unknown_dxf_DIMASSOC_784;
unknown_dxf[785].fields = unknown_dxf_DIMASSOC_785;
unknown_dxf[786].fields = unknown_dxf_DIMASSOC_786;
unknown_dxf[787].fields = unknown_dxf_DIMASSOC_787;
unknown_dxf[788].fields = unknown_dxf_DIMASSOC_788;
unknown_dxf[789].fields = unknown_dxf_DIMASSOC_789;
unknown_dxf[790].fields = unknown_dxf_DIMASSOC_790;
unknown_dxf[791].fields = unknown_dxf_DIMASSOC_791;
unknown_dxf[792].fields = unknown_dxf_DIMASSOC_792;
unknown_dxf[793].fields = unknown_dxf_DIMASSOC_793;
unknown_dxf[794].fields = unknown_dxf_DIMASSOC_794;
unknown_dxf[795].fields = unknown_dxf_DIMASSOC_795;
unknown_dxf[796].fields = unknown_dxf_DIMASSOC_796;
unknown_dxf[797].fields = unknown_dxf_DIMASSOC_797;
unknown_dxf[798].fields = unknown_dxf_DIMASSOC_798;
unknown_dxf[799].fields = unknown_dxf_DIMASSOC_799;
unknown_dxf[800].fields = unknown_dxf_DIMASSOC_800;
unknown_dxf[801].fields = unknown_dxf_DIMASSOC_801;
unknown_dxf[802].fields = unknown_dxf_DIMASSOC_802;
unknown_dxf[803].fields = unknown_dxf_DIMASSOC_803;
unknown_dxf[804].fields = unknown_dxf_DIMASSOC_804;
unknown_dxf[805].fields = unknown_dxf_DIMASSOC_805;
unknown_dxf[806].fields = unknown_dxf_DIMASSOC_806;
unknown_dxf[807].fields = unknown_dxf_DIMASSOC_807;
unknown_dxf[808].fields = unknown_dxf_DIMASSOC_808;
unknown_dxf[809].fields = unknown_dxf_DIMASSOC_809;
unknown_dxf[810].fields = unknown_dxf_DIMASSOC_810;
unknown_dxf[811].fields = unknown_dxf_DIMASSOC_811;
unknown_dxf[812].fields = unknown_dxf_DIMASSOC_812;
unknown_dxf[813].fields = unknown_dxf_DIMASSOC_813;
unknown_dxf[814].fields = unknown_dxf_DIMASSOC_814;
unknown_dxf[815].fields = unknown_dxf_DIMASSOC_815;
unknown_dxf[816].fields = unknown_dxf_DIMASSOC_816;
unknown_dxf[817].fields = unknown_dxf_DIMASSOC_817;
unknown_dxf[818].fields = unknown_dxf_DIMASSOC_818;
unknown_dxf[819].fields = unknown_dxf_DIMASSOC_819;
unknown_dxf[820].fields = unknown_dxf_DIMASSOC_820;
unknown_dxf[821].fields = unknown_dxf_DIMASSOC_821;
unknown_dxf[822].fields = unknown_dxf_DIMASSOC_822;
unknown_dxf[823].fields = unknown_dxf_DIMASSOC_823;
unknown_dxf[824].fields = unknown_dxf_DIMASSOC_824;
unknown_dxf[825].fields = unknown_dxf_DIMASSOC_825;
unknown_dxf[826].fields = unknown_dxf_DIMASSOC_826;
unknown_dxf[827].fields = unknown_dxf_DIMASSOC_827;
unknown_dxf[828].fields = unknown_dxf_DIMASSOC_828;
unknown_dxf[829].fields = unknown_dxf_DIMASSOC_829;
unknown_dxf[830].fields = unknown_dxf_DIMASSOC_830;
unknown_dxf[831].fields = unknown_dxf_DIMASSOC_831;
unknown_dxf[832].fields = unknown_dxf_DIMASSOC_832;
unknown_dxf[833].fields = unknown_dxf_DIMASSOC_833;
unknown_dxf[834].fields = unknown_dxf_DIMASSOC_834;
unknown_dxf[835].fields = unknown_dxf_DIMASSOC_835;
unknown_dxf[836].fields = unknown_dxf_DIMASSOC_836;
unknown_dxf[837].fields = unknown_dxf_DIMASSOC_837;
unknown_dxf[838].fields = unknown_dxf_DIMASSOC_838;
unknown_dxf[839].fields = unknown_dxf_DIMASSOC_839;
unknown_dxf[840].fields = unknown_dxf_DIMASSOC_840;
unknown_dxf[841].fields = unknown_dxf_DIMASSOC_841;
unknown_dxf[842].fields = unknown_dxf_DIMASSOC_842;
unknown_dxf[843].fields = unknown_dxf_DIMASSOC_843;
unknown_dxf[844].fields = unknown_dxf_DIMASSOC_844;
unknown_dxf[845].fields = unknown_dxf_DIMASSOC_845;
unknown_dxf[846].fields = unknown_dxf_DIMASSOC_846;
unknown_dxf[847].fields = unknown_dxf_DIMASSOC_847;
unknown_dxf[848].fields = unknown_dxf_DIMASSOC_848;
unknown_dxf[849].fields = unknown_dxf_DIMASSOC_849;
unknown_dxf[850].fields = unknown_dxf_DIMASSOC_850;
unknown_dxf[851].fields = unknown_dxf_DIMASSOC_851;
unknown_dxf[852].fields = unknown_dxf_DIMASSOC_852;
unknown_dxf[853].fields = unknown_dxf_DIMASSOC_853;
unknown_dxf[854].fields = unknown_dxf_DIMASSOC_854;
unknown_dxf[855].fields = unknown_dxf_DIMASSOC_855;
unknown_dxf[856].fields = unknown_dxf_DIMASSOC_856;
unknown_dxf[857].fields = unknown_dxf_DIMASSOC_857;
unknown_dxf[858].fields = unknown_dxf_DIMASSOC_858;
unknown_dxf[859].fields = unknown_dxf_DIMASSOC_859;
unknown_dxf[860].fields = unknown_dxf_DIMASSOC_860;
unknown_dxf[861].fields = unknown_dxf_DIMASSOC_861;
unknown_dxf[862].fields = unknown_dxf_DIMASSOC_862;
unknown_dxf[863].fields = unknown_dxf_DIMASSOC_863;
unknown_dxf[864].fields = unknown_dxf_DIMASSOC_864;
unknown_dxf[865].fields = unknown_dxf_DIMASSOC_865;
unknown_dxf[866].fields = unknown_dxf_DIMASSOC_866;
unknown_dxf[867].fields = unknown_dxf_DIMASSOC_867;
unknown_dxf[868].fields = unknown_dxf_DIMASSOC_868;
unknown_dxf[869].fields = unknown_dxf_DIMASSOC_869;
unknown_dxf[870].fields = unknown_dxf_DIMASSOC_870;
unknown_dxf[871].fields = unknown_dxf_DIMASSOC_871;
unknown_dxf[872].fields = unknown_dxf_DIMASSOC_872;
unknown_dxf[873].fields = unknown_dxf_DIMASSOC_873;
unknown_dxf[874].fields = unknown_dxf_DIMASSOC_874;
unknown_dxf[875].fields = unknown_dxf_DIMASSOC_875;
unknown_dxf[876].fields = unknown_dxf_DIMASSOC_876;
unknown_dxf[877].fields = unknown_dxf_DIMASSOC_877;
unknown_dxf[878].fields = unknown_dxf_DIMASSOC_878;
unknown_dxf[879].fields = unknown_dxf_DIMASSOC_879;
unknown_dxf[880].fields = unknown_dxf_DIMASSOC_880;
unknown_dxf[881].fields = unknown_dxf_DIMASSOC_881;
unknown_dxf[882].fields = unknown_dxf_DIMASSOC_882;
unknown_dxf[883].fields = unknown_dxf_DIMASSOC_883;
unknown_dxf[884].fields = unknown_dxf_DIMASSOC_884;
unknown_dxf[885].fields = unknown_dxf_DIMASSOC_885;
unknown_dxf[886].fields = unknown_dxf_DIMASSOC_886;
unknown_dxf[887].fields = unknown_dxf_DIMASSOC_887;
unknown_dxf[888].fields = unknown_dxf_DIMASSOC_888;
unknown_dxf[889].fields = unknown_dxf_DIMASSOC_889;
unknown_dxf[890].fields = unknown_dxf_DIMASSOC_890;
unknown_dxf[891].fields = unknown_dxf_DIMASSOC_891;
unknown_dxf[892].fields = unknown_dxf_DIMASSOC_892;
unknown_dxf[893].fields = unknown_dxf_DIMASSOC_893;
unknown_dxf[894].fields = unknown_dxf_DIMASSOC_894;
unknown_dxf[895].fields = unknown_dxf_DIMASSOC_895;
unknown_dxf[896].fields = unknown_dxf_DIMASSOC_896;
unknown_dxf[897].fields = unknown_dxf_DIMASSOC_897;
unknown_dxf[898].fields = unknown_dxf_DIMASSOC_898;
unknown_dxf[899].fields = unknown_dxf_DIMASSOC_899;
unknown_dxf[900].fields = unknown_dxf_DIMASSOC_900;
unknown_dxf[901].fields = unknown_dxf_DIMASSOC_901;
unknown_dxf[902].fields = unknown_dxf_DIMASSOC_902;
unknown_dxf[903].fields = unknown_dxf_DIMASSOC_903;
unknown_dxf[904].fields = unknown_dxf_DIMASSOC_904;
unknown_dxf[905].fields = unknown_dxf_DIMASSOC_905;
unknown_dxf[906].fields = unknown_dxf_DIMASSOC_906;
unknown_dxf[907].fields = unknown_dxf_DIMASSOC_907;
unknown_dxf[908].fields = unknown_dxf_DIMASSOC_908;
unknown_dxf[909].fields = unknown_dxf_DIMASSOC_909;
unknown_dxf[910].fields = unknown_dxf_DIMASSOC_910;
unknown_dxf[911].fields = unknown_dxf_DIMASSOC_911;
unknown_dxf[912].fields = unknown_dxf_DIMASSOC_912;
unknown_dxf[913].fields = unknown_dxf_DIMASSOC_913;
unknown_dxf[914].fields = unknown_dxf_DIMASSOC_914;
unknown_dxf[915].fields = unknown_dxf_DIMASSOC_915;
unknown_dxf[916].fields = unknown_dxf_DIMASSOC_916;
unknown_dxf[917].fields = unknown_dxf_DIMASSOC_917;
unknown_dxf[918].fields = unknown_dxf_DIMASSOC_918;
unknown_dxf[919].fields = unknown_dxf_DIMASSOC_919;
unknown_dxf[920].fields = unknown_dxf_DIMASSOC_920;
unknown_dxf[921].fields = unknown_dxf_DIMASSOC_921;
unknown_dxf[922].fields = unknown_dxf_DIMASSOC_922;
unknown_dxf[923].fields = unknown_dxf_DIMASSOC_923;
unknown_dxf[924].fields = unknown_dxf_DIMASSOC_924;
unknown_dxf[925].fields = unknown_dxf_DIMASSOC_925;
unknown_dxf[926].fields = unknown_dxf_DIMASSOC_926;
unknown_dxf[927].fields = unknown_dxf_DIMASSOC_927;
unknown_dxf[928].fields = unknown_dxf_DIMASSOC_928;
unknown_dxf[929].fields = unknown_dxf_DIMASSOC_929;
unknown_dxf[930].fields = unknown_dxf_DIMASSOC_930;
unknown_dxf[931].fields = unknown_dxf_DIMASSOC_931;
unknown_dxf[932].fields = unknown_dxf_DIMASSOC_932;
unknown_dxf[933].fields = unknown_dxf_DIMASSOC_933;
unknown_dxf[934].fields = unknown_dxf_DIMASSOC_934;
unknown_dxf[935].fields = unknown_dxf_DIMASSOC_935;
unknown_dxf[936].fields = unknown_dxf_DIMASSOC_936;
unknown_dxf[937].fields = unknown_dxf_DIMASSOC_937;
unknown_dxf[938].fields = unknown_dxf_DIMASSOC_938;
unknown_dxf[939].fields = unknown_dxf_DIMASSOC_939;
unknown_dxf[940].fields = unknown_dxf_DIMASSOC_940;
unknown_dxf[941].fields = unknown_dxf_DIMASSOC_941;
unknown_dxf[942].fields = unknown_dxf_DIMASSOC_942;
unknown_dxf[943].fields = unknown_dxf_DIMASSOC_943;
unknown_dxf[944].fields = unknown_dxf_DIMASSOC_944;
unknown_dxf[945].fields = unknown_dxf_DIMASSOC_945;
unknown_dxf[946].fields = unknown_dxf_DIMASSOC_946;
unknown_dxf[947].fields = unknown_dxf_DIMASSOC_947;
unknown_dxf[948].fields = unknown_dxf_DIMASSOC_948;
unknown_dxf[949].fields = unknown_dxf_DIMASSOC_949;
unknown_dxf[950].fields = unknown_dxf_DIMASSOC_950;
unknown_dxf[951].fields = unknown_dxf_DIMASSOC_951;
unknown_dxf[952].fields = unknown_dxf_DIMASSOC_952;
unknown_dxf[953].fields = unknown_dxf_DIMASSOC_953;
unknown_dxf[954].fields = unknown_dxf_DIMASSOC_954;
unknown_dxf[955].fields = unknown_dxf_DIMASSOC_955;
unknown_dxf[956].fields = unknown_dxf_DIMASSOC_956;
unknown_dxf[957].fields = unknown_dxf_DIMASSOC_957;
unknown_dxf[958].fields = unknown_dxf_DIMASSOC_958;
unknown_dxf[959].fields = unknown_dxf_EXTRUDEDSURFACE_959;
unknown_dxf[960].fields = unknown_dxf_EXTRUDEDSURFACE_960;
unknown_dxf[961].fields = unknown_dxf_HELIX_961;
unknown_dxf[962].fields = unknown_dxf_HELIX_962;
unknown_dxf[963].fields = unknown_dxf_HELIX_963;
unknown_dxf[964].fields = unknown_dxf_HELIX_964;
unknown_dxf[965].fields = unknown_dxf_HELIX_965;
unknown_dxf[966].fields = unknown_dxf_HELIX_966;
unknown_dxf[967].fields = unknown_dxf_HELIX_967;
unknown_dxf[968].fields = unknown_dxf_PLANESURFACE_968;
unknown_dxf[969].fields = unknown_dxf_PLANESURFACE_969;
unknown_dxf[970].fields = unknown_dxf_PLANESURFACE_970;
unknown_dxf[971].fields = unknown_dxf_PLANESURFACE_971;
unknown_dxf[972].fields = unknown_dxf_REVOLVEDSURFACE_972;
unknown_dxf[973].fields = unknown_dxf_SECTION_SETTINGS_973;
unknown_dxf[974].fields = unknown_dxf_SECTION_SETTINGS_974;
unknown_dxf[975].fields = unknown_dxf_SWEPTSURFACE_975;
unknown_dxf[976].fields = unknown_dxf_SWEPTSURFACE_976;
unknown_dxf[977].fields = unknown_dxf_TABLESTYLE_977;
unknown_dxf[978].fields = unknown_dxf_TABLESTYLE_978;
unknown_dxf[979].fields = unknown_dxf_TABLESTYLE_979;
unknown_dxf[980].fields = unknown_dxf_TABLESTYLE_980;
unknown_dxf[981].fields = unknown_dxf_TABLESTYLE_981;
unknown_dxf[982].fields = unknown_dxf_TABLESTYLE_982;
unknown_dxf[983].fields = unknown_dxf_TABLESTYLE_983;
unknown_dxf[984].fields = unknown_dxf_TABLESTYLE_984;
unknown_dxf[985].fields = unknown_dxf_TABLESTYLE_985;
unknown_dxf[986].fields = unknown_dxf_TABLESTYLE_986;
unknown_dxf[987].fields = unknown_dxf_TABLESTYLE_987;
unknown_dxf[988].fields = unknown_dxf_TABLESTYLE_988;
unknown_dxf[989].fields = unknown_dxf_TABLESTYLE_989;
unknown_dxf[990].fields = unknown_dxf_TABLESTYLE_990;
unknown_dxf[991].fields = unknown_dxf_TABLESTYLE_991;
unknown_dxf[992].fields = unknown_dxf_TABLESTYLE_992;
unknown_dxf[993].fields = unknown_dxf_TABLESTYLE_993;
unknown_dxf[994].fields = unknown_dxf_TABLESTYLE_994;
unknown_dxf[995].fields = unknown_dxf_TABLESTYLE_995;
unknown_dxf[996].fields = unknown_dxf_TABLESTYLE_996;
unknown_dxf[997].fields = unknown_dxf_TABLESTYLE_997;
unknown_dxf[998].fields = unknown_dxf_TABLESTYLE_998;
unknown_dxf[999].fields = unknown_dxf_TABLESTYLE_999;
unknown_dxf[1000].fields = unknown_dxf_TABLESTYLE_1000;
unknown_dxf[1001].fields = unknown_dxf_TABLESTYLE_1001;
unknown_dxf[1002].fields = unknown_dxf_TABLESTYLE_1002;
unknown_dxf[1003].fields = unknown_dxf_TABLESTYLE_1003;
unknown_dxf[1004].fields = unknown_dxf_TABLESTYLE_1004;
unknown_dxf[1005].fields = unknown_dxf_TABLESTYLE_1005;
unknown_dxf[1006].fields = unknown_dxf_TABLESTYLE_1006;
unknown_dxf[1007].fields = unknown_dxf_TABLESTYLE_1007;
unknown_dxf[1008].fields = unknown_dxf_TABLESTYLE_1008;
unknown_dxf[1009].fields = unknown_dxf_TABLESTYLE_1009;
unknown_dxf[1010].fields = unknown_dxf_TABLESTYLE_1010;
unknown_dxf[1011].fields = unknown_dxf_TABLESTYLE_1011;
unknown_dxf[1012].fields = unknown_dxf_TABLESTYLE_1012;
unknown_dxf[1013].fields = unknown_dxf_TABLESTYLE_1013;
unknown_dxf[1014].fields = unknown_dxf_TABLESTYLE_1014;
unknown_dxf[1015].fields = unknown_dxf_TABLESTYLE_1015;
unknown_dxf[1016].fields = unknown_dxf_TABLESTYLE_1016;
unknown_dxf[1017].fields = unknown_dxf_TABLESTYLE_1017;
unknown_dxf[1018].fields = unknown_dxf_TABLESTYLE_1018;
unknown_dxf[1019].fields = unknown_dxf_TABLESTYLE_1019;
unknown_dxf[1020].fields = unknown_dxf_TABLESTYLE_1020;
unknown_dxf[1021].fields = unknown_dxf_TABLESTYLE_1021;
unknown_dxf[1022].fields = unknown_dxf_TABLESTYLE_1022;
unknown_dxf[1023].fields = unknown_dxf_TABLESTYLE_1023;
unknown_dxf[1024].fields = unknown_dxf_TABLESTYLE_1024;
unknown_dxf[1025].fields = unknown_dxf_TABLESTYLE_1025;
unknown_dxf[1026].fields = unknown_dxf_TABLESTYLE_1026;
unknown_dxf[1027].fields = unknown_dxf_TABLESTYLE_1027;
unknown_dxf[1028].fields = unknown_dxf_TABLESTYLE_1028;
unknown_dxf[1029].fields = unknown_dxf_TABLESTYLE_1029;
unknown_dxf[1030].fields = unknown_dxf_TABLESTYLE_1030;
unknown_dxf[1031].fields = unknown_dxf_TABLESTYLE_1031;
unknown_dxf[1032].fields = unknown_dxf_TABLESTYLE_1032;
unknown_dxf[1033].fields = unknown_dxf_TABLESTYLE_1033;
unknown_dxf[1034].fields = unknown_dxf_TABLESTYLE_1034;
unknown_dxf[1035].fields = unknown_dxf_TABLESTYLE_1035;
unknown_dxf[1036].fields = unknown_dxf_TABLESTYLE_1036;
unknown_dxf[1037].fields = unknown_dxf_TABLESTYLE_1037;
unknown_dxf[1038].fields = unknown_dxf_TABLESTYLE_1038;
unknown_dxf[1039].fields = unknown_dxf_TABLESTYLE_1039;
unknown_dxf[1040].fields = unknown_dxf_TABLESTYLE_1040;
unknown_dxf[1041].fields = unknown_dxf_TABLESTYLE_1041;
unknown_dxf[1042].fields = unknown_dxf_TABLESTYLE_1042;
unknown_dxf[1043].fields = unknown_dxf_TABLESTYLE_1043;
unknown_dxf[1044].fields = unknown_dxf_TABLESTYLE_1044;
unknown_dxf[1045].fields = unknown_dxf_TABLESTYLE_1045;
unknown_dxf[1046].fields = unknown_dxf_TABLESTYLE_1046;
unknown_dxf[1047].fields = unknown_dxf_TABLESTYLE_1047;
unknown_dxf[1048].fields = unknown_dxf_TABLESTYLE_1048;
unknown_dxf[1049].fields = unknown_dxf_TABLESTYLE_1049;
unknown_dxf[1050].fields = unknown_dxf_TABLESTYLE_1050;
unknown_dxf[1051].fields = unknown_dxf_TABLESTYLE_1051;
unknown_dxf[1052].fields = unknown_dxf_TABLESTYLE_1052;
unknown_dxf[1053].fields = unknown_dxf_TABLESTYLE_1053;
unknown_dxf[1054].fields = unknown_dxf_TABLESTYLE_1054;
unknown_dxf[1055].fields = unknown_dxf_TABLESTYLE_1055;
unknown_dxf[1056].fields = unknown_dxf_TABLESTYLE_1056;
unknown_dxf[1057].fields = unknown_dxf_TABLESTYLE_1057;
unknown_dxf[1058].fields = unknown_dxf_TABLESTYLE_1058;
unknown_dxf[1059].fields = unknown_dxf_TABLESTYLE_1059;
unknown_dxf[1060].fields = unknown_dxf_TABLESTYLE_1060;
unknown_dxf[1061].fields = unknown_dxf_TABLESTYLE_1061;
unknown_dxf[1062].fields = unknown_dxf_TABLESTYLE_1062;
unknown_dxf[1063].fields = unknown_dxf_TABLESTYLE_1063;
unknown_dxf[1064].fields = unknown_dxf_TABLESTYLE_1064;
unknown_dxf[1065].fields = unknown_dxf_TABLESTYLE_1065;
unknown_dxf[1066].fields = unknown_dxf_TABLESTYLE_1066;
unknown_dxf[1067].fields = unknown_dxf_TABLESTYLE_1067;
unknown_dxf[1068].fields = unknown_dxf_TABLESTYLE_1068;
unknown_dxf[1069].fields = unknown_dxf_TABLESTYLE_1069;
unknown_dxf[1070].fields = unknown_dxf_TABLESTYLE_1070;
unknown_dxf[1071].fields = unknown_dxf_TABLESTYLE_1071;
unknown_dxf[1072].fields = unknown_dxf_TABLESTYLE_1072;
unknown_dxf[1073].fields = unknown_dxf_TABLESTYLE_1073;
unknown_dxf[1074].fields = unknown_dxf_TABLESTYLE_1074;
unknown_dxf[1075].fields = unknown_dxf_TABLESTYLE_1075;
unknown_dxf[1076].fields = unknown_dxf_TABLESTYLE_1076;
unknown_dxf[1077].fields = unknown_dxf_TABLESTYLE_1077;
unknown_dxf[1078].fields = unknown_dxf_TABLESTYLE_1078;
unknown_dxf[1079].fields = unknown_dxf_TABLESTYLE_1079;
unknown_dxf[1080].fields = unknown_dxf_TABLESTYLE_1080;
unknown_dxf[1081].fields = unknown_dxf_TABLESTYLE_1081;
unknown_dxf[1082].fields = unknown_dxf_TABLESTYLE_1082;
unknown_dxf[1083].fields = unknown_dxf_TABLESTYLE_1083;
unknown_dxf[1084].fields = unknown_dxf_TABLESTYLE_1084;
unknown_dxf[1085].fields = unknown_dxf_TABLESTYLE_1085;
unknown_dxf[1086].fields = unknown_dxf_TABLESTYLE_1086;
unknown_dxf[1087].fields = unknown_dxf_TABLESTYLE_1087;
unknown_dxf[1088].fields = unknown_dxf_TABLESTYLE_1088;
unknown_dxf[1089].fields = unknown_dxf_TABLESTYLE_1089;
unknown_dxf[1090].fields = unknown_dxf_TABLESTYLE_1090;
unknown_dxf[1091].fields = unknown_dxf_TABLESTYLE_1091;
unknown_dxf[1092].fields = unknown_dxf_WIPEOUT_1092;
unknown_dxf[1093].fields = unknown_dxf_WIPEOUT_1093;
unknown_dxf[1094].fields = unknown_dxf_WIPEOUT_1094;
unknown_dxf[1095].fields = unknown_dxf_WIPEOUT_1095;
unknown_dxf[1096].fields = unknown_dxf_WIPEOUT_1096;
unknown_dxf[1097].fields = unknown_dxf_WIPEOUT_1097;
unknown_dxf[1098].fields = unknown_dxf_WIPEOUT_1098;
unknown_dxf[1099].fields = unknown_dxf_WIPEOUT_1099;
unknown_dxf[1100].fields = unknown_dxf_WIPEOUT_1100;
unknown_dxf[1101].fields = unknown_dxf_WIPEOUT_1101;
unknown_dxf[1102].fields = unknown_dxf_WIPEOUT_1102;
unknown_dxf[1103].fields = unknown_dxf_WIPEOUT_1103;
unknown_dxf[1104].fields = unknown_dxf_WIPEOUT_1104;
unknown_dxf[1105].fields = unknown_dxf_WIPEOUT_1105;
unknown_dxf[1106].fields = unknown_dxf_WIPEOUT_1106;
unknown_dxf[1107].fields = unknown_dxf_WIPEOUT_1107;
unknown_dxf[1108].fields = unknown_dxf_WIPEOUT_1108;
unknown_dxf[1109].fields = unknown_dxf_WIPEOUT_1109;
unknown_dxf[1110].fields = unknown_dxf_WIPEOUT_1110;
unknown_dxf[1111].fields = unknown_dxf_ATTDEF_2F7F;
unknown_dxf[1112].fields = unknown_dxf_ATTDEF_2AF3;
