/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
/* generated by make regen-unknown, do not modify */

// code, value, bits, pre_bits, num_bits, type, name, num, pos[]
/* BLOCKPARAMDEPENDENCYBODY 6EF in test/test-data/2010/civil_example-imperial.dxf */
static struct _unknown_field unknown_dxf_BLOCKPARAMDEPENDENCYBODY_729[] = {
  //{ 5, "6EF", NULL, 0, BITS_UNKNOWN, "handle", 1, {-1,-1,-1,-1,-1} },
    { 330, "6EE", NULL, 0, BITS_UNKNOWN, "ownerhandle", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbAssocDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 2, {-1,-1,-1,-1,-1} },
    { 100, "AcDbImpAssocDimDependencyBodyBase", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "1", NULL, 0, BITS_UNKNOWN, "", 2, {-1,-1,-1,-1,-1} },
    { 1, "d1=40", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 100, "AcDbBlockParameterDependencyBody", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 90, "0", NULL, 0, BITS_UNKNOWN, "", 1, {-1,-1,-1,-1,-1} },
    { 0, NULL, NULL, 0, BITS_UNKNOWN, NULL, 0, {-1,-1,-1,-1,-1}}
};
