/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
// Copyright (C) 2023,2024 Free Software Foundation, Inc.
// This file is part of LibreDWG.
// Created from scratch by Reini Urban
// Generated by txt2h.pl.

#include <stdint.h>

/* unsorted rhs values */
static const uint16_t cptbl_cp869[] = {
  /*[0x0] = */255,	// size of vector
  /*[0x1] = */1,
  /*[0x2] = */2,
  /*[0x3] = */3,
  /*[0x4] = */4,
  /*[0x5] = */5,
  /*[0x6] = */6,
  /*[0x7] = */7,
  /*[0x8] = */8,
  /*[0x9] = */9,
  /*[0xa] = */10,
  /*[0xb] = */11,
  /*[0xc] = */12,
  /*[0xd] = */13,
  /*[0xe] = */14,
  /*[0xf] = */15,
  /*[0x10] = */16,
  /*[0x11] = */17,
  /*[0x12] = */18,
  /*[0x13] = */19,
  /*[0x14] = */20,
  /*[0x15] = */21,
  /*[0x16] = */22,
  /*[0x17] = */23,
  /*[0x18] = */24,
  /*[0x19] = */25,
  /*[0x1a] = */26,
  /*[0x1b] = */27,
  /*[0x1c] = */28,
  /*[0x1d] = */29,
  /*[0x1e] = */30,
  /*[0x1f] = */31,
  /*[0x20] = */32,
  /*[0x21] = */33,
  /*[0x22] = */34,
  /*[0x23] = */35,
  /*[0x24] = */36,
  /*[0x25] = */37,
  /*[0x26] = */38,
  /*[0x27] = */39,
  /*[0x28] = */40,
  /*[0x29] = */41,
  /*[0x2a] = */42,
  /*[0x2b] = */43,
  /*[0x2c] = */44,
  /*[0x2d] = */45,
  /*[0x2e] = */46,
  /*[0x2f] = */47,
  /*[0x30] = */48,
  /*[0x31] = */49,
  /*[0x32] = */50,
  /*[0x33] = */51,
  /*[0x34] = */52,
  /*[0x35] = */53,
  /*[0x36] = */54,
  /*[0x37] = */55,
  /*[0x38] = */56,
  /*[0x39] = */57,
  /*[0x3a] = */58,
  /*[0x3b] = */59,
  /*[0x3c] = */60,
  /*[0x3d] = */61,
  /*[0x3e] = */62,
  /*[0x3f] = */63,
  /*[0x40] = */64,
  /*[0x41] = */65,
  /*[0x42] = */66,
  /*[0x43] = */67,
  /*[0x44] = */68,
  /*[0x45] = */69,
  /*[0x46] = */70,
  /*[0x47] = */71,
  /*[0x48] = */72,
  /*[0x49] = */73,
  /*[0x4a] = */74,
  /*[0x4b] = */75,
  /*[0x4c] = */76,
  /*[0x4d] = */77,
  /*[0x4e] = */78,
  /*[0x4f] = */79,
  /*[0x50] = */80,
  /*[0x51] = */81,
  /*[0x52] = */82,
  /*[0x53] = */83,
  /*[0x54] = */84,
  /*[0x55] = */85,
  /*[0x56] = */86,
  /*[0x57] = */87,
  /*[0x58] = */88,
  /*[0x59] = */89,
  /*[0x5a] = */90,
  /*[0x5b] = */91,
  /*[0x5c] = */92,
  /*[0x5d] = */93,
  /*[0x5e] = */94,
  /*[0x5f] = */95,
  /*[0x60] = */96,
  /*[0x61] = */97,
  /*[0x62] = */98,
  /*[0x63] = */99,
  /*[0x64] = */100,
  /*[0x65] = */101,
  /*[0x66] = */102,
  /*[0x67] = */103,
  /*[0x68] = */104,
  /*[0x69] = */105,
  /*[0x6a] = */106,
  /*[0x6b] = */107,
  /*[0x6c] = */108,
  /*[0x6d] = */109,
  /*[0x6e] = */110,
  /*[0x6f] = */111,
  /*[0x70] = */112,
  /*[0x71] = */113,
  /*[0x72] = */114,
  /*[0x73] = */115,
  /*[0x74] = */116,
  /*[0x75] = */117,
  /*[0x76] = */118,
  /*[0x77] = */119,
  /*[0x78] = */120,
  /*[0x79] = */121,
  /*[0x7a] = */122,
  /*[0x7b] = */123,
  /*[0x7c] = */124,
  /*[0x7d] = */125,
  /*[0x7e] = */126,
  /*[0x7f] = */0,
  /*[0x80] = */0,
  /*[0x81] = */0,
  /*[0x82] = */0,
  /*[0x83] = */0,
  /*[0x84] = */0,
  /*[0x85] = */0,
  /*[0x86] = */0x0386,	// GREEK CAPITAL LETTER ALPHA WITH TONOS
  /*[0x87] = */0,
  /*[0x88] = */0x00b7,	// MIDDLE DOT
  /*[0x89] = */0x00ac,	// NOT SIGN
  /*[0x8a] = */0x00a6,	// BROKEN BAR
  /*[0x8b] = */0x2018,	// LEFT SINGLE QUOTATION MARK
  /*[0x8c] = */0x2019,	// RIGHT SINGLE QUOTATION MARK
  /*[0x8d] = */0x0388,	// GREEK CAPITAL LETTER EPSILON WITH TONOS
  /*[0x8e] = */0x2015,	// HORIZONTAL BAR
  /*[0x8f] = */0x0389,	// GREEK CAPITAL LETTER ETA WITH TONOS
  /*[0x90] = */0x038a,	// GREEK CAPITAL LETTER IOTA WITH TONOS
  /*[0x91] = */0x03aa,	// GREEK CAPITAL LETTER IOTA WITH DIALYTIKA
  /*[0x92] = */0x038c,	// GREEK CAPITAL LETTER OMICRON WITH TONOS
  /*[0x93] = */0,
  /*[0x94] = */0,
  /*[0x95] = */0x038e,	// GREEK CAPITAL LETTER UPSILON WITH TONOS
  /*[0x96] = */0x03ab,	// GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA
  /*[0x97] = */0x00a9,	// COPYRIGHT SIGN
  /*[0x98] = */0x038f,	// GREEK CAPITAL LETTER OMEGA WITH TONOS
  /*[0x99] = */0x00b2,	// SUPERSCRIPT TWO
  /*[0x9a] = */0x00b3,	// SUPERSCRIPT THREE
  /*[0x9b] = */0x03ac,	// GREEK SMALL LETTER ALPHA WITH TONOS
  /*[0x9c] = */0x00a3,	// POUND SIGN
  /*[0x9d] = */0x03ad,	// GREEK SMALL LETTER EPSILON WITH TONOS
  /*[0x9e] = */0x03ae,	// GREEK SMALL LETTER ETA WITH TONOS
  /*[0x9f] = */0x03af,	// GREEK SMALL LETTER IOTA WITH TONOS
  /*[0xa0] = */0x03ca,	// GREEK SMALL LETTER IOTA WITH DIALYTIKA
  /*[0xa1] = */0x0390,	// GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS
  /*[0xa2] = */0x03cc,	// GREEK SMALL LETTER OMICRON WITH TONOS
  /*[0xa3] = */0x03cd,	// GREEK SMALL LETTER UPSILON WITH TONOS
  /*[0xa4] = */0x0391,	// GREEK CAPITAL LETTER ALPHA
  /*[0xa5] = */0x0392,	// GREEK CAPITAL LETTER BETA
  /*[0xa6] = */0x0393,	// GREEK CAPITAL LETTER GAMMA
  /*[0xa7] = */0x0394,	// GREEK CAPITAL LETTER DELTA
  /*[0xa8] = */0x0395,	// GREEK CAPITAL LETTER EPSILON
  /*[0xa9] = */0x0396,	// GREEK CAPITAL LETTER ZETA
  /*[0xaa] = */0x0397,	// GREEK CAPITAL LETTER ETA
  /*[0xab] = */0x00bd,	// VULGAR FRACTION ONE HALF
  /*[0xac] = */0x0398,	// GREEK CAPITAL LETTER THETA
  /*[0xad] = */0x0399,	// GREEK CAPITAL LETTER IOTA
  /*[0xae] = */0x00ab,	// LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
  /*[0xaf] = */0x00bb,	// RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
  /*[0xb0] = */0x2591,	// LIGHT SHADE
  /*[0xb1] = */0x2592,	// MEDIUM SHADE
  /*[0xb2] = */0x2593,	// DARK SHADE
  /*[0xb3] = */0x2502,	// BOX DRAWINGS LIGHT VERTICAL
  /*[0xb4] = */0x2524,	// BOX DRAWINGS LIGHT VERTICAL AND LEFT
  /*[0xb5] = */0x039a,	// GREEK CAPITAL LETTER KAPPA
  /*[0xb6] = */0x039b,	// GREEK CAPITAL LETTER LAMDA
  /*[0xb7] = */0x039c,	// GREEK CAPITAL LETTER MU
  /*[0xb8] = */0x039d,	// GREEK CAPITAL LETTER NU
  /*[0xb9] = */0x2563,	// BOX DRAWINGS DOUBLE VERTICAL AND LEFT
  /*[0xba] = */0x2551,	// BOX DRAWINGS DOUBLE VERTICAL
  /*[0xbb] = */0x2557,	// BOX DRAWINGS DOUBLE DOWN AND LEFT
  /*[0xbc] = */0x255d,	// BOX DRAWINGS DOUBLE UP AND LEFT
  /*[0xbd] = */0x039e,	// GREEK CAPITAL LETTER XI
  /*[0xbe] = */0x039f,	// GREEK CAPITAL LETTER OMICRON
  /*[0xbf] = */0x2510,	// BOX DRAWINGS LIGHT DOWN AND LEFT
  /*[0xc0] = */0x2514,	// BOX DRAWINGS LIGHT UP AND RIGHT
  /*[0xc1] = */0x2534,	// BOX DRAWINGS LIGHT UP AND HORIZONTAL
  /*[0xc2] = */0x252c,	// BOX DRAWINGS LIGHT DOWN AND HORIZONTAL
  /*[0xc3] = */0x251c,	// BOX DRAWINGS LIGHT VERTICAL AND RIGHT
  /*[0xc4] = */0x2500,	// BOX DRAWINGS LIGHT HORIZONTAL
  /*[0xc5] = */0x253c,	// BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL
  /*[0xc6] = */0x03a0,	// GREEK CAPITAL LETTER PI
  /*[0xc7] = */0x03a1,	// GREEK CAPITAL LETTER RHO
  /*[0xc8] = */0x255a,	// BOX DRAWINGS DOUBLE UP AND RIGHT
  /*[0xc9] = */0x2554,	// BOX DRAWINGS DOUBLE DOWN AND RIGHT
  /*[0xca] = */0x2569,	// BOX DRAWINGS DOUBLE UP AND HORIZONTAL
  /*[0xcb] = */0x2566,	// BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL
  /*[0xcc] = */0x2560,	// BOX DRAWINGS DOUBLE VERTICAL AND RIGHT
  /*[0xcd] = */0x2550,	// BOX DRAWINGS DOUBLE HORIZONTAL
  /*[0xce] = */0x256c,	// BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL
  /*[0xcf] = */0x03a3,	// GREEK CAPITAL LETTER SIGMA
  /*[0xd0] = */0x03a4,	// GREEK CAPITAL LETTER TAU
  /*[0xd1] = */0x03a5,	// GREEK CAPITAL LETTER UPSILON
  /*[0xd2] = */0x03a6,	// GREEK CAPITAL LETTER PHI
  /*[0xd3] = */0x03a7,	// GREEK CAPITAL LETTER CHI
  /*[0xd4] = */0x03a8,	// GREEK CAPITAL LETTER PSI
  /*[0xd5] = */0x03a9,	// GREEK CAPITAL LETTER OMEGA
  /*[0xd6] = */0x03b1,	// GREEK SMALL LETTER ALPHA
  /*[0xd7] = */0x03b2,	// GREEK SMALL LETTER BETA
  /*[0xd8] = */0x03b3,	// GREEK SMALL LETTER GAMMA
  /*[0xd9] = */0x2518,	// BOX DRAWINGS LIGHT UP AND LEFT
  /*[0xda] = */0x250c,	// BOX DRAWINGS LIGHT DOWN AND RIGHT
  /*[0xdb] = */0x2588,	// FULL BLOCK
  /*[0xdc] = */0x2584,	// LOWER HALF BLOCK
  /*[0xdd] = */0x03b4,	// GREEK SMALL LETTER DELTA
  /*[0xde] = */0x03b5,	// GREEK SMALL LETTER EPSILON
  /*[0xdf] = */0x2580,	// UPPER HALF BLOCK
  /*[0xe0] = */0x03b6,	// GREEK SMALL LETTER ZETA
  /*[0xe1] = */0x03b7,	// GREEK SMALL LETTER ETA
  /*[0xe2] = */0x03b8,	// GREEK SMALL LETTER THETA
  /*[0xe3] = */0x03b9,	// GREEK SMALL LETTER IOTA
  /*[0xe4] = */0x03ba,	// GREEK SMALL LETTER KAPPA
  /*[0xe5] = */0x03bb,	// GREEK SMALL LETTER LAMDA
  /*[0xe6] = */0x03bc,	// GREEK SMALL LETTER MU
  /*[0xe7] = */0x03bd,	// GREEK SMALL LETTER NU
  /*[0xe8] = */0x03be,	// GREEK SMALL LETTER XI
  /*[0xe9] = */0x03bf,	// GREEK SMALL LETTER OMICRON
  /*[0xea] = */0x03c0,	// GREEK SMALL LETTER PI
  /*[0xeb] = */0x03c1,	// GREEK SMALL LETTER RHO
  /*[0xec] = */0x03c3,	// GREEK SMALL LETTER SIGMA
  /*[0xed] = */0x03c2,	// GREEK SMALL LETTER FINAL SIGMA
  /*[0xee] = */0x03c4,	// GREEK SMALL LETTER TAU
  /*[0xef] = */0x0384,	// GREEK TONOS
  /*[0xf0] = */0x00ad,	// SOFT HYPHEN
  /*[0xf1] = */0x00b1,	// PLUS-MINUS SIGN
  /*[0xf2] = */0x03c5,	// GREEK SMALL LETTER UPSILON
  /*[0xf3] = */0x03c6,	// GREEK SMALL LETTER PHI
  /*[0xf4] = */0x03c7,	// GREEK SMALL LETTER CHI
  /*[0xf5] = */0x00a7,	// SECTION SIGN
  /*[0xf6] = */0x03c8,	// GREEK SMALL LETTER PSI
  /*[0xf7] = */0x0385,	// GREEK DIALYTIKA TONOS
  /*[0xf8] = */0x00b0,	// DEGREE SIGN
  /*[0xf9] = */0x00a8,	// DIAERESIS
  /*[0xfa] = */0x03c9,	// GREEK SMALL LETTER OMEGA
  /*[0xfb] = */0x03cb,	// GREEK SMALL LETTER UPSILON WITH DIALYTIKA
  /*[0xfc] = */0x03b0,	// GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS
  /*[0xfd] = */0x03ce,	// GREEK SMALL LETTER OMEGA WITH TONOS
  /*[0xfe] = */0x25a0,	// BLACK SQUARE
  /*[0xff] = */0x00a0,	// NO-BREAK SPACE
};
/* sorted, without the first 127 characters */
static const uint8_t cptbl_alnum_cp869[] = {
  69,		// size of vector
  0x86,	// GREEK CAPITAL LETTER ALPHA WITH TONOS
  0x8d,	// GREEK CAPITAL LETTER EPSILON WITH TONOS
  0x8f,	// GREEK CAPITAL LETTER ETA WITH TONOS
  0x90,	// GREEK CAPITAL LETTER IOTA WITH TONOS
  0x91,	// GREEK CAPITAL LETTER IOTA WITH DIALYTIKA
  0x92,	// GREEK CAPITAL LETTER OMICRON WITH TONOS
  0x95,	// GREEK CAPITAL LETTER UPSILON WITH TONOS
  0x96,	// GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA
  0x98,	// GREEK CAPITAL LETTER OMEGA WITH TONOS
  0x9b,	// GREEK SMALL LETTER ALPHA WITH TONOS
  0x9d,	// GREEK SMALL LETTER EPSILON WITH TONOS
  0x9e,	// GREEK SMALL LETTER ETA WITH TONOS
  0x9f,	// GREEK SMALL LETTER IOTA WITH TONOS
  0xa0,	// GREEK SMALL LETTER IOTA WITH DIALYTIKA
  0xa1,	// GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS
  0xa2,	// GREEK SMALL LETTER OMICRON WITH TONOS
  0xa3,	// GREEK SMALL LETTER UPSILON WITH TONOS
  0xa4,	// GREEK CAPITAL LETTER ALPHA
  0xa5,	// GREEK CAPITAL LETTER BETA
  0xa6,	// GREEK CAPITAL LETTER GAMMA
  0xa7,	// GREEK CAPITAL LETTER DELTA
  0xa8,	// GREEK CAPITAL LETTER EPSILON
  0xa9,	// GREEK CAPITAL LETTER ZETA
  0xaa,	// GREEK CAPITAL LETTER ETA
  0xac,	// GREEK CAPITAL LETTER THETA
  0xad,	// GREEK CAPITAL LETTER IOTA
  0xb5,	// GREEK CAPITAL LETTER KAPPA
  0xb6,	// GREEK CAPITAL LETTER LAMDA
  0xb7,	// GREEK CAPITAL LETTER MU
  0xb8,	// GREEK CAPITAL LETTER NU
  0xbd,	// GREEK CAPITAL LETTER XI
  0xbe,	// GREEK CAPITAL LETTER OMICRON
  0xc6,	// GREEK CAPITAL LETTER PI
  0xc7,	// GREEK CAPITAL LETTER RHO
  0xcf,	// GREEK CAPITAL LETTER SIGMA
  0xd0,	// GREEK CAPITAL LETTER TAU
  0xd1,	// GREEK CAPITAL LETTER UPSILON
  0xd2,	// GREEK CAPITAL LETTER PHI
  0xd3,	// GREEK CAPITAL LETTER CHI
  0xd4,	// GREEK CAPITAL LETTER PSI
  0xd5,	// GREEK CAPITAL LETTER OMEGA
  0xd6,	// GREEK SMALL LETTER ALPHA
  0xd7,	// GREEK SMALL LETTER BETA
  0xd8,	// GREEK SMALL LETTER GAMMA
  0xdd,	// GREEK SMALL LETTER DELTA
  0xde,	// GREEK SMALL LETTER EPSILON
  0xe0,	// GREEK SMALL LETTER ZETA
  0xe1,	// GREEK SMALL LETTER ETA
  0xe2,	// GREEK SMALL LETTER THETA
  0xe3,	// GREEK SMALL LETTER IOTA
  0xe4,	// GREEK SMALL LETTER KAPPA
  0xe5,	// GREEK SMALL LETTER LAMDA
  0xe6,	// GREEK SMALL LETTER MU
  0xe7,	// GREEK SMALL LETTER NU
  0xe8,	// GREEK SMALL LETTER XI
  0xe9,	// GREEK SMALL LETTER OMICRON
  0xea,	// GREEK SMALL LETTER PI
  0xeb,	// GREEK SMALL LETTER RHO
  0xec,	// GREEK SMALL LETTER SIGMA
  0xed,	// GREEK SMALL LETTER FINAL SIGMA
  0xee,	// GREEK SMALL LETTER TAU
  0xf2,	// GREEK SMALL LETTER UPSILON
  0xf3,	// GREEK SMALL LETTER PHI
  0xf4,	// GREEK SMALL LETTER CHI
  0xf6,	// GREEK SMALL LETTER PSI
  0xfa,	// GREEK SMALL LETTER OMEGA
  0xfb,	// GREEK SMALL LETTER UPSILON WITH DIALYTIKA
  0xfc,	// GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS
  0xfd,	// GREEK SMALL LETTER OMEGA WITH TONOS
};
#define MIN_CP869	0
#define MAX_CP869	255
#define MIN_CP869_UC	0
#define MAX_CP869_UC	9632
