/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
// Copyright (C) 2023,2024 Free Software Foundation, Inc.
// This file is part of LibreDWG.
// Created from scratch by Reini Urban
// Generated by txt2h.pl.

#include <stdint.h>

/* unsorted rhs values */
static const uint16_t cptbl_iso_8859_5[] = {
  /*[0x0] = */255,	// size of vector
  /*[0x1] = */1,
  /*[0x2] = */2,
  /*[0x3] = */3,
  /*[0x4] = */4,
  /*[0x5] = */5,
  /*[0x6] = */6,
  /*[0x7] = */7,
  /*[0x8] = */8,
  /*[0x9] = */9,
  /*[0xa] = */10,
  /*[0xb] = */11,
  /*[0xc] = */12,
  /*[0xd] = */13,
  /*[0xe] = */14,
  /*[0xf] = */15,
  /*[0x10] = */16,
  /*[0x11] = */17,
  /*[0x12] = */18,
  /*[0x13] = */19,
  /*[0x14] = */20,
  /*[0x15] = */21,
  /*[0x16] = */22,
  /*[0x17] = */23,
  /*[0x18] = */24,
  /*[0x19] = */25,
  /*[0x1a] = */26,
  /*[0x1b] = */27,
  /*[0x1c] = */28,
  /*[0x1d] = */29,
  /*[0x1e] = */30,
  /*[0x1f] = */31,
  /*[0x20] = */32,
  /*[0x21] = */33,
  /*[0x22] = */34,
  /*[0x23] = */35,
  /*[0x24] = */36,
  /*[0x25] = */37,
  /*[0x26] = */38,
  /*[0x27] = */39,
  /*[0x28] = */40,
  /*[0x29] = */41,
  /*[0x2a] = */42,
  /*[0x2b] = */43,
  /*[0x2c] = */44,
  /*[0x2d] = */45,
  /*[0x2e] = */46,
  /*[0x2f] = */47,
  /*[0x30] = */48,
  /*[0x31] = */49,
  /*[0x32] = */50,
  /*[0x33] = */51,
  /*[0x34] = */52,
  /*[0x35] = */53,
  /*[0x36] = */54,
  /*[0x37] = */55,
  /*[0x38] = */56,
  /*[0x39] = */57,
  /*[0x3a] = */58,
  /*[0x3b] = */59,
  /*[0x3c] = */60,
  /*[0x3d] = */61,
  /*[0x3e] = */62,
  /*[0x3f] = */63,
  /*[0x40] = */64,
  /*[0x41] = */65,
  /*[0x42] = */66,
  /*[0x43] = */67,
  /*[0x44] = */68,
  /*[0x45] = */69,
  /*[0x46] = */70,
  /*[0x47] = */71,
  /*[0x48] = */72,
  /*[0x49] = */73,
  /*[0x4a] = */74,
  /*[0x4b] = */75,
  /*[0x4c] = */76,
  /*[0x4d] = */77,
  /*[0x4e] = */78,
  /*[0x4f] = */79,
  /*[0x50] = */80,
  /*[0x51] = */81,
  /*[0x52] = */82,
  /*[0x53] = */83,
  /*[0x54] = */84,
  /*[0x55] = */85,
  /*[0x56] = */86,
  /*[0x57] = */87,
  /*[0x58] = */88,
  /*[0x59] = */89,
  /*[0x5a] = */90,
  /*[0x5b] = */91,
  /*[0x5c] = */92,
  /*[0x5d] = */93,
  /*[0x5e] = */94,
  /*[0x5f] = */95,
  /*[0x60] = */96,
  /*[0x61] = */97,
  /*[0x62] = */98,
  /*[0x63] = */99,
  /*[0x64] = */100,
  /*[0x65] = */101,
  /*[0x66] = */102,
  /*[0x67] = */103,
  /*[0x68] = */104,
  /*[0x69] = */105,
  /*[0x6a] = */106,
  /*[0x6b] = */107,
  /*[0x6c] = */108,
  /*[0x6d] = */109,
  /*[0x6e] = */110,
  /*[0x6f] = */111,
  /*[0x70] = */112,
  /*[0x71] = */113,
  /*[0x72] = */114,
  /*[0x73] = */115,
  /*[0x74] = */116,
  /*[0x75] = */117,
  /*[0x76] = */118,
  /*[0x77] = */119,
  /*[0x78] = */120,
  /*[0x79] = */121,
  /*[0x7a] = */122,
  /*[0x7b] = */123,
  /*[0x7c] = */124,
  /*[0x7d] = */125,
  /*[0x7e] = */126,
  /*[0x7f] = */0,
  /*[0x80] = */0x0080,	// 	<control>
  /*[0x81] = */0x0081,	// 	<control>
  /*[0x82] = */0x0082,	// 	<control>
  /*[0x83] = */0x0083,	// 	<control>
  /*[0x84] = */0x0084,	// 	<control>
  /*[0x85] = */0x0085,	// 	<control>
  /*[0x86] = */0x0086,	// 	<control>
  /*[0x87] = */0x0087,	// 	<control>
  /*[0x88] = */0x0088,	// 	<control>
  /*[0x89] = */0x0089,	// 	<control>
  /*[0x8A] = */0x008A,	// 	<control>
  /*[0x8B] = */0x008B,	// 	<control>
  /*[0x8C] = */0x008C,	// 	<control>
  /*[0x8D] = */0x008D,	// 	<control>
  /*[0x8E] = */0x008E,	// 	<control>
  /*[0x8F] = */0x008F,	// 	<control>
  /*[0x90] = */0x0090,	// 	<control>
  /*[0x91] = */0x0091,	// 	<control>
  /*[0x92] = */0x0092,	// 	<control>
  /*[0x93] = */0x0093,	// 	<control>
  /*[0x94] = */0x0094,	// 	<control>
  /*[0x95] = */0x0095,	// 	<control>
  /*[0x96] = */0x0096,	// 	<control>
  /*[0x97] = */0x0097,	// 	<control>
  /*[0x98] = */0x0098,	// 	<control>
  /*[0x99] = */0x0099,	// 	<control>
  /*[0x9A] = */0x009A,	// 	<control>
  /*[0x9B] = */0x009B,	// 	<control>
  /*[0x9C] = */0x009C,	// 	<control>
  /*[0x9D] = */0x009D,	// 	<control>
  /*[0x9E] = */0x009E,	// 	<control>
  /*[0x9F] = */0x009F,	// 	<control>
  /*[0xA0] = */0x00A0,	// 	NO-BREAK SPACE
  /*[0xA1] = */0x0401,	// 	CYRILLIC CAPITAL LETTER IO
  /*[0xA2] = */0x0402,	// 	CYRILLIC CAPITAL LETTER DJE
  /*[0xA3] = */0x0403,	// 	CYRILLIC CAPITAL LETTER GJE
  /*[0xA4] = */0x0404,	// 	CYRILLIC CAPITAL LETTER UKRAINIAN IE
  /*[0xA5] = */0x0405,	// 	CYRILLIC CAPITAL LETTER DZE
  /*[0xA6] = */0x0406,	// 	CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I
  /*[0xA7] = */0x0407,	// 	CYRILLIC CAPITAL LETTER YI
  /*[0xA8] = */0x0408,	// 	CYRILLIC CAPITAL LETTER JE
  /*[0xA9] = */0x0409,	// 	CYRILLIC CAPITAL LETTER LJE
  /*[0xAA] = */0x040A,	// 	CYRILLIC CAPITAL LETTER NJE
  /*[0xAB] = */0x040B,	// 	CYRILLIC CAPITAL LETTER TSHE
  /*[0xAC] = */0x040C,	// 	CYRILLIC CAPITAL LETTER KJE
  /*[0xAD] = */0x00AD,	// 	SOFT HYPHEN
  /*[0xAE] = */0x040E,	// 	CYRILLIC CAPITAL LETTER SHORT U
  /*[0xAF] = */0x040F,	// 	CYRILLIC CAPITAL LETTER DZHE
  /*[0xB0] = */0x0410,	// 	CYRILLIC CAPITAL LETTER A
  /*[0xB1] = */0x0411,	// 	CYRILLIC CAPITAL LETTER BE
  /*[0xB2] = */0x0412,	// 	CYRILLIC CAPITAL LETTER VE
  /*[0xB3] = */0x0413,	// 	CYRILLIC CAPITAL LETTER GHE
  /*[0xB4] = */0x0414,	// 	CYRILLIC CAPITAL LETTER DE
  /*[0xB5] = */0x0415,	// 	CYRILLIC CAPITAL LETTER IE
  /*[0xB6] = */0x0416,	// 	CYRILLIC CAPITAL LETTER ZHE
  /*[0xB7] = */0x0417,	// 	CYRILLIC CAPITAL LETTER ZE
  /*[0xB8] = */0x0418,	// 	CYRILLIC CAPITAL LETTER I
  /*[0xB9] = */0x0419,	// 	CYRILLIC CAPITAL LETTER SHORT I
  /*[0xBA] = */0x041A,	// 	CYRILLIC CAPITAL LETTER KA
  /*[0xBB] = */0x041B,	// 	CYRILLIC CAPITAL LETTER EL
  /*[0xBC] = */0x041C,	// 	CYRILLIC CAPITAL LETTER EM
  /*[0xBD] = */0x041D,	// 	CYRILLIC CAPITAL LETTER EN
  /*[0xBE] = */0x041E,	// 	CYRILLIC CAPITAL LETTER O
  /*[0xBF] = */0x041F,	// 	CYRILLIC CAPITAL LETTER PE
  /*[0xC0] = */0x0420,	// 	CYRILLIC CAPITAL LETTER ER
  /*[0xC1] = */0x0421,	// 	CYRILLIC CAPITAL LETTER ES
  /*[0xC2] = */0x0422,	// 	CYRILLIC CAPITAL LETTER TE
  /*[0xC3] = */0x0423,	// 	CYRILLIC CAPITAL LETTER U
  /*[0xC4] = */0x0424,	// 	CYRILLIC CAPITAL LETTER EF
  /*[0xC5] = */0x0425,	// 	CYRILLIC CAPITAL LETTER HA
  /*[0xC6] = */0x0426,	// 	CYRILLIC CAPITAL LETTER TSE
  /*[0xC7] = */0x0427,	// 	CYRILLIC CAPITAL LETTER CHE
  /*[0xC8] = */0x0428,	// 	CYRILLIC CAPITAL LETTER SHA
  /*[0xC9] = */0x0429,	// 	CYRILLIC CAPITAL LETTER SHCHA
  /*[0xCA] = */0x042A,	// 	CYRILLIC CAPITAL LETTER HARD SIGN
  /*[0xCB] = */0x042B,	// 	CYRILLIC CAPITAL LETTER YERU
  /*[0xCC] = */0x042C,	// 	CYRILLIC CAPITAL LETTER SOFT SIGN
  /*[0xCD] = */0x042D,	// 	CYRILLIC CAPITAL LETTER E
  /*[0xCE] = */0x042E,	// 	CYRILLIC CAPITAL LETTER YU
  /*[0xCF] = */0x042F,	// 	CYRILLIC CAPITAL LETTER YA
  /*[0xD0] = */0x0430,	// 	CYRILLIC SMALL LETTER A
  /*[0xD1] = */0x0431,	// 	CYRILLIC SMALL LETTER BE
  /*[0xD2] = */0x0432,	// 	CYRILLIC SMALL LETTER VE
  /*[0xD3] = */0x0433,	// 	CYRILLIC SMALL LETTER GHE
  /*[0xD4] = */0x0434,	// 	CYRILLIC SMALL LETTER DE
  /*[0xD5] = */0x0435,	// 	CYRILLIC SMALL LETTER IE
  /*[0xD6] = */0x0436,	// 	CYRILLIC SMALL LETTER ZHE
  /*[0xD7] = */0x0437,	// 	CYRILLIC SMALL LETTER ZE
  /*[0xD8] = */0x0438,	// 	CYRILLIC SMALL LETTER I
  /*[0xD9] = */0x0439,	// 	CYRILLIC SMALL LETTER SHORT I
  /*[0xDA] = */0x043A,	// 	CYRILLIC SMALL LETTER KA
  /*[0xDB] = */0x043B,	// 	CYRILLIC SMALL LETTER EL
  /*[0xDC] = */0x043C,	// 	CYRILLIC SMALL LETTER EM
  /*[0xDD] = */0x043D,	// 	CYRILLIC SMALL LETTER EN
  /*[0xDE] = */0x043E,	// 	CYRILLIC SMALL LETTER O
  /*[0xDF] = */0x043F,	// 	CYRILLIC SMALL LETTER PE
  /*[0xE0] = */0x0440,	// 	CYRILLIC SMALL LETTER ER
  /*[0xE1] = */0x0441,	// 	CYRILLIC SMALL LETTER ES
  /*[0xE2] = */0x0442,	// 	CYRILLIC SMALL LETTER TE
  /*[0xE3] = */0x0443,	// 	CYRILLIC SMALL LETTER U
  /*[0xE4] = */0x0444,	// 	CYRILLIC SMALL LETTER EF
  /*[0xE5] = */0x0445,	// 	CYRILLIC SMALL LETTER HA
  /*[0xE6] = */0x0446,	// 	CYRILLIC SMALL LETTER TSE
  /*[0xE7] = */0x0447,	// 	CYRILLIC SMALL LETTER CHE
  /*[0xE8] = */0x0448,	// 	CYRILLIC SMALL LETTER SHA
  /*[0xE9] = */0x0449,	// 	CYRILLIC SMALL LETTER SHCHA
  /*[0xEA] = */0x044A,	// 	CYRILLIC SMALL LETTER HARD SIGN
  /*[0xEB] = */0x044B,	// 	CYRILLIC SMALL LETTER YERU
  /*[0xEC] = */0x044C,	// 	CYRILLIC SMALL LETTER SOFT SIGN
  /*[0xED] = */0x044D,	// 	CYRILLIC SMALL LETTER E
  /*[0xEE] = */0x044E,	// 	CYRILLIC SMALL LETTER YU
  /*[0xEF] = */0x044F,	// 	CYRILLIC SMALL LETTER YA
  /*[0xF0] = */0x2116,	// 	NUMERO SIGN
  /*[0xF1] = */0x0451,	// 	CYRILLIC SMALL LETTER IO
  /*[0xF2] = */0x0452,	// 	CYRILLIC SMALL LETTER DJE
  /*[0xF3] = */0x0453,	// 	CYRILLIC SMALL LETTER GJE
  /*[0xF4] = */0x0454,	// 	CYRILLIC SMALL LETTER UKRAINIAN IE
  /*[0xF5] = */0x0455,	// 	CYRILLIC SMALL LETTER DZE
  /*[0xF6] = */0x0456,	// 	CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
  /*[0xF7] = */0x0457,	// 	CYRILLIC SMALL LETTER YI
  /*[0xF8] = */0x0458,	// 	CYRILLIC SMALL LETTER JE
  /*[0xF9] = */0x0459,	// 	CYRILLIC SMALL LETTER LJE
  /*[0xFA] = */0x045A,	// 	CYRILLIC SMALL LETTER NJE
  /*[0xFB] = */0x045B,	// 	CYRILLIC SMALL LETTER TSHE
  /*[0xFC] = */0x045C,	// 	CYRILLIC SMALL LETTER KJE
  /*[0xFD] = */0x00A7,	// 	SECTION SIGN
  /*[0xFE] = */0x045E,	// 	CYRILLIC SMALL LETTER SHORT U
  /*[0xFF] = */0x045F,	// 	CYRILLIC SMALL LETTER DZHE
};
/* sorted, without the first 127 characters */
static const uint8_t cptbl_alnum_iso_8859_5[] = {
  92,		// size of vector
  0xA1,	// 	CYRILLIC CAPITAL LETTER IO
  0xA2,	// 	CYRILLIC CAPITAL LETTER DJE
  0xA3,	// 	CYRILLIC CAPITAL LETTER GJE
  0xA4,	// 	CYRILLIC CAPITAL LETTER UKRAINIAN IE
  0xA5,	// 	CYRILLIC CAPITAL LETTER DZE
  0xA6,	// 	CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I
  0xA7,	// 	CYRILLIC CAPITAL LETTER YI
  0xA8,	// 	CYRILLIC CAPITAL LETTER JE
  0xA9,	// 	CYRILLIC CAPITAL LETTER LJE
  0xAA,	// 	CYRILLIC CAPITAL LETTER NJE
  0xAB,	// 	CYRILLIC CAPITAL LETTER TSHE
  0xAC,	// 	CYRILLIC CAPITAL LETTER KJE
  0xAE,	// 	CYRILLIC CAPITAL LETTER SHORT U
  0xAF,	// 	CYRILLIC CAPITAL LETTER DZHE
  0xB0,	// 	CYRILLIC CAPITAL LETTER A
  0xB1,	// 	CYRILLIC CAPITAL LETTER BE
  0xB2,	// 	CYRILLIC CAPITAL LETTER VE
  0xB3,	// 	CYRILLIC CAPITAL LETTER GHE
  0xB4,	// 	CYRILLIC CAPITAL LETTER DE
  0xB5,	// 	CYRILLIC CAPITAL LETTER IE
  0xB6,	// 	CYRILLIC CAPITAL LETTER ZHE
  0xB7,	// 	CYRILLIC CAPITAL LETTER ZE
  0xB8,	// 	CYRILLIC CAPITAL LETTER I
  0xB9,	// 	CYRILLIC CAPITAL LETTER SHORT I
  0xBA,	// 	CYRILLIC CAPITAL LETTER KA
  0xBB,	// 	CYRILLIC CAPITAL LETTER EL
  0xBC,	// 	CYRILLIC CAPITAL LETTER EM
  0xBD,	// 	CYRILLIC CAPITAL LETTER EN
  0xBE,	// 	CYRILLIC CAPITAL LETTER O
  0xBF,	// 	CYRILLIC CAPITAL LETTER PE
  0xC0,	// 	CYRILLIC CAPITAL LETTER ER
  0xC1,	// 	CYRILLIC CAPITAL LETTER ES
  0xC2,	// 	CYRILLIC CAPITAL LETTER TE
  0xC3,	// 	CYRILLIC CAPITAL LETTER U
  0xC4,	// 	CYRILLIC CAPITAL LETTER EF
  0xC5,	// 	CYRILLIC CAPITAL LETTER HA
  0xC6,	// 	CYRILLIC CAPITAL LETTER TSE
  0xC7,	// 	CYRILLIC CAPITAL LETTER CHE
  0xC8,	// 	CYRILLIC CAPITAL LETTER SHA
  0xC9,	// 	CYRILLIC CAPITAL LETTER SHCHA
  0xCA,	// 	CYRILLIC CAPITAL LETTER HARD SIGN
  0xCB,	// 	CYRILLIC CAPITAL LETTER YERU
  0xCC,	// 	CYRILLIC CAPITAL LETTER SOFT SIGN
  0xCD,	// 	CYRILLIC CAPITAL LETTER E
  0xCE,	// 	CYRILLIC CAPITAL LETTER YU
  0xCF,	// 	CYRILLIC CAPITAL LETTER YA
  0xD0,	// 	CYRILLIC SMALL LETTER A
  0xD1,	// 	CYRILLIC SMALL LETTER BE
  0xD2,	// 	CYRILLIC SMALL LETTER VE
  0xD3,	// 	CYRILLIC SMALL LETTER GHE
  0xD4,	// 	CYRILLIC SMALL LETTER DE
  0xD5,	// 	CYRILLIC SMALL LETTER IE
  0xD6,	// 	CYRILLIC SMALL LETTER ZHE
  0xD7,	// 	CYRILLIC SMALL LETTER ZE
  0xD8,	// 	CYRILLIC SMALL LETTER I
  0xD9,	// 	CYRILLIC SMALL LETTER SHORT I
  0xDA,	// 	CYRILLIC SMALL LETTER KA
  0xDB,	// 	CYRILLIC SMALL LETTER EL
  0xDC,	// 	CYRILLIC SMALL LETTER EM
  0xDD,	// 	CYRILLIC SMALL LETTER EN
  0xDE,	// 	CYRILLIC SMALL LETTER O
  0xDF,	// 	CYRILLIC SMALL LETTER PE
  0xE0,	// 	CYRILLIC SMALL LETTER ER
  0xE1,	// 	CYRILLIC SMALL LETTER ES
  0xE2,	// 	CYRILLIC SMALL LETTER TE
  0xE3,	// 	CYRILLIC SMALL LETTER U
  0xE4,	// 	CYRILLIC SMALL LETTER EF
  0xE5,	// 	CYRILLIC SMALL LETTER HA
  0xE6,	// 	CYRILLIC SMALL LETTER TSE
  0xE7,	// 	CYRILLIC SMALL LETTER CHE
  0xE8,	// 	CYRILLIC SMALL LETTER SHA
  0xE9,	// 	CYRILLIC SMALL LETTER SHCHA
  0xEA,	// 	CYRILLIC SMALL LETTER HARD SIGN
  0xEB,	// 	CYRILLIC SMALL LETTER YERU
  0xEC,	// 	CYRILLIC SMALL LETTER SOFT SIGN
  0xED,	// 	CYRILLIC SMALL LETTER E
  0xEE,	// 	CYRILLIC SMALL LETTER YU
  0xEF,	// 	CYRILLIC SMALL LETTER YA
  0xF1,	// 	CYRILLIC SMALL LETTER IO
  0xF2,	// 	CYRILLIC SMALL LETTER DJE
  0xF3,	// 	CYRILLIC SMALL LETTER GJE
  0xF4,	// 	CYRILLIC SMALL LETTER UKRAINIAN IE
  0xF5,	// 	CYRILLIC SMALL LETTER DZE
  0xF6,	// 	CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
  0xF7,	// 	CYRILLIC SMALL LETTER YI
  0xF8,	// 	CYRILLIC SMALL LETTER JE
  0xF9,	// 	CYRILLIC SMALL LETTER LJE
  0xFA,	// 	CYRILLIC SMALL LETTER NJE
  0xFB,	// 	CYRILLIC SMALL LETTER TSHE
  0xFC,	// 	CYRILLIC SMALL LETTER KJE
  0xFE,	// 	CYRILLIC SMALL LETTER SHORT U
  0xFF,	// 	CYRILLIC SMALL LETTER DZHE
};
#define MIN_ISO_8859_5	0
#define MAX_ISO_8859_5	255
#define MIN_ISO_8859_5_UC	0
#define MAX_ISO_8859_5_UC	8470
