/* -*- c -*- */
/*****************************************************************************/
/*  LibreDWG - free implementation of the DWG file format                    */
/*                                                                           */
/*  Copyright (C) 2018-2025 Free Software Foundation, Inc.                   */
/*                                                                           */
/*  This library is free software, licensed under the terms of the GNU       */
/*  General Public License as published by the Free Software Foundation,     */
/*  either version 3 of the License, or (at your option) any later version.  */
/*  You should have received a copy of the GNU General Public License        */
/*  along with this program.  If not, see <http://www.gnu.org/licenses/>.    */
/*****************************************************************************/

/*
 * classes.inc: the dynamic class dispatcher
 * for all types > 500
 * written by Reini Urban
 */

/**
 * Variable-typed classes and its stability
 */

#ifndef ACTION
# error ACTION define missing: decode, encode, dxf, indxf, print, free, ...
#endif

#define WARN_UNHANDLED_CLASS                                                  \
  LOG_WARN ("Unhandled Class %s %d %s (0x%x%s) %u/" FORMAT_HV,              \
            is_entity ? "entity" : "object", klass->number, klass->dxfname,   \
            klass->proxyflag, klass->is_zombie ? "is_zombie" : "",            \
            obj->index, obj->handle.value)
#define WARN_UNSTABLE_CLASS                                                   \
  LOG_WARN ("Unstable Class %s %d %s (0x%x%s) %u/" FORMAT_HV,               \
            is_entity ? "entity" : "object", klass->number, klass->dxfname,   \
            klass->proxyflag, klass->is_zombie ? "is_zombie" : "",            \
            obj->index, obj->handle.value)
#if defined(IS_FREE) || defined(IS_PRINT)
#  undef WARN_UNSTABLE_CLASS
#  undef WARN_UNHANDLED_CLASS
#  define WARN_UNSTABLE_CLASS
#  define WARN_UNHANDLED_CLASS
#endif

#define DISALLOW_DXF(action, _name) \
  (strEQc (_STR(action), "dxf")) ? DWG_ERR_NOTYETSUPPORTED :
#define ALLOW_DXF(action, _name)

#ifdef DEBUG_CLASSES
# define DEBUGGING_DXF(action, name)                 ALLOW_DXF(action, name)
# define DEBUGGING_CLASS(action, name)               UNSTABLE_CLASS(action, name)
# define DEBUGGING_CLASS_DXF(action, name, _dxfname) UNSTABLE_CLASS_DXF(action, name, _dxfname)
# define DEBUGGING_CLASS_CPP(action, name, _cppname) UNSTABLE_CLASS_CPP(action, name, _cppname)
#else
# define DEBUGGING_DXF(action, name)                 DISALLOW_DXF(action, name)
# define DEBUGGING_CLASS(action, name)               UNHANDLED_CLASS(action, name)
# define DEBUGGING_CLASS_DXF(action, name, _dxfname) UNHANDLED_CLASS_DXF(action, name, _dxfname)
# define DEBUGGING_CLASS_CPP(action, name, _cppname) UNHANDLED_CLASS_CPP(action, name, _cppname)
#endif

#define STABLE_CLASS(action, _name) \
  if (klass->dxfname && strEQc (klass->dxfname, #_name)) \
    { \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_name; \
        obj->fixedtype = DWG_TYPE_##_name; \
        obj->klass = klass; \
      } \
      return DWG_FUNC_N(action,_name) (dat, obj); \
    }
#define STABLE_CLASS_DXF(action, _name, _dxfname) \
  if (klass->dxfname && strEQc (klass->dxfname, #_dxfname)) \
    { \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_dxfname; \
        obj->fixedtype = DWG_TYPE_##_name; \
        obj->klass = klass; \
      } \
      return DWG_FUNC_N(action,_name) (dat, obj); \
    }
#define STABLE_CLASS_CPP(action, _name, _cppname) \
  if (klass->cppname && strEQc (klass->cppname, #_name)) \
    { \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_name; \
        obj->fixedtype = DWG_TYPE_##_name; \
        obj->klass = klass; \
      } \
      return DWG_FUNC_N(action,_name) (dat, obj); \
    }
#define UNSTABLE_CLASS(action, _name) \
  if (klass->dxfname && strEQc (klass->dxfname, #_name)) \
    { \
      WARN_UNSTABLE_CLASS; \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_name; \
        obj->fixedtype = DWG_TYPE_##_name; \
        obj->klass = klass; \
      } \
      return DEBUGGING_DXF(action, name) DWG_FUNC_N(action,_name) (dat, obj); \
    }
#define UNSTABLE_CLASS_DXF(action, _name, _dxfname) \
  if (klass->dxfname && strEQc (klass->dxfname, #_dxfname)) \
    { \
      WARN_UNSTABLE_CLASS; \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_dxfname; \
        obj->fixedtype = DWG_TYPE_##_name; \
        obj->klass = klass; \
      } \
      return DEBUGGING_DXF(action, name) DWG_FUNC_N(action,_name) (dat, obj); \
    }
#define UNSTABLE_CLASS_CPP(action, _name, _cppname) \
  if (klass->cppname && strEQc (klass->cppname, #_cppname)) \
    { \
      WARN_UNSTABLE_CLASS; \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_name; \
        obj->fixedtype = DWG_TYPE_##_name; \
        obj->klass = klass; \
      } \
      return DEBUGGING_DXF(action, name) DWG_FUNC_N(action,_name) (dat, obj); \
    }
#define UNHANDLED_CLASS(action, _name) \
  if (klass->dxfname && strEQc (klass->dxfname, #_name)) \
    { \
      WARN_UNHANDLED_CLASS; \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_name; \
        obj->klass = klass; \
      } \
      return DWG_ERR_UNHANDLEDCLASS; \
    }
#define UNHANDLED_CLASS_DXF(action, _name, _dxfname) \
  if (klass->dxfname && strEQc (klass->dxfname, #_dxfname)) \
    { \
      WARN_UNHANDLED_CLASS; \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_dxfname; \
        obj->klass = klass; \
      } \
      return DWG_ERR_UNHANDLEDCLASS; \
    }
#define UNHANDLED_CLASS_CPP(action, _name, _cppname) \
  if (klass->cppname && strEQc (klass->cppname, #_cppname)) \
    { \
      WARN_UNHANDLED_CLASS; \
      if (strEQc (_STR(action), "decode") || !memcmp (_STR(action), "in", 2)) { \
        obj->name = (char*) #_name; \
        obj->dxfname = (char*) #_name; \
        obj->klass = klass; \
      } \
      return DWG_ERR_UNHANDLEDCLASS; \
    }

  /* Entities */
  STABLE_CLASS    (ACTION, CAMERA)    /* not persistent in a DWG */
  STABLE_CLASS    (ACTION, IMAGE)
  STABLE_CLASS    (ACTION, LIGHT)
  STABLE_CLASS    (ACTION, LWPOLYLINE)
  STABLE_CLASS    (ACTION, HATCH)
  STABLE_CLASS    (ACTION, MESH)
  STABLE_CLASS    (ACTION, MULTILEADER)
  STABLE_CLASS    (ACTION, OLE2FRAME)
  STABLE_CLASS    (ACTION, SECTIONOBJECT)
  STABLE_CLASS    (ACTION, PDFUNDERLAY)
  STABLE_CLASS    (ACTION, DGNUNDERLAY)
  STABLE_CLASS    (ACTION, DWFUNDERLAY)
  // preR13
  STABLE_CLASS    (ACTION, REPEAT)
  STABLE_CLASS    (ACTION, ENDREP)
  STABLE_CLASS_DXF(ACTION, _3DLINE, 3DLINE)
  STABLE_CLASS    (ACTION, JUMP)

  /* Objects */
  STABLE_CLASS    (ACTION, ACSH_BOOLEAN_CLASS)
  STABLE_CLASS    (ACTION, ACSH_BOX_CLASS)
  STABLE_CLASS    (ACTION, ACSH_CONE_CLASS)
  STABLE_CLASS    (ACTION, ACSH_CYLINDER_CLASS)
  STABLE_CLASS    (ACTION, ACSH_FILLET_CLASS)
  STABLE_CLASS    (ACTION, ACSH_HISTORY_CLASS)
  STABLE_CLASS    (ACTION, ACSH_SPHERE_CLASS)
  STABLE_CLASS    (ACTION, ACSH_TORUS_CLASS)
  STABLE_CLASS    (ACTION, ACSH_WEDGE_CLASS) // the same as box
  STABLE_CLASS_DXF (ACTION, ASSOCNETWORK, ACDBASSOCNETWORK)
  STABLE_CLASS_DXF (ACTION, ASSOCGEOMDEPENDENCY, ACDBASSOCGEOMDEPENDENCY)
  STABLE_CLASS    (ACTION, BLOCKGRIPLOCATIONCOMPONENT) // AcDbBlockGripExpr
  STABLE_CLASS    (ACTION, BLOCKALIGNMENTPARAMETER)
  STABLE_CLASS    (ACTION, BLOCKALIGNMENTGRIP)
  STABLE_CLASS    (ACTION, BLOCKBASEPOINTPARAMETER)
  STABLE_CLASS    (ACTION, BLOCKFLIPACTION)
  STABLE_CLASS    (ACTION, BLOCKFLIPPARAMETER)
  STABLE_CLASS    (ACTION, BLOCKFLIPGRIP)
  STABLE_CLASS    (ACTION, BLOCKLINEARGRIP)
  STABLE_CLASS    (ACTION, BLOCKLOOKUPGRIP)
  STABLE_CLASS    (ACTION, BLOCKROTATIONGRIP)
  STABLE_CLASS    (ACTION, BLOCKMOVEACTION)
  STABLE_CLASS    (ACTION, BLOCKROTATEACTION)
  STABLE_CLASS    (ACTION, BLOCKSCALEACTION)
  STABLE_CLASS    (ACTION, BLOCKVISIBILITYGRIP)
  STABLE_CLASS    (ACTION, CELLSTYLEMAP)
  STABLE_CLASS    (ACTION, DICTIONARYVAR)
  STABLE_CLASS_DXF(ACTION, DICTIONARYWDFLT, ACDBDICTIONARYWDFLT)
  STABLE_CLASS_DXF(ACTION, DYNAMICBLOCKPURGEPREVENTER, ACDB_DYNAMICBLOCKPURGEPREVENTER_VERSION)
  STABLE_CLASS    (ACTION, FIELD)
  STABLE_CLASS    (ACTION, FIELDLIST)
  STABLE_CLASS    (ACTION, GEODATA)
  STABLE_CLASS    (ACTION, GROUP)
  STABLE_CLASS    (ACTION, IDBUFFER)
  STABLE_CLASS    (ACTION, INDEX)
  STABLE_CLASS    (ACTION, IMAGEDEF)
  STABLE_CLASS    (ACTION, IMAGEDEF_REACTOR)
  STABLE_CLASS    (ACTION, LAYER_INDEX)
  STABLE_CLASS    (ACTION, LAYERFILTER)
  STABLE_CLASS    (ACTION, LAYOUT)       // includes PLOTSETTINGS
  STABLE_CLASS_DXF(ACTION, PLACEHOLDER, ACDBPLACEHOLDER)
  STABLE_CLASS    (ACTION, PLOTSETTINGS)
  STABLE_CLASS    (ACTION, RASTERVARIABLES)
  STABLE_CLASS    (ACTION, RENDERENVIRONMENT)
  STABLE_CLASS    (ACTION, SCALE)
  STABLE_CLASS    (ACTION, SECTION_MANAGER)
  STABLE_CLASS_DXF(ACTION, DETAILVIEWSTYLE, ACDBDETAILVIEWSTYLE) // excellent coverage
  STABLE_CLASS_DXF(ACTION, SECTIONVIEWSTYLE, ACDBSECTIONVIEWSTYLE)
  STABLE_CLASS    (ACTION, SORTENTSTABLE)
  STABLE_CLASS    (ACTION, SPATIAL_FILTER)
  STABLE_CLASS    (ACTION, SUN)
  STABLE_CLASS    (ACTION, TABLEGEOMETRY)
  STABLE_CLASS    (ACTION, PDFDEFINITION)
  STABLE_CLASS    (ACTION, DGNDEFINITION)
  STABLE_CLASS    (ACTION, DWFDEFINITION)
  STABLE_CLASS    (ACTION, VBA_PROJECT)
  STABLE_CLASS    (ACTION, VISUALSTYLE)
  STABLE_CLASS    (ACTION, WIPEOUT)   /* ent, stable but causes acad redraw to crash (GH #244) */
  STABLE_CLASS    (ACTION, WIPEOUTVARIABLES)
  STABLE_CLASS    (ACTION, XRECORD)

  // unstable/undertested.
  // not enough coverage, but assumed mostly ok. field names may change.
  // DXF output skipped

  UNSTABLE_CLASS     (ACTION, MLEADERSTYLE) // TODO class_version
  UNSTABLE_CLASS     (ACTION, ARC_DIMENSION)     /* ent */
  UNSTABLE_CLASS     (ACTION, PLANESURFACE)      /* ent */
  UNSTABLE_CLASS     (ACTION, ACSH_BREP_CLASS)
  UNSTABLE_CLASS     (ACTION, ACSH_CHAMFER_CLASS)
  UNSTABLE_CLASS     (ACTION, ACSH_PYRAMID_CLASS)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCACTION, ACDBASSOCACTION)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCDEPENDENCY, ACDBASSOCDEPENDENCY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCVALUEDEPENDENCY, ACDBASSOCVALUEDEPENDENCY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCARRAYACTIONBODY, ACDBASSOCARRAYACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCBLENDSURFACEACTIONBODY, ACDBASSOCBLENDSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCEXTENDSURFACEACTIONBODY, ACDBASSOCEXTENDSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCEXTRUDEDSURFACEACTIONBODY, ACDBASSOCEXTRUDEDSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCFILLETSURFACEACTIONBODY, ACDBASSOCFILLETSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCLOFTEDSURFACEACTIONBODY, ACDBASSOCLOFTEDSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCNETWORKSURFACEACTIONBODY, ACDBASSOCNETWORKSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCOFFSETSURFACEACTIONBODY, ACDBASSOCOFFSETSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCPATCHSURFACEACTIONBODY, ACDBASSOCPATCHSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCPLANESURFACEACTIONBODY, ACDBASSOCPLANESURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCREVOLVEDSURFACEACTIONBODY, ACDBASSOCREVOLVEDSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCTRIMSURFACEACTIONBODY, ACDBASSOCTRIMSURFACEACTIONBODY)
  UNSTABLE_CLASS_DXF (ACTION, EVALUATION_GRAPH, ACAD_EVALUATION_GRAPH) /* almost */
  UNSTABLE_CLASS     (ACTION, DATALINK)
  UNSTABLE_CLASS     (ACTION, DBCOLOR)
  UNSTABLE_CLASS     (ACTION, HELIX)     /* ent, looks very stable */
  UNSTABLE_CLASS     (ACTION, LARGE_RADIAL_DIMENSION) /* ent */
  UNSTABLE_CLASS     (ACTION, LAYOUTPRINTCONFIG) /* ent. CSD_APP, probably a mistake as ent */
  UNSTABLE_CLASS     (ACTION, LIGHTLIST)
  UNSTABLE_CLASS     (ACTION, MATERIAL) // encode fails ODA
  UNSTABLE_CLASS     (ACTION, MENTALRAYRENDERSETTINGS) // not exported by Teigha
  UNSTABLE_CLASS     (ACTION, OBJECT_PTR)
  UNSTABLE_CLASS_CPP (ACTION, OBJECT_PTR, CAseDLPNTableRecord)
  UNSTABLE_CLASS_DXF (ACTION, PROXY_OBJECT, ACAD_PROXY_OBJECT)
  UNSTABLE_CLASS     (ACTION, RAPIDRTRENDERSETTINGS)
  UNSTABLE_CLASS     (ACTION, RENDERSETTINGS)
  UNSTABLE_CLASS     (ACTION, RENDERENTRY)
  UNSTABLE_CLASS     (ACTION, RENDERGLOBAL)
  UNSTABLE_CLASS     (ACTION, SPATIAL_INDEX)
  UNSTABLE_CLASS     (ACTION, TABLESTYLE)    // for r2010+ some bits missing at the end
  UNSTABLE_CLASS     (ACTION, SECTION_SETTINGS)
  UNSTABLE_CLASS     (ACTION, SOLID_BACKGROUND)
  UNSTABLE_CLASS     (ACTION, SKYLIGHT_BACKGROUND)
  UNSTABLE_CLASS     (ACTION, GROUND_PLANE_BACKGROUND)
  UNSTABLE_CLASS     (ACTION, GRADIENT_BACKGROUND)
  UNSTABLE_CLASS     (ACTION, IMAGE_BACKGROUND)
  UNSTABLE_CLASS     (ACTION, IBL_BACKGROUND)
  UNSTABLE_CLASS     (ACTION, BLOCKLINEARPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKROTATIONPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKXYPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKVISIBILITYPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKPOLARPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKPOLARGRIP)
  UNSTABLE_CLASS_DXF (ACTION, BLOCKREPRESENTATION, ACDB_BLOCKREPRESENTATION_DATA)
  UNSTABLE_CLASS     (ACTION, ASSOCDIMDEPENDENCYBODY)
  UNSTABLE_CLASS     (ACTION, BLOCKPARAMDEPENDENCYBODY)
  UNSTABLE_CLASS_DXF (ACTION, ALDIMOBJECTCONTEXTDATA, ACDB_ALDIMOBJECTCONTEXTDATA_CLASS)
  UNSTABLE_CLASS_DXF (ACTION, MTEXTOBJECTCONTEXTDATA, ACDB_MTEXTOBJECTCONTEXTDATA_CLASS)
  UNSTABLE_CLASS_DXF (ACTION, BLKREFOBJECTCONTEXTDATA, ACDB_BLKREFOBJECTCONTEXTDATA_CLASS)
  UNSTABLE_CLASS_DXF (ACTION, LEADEROBJECTCONTEXTDATA, ACDB_LEADEROBJECTCONTEXTDATA_CLASS)
  UNSTABLE_CLASS_DXF (ACTION, TEXTOBJECTCONTEXTDATA,  ACDB_TEXTOBJECTCONTEXTDATA_CLASS)
  UNSTABLE_CLASS_DXF (ACTION, FCFOBJECTCONTEXTDATA, ACDB_FCFOBJECTCONTEXTDATA_CLASS)

  UNSTABLE_CLASS_DXF (ACTION, ASSOCVARIABLE, ACDBASSOCVARIABLE)
  UNSTABLE_CLASS_DXF (ACTION, ASSOC2DCONSTRAINTGROUP, ACDBASSOC2DCONSTRAINTGROUP)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCOSNAPPOINTREFACTIONPARAM, ACDBASSOCOSNAPPOINTREFACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCPOINTREFACTIONPARAM, ACDBASSOCPointRefActionParam)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCASMBODYACTIONPARAM, ACDBASSOCASMBODYACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCACTIONPARAM, ACDBASSOCACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCCOMPOUNDACTIONPARAM, ACDBASSOCCOMPOUNDACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCFACEACTIONPARAM, ACDBASSOCFACEACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCOBJECTACTIONPARAM, ACDBASSOCOBJECTACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCPATHACTIONPARAM, ACDBASSOCPATHACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCVERTEXACTIONPARAM, ACDBASSOCVERTEXACTIONPARAM)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCARRAYMODIFYPARAMETERS, ACDBASSOCARRAYMODIFYPARAMETERS)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCARRAYPATHPARAMETERS, ACDBASSOCARRAYPATHPARAMETERS)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCARRAYPOLARPARAMETERS, ACDBASSOCARRAYPOLARPARAMETERS)
  UNSTABLE_CLASS_DXF (ACTION, ASSOCARRAYRECTANGULARPARAMETERS, ACDBASSOCARRAYRECTANGULARPARAMETERS)

  UNSTABLE_CLASS     (ACTION, BLOCKALIGNEDCONSTRAINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKANGULARCONSTRAINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKARRAYACTION)
  UNSTABLE_CLASS     (ACTION, BLOCKDIAMETRICCONSTRAINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKHORIZONTALCONSTRAINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKLINEARCONSTRAINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKRADIALCONSTRAINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKVERTICALCONSTRAINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKLOOKUPACTION)
  UNSTABLE_CLASS     (ACTION, BLOCKLOOKUPPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKPOINTPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKPOLARSTRETCHACTION)
  UNSTABLE_CLASS     (ACTION, BLOCKSTRETCHACTION)
  UNSTABLE_CLASS     (ACTION, BLOCKUSERPARAMETER)
  UNSTABLE_CLASS     (ACTION, BLOCKXYGRIP)

  UNSTABLE_CLASS_DXF (ACTION, POINTCLOUD, ACDBPOINTCLOUD)	/* ent */
  UNSTABLE_CLASS_DXF (ACTION, POINTCLOUDEX, ACDBPOINTCLOUDEX)	/* ent */
  UNSTABLE_CLASS_DXF (ACTION, POINTCLOUDDEF, ACDBPOINTCLOUDDEF)
  UNSTABLE_CLASS_DXF (ACTION, POINTCLOUDDEFEX, ACDBPOINTCLOUDDEFEX)
  UNSTABLE_CLASS_DXF (ACTION, POINTCLOUDDEF_REACTOR, ACDBPOINTCLOUDDEF_REACTOR)
  UNSTABLE_CLASS_DXF (ACTION, POINTCLOUDDEF_REACTOR_EX, ACDBPOINTCLOUDDEF_REACTOR_EX)
  UNSTABLE_CLASS_DXF (ACTION, POINTCLOUDCOLORMAP, ACDBPOINTCLOUDCOLORMAP)
  //UNSTABLE_CLASS     (ACTION, PARTIAL_VIEWING_FILTER)
  UNSTABLE_CLASS     (ACTION, PARTIAL_VIEWING_INDEX)

  // coverage exists, but broken and being worked on. needs -DDEBUG_CLASSES. See also
  // examples/unknown with all the unknown blobs of these.
  // PROXY_ENTITY has a fixed type
  // DXF output skipped, but some INDXF are done.
  DEBUGGING_CLASS     (ACTION, SUNSTUDY)          // almost

  DEBUGGING_CLASS_DXF (ACTION, GEOPOSITIONMARKER, POSITIONMARKER) /* ent, yet unsorted. no coverage */
  DEBUGGING_CLASS     (ACTION, NURBSURFACE)       /* ent */
  DEBUGGING_CLASS     (ACTION, EXTRUDEDSURFACE)   /* ent */
  DEBUGGING_CLASS     (ACTION, LOFTEDSURFACE)     /* ent */
  DEBUGGING_CLASS     (ACTION, REVOLVEDSURFACE)   /* ent */
  DEBUGGING_CLASS     (ACTION, SWEPTSURFACE)      /* ent */
  DEBUGGING_CLASS_DXF (ACTION, TABLE, ACAD_TABLE) /* ent, r2010+ needs subclassing */
  DEBUGGING_CLASS     (ACTION, ARCALIGNEDTEXT)    // ent expresstools
  DEBUGGING_CLASS     (ACTION, RTEXT)             // ent expresstools
  DEBUGGING_CLASS     (ACTION, MPOLYGON)          // hatch-like ent
  DEBUGGING_CLASS_CPP (ACTION, NAVISWORKSMODEL, AcDbNavisworksModel)   /* ent,
                                                     dxfname COORDINATION_MODEL? */
  DEBUGGING_CLASS     (ACTION, DIMASSOC)
  DEBUGGING_CLASS_CPP (ACTION, TABLECONTENT, AcDbTableContent) // dxfname: "TABLE"
  DEBUGGING_CLASS     (ACTION, ACSH_EXTRUSION_CLASS)
  DEBUGGING_CLASS     (ACTION, ACSH_LOFT_CLASS)
  DEBUGGING_CLASS     (ACTION, ACSH_REVOLVE_CLASS)
  DEBUGGING_CLASS     (ACTION, ACSH_SWEEP_CLASS)
  DEBUGGING_CLASS     (ACTION, GEOMAPIMAGE)  // LiveMap image overlay
  DEBUGGING_CLASS_DXF (ACTION, NAVISWORKSMODELDEF, ACDBNAVISWORKSMODELDEF)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCRESTOREENTITYSTATEACTIONBODY, ACDBASSOCRESTOREENTITYSTATEACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCMLEADERACTIONBODY, ACDBASSOCMLEADERACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOC3POINTANGULARDIMACTIONBODY, ACDBASSOC3POINTANGULARDIMACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCARRAYMODIFYACTIONBODY, ACDBASSOCARRAYMODIFYACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCALIGNEDDIMACTIONBODY, ACDBASSOCALIGNEDDIMACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCORDINATEDIMACTIONBODY, ACDBASSOCORDINATEDIMACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCROTATEDDIMACTIONBODY, ACDBASSOCROTATEDDIMACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCSWEPTSURFACEACTIONBODY, ACDBASSOCSWEPTSURFACEACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCEDGECHAMFERACTIONBODY, ACDbAssocEDGECHAMFERACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCEDGEFILLETACTIONBODY, ASSOCEDGEFILLETACTIONBODY)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCPERSSUBENTMANAGER, ACDBASSOCPERSSUBENTMANAGER)
  DEBUGGING_CLASS_DXF (ACTION, PERSUBENTMGR, ACDBPERSSUBENTMANAGER)
  DEBUGGING_CLASS     (ACTION, DATATABLE)
  DEBUGGING_CLASS     (ACTION, CONTEXTDATAMANAGER)
  DEBUGGING_CLASS_DXF (ACTION, ANNOTSCALEOBJECTCONTEXTDATA, ACDB_ANNOTSCALEOBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS_DXF (ACTION, ANGDIMOBJECTCONTEXTDATA, ACDB_ANGDIMOBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS_DXF (ACTION, DMDIMOBJECTCONTEXTDATA, ACDB_DMDIMOBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS_DXF (ACTION, ORDDIMOBJECTCONTEXTDATA, ACDB_ORDDIMOBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS_DXF (ACTION, RADIMOBJECTCONTEXTDATA, ACDB_RADIMOBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS_DXF (ACTION, RADIMLGOBJECTCONTEXTDATA, ACDB_RADIMLGOBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS_DXF (ACTION, MLEADEROBJECTCONTEXTDATA, ACDB_MLEADEROBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS_DXF (ACTION, MTEXTATTRIBUTEOBJECTCONTEXTDATA, ACDB_MTEXTATTRIBUTEOBJECTCONTEXTDATA_CLASS)
  DEBUGGING_CLASS     (ACTION, ACMECOMMANDHISTORY)
  DEBUGGING_CLASS     (ACTION, ACMESCOPE)
  DEBUGGING_CLASS     (ACTION, ACMESTATEMGR)
  DEBUGGING_CLASS_DXF (ACTION, ASSOCEDGEACTIONPARAM, ACDBASSOCEDGEACTIONPARAM)
  DEBUGGING_CLASS     (ACTION, CSACDOCUMENTOPTIONS)
  DEBUGGING_CLASS_DXF (ACTION, CURVEPATH, ACDBCURVEPATH)
  DEBUGGING_CLASS     (ACTION, MOTIONPATH)
  DEBUGGING_CLASS_DXF (ACTION, POINTPATH, ACDBPOINTPATH)
  DEBUGGING_CLASS     (ACTION, TVDEVICEPROPERTIES)
  DEBUGGING_CLASS     (ACTION, ALIGNMENTPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, BASEPOINTPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, FLIPPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, LINEARPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, POINTPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, ROTATIONPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, VISIBILITYPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, FLIPGRIPENTITY)
  DEBUGGING_CLASS     (ACTION, LINEARGRIPENTITY)
  DEBUGGING_CLASS     (ACTION, POLARGRIPENTITY)
  DEBUGGING_CLASS     (ACTION, ROTATIONGRIPENTITY)
  DEBUGGING_CLASS     (ACTION, VISIBILITYGRIPENTITY)
  DEBUGGING_CLASS     (ACTION, XYGRIPENTITY)
  DEBUGGING_CLASS     (ACTION, XYPARAMETERENTITY)
  DEBUGGING_CLASS     (ACTION, BLOCKPROPERTIESTABLE)
  DEBUGGING_CLASS     (ACTION, BLOCKPROPERTIESTABLEGRIP)
  DEBUGGING_CLASS_DXF (ACTION, DYNAMICBLOCKPROXYNODE, ACDB_DYNAMICBLOCKPROXYNODE)
  DEBUGGING_CLASS     (ACTION, BREAKDATA)
  DEBUGGING_CLASS     (ACTION, BREAKPOINTREF)

  // unimplemented, passed through. coverage exists mostly, but documentation not.
  // DXF output skipped
  UNHANDLED_CLASS     (ACTION, PROXY_LWPOLYLINE) 		/* ent */
  UNHANDLED_CLASS     (ACTION, ACDSRECORD)
  UNHANDLED_CLASS     (ACTION, ACDSSCHEMA)
  UNHANDLED_CLASS     (ACTION, NPOCOLLECTION)
  UNHANDLED_CLASS     (ACTION, RAPIDRTRENDERENVIRONMENT)  //no coverage
  UNHANDLED_CLASS_DXF (ACTION, XREFPANELOBJECT, EXACXREFPANELOBJECT)
  UNHANDLED_CLASS     (ACTION, FLIPACTIONENTITY) // AcDbBlockFlipActionEntity
  UNHANDLED_CLASS     (ACTION, MOVEACTIONENTITY)
  UNHANDLED_CLASS     (ACTION, ROTATEACTIONENTITY)
  UNHANDLED_CLASS     (ACTION, SCALEACTIONENTITY)
  UNHANDLED_CLASS     (ACTION, STRETCHACTIONENTITY)

/* Missing DXF names:
ACAD_PROXY_ENTITY
ACAD_PROXY_OBJECT
ARRAY
ATTBLOCKREF
ATTDYNBLOCKREF
BLOCKREF
DYNBLOCKREF
XREF
CENTERMARK
CENTERLINE

// Declared are these DXF names, but not yet added:
ACAD_PROXY_OBJECT_WRAPPER
*/

  UNHANDLED_CLASS     (ACTION, ABSHDRAWINGSETTINGS)
  UNHANDLED_CLASS     (ACTION, ACAECUSTOBJ)
  UNHANDLED_CLASS     (ACTION, ACAEEEMGROBJ)
  UNHANDLED_CLASS     (ACTION, ACAMCOMP)
  UNHANDLED_CLASS     (ACTION, ACAMCOMPDEF)
  UNHANDLED_CLASS     (ACTION, ACAMCOMPDEFMGR)
  UNHANDLED_CLASS     (ACTION, ACAMCONTEXTMODELER)
  UNHANDLED_CLASS     (ACTION, ACAMGDIMSTD)
  UNHANDLED_CLASS     (ACTION, ACAMGFILTERDAT)
  UNHANDLED_CLASS     (ACTION, ACAMGHOLECHARTSTDCSN)
  UNHANDLED_CLASS     (ACTION, ACAMGHOLECHARTSTDDIN)
  UNHANDLED_CLASS     (ACTION, ACAMGHOLECHARTSTDISO)
  UNHANDLED_CLASS     (ACTION, ACAMGLAYSTD)
  UNHANDLED_CLASS     (ACTION, ACAMGRCOMPDEF)
  UNHANDLED_CLASS     (ACTION, ACAMGRCOMPDEFSET)
  UNHANDLED_CLASS     (ACTION, ACAMGTITLESTD)
  UNHANDLED_CLASS     (ACTION, ACAMMVDBACKUPOBJECT)
  UNHANDLED_CLASS     (ACTION, ACAMPROJECT)
  UNHANDLED_CLASS     (ACTION, ACAMSHAFTCOMPDEF)
  UNHANDLED_CLASS     (ACTION, ACAMSTDPCOMPDEF)
  UNHANDLED_CLASS     (ACTION, ACAMWBLOCKTEMPENTS)
  UNHANDLED_CLASS     (ACTION, ACARRAYJIGENTITY)
  UNHANDLED_CLASS     (ACTION, ACCMCONTEXT)
  UNHANDLED_CLASS     (ACTION, ACDBCIRCARCRES)
  UNHANDLED_CLASS     (ACTION, ACDBDIMENSIONRES)
  UNHANDLED_CLASS     (ACTION, ACDBENTITYCACHE)
  UNHANDLED_CLASS     (ACTION, ACDBLINERES)
  UNHANDLED_CLASS     (ACTION, ACDBSTDPARTRES_ARC)
  UNHANDLED_CLASS     (ACTION, ACDBSTDPARTRES_LINE)
  UNHANDLED_CLASS     (ACTION, ACDB_HATCHSCALECONTEXTDATA_CLASS)
  UNHANDLED_CLASS     (ACTION, ACDB_HATCHVIEWCONTEXTDATA_CLASS)
  UNHANDLED_CLASS     (ACTION, ACDB_PROXY_ENTITY_DATA)
  UNHANDLED_CLASS     (ACTION, ACGREFACADMASTER)
  UNHANDLED_CLASS     (ACTION, ACGREFMASTER)
  UNHANDLED_CLASS     (ACTION, ACIMINTSYSVAR)
  UNHANDLED_CLASS     (ACTION, ACIMREALSYSVAR)
  UNHANDLED_CLASS     (ACTION, ACIMSTRSYSVAR)
  UNHANDLED_CLASS     (ACTION, ACIMSYSVARMAN)
  UNHANDLED_CLASS     (ACTION, ACMANOOTATIONVIEWSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMANOOTATIONVIEWSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMANOOTATIONVIEWSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMANOOTATIONVIEWSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMAPLEGENDDBOBJECT)
  UNHANDLED_CLASS     (ACTION, ACMAPLEGENDITEMDBOBJECT)
  UNHANDLED_CLASS     (ACTION, ACMAPMAPVIEWPORTDBOBJECT)
  UNHANDLED_CLASS     (ACTION, ACMAPPRINTLAYOUTELEMENTDBOBJECTCONTAINER)
  UNHANDLED_CLASS     (ACTION, ACMBALLOON)
  UNHANDLED_CLASS     (ACTION, ACMBOM)
  UNHANDLED_CLASS     (ACTION, ACMBOMROW)
  UNHANDLED_CLASS     (ACTION, ACMBOMROWSTRUCT)
  UNHANDLED_CLASS     (ACTION, ACMBOMSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMBOMSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMBOMSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMBOMSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMCENTERLINESTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMCENTERLINESTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMCENTERLINESTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMCENTERLINESTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMDATADICTIONARY)
  UNHANDLED_CLASS     (ACTION, ACMDATAENTRY)
  UNHANDLED_CLASS     (ACTION, ACMDATAENTRYBLOCK)
  UNHANDLED_CLASS     (ACTION, ACMDATUMID)
  UNHANDLED_CLASS     (ACTION, ACMDATUMSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMDATUMSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMDATUMSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMDATUMSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMDATUMSTANDARDISO2012)
  UNHANDLED_CLASS     (ACTION, ACMDETAILSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMDETAILSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMDETAILSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMDETAILSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMDETAILTANDARDCUSTOM)
  UNHANDLED_CLASS     (ACTION, ACMDIMBREAKPERSREACTOR)
  UNHANDLED_CLASS     (ACTION, ACMEDRAWINGMAN)
  UNHANDLED_CLASS     (ACTION, ACMEVIEW)
  UNHANDLED_CLASS     (ACTION, ACME_DATABASE)
  UNHANDLED_CLASS     (ACTION, ACME_DOCUMENT)
  UNHANDLED_CLASS     (ACTION, ACMFCFRAME)
  UNHANDLED_CLASS     (ACTION, ACMFCFSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMFCFSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMFCFSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMFCFSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMFCFSTANDARDISO2004)
  UNHANDLED_CLASS     (ACTION, ACMFCFSTANDARDISO2012)
  UNHANDLED_CLASS     (ACTION, ACMIDSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMIDSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMIDSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMIDSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMIDSTANDARDISO2004)
  UNHANDLED_CLASS     (ACTION, ACMIDSTANDARDISO2012)
  UNHANDLED_CLASS     (ACTION, ACMNOTESTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMNOTESTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMNOTESTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMNOTESTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMPARTLIST)
  UNHANDLED_CLASS     (ACTION, ACMPICKOBJ)
  UNHANDLED_CLASS     (ACTION, ACMSECTIONSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMSECTIONSTANDARDCSN2002)
  UNHANDLED_CLASS     (ACTION, ACMSECTIONSTANDARDCUSTOM)
  UNHANDLED_CLASS     (ACTION, ACMSECTIONSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMSECTIONSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMSECTIONSTANDARDISO2001)
  UNHANDLED_CLASS     (ACTION, ACMSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMSURFSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMSURFSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMSURFSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMSURFSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMSURFSTANDARDISO2002)
  UNHANDLED_CLASS     (ACTION, ACMSURFSYM)
  UNHANDLED_CLASS     (ACTION, ACMTAPERSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMTAPERSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMTAPERSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMTAPERSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMTHREADLINESTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMTHREADLINESTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMTHREADLINESTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMTHREADLINESTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMWELDSTANDARDANSI)
  UNHANDLED_CLASS     (ACTION, ACMWELDSTANDARDCSN)
  UNHANDLED_CLASS     (ACTION, ACMWELDSTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, ACMWELDSTANDARDISO)
  UNHANDLED_CLASS     (ACTION, ACMWELDSYM)
  UNHANDLED_CLASS     (ACTION, ACRFATTGENMGR)
  UNHANDLED_CLASS     (ACTION, ACRFINSADJ)
  UNHANDLED_CLASS     (ACTION, ACRFINSADJUSTERMGR)
  UNHANDLED_CLASS     (ACTION, ACRFMCADAPIATTHOLDER)
  UNHANDLED_CLASS     (ACTION, ACRFOBJATTMGR)
  UNHANDLED_CLASS     (ACTION, ACSH_SUBENT_MATERIAL_CLASS)
  UNHANDLED_CLASS     (ACTION, AC_AM_2D_XREF_MGR)
  UNHANDLED_CLASS     (ACTION, AC_AM_BASIC_VIEW)
  UNHANDLED_CLASS     (ACTION, AC_AM_BASIC_VIEW_DEF)
  UNHANDLED_CLASS     (ACTION, AC_AM_COMPLEX_HIDE_SITUATION)
  UNHANDLED_CLASS     (ACTION, AC_AM_COMP_VIEW_DEF)
  UNHANDLED_CLASS     (ACTION, AC_AM_COMP_VIEW_INST)
  UNHANDLED_CLASS     (ACTION, AC_AM_DIRTY_NODES)
  UNHANDLED_CLASS     (ACTION, AC_AM_HIDE_SITUATION)
  UNHANDLED_CLASS     (ACTION, AC_AM_MAPPER_CACHE)
  UNHANDLED_CLASS     (ACTION, AC_AM_MASTER_VIEW_DEF)
  UNHANDLED_CLASS     (ACTION, AC_AM_MVD_DEP_MGR)
  UNHANDLED_CLASS     (ACTION, AC_AM_OVERRIDE_FILTER)
  UNHANDLED_CLASS     (ACTION, AC_AM_PROPS_OVERRIDE)
  UNHANDLED_CLASS     (ACTION, AC_AM_SHAFT_HIDE_SITUATION)
  UNHANDLED_CLASS     (ACTION, AC_AM_STDP_VIEW_DEF)
  UNHANDLED_CLASS     (ACTION, AC_AM_TRANSFORM_GHOST)
  UNHANDLED_CLASS     (ACTION, ADAPPL)
  UNHANDLED_CLASS     (ACTION, AECC_ALIGNMENT_DESIGN_CHECK_SET)
  UNHANDLED_CLASS     (ACTION, AECC_ALIGNMENT_LABEL_SET)
  UNHANDLED_CLASS     (ACTION, AECC_ALIGNMENT_LABEL_SET_EXT)
  UNHANDLED_CLASS     (ACTION, AECC_ALIGNMENT_PARCEL_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_ALIGNMENT_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_APPURTENANCE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_ASSEMBLY_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_BUILDING_SITE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_CANT_DIAGRAM_VIEW_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_CATCHMENT_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_CLASS_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_CONTOURVIEW)
  UNHANDLED_CLASS     (ACTION, AECC_CORRIDOR_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_CANT_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_CURVE_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_DESIGNSPEED_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_GEOMPT_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_INDEXED_PI_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_MINOR_STATION_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_PI_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_SPIRAL_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_STAEQU_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_STATION_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_STATION_OFFSET_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_SUPERELEVATION_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_TANGENT_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ALIGNMENT_VERTICAL_GEOMPT_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_APPURTENANCE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_APPURTENANCE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_APPURTENANCE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_APPURTENANCE_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_ASSEMBLY)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_PROFILE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_SECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_AUTO_FEATURE_LINE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_AUTO_FEATURE_LINE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_AUTO_FEATURE_LINE_PROFILE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_AUTO_FEATURE_LINE_SECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_BUILDINGSITE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_BUILDINGUTIL_CONNECTOR)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_CANT_DIAGRAM_VIEW)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_CATCHMENT_AREA)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_CATCHMENT_AREA_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_CORRIDOR)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_CROSSING_PIPE_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_CROSSING_PRESSURE_PIPE_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_CSVSTATIONSLIDER)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FACE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FEATURE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FEATURE_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FEATURE_LINE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FEATURE_LINE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FEATURE_LINE_PROFILE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FEATURE_LINE_SECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FITTING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FITTING_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FITTING_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FITTING_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_FLOW_SEGMENT_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_GENERAL_SEGMENT_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_GRADING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_GRAPH)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_GRAPHPROFILE_NETWORKPART)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_GRAPHPROFILE_PRESSUREPART)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_GRID_SURFACE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_GRID_SURFACE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_HORGEOMETRY_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_HYDRO_REGION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_INTERFERENCE_CHECK)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_INTERFERENCE_PART)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_INTERFERENCE_PART_SECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_INTERSECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_INTERSECTION_LOCATION_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_LEGEND_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_LINE_BETWEEN_POINTS_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_LOTLINE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_MASSHAULLINE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_MASS_HAUL_VIEW)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_MATCHLINE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_MATCH_LINE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_MATERIAL_SECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_NETWORK)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_NOTE_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_OFFSET_ELEV_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PARCEL_BOUNDARY)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PARCEL_FACE_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PARCEL_SEGMENT)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PARCEL_SEGMENT_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PARCEL_SEGMENT_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PARCEL_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PIPE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PIPENETWORK_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PIPE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PIPE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PIPE_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PIPE_SECTION_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PIPE_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_POINT_ENT)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_POINT_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_POINT_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PRESSUREPIPENETWORK)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PRESSURE_PART_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PRESSURE_PIPE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PRESSURE_PIPE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PRESSURE_PIPE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PRESSURE_PIPE_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PRESSURE_PIPE_SECTION_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PROFILE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PROFILEDATA_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PROFILE_PROJECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PROFILE_PROJECTION_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PROFILE_VIEW)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_PROFILE_VIEW_DEPTH_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_QUANTITY_TAKEOFF_AGGREGATE_EARTHWORK_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_RIGHT_OF_WAY)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SAMPLELINE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SAMPLE_LINE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SAMPLE_LINE_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTIONALDATA_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTIONDATA_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTIONSEGMENT_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_CORRIDOR)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_CORRIDOR_POINT_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_GRADEBREAK_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_MINOR_OFFSET_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_OFFSET_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_PIPENETWORK)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_PRESSUREPIPENETWORK)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_PROJECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_PROJECTION_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_SEGMENT_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_VIEW)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_VIEW_DEPTH_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SECTION_VIEW_QUANTITY_TAKEOFF_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SHEET)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SPANNING_PIPE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SPANNING_PIPE_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_STATION_ELEV_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_STRUCTURE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_STRUCTURE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_STRUCTURE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_STRUCTURE_PROFILE_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_STRUCTURE_SECTION_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SUBASSEMBLY)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SUPERELEVATION_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SUPERELEVATION_DIAGRAM_VIEW)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SURFACE_CONTOUR_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SURFACE_ELEVATION_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SURFACE_SLOPE_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SURVEY_FIGURE_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SVFIGURE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SVFIGURE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SVFIGURE_PROFILE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SVFIGURE_SECTION)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SVFIGURE_SEGMENT_LABEL)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_SVNETWORK)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_TANGENT_INTERSECTION_TABLE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_TIN_SURFACE)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_TIN_SURFACE_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_CRESTCURVE_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_CSV)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_HAGEOMPT_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_LINE_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_MINOR_STATION_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_PVI_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_SAGCURVE_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VALIGNMENT_STATION_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VERTICALGEOMETRY_BAND_LABEL_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VIEWFRAME_LABELING)
  UNHANDLED_CLASS     (ACTION, AECC_DISP_REP_VIEW_FRAME)
  UNHANDLED_CLASS     (ACTION, AECC_FEATURELINE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_FEATURE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_FITTING_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_FORMAT_MANAGER_OBJECT)
  UNHANDLED_CLASS     (ACTION, AECC_GRADEVIEW)
  UNHANDLED_CLASS     (ACTION, AECC_GRADING_CRITERIA)
  UNHANDLED_CLASS     (ACTION, AECC_GRADING_CRITERIA_SET)
  UNHANDLED_CLASS     (ACTION, AECC_GRADING_GROUP)
  UNHANDLED_CLASS     (ACTION, AECC_GRADING_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_IMPORT_STORM_SEWER_DEFAULTS)
  UNHANDLED_CLASS     (ACTION, AECC_INTERFERENCE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_INTERSECTION_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_LABEL_COLLECTOR_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_LABEL_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_LABEL_RADIAL_LINE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_LABEL_TEXT_ITERATOR_CURVE_OR_SPIRAL_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_LABEL_TEXT_ITERATOR_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_LABEL_TEXT_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_LABEL_VECTOR_ARROW_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_LEGEND_TABLE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_MASS_HAUL_LINE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_MASS_HAUL_VIEW_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_MATCHLINE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_MATERIAL_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_NETWORK_PART_CATALOG_DEF_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_NETWORK_PART_FAMILY_ITEM)
  UNHANDLED_CLASS     (ACTION, AECC_NETWORK_PART_LIST)
  UNHANDLED_CLASS     (ACTION, AECC_NETWORK_RULE)
  UNHANDLED_CLASS     (ACTION, AECC_PARCEL_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_PARCEL_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PART_SIZE_FILTER)
  UNHANDLED_CLASS     (ACTION, AECC_PIPE_RULES)
  UNHANDLED_CLASS     (ACTION, AECC_PIPE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PIPE_STYLE_EXTENSION)
  UNHANDLED_CLASS     (ACTION, AECC_POINTCLOUD_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_POINTVIEW)
  UNHANDLED_CLASS     (ACTION, AECC_POINT_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PRESSURE_PART_LIST)
  UNHANDLED_CLASS     (ACTION, AECC_PRESSURE_PIPE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILESECTIONENTITY_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_DESIGN_CHECK_SET)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_LABEL_SET)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_BAND_STYLE_SET)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_DATA_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_HORIZONTAL_GEOMETRY_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_PIPE_NETWORK_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_SECTIONAL_DATA_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_SUPERELEVATION_DIAGRAM_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_PROFILE_VIEW_VERTICAL_GEOMETRY_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_QUANTITY_TAKEOFF_CRITERIA)
  UNHANDLED_CLASS     (ACTION, AECC_ROADWAYLINK_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_ROADWAYMARKER_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_ROADWAYSHAPE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_ROADWAY_STYLE_SET)
  UNHANDLED_CLASS     (ACTION, AECC_ROOT_SETTINGS_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_SAMPLE_LINE_GROUP_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SAMPLE_LINE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SECTION_LABEL_SET)
  UNHANDLED_CLASS     (ACTION, AECC_SECTION_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SECTION_VIEW_BAND_STYLE_SET)
  UNHANDLED_CLASS     (ACTION, AECC_SECTION_VIEW_DATA_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SECTION_VIEW_ROAD_SURFACE_BAND_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SECTION_VIEW_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SETTINGS_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_SHEET_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SLOPE_PATTERN_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_STATION_FORMAT_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_STRUCTURE_RULES)
  UNHANDLED_CLASS     (ACTION, AECC_STUCTURE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SUPERELEVATION_DIAGRAM_VIEW_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SURFACE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SVFIGURE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_SVNETWORK_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_TABLE_STYLE)
  UNHANDLED_CLASS     (ACTION, AECC_TAG_MANAGER)
  UNHANDLED_CLASS     (ACTION, AECC_TREE_NODE)
  UNHANDLED_CLASS     (ACTION, AECC_USER_DEFINED_ATTRIBUTE_CLASSIFICATION)
  UNHANDLED_CLASS     (ACTION, AECC_VALIGNMENT_STYLE_EXTENSION)
  UNHANDLED_CLASS     (ACTION, AECC_VIEW_FRAME_STYLE)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_PROPS_MEMBER)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_PROPS_MEMBER_LOGICAL)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_PROPS_MEMBER_PLAN)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_PROPS_MEMBER_PLAN_SKETCH)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_PROPS_MEMBER_PROJECTED)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_ELEVATION_DESIGN)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_ELEVATION_DETAIL)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_LOGICAL)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_MODEL_DESIGN)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_MODEL_DETAIL)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_PLAN_DESIGN)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_PLAN_DETAIL)
  UNHANDLED_CLASS     (ACTION, AECS_DISP_REP_MEMBER_PLAN_SKETCH)
  UNHANDLED_CLASS     (ACTION, AECS_MEMBER_NODE_SHAPE)
  UNHANDLED_CLASS     (ACTION, AECS_MEMBER_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_2DSECTION_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPBDGELEVLINEPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPBDGELEVLINEPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPBDGSECTIONLINEPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPBDGSECTIONLINEPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCEILINGGRIDPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCEILINGGRIDPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCOLUMNGRIDPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCOLUMNGRIDPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCURTAINWALLUNITPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPCURTAINWALLUNITPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPMVBLOCKREFPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPMVBLOCKREFPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPROOFPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPROOFPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPROOFSLABPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPROOFSLABPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPSLABPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPSLABPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPSPACEPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPSPACEPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPWALLPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPWALLPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPWINDOWASSEMBLYPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPWINDOWASSEMBLYPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPZONE100)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBDISPREPZONE50)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBZONEDEF)
  UNHANDLED_CLASS     (ACTION, AEC_AECDBZONESTYLE)
  UNHANDLED_CLASS     (ACTION, AEC_ANCHOR_OPENINGBASE_TO_WALL)
  UNHANDLED_CLASS     (ACTION, AEC_CLASSIFICATION_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_CLASSIFICATION_SYSTEM_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_CLEANUP_GROUP_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_CURTAIN_WALL_LAYOUT_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_CURTAIN_WALL_UNIT_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_CVSECTIONVIEW)
  UNHANDLED_CLASS     (ACTION, AEC_DB_DISP_REP_DIM_GROUP_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DB_DISP_REP_DIM_GROUP_PLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_DB_DISP_REP_DIM_GROUP_PLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_DIM_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_DISPLAYTHEME_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_2D_SECTION)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_CLIP_VOLUME)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_CLIP_VOLUME_RESULT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DIM)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DISPLAYTHEME)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DOOR)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DOOR_NOMINAL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DOOR_PLAN_100)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DOOR_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DOOR_THRESHOLD_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_DOOR_THRESHOLD_SYMBOL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_EDITINPLACEPROFILE_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_ENT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_ENT_REF)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_GRID_ASSEMBLY_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_GRID_ASSEMBLY_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_LAYOUT_CURVE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_LAYOUT_GRID2D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_LAYOUT_GRID3D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_MASKBLOCK)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_MASS_ELEM_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_MASS_GROUP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_MATERIAL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_OPENING)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_POLYGON_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_POLYGON_TRUECOLOUR)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_RAILING_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_RAILING_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_ROOF)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_ROOFSLAB)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_ROOFSLAB_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_SCHEDULE_TABLE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_SLAB)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_SLAB_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_SLICE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_SPACE_DECOMPOSED)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_SPACE_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_SPACE_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_STAIR_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_STAIR_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_STAIR_PLAN_OVERLAPPING)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WALL_GRAPH)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WALL_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WALL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WALL_SCHEM)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WINDOW)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WINDOW_ASSEMBLY_SILL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WINDOW_NOMINAL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WINDOW_PLAN_100)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WINDOW_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_WINDOW_SILL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_PROPS_ZONE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_2D_SECTION)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPMASSELEMPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPMASSELEMPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPMASSGROUPPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPMASSGROUPPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPOPENINGPLAN100)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPOPENINGPLAN50)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPOPENINGPLANREFLECTED)
  UNHANDLED_CLASS     (ACTION, AEC_DISPREPAECDBDISPREPOPENINGSILLPLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ANCHOR)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ANCHOR_BUB_TO_GRID)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ANCHOR_BUB_TO_GRID_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ANCHOR_BUB_TO_GRID_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ANCHOR_ENT_TO_NODE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ANCHOR_EXT_TAG_TO_ENT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ANCHOR_TAG_TO_ENT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_ELEVLINE_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_ELEVLINE_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_ELEVLINE_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_SECTIONLINE_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_SECTIONLINE_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_SECTIONLINE_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_SECTION_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_BDG_SECTION_SUBDIV)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CEILING_GRID)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CEILING_GRID_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CEILING_GRID_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CLIP_VOLUME_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CLIP_VOLUME_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CLIP_VOLUME_RESULT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CLIP_VOLUME_RESULT_SUBDIV)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_BLOCK)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CIRCARC2D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONCOINCIDENT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONCONCENTRIC)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONEQUALDISTANCE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONMIDPOINT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONNECTOR)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONNORMAL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONPARALLEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONPERPENDICULAR)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONSYMMETRIC)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_CONTANGENT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_DIMANGLE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_DIMDIAMETER)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_DIMDISTANCE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_DIMLENGTH)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_DIMMAJORRADIUS)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_DIMMINORRADIUS)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_ELLIPARC2D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_LAYOUTDATA)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_LINE2D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_ADD)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_CUTPLANE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_EXTRUSION)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_GROUP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_LOFT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_PATH)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_REVOLVE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_SUBTRACT)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_MODIFIER_TRANSITION)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_POINT2D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_PROFILE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COLUMN_GRID)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COLUMN_GRID_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COLUMN_GRID_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_WORKPLANE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_COL_WORKPLANE_REF)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CONFIG)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CURTAIN_WALL_LAYOUT_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CURTAIN_WALL_LAYOUT_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CURTAIN_WALL_UNIT_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_CURTAIN_WALL_UNIT_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DCM_DIMRADIUS)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DISPLAYTHEME)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_ELEV)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_NOMINAL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_PLAN_HEKTO)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_THRESHOLD_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_DOOR_THRESHOLD_SYMBOL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_EDITINPLACEPROFILE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ENT_REF)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_LAYOUT_CURVE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_LAYOUT_GRID2D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_LAYOUT_GRID3D)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASKBLOCK_REF)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASKBLOCK_REF_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASS_ELEM_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASS_ELEM_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASS_ELEM_SCHEM)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASS_GROUP_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASS_GROUP_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MASS_GROUP_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MATERIAL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MVBLOCK_REF)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MVBLOCK_REF_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_MVBLOCK_REF_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_OPENING)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_OPENING_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_POLYGON_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_POLYGON_TRUECOLOUR)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_RAILING_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_RAILING_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_RAILING_PLAN_100)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_RAILING_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ROOF_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ROOF_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ROOF_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ROOFSLAB_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ROOFSLAB_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SCHEDULE_TABLE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SET)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SLAB_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SLAB_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SLICE)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SPACE_DECOMPOSED)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SPACE_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SPACE_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SPACE_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_SPACE_VOLUME)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_STAIR_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_STAIR_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_STAIR_PLAN_100)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_STAIR_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_STAIR_PLAN_OVERLAPPING)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_STAIR_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WALL_GRAPH)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WALL_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WALL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WALL_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WALL_SCHEM)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_ASSEMBLY_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_ASSEMBLY_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOWASSEMBLY_SILL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_ELEV)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_MODEL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_NOMINAL)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_PLAN_100)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_RCP)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_WINDOW_SILL_PLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_REP_ZONE)
  UNHANDLED_CLASS     (ACTION, AEC_DISPROPSMASSELEMPLANCOMMON)
  UNHANDLED_CLASS     (ACTION, AEC_DISPROPSMASSGROUPPLANCOMMON)
  UNHANDLED_CLASS     (ACTION, AEC_DISPROPSOPENINGPLANCOMMON)
  UNHANDLED_CLASS     (ACTION, AEC_DISPROPSOPENINGPLANCOMMONHATCHED)
  UNHANDLED_CLASS     (ACTION, AEC_DISPROPSOPENINGSILLPLAN)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_ROPS_RAILING_PLAN_100)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_ROPS_RAILING_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_ROPS_STAIR_PLAN_100)
  UNHANDLED_CLASS     (ACTION, AEC_DISP_ROPS_STAIR_PLAN_50)
  UNHANDLED_CLASS     (ACTION, AEC_DOOR_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_ENDCAP_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_FRAME_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_LAYERKEY_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_LIST_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_MASKBLOCK_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_MASS_ELEM_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_MATERIAL_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_MVBLOCK_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_MVBLOCK_REF)
  UNHANDLED_CLASS     (ACTION, AEC_NOTIFICATION_TRACKER)
  UNHANDLED_CLASS     (ACTION, AEC_POLYGON)
  UNHANDLED_CLASS     (ACTION, AEC_POLYGON_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_PROPERTY_SET_DEF)
  UNHANDLED_CLASS     (ACTION, AEC_RAILING_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_REFEDIT_STATUS_TRACKER) // AecDbRefEditStatusTracker / AecArchBase80|...)
  UNHANDLED_CLASS     (ACTION, AEC_ROOFSLABEDGE_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_ROOFSLAB_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_SCHEDULE_DATA_FORMAT)
  UNHANDLED_CLASS     (ACTION, AEC_SLABEDGE_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_SLAB_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_SPACE_STYLES)
  UNHANDLED_CLASS     (ACTION, AEC_STAIR_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_STAIR_WINDER_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_STAIR_WINDER_TYPE_BALANCED)
  UNHANDLED_CLASS     (ACTION, AEC_STAIR_WINDER_TYPE_MANUAL)
  UNHANDLED_CLASS     (ACTION, AEC_STAIR_WINDER_TYPE_SINGLE_POINT)
  UNHANDLED_CLASS     (ACTION, AEC_VARS_AECBBLDSRV)
  UNHANDLED_CLASS     (ACTION, AEC_VARS_ARCHBASE)
  UNHANDLED_CLASS     (ACTION, AEC_VARS_DWG_SETUP)
  UNHANDLED_CLASS     (ACTION, AEC_VARS_MUNICH)
  UNHANDLED_CLASS     (ACTION, AEC_VARS_STRUCTUREBASE)
  UNHANDLED_CLASS     (ACTION, AEC_WALLMOD_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_WALL_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_WINDOW_ASSEMBLY_STYLE)
  UNHANDLED_CLASS     (ACTION, AEC_WINDOW_STYLE)
  UNHANDLED_CLASS     (ACTION, ALIGNMENTGRIPENTITY)
  UNHANDLED_CLASS     (ACTION, AMCONTEXTMGR)
  UNHANDLED_CLASS     (ACTION, AMDTADMENUSTATE)
  UNHANDLED_CLASS     (ACTION, AMDTAMMENUSTATE)
  UNHANDLED_CLASS     (ACTION, AMDTBROWSERDBTAB)
  UNHANDLED_CLASS     (ACTION, AMDTDMMENUSTATE)
  UNHANDLED_CLASS     (ACTION, AMDTEDGESTANDARDDIN)
  UNHANDLED_CLASS     (ACTION, AMDTEDGESTANDARDDIN13715)
  UNHANDLED_CLASS     (ACTION, AMDTEDGESTANDARDISO)
  UNHANDLED_CLASS     (ACTION, AMDTEDGESTANDARDISO13715)
  UNHANDLED_CLASS     (ACTION, AMDTFORMULAUPDATEDISPATCHER)
  UNHANDLED_CLASS     (ACTION, AMDTINTERNALREACTOR)
  UNHANDLED_CLASS     (ACTION, AMDTMCOMMENUSTATE)
  UNHANDLED_CLASS     (ACTION, AMDTMENUSTATEMGR)
  UNHANDLED_CLASS     (ACTION, AMDTNOTE)
  UNHANDLED_CLASS     (ACTION, AMDTNOTETEMPLATEDB)
  UNHANDLED_CLASS     (ACTION, AMDTSECTIONSYM)
  UNHANDLED_CLASS     (ACTION, AMDTSECTIONSYMLABEL)
  UNHANDLED_CLASS     (ACTION, AMDTSYSATTR)
  UNHANDLED_CLASS     (ACTION, AMGOBJPROPCFG)
  UNHANDLED_CLASS     (ACTION, AMGSETTINGSOBJ)
  UNHANDLED_CLASS     (ACTION, AMIMASTER)
  UNHANDLED_CLASS     (ACTION, AM_DRAWING_MGR)
  UNHANDLED_CLASS     (ACTION, AM_DWGMGR_NAME)
  UNHANDLED_CLASS     (ACTION, AM_DWG_DOCUMENT)
  UNHANDLED_CLASS     (ACTION, AM_DWG_SHEET)
  UNHANDLED_CLASS     (ACTION, AM_VIEWDIMPARMAP)
  UNHANDLED_CLASS     (ACTION, BINRECORD)
  UNHANDLED_CLASS     (ACTION, CAMSCATALOGAPPOBJECT)
  UNHANDLED_CLASS     (ACTION, CAMSSTRUCTBTNSTATE)
  UNHANDLED_CLASS     (ACTION, CATALOGSTATE)
  UNHANDLED_CLASS     (ACTION, CBROWSERAPPOBJECT)
  UNHANDLED_CLASS     (ACTION, DEPMGR) // AcAdDepMgr / DesignerAssemblies)
  UNHANDLED_CLASS     (ACTION, DMBASEELEMENT) // AcMapDMBaseElement / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMDEFAULTSTYLE) // AcMapDMDefaultStyle / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMLEGEND)
  UNHANDLED_CLASS     (ACTION, DMMAP) // AcMapDMMap / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMMAPMANAGER) // AcMapDMMapManager / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMSTYLECATEGORY) // AcMapDMStyleCategory / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMSTYLELIBRARY) // AcMapDMStyleLibrary / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMSTYLEREFERENCE) // AcMapDMStyleReference / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMSTYLIZEDENTITIESTABLE) // AcMapDMStylizedEntitiesTable / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DMSURROGATESTYLESETS) // AcMapDMSurrogateStyleSets / AcMapDisplayManagement)
  UNHANDLED_CLASS     (ACTION, DM_PLACEHOLDER) // AcDmSelsetPlaceholder / McadcomServices)
  UNHANDLED_CLASS     (ACTION, EXACTERMXREFMAP) // ExAcTermXrefMap / EXAC_SERVICE)
  UNHANDLED_CLASS     (ACTION, EXACXREFPANELOBJECT) // ExAcXREFPanelObject / EXAC_ESW)
  UNHANDLED_CLASS     (ACTION, EXPO_NOTIFYBLOCK)
  UNHANDLED_CLASS     (ACTION, EXPO_NOTIFYHALL)
  UNHANDLED_CLASS     (ACTION, EXPO_NOTIFYPILLAR)
  UNHANDLED_CLASS     (ACTION, EXPO_NOTIFYSTAND)
  UNHANDLED_CLASS     (ACTION, EXPO_NOTIFYSTANDNOPOLY)
  UNHANDLED_CLASS     (ACTION, GSMANAGER) // AcMeGSManager / AcMeServices
  UNHANDLED_CLASS     (ACTION, IRD_DSC_DICT)
  UNHANDLED_CLASS     (ACTION, IRD_DSC_RECORD)
  UNHANDLED_CLASS     (ACTION, IRD_OBJ_RECORD)
  UNHANDLED_CLASS     (ACTION, MAPFSMRVOBJECT)
  UNHANDLED_CLASS_CPP (ACTION, MAPGWSUNDOOBJECT, AcMapGWSUndoRecorder) // AcMapFeatureSourceManager
  UNHANDLED_CLASS     (ACTION, MAPIAMMOUDLE)
  UNHANDLED_CLASS     (ACTION, MAPMETADATAOBJECT)
  UNHANDLED_CLASS     (ACTION, MAPRESOURCEMANAGEROBJECT)
  UNHANDLED_CLASS     (ACTION, McDbContainer2)
  UNHANDLED_CLASS     (ACTION, McDbMarker)
  UNHANDLED_CLASS     (ACTION, NAMEDAPPL)
  UNHANDLED_CLASS     (ACTION, NEWSTDPARTPARLIST)
  UNHANDLED_CLASS     (ACTION, NOTEPOSITION / mcsDbObjectNotePosition)
  UNHANDLED_CLASS     (ACTION, OBJCLONER)
  UNHANDLED_CLASS     (ACTION, PARAMMGR)
  UNHANDLED_CLASS     (ACTION, PARAMSCOPE)
  UNHANDLED_CLASS     (ACTION, PILLAR)
  UNHANDLED_CLASS     (ACTION, STDPART2D)
  UNHANDLED_CLASS     (ACTION, TCH_ARROW)
  UNHANDLED_CLASS     (ACTION, TCH_AXIS_LABEL)
  UNHANDLED_CLASS     (ACTION, TCH_BLOCK_INSERT)
  UNHANDLED_CLASS     (ACTION, TCH_COLUMN)
  UNHANDLED_CLASS     (ACTION, TCH_DBCONFIG)
  UNHANDLED_CLASS     (ACTION, TCH_DIMENSION2)
  UNHANDLED_CLASS     (ACTION, TCH_DRAWINGINDEX)
  UNHANDLED_CLASS     (ACTION, TCH_HANDRAIL)
  UNHANDLED_CLASS     (ACTION, TCH_LINESTAIR)
  UNHANDLED_CLASS     (ACTION, TCH_OPENING)
  UNHANDLED_CLASS     (ACTION, TCH_RECTSTAIR)
  UNHANDLED_CLASS     (ACTION, TCH_SLAB)
  UNHANDLED_CLASS     (ACTION, TCH_SPACE)
  UNHANDLED_CLASS     (ACTION, TCH_TEXT)
  UNHANDLED_CLASS     (ACTION, TCH_WALL)
  UNHANDLED_CLASS     (ACTION, TGrupoPuntos)
  UNHANDLED_CLASS     (ACTION, VAACIMAGEINVENTORY)
  UNHANDLED_CLASS     (ACTION, VAACXREFPANELOBJECT)
