/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
// Copyright (C) 2023,2024 Free Software Foundation, Inc.
// This file is part of LibreDWG.
// Created from scratch by Reini Urban
// Generated by txt2h.pl.

#include <stdint.h>

/* unsorted rhs values */
static const uint16_t cptbl_windows_1257[] = {
  /*[0x0] = */255,	// size of vector
  /*[0x1] = */1,
  /*[0x2] = */2,
  /*[0x3] = */3,
  /*[0x4] = */4,
  /*[0x5] = */5,
  /*[0x6] = */6,
  /*[0x7] = */7,
  /*[0x8] = */8,
  /*[0x9] = */9,
  /*[0xa] = */10,
  /*[0xb] = */11,
  /*[0xc] = */12,
  /*[0xd] = */13,
  /*[0xe] = */14,
  /*[0xf] = */15,
  /*[0x10] = */16,
  /*[0x11] = */17,
  /*[0x12] = */18,
  /*[0x13] = */19,
  /*[0x14] = */20,
  /*[0x15] = */21,
  /*[0x16] = */22,
  /*[0x17] = */23,
  /*[0x18] = */24,
  /*[0x19] = */25,
  /*[0x1a] = */26,
  /*[0x1b] = */27,
  /*[0x1c] = */28,
  /*[0x1d] = */29,
  /*[0x1e] = */30,
  /*[0x1f] = */31,
  /*[0x20] = */32,
  /*[0x21] = */33,
  /*[0x22] = */34,
  /*[0x23] = */35,
  /*[0x24] = */36,
  /*[0x25] = */37,
  /*[0x26] = */38,
  /*[0x27] = */39,
  /*[0x28] = */40,
  /*[0x29] = */41,
  /*[0x2a] = */42,
  /*[0x2b] = */43,
  /*[0x2c] = */44,
  /*[0x2d] = */45,
  /*[0x2e] = */46,
  /*[0x2f] = */47,
  /*[0x30] = */48,
  /*[0x31] = */49,
  /*[0x32] = */50,
  /*[0x33] = */51,
  /*[0x34] = */52,
  /*[0x35] = */53,
  /*[0x36] = */54,
  /*[0x37] = */55,
  /*[0x38] = */56,
  /*[0x39] = */57,
  /*[0x3a] = */58,
  /*[0x3b] = */59,
  /*[0x3c] = */60,
  /*[0x3d] = */61,
  /*[0x3e] = */62,
  /*[0x3f] = */63,
  /*[0x40] = */64,
  /*[0x41] = */65,
  /*[0x42] = */66,
  /*[0x43] = */67,
  /*[0x44] = */68,
  /*[0x45] = */69,
  /*[0x46] = */70,
  /*[0x47] = */71,
  /*[0x48] = */72,
  /*[0x49] = */73,
  /*[0x4a] = */74,
  /*[0x4b] = */75,
  /*[0x4c] = */76,
  /*[0x4d] = */77,
  /*[0x4e] = */78,
  /*[0x4f] = */79,
  /*[0x50] = */80,
  /*[0x51] = */81,
  /*[0x52] = */82,
  /*[0x53] = */83,
  /*[0x54] = */84,
  /*[0x55] = */85,
  /*[0x56] = */86,
  /*[0x57] = */87,
  /*[0x58] = */88,
  /*[0x59] = */89,
  /*[0x5a] = */90,
  /*[0x5b] = */91,
  /*[0x5c] = */92,
  /*[0x5d] = */93,
  /*[0x5e] = */94,
  /*[0x5f] = */95,
  /*[0x60] = */96,
  /*[0x61] = */97,
  /*[0x62] = */98,
  /*[0x63] = */99,
  /*[0x64] = */100,
  /*[0x65] = */101,
  /*[0x66] = */102,
  /*[0x67] = */103,
  /*[0x68] = */104,
  /*[0x69] = */105,
  /*[0x6a] = */106,
  /*[0x6b] = */107,
  /*[0x6c] = */108,
  /*[0x6d] = */109,
  /*[0x6e] = */110,
  /*[0x6f] = */111,
  /*[0x70] = */112,
  /*[0x71] = */113,
  /*[0x72] = */114,
  /*[0x73] = */115,
  /*[0x74] = */116,
  /*[0x75] = */117,
  /*[0x76] = */118,
  /*[0x77] = */119,
  /*[0x78] = */120,
  /*[0x79] = */121,
  /*[0x7a] = */122,
  /*[0x7b] = */123,
  /*[0x7c] = */124,
  /*[0x7d] = */125,
  /*[0x7e] = */126,
  /*[0x7f] = */0,
  /*[0x80] = */0x20AC,	// EURO SIGN
  /*[0x81] = */0,
  /*[0x82] = */0x201A,	// SINGLE LOW-9 QUOTATION MARK
  /*[0x83] = */0,
  /*[0x84] = */0x201E,	// DOUBLE LOW-9 QUOTATION MARK
  /*[0x85] = */0x2026,	// HORIZONTAL ELLIPSIS
  /*[0x86] = */0x2020,	// DAGGER
  /*[0x87] = */0x2021,	// DOUBLE DAGGER
  /*[0x88] = */0,
  /*[0x89] = */0x2030,	// PER MILLE SIGN
  /*[0x8a] = */0,
  /*[0x8B] = */0x2039,	// SINGLE LEFT-POINTING ANGLE QUOTATION MARK
  /*[0x8c] = */0,
  /*[0x8D] = */0x00A8,	// DIAERESIS
  /*[0x8E] = */0x02C7,	// CARON
  /*[0x8F] = */0x00B8,	// CEDILLA
  /*[0x90] = */0,
  /*[0x91] = */0x2018,	// LEFT SINGLE QUOTATION MARK
  /*[0x92] = */0x2019,	// RIGHT SINGLE QUOTATION MARK
  /*[0x93] = */0x201C,	// LEFT DOUBLE QUOTATION MARK
  /*[0x94] = */0x201D,	// RIGHT DOUBLE QUOTATION MARK
  /*[0x95] = */0x2022,	// BULLET
  /*[0x96] = */0x2013,	// EN DASH
  /*[0x97] = */0x2014,	// EM DASH
  /*[0x98] = */0,
  /*[0x99] = */0x2122,	// TRADE MARK SIGN
  /*[0x9a] = */0,
  /*[0x9B] = */0x203A,	// SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
  /*[0x9c] = */0,
  /*[0x9D] = */0x00AF,	// MACRON
  /*[0x9E] = */0x02DB,	// OGONEK
  /*[0x9f] = */0,
  /*[0xA0] = */0x00A0,	// NO-BREAK SPACE
  /*[0xa1] = */0,
  /*[0xA2] = */0x00A2,	// CENT SIGN
  /*[0xA3] = */0x00A3,	// POUND SIGN
  /*[0xA4] = */0x00A4,	// CURRENCY SIGN
  /*[0xa5] = */0,
  /*[0xA6] = */0x00A6,	// BROKEN BAR
  /*[0xA7] = */0x00A7,	// SECTION SIGN
  /*[0xA8] = */0x00D8,	// LATIN CAPITAL LETTER O WITH STROKE
  /*[0xA9] = */0x00A9,	// COPYRIGHT SIGN
  /*[0xAA] = */0x0156,	// LATIN CAPITAL LETTER R WITH CEDILLA
  /*[0xAB] = */0x00AB,	// LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
  /*[0xAC] = */0x00AC,	// NOT SIGN
  /*[0xAD] = */0x00AD,	// SOFT HYPHEN
  /*[0xAE] = */0x00AE,	// REGISTERED SIGN
  /*[0xAF] = */0x00C6,	// LATIN CAPITAL LETTER AE
  /*[0xB0] = */0x00B0,	// DEGREE SIGN
  /*[0xB1] = */0x00B1,	// PLUS-MINUS SIGN
  /*[0xB2] = */0x00B2,	// SUPERSCRIPT TWO
  /*[0xB3] = */0x00B3,	// SUPERSCRIPT THREE
  /*[0xB4] = */0x00B4,	// ACUTE ACCENT
  /*[0xB5] = */0x00B5,	// MICRO SIGN
  /*[0xB6] = */0x00B6,	// PILCROW SIGN
  /*[0xB7] = */0x00B7,	// MIDDLE DOT
  /*[0xB8] = */0x00F8,	// LATIN SMALL LETTER O WITH STROKE
  /*[0xB9] = */0x00B9,	// SUPERSCRIPT ONE
  /*[0xBA] = */0x0157,	// LATIN SMALL LETTER R WITH CEDILLA
  /*[0xBB] = */0x00BB,	// RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
  /*[0xBC] = */0x00BC,	// VULGAR FRACTION ONE QUARTER
  /*[0xBD] = */0x00BD,	// VULGAR FRACTION ONE HALF
  /*[0xBE] = */0x00BE,	// VULGAR FRACTION THREE QUARTERS
  /*[0xBF] = */0x00E6,	// LATIN SMALL LETTER AE
  /*[0xC0] = */0x0104,	// LATIN CAPITAL LETTER A WITH OGONEK
  /*[0xC1] = */0x012E,	// LATIN CAPITAL LETTER I WITH OGONEK
  /*[0xC2] = */0x0100,	// LATIN CAPITAL LETTER A WITH MACRON
  /*[0xC3] = */0x0106,	// LATIN CAPITAL LETTER C WITH ACUTE
  /*[0xC4] = */0x00C4,	// LATIN CAPITAL LETTER A WITH DIAERESIS
  /*[0xC5] = */0x00C5,	// LATIN CAPITAL LETTER A WITH RING ABOVE
  /*[0xC6] = */0x0118,	// LATIN CAPITAL LETTER E WITH OGONEK
  /*[0xC7] = */0x0112,	// LATIN CAPITAL LETTER E WITH MACRON
  /*[0xC8] = */0x010C,	// LATIN CAPITAL LETTER C WITH CARON
  /*[0xC9] = */0x00C9,	// LATIN CAPITAL LETTER E WITH ACUTE
  /*[0xCA] = */0x0179,	// LATIN CAPITAL LETTER Z WITH ACUTE
  /*[0xCB] = */0x0116,	// LATIN CAPITAL LETTER E WITH DOT ABOVE
  /*[0xCC] = */0x0122,	// LATIN CAPITAL LETTER G WITH CEDILLA
  /*[0xCD] = */0x0136,	// LATIN CAPITAL LETTER K WITH CEDILLA
  /*[0xCE] = */0x012A,	// LATIN CAPITAL LETTER I WITH MACRON
  /*[0xCF] = */0x013B,	// LATIN CAPITAL LETTER L WITH CEDILLA
  /*[0xD0] = */0x0160,	// LATIN CAPITAL LETTER S WITH CARON
  /*[0xD1] = */0x0143,	// LATIN CAPITAL LETTER N WITH ACUTE
  /*[0xD2] = */0x0145,	// LATIN CAPITAL LETTER N WITH CEDILLA
  /*[0xD3] = */0x00D3,	// LATIN CAPITAL LETTER O WITH ACUTE
  /*[0xD4] = */0x014C,	// LATIN CAPITAL LETTER O WITH MACRON
  /*[0xD5] = */0x00D5,	// LATIN CAPITAL LETTER O WITH TILDE
  /*[0xD6] = */0x00D6,	// LATIN CAPITAL LETTER O WITH DIAERESIS
  /*[0xD7] = */0x00D7,	// MULTIPLICATION SIGN
  /*[0xD8] = */0x0172,	// LATIN CAPITAL LETTER U WITH OGONEK
  /*[0xD9] = */0x0141,	// LATIN CAPITAL LETTER L WITH STROKE
  /*[0xDA] = */0x015A,	// LATIN CAPITAL LETTER S WITH ACUTE
  /*[0xDB] = */0x016A,	// LATIN CAPITAL LETTER U WITH MACRON
  /*[0xDC] = */0x00DC,	// LATIN CAPITAL LETTER U WITH DIAERESIS
  /*[0xDD] = */0x017B,	// LATIN CAPITAL LETTER Z WITH DOT ABOVE
  /*[0xDE] = */0x017D,	// LATIN CAPITAL LETTER Z WITH CARON
  /*[0xDF] = */0x00DF,	// LATIN SMALL LETTER SHARP S
  /*[0xE0] = */0x0105,	// LATIN SMALL LETTER A WITH OGONEK
  /*[0xE1] = */0x012F,	// LATIN SMALL LETTER I WITH OGONEK
  /*[0xE2] = */0x0101,	// LATIN SMALL LETTER A WITH MACRON
  /*[0xE3] = */0x0107,	// LATIN SMALL LETTER C WITH ACUTE
  /*[0xE4] = */0x00E4,	// LATIN SMALL LETTER A WITH DIAERESIS
  /*[0xE5] = */0x00E5,	// LATIN SMALL LETTER A WITH RING ABOVE
  /*[0xE6] = */0x0119,	// LATIN SMALL LETTER E WITH OGONEK
  /*[0xE7] = */0x0113,	// LATIN SMALL LETTER E WITH MACRON
  /*[0xE8] = */0x010D,	// LATIN SMALL LETTER C WITH CARON
  /*[0xE9] = */0x00E9,	// LATIN SMALL LETTER E WITH ACUTE
  /*[0xEA] = */0x017A,	// LATIN SMALL LETTER Z WITH ACUTE
  /*[0xEB] = */0x0117,	// LATIN SMALL LETTER E WITH DOT ABOVE
  /*[0xEC] = */0x0123,	// LATIN SMALL LETTER G WITH CEDILLA
  /*[0xED] = */0x0137,	// LATIN SMALL LETTER K WITH CEDILLA
  /*[0xEE] = */0x012B,	// LATIN SMALL LETTER I WITH MACRON
  /*[0xEF] = */0x013C,	// LATIN SMALL LETTER L WITH CEDILLA
  /*[0xF0] = */0x0161,	// LATIN SMALL LETTER S WITH CARON
  /*[0xF1] = */0x0144,	// LATIN SMALL LETTER N WITH ACUTE
  /*[0xF2] = */0x0146,	// LATIN SMALL LETTER N WITH CEDILLA
  /*[0xF3] = */0x00F3,	// LATIN SMALL LETTER O WITH ACUTE
  /*[0xF4] = */0x014D,	// LATIN SMALL LETTER O WITH MACRON
  /*[0xF5] = */0x00F5,	// LATIN SMALL LETTER O WITH TILDE
  /*[0xF6] = */0x00F6,	// LATIN SMALL LETTER O WITH DIAERESIS
  /*[0xF7] = */0x00F7,	// DIVISION SIGN
  /*[0xF8] = */0x0173,	// LATIN SMALL LETTER U WITH OGONEK
  /*[0xF9] = */0x0142,	// LATIN SMALL LETTER L WITH STROKE
  /*[0xFA] = */0x015B,	// LATIN SMALL LETTER S WITH ACUTE
  /*[0xFB] = */0x016B,	// LATIN SMALL LETTER U WITH MACRON
  /*[0xFC] = */0x00FC,	// LATIN SMALL LETTER U WITH DIAERESIS
  /*[0xFD] = */0x017C,	// LATIN SMALL LETTER Z WITH DOT ABOVE
  /*[0xFE] = */0x017E,	// LATIN SMALL LETTER Z WITH CARON
  /*[0xFF] = */0x02D9,	// DOT ABOVE
};
/* sorted, without the first 127 characters */
static const uint8_t cptbl_alnum_windows_1257[] = {
  67,		// size of vector
  0xA8,	// LATIN CAPITAL LETTER O WITH STROKE
  0xAA,	// LATIN CAPITAL LETTER R WITH CEDILLA
  0xAF,	// LATIN CAPITAL LETTER AE
  0xB8,	// LATIN SMALL LETTER O WITH STROKE
  0xBA,	// LATIN SMALL LETTER R WITH CEDILLA
  0xBF,	// LATIN SMALL LETTER AE
  0xC0,	// LATIN CAPITAL LETTER A WITH OGONEK
  0xC1,	// LATIN CAPITAL LETTER I WITH OGONEK
  0xC2,	// LATIN CAPITAL LETTER A WITH MACRON
  0xC3,	// LATIN CAPITAL LETTER C WITH ACUTE
  0xC4,	// LATIN CAPITAL LETTER A WITH DIAERESIS
  0xC5,	// LATIN CAPITAL LETTER A WITH RING ABOVE
  0xC6,	// LATIN CAPITAL LETTER E WITH OGONEK
  0xC7,	// LATIN CAPITAL LETTER E WITH MACRON
  0xC8,	// LATIN CAPITAL LETTER C WITH CARON
  0xC9,	// LATIN CAPITAL LETTER E WITH ACUTE
  0xCA,	// LATIN CAPITAL LETTER Z WITH ACUTE
  0xCB,	// LATIN CAPITAL LETTER E WITH DOT ABOVE
  0xCC,	// LATIN CAPITAL LETTER G WITH CEDILLA
  0xCD,	// LATIN CAPITAL LETTER K WITH CEDILLA
  0xCE,	// LATIN CAPITAL LETTER I WITH MACRON
  0xCF,	// LATIN CAPITAL LETTER L WITH CEDILLA
  0xD0,	// LATIN CAPITAL LETTER S WITH CARON
  0xD1,	// LATIN CAPITAL LETTER N WITH ACUTE
  0xD2,	// LATIN CAPITAL LETTER N WITH CEDILLA
  0xD3,	// LATIN CAPITAL LETTER O WITH ACUTE
  0xD4,	// LATIN CAPITAL LETTER O WITH MACRON
  0xD5,	// LATIN CAPITAL LETTER O WITH TILDE
  0xD6,	// LATIN CAPITAL LETTER O WITH DIAERESIS
  0xD8,	// LATIN CAPITAL LETTER U WITH OGONEK
  0xD9,	// LATIN CAPITAL LETTER L WITH STROKE
  0xDA,	// LATIN CAPITAL LETTER S WITH ACUTE
  0xDB,	// LATIN CAPITAL LETTER U WITH MACRON
  0xDC,	// LATIN CAPITAL LETTER U WITH DIAERESIS
  0xDD,	// LATIN CAPITAL LETTER Z WITH DOT ABOVE
  0xDE,	// LATIN CAPITAL LETTER Z WITH CARON
  0xDF,	// LATIN SMALL LETTER SHARP S
  0xE0,	// LATIN SMALL LETTER A WITH OGONEK
  0xE1,	// LATIN SMALL LETTER I WITH OGONEK
  0xE2,	// LATIN SMALL LETTER A WITH MACRON
  0xE3,	// LATIN SMALL LETTER C WITH ACUTE
  0xE4,	// LATIN SMALL LETTER A WITH DIAERESIS
  0xE5,	// LATIN SMALL LETTER A WITH RING ABOVE
  0xE6,	// LATIN SMALL LETTER E WITH OGONEK
  0xE7,	// LATIN SMALL LETTER E WITH MACRON
  0xE8,	// LATIN SMALL LETTER C WITH CARON
  0xE9,	// LATIN SMALL LETTER E WITH ACUTE
  0xEA,	// LATIN SMALL LETTER Z WITH ACUTE
  0xEB,	// LATIN SMALL LETTER E WITH DOT ABOVE
  0xEC,	// LATIN SMALL LETTER G WITH CEDILLA
  0xED,	// LATIN SMALL LETTER K WITH CEDILLA
  0xEE,	// LATIN SMALL LETTER I WITH MACRON
  0xEF,	// LATIN SMALL LETTER L WITH CEDILLA
  0xF0,	// LATIN SMALL LETTER S WITH CARON
  0xF1,	// LATIN SMALL LETTER N WITH ACUTE
  0xF2,	// LATIN SMALL LETTER N WITH CEDILLA
  0xF3,	// LATIN SMALL LETTER O WITH ACUTE
  0xF4,	// LATIN SMALL LETTER O WITH MACRON
  0xF5,	// LATIN SMALL LETTER O WITH TILDE
  0xF6,	// LATIN SMALL LETTER O WITH DIAERESIS
  0xF8,	// LATIN SMALL LETTER U WITH OGONEK
  0xF9,	// LATIN SMALL LETTER L WITH STROKE
  0xFA,	// LATIN SMALL LETTER S WITH ACUTE
  0xFB,	// LATIN SMALL LETTER U WITH MACRON
  0xFC,	// LATIN SMALL LETTER U WITH DIAERESIS
  0xFD,	// LATIN SMALL LETTER Z WITH DOT ABOVE
  0xFE,	// LATIN SMALL LETTER Z WITH CARON
};
#define MIN_WINDOWS_1257	0
#define MAX_WINDOWS_1257	255
#define MIN_WINDOWS_1257_UC	0
#define MAX_WINDOWS_1257_UC	8482
