/* ex: set ro ft=c: -*- mode: c; buffer-read-only: t -*- */
// Copyright (C) 2023,2024 Free Software Foundation, Inc.
// This file is part of LibreDWG.
// Created from scratch by Reini Urban
// Generated by txt2h.pl.

#include <stdint.h>

/* unsorted rhs values */
static const uint16_t cptbl_iso_8859_7[] = {
  /*[0x0] = */254,	// size of vector
  /*[0x1] = */1,
  /*[0x2] = */2,
  /*[0x3] = */3,
  /*[0x4] = */4,
  /*[0x5] = */5,
  /*[0x6] = */6,
  /*[0x7] = */7,
  /*[0x8] = */8,
  /*[0x9] = */9,
  /*[0xa] = */10,
  /*[0xb] = */11,
  /*[0xc] = */12,
  /*[0xd] = */13,
  /*[0xe] = */14,
  /*[0xf] = */15,
  /*[0x10] = */16,
  /*[0x11] = */17,
  /*[0x12] = */18,
  /*[0x13] = */19,
  /*[0x14] = */20,
  /*[0x15] = */21,
  /*[0x16] = */22,
  /*[0x17] = */23,
  /*[0x18] = */24,
  /*[0x19] = */25,
  /*[0x1a] = */26,
  /*[0x1b] = */27,
  /*[0x1c] = */28,
  /*[0x1d] = */29,
  /*[0x1e] = */30,
  /*[0x1f] = */31,
  /*[0x20] = */32,
  /*[0x21] = */33,
  /*[0x22] = */34,
  /*[0x23] = */35,
  /*[0x24] = */36,
  /*[0x25] = */37,
  /*[0x26] = */38,
  /*[0x27] = */39,
  /*[0x28] = */40,
  /*[0x29] = */41,
  /*[0x2a] = */42,
  /*[0x2b] = */43,
  /*[0x2c] = */44,
  /*[0x2d] = */45,
  /*[0x2e] = */46,
  /*[0x2f] = */47,
  /*[0x30] = */48,
  /*[0x31] = */49,
  /*[0x32] = */50,
  /*[0x33] = */51,
  /*[0x34] = */52,
  /*[0x35] = */53,
  /*[0x36] = */54,
  /*[0x37] = */55,
  /*[0x38] = */56,
  /*[0x39] = */57,
  /*[0x3a] = */58,
  /*[0x3b] = */59,
  /*[0x3c] = */60,
  /*[0x3d] = */61,
  /*[0x3e] = */62,
  /*[0x3f] = */63,
  /*[0x40] = */64,
  /*[0x41] = */65,
  /*[0x42] = */66,
  /*[0x43] = */67,
  /*[0x44] = */68,
  /*[0x45] = */69,
  /*[0x46] = */70,
  /*[0x47] = */71,
  /*[0x48] = */72,
  /*[0x49] = */73,
  /*[0x4a] = */74,
  /*[0x4b] = */75,
  /*[0x4c] = */76,
  /*[0x4d] = */77,
  /*[0x4e] = */78,
  /*[0x4f] = */79,
  /*[0x50] = */80,
  /*[0x51] = */81,
  /*[0x52] = */82,
  /*[0x53] = */83,
  /*[0x54] = */84,
  /*[0x55] = */85,
  /*[0x56] = */86,
  /*[0x57] = */87,
  /*[0x58] = */88,
  /*[0x59] = */89,
  /*[0x5a] = */90,
  /*[0x5b] = */91,
  /*[0x5c] = */92,
  /*[0x5d] = */93,
  /*[0x5e] = */94,
  /*[0x5f] = */95,
  /*[0x60] = */96,
  /*[0x61] = */97,
  /*[0x62] = */98,
  /*[0x63] = */99,
  /*[0x64] = */100,
  /*[0x65] = */101,
  /*[0x66] = */102,
  /*[0x67] = */103,
  /*[0x68] = */104,
  /*[0x69] = */105,
  /*[0x6a] = */106,
  /*[0x6b] = */107,
  /*[0x6c] = */108,
  /*[0x6d] = */109,
  /*[0x6e] = */110,
  /*[0x6f] = */111,
  /*[0x70] = */112,
  /*[0x71] = */113,
  /*[0x72] = */114,
  /*[0x73] = */115,
  /*[0x74] = */116,
  /*[0x75] = */117,
  /*[0x76] = */118,
  /*[0x77] = */119,
  /*[0x78] = */120,
  /*[0x79] = */121,
  /*[0x7a] = */122,
  /*[0x7b] = */123,
  /*[0x7c] = */124,
  /*[0x7d] = */125,
  /*[0x7e] = */126,
  /*[0x7f] = */0,
  /*[0x80] = */0x0080,	// 	<control>
  /*[0x81] = */0x0081,	// 	<control>
  /*[0x82] = */0x0082,	// 	<control>
  /*[0x83] = */0x0083,	// 	<control>
  /*[0x84] = */0x0084,	// 	<control>
  /*[0x85] = */0x0085,	// 	<control>
  /*[0x86] = */0x0086,	// 	<control>
  /*[0x87] = */0x0087,	// 	<control>
  /*[0x88] = */0x0088,	// 	<control>
  /*[0x89] = */0x0089,	// 	<control>
  /*[0x8A] = */0x008A,	// 	<control>
  /*[0x8B] = */0x008B,	// 	<control>
  /*[0x8C] = */0x008C,	// 	<control>
  /*[0x8D] = */0x008D,	// 	<control>
  /*[0x8E] = */0x008E,	// 	<control>
  /*[0x8F] = */0x008F,	// 	<control>
  /*[0x90] = */0x0090,	// 	<control>
  /*[0x91] = */0x0091,	// 	<control>
  /*[0x92] = */0x0092,	// 	<control>
  /*[0x93] = */0x0093,	// 	<control>
  /*[0x94] = */0x0094,	// 	<control>
  /*[0x95] = */0x0095,	// 	<control>
  /*[0x96] = */0x0096,	// 	<control>
  /*[0x97] = */0x0097,	// 	<control>
  /*[0x98] = */0x0098,	// 	<control>
  /*[0x99] = */0x0099,	// 	<control>
  /*[0x9A] = */0x009A,	// 	<control>
  /*[0x9B] = */0x009B,	// 	<control>
  /*[0x9C] = */0x009C,	// 	<control>
  /*[0x9D] = */0x009D,	// 	<control>
  /*[0x9E] = */0x009E,	// 	<control>
  /*[0x9F] = */0x009F,	// 	<control>
  /*[0xA0] = */0x00A0,	// 	NO-BREAK SPACE
  /*[0xA1] = */0x2018,	// 	LEFT SINGLE QUOTATION MARK
  /*[0xA2] = */0x2019,	// 	RIGHT SINGLE QUOTATION MARK
  /*[0xA3] = */0x00A3,	// 	POUND SIGN
  /*[0xA4] = */0x20AC,	// 	EURO SIGN
  /*[0xA5] = */0x20AF,	// 	DRACHMA SIGN
  /*[0xA6] = */0x00A6,	// 	BROKEN BAR
  /*[0xA7] = */0x00A7,	// 	SECTION SIGN
  /*[0xA8] = */0x00A8,	// 	DIAERESIS
  /*[0xA9] = */0x00A9,	// 	COPYRIGHT SIGN
  /*[0xAA] = */0x037A,	// 	GREEK YPOGEGRAMMENI
  /*[0xAB] = */0x00AB,	// 	LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
  /*[0xAC] = */0x00AC,	// 	NOT SIGN
  /*[0xAD] = */0x00AD,	// 	SOFT HYPHEN
  /*[0xae] = */0,
  /*[0xAF] = */0x2015,	// 	HORIZONTAL BAR
  /*[0xB0] = */0x00B0,	// 	DEGREE SIGN
  /*[0xB1] = */0x00B1,	// 	PLUS-MINUS SIGN
  /*[0xB2] = */0x00B2,	// 	SUPERSCRIPT TWO
  /*[0xB3] = */0x00B3,	// 	SUPERSCRIPT THREE
  /*[0xB4] = */0x0384,	// 	GREEK TONOS
  /*[0xB5] = */0x0385,	// 	GREEK DIALYTIKA TONOS
  /*[0xB6] = */0x0386,	// 	GREEK CAPITAL LETTER ALPHA WITH TONOS
  /*[0xB7] = */0x00B7,	// 	MIDDLE DOT
  /*[0xB8] = */0x0388,	// 	GREEK CAPITAL LETTER EPSILON WITH TONOS
  /*[0xB9] = */0x0389,	// 	GREEK CAPITAL LETTER ETA WITH TONOS
  /*[0xBA] = */0x038A,	// 	GREEK CAPITAL LETTER IOTA WITH TONOS
  /*[0xBB] = */0x00BB,	// 	RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
  /*[0xBC] = */0x038C,	// 	GREEK CAPITAL LETTER OMICRON WITH TONOS
  /*[0xBD] = */0x00BD,	// 	VULGAR FRACTION ONE HALF
  /*[0xBE] = */0x038E,	// 	GREEK CAPITAL LETTER UPSILON WITH TONOS
  /*[0xBF] = */0x038F,	// 	GREEK CAPITAL LETTER OMEGA WITH TONOS
  /*[0xC0] = */0x0390,	// 	GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS
  /*[0xC1] = */0x0391,	// 	GREEK CAPITAL LETTER ALPHA
  /*[0xC2] = */0x0392,	// 	GREEK CAPITAL LETTER BETA
  /*[0xC3] = */0x0393,	// 	GREEK CAPITAL LETTER GAMMA
  /*[0xC4] = */0x0394,	// 	GREEK CAPITAL LETTER DELTA
  /*[0xC5] = */0x0395,	// 	GREEK CAPITAL LETTER EPSILON
  /*[0xC6] = */0x0396,	// 	GREEK CAPITAL LETTER ZETA
  /*[0xC7] = */0x0397,	// 	GREEK CAPITAL LETTER ETA
  /*[0xC8] = */0x0398,	// 	GREEK CAPITAL LETTER THETA
  /*[0xC9] = */0x0399,	// 	GREEK CAPITAL LETTER IOTA
  /*[0xCA] = */0x039A,	// 	GREEK CAPITAL LETTER KAPPA
  /*[0xCB] = */0x039B,	// 	GREEK CAPITAL LETTER LAMDA
  /*[0xCC] = */0x039C,	// 	GREEK CAPITAL LETTER MU
  /*[0xCD] = */0x039D,	// 	GREEK CAPITAL LETTER NU
  /*[0xCE] = */0x039E,	// 	GREEK CAPITAL LETTER XI
  /*[0xCF] = */0x039F,	// 	GREEK CAPITAL LETTER OMICRON
  /*[0xD0] = */0x03A0,	// 	GREEK CAPITAL LETTER PI
  /*[0xD1] = */0x03A1,	// 	GREEK CAPITAL LETTER RHO
  /*[0xd2] = */0,
  /*[0xD3] = */0x03A3,	// 	GREEK CAPITAL LETTER SIGMA
  /*[0xD4] = */0x03A4,	// 	GREEK CAPITAL LETTER TAU
  /*[0xD5] = */0x03A5,	// 	GREEK CAPITAL LETTER UPSILON
  /*[0xD6] = */0x03A6,	// 	GREEK CAPITAL LETTER PHI
  /*[0xD7] = */0x03A7,	// 	GREEK CAPITAL LETTER CHI
  /*[0xD8] = */0x03A8,	// 	GREEK CAPITAL LETTER PSI
  /*[0xD9] = */0x03A9,	// 	GREEK CAPITAL LETTER OMEGA
  /*[0xDA] = */0x03AA,	// 	GREEK CAPITAL LETTER IOTA WITH DIALYTIKA
  /*[0xDB] = */0x03AB,	// 	GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA
  /*[0xDC] = */0x03AC,	// 	GREEK SMALL LETTER ALPHA WITH TONOS
  /*[0xDD] = */0x03AD,	// 	GREEK SMALL LETTER EPSILON WITH TONOS
  /*[0xDE] = */0x03AE,	// 	GREEK SMALL LETTER ETA WITH TONOS
  /*[0xDF] = */0x03AF,	// 	GREEK SMALL LETTER IOTA WITH TONOS
  /*[0xE0] = */0x03B0,	// 	GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS
  /*[0xE1] = */0x03B1,	// 	GREEK SMALL LETTER ALPHA
  /*[0xE2] = */0x03B2,	// 	GREEK SMALL LETTER BETA
  /*[0xE3] = */0x03B3,	// 	GREEK SMALL LETTER GAMMA
  /*[0xE4] = */0x03B4,	// 	GREEK SMALL LETTER DELTA
  /*[0xE5] = */0x03B5,	// 	GREEK SMALL LETTER EPSILON
  /*[0xE6] = */0x03B6,	// 	GREEK SMALL LETTER ZETA
  /*[0xE7] = */0x03B7,	// 	GREEK SMALL LETTER ETA
  /*[0xE8] = */0x03B8,	// 	GREEK SMALL LETTER THETA
  /*[0xE9] = */0x03B9,	// 	GREEK SMALL LETTER IOTA
  /*[0xEA] = */0x03BA,	// 	GREEK SMALL LETTER KAPPA
  /*[0xEB] = */0x03BB,	// 	GREEK SMALL LETTER LAMDA
  /*[0xEC] = */0x03BC,	// 	GREEK SMALL LETTER MU
  /*[0xED] = */0x03BD,	// 	GREEK SMALL LETTER NU
  /*[0xEE] = */0x03BE,	// 	GREEK SMALL LETTER XI
  /*[0xEF] = */0x03BF,	// 	GREEK SMALL LETTER OMICRON
  /*[0xF0] = */0x03C0,	// 	GREEK SMALL LETTER PI
  /*[0xF1] = */0x03C1,	// 	GREEK SMALL LETTER RHO
  /*[0xF2] = */0x03C2,	// 	GREEK SMALL LETTER FINAL SIGMA
  /*[0xF3] = */0x03C3,	// 	GREEK SMALL LETTER SIGMA
  /*[0xF4] = */0x03C4,	// 	GREEK SMALL LETTER TAU
  /*[0xF5] = */0x03C5,	// 	GREEK SMALL LETTER UPSILON
  /*[0xF6] = */0x03C6,	// 	GREEK SMALL LETTER PHI
  /*[0xF7] = */0x03C7,	// 	GREEK SMALL LETTER CHI
  /*[0xF8] = */0x03C8,	// 	GREEK SMALL LETTER PSI
  /*[0xF9] = */0x03C9,	// 	GREEK SMALL LETTER OMEGA
  /*[0xFA] = */0x03CA,	// 	GREEK SMALL LETTER IOTA WITH DIALYTIKA
  /*[0xFB] = */0x03CB,	// 	GREEK SMALL LETTER UPSILON WITH DIALYTIKA
  /*[0xFC] = */0x03CC,	// 	GREEK SMALL LETTER OMICRON WITH TONOS
  /*[0xFD] = */0x03CD,	// 	GREEK SMALL LETTER UPSILON WITH TONOS
  /*[0xFE] = */0x03CE,	// 	GREEK SMALL LETTER OMEGA WITH TONOS
};
/* sorted, without the first 127 characters */
static const uint8_t cptbl_alnum_iso_8859_7[] = {
  69,		// size of vector
  0xB6,	// 	GREEK CAPITAL LETTER ALPHA WITH TONOS
  0xB8,	// 	GREEK CAPITAL LETTER EPSILON WITH TONOS
  0xB9,	// 	GREEK CAPITAL LETTER ETA WITH TONOS
  0xBA,	// 	GREEK CAPITAL LETTER IOTA WITH TONOS
  0xBC,	// 	GREEK CAPITAL LETTER OMICRON WITH TONOS
  0xBE,	// 	GREEK CAPITAL LETTER UPSILON WITH TONOS
  0xBF,	// 	GREEK CAPITAL LETTER OMEGA WITH TONOS
  0xC0,	// 	GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS
  0xC1,	// 	GREEK CAPITAL LETTER ALPHA
  0xC2,	// 	GREEK CAPITAL LETTER BETA
  0xC3,	// 	GREEK CAPITAL LETTER GAMMA
  0xC4,	// 	GREEK CAPITAL LETTER DELTA
  0xC5,	// 	GREEK CAPITAL LETTER EPSILON
  0xC6,	// 	GREEK CAPITAL LETTER ZETA
  0xC7,	// 	GREEK CAPITAL LETTER ETA
  0xC8,	// 	GREEK CAPITAL LETTER THETA
  0xC9,	// 	GREEK CAPITAL LETTER IOTA
  0xCA,	// 	GREEK CAPITAL LETTER KAPPA
  0xCB,	// 	GREEK CAPITAL LETTER LAMDA
  0xCC,	// 	GREEK CAPITAL LETTER MU
  0xCD,	// 	GREEK CAPITAL LETTER NU
  0xCE,	// 	GREEK CAPITAL LETTER XI
  0xCF,	// 	GREEK CAPITAL LETTER OMICRON
  0xD0,	// 	GREEK CAPITAL LETTER PI
  0xD1,	// 	GREEK CAPITAL LETTER RHO
  0xD3,	// 	GREEK CAPITAL LETTER SIGMA
  0xD4,	// 	GREEK CAPITAL LETTER TAU
  0xD5,	// 	GREEK CAPITAL LETTER UPSILON
  0xD6,	// 	GREEK CAPITAL LETTER PHI
  0xD7,	// 	GREEK CAPITAL LETTER CHI
  0xD8,	// 	GREEK CAPITAL LETTER PSI
  0xD9,	// 	GREEK CAPITAL LETTER OMEGA
  0xDA,	// 	GREEK CAPITAL LETTER IOTA WITH DIALYTIKA
  0xDB,	// 	GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA
  0xDC,	// 	GREEK SMALL LETTER ALPHA WITH TONOS
  0xDD,	// 	GREEK SMALL LETTER EPSILON WITH TONOS
  0xDE,	// 	GREEK SMALL LETTER ETA WITH TONOS
  0xDF,	// 	GREEK SMALL LETTER IOTA WITH TONOS
  0xE0,	// 	GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS
  0xE1,	// 	GREEK SMALL LETTER ALPHA
  0xE2,	// 	GREEK SMALL LETTER BETA
  0xE3,	// 	GREEK SMALL LETTER GAMMA
  0xE4,	// 	GREEK SMALL LETTER DELTA
  0xE5,	// 	GREEK SMALL LETTER EPSILON
  0xE6,	// 	GREEK SMALL LETTER ZETA
  0xE7,	// 	GREEK SMALL LETTER ETA
  0xE8,	// 	GREEK SMALL LETTER THETA
  0xE9,	// 	GREEK SMALL LETTER IOTA
  0xEA,	// 	GREEK SMALL LETTER KAPPA
  0xEB,	// 	GREEK SMALL LETTER LAMDA
  0xEC,	// 	GREEK SMALL LETTER MU
  0xED,	// 	GREEK SMALL LETTER NU
  0xEE,	// 	GREEK SMALL LETTER XI
  0xEF,	// 	GREEK SMALL LETTER OMICRON
  0xF0,	// 	GREEK SMALL LETTER PI
  0xF1,	// 	GREEK SMALL LETTER RHO
  0xF2,	// 	GREEK SMALL LETTER FINAL SIGMA
  0xF3,	// 	GREEK SMALL LETTER SIGMA
  0xF4,	// 	GREEK SMALL LETTER TAU
  0xF5,	// 	GREEK SMALL LETTER UPSILON
  0xF6,	// 	GREEK SMALL LETTER PHI
  0xF7,	// 	GREEK SMALL LETTER CHI
  0xF8,	// 	GREEK SMALL LETTER PSI
  0xF9,	// 	GREEK SMALL LETTER OMEGA
  0xFA,	// 	GREEK SMALL LETTER IOTA WITH DIALYTIKA
  0xFB,	// 	GREEK SMALL LETTER UPSILON WITH DIALYTIKA
  0xFC,	// 	GREEK SMALL LETTER OMICRON WITH TONOS
  0xFD,	// 	GREEK SMALL LETTER UPSILON WITH TONOS
  0xFE,	// 	GREEK SMALL LETTER OMEGA WITH TONOS
};
#define MIN_ISO_8859_7	0
#define MAX_ISO_8859_7	254
#define MIN_ISO_8859_7_UC	0
#define MAX_ISO_8859_7_UC	8367
