# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.3.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

package LibreDWG;
use base qw(Exporter);
use base qw(DynaLoader);
package LibreDWGc;
bootstrap LibreDWG;
package LibreDWG;
@EXPORT = qw();

# ---------- BASE METHODS -------------

package LibreDWG;

sub TIEHASH {
    my ($classname,$obj) = @_;
    return bless $obj, $classname;
}

sub CLEAR { }

sub FIRSTKEY { }

sub NEXTKEY { }

sub FETCH {
    my ($self,$field) = @_;
    my $member_func = "swig_${field}_get";
    $self->$member_func();
}

sub STORE {
    my ($self,$field,$newval) = @_;
    my $member_func = "swig_${field}_set";
    $self->$member_func($newval);
}

sub this {
    my $ptr = shift;
    return tied(%$ptr);
}


# ------- FUNCTION WRAPPERS --------

package LibreDWG;

*new_Dwg_Object_Array = *LibreDWGc::new_Dwg_Object_Array;
*delete_Dwg_Object_Array = *LibreDWGc::delete_Dwg_Object_Array;
*Dwg_Object_Array_getitem = *LibreDWGc::Dwg_Object_Array_getitem;
*Dwg_Object_Array_setitem = *LibreDWGc::Dwg_Object_Array_setitem;
*dwg_color_method_name = *LibreDWGc::dwg_color_method_name;
*dwg_read_file = *LibreDWGc::dwg_read_file;
*dxf_read_file = *LibreDWGc::dxf_read_file;
*dwg_write_file = *LibreDWGc::dwg_write_file;
*dwg_bmp = *LibreDWGc::dwg_bmp;
*dxf_cvt_lweight = *LibreDWGc::dxf_cvt_lweight;
*dxf_revcvt_lweight = *LibreDWGc::dxf_revcvt_lweight;
*dwg_ctrl_table = *LibreDWGc::dwg_ctrl_table;
*dwg_find_tablehandle = *LibreDWGc::dwg_find_tablehandle;
*dwg_find_tablehandle_index = *LibreDWGc::dwg_find_tablehandle_index;
*dwg_handle_name = *LibreDWGc::dwg_handle_name;
*dwg_find_table_control = *LibreDWGc::dwg_find_table_control;
*dwg_find_dictionary = *LibreDWGc::dwg_find_dictionary;
*dwg_find_dicthandle = *LibreDWGc::dwg_find_dicthandle;
*dwg_find_dicthandle_objname = *LibreDWGc::dwg_find_dicthandle_objname;
*dwg_find_table_extname = *LibreDWGc::dwg_find_table_extname;
*dwg_variable_dict = *LibreDWGc::dwg_variable_dict;
*dwg_model_x_min = *LibreDWGc::dwg_model_x_min;
*dwg_model_x_max = *LibreDWGc::dwg_model_x_max;
*dwg_model_y_min = *LibreDWGc::dwg_model_y_min;
*dwg_model_y_max = *LibreDWGc::dwg_model_y_max;
*dwg_model_z_min = *LibreDWGc::dwg_model_z_min;
*dwg_model_z_max = *LibreDWGc::dwg_model_z_max;
*dwg_page_x_min = *LibreDWGc::dwg_page_x_min;
*dwg_page_x_max = *LibreDWGc::dwg_page_x_max;
*dwg_page_y_min = *LibreDWGc::dwg_page_y_min;
*dwg_page_y_max = *LibreDWGc::dwg_page_y_max;
*dwg_block_control = *LibreDWGc::dwg_block_control;
*dwg_model_space_ref = *LibreDWGc::dwg_model_space_ref;
*dwg_paper_space_ref = *LibreDWGc::dwg_paper_space_ref;
*dwg_model_space_object = *LibreDWGc::dwg_model_space_object;
*dwg_paper_space_object = *LibreDWGc::dwg_paper_space_object;
*dwg_get_layer_count = *LibreDWGc::dwg_get_layer_count;
*dwg_get_layers = *LibreDWGc::dwg_get_layers;
*dwg_get_num_objects = *LibreDWGc::dwg_get_num_objects;
*dwg_get_object_num_objects = *LibreDWGc::dwg_get_object_num_objects;
*dwg_class_is_entity = *LibreDWGc::dwg_class_is_entity;
*dwg_obj_is_control = *LibreDWGc::dwg_obj_is_control;
*dwg_obj_is_table = *LibreDWGc::dwg_obj_is_table;
*dwg_obj_is_subentity = *LibreDWGc::dwg_obj_is_subentity;
*dwg_obj_has_subentity = *LibreDWGc::dwg_obj_has_subentity;
*dwg_obj_is_3dsolid = *LibreDWGc::dwg_obj_is_3dsolid;
*dwg_obj_is_acsh = *LibreDWGc::dwg_obj_is_acsh;
*dwg_get_num_entities = *LibreDWGc::dwg_get_num_entities;
*dwg_get_entities = *LibreDWGc::dwg_get_entities;
*dwg_get_entity_layer = *LibreDWGc::dwg_get_entity_layer;
*dwg_next_object = *LibreDWGc::dwg_next_object;
*dwg_next_entity = *LibreDWGc::dwg_next_entity;
*dwg_next_handle = *LibreDWGc::dwg_next_handle;
*dwg_next_handseed = *LibreDWGc::dwg_next_handseed;
*dwg_ref_object = *LibreDWGc::dwg_ref_object;
*dwg_ref_object_relative = *LibreDWGc::dwg_ref_object_relative;
*dwg_ref_object_silent = *LibreDWGc::dwg_ref_object_silent;
*get_first_owned_entity = *LibreDWGc::get_first_owned_entity;
*get_next_owned_entity = *LibreDWGc::get_next_owned_entity;
*get_first_owned_subentity = *LibreDWGc::get_first_owned_subentity;
*get_next_owned_subentity = *LibreDWGc::get_next_owned_subentity;
*get_first_owned_block = *LibreDWGc::get_first_owned_block;
*get_last_owned_block = *LibreDWGc::get_last_owned_block;
*get_next_owned_block = *LibreDWGc::get_next_owned_block;
*get_next_owned_block_entity = *LibreDWGc::get_next_owned_block_entity;
*dwg_get_first_object = *LibreDWGc::dwg_get_first_object;
*dwg_get_next_object = *LibreDWGc::dwg_get_next_object;
*dwg_resolve_handle = *LibreDWGc::dwg_resolve_handle;
*dwg_resolve_handle_silent = *LibreDWGc::dwg_resolve_handle_silent;
*dwg_resolve_handleref = *LibreDWGc::dwg_resolve_handleref;
*dwg_resolve_jump = *LibreDWGc::dwg_resolve_jump;
*dwg_section_type = *LibreDWGc::dwg_section_type;
*dwg_section_wtype = *LibreDWGc::dwg_section_wtype;
*dwg_section_name = *LibreDWGc::dwg_section_name;
*dwg_resbuf_value_type = *LibreDWGc::dwg_resbuf_value_type;
*dwg_free = *LibreDWGc::dwg_free;
*dwg_free_object = *LibreDWGc::dwg_free_object;
*dwg_new_ref = *LibreDWGc::dwg_new_ref;
*dwg_add_handle = *LibreDWGc::dwg_add_handle;
*dwg_add_handleref = *LibreDWGc::dwg_add_handleref;
*dwg_dup_handleref = *LibreDWGc::dwg_dup_handleref;
*dwg_add_handleref_free = *LibreDWGc::dwg_add_handleref_free;
*dwg_version_type = *LibreDWGc::dwg_version_type;
*dwg_version_as = *LibreDWGc::dwg_version_as;
*dwg_version_hdr_type = *LibreDWGc::dwg_version_hdr_type;
*dwg_supports_eed = *LibreDWGc::dwg_supports_eed;
*dwg_supports_obj = *LibreDWGc::dwg_supports_obj;
*dwg_errstrings = *LibreDWGc::dwg_errstrings;
*dwg_encrypt_SAT1 = *LibreDWGc::dwg_encrypt_SAT1;
*dwg_convert_SAB_to_SAT1 = *LibreDWGc::dwg_convert_SAB_to_SAT1;
*dwg_rgb_palette = *LibreDWGc::dwg_rgb_palette;
*dwg_rgb_palette_index = *LibreDWGc::dwg_rgb_palette_index;
*dwg_find_color_index = *LibreDWGc::dwg_find_color_index;
*dwg_add_object = *LibreDWGc::dwg_add_object;
*dwg_object_name = *LibreDWGc::dwg_object_name;
*dwg_setup__3DFACE = *LibreDWGc::dwg_setup__3DFACE;
*dwg_setup__3DSOLID = *LibreDWGc::dwg_setup__3DSOLID;
*dwg_setup_ARC = *LibreDWGc::dwg_setup_ARC;
*dwg_setup_ATTDEF = *LibreDWGc::dwg_setup_ATTDEF;
*dwg_setup_ATTRIB = *LibreDWGc::dwg_setup_ATTRIB;
*dwg_setup_BLOCK = *LibreDWGc::dwg_setup_BLOCK;
*dwg_setup_BODY = *LibreDWGc::dwg_setup_BODY;
*dwg_setup_CIRCLE = *LibreDWGc::dwg_setup_CIRCLE;
*dwg_setup_DIMENSION_ALIGNED = *LibreDWGc::dwg_setup_DIMENSION_ALIGNED;
*dwg_setup_DIMENSION_ANG2LN = *LibreDWGc::dwg_setup_DIMENSION_ANG2LN;
*dwg_setup_DIMENSION_ANG3PT = *LibreDWGc::dwg_setup_DIMENSION_ANG3PT;
*dwg_setup_DIMENSION_DIAMETER = *LibreDWGc::dwg_setup_DIMENSION_DIAMETER;
*dwg_setup_DIMENSION_LINEAR = *LibreDWGc::dwg_setup_DIMENSION_LINEAR;
*dwg_setup_DIMENSION_ORDINATE = *LibreDWGc::dwg_setup_DIMENSION_ORDINATE;
*dwg_setup_DIMENSION_RADIUS = *LibreDWGc::dwg_setup_DIMENSION_RADIUS;
*dwg_setup_ELLIPSE = *LibreDWGc::dwg_setup_ELLIPSE;
*dwg_setup_ENDBLK = *LibreDWGc::dwg_setup_ENDBLK;
*dwg_setup_INSERT = *LibreDWGc::dwg_setup_INSERT;
*dwg_setup_LEADER = *LibreDWGc::dwg_setup_LEADER;
*dwg_setup_LINE = *LibreDWGc::dwg_setup_LINE;
*dwg_setup_LOAD = *LibreDWGc::dwg_setup_LOAD;
*dwg_setup_MINSERT = *LibreDWGc::dwg_setup_MINSERT;
*dwg_setup_MLINE = *LibreDWGc::dwg_setup_MLINE;
*dwg_setup_MTEXT = *LibreDWGc::dwg_setup_MTEXT;
*dwg_setup_OLEFRAME = *LibreDWGc::dwg_setup_OLEFRAME;
*dwg_setup_POINT = *LibreDWGc::dwg_setup_POINT;
*dwg_setup_POLYLINE_2D = *LibreDWGc::dwg_setup_POLYLINE_2D;
*dwg_setup_POLYLINE_3D = *LibreDWGc::dwg_setup_POLYLINE_3D;
*dwg_setup_POLYLINE_MESH = *LibreDWGc::dwg_setup_POLYLINE_MESH;
*dwg_setup_POLYLINE_PFACE = *LibreDWGc::dwg_setup_POLYLINE_PFACE;
*dwg_setup_PROXY_ENTITY = *LibreDWGc::dwg_setup_PROXY_ENTITY;
*dwg_setup_RAY = *LibreDWGc::dwg_setup_RAY;
*dwg_setup_REGION = *LibreDWGc::dwg_setup_REGION;
*dwg_setup_SEQEND = *LibreDWGc::dwg_setup_SEQEND;
*dwg_setup_SHAPE = *LibreDWGc::dwg_setup_SHAPE;
*dwg_setup_SOLID = *LibreDWGc::dwg_setup_SOLID;
*dwg_setup_SPLINE = *LibreDWGc::dwg_setup_SPLINE;
*dwg_setup_TEXT = *LibreDWGc::dwg_setup_TEXT;
*dwg_setup_TOLERANCE = *LibreDWGc::dwg_setup_TOLERANCE;
*dwg_setup_TRACE = *LibreDWGc::dwg_setup_TRACE;
*dwg_setup_UNKNOWN_ENT = *LibreDWGc::dwg_setup_UNKNOWN_ENT;
*dwg_setup_VERTEX_2D = *LibreDWGc::dwg_setup_VERTEX_2D;
*dwg_setup_VERTEX_3D = *LibreDWGc::dwg_setup_VERTEX_3D;
*dwg_setup_VERTEX_MESH = *LibreDWGc::dwg_setup_VERTEX_MESH;
*dwg_setup_VERTEX_PFACE = *LibreDWGc::dwg_setup_VERTEX_PFACE;
*dwg_setup_VERTEX_PFACE_FACE = *LibreDWGc::dwg_setup_VERTEX_PFACE_FACE;
*dwg_setup_VIEWPORT = *LibreDWGc::dwg_setup_VIEWPORT;
*dwg_setup_XLINE = *LibreDWGc::dwg_setup_XLINE;
*dwg_setup_APPID = *LibreDWGc::dwg_setup_APPID;
*dwg_setup_APPID_CONTROL = *LibreDWGc::dwg_setup_APPID_CONTROL;
*dwg_setup_BLOCK_CONTROL = *LibreDWGc::dwg_setup_BLOCK_CONTROL;
*dwg_setup_BLOCK_HEADER = *LibreDWGc::dwg_setup_BLOCK_HEADER;
*dwg_setup_DICTIONARY = *LibreDWGc::dwg_setup_DICTIONARY;
*dwg_setup_DIMSTYLE = *LibreDWGc::dwg_setup_DIMSTYLE;
*dwg_setup_DIMSTYLE_CONTROL = *LibreDWGc::dwg_setup_DIMSTYLE_CONTROL;
*dwg_setup_DUMMY = *LibreDWGc::dwg_setup_DUMMY;
*dwg_setup_LAYER = *LibreDWGc::dwg_setup_LAYER;
*dwg_setup_LAYER_CONTROL = *LibreDWGc::dwg_setup_LAYER_CONTROL;
*dwg_setup_LONG_TRANSACTION = *LibreDWGc::dwg_setup_LONG_TRANSACTION;
*dwg_setup_LTYPE = *LibreDWGc::dwg_setup_LTYPE;
*dwg_setup_LTYPE_CONTROL = *LibreDWGc::dwg_setup_LTYPE_CONTROL;
*dwg_setup_MLINESTYLE = *LibreDWGc::dwg_setup_MLINESTYLE;
*dwg_setup_STYLE = *LibreDWGc::dwg_setup_STYLE;
*dwg_setup_STYLE_CONTROL = *LibreDWGc::dwg_setup_STYLE_CONTROL;
*dwg_setup_UCS = *LibreDWGc::dwg_setup_UCS;
*dwg_setup_UCS_CONTROL = *LibreDWGc::dwg_setup_UCS_CONTROL;
*dwg_setup_UNKNOWN_OBJ = *LibreDWGc::dwg_setup_UNKNOWN_OBJ;
*dwg_setup_VIEW = *LibreDWGc::dwg_setup_VIEW;
*dwg_setup_VIEW_CONTROL = *LibreDWGc::dwg_setup_VIEW_CONTROL;
*dwg_setup_VPORT = *LibreDWGc::dwg_setup_VPORT;
*dwg_setup_VPORT_CONTROL = *LibreDWGc::dwg_setup_VPORT_CONTROL;
*dwg_setup_VX_CONTROL = *LibreDWGc::dwg_setup_VX_CONTROL;
*dwg_setup_VX_TABLE_RECORD = *LibreDWGc::dwg_setup_VX_TABLE_RECORD;
*dwg_setup__3DLINE = *LibreDWGc::dwg_setup__3DLINE;
*dwg_setup_CAMERA = *LibreDWGc::dwg_setup_CAMERA;
*dwg_setup_DGNUNDERLAY = *LibreDWGc::dwg_setup_DGNUNDERLAY;
*dwg_setup_DWFUNDERLAY = *LibreDWGc::dwg_setup_DWFUNDERLAY;
*dwg_setup_ENDREP = *LibreDWGc::dwg_setup_ENDREP;
*dwg_setup_HATCH = *LibreDWGc::dwg_setup_HATCH;
*dwg_setup_IMAGE = *LibreDWGc::dwg_setup_IMAGE;
*dwg_setup_JUMP = *LibreDWGc::dwg_setup_JUMP;
*dwg_setup_LIGHT = *LibreDWGc::dwg_setup_LIGHT;
*dwg_setup_LWPOLYLINE = *LibreDWGc::dwg_setup_LWPOLYLINE;
*dwg_setup_MESH = *LibreDWGc::dwg_setup_MESH;
*dwg_setup_MULTILEADER = *LibreDWGc::dwg_setup_MULTILEADER;
*dwg_setup_OLE2FRAME = *LibreDWGc::dwg_setup_OLE2FRAME;
*dwg_setup_PDFUNDERLAY = *LibreDWGc::dwg_setup_PDFUNDERLAY;
*dwg_setup_REPEAT = *LibreDWGc::dwg_setup_REPEAT;
*dwg_setup_SECTIONOBJECT = *LibreDWGc::dwg_setup_SECTIONOBJECT;
*dwg_setup_WIPEOUT = *LibreDWGc::dwg_setup_WIPEOUT;
*dwg_setup_ACSH_BOOLEAN_CLASS = *LibreDWGc::dwg_setup_ACSH_BOOLEAN_CLASS;
*dwg_setup_ACSH_BOX_CLASS = *LibreDWGc::dwg_setup_ACSH_BOX_CLASS;
*dwg_setup_ACSH_CONE_CLASS = *LibreDWGc::dwg_setup_ACSH_CONE_CLASS;
*dwg_setup_ACSH_CYLINDER_CLASS = *LibreDWGc::dwg_setup_ACSH_CYLINDER_CLASS;
*dwg_setup_ACSH_FILLET_CLASS = *LibreDWGc::dwg_setup_ACSH_FILLET_CLASS;
*dwg_setup_ACSH_HISTORY_CLASS = *LibreDWGc::dwg_setup_ACSH_HISTORY_CLASS;
*dwg_setup_ACSH_SPHERE_CLASS = *LibreDWGc::dwg_setup_ACSH_SPHERE_CLASS;
*dwg_setup_ACSH_TORUS_CLASS = *LibreDWGc::dwg_setup_ACSH_TORUS_CLASS;
*dwg_setup_ACSH_WEDGE_CLASS = *LibreDWGc::dwg_setup_ACSH_WEDGE_CLASS;
*dwg_setup_ASSOCGEOMDEPENDENCY = *LibreDWGc::dwg_setup_ASSOCGEOMDEPENDENCY;
*dwg_setup_ASSOCNETWORK = *LibreDWGc::dwg_setup_ASSOCNETWORK;
*dwg_setup_BLOCKALIGNMENTGRIP = *LibreDWGc::dwg_setup_BLOCKALIGNMENTGRIP;
*dwg_setup_BLOCKALIGNMENTPARAMETER = *LibreDWGc::dwg_setup_BLOCKALIGNMENTPARAMETER;
*dwg_setup_BLOCKBASEPOINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKBASEPOINTPARAMETER;
*dwg_setup_BLOCKFLIPACTION = *LibreDWGc::dwg_setup_BLOCKFLIPACTION;
*dwg_setup_BLOCKFLIPGRIP = *LibreDWGc::dwg_setup_BLOCKFLIPGRIP;
*dwg_setup_BLOCKFLIPPARAMETER = *LibreDWGc::dwg_setup_BLOCKFLIPPARAMETER;
*dwg_setup_BLOCKGRIPLOCATIONCOMPONENT = *LibreDWGc::dwg_setup_BLOCKGRIPLOCATIONCOMPONENT;
*dwg_setup_BLOCKLINEARGRIP = *LibreDWGc::dwg_setup_BLOCKLINEARGRIP;
*dwg_setup_BLOCKLOOKUPGRIP = *LibreDWGc::dwg_setup_BLOCKLOOKUPGRIP;
*dwg_setup_BLOCKMOVEACTION = *LibreDWGc::dwg_setup_BLOCKMOVEACTION;
*dwg_setup_BLOCKROTATEACTION = *LibreDWGc::dwg_setup_BLOCKROTATEACTION;
*dwg_setup_BLOCKROTATIONGRIP = *LibreDWGc::dwg_setup_BLOCKROTATIONGRIP;
*dwg_setup_BLOCKSCALEACTION = *LibreDWGc::dwg_setup_BLOCKSCALEACTION;
*dwg_setup_BLOCKVISIBILITYGRIP = *LibreDWGc::dwg_setup_BLOCKVISIBILITYGRIP;
*dwg_setup_CELLSTYLEMAP = *LibreDWGc::dwg_setup_CELLSTYLEMAP;
*dwg_setup_DETAILVIEWSTYLE = *LibreDWGc::dwg_setup_DETAILVIEWSTYLE;
*dwg_setup_DICTIONARYVAR = *LibreDWGc::dwg_setup_DICTIONARYVAR;
*dwg_setup_DICTIONARYWDFLT = *LibreDWGc::dwg_setup_DICTIONARYWDFLT;
*dwg_setup_DYNAMICBLOCKPURGEPREVENTER = *LibreDWGc::dwg_setup_DYNAMICBLOCKPURGEPREVENTER;
*dwg_setup_FIELD = *LibreDWGc::dwg_setup_FIELD;
*dwg_setup_FIELDLIST = *LibreDWGc::dwg_setup_FIELDLIST;
*dwg_setup_GEODATA = *LibreDWGc::dwg_setup_GEODATA;
*dwg_setup_GROUP = *LibreDWGc::dwg_setup_GROUP;
*dwg_setup_IDBUFFER = *LibreDWGc::dwg_setup_IDBUFFER;
*dwg_setup_IMAGEDEF = *LibreDWGc::dwg_setup_IMAGEDEF;
*dwg_setup_IMAGEDEF_REACTOR = *LibreDWGc::dwg_setup_IMAGEDEF_REACTOR;
*dwg_setup_INDEX = *LibreDWGc::dwg_setup_INDEX;
*dwg_setup_LAYERFILTER = *LibreDWGc::dwg_setup_LAYERFILTER;
*dwg_setup_LAYER_INDEX = *LibreDWGc::dwg_setup_LAYER_INDEX;
*dwg_setup_LAYOUT = *LibreDWGc::dwg_setup_LAYOUT;
*dwg_setup_PLACEHOLDER = *LibreDWGc::dwg_setup_PLACEHOLDER;
*dwg_setup_PLOTSETTINGS = *LibreDWGc::dwg_setup_PLOTSETTINGS;
*dwg_setup_RASTERVARIABLES = *LibreDWGc::dwg_setup_RASTERVARIABLES;
*dwg_setup_RENDERENVIRONMENT = *LibreDWGc::dwg_setup_RENDERENVIRONMENT;
*dwg_setup_SCALE = *LibreDWGc::dwg_setup_SCALE;
*dwg_setup_SECTIONVIEWSTYLE = *LibreDWGc::dwg_setup_SECTIONVIEWSTYLE;
*dwg_setup_SECTION_MANAGER = *LibreDWGc::dwg_setup_SECTION_MANAGER;
*dwg_setup_SORTENTSTABLE = *LibreDWGc::dwg_setup_SORTENTSTABLE;
*dwg_setup_SPATIAL_FILTER = *LibreDWGc::dwg_setup_SPATIAL_FILTER;
*dwg_setup_SUN = *LibreDWGc::dwg_setup_SUN;
*dwg_setup_TABLEGEOMETRY = *LibreDWGc::dwg_setup_TABLEGEOMETRY;
*dwg_setup_VBA_PROJECT = *LibreDWGc::dwg_setup_VBA_PROJECT;
*dwg_setup_VISUALSTYLE = *LibreDWGc::dwg_setup_VISUALSTYLE;
*dwg_setup_WIPEOUTVARIABLES = *LibreDWGc::dwg_setup_WIPEOUTVARIABLES;
*dwg_setup_XRECORD = *LibreDWGc::dwg_setup_XRECORD;
*dwg_setup_PDFDEFINITION = *LibreDWGc::dwg_setup_PDFDEFINITION;
*dwg_setup_DGNDEFINITION = *LibreDWGc::dwg_setup_DGNDEFINITION;
*dwg_setup_DWFDEFINITION = *LibreDWGc::dwg_setup_DWFDEFINITION;
*dwg_setup_ARC_DIMENSION = *LibreDWGc::dwg_setup_ARC_DIMENSION;
*dwg_setup_HELIX = *LibreDWGc::dwg_setup_HELIX;
*dwg_setup_LARGE_RADIAL_DIMENSION = *LibreDWGc::dwg_setup_LARGE_RADIAL_DIMENSION;
*dwg_setup_LAYOUTPRINTCONFIG = *LibreDWGc::dwg_setup_LAYOUTPRINTCONFIG;
*dwg_setup_PLANESURFACE = *LibreDWGc::dwg_setup_PLANESURFACE;
*dwg_setup_POINTCLOUD = *LibreDWGc::dwg_setup_POINTCLOUD;
*dwg_setup_POINTCLOUDEX = *LibreDWGc::dwg_setup_POINTCLOUDEX;
*dwg_setup_ACSH_BREP_CLASS = *LibreDWGc::dwg_setup_ACSH_BREP_CLASS;
*dwg_setup_ACSH_CHAMFER_CLASS = *LibreDWGc::dwg_setup_ACSH_CHAMFER_CLASS;
*dwg_setup_ACSH_PYRAMID_CLASS = *LibreDWGc::dwg_setup_ACSH_PYRAMID_CLASS;
*dwg_setup_ALDIMOBJECTCONTEXTDATA = *LibreDWGc::dwg_setup_ALDIMOBJECTCONTEXTDATA;
*dwg_setup_ASSOC2DCONSTRAINTGROUP = *LibreDWGc::dwg_setup_ASSOC2DCONSTRAINTGROUP;
*dwg_setup_ASSOCACTION = *LibreDWGc::dwg_setup_ASSOCACTION;
*dwg_setup_ASSOCACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCACTIONPARAM;
*dwg_setup_ASSOCARRAYACTIONBODY = *LibreDWGc::dwg_setup_ASSOCARRAYACTIONBODY;
*dwg_setup_ASSOCASMBODYACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCASMBODYACTIONPARAM;
*dwg_setup_ASSOCBLENDSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCBLENDSURFACEACTIONBODY;
*dwg_setup_ASSOCCOMPOUNDACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCCOMPOUNDACTIONPARAM;
*dwg_setup_ASSOCDEPENDENCY = *LibreDWGc::dwg_setup_ASSOCDEPENDENCY;
*dwg_setup_ASSOCDIMDEPENDENCYBODY = *LibreDWGc::dwg_setup_ASSOCDIMDEPENDENCYBODY;
*dwg_setup_ASSOCEXTENDSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCEXTENDSURFACEACTIONBODY;
*dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCEXTRUDEDSURFACEACTIONBODY;
*dwg_setup_ASSOCFACEACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCFACEACTIONPARAM;
*dwg_setup_ASSOCFILLETSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCFILLETSURFACEACTIONBODY;
*dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCLOFTEDSURFACEACTIONBODY;
*dwg_setup_ASSOCNETWORKSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCNETWORKSURFACEACTIONBODY;
*dwg_setup_ASSOCOBJECTACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCOBJECTACTIONPARAM;
*dwg_setup_ASSOCOFFSETSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCOFFSETSURFACEACTIONBODY;
*dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCOSNAPPOINTREFACTIONPARAM;
*dwg_setup_ASSOCPATCHSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCPATCHSURFACEACTIONBODY;
*dwg_setup_ASSOCPATHACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCPATHACTIONPARAM;
*dwg_setup_ASSOCPLANESURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCPLANESURFACEACTIONBODY;
*dwg_setup_ASSOCPOINTREFACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCPOINTREFACTIONPARAM;
*dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCREVOLVEDSURFACEACTIONBODY;
*dwg_setup_ASSOCTRIMSURFACEACTIONBODY = *LibreDWGc::dwg_setup_ASSOCTRIMSURFACEACTIONBODY;
*dwg_setup_ASSOCVALUEDEPENDENCY = *LibreDWGc::dwg_setup_ASSOCVALUEDEPENDENCY;
*dwg_setup_ASSOCVARIABLE = *LibreDWGc::dwg_setup_ASSOCVARIABLE;
*dwg_setup_ASSOCVERTEXACTIONPARAM = *LibreDWGc::dwg_setup_ASSOCVERTEXACTIONPARAM;
*dwg_setup_BLKREFOBJECTCONTEXTDATA = *LibreDWGc::dwg_setup_BLKREFOBJECTCONTEXTDATA;
*dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKALIGNEDCONSTRAINTPARAMETER;
*dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKANGULARCONSTRAINTPARAMETER;
*dwg_setup_BLOCKARRAYACTION = *LibreDWGc::dwg_setup_BLOCKARRAYACTION;
*dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKDIAMETRICCONSTRAINTPARAMETER;
*dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKHORIZONTALCONSTRAINTPARAMETER;
*dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKLINEARCONSTRAINTPARAMETER;
*dwg_setup_BLOCKLINEARPARAMETER = *LibreDWGc::dwg_setup_BLOCKLINEARPARAMETER;
*dwg_setup_BLOCKLOOKUPACTION = *LibreDWGc::dwg_setup_BLOCKLOOKUPACTION;
*dwg_setup_BLOCKLOOKUPPARAMETER = *LibreDWGc::dwg_setup_BLOCKLOOKUPPARAMETER;
*dwg_setup_BLOCKPARAMDEPENDENCYBODY = *LibreDWGc::dwg_setup_BLOCKPARAMDEPENDENCYBODY;
*dwg_setup_BLOCKPOINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKPOINTPARAMETER;
*dwg_setup_BLOCKPOLARGRIP = *LibreDWGc::dwg_setup_BLOCKPOLARGRIP;
*dwg_setup_BLOCKPOLARPARAMETER = *LibreDWGc::dwg_setup_BLOCKPOLARPARAMETER;
*dwg_setup_BLOCKPOLARSTRETCHACTION = *LibreDWGc::dwg_setup_BLOCKPOLARSTRETCHACTION;
*dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKRADIALCONSTRAINTPARAMETER;
*dwg_setup_BLOCKREPRESENTATION = *LibreDWGc::dwg_setup_BLOCKREPRESENTATION;
*dwg_setup_BLOCKROTATIONPARAMETER = *LibreDWGc::dwg_setup_BLOCKROTATIONPARAMETER;
*dwg_setup_BLOCKSTRETCHACTION = *LibreDWGc::dwg_setup_BLOCKSTRETCHACTION;
*dwg_setup_BLOCKUSERPARAMETER = *LibreDWGc::dwg_setup_BLOCKUSERPARAMETER;
*dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER = *LibreDWGc::dwg_setup_BLOCKVERTICALCONSTRAINTPARAMETER;
*dwg_setup_BLOCKVISIBILITYPARAMETER = *LibreDWGc::dwg_setup_BLOCKVISIBILITYPARAMETER;
*dwg_setup_BLOCKXYGRIP = *LibreDWGc::dwg_setup_BLOCKXYGRIP;
*dwg_setup_BLOCKXYPARAMETER = *LibreDWGc::dwg_setup_BLOCKXYPARAMETER;
*dwg_setup_DATALINK = *LibreDWGc::dwg_setup_DATALINK;
*dwg_setup_DBCOLOR = *LibreDWGc::dwg_setup_DBCOLOR;
*dwg_setup_EVALUATION_GRAPH = *LibreDWGc::dwg_setup_EVALUATION_GRAPH;
*dwg_setup_FCFOBJECTCONTEXTDATA = *LibreDWGc::dwg_setup_FCFOBJECTCONTEXTDATA;
*dwg_setup_GRADIENT_BACKGROUND = *LibreDWGc::dwg_setup_GRADIENT_BACKGROUND;
*dwg_setup_GROUND_PLANE_BACKGROUND = *LibreDWGc::dwg_setup_GROUND_PLANE_BACKGROUND;
*dwg_setup_IBL_BACKGROUND = *LibreDWGc::dwg_setup_IBL_BACKGROUND;
*dwg_setup_IMAGE_BACKGROUND = *LibreDWGc::dwg_setup_IMAGE_BACKGROUND;
*dwg_setup_LEADEROBJECTCONTEXTDATA = *LibreDWGc::dwg_setup_LEADEROBJECTCONTEXTDATA;
*dwg_setup_LIGHTLIST = *LibreDWGc::dwg_setup_LIGHTLIST;
*dwg_setup_MATERIAL = *LibreDWGc::dwg_setup_MATERIAL;
*dwg_setup_MENTALRAYRENDERSETTINGS = *LibreDWGc::dwg_setup_MENTALRAYRENDERSETTINGS;
*dwg_setup_MLEADERSTYLE = *LibreDWGc::dwg_setup_MLEADERSTYLE;
*dwg_setup_MTEXTOBJECTCONTEXTDATA = *LibreDWGc::dwg_setup_MTEXTOBJECTCONTEXTDATA;
*dwg_setup_OBJECT_PTR = *LibreDWGc::dwg_setup_OBJECT_PTR;
*dwg_setup_PARTIAL_VIEWING_INDEX = *LibreDWGc::dwg_setup_PARTIAL_VIEWING_INDEX;
*dwg_setup_POINTCLOUDCOLORMAP = *LibreDWGc::dwg_setup_POINTCLOUDCOLORMAP;
*dwg_setup_POINTCLOUDDEF = *LibreDWGc::dwg_setup_POINTCLOUDDEF;
*dwg_setup_POINTCLOUDDEFEX = *LibreDWGc::dwg_setup_POINTCLOUDDEFEX;
*dwg_setup_POINTCLOUDDEF_REACTOR = *LibreDWGc::dwg_setup_POINTCLOUDDEF_REACTOR;
*dwg_setup_POINTCLOUDDEF_REACTOR_EX = *LibreDWGc::dwg_setup_POINTCLOUDDEF_REACTOR_EX;
*dwg_setup_PROXY_OBJECT = *LibreDWGc::dwg_setup_PROXY_OBJECT;
*dwg_setup_RAPIDRTRENDERSETTINGS = *LibreDWGc::dwg_setup_RAPIDRTRENDERSETTINGS;
*dwg_setup_RENDERENTRY = *LibreDWGc::dwg_setup_RENDERENTRY;
*dwg_setup_RENDERGLOBAL = *LibreDWGc::dwg_setup_RENDERGLOBAL;
*dwg_setup_RENDERSETTINGS = *LibreDWGc::dwg_setup_RENDERSETTINGS;
*dwg_setup_SECTION_SETTINGS = *LibreDWGc::dwg_setup_SECTION_SETTINGS;
*dwg_setup_SKYLIGHT_BACKGROUND = *LibreDWGc::dwg_setup_SKYLIGHT_BACKGROUND;
*dwg_setup_SOLID_BACKGROUND = *LibreDWGc::dwg_setup_SOLID_BACKGROUND;
*dwg_setup_SPATIAL_INDEX = *LibreDWGc::dwg_setup_SPATIAL_INDEX;
*dwg_setup_TABLESTYLE = *LibreDWGc::dwg_setup_TABLESTYLE;
*dwg_setup_TEXTOBJECTCONTEXTDATA = *LibreDWGc::dwg_setup_TEXTOBJECTCONTEXTDATA;
*dwg_setup_ASSOCARRAYMODIFYPARAMETERS = *LibreDWGc::dwg_setup_ASSOCARRAYMODIFYPARAMETERS;
*dwg_setup_ASSOCARRAYPATHPARAMETERS = *LibreDWGc::dwg_setup_ASSOCARRAYPATHPARAMETERS;
*dwg_setup_ASSOCARRAYPOLARPARAMETERS = *LibreDWGc::dwg_setup_ASSOCARRAYPOLARPARAMETERS;
*dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS = *LibreDWGc::dwg_setup_ASSOCARRAYRECTANGULARPARAMETERS;
*is_dwg_entity = *LibreDWGc::is_dwg_entity;
*is_dwg_object = *LibreDWGc::is_dwg_object;
*dwg_dynapi_header_value = *LibreDWGc::dwg_dynapi_header_value;
*dwg_dynapi_entity_value = *LibreDWGc::dwg_dynapi_entity_value;
*dwg_dynapi_common_value = *LibreDWGc::dwg_dynapi_common_value;
*dwg_dynapi_subclass_value = *LibreDWGc::dwg_dynapi_subclass_value;
*dwg_dynapi_header_utf8text = *LibreDWGc::dwg_dynapi_header_utf8text;
*dwg_dynapi_entity_utf8text = *LibreDWGc::dwg_dynapi_entity_utf8text;
*dwg_dynapi_common_utf8text = *LibreDWGc::dwg_dynapi_common_utf8text;
*dwg_dynapi_header_set_value = *LibreDWGc::dwg_dynapi_header_set_value;
*dwg_dynapi_entity_set_value = *LibreDWGc::dwg_dynapi_entity_set_value;
*dwg_dynapi_common_set_value = *LibreDWGc::dwg_dynapi_common_set_value;
*dwg_dynapi_handle_name = *LibreDWGc::dwg_dynapi_handle_name;
*dwg_dynapi_header_field = *LibreDWGc::dwg_dynapi_header_field;
*dwg_dynapi_entity_field = *LibreDWGc::dwg_dynapi_entity_field;
*dwg_dynapi_subclass_field = *LibreDWGc::dwg_dynapi_subclass_field;
*dwg_dynapi_common_entity_field = *LibreDWGc::dwg_dynapi_common_entity_field;
*dwg_dynapi_common_object_field = *LibreDWGc::dwg_dynapi_common_object_field;
*dwg_dynapi_entity_fields = *LibreDWGc::dwg_dynapi_entity_fields;
*dwg_dynapi_common_entity_fields = *LibreDWGc::dwg_dynapi_common_entity_fields;
*dwg_dynapi_common_object_fields = *LibreDWGc::dwg_dynapi_common_object_fields;
*dwg_dynapi_subclass_fields = *LibreDWGc::dwg_dynapi_subclass_fields;
*dwg_dynapi_fields_size = *LibreDWGc::dwg_dynapi_fields_size;
*dwg_object_tablectrl_get_entries = *LibreDWGc::dwg_object_tablectrl_get_entries;
*dwg_ref_get_table_name = *LibreDWGc::dwg_ref_get_table_name;
*dwg_ent_get_layer_name = *LibreDWGc::dwg_ent_get_layer_name;
*dwg_obj_layer_get_name = *LibreDWGc::dwg_obj_layer_get_name;
*dwg_obj_table_get_name = *LibreDWGc::dwg_obj_table_get_name;
*dwg_ent_to_object = *LibreDWGc::dwg_ent_to_object;
*dwg_object_to_object = *LibreDWGc::dwg_object_to_object;
*dwg_obj_obj_to_object = *LibreDWGc::dwg_obj_obj_to_object;
*dwg_object_to_entity = *LibreDWGc::dwg_object_to_entity;
*dwg_object_get_type = *LibreDWGc::dwg_object_get_type;
*dwg_object_get_fixedtype = *LibreDWGc::dwg_object_get_fixedtype;
*dwg_object_get_dxfname = *LibreDWGc::dwg_object_get_dxfname;
*dwg_ref_get_absref = *LibreDWGc::dwg_ref_get_absref;
*dwg_ref_get_object = *LibreDWGc::dwg_ref_get_object;
*dwg_absref_get_object = *LibreDWGc::dwg_absref_get_object;
*dwg_get_num_classes = *LibreDWGc::dwg_get_num_classes;
*dwg_get_class = *LibreDWGc::dwg_get_class;
*dwg_get_object = *LibreDWGc::dwg_get_object;
*dwg_obj_generic_to_object = *LibreDWGc::dwg_obj_generic_to_object;
*dwg_obj_generic_parent = *LibreDWGc::dwg_obj_generic_parent;
*dwg_object_to_DIMENSION = *LibreDWGc::dwg_object_to_DIMENSION;
*dwg_getall__3DFACE = *LibreDWGc::dwg_getall__3DFACE;
*dwg_getall__3DLINE = *LibreDWGc::dwg_getall__3DLINE;
*dwg_getall__3DSOLID = *LibreDWGc::dwg_getall__3DSOLID;
*dwg_getall_ARC = *LibreDWGc::dwg_getall_ARC;
*dwg_getall_ATTDEF = *LibreDWGc::dwg_getall_ATTDEF;
*dwg_getall_ATTRIB = *LibreDWGc::dwg_getall_ATTRIB;
*dwg_getall_BLOCK = *LibreDWGc::dwg_getall_BLOCK;
*dwg_getall_BODY = *LibreDWGc::dwg_getall_BODY;
*dwg_getall_CAMERA = *LibreDWGc::dwg_getall_CAMERA;
*dwg_getall_CIRCLE = *LibreDWGc::dwg_getall_CIRCLE;
*dwg_getall_DGNUNDERLAY = *LibreDWGc::dwg_getall_DGNUNDERLAY;
*dwg_getall_DIMENSION_ALIGNED = *LibreDWGc::dwg_getall_DIMENSION_ALIGNED;
*dwg_getall_DIMENSION_ANG2LN = *LibreDWGc::dwg_getall_DIMENSION_ANG2LN;
*dwg_getall_DIMENSION_ANG3PT = *LibreDWGc::dwg_getall_DIMENSION_ANG3PT;
*dwg_getall_DIMENSION_DIAMETER = *LibreDWGc::dwg_getall_DIMENSION_DIAMETER;
*dwg_getall_DIMENSION_LINEAR = *LibreDWGc::dwg_getall_DIMENSION_LINEAR;
*dwg_getall_DIMENSION_ORDINATE = *LibreDWGc::dwg_getall_DIMENSION_ORDINATE;
*dwg_getall_DIMENSION_RADIUS = *LibreDWGc::dwg_getall_DIMENSION_RADIUS;
*dwg_getall_DWFUNDERLAY = *LibreDWGc::dwg_getall_DWFUNDERLAY;
*dwg_getall_ELLIPSE = *LibreDWGc::dwg_getall_ELLIPSE;
*dwg_getall_ENDBLK = *LibreDWGc::dwg_getall_ENDBLK;
*dwg_getall_ENDREP = *LibreDWGc::dwg_getall_ENDREP;
*dwg_getall_HATCH = *LibreDWGc::dwg_getall_HATCH;
*dwg_getall_IMAGE = *LibreDWGc::dwg_getall_IMAGE;
*dwg_getall_INSERT = *LibreDWGc::dwg_getall_INSERT;
*dwg_getall_JUMP = *LibreDWGc::dwg_getall_JUMP;
*dwg_getall_LEADER = *LibreDWGc::dwg_getall_LEADER;
*dwg_getall_LIGHT = *LibreDWGc::dwg_getall_LIGHT;
*dwg_getall_LINE = *LibreDWGc::dwg_getall_LINE;
*dwg_getall_LOAD = *LibreDWGc::dwg_getall_LOAD;
*dwg_getall_LWPOLYLINE = *LibreDWGc::dwg_getall_LWPOLYLINE;
*dwg_getall_MESH = *LibreDWGc::dwg_getall_MESH;
*dwg_getall_MINSERT = *LibreDWGc::dwg_getall_MINSERT;
*dwg_getall_MLINE = *LibreDWGc::dwg_getall_MLINE;
*dwg_getall_MTEXT = *LibreDWGc::dwg_getall_MTEXT;
*dwg_getall_MULTILEADER = *LibreDWGc::dwg_getall_MULTILEADER;
*dwg_getall_OLE2FRAME = *LibreDWGc::dwg_getall_OLE2FRAME;
*dwg_getall_OLEFRAME = *LibreDWGc::dwg_getall_OLEFRAME;
*dwg_getall_PDFUNDERLAY = *LibreDWGc::dwg_getall_PDFUNDERLAY;
*dwg_getall_POINT = *LibreDWGc::dwg_getall_POINT;
*dwg_getall_POLYLINE_2D = *LibreDWGc::dwg_getall_POLYLINE_2D;
*dwg_getall_POLYLINE_3D = *LibreDWGc::dwg_getall_POLYLINE_3D;
*dwg_getall_POLYLINE_MESH = *LibreDWGc::dwg_getall_POLYLINE_MESH;
*dwg_getall_POLYLINE_PFACE = *LibreDWGc::dwg_getall_POLYLINE_PFACE;
*dwg_getall_PROXY_ENTITY = *LibreDWGc::dwg_getall_PROXY_ENTITY;
*dwg_getall_RAY = *LibreDWGc::dwg_getall_RAY;
*dwg_getall_REGION = *LibreDWGc::dwg_getall_REGION;
*dwg_getall_REPEAT = *LibreDWGc::dwg_getall_REPEAT;
*dwg_getall_SECTIONOBJECT = *LibreDWGc::dwg_getall_SECTIONOBJECT;
*dwg_getall_SEQEND = *LibreDWGc::dwg_getall_SEQEND;
*dwg_getall_SHAPE = *LibreDWGc::dwg_getall_SHAPE;
*dwg_getall_SOLID = *LibreDWGc::dwg_getall_SOLID;
*dwg_getall_SPLINE = *LibreDWGc::dwg_getall_SPLINE;
*dwg_getall_TEXT = *LibreDWGc::dwg_getall_TEXT;
*dwg_getall_TOLERANCE = *LibreDWGc::dwg_getall_TOLERANCE;
*dwg_getall_TRACE = *LibreDWGc::dwg_getall_TRACE;
*dwg_getall_UNKNOWN_ENT = *LibreDWGc::dwg_getall_UNKNOWN_ENT;
*dwg_getall_VERTEX_2D = *LibreDWGc::dwg_getall_VERTEX_2D;
*dwg_getall_VERTEX_3D = *LibreDWGc::dwg_getall_VERTEX_3D;
*dwg_getall_VERTEX_MESH = *LibreDWGc::dwg_getall_VERTEX_MESH;
*dwg_getall_VERTEX_PFACE = *LibreDWGc::dwg_getall_VERTEX_PFACE;
*dwg_getall_VERTEX_PFACE_FACE = *LibreDWGc::dwg_getall_VERTEX_PFACE_FACE;
*dwg_getall_VIEWPORT = *LibreDWGc::dwg_getall_VIEWPORT;
*dwg_getall_WIPEOUT = *LibreDWGc::dwg_getall_WIPEOUT;
*dwg_getall_XLINE = *LibreDWGc::dwg_getall_XLINE;
*dwg_getall_ARC_DIMENSION = *LibreDWGc::dwg_getall_ARC_DIMENSION;
*dwg_getall_HELIX = *LibreDWGc::dwg_getall_HELIX;
*dwg_getall_LARGE_RADIAL_DIMENSION = *LibreDWGc::dwg_getall_LARGE_RADIAL_DIMENSION;
*dwg_getall_LAYOUTPRINTCONFIG = *LibreDWGc::dwg_getall_LAYOUTPRINTCONFIG;
*dwg_getall_PLANESURFACE = *LibreDWGc::dwg_getall_PLANESURFACE;
*dwg_getall_POINTCLOUD = *LibreDWGc::dwg_getall_POINTCLOUD;
*dwg_getall_POINTCLOUDEX = *LibreDWGc::dwg_getall_POINTCLOUDEX;
*dwg_getall_ACSH_BOOLEAN_CLASS = *LibreDWGc::dwg_getall_ACSH_BOOLEAN_CLASS;
*dwg_getall_ACSH_BOX_CLASS = *LibreDWGc::dwg_getall_ACSH_BOX_CLASS;
*dwg_getall_ACSH_CONE_CLASS = *LibreDWGc::dwg_getall_ACSH_CONE_CLASS;
*dwg_getall_ACSH_CYLINDER_CLASS = *LibreDWGc::dwg_getall_ACSH_CYLINDER_CLASS;
*dwg_getall_ACSH_FILLET_CLASS = *LibreDWGc::dwg_getall_ACSH_FILLET_CLASS;
*dwg_getall_ACSH_HISTORY_CLASS = *LibreDWGc::dwg_getall_ACSH_HISTORY_CLASS;
*dwg_getall_ACSH_SPHERE_CLASS = *LibreDWGc::dwg_getall_ACSH_SPHERE_CLASS;
*dwg_getall_ACSH_TORUS_CLASS = *LibreDWGc::dwg_getall_ACSH_TORUS_CLASS;
*dwg_getall_ACSH_WEDGE_CLASS = *LibreDWGc::dwg_getall_ACSH_WEDGE_CLASS;
*dwg_getall_APPID = *LibreDWGc::dwg_getall_APPID;
*dwg_getall_APPID_CONTROL = *LibreDWGc::dwg_getall_APPID_CONTROL;
*dwg_getall_ASSOCGEOMDEPENDENCY = *LibreDWGc::dwg_getall_ASSOCGEOMDEPENDENCY;
*dwg_getall_ASSOCNETWORK = *LibreDWGc::dwg_getall_ASSOCNETWORK;
*dwg_getall_BLOCKALIGNMENTGRIP = *LibreDWGc::dwg_getall_BLOCKALIGNMENTGRIP;
*dwg_getall_BLOCKALIGNMENTPARAMETER = *LibreDWGc::dwg_getall_BLOCKALIGNMENTPARAMETER;
*dwg_getall_BLOCKBASEPOINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKBASEPOINTPARAMETER;
*dwg_getall_BLOCKFLIPACTION = *LibreDWGc::dwg_getall_BLOCKFLIPACTION;
*dwg_getall_BLOCKFLIPGRIP = *LibreDWGc::dwg_getall_BLOCKFLIPGRIP;
*dwg_getall_BLOCKFLIPPARAMETER = *LibreDWGc::dwg_getall_BLOCKFLIPPARAMETER;
*dwg_getall_BLOCKGRIPLOCATIONCOMPONENT = *LibreDWGc::dwg_getall_BLOCKGRIPLOCATIONCOMPONENT;
*dwg_getall_BLOCKLINEARGRIP = *LibreDWGc::dwg_getall_BLOCKLINEARGRIP;
*dwg_getall_BLOCKLOOKUPGRIP = *LibreDWGc::dwg_getall_BLOCKLOOKUPGRIP;
*dwg_getall_BLOCKMOVEACTION = *LibreDWGc::dwg_getall_BLOCKMOVEACTION;
*dwg_getall_BLOCKROTATEACTION = *LibreDWGc::dwg_getall_BLOCKROTATEACTION;
*dwg_getall_BLOCKROTATIONGRIP = *LibreDWGc::dwg_getall_BLOCKROTATIONGRIP;
*dwg_getall_BLOCKSCALEACTION = *LibreDWGc::dwg_getall_BLOCKSCALEACTION;
*dwg_getall_BLOCKVISIBILITYGRIP = *LibreDWGc::dwg_getall_BLOCKVISIBILITYGRIP;
*dwg_getall_BLOCK_CONTROL = *LibreDWGc::dwg_getall_BLOCK_CONTROL;
*dwg_getall_BLOCK_HEADER = *LibreDWGc::dwg_getall_BLOCK_HEADER;
*dwg_getall_CELLSTYLEMAP = *LibreDWGc::dwg_getall_CELLSTYLEMAP;
*dwg_getall_DETAILVIEWSTYLE = *LibreDWGc::dwg_getall_DETAILVIEWSTYLE;
*dwg_getall_DICTIONARY = *LibreDWGc::dwg_getall_DICTIONARY;
*dwg_getall_DICTIONARYVAR = *LibreDWGc::dwg_getall_DICTIONARYVAR;
*dwg_getall_DICTIONARYWDFLT = *LibreDWGc::dwg_getall_DICTIONARYWDFLT;
*dwg_getall_DIMSTYLE = *LibreDWGc::dwg_getall_DIMSTYLE;
*dwg_getall_DIMSTYLE_CONTROL = *LibreDWGc::dwg_getall_DIMSTYLE_CONTROL;
*dwg_getall_DUMMY = *LibreDWGc::dwg_getall_DUMMY;
*dwg_getall_DYNAMICBLOCKPURGEPREVENTER = *LibreDWGc::dwg_getall_DYNAMICBLOCKPURGEPREVENTER;
*dwg_getall_FIELD = *LibreDWGc::dwg_getall_FIELD;
*dwg_getall_FIELDLIST = *LibreDWGc::dwg_getall_FIELDLIST;
*dwg_getall_GEODATA = *LibreDWGc::dwg_getall_GEODATA;
*dwg_getall_GROUP = *LibreDWGc::dwg_getall_GROUP;
*dwg_getall_IDBUFFER = *LibreDWGc::dwg_getall_IDBUFFER;
*dwg_getall_IMAGEDEF = *LibreDWGc::dwg_getall_IMAGEDEF;
*dwg_getall_IMAGEDEF_REACTOR = *LibreDWGc::dwg_getall_IMAGEDEF_REACTOR;
*dwg_getall_INDEX = *LibreDWGc::dwg_getall_INDEX;
*dwg_getall_LAYER = *LibreDWGc::dwg_getall_LAYER;
*dwg_getall_LAYERFILTER = *LibreDWGc::dwg_getall_LAYERFILTER;
*dwg_getall_LAYER_CONTROL = *LibreDWGc::dwg_getall_LAYER_CONTROL;
*dwg_getall_LAYER_INDEX = *LibreDWGc::dwg_getall_LAYER_INDEX;
*dwg_getall_LAYOUT = *LibreDWGc::dwg_getall_LAYOUT;
*dwg_getall_LONG_TRANSACTION = *LibreDWGc::dwg_getall_LONG_TRANSACTION;
*dwg_getall_LTYPE = *LibreDWGc::dwg_getall_LTYPE;
*dwg_getall_LTYPE_CONTROL = *LibreDWGc::dwg_getall_LTYPE_CONTROL;
*dwg_getall_MLINESTYLE = *LibreDWGc::dwg_getall_MLINESTYLE;
*dwg_getall_PLACEHOLDER = *LibreDWGc::dwg_getall_PLACEHOLDER;
*dwg_getall_PLOTSETTINGS = *LibreDWGc::dwg_getall_PLOTSETTINGS;
*dwg_getall_RASTERVARIABLES = *LibreDWGc::dwg_getall_RASTERVARIABLES;
*dwg_getall_RENDERENVIRONMENT = *LibreDWGc::dwg_getall_RENDERENVIRONMENT;
*dwg_getall_SCALE = *LibreDWGc::dwg_getall_SCALE;
*dwg_getall_SECTIONVIEWSTYLE = *LibreDWGc::dwg_getall_SECTIONVIEWSTYLE;
*dwg_getall_SECTION_MANAGER = *LibreDWGc::dwg_getall_SECTION_MANAGER;
*dwg_getall_SORTENTSTABLE = *LibreDWGc::dwg_getall_SORTENTSTABLE;
*dwg_getall_SPATIAL_FILTER = *LibreDWGc::dwg_getall_SPATIAL_FILTER;
*dwg_getall_STYLE = *LibreDWGc::dwg_getall_STYLE;
*dwg_getall_STYLE_CONTROL = *LibreDWGc::dwg_getall_STYLE_CONTROL;
*dwg_getall_SUN = *LibreDWGc::dwg_getall_SUN;
*dwg_getall_TABLEGEOMETRY = *LibreDWGc::dwg_getall_TABLEGEOMETRY;
*dwg_getall_UCS = *LibreDWGc::dwg_getall_UCS;
*dwg_getall_UCS_CONTROL = *LibreDWGc::dwg_getall_UCS_CONTROL;
*dwg_getall_UNKNOWN_OBJ = *LibreDWGc::dwg_getall_UNKNOWN_OBJ;
*dwg_getall_VBA_PROJECT = *LibreDWGc::dwg_getall_VBA_PROJECT;
*dwg_getall_VIEW = *LibreDWGc::dwg_getall_VIEW;
*dwg_getall_VIEW_CONTROL = *LibreDWGc::dwg_getall_VIEW_CONTROL;
*dwg_getall_VISUALSTYLE = *LibreDWGc::dwg_getall_VISUALSTYLE;
*dwg_getall_VPORT = *LibreDWGc::dwg_getall_VPORT;
*dwg_getall_VPORT_CONTROL = *LibreDWGc::dwg_getall_VPORT_CONTROL;
*dwg_getall_VX_CONTROL = *LibreDWGc::dwg_getall_VX_CONTROL;
*dwg_getall_VX_TABLE_RECORD = *LibreDWGc::dwg_getall_VX_TABLE_RECORD;
*dwg_getall_WIPEOUTVARIABLES = *LibreDWGc::dwg_getall_WIPEOUTVARIABLES;
*dwg_getall_XRECORD = *LibreDWGc::dwg_getall_XRECORD;
*dwg_getall_PDFDEFINITION = *LibreDWGc::dwg_getall_PDFDEFINITION;
*dwg_getall_DGNDEFINITION = *LibreDWGc::dwg_getall_DGNDEFINITION;
*dwg_getall_DWFDEFINITION = *LibreDWGc::dwg_getall_DWFDEFINITION;
*dwg_getall_ACSH_BREP_CLASS = *LibreDWGc::dwg_getall_ACSH_BREP_CLASS;
*dwg_getall_ACSH_CHAMFER_CLASS = *LibreDWGc::dwg_getall_ACSH_CHAMFER_CLASS;
*dwg_getall_ACSH_PYRAMID_CLASS = *LibreDWGc::dwg_getall_ACSH_PYRAMID_CLASS;
*dwg_getall_ALDIMOBJECTCONTEXTDATA = *LibreDWGc::dwg_getall_ALDIMOBJECTCONTEXTDATA;
*dwg_getall_ASSOC2DCONSTRAINTGROUP = *LibreDWGc::dwg_getall_ASSOC2DCONSTRAINTGROUP;
*dwg_getall_ASSOCACTION = *LibreDWGc::dwg_getall_ASSOCACTION;
*dwg_getall_ASSOCACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCACTIONPARAM;
*dwg_getall_ASSOCARRAYACTIONBODY = *LibreDWGc::dwg_getall_ASSOCARRAYACTIONBODY;
*dwg_getall_ASSOCASMBODYACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCASMBODYACTIONPARAM;
*dwg_getall_ASSOCBLENDSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCBLENDSURFACEACTIONBODY;
*dwg_getall_ASSOCCOMPOUNDACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCCOMPOUNDACTIONPARAM;
*dwg_getall_ASSOCDEPENDENCY = *LibreDWGc::dwg_getall_ASSOCDEPENDENCY;
*dwg_getall_ASSOCDIMDEPENDENCYBODY = *LibreDWGc::dwg_getall_ASSOCDIMDEPENDENCYBODY;
*dwg_getall_ASSOCEXTENDSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCEXTENDSURFACEACTIONBODY;
*dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCEXTRUDEDSURFACEACTIONBODY;
*dwg_getall_ASSOCFACEACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCFACEACTIONPARAM;
*dwg_getall_ASSOCFILLETSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCFILLETSURFACEACTIONBODY;
*dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCLOFTEDSURFACEACTIONBODY;
*dwg_getall_ASSOCNETWORKSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCNETWORKSURFACEACTIONBODY;
*dwg_getall_ASSOCOBJECTACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCOBJECTACTIONPARAM;
*dwg_getall_ASSOCOFFSETSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCOFFSETSURFACEACTIONBODY;
*dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCOSNAPPOINTREFACTIONPARAM;
*dwg_getall_ASSOCPATCHSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCPATCHSURFACEACTIONBODY;
*dwg_getall_ASSOCPATHACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCPATHACTIONPARAM;
*dwg_getall_ASSOCPLANESURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCPLANESURFACEACTIONBODY;
*dwg_getall_ASSOCPOINTREFACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCPOINTREFACTIONPARAM;
*dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCREVOLVEDSURFACEACTIONBODY;
*dwg_getall_ASSOCTRIMSURFACEACTIONBODY = *LibreDWGc::dwg_getall_ASSOCTRIMSURFACEACTIONBODY;
*dwg_getall_ASSOCVALUEDEPENDENCY = *LibreDWGc::dwg_getall_ASSOCVALUEDEPENDENCY;
*dwg_getall_ASSOCVARIABLE = *LibreDWGc::dwg_getall_ASSOCVARIABLE;
*dwg_getall_ASSOCVERTEXACTIONPARAM = *LibreDWGc::dwg_getall_ASSOCVERTEXACTIONPARAM;
*dwg_getall_BLKREFOBJECTCONTEXTDATA = *LibreDWGc::dwg_getall_BLKREFOBJECTCONTEXTDATA;
*dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKALIGNEDCONSTRAINTPARAMETER;
*dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKANGULARCONSTRAINTPARAMETER;
*dwg_getall_BLOCKARRAYACTION = *LibreDWGc::dwg_getall_BLOCKARRAYACTION;
*dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKDIAMETRICCONSTRAINTPARAMETER;
*dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKHORIZONTALCONSTRAINTPARAMETER;
*dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKLINEARCONSTRAINTPARAMETER;
*dwg_getall_BLOCKLINEARPARAMETER = *LibreDWGc::dwg_getall_BLOCKLINEARPARAMETER;
*dwg_getall_BLOCKLOOKUPACTION = *LibreDWGc::dwg_getall_BLOCKLOOKUPACTION;
*dwg_getall_BLOCKLOOKUPPARAMETER = *LibreDWGc::dwg_getall_BLOCKLOOKUPPARAMETER;
*dwg_getall_BLOCKPARAMDEPENDENCYBODY = *LibreDWGc::dwg_getall_BLOCKPARAMDEPENDENCYBODY;
*dwg_getall_BLOCKPOINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKPOINTPARAMETER;
*dwg_getall_BLOCKPOLARGRIP = *LibreDWGc::dwg_getall_BLOCKPOLARGRIP;
*dwg_getall_BLOCKPOLARPARAMETER = *LibreDWGc::dwg_getall_BLOCKPOLARPARAMETER;
*dwg_getall_BLOCKPOLARSTRETCHACTION = *LibreDWGc::dwg_getall_BLOCKPOLARSTRETCHACTION;
*dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKRADIALCONSTRAINTPARAMETER;
*dwg_getall_BLOCKREPRESENTATION = *LibreDWGc::dwg_getall_BLOCKREPRESENTATION;
*dwg_getall_BLOCKROTATIONPARAMETER = *LibreDWGc::dwg_getall_BLOCKROTATIONPARAMETER;
*dwg_getall_BLOCKSTRETCHACTION = *LibreDWGc::dwg_getall_BLOCKSTRETCHACTION;
*dwg_getall_BLOCKUSERPARAMETER = *LibreDWGc::dwg_getall_BLOCKUSERPARAMETER;
*dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER = *LibreDWGc::dwg_getall_BLOCKVERTICALCONSTRAINTPARAMETER;
*dwg_getall_BLOCKVISIBILITYPARAMETER = *LibreDWGc::dwg_getall_BLOCKVISIBILITYPARAMETER;
*dwg_getall_BLOCKXYGRIP = *LibreDWGc::dwg_getall_BLOCKXYGRIP;
*dwg_getall_BLOCKXYPARAMETER = *LibreDWGc::dwg_getall_BLOCKXYPARAMETER;
*dwg_getall_DATALINK = *LibreDWGc::dwg_getall_DATALINK;
*dwg_getall_DBCOLOR = *LibreDWGc::dwg_getall_DBCOLOR;
*dwg_getall_EVALUATION_GRAPH = *LibreDWGc::dwg_getall_EVALUATION_GRAPH;
*dwg_getall_FCFOBJECTCONTEXTDATA = *LibreDWGc::dwg_getall_FCFOBJECTCONTEXTDATA;
*dwg_getall_GRADIENT_BACKGROUND = *LibreDWGc::dwg_getall_GRADIENT_BACKGROUND;
*dwg_getall_GROUND_PLANE_BACKGROUND = *LibreDWGc::dwg_getall_GROUND_PLANE_BACKGROUND;
*dwg_getall_IBL_BACKGROUND = *LibreDWGc::dwg_getall_IBL_BACKGROUND;
*dwg_getall_IMAGE_BACKGROUND = *LibreDWGc::dwg_getall_IMAGE_BACKGROUND;
*dwg_getall_LEADEROBJECTCONTEXTDATA = *LibreDWGc::dwg_getall_LEADEROBJECTCONTEXTDATA;
*dwg_getall_LIGHTLIST = *LibreDWGc::dwg_getall_LIGHTLIST;
*dwg_getall_MATERIAL = *LibreDWGc::dwg_getall_MATERIAL;
*dwg_getall_MENTALRAYRENDERSETTINGS = *LibreDWGc::dwg_getall_MENTALRAYRENDERSETTINGS;
*dwg_getall_MLEADERSTYLE = *LibreDWGc::dwg_getall_MLEADERSTYLE;
*dwg_getall_MTEXTOBJECTCONTEXTDATA = *LibreDWGc::dwg_getall_MTEXTOBJECTCONTEXTDATA;
*dwg_getall_OBJECT_PTR = *LibreDWGc::dwg_getall_OBJECT_PTR;
*dwg_getall_PARTIAL_VIEWING_INDEX = *LibreDWGc::dwg_getall_PARTIAL_VIEWING_INDEX;
*dwg_getall_POINTCLOUDCOLORMAP = *LibreDWGc::dwg_getall_POINTCLOUDCOLORMAP;
*dwg_getall_POINTCLOUDDEF = *LibreDWGc::dwg_getall_POINTCLOUDDEF;
*dwg_getall_POINTCLOUDDEFEX = *LibreDWGc::dwg_getall_POINTCLOUDDEFEX;
*dwg_getall_POINTCLOUDDEF_REACTOR = *LibreDWGc::dwg_getall_POINTCLOUDDEF_REACTOR;
*dwg_getall_POINTCLOUDDEF_REACTOR_EX = *LibreDWGc::dwg_getall_POINTCLOUDDEF_REACTOR_EX;
*dwg_getall_PROXY_OBJECT = *LibreDWGc::dwg_getall_PROXY_OBJECT;
*dwg_getall_RAPIDRTRENDERSETTINGS = *LibreDWGc::dwg_getall_RAPIDRTRENDERSETTINGS;
*dwg_getall_RENDERENTRY = *LibreDWGc::dwg_getall_RENDERENTRY;
*dwg_getall_RENDERGLOBAL = *LibreDWGc::dwg_getall_RENDERGLOBAL;
*dwg_getall_RENDERSETTINGS = *LibreDWGc::dwg_getall_RENDERSETTINGS;
*dwg_getall_SECTION_SETTINGS = *LibreDWGc::dwg_getall_SECTION_SETTINGS;
*dwg_getall_SKYLIGHT_BACKGROUND = *LibreDWGc::dwg_getall_SKYLIGHT_BACKGROUND;
*dwg_getall_SOLID_BACKGROUND = *LibreDWGc::dwg_getall_SOLID_BACKGROUND;
*dwg_getall_SPATIAL_INDEX = *LibreDWGc::dwg_getall_SPATIAL_INDEX;
*dwg_getall_TABLESTYLE = *LibreDWGc::dwg_getall_TABLESTYLE;
*dwg_getall_TEXTOBJECTCONTEXTDATA = *LibreDWGc::dwg_getall_TEXTOBJECTCONTEXTDATA;
*dwg_getall_ASSOCARRAYMODIFYPARAMETERS = *LibreDWGc::dwg_getall_ASSOCARRAYMODIFYPARAMETERS;
*dwg_getall_ASSOCARRAYPATHPARAMETERS = *LibreDWGc::dwg_getall_ASSOCARRAYPATHPARAMETERS;
*dwg_getall_ASSOCARRAYPOLARPARAMETERS = *LibreDWGc::dwg_getall_ASSOCARRAYPOLARPARAMETERS;
*dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS = *LibreDWGc::dwg_getall_ASSOCARRAYRECTANGULARPARAMETERS;
*dwg_object_to__3DFACE = *LibreDWGc::dwg_object_to__3DFACE;
*dwg_object_to__3DLINE = *LibreDWGc::dwg_object_to__3DLINE;
*dwg_object_to__3DSOLID = *LibreDWGc::dwg_object_to__3DSOLID;
*dwg_object_to_ARC = *LibreDWGc::dwg_object_to_ARC;
*dwg_object_to_ATTDEF = *LibreDWGc::dwg_object_to_ATTDEF;
*dwg_object_to_ATTRIB = *LibreDWGc::dwg_object_to_ATTRIB;
*dwg_object_to_BLOCK = *LibreDWGc::dwg_object_to_BLOCK;
*dwg_object_to_BODY = *LibreDWGc::dwg_object_to_BODY;
*dwg_object_to_CAMERA = *LibreDWGc::dwg_object_to_CAMERA;
*dwg_object_to_CIRCLE = *LibreDWGc::dwg_object_to_CIRCLE;
*dwg_object_to_DGNUNDERLAY = *LibreDWGc::dwg_object_to_DGNUNDERLAY;
*dwg_object_to_DIMENSION_ALIGNED = *LibreDWGc::dwg_object_to_DIMENSION_ALIGNED;
*dwg_object_to_DIMENSION_ANG2LN = *LibreDWGc::dwg_object_to_DIMENSION_ANG2LN;
*dwg_object_to_DIMENSION_ANG3PT = *LibreDWGc::dwg_object_to_DIMENSION_ANG3PT;
*dwg_object_to_DIMENSION_DIAMETER = *LibreDWGc::dwg_object_to_DIMENSION_DIAMETER;
*dwg_object_to_DIMENSION_LINEAR = *LibreDWGc::dwg_object_to_DIMENSION_LINEAR;
*dwg_object_to_DIMENSION_ORDINATE = *LibreDWGc::dwg_object_to_DIMENSION_ORDINATE;
*dwg_object_to_DIMENSION_RADIUS = *LibreDWGc::dwg_object_to_DIMENSION_RADIUS;
*dwg_object_to_DWFUNDERLAY = *LibreDWGc::dwg_object_to_DWFUNDERLAY;
*dwg_object_to_ELLIPSE = *LibreDWGc::dwg_object_to_ELLIPSE;
*dwg_object_to_ENDBLK = *LibreDWGc::dwg_object_to_ENDBLK;
*dwg_object_to_ENDREP = *LibreDWGc::dwg_object_to_ENDREP;
*dwg_object_to_HATCH = *LibreDWGc::dwg_object_to_HATCH;
*dwg_object_to_IMAGE = *LibreDWGc::dwg_object_to_IMAGE;
*dwg_object_to_INSERT = *LibreDWGc::dwg_object_to_INSERT;
*dwg_object_to_JUMP = *LibreDWGc::dwg_object_to_JUMP;
*dwg_object_to_LEADER = *LibreDWGc::dwg_object_to_LEADER;
*dwg_object_to_LIGHT = *LibreDWGc::dwg_object_to_LIGHT;
*dwg_object_to_LINE = *LibreDWGc::dwg_object_to_LINE;
*dwg_object_to_LOAD = *LibreDWGc::dwg_object_to_LOAD;
*dwg_object_to_LWPOLYLINE = *LibreDWGc::dwg_object_to_LWPOLYLINE;
*dwg_object_to_MESH = *LibreDWGc::dwg_object_to_MESH;
*dwg_object_to_MINSERT = *LibreDWGc::dwg_object_to_MINSERT;
*dwg_object_to_MLINE = *LibreDWGc::dwg_object_to_MLINE;
*dwg_object_to_MTEXT = *LibreDWGc::dwg_object_to_MTEXT;
*dwg_object_to_MULTILEADER = *LibreDWGc::dwg_object_to_MULTILEADER;
*dwg_object_to_OLE2FRAME = *LibreDWGc::dwg_object_to_OLE2FRAME;
*dwg_object_to_OLEFRAME = *LibreDWGc::dwg_object_to_OLEFRAME;
*dwg_object_to_PDFUNDERLAY = *LibreDWGc::dwg_object_to_PDFUNDERLAY;
*dwg_object_to_POINT = *LibreDWGc::dwg_object_to_POINT;
*dwg_object_to_POLYLINE_2D = *LibreDWGc::dwg_object_to_POLYLINE_2D;
*dwg_object_to_POLYLINE_3D = *LibreDWGc::dwg_object_to_POLYLINE_3D;
*dwg_object_to_POLYLINE_MESH = *LibreDWGc::dwg_object_to_POLYLINE_MESH;
*dwg_object_to_POLYLINE_PFACE = *LibreDWGc::dwg_object_to_POLYLINE_PFACE;
*dwg_object_to_PROXY_ENTITY = *LibreDWGc::dwg_object_to_PROXY_ENTITY;
*dwg_object_to_RAY = *LibreDWGc::dwg_object_to_RAY;
*dwg_object_to_REGION = *LibreDWGc::dwg_object_to_REGION;
*dwg_object_to_REPEAT = *LibreDWGc::dwg_object_to_REPEAT;
*dwg_object_to_SECTIONOBJECT = *LibreDWGc::dwg_object_to_SECTIONOBJECT;
*dwg_object_to_SEQEND = *LibreDWGc::dwg_object_to_SEQEND;
*dwg_object_to_SHAPE = *LibreDWGc::dwg_object_to_SHAPE;
*dwg_object_to_SOLID = *LibreDWGc::dwg_object_to_SOLID;
*dwg_object_to_SPLINE = *LibreDWGc::dwg_object_to_SPLINE;
*dwg_object_to_TEXT = *LibreDWGc::dwg_object_to_TEXT;
*dwg_object_to_TOLERANCE = *LibreDWGc::dwg_object_to_TOLERANCE;
*dwg_object_to_TRACE = *LibreDWGc::dwg_object_to_TRACE;
*dwg_object_to_UNKNOWN_ENT = *LibreDWGc::dwg_object_to_UNKNOWN_ENT;
*dwg_object_to_VERTEX_2D = *LibreDWGc::dwg_object_to_VERTEX_2D;
*dwg_object_to_VERTEX_3D = *LibreDWGc::dwg_object_to_VERTEX_3D;
*dwg_object_to_VERTEX_MESH = *LibreDWGc::dwg_object_to_VERTEX_MESH;
*dwg_object_to_VERTEX_PFACE = *LibreDWGc::dwg_object_to_VERTEX_PFACE;
*dwg_object_to_VERTEX_PFACE_FACE = *LibreDWGc::dwg_object_to_VERTEX_PFACE_FACE;
*dwg_object_to_VIEWPORT = *LibreDWGc::dwg_object_to_VIEWPORT;
*dwg_object_to_WIPEOUT = *LibreDWGc::dwg_object_to_WIPEOUT;
*dwg_object_to_XLINE = *LibreDWGc::dwg_object_to_XLINE;
*dwg_object_to_ARC_DIMENSION = *LibreDWGc::dwg_object_to_ARC_DIMENSION;
*dwg_object_to_HELIX = *LibreDWGc::dwg_object_to_HELIX;
*dwg_object_to_LARGE_RADIAL_DIMENSION = *LibreDWGc::dwg_object_to_LARGE_RADIAL_DIMENSION;
*dwg_object_to_LAYOUTPRINTCONFIG = *LibreDWGc::dwg_object_to_LAYOUTPRINTCONFIG;
*dwg_object_to_PLANESURFACE = *LibreDWGc::dwg_object_to_PLANESURFACE;
*dwg_object_to_POINTCLOUD = *LibreDWGc::dwg_object_to_POINTCLOUD;
*dwg_object_to_POINTCLOUDEX = *LibreDWGc::dwg_object_to_POINTCLOUDEX;
*dwg_object_to_ACSH_BOOLEAN_CLASS = *LibreDWGc::dwg_object_to_ACSH_BOOLEAN_CLASS;
*dwg_object_to_ACSH_BOX_CLASS = *LibreDWGc::dwg_object_to_ACSH_BOX_CLASS;
*dwg_object_to_ACSH_CONE_CLASS = *LibreDWGc::dwg_object_to_ACSH_CONE_CLASS;
*dwg_object_to_ACSH_CYLINDER_CLASS = *LibreDWGc::dwg_object_to_ACSH_CYLINDER_CLASS;
*dwg_object_to_ACSH_FILLET_CLASS = *LibreDWGc::dwg_object_to_ACSH_FILLET_CLASS;
*dwg_object_to_ACSH_HISTORY_CLASS = *LibreDWGc::dwg_object_to_ACSH_HISTORY_CLASS;
*dwg_object_to_ACSH_SPHERE_CLASS = *LibreDWGc::dwg_object_to_ACSH_SPHERE_CLASS;
*dwg_object_to_ACSH_TORUS_CLASS = *LibreDWGc::dwg_object_to_ACSH_TORUS_CLASS;
*dwg_object_to_ACSH_WEDGE_CLASS = *LibreDWGc::dwg_object_to_ACSH_WEDGE_CLASS;
*dwg_object_to_APPID = *LibreDWGc::dwg_object_to_APPID;
*dwg_object_to_APPID_CONTROL = *LibreDWGc::dwg_object_to_APPID_CONTROL;
*dwg_object_to_ASSOCGEOMDEPENDENCY = *LibreDWGc::dwg_object_to_ASSOCGEOMDEPENDENCY;
*dwg_object_to_ASSOCNETWORK = *LibreDWGc::dwg_object_to_ASSOCNETWORK;
*dwg_object_to_BLOCKALIGNMENTGRIP = *LibreDWGc::dwg_object_to_BLOCKALIGNMENTGRIP;
*dwg_object_to_BLOCKALIGNMENTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKALIGNMENTPARAMETER;
*dwg_object_to_BLOCKBASEPOINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKBASEPOINTPARAMETER;
*dwg_object_to_BLOCKFLIPACTION = *LibreDWGc::dwg_object_to_BLOCKFLIPACTION;
*dwg_object_to_BLOCKFLIPGRIP = *LibreDWGc::dwg_object_to_BLOCKFLIPGRIP;
*dwg_object_to_BLOCKFLIPPARAMETER = *LibreDWGc::dwg_object_to_BLOCKFLIPPARAMETER;
*dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT = *LibreDWGc::dwg_object_to_BLOCKGRIPLOCATIONCOMPONENT;
*dwg_object_to_BLOCKLINEARGRIP = *LibreDWGc::dwg_object_to_BLOCKLINEARGRIP;
*dwg_object_to_BLOCKLOOKUPGRIP = *LibreDWGc::dwg_object_to_BLOCKLOOKUPGRIP;
*dwg_object_to_BLOCKMOVEACTION = *LibreDWGc::dwg_object_to_BLOCKMOVEACTION;
*dwg_object_to_BLOCKROTATEACTION = *LibreDWGc::dwg_object_to_BLOCKROTATEACTION;
*dwg_object_to_BLOCKROTATIONGRIP = *LibreDWGc::dwg_object_to_BLOCKROTATIONGRIP;
*dwg_object_to_BLOCKSCALEACTION = *LibreDWGc::dwg_object_to_BLOCKSCALEACTION;
*dwg_object_to_BLOCKVISIBILITYGRIP = *LibreDWGc::dwg_object_to_BLOCKVISIBILITYGRIP;
*dwg_object_to_BLOCK_CONTROL = *LibreDWGc::dwg_object_to_BLOCK_CONTROL;
*dwg_object_to_BLOCK_HEADER = *LibreDWGc::dwg_object_to_BLOCK_HEADER;
*dwg_object_to_CELLSTYLEMAP = *LibreDWGc::dwg_object_to_CELLSTYLEMAP;
*dwg_object_to_DETAILVIEWSTYLE = *LibreDWGc::dwg_object_to_DETAILVIEWSTYLE;
*dwg_object_to_DICTIONARY = *LibreDWGc::dwg_object_to_DICTIONARY;
*dwg_object_to_DICTIONARYVAR = *LibreDWGc::dwg_object_to_DICTIONARYVAR;
*dwg_object_to_DICTIONARYWDFLT = *LibreDWGc::dwg_object_to_DICTIONARYWDFLT;
*dwg_object_to_DIMSTYLE = *LibreDWGc::dwg_object_to_DIMSTYLE;
*dwg_object_to_DIMSTYLE_CONTROL = *LibreDWGc::dwg_object_to_DIMSTYLE_CONTROL;
*dwg_object_to_DUMMY = *LibreDWGc::dwg_object_to_DUMMY;
*dwg_object_to_DYNAMICBLOCKPURGEPREVENTER = *LibreDWGc::dwg_object_to_DYNAMICBLOCKPURGEPREVENTER;
*dwg_object_to_FIELD = *LibreDWGc::dwg_object_to_FIELD;
*dwg_object_to_FIELDLIST = *LibreDWGc::dwg_object_to_FIELDLIST;
*dwg_object_to_GEODATA = *LibreDWGc::dwg_object_to_GEODATA;
*dwg_object_to_GROUP = *LibreDWGc::dwg_object_to_GROUP;
*dwg_object_to_IDBUFFER = *LibreDWGc::dwg_object_to_IDBUFFER;
*dwg_object_to_IMAGEDEF = *LibreDWGc::dwg_object_to_IMAGEDEF;
*dwg_object_to_IMAGEDEF_REACTOR = *LibreDWGc::dwg_object_to_IMAGEDEF_REACTOR;
*dwg_object_to_INDEX = *LibreDWGc::dwg_object_to_INDEX;
*dwg_object_to_LAYER = *LibreDWGc::dwg_object_to_LAYER;
*dwg_object_to_LAYERFILTER = *LibreDWGc::dwg_object_to_LAYERFILTER;
*dwg_object_to_LAYER_CONTROL = *LibreDWGc::dwg_object_to_LAYER_CONTROL;
*dwg_object_to_LAYER_INDEX = *LibreDWGc::dwg_object_to_LAYER_INDEX;
*dwg_object_to_LAYOUT = *LibreDWGc::dwg_object_to_LAYOUT;
*dwg_object_to_LONG_TRANSACTION = *LibreDWGc::dwg_object_to_LONG_TRANSACTION;
*dwg_object_to_LTYPE = *LibreDWGc::dwg_object_to_LTYPE;
*dwg_object_to_LTYPE_CONTROL = *LibreDWGc::dwg_object_to_LTYPE_CONTROL;
*dwg_object_to_MLINESTYLE = *LibreDWGc::dwg_object_to_MLINESTYLE;
*dwg_object_to_PLACEHOLDER = *LibreDWGc::dwg_object_to_PLACEHOLDER;
*dwg_object_to_PLOTSETTINGS = *LibreDWGc::dwg_object_to_PLOTSETTINGS;
*dwg_object_to_RASTERVARIABLES = *LibreDWGc::dwg_object_to_RASTERVARIABLES;
*dwg_object_to_RENDERENVIRONMENT = *LibreDWGc::dwg_object_to_RENDERENVIRONMENT;
*dwg_object_to_SCALE = *LibreDWGc::dwg_object_to_SCALE;
*dwg_object_to_SECTIONVIEWSTYLE = *LibreDWGc::dwg_object_to_SECTIONVIEWSTYLE;
*dwg_object_to_SECTION_MANAGER = *LibreDWGc::dwg_object_to_SECTION_MANAGER;
*dwg_object_to_SORTENTSTABLE = *LibreDWGc::dwg_object_to_SORTENTSTABLE;
*dwg_object_to_SPATIAL_FILTER = *LibreDWGc::dwg_object_to_SPATIAL_FILTER;
*dwg_object_to_STYLE = *LibreDWGc::dwg_object_to_STYLE;
*dwg_object_to_STYLE_CONTROL = *LibreDWGc::dwg_object_to_STYLE_CONTROL;
*dwg_object_to_SUN = *LibreDWGc::dwg_object_to_SUN;
*dwg_object_to_TABLEGEOMETRY = *LibreDWGc::dwg_object_to_TABLEGEOMETRY;
*dwg_object_to_UCS = *LibreDWGc::dwg_object_to_UCS;
*dwg_object_to_UCS_CONTROL = *LibreDWGc::dwg_object_to_UCS_CONTROL;
*dwg_object_to_UNKNOWN_OBJ = *LibreDWGc::dwg_object_to_UNKNOWN_OBJ;
*dwg_object_to_VBA_PROJECT = *LibreDWGc::dwg_object_to_VBA_PROJECT;
*dwg_object_to_VIEW = *LibreDWGc::dwg_object_to_VIEW;
*dwg_object_to_VIEW_CONTROL = *LibreDWGc::dwg_object_to_VIEW_CONTROL;
*dwg_object_to_VISUALSTYLE = *LibreDWGc::dwg_object_to_VISUALSTYLE;
*dwg_object_to_VPORT = *LibreDWGc::dwg_object_to_VPORT;
*dwg_object_to_VPORT_CONTROL = *LibreDWGc::dwg_object_to_VPORT_CONTROL;
*dwg_object_to_VX_CONTROL = *LibreDWGc::dwg_object_to_VX_CONTROL;
*dwg_object_to_VX_TABLE_RECORD = *LibreDWGc::dwg_object_to_VX_TABLE_RECORD;
*dwg_object_to_WIPEOUTVARIABLES = *LibreDWGc::dwg_object_to_WIPEOUTVARIABLES;
*dwg_object_to_XRECORD = *LibreDWGc::dwg_object_to_XRECORD;
*dwg_object_to_PDFDEFINITION = *LibreDWGc::dwg_object_to_PDFDEFINITION;
*dwg_object_to_DGNDEFINITION = *LibreDWGc::dwg_object_to_DGNDEFINITION;
*dwg_object_to_DWFDEFINITION = *LibreDWGc::dwg_object_to_DWFDEFINITION;
*dwg_object_to_ACSH_BREP_CLASS = *LibreDWGc::dwg_object_to_ACSH_BREP_CLASS;
*dwg_object_to_ACSH_CHAMFER_CLASS = *LibreDWGc::dwg_object_to_ACSH_CHAMFER_CLASS;
*dwg_object_to_ACSH_PYRAMID_CLASS = *LibreDWGc::dwg_object_to_ACSH_PYRAMID_CLASS;
*dwg_object_to_ALDIMOBJECTCONTEXTDATA = *LibreDWGc::dwg_object_to_ALDIMOBJECTCONTEXTDATA;
*dwg_object_to_ASSOC2DCONSTRAINTGROUP = *LibreDWGc::dwg_object_to_ASSOC2DCONSTRAINTGROUP;
*dwg_object_to_ASSOCACTION = *LibreDWGc::dwg_object_to_ASSOCACTION;
*dwg_object_to_ASSOCACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCACTIONPARAM;
*dwg_object_to_ASSOCARRAYACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCARRAYACTIONBODY;
*dwg_object_to_ASSOCASMBODYACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCASMBODYACTIONPARAM;
*dwg_object_to_ASSOCBLENDSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCBLENDSURFACEACTIONBODY;
*dwg_object_to_ASSOCCOMPOUNDACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCCOMPOUNDACTIONPARAM;
*dwg_object_to_ASSOCDEPENDENCY = *LibreDWGc::dwg_object_to_ASSOCDEPENDENCY;
*dwg_object_to_ASSOCDIMDEPENDENCYBODY = *LibreDWGc::dwg_object_to_ASSOCDIMDEPENDENCYBODY;
*dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCEXTENDSURFACEACTIONBODY;
*dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCEXTRUDEDSURFACEACTIONBODY;
*dwg_object_to_ASSOCFACEACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCFACEACTIONPARAM;
*dwg_object_to_ASSOCFILLETSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCFILLETSURFACEACTIONBODY;
*dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY;
*dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCNETWORKSURFACEACTIONBODY;
*dwg_object_to_ASSOCOBJECTACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCOBJECTACTIONPARAM;
*dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCOFFSETSURFACEACTIONBODY;
*dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCOSNAPPOINTREFACTIONPARAM;
*dwg_object_to_ASSOCPATCHSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCPATCHSURFACEACTIONBODY;
*dwg_object_to_ASSOCPATHACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCPATHACTIONPARAM;
*dwg_object_to_ASSOCPLANESURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCPLANESURFACEACTIONBODY;
*dwg_object_to_ASSOCPOINTREFACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCPOINTREFACTIONPARAM;
*dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCREVOLVEDSURFACEACTIONBODY;
*dwg_object_to_ASSOCTRIMSURFACEACTIONBODY = *LibreDWGc::dwg_object_to_ASSOCTRIMSURFACEACTIONBODY;
*dwg_object_to_ASSOCVALUEDEPENDENCY = *LibreDWGc::dwg_object_to_ASSOCVALUEDEPENDENCY;
*dwg_object_to_ASSOCVARIABLE = *LibreDWGc::dwg_object_to_ASSOCVARIABLE;
*dwg_object_to_ASSOCVERTEXACTIONPARAM = *LibreDWGc::dwg_object_to_ASSOCVERTEXACTIONPARAM;
*dwg_object_to_BLKREFOBJECTCONTEXTDATA = *LibreDWGc::dwg_object_to_BLKREFOBJECTCONTEXTDATA;
*dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKALIGNEDCONSTRAINTPARAMETER;
*dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKANGULARCONSTRAINTPARAMETER;
*dwg_object_to_BLOCKARRAYACTION = *LibreDWGc::dwg_object_to_BLOCKARRAYACTION;
*dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKDIAMETRICCONSTRAINTPARAMETER;
*dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKHORIZONTALCONSTRAINTPARAMETER;
*dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKLINEARCONSTRAINTPARAMETER;
*dwg_object_to_BLOCKLINEARPARAMETER = *LibreDWGc::dwg_object_to_BLOCKLINEARPARAMETER;
*dwg_object_to_BLOCKLOOKUPACTION = *LibreDWGc::dwg_object_to_BLOCKLOOKUPACTION;
*dwg_object_to_BLOCKLOOKUPPARAMETER = *LibreDWGc::dwg_object_to_BLOCKLOOKUPPARAMETER;
*dwg_object_to_BLOCKPARAMDEPENDENCYBODY = *LibreDWGc::dwg_object_to_BLOCKPARAMDEPENDENCYBODY;
*dwg_object_to_BLOCKPOINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKPOINTPARAMETER;
*dwg_object_to_BLOCKPOLARGRIP = *LibreDWGc::dwg_object_to_BLOCKPOLARGRIP;
*dwg_object_to_BLOCKPOLARPARAMETER = *LibreDWGc::dwg_object_to_BLOCKPOLARPARAMETER;
*dwg_object_to_BLOCKPOLARSTRETCHACTION = *LibreDWGc::dwg_object_to_BLOCKPOLARSTRETCHACTION;
*dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKRADIALCONSTRAINTPARAMETER;
*dwg_object_to_BLOCKREPRESENTATION = *LibreDWGc::dwg_object_to_BLOCKREPRESENTATION;
*dwg_object_to_BLOCKROTATIONPARAMETER = *LibreDWGc::dwg_object_to_BLOCKROTATIONPARAMETER;
*dwg_object_to_BLOCKSTRETCHACTION = *LibreDWGc::dwg_object_to_BLOCKSTRETCHACTION;
*dwg_object_to_BLOCKUSERPARAMETER = *LibreDWGc::dwg_object_to_BLOCKUSERPARAMETER;
*dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER = *LibreDWGc::dwg_object_to_BLOCKVERTICALCONSTRAINTPARAMETER;
*dwg_object_to_BLOCKVISIBILITYPARAMETER = *LibreDWGc::dwg_object_to_BLOCKVISIBILITYPARAMETER;
*dwg_object_to_BLOCKXYGRIP = *LibreDWGc::dwg_object_to_BLOCKXYGRIP;
*dwg_object_to_BLOCKXYPARAMETER = *LibreDWGc::dwg_object_to_BLOCKXYPARAMETER;
*dwg_object_to_DATALINK = *LibreDWGc::dwg_object_to_DATALINK;
*dwg_object_to_DBCOLOR = *LibreDWGc::dwg_object_to_DBCOLOR;
*dwg_object_to_EVALUATION_GRAPH = *LibreDWGc::dwg_object_to_EVALUATION_GRAPH;
*dwg_object_to_FCFOBJECTCONTEXTDATA = *LibreDWGc::dwg_object_to_FCFOBJECTCONTEXTDATA;
*dwg_object_to_GRADIENT_BACKGROUND = *LibreDWGc::dwg_object_to_GRADIENT_BACKGROUND;
*dwg_object_to_GROUND_PLANE_BACKGROUND = *LibreDWGc::dwg_object_to_GROUND_PLANE_BACKGROUND;
*dwg_object_to_IBL_BACKGROUND = *LibreDWGc::dwg_object_to_IBL_BACKGROUND;
*dwg_object_to_IMAGE_BACKGROUND = *LibreDWGc::dwg_object_to_IMAGE_BACKGROUND;
*dwg_object_to_LEADEROBJECTCONTEXTDATA = *LibreDWGc::dwg_object_to_LEADEROBJECTCONTEXTDATA;
*dwg_object_to_LIGHTLIST = *LibreDWGc::dwg_object_to_LIGHTLIST;
*dwg_object_to_MATERIAL = *LibreDWGc::dwg_object_to_MATERIAL;
*dwg_object_to_MENTALRAYRENDERSETTINGS = *LibreDWGc::dwg_object_to_MENTALRAYRENDERSETTINGS;
*dwg_object_to_MLEADERSTYLE = *LibreDWGc::dwg_object_to_MLEADERSTYLE;
*dwg_object_to_MTEXTOBJECTCONTEXTDATA = *LibreDWGc::dwg_object_to_MTEXTOBJECTCONTEXTDATA;
*dwg_object_to_OBJECT_PTR = *LibreDWGc::dwg_object_to_OBJECT_PTR;
*dwg_object_to_PARTIAL_VIEWING_INDEX = *LibreDWGc::dwg_object_to_PARTIAL_VIEWING_INDEX;
*dwg_object_to_POINTCLOUDCOLORMAP = *LibreDWGc::dwg_object_to_POINTCLOUDCOLORMAP;
*dwg_object_to_POINTCLOUDDEF = *LibreDWGc::dwg_object_to_POINTCLOUDDEF;
*dwg_object_to_POINTCLOUDDEFEX = *LibreDWGc::dwg_object_to_POINTCLOUDDEFEX;
*dwg_object_to_POINTCLOUDDEF_REACTOR = *LibreDWGc::dwg_object_to_POINTCLOUDDEF_REACTOR;
*dwg_object_to_POINTCLOUDDEF_REACTOR_EX = *LibreDWGc::dwg_object_to_POINTCLOUDDEF_REACTOR_EX;
*dwg_object_to_PROXY_OBJECT = *LibreDWGc::dwg_object_to_PROXY_OBJECT;
*dwg_object_to_RAPIDRTRENDERSETTINGS = *LibreDWGc::dwg_object_to_RAPIDRTRENDERSETTINGS;
*dwg_object_to_RENDERENTRY = *LibreDWGc::dwg_object_to_RENDERENTRY;
*dwg_object_to_RENDERGLOBAL = *LibreDWGc::dwg_object_to_RENDERGLOBAL;
*dwg_object_to_RENDERSETTINGS = *LibreDWGc::dwg_object_to_RENDERSETTINGS;
*dwg_object_to_SECTION_SETTINGS = *LibreDWGc::dwg_object_to_SECTION_SETTINGS;
*dwg_object_to_SKYLIGHT_BACKGROUND = *LibreDWGc::dwg_object_to_SKYLIGHT_BACKGROUND;
*dwg_object_to_SOLID_BACKGROUND = *LibreDWGc::dwg_object_to_SOLID_BACKGROUND;
*dwg_object_to_SPATIAL_INDEX = *LibreDWGc::dwg_object_to_SPATIAL_INDEX;
*dwg_object_to_TABLESTYLE = *LibreDWGc::dwg_object_to_TABLESTYLE;
*dwg_object_to_TEXTOBJECTCONTEXTDATA = *LibreDWGc::dwg_object_to_TEXTOBJECTCONTEXTDATA;
*dwg_object_to_ASSOCARRAYMODIFYPARAMETERS = *LibreDWGc::dwg_object_to_ASSOCARRAYMODIFYPARAMETERS;
*dwg_object_to_ASSOCARRAYPATHPARAMETERS = *LibreDWGc::dwg_object_to_ASSOCARRAYPATHPARAMETERS;
*dwg_object_to_ASSOCARRAYPOLARPARAMETERS = *LibreDWGc::dwg_object_to_ASSOCARRAYPOLARPARAMETERS;
*dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS = *LibreDWGc::dwg_object_to_ASSOCARRAYRECTANGULARPARAMETERS;

############# Class : LibreDWG::Dwg_Bitcode_TimeBLL ##############

package LibreDWG::Dwg_Bitcode_TimeBLL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_days_get = *LibreDWGc::Dwg_Bitcode_TimeBLL_days_get;
*swig_days_set = *LibreDWGc::Dwg_Bitcode_TimeBLL_days_set;
*swig_ms_get = *LibreDWGc::Dwg_Bitcode_TimeBLL_ms_get;
*swig_ms_set = *LibreDWGc::Dwg_Bitcode_TimeBLL_ms_set;
*swig_value_get = *LibreDWGc::Dwg_Bitcode_TimeBLL_value_get;
*swig_value_set = *LibreDWGc::Dwg_Bitcode_TimeBLL_value_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Bitcode_TimeBLL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Bitcode_TimeBLL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Bitcode_2RD ##############

package LibreDWG::Dwg_Bitcode_2RD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_x_get = *LibreDWGc::Dwg_Bitcode_2RD_x_get;
*swig_x_set = *LibreDWGc::Dwg_Bitcode_2RD_x_set;
*swig_y_get = *LibreDWGc::Dwg_Bitcode_2RD_y_get;
*swig_y_set = *LibreDWGc::Dwg_Bitcode_2RD_y_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Bitcode_2RD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Bitcode_2RD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Bitcode_2BD ##############

package LibreDWG::Dwg_Bitcode_2BD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_x_get = *LibreDWGc::Dwg_Bitcode_2BD_x_get;
*swig_x_set = *LibreDWGc::Dwg_Bitcode_2BD_x_set;
*swig_y_get = *LibreDWGc::Dwg_Bitcode_2BD_y_get;
*swig_y_set = *LibreDWGc::Dwg_Bitcode_2BD_y_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Bitcode_2BD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Bitcode_2BD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Bitcode_3RD ##############

package LibreDWG::Dwg_Bitcode_3RD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_x_get = *LibreDWGc::Dwg_Bitcode_3RD_x_get;
*swig_x_set = *LibreDWGc::Dwg_Bitcode_3RD_x_set;
*swig_y_get = *LibreDWGc::Dwg_Bitcode_3RD_y_get;
*swig_y_set = *LibreDWGc::Dwg_Bitcode_3RD_y_set;
*swig_z_get = *LibreDWGc::Dwg_Bitcode_3RD_z_get;
*swig_z_set = *LibreDWGc::Dwg_Bitcode_3RD_z_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Bitcode_3RD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Bitcode_3RD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Bitcode_3BD ##############

package LibreDWG::Dwg_Bitcode_3BD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_x_get = *LibreDWGc::Dwg_Bitcode_3BD_x_get;
*swig_x_set = *LibreDWGc::Dwg_Bitcode_3BD_x_set;
*swig_y_get = *LibreDWGc::Dwg_Bitcode_3BD_y_get;
*swig_y_set = *LibreDWGc::Dwg_Bitcode_3BD_y_set;
*swig_z_get = *LibreDWGc::Dwg_Bitcode_3BD_z_get;
*swig_z_set = *LibreDWGc::Dwg_Bitcode_3BD_z_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Bitcode_3BD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Bitcode_3BD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Versions ##############

package LibreDWG::Dwg_Versions;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_r_get = *LibreDWGc::Dwg_Versions_r_get;
*swig_r_set = *LibreDWGc::Dwg_Versions_r_set;
*swig_type_get = *LibreDWGc::Dwg_Versions_type_get;
*swig_type_set = *LibreDWGc::Dwg_Versions_type_set;
*swig_hdr_get = *LibreDWGc::Dwg_Versions_hdr_get;
*swig_hdr_set = *LibreDWGc::Dwg_Versions_hdr_set;
*swig_desc_get = *LibreDWGc::Dwg_Versions_desc_get;
*swig_desc_set = *LibreDWGc::Dwg_Versions_desc_set;
*swig_dwg_version_get = *LibreDWGc::Dwg_Versions_dwg_version_get;
*swig_dwg_version_set = *LibreDWGc::Dwg_Versions_dwg_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Versions(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Versions($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Handle ##############

package LibreDWG::Dwg_Handle;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_code_get = *LibreDWGc::Dwg_Handle_code_get;
*swig_code_set = *LibreDWGc::Dwg_Handle_code_set;
*swig_size_get = *LibreDWGc::Dwg_Handle_size_get;
*swig_size_set = *LibreDWGc::Dwg_Handle_size_set;
*swig_value_get = *LibreDWGc::Dwg_Handle_value_get;
*swig_value_set = *LibreDWGc::Dwg_Handle_value_set;
*swig_is_global_get = *LibreDWGc::Dwg_Handle_is_global_get;
*swig_is_global_set = *LibreDWGc::Dwg_Handle_is_global_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Handle(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Handle($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_Ref ##############

package LibreDWG::Dwg_Object_Ref;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_obj_get = *LibreDWGc::Dwg_Object_Ref_obj_get;
*swig_obj_set = *LibreDWGc::Dwg_Object_Ref_obj_set;
*swig_handleref_get = *LibreDWGc::Dwg_Object_Ref_handleref_get;
*swig_handleref_set = *LibreDWGc::Dwg_Object_Ref_handleref_set;
*swig_absolute_ref_get = *LibreDWGc::Dwg_Object_Ref_absolute_ref_get;
*swig_absolute_ref_set = *LibreDWGc::Dwg_Object_Ref_absolute_ref_set;
*swig_r11_idx_get = *LibreDWGc::Dwg_Object_Ref_r11_idx_get;
*swig_r11_idx_set = *LibreDWGc::Dwg_Object_Ref_r11_idx_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_Ref(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_Ref($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Color ##############

package LibreDWG::Dwg_Color;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_index_get = *LibreDWGc::Dwg_Color_index_get;
*swig_index_set = *LibreDWGc::Dwg_Color_index_set;
*swig_flag_get = *LibreDWGc::Dwg_Color_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Color_flag_set;
*swig_raw_get = *LibreDWGc::Dwg_Color_raw_get;
*swig_raw_set = *LibreDWGc::Dwg_Color_raw_set;
*swig_rgb_get = *LibreDWGc::Dwg_Color_rgb_get;
*swig_rgb_set = *LibreDWGc::Dwg_Color_rgb_set;
*swig_method_get = *LibreDWGc::Dwg_Color_method_get;
*swig_method_set = *LibreDWGc::Dwg_Color_method_set;
*swig_name_get = *LibreDWGc::Dwg_Color_name_get;
*swig_name_set = *LibreDWGc::Dwg_Color_name_set;
*swig_book_name_get = *LibreDWGc::Dwg_Color_book_name_get;
*swig_book_name_set = *LibreDWGc::Dwg_Color_book_name_set;
*swig_handle_get = *LibreDWGc::Dwg_Color_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_Color_handle_set;
*swig_alpha_raw_get = *LibreDWGc::Dwg_Color_alpha_raw_get;
*swig_alpha_raw_set = *LibreDWGc::Dwg_Color_alpha_raw_set;
*swig_alpha_type_get = *LibreDWGc::Dwg_Color_alpha_type_get;
*swig_alpha_type_set = *LibreDWGc::Dwg_Color_alpha_type_set;
*swig_alpha_get = *LibreDWGc::Dwg_Color_alpha_get;
*swig_alpha_set = *LibreDWGc::Dwg_Color_alpha_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Color(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Color($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_binary_chunk ##############

package LibreDWG::_dwg_binary_chunk;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::_dwg_binary_chunk_size_get;
*swig_size_set = *LibreDWGc::_dwg_binary_chunk_size_set;
*swig_codepage_get = *LibreDWGc::_dwg_binary_chunk_codepage_get;
*swig_codepage_set = *LibreDWGc::_dwg_binary_chunk_codepage_set;
*swig_is_tu_get = *LibreDWGc::_dwg_binary_chunk_is_tu_get;
*swig_is_tu_set = *LibreDWGc::_dwg_binary_chunk_is_tu_set;
*swig_u_get = *LibreDWGc::_dwg_binary_chunk_u_get;
*swig_u_set = *LibreDWGc::_dwg_binary_chunk_u_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_binary_chunk(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_binary_chunk($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_binary_chunk_u ##############

package LibreDWG::_dwg_binary_chunk_u;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_data_get = *LibreDWGc::_dwg_binary_chunk_u_data_get;
*swig_data_set = *LibreDWGc::_dwg_binary_chunk_u_data_set;
*swig_wdata_get = *LibreDWGc::_dwg_binary_chunk_u_wdata_get;
*swig_wdata_set = *LibreDWGc::_dwg_binary_chunk_u_wdata_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_binary_chunk_u(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_binary_chunk_u($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Resbuf ##############

package LibreDWG::Dwg_Resbuf;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_type_get = *LibreDWGc::Dwg_Resbuf_type_get;
*swig_type_set = *LibreDWGc::Dwg_Resbuf_type_set;
*swig_value_get = *LibreDWGc::Dwg_Resbuf_value_get;
*swig_value_set = *LibreDWGc::Dwg_Resbuf_value_set;
*swig_nextrb_get = *LibreDWGc::Dwg_Resbuf_nextrb_get;
*swig_nextrb_set = *LibreDWGc::Dwg_Resbuf_nextrb_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Resbuf(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Resbuf($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_resbuf_value ##############

package LibreDWG::_dwg_resbuf_value;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_pt_get = *LibreDWGc::_dwg_resbuf_value_pt_get;
*swig_pt_set = *LibreDWGc::_dwg_resbuf_value_pt_set;
*swig_i8_get = *LibreDWGc::_dwg_resbuf_value_i8_get;
*swig_i8_set = *LibreDWGc::_dwg_resbuf_value_i8_set;
*swig_i16_get = *LibreDWGc::_dwg_resbuf_value_i16_get;
*swig_i16_set = *LibreDWGc::_dwg_resbuf_value_i16_set;
*swig_i32_get = *LibreDWGc::_dwg_resbuf_value_i32_get;
*swig_i32_set = *LibreDWGc::_dwg_resbuf_value_i32_set;
*swig_i64_get = *LibreDWGc::_dwg_resbuf_value_i64_get;
*swig_i64_set = *LibreDWGc::_dwg_resbuf_value_i64_set;
*swig_absref_get = *LibreDWGc::_dwg_resbuf_value_absref_get;
*swig_absref_set = *LibreDWGc::_dwg_resbuf_value_absref_set;
*swig_dbl_get = *LibreDWGc::_dwg_resbuf_value_dbl_get;
*swig_dbl_set = *LibreDWGc::_dwg_resbuf_value_dbl_set;
*swig_str_get = *LibreDWGc::_dwg_resbuf_value_str_get;
*swig_str_set = *LibreDWGc::_dwg_resbuf_value_str_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_resbuf_value(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_resbuf_value($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Header_Variables ##############

package LibreDWG::Dwg_Header_Variables;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_Header_Variables_size_get;
*swig_size_set = *LibreDWGc::Dwg_Header_Variables_size_set;
*swig_bitsize_hi_get = *LibreDWGc::Dwg_Header_Variables_bitsize_hi_get;
*swig_bitsize_hi_set = *LibreDWGc::Dwg_Header_Variables_bitsize_hi_set;
*swig_bitsize_get = *LibreDWGc::Dwg_Header_Variables_bitsize_get;
*swig_bitsize_set = *LibreDWGc::Dwg_Header_Variables_bitsize_set;
*swig_ACADMAINTVER_get = *LibreDWGc::Dwg_Header_Variables_ACADMAINTVER_get;
*swig_ACADMAINTVER_set = *LibreDWGc::Dwg_Header_Variables_ACADMAINTVER_set;
*swig_REQUIREDVERSIONS_get = *LibreDWGc::Dwg_Header_Variables_REQUIREDVERSIONS_get;
*swig_REQUIREDVERSIONS_set = *LibreDWGc::Dwg_Header_Variables_REQUIREDVERSIONS_set;
*swig_DWGCODEPAGE_get = *LibreDWGc::Dwg_Header_Variables_DWGCODEPAGE_get;
*swig_DWGCODEPAGE_set = *LibreDWGc::Dwg_Header_Variables_DWGCODEPAGE_set;
*swig_codepage_get = *LibreDWGc::Dwg_Header_Variables_codepage_get;
*swig_codepage_set = *LibreDWGc::Dwg_Header_Variables_codepage_set;
*swig_unit1_ratio_get = *LibreDWGc::Dwg_Header_Variables_unit1_ratio_get;
*swig_unit1_ratio_set = *LibreDWGc::Dwg_Header_Variables_unit1_ratio_set;
*swig_unit2_ratio_get = *LibreDWGc::Dwg_Header_Variables_unit2_ratio_get;
*swig_unit2_ratio_set = *LibreDWGc::Dwg_Header_Variables_unit2_ratio_set;
*swig_unit3_ratio_get = *LibreDWGc::Dwg_Header_Variables_unit3_ratio_get;
*swig_unit3_ratio_set = *LibreDWGc::Dwg_Header_Variables_unit3_ratio_set;
*swig_unit4_ratio_get = *LibreDWGc::Dwg_Header_Variables_unit4_ratio_get;
*swig_unit4_ratio_set = *LibreDWGc::Dwg_Header_Variables_unit4_ratio_set;
*swig_unit1_name_get = *LibreDWGc::Dwg_Header_Variables_unit1_name_get;
*swig_unit1_name_set = *LibreDWGc::Dwg_Header_Variables_unit1_name_set;
*swig_unit2_name_get = *LibreDWGc::Dwg_Header_Variables_unit2_name_get;
*swig_unit2_name_set = *LibreDWGc::Dwg_Header_Variables_unit2_name_set;
*swig_unit3_name_get = *LibreDWGc::Dwg_Header_Variables_unit3_name_get;
*swig_unit3_name_set = *LibreDWGc::Dwg_Header_Variables_unit3_name_set;
*swig_unit4_name_get = *LibreDWGc::Dwg_Header_Variables_unit4_name_get;
*swig_unit4_name_set = *LibreDWGc::Dwg_Header_Variables_unit4_name_set;
*swig_unknown_8_get = *LibreDWGc::Dwg_Header_Variables_unknown_8_get;
*swig_unknown_8_set = *LibreDWGc::Dwg_Header_Variables_unknown_8_set;
*swig_unknown_9_get = *LibreDWGc::Dwg_Header_Variables_unknown_9_get;
*swig_unknown_9_set = *LibreDWGc::Dwg_Header_Variables_unknown_9_set;
*swig_unknown_10_get = *LibreDWGc::Dwg_Header_Variables_unknown_10_get;
*swig_unknown_10_set = *LibreDWGc::Dwg_Header_Variables_unknown_10_set;
*swig_VX_TABLE_RECORD_get = *LibreDWGc::Dwg_Header_Variables_VX_TABLE_RECORD_get;
*swig_VX_TABLE_RECORD_set = *LibreDWGc::Dwg_Header_Variables_VX_TABLE_RECORD_set;
*swig_DIMASO_get = *LibreDWGc::Dwg_Header_Variables_DIMASO_get;
*swig_DIMASO_set = *LibreDWGc::Dwg_Header_Variables_DIMASO_set;
*swig_DIMSHO_get = *LibreDWGc::Dwg_Header_Variables_DIMSHO_get;
*swig_DIMSHO_set = *LibreDWGc::Dwg_Header_Variables_DIMSHO_set;
*swig_DIMSAV_get = *LibreDWGc::Dwg_Header_Variables_DIMSAV_get;
*swig_DIMSAV_set = *LibreDWGc::Dwg_Header_Variables_DIMSAV_set;
*swig_PLINEGEN_get = *LibreDWGc::Dwg_Header_Variables_PLINEGEN_get;
*swig_PLINEGEN_set = *LibreDWGc::Dwg_Header_Variables_PLINEGEN_set;
*swig_ORTHOMODE_get = *LibreDWGc::Dwg_Header_Variables_ORTHOMODE_get;
*swig_ORTHOMODE_set = *LibreDWGc::Dwg_Header_Variables_ORTHOMODE_set;
*swig_REGENMODE_get = *LibreDWGc::Dwg_Header_Variables_REGENMODE_get;
*swig_REGENMODE_set = *LibreDWGc::Dwg_Header_Variables_REGENMODE_set;
*swig_FILLMODE_get = *LibreDWGc::Dwg_Header_Variables_FILLMODE_get;
*swig_FILLMODE_set = *LibreDWGc::Dwg_Header_Variables_FILLMODE_set;
*swig_QTEXTMODE_get = *LibreDWGc::Dwg_Header_Variables_QTEXTMODE_get;
*swig_QTEXTMODE_set = *LibreDWGc::Dwg_Header_Variables_QTEXTMODE_set;
*swig_PSLTSCALE_get = *LibreDWGc::Dwg_Header_Variables_PSLTSCALE_get;
*swig_PSLTSCALE_set = *LibreDWGc::Dwg_Header_Variables_PSLTSCALE_set;
*swig_LIMCHECK_get = *LibreDWGc::Dwg_Header_Variables_LIMCHECK_get;
*swig_LIMCHECK_set = *LibreDWGc::Dwg_Header_Variables_LIMCHECK_set;
*swig_MENUEXT_get = *LibreDWGc::Dwg_Header_Variables_MENUEXT_get;
*swig_MENUEXT_set = *LibreDWGc::Dwg_Header_Variables_MENUEXT_set;
*swig_BLIPMODE_get = *LibreDWGc::Dwg_Header_Variables_BLIPMODE_get;
*swig_BLIPMODE_set = *LibreDWGc::Dwg_Header_Variables_BLIPMODE_set;
*swig_unknown_11_get = *LibreDWGc::Dwg_Header_Variables_unknown_11_get;
*swig_unknown_11_set = *LibreDWGc::Dwg_Header_Variables_unknown_11_set;
*swig_USRTIMER_get = *LibreDWGc::Dwg_Header_Variables_USRTIMER_get;
*swig_USRTIMER_set = *LibreDWGc::Dwg_Header_Variables_USRTIMER_set;
*swig_FASTZOOM_get = *LibreDWGc::Dwg_Header_Variables_FASTZOOM_get;
*swig_FASTZOOM_set = *LibreDWGc::Dwg_Header_Variables_FASTZOOM_set;
*swig_FLATLAND_get = *LibreDWGc::Dwg_Header_Variables_FLATLAND_get;
*swig_FLATLAND_set = *LibreDWGc::Dwg_Header_Variables_FLATLAND_set;
*swig_VIEWMODE_get = *LibreDWGc::Dwg_Header_Variables_VIEWMODE_get;
*swig_VIEWMODE_set = *LibreDWGc::Dwg_Header_Variables_VIEWMODE_set;
*swig_SKPOLY_get = *LibreDWGc::Dwg_Header_Variables_SKPOLY_get;
*swig_SKPOLY_set = *LibreDWGc::Dwg_Header_Variables_SKPOLY_set;
*swig_unknown_mon_get = *LibreDWGc::Dwg_Header_Variables_unknown_mon_get;
*swig_unknown_mon_set = *LibreDWGc::Dwg_Header_Variables_unknown_mon_set;
*swig_unknown_day_get = *LibreDWGc::Dwg_Header_Variables_unknown_day_get;
*swig_unknown_day_set = *LibreDWGc::Dwg_Header_Variables_unknown_day_set;
*swig_unknown_year_get = *LibreDWGc::Dwg_Header_Variables_unknown_year_get;
*swig_unknown_year_set = *LibreDWGc::Dwg_Header_Variables_unknown_year_set;
*swig_unknown_hour_get = *LibreDWGc::Dwg_Header_Variables_unknown_hour_get;
*swig_unknown_hour_set = *LibreDWGc::Dwg_Header_Variables_unknown_hour_set;
*swig_unknown_min_get = *LibreDWGc::Dwg_Header_Variables_unknown_min_get;
*swig_unknown_min_set = *LibreDWGc::Dwg_Header_Variables_unknown_min_set;
*swig_unknown_sec_get = *LibreDWGc::Dwg_Header_Variables_unknown_sec_get;
*swig_unknown_sec_set = *LibreDWGc::Dwg_Header_Variables_unknown_sec_set;
*swig_unknown_ms_get = *LibreDWGc::Dwg_Header_Variables_unknown_ms_get;
*swig_unknown_ms_set = *LibreDWGc::Dwg_Header_Variables_unknown_ms_set;
*swig_ANGDIR_get = *LibreDWGc::Dwg_Header_Variables_ANGDIR_get;
*swig_ANGDIR_set = *LibreDWGc::Dwg_Header_Variables_ANGDIR_set;
*swig_SPLFRAME_get = *LibreDWGc::Dwg_Header_Variables_SPLFRAME_get;
*swig_SPLFRAME_set = *LibreDWGc::Dwg_Header_Variables_SPLFRAME_set;
*swig_ATTREQ_get = *LibreDWGc::Dwg_Header_Variables_ATTREQ_get;
*swig_ATTREQ_set = *LibreDWGc::Dwg_Header_Variables_ATTREQ_set;
*swig_ATTDIA_get = *LibreDWGc::Dwg_Header_Variables_ATTDIA_get;
*swig_ATTDIA_set = *LibreDWGc::Dwg_Header_Variables_ATTDIA_set;
*swig_MIRRTEXT_get = *LibreDWGc::Dwg_Header_Variables_MIRRTEXT_get;
*swig_MIRRTEXT_set = *LibreDWGc::Dwg_Header_Variables_MIRRTEXT_set;
*swig_WORLDVIEW_get = *LibreDWGc::Dwg_Header_Variables_WORLDVIEW_get;
*swig_WORLDVIEW_set = *LibreDWGc::Dwg_Header_Variables_WORLDVIEW_set;
*swig_WIREFRAME_get = *LibreDWGc::Dwg_Header_Variables_WIREFRAME_get;
*swig_WIREFRAME_set = *LibreDWGc::Dwg_Header_Variables_WIREFRAME_set;
*swig_TILEMODE_get = *LibreDWGc::Dwg_Header_Variables_TILEMODE_get;
*swig_TILEMODE_set = *LibreDWGc::Dwg_Header_Variables_TILEMODE_set;
*swig_PLIMCHECK_get = *LibreDWGc::Dwg_Header_Variables_PLIMCHECK_get;
*swig_PLIMCHECK_set = *LibreDWGc::Dwg_Header_Variables_PLIMCHECK_set;
*swig_VISRETAIN_get = *LibreDWGc::Dwg_Header_Variables_VISRETAIN_get;
*swig_VISRETAIN_set = *LibreDWGc::Dwg_Header_Variables_VISRETAIN_set;
*swig_DELOBJ_get = *LibreDWGc::Dwg_Header_Variables_DELOBJ_get;
*swig_DELOBJ_set = *LibreDWGc::Dwg_Header_Variables_DELOBJ_set;
*swig_DISPSILH_get = *LibreDWGc::Dwg_Header_Variables_DISPSILH_get;
*swig_DISPSILH_set = *LibreDWGc::Dwg_Header_Variables_DISPSILH_set;
*swig_PELLIPSE_get = *LibreDWGc::Dwg_Header_Variables_PELLIPSE_get;
*swig_PELLIPSE_set = *LibreDWGc::Dwg_Header_Variables_PELLIPSE_set;
*swig_SAVEIMAGES_get = *LibreDWGc::Dwg_Header_Variables_SAVEIMAGES_get;
*swig_SAVEIMAGES_set = *LibreDWGc::Dwg_Header_Variables_SAVEIMAGES_set;
*swig_PROXYGRAPHICS_get = *LibreDWGc::Dwg_Header_Variables_PROXYGRAPHICS_get;
*swig_PROXYGRAPHICS_set = *LibreDWGc::Dwg_Header_Variables_PROXYGRAPHICS_set;
*swig_MEASUREMENT_get = *LibreDWGc::Dwg_Header_Variables_MEASUREMENT_get;
*swig_MEASUREMENT_set = *LibreDWGc::Dwg_Header_Variables_MEASUREMENT_set;
*swig_DRAGMODE_get = *LibreDWGc::Dwg_Header_Variables_DRAGMODE_get;
*swig_DRAGMODE_set = *LibreDWGc::Dwg_Header_Variables_DRAGMODE_set;
*swig_TREEDEPTH_get = *LibreDWGc::Dwg_Header_Variables_TREEDEPTH_get;
*swig_TREEDEPTH_set = *LibreDWGc::Dwg_Header_Variables_TREEDEPTH_set;
*swig_LUNITS_get = *LibreDWGc::Dwg_Header_Variables_LUNITS_get;
*swig_LUNITS_set = *LibreDWGc::Dwg_Header_Variables_LUNITS_set;
*swig_LUPREC_get = *LibreDWGc::Dwg_Header_Variables_LUPREC_get;
*swig_LUPREC_set = *LibreDWGc::Dwg_Header_Variables_LUPREC_set;
*swig_AUNITS_get = *LibreDWGc::Dwg_Header_Variables_AUNITS_get;
*swig_AUNITS_set = *LibreDWGc::Dwg_Header_Variables_AUNITS_set;
*swig_AUPREC_get = *LibreDWGc::Dwg_Header_Variables_AUPREC_get;
*swig_AUPREC_set = *LibreDWGc::Dwg_Header_Variables_AUPREC_set;
*swig_ATTMODE_get = *LibreDWGc::Dwg_Header_Variables_ATTMODE_get;
*swig_ATTMODE_set = *LibreDWGc::Dwg_Header_Variables_ATTMODE_set;
*swig_COORDS_get = *LibreDWGc::Dwg_Header_Variables_COORDS_get;
*swig_COORDS_set = *LibreDWGc::Dwg_Header_Variables_COORDS_set;
*swig_PDMODE_get = *LibreDWGc::Dwg_Header_Variables_PDMODE_get;
*swig_PDMODE_set = *LibreDWGc::Dwg_Header_Variables_PDMODE_set;
*swig_PICKSTYLE_get = *LibreDWGc::Dwg_Header_Variables_PICKSTYLE_get;
*swig_PICKSTYLE_set = *LibreDWGc::Dwg_Header_Variables_PICKSTYLE_set;
*swig_OSMODE_get = *LibreDWGc::Dwg_Header_Variables_OSMODE_get;
*swig_OSMODE_set = *LibreDWGc::Dwg_Header_Variables_OSMODE_set;
*swig_unknown_12_get = *LibreDWGc::Dwg_Header_Variables_unknown_12_get;
*swig_unknown_12_set = *LibreDWGc::Dwg_Header_Variables_unknown_12_set;
*swig_unknown_13_get = *LibreDWGc::Dwg_Header_Variables_unknown_13_get;
*swig_unknown_13_set = *LibreDWGc::Dwg_Header_Variables_unknown_13_set;
*swig_unknown_14_get = *LibreDWGc::Dwg_Header_Variables_unknown_14_get;
*swig_unknown_14_set = *LibreDWGc::Dwg_Header_Variables_unknown_14_set;
*swig_USERI1_get = *LibreDWGc::Dwg_Header_Variables_USERI1_get;
*swig_USERI1_set = *LibreDWGc::Dwg_Header_Variables_USERI1_set;
*swig_USERI2_get = *LibreDWGc::Dwg_Header_Variables_USERI2_get;
*swig_USERI2_set = *LibreDWGc::Dwg_Header_Variables_USERI2_set;
*swig_USERI3_get = *LibreDWGc::Dwg_Header_Variables_USERI3_get;
*swig_USERI3_set = *LibreDWGc::Dwg_Header_Variables_USERI3_set;
*swig_USERI4_get = *LibreDWGc::Dwg_Header_Variables_USERI4_get;
*swig_USERI4_set = *LibreDWGc::Dwg_Header_Variables_USERI4_set;
*swig_USERI5_get = *LibreDWGc::Dwg_Header_Variables_USERI5_get;
*swig_USERI5_set = *LibreDWGc::Dwg_Header_Variables_USERI5_set;
*swig_SPLINESEGS_get = *LibreDWGc::Dwg_Header_Variables_SPLINESEGS_get;
*swig_SPLINESEGS_set = *LibreDWGc::Dwg_Header_Variables_SPLINESEGS_set;
*swig_SURFU_get = *LibreDWGc::Dwg_Header_Variables_SURFU_get;
*swig_SURFU_set = *LibreDWGc::Dwg_Header_Variables_SURFU_set;
*swig_SURFV_get = *LibreDWGc::Dwg_Header_Variables_SURFV_get;
*swig_SURFV_set = *LibreDWGc::Dwg_Header_Variables_SURFV_set;
*swig_SURFTYPE_get = *LibreDWGc::Dwg_Header_Variables_SURFTYPE_get;
*swig_SURFTYPE_set = *LibreDWGc::Dwg_Header_Variables_SURFTYPE_set;
*swig_SURFTAB1_get = *LibreDWGc::Dwg_Header_Variables_SURFTAB1_get;
*swig_SURFTAB1_set = *LibreDWGc::Dwg_Header_Variables_SURFTAB1_set;
*swig_SURFTAB2_get = *LibreDWGc::Dwg_Header_Variables_SURFTAB2_get;
*swig_SURFTAB2_set = *LibreDWGc::Dwg_Header_Variables_SURFTAB2_set;
*swig_SPLINETYPE_get = *LibreDWGc::Dwg_Header_Variables_SPLINETYPE_get;
*swig_SPLINETYPE_set = *LibreDWGc::Dwg_Header_Variables_SPLINETYPE_set;
*swig_SHADEDGE_get = *LibreDWGc::Dwg_Header_Variables_SHADEDGE_get;
*swig_SHADEDGE_set = *LibreDWGc::Dwg_Header_Variables_SHADEDGE_set;
*swig_SHADEDIF_get = *LibreDWGc::Dwg_Header_Variables_SHADEDIF_get;
*swig_SHADEDIF_set = *LibreDWGc::Dwg_Header_Variables_SHADEDIF_set;
*swig_UNITMODE_get = *LibreDWGc::Dwg_Header_Variables_UNITMODE_get;
*swig_UNITMODE_set = *LibreDWGc::Dwg_Header_Variables_UNITMODE_set;
*swig_MAXACTVP_get = *LibreDWGc::Dwg_Header_Variables_MAXACTVP_get;
*swig_MAXACTVP_set = *LibreDWGc::Dwg_Header_Variables_MAXACTVP_set;
*swig_ISOLINES_get = *LibreDWGc::Dwg_Header_Variables_ISOLINES_get;
*swig_ISOLINES_set = *LibreDWGc::Dwg_Header_Variables_ISOLINES_set;
*swig_CMLJUST_get = *LibreDWGc::Dwg_Header_Variables_CMLJUST_get;
*swig_CMLJUST_set = *LibreDWGc::Dwg_Header_Variables_CMLJUST_set;
*swig_TEXTQLTY_get = *LibreDWGc::Dwg_Header_Variables_TEXTQLTY_get;
*swig_TEXTQLTY_set = *LibreDWGc::Dwg_Header_Variables_TEXTQLTY_set;
*swig_unknown_14b_get = *LibreDWGc::Dwg_Header_Variables_unknown_14b_get;
*swig_unknown_14b_set = *LibreDWGc::Dwg_Header_Variables_unknown_14b_set;
*swig_LTSCALE_get = *LibreDWGc::Dwg_Header_Variables_LTSCALE_get;
*swig_LTSCALE_set = *LibreDWGc::Dwg_Header_Variables_LTSCALE_set;
*swig_TEXTSIZE_get = *LibreDWGc::Dwg_Header_Variables_TEXTSIZE_get;
*swig_TEXTSIZE_set = *LibreDWGc::Dwg_Header_Variables_TEXTSIZE_set;
*swig_TRACEWID_get = *LibreDWGc::Dwg_Header_Variables_TRACEWID_get;
*swig_TRACEWID_set = *LibreDWGc::Dwg_Header_Variables_TRACEWID_set;
*swig_SKETCHINC_get = *LibreDWGc::Dwg_Header_Variables_SKETCHINC_get;
*swig_SKETCHINC_set = *LibreDWGc::Dwg_Header_Variables_SKETCHINC_set;
*swig_FILLETRAD_get = *LibreDWGc::Dwg_Header_Variables_FILLETRAD_get;
*swig_FILLETRAD_set = *LibreDWGc::Dwg_Header_Variables_FILLETRAD_set;
*swig_THICKNESS_get = *LibreDWGc::Dwg_Header_Variables_THICKNESS_get;
*swig_THICKNESS_set = *LibreDWGc::Dwg_Header_Variables_THICKNESS_set;
*swig_ANGBASE_get = *LibreDWGc::Dwg_Header_Variables_ANGBASE_get;
*swig_ANGBASE_set = *LibreDWGc::Dwg_Header_Variables_ANGBASE_set;
*swig_PDSIZE_get = *LibreDWGc::Dwg_Header_Variables_PDSIZE_get;
*swig_PDSIZE_set = *LibreDWGc::Dwg_Header_Variables_PDSIZE_set;
*swig_PLINEWID_get = *LibreDWGc::Dwg_Header_Variables_PLINEWID_get;
*swig_PLINEWID_set = *LibreDWGc::Dwg_Header_Variables_PLINEWID_set;
*swig_USERR1_get = *LibreDWGc::Dwg_Header_Variables_USERR1_get;
*swig_USERR1_set = *LibreDWGc::Dwg_Header_Variables_USERR1_set;
*swig_USERR2_get = *LibreDWGc::Dwg_Header_Variables_USERR2_get;
*swig_USERR2_set = *LibreDWGc::Dwg_Header_Variables_USERR2_set;
*swig_USERR3_get = *LibreDWGc::Dwg_Header_Variables_USERR3_get;
*swig_USERR3_set = *LibreDWGc::Dwg_Header_Variables_USERR3_set;
*swig_USERR4_get = *LibreDWGc::Dwg_Header_Variables_USERR4_get;
*swig_USERR4_set = *LibreDWGc::Dwg_Header_Variables_USERR4_set;
*swig_USERR5_get = *LibreDWGc::Dwg_Header_Variables_USERR5_get;
*swig_USERR5_set = *LibreDWGc::Dwg_Header_Variables_USERR5_set;
*swig_CHAMFERA_get = *LibreDWGc::Dwg_Header_Variables_CHAMFERA_get;
*swig_CHAMFERA_set = *LibreDWGc::Dwg_Header_Variables_CHAMFERA_set;
*swig_CHAMFERB_get = *LibreDWGc::Dwg_Header_Variables_CHAMFERB_get;
*swig_CHAMFERB_set = *LibreDWGc::Dwg_Header_Variables_CHAMFERB_set;
*swig_CHAMFERC_get = *LibreDWGc::Dwg_Header_Variables_CHAMFERC_get;
*swig_CHAMFERC_set = *LibreDWGc::Dwg_Header_Variables_CHAMFERC_set;
*swig_CHAMFERD_get = *LibreDWGc::Dwg_Header_Variables_CHAMFERD_get;
*swig_CHAMFERD_set = *LibreDWGc::Dwg_Header_Variables_CHAMFERD_set;
*swig_FACETRES_get = *LibreDWGc::Dwg_Header_Variables_FACETRES_get;
*swig_FACETRES_set = *LibreDWGc::Dwg_Header_Variables_FACETRES_set;
*swig_CMLSCALE_get = *LibreDWGc::Dwg_Header_Variables_CMLSCALE_get;
*swig_CMLSCALE_set = *LibreDWGc::Dwg_Header_Variables_CMLSCALE_set;
*swig_CELTSCALE_get = *LibreDWGc::Dwg_Header_Variables_CELTSCALE_get;
*swig_CELTSCALE_set = *LibreDWGc::Dwg_Header_Variables_CELTSCALE_set;
*swig_VIEWTWIST_get = *LibreDWGc::Dwg_Header_Variables_VIEWTWIST_get;
*swig_VIEWTWIST_set = *LibreDWGc::Dwg_Header_Variables_VIEWTWIST_set;
*swig_MENU_get = *LibreDWGc::Dwg_Header_Variables_MENU_get;
*swig_MENU_set = *LibreDWGc::Dwg_Header_Variables_MENU_set;
*swig_TDCREATE_get = *LibreDWGc::Dwg_Header_Variables_TDCREATE_get;
*swig_TDCREATE_set = *LibreDWGc::Dwg_Header_Variables_TDCREATE_set;
*swig_TDUPDATE_get = *LibreDWGc::Dwg_Header_Variables_TDUPDATE_get;
*swig_TDUPDATE_set = *LibreDWGc::Dwg_Header_Variables_TDUPDATE_set;
*swig_TDUCREATE_get = *LibreDWGc::Dwg_Header_Variables_TDUCREATE_get;
*swig_TDUCREATE_set = *LibreDWGc::Dwg_Header_Variables_TDUCREATE_set;
*swig_TDUUPDATE_get = *LibreDWGc::Dwg_Header_Variables_TDUUPDATE_get;
*swig_TDUUPDATE_set = *LibreDWGc::Dwg_Header_Variables_TDUUPDATE_set;
*swig_unknown_15_get = *LibreDWGc::Dwg_Header_Variables_unknown_15_get;
*swig_unknown_15_set = *LibreDWGc::Dwg_Header_Variables_unknown_15_set;
*swig_unknown_16_get = *LibreDWGc::Dwg_Header_Variables_unknown_16_get;
*swig_unknown_16_set = *LibreDWGc::Dwg_Header_Variables_unknown_16_set;
*swig_unknown_17_get = *LibreDWGc::Dwg_Header_Variables_unknown_17_get;
*swig_unknown_17_set = *LibreDWGc::Dwg_Header_Variables_unknown_17_set;
*swig_TDINDWG_get = *LibreDWGc::Dwg_Header_Variables_TDINDWG_get;
*swig_TDINDWG_set = *LibreDWGc::Dwg_Header_Variables_TDINDWG_set;
*swig_TDUSRTIMER_get = *LibreDWGc::Dwg_Header_Variables_TDUSRTIMER_get;
*swig_TDUSRTIMER_set = *LibreDWGc::Dwg_Header_Variables_TDUSRTIMER_set;
*swig_CECOLOR_get = *LibreDWGc::Dwg_Header_Variables_CECOLOR_get;
*swig_CECOLOR_set = *LibreDWGc::Dwg_Header_Variables_CECOLOR_set;
*swig_HANDLING_get = *LibreDWGc::Dwg_Header_Variables_HANDLING_get;
*swig_HANDLING_set = *LibreDWGc::Dwg_Header_Variables_HANDLING_set;
*swig_HANDSEED_get = *LibreDWGc::Dwg_Header_Variables_HANDSEED_get;
*swig_HANDSEED_set = *LibreDWGc::Dwg_Header_Variables_HANDSEED_set;
*swig_unknown_4f2_get = *LibreDWGc::Dwg_Header_Variables_unknown_4f2_get;
*swig_unknown_4f2_set = *LibreDWGc::Dwg_Header_Variables_unknown_4f2_set;
*swig_unknown_5_get = *LibreDWGc::Dwg_Header_Variables_unknown_5_get;
*swig_unknown_5_set = *LibreDWGc::Dwg_Header_Variables_unknown_5_set;
*swig_unknown_6_get = *LibreDWGc::Dwg_Header_Variables_unknown_6_get;
*swig_unknown_6_set = *LibreDWGc::Dwg_Header_Variables_unknown_6_set;
*swig_unknown_6a_get = *LibreDWGc::Dwg_Header_Variables_unknown_6a_get;
*swig_unknown_6a_set = *LibreDWGc::Dwg_Header_Variables_unknown_6a_set;
*swig_unknown_6b_get = *LibreDWGc::Dwg_Header_Variables_unknown_6b_get;
*swig_unknown_6b_set = *LibreDWGc::Dwg_Header_Variables_unknown_6b_set;
*swig_unknown_6c_get = *LibreDWGc::Dwg_Header_Variables_unknown_6c_get;
*swig_unknown_6c_set = *LibreDWGc::Dwg_Header_Variables_unknown_6c_set;
*swig_CLAYER_get = *LibreDWGc::Dwg_Header_Variables_CLAYER_get;
*swig_CLAYER_set = *LibreDWGc::Dwg_Header_Variables_CLAYER_set;
*swig_TEXTSTYLE_get = *LibreDWGc::Dwg_Header_Variables_TEXTSTYLE_get;
*swig_TEXTSTYLE_set = *LibreDWGc::Dwg_Header_Variables_TEXTSTYLE_set;
*swig_CELTYPE_get = *LibreDWGc::Dwg_Header_Variables_CELTYPE_get;
*swig_CELTYPE_set = *LibreDWGc::Dwg_Header_Variables_CELTYPE_set;
*swig_CMATERIAL_get = *LibreDWGc::Dwg_Header_Variables_CMATERIAL_get;
*swig_CMATERIAL_set = *LibreDWGc::Dwg_Header_Variables_CMATERIAL_set;
*swig_DIMSTYLE_get = *LibreDWGc::Dwg_Header_Variables_DIMSTYLE_get;
*swig_DIMSTYLE_set = *LibreDWGc::Dwg_Header_Variables_DIMSTYLE_set;
*swig_CMLSTYLE_get = *LibreDWGc::Dwg_Header_Variables_CMLSTYLE_get;
*swig_CMLSTYLE_set = *LibreDWGc::Dwg_Header_Variables_CMLSTYLE_set;
*swig_PSVPSCALE_get = *LibreDWGc::Dwg_Header_Variables_PSVPSCALE_get;
*swig_PSVPSCALE_set = *LibreDWGc::Dwg_Header_Variables_PSVPSCALE_set;
*swig_PINSBASE_get = *LibreDWGc::Dwg_Header_Variables_PINSBASE_get;
*swig_PINSBASE_set = *LibreDWGc::Dwg_Header_Variables_PINSBASE_set;
*swig_PEXTMIN_get = *LibreDWGc::Dwg_Header_Variables_PEXTMIN_get;
*swig_PEXTMIN_set = *LibreDWGc::Dwg_Header_Variables_PEXTMIN_set;
*swig_PEXTMAX_get = *LibreDWGc::Dwg_Header_Variables_PEXTMAX_get;
*swig_PEXTMAX_set = *LibreDWGc::Dwg_Header_Variables_PEXTMAX_set;
*swig_PLIMMIN_get = *LibreDWGc::Dwg_Header_Variables_PLIMMIN_get;
*swig_PLIMMIN_set = *LibreDWGc::Dwg_Header_Variables_PLIMMIN_set;
*swig_PLIMMAX_get = *LibreDWGc::Dwg_Header_Variables_PLIMMAX_get;
*swig_PLIMMAX_set = *LibreDWGc::Dwg_Header_Variables_PLIMMAX_set;
*swig_PELEVATION_get = *LibreDWGc::Dwg_Header_Variables_PELEVATION_get;
*swig_PELEVATION_set = *LibreDWGc::Dwg_Header_Variables_PELEVATION_set;
*swig_PUCSORG_get = *LibreDWGc::Dwg_Header_Variables_PUCSORG_get;
*swig_PUCSORG_set = *LibreDWGc::Dwg_Header_Variables_PUCSORG_set;
*swig_PUCSXDIR_get = *LibreDWGc::Dwg_Header_Variables_PUCSXDIR_get;
*swig_PUCSXDIR_set = *LibreDWGc::Dwg_Header_Variables_PUCSXDIR_set;
*swig_PUCSYDIR_get = *LibreDWGc::Dwg_Header_Variables_PUCSYDIR_get;
*swig_PUCSYDIR_set = *LibreDWGc::Dwg_Header_Variables_PUCSYDIR_set;
*swig_PUCSNAME_get = *LibreDWGc::Dwg_Header_Variables_PUCSNAME_get;
*swig_PUCSNAME_set = *LibreDWGc::Dwg_Header_Variables_PUCSNAME_set;
*swig_PUCSBASE_get = *LibreDWGc::Dwg_Header_Variables_PUCSBASE_get;
*swig_PUCSBASE_set = *LibreDWGc::Dwg_Header_Variables_PUCSBASE_set;
*swig_PUCSORTHOREF_get = *LibreDWGc::Dwg_Header_Variables_PUCSORTHOREF_get;
*swig_PUCSORTHOREF_set = *LibreDWGc::Dwg_Header_Variables_PUCSORTHOREF_set;
*swig_PUCSORTHOVIEW_get = *LibreDWGc::Dwg_Header_Variables_PUCSORTHOVIEW_get;
*swig_PUCSORTHOVIEW_set = *LibreDWGc::Dwg_Header_Variables_PUCSORTHOVIEW_set;
*swig_PUCSORGTOP_get = *LibreDWGc::Dwg_Header_Variables_PUCSORGTOP_get;
*swig_PUCSORGTOP_set = *LibreDWGc::Dwg_Header_Variables_PUCSORGTOP_set;
*swig_PUCSORGBOTTOM_get = *LibreDWGc::Dwg_Header_Variables_PUCSORGBOTTOM_get;
*swig_PUCSORGBOTTOM_set = *LibreDWGc::Dwg_Header_Variables_PUCSORGBOTTOM_set;
*swig_PUCSORGLEFT_get = *LibreDWGc::Dwg_Header_Variables_PUCSORGLEFT_get;
*swig_PUCSORGLEFT_set = *LibreDWGc::Dwg_Header_Variables_PUCSORGLEFT_set;
*swig_PUCSORGRIGHT_get = *LibreDWGc::Dwg_Header_Variables_PUCSORGRIGHT_get;
*swig_PUCSORGRIGHT_set = *LibreDWGc::Dwg_Header_Variables_PUCSORGRIGHT_set;
*swig_PUCSORGFRONT_get = *LibreDWGc::Dwg_Header_Variables_PUCSORGFRONT_get;
*swig_PUCSORGFRONT_set = *LibreDWGc::Dwg_Header_Variables_PUCSORGFRONT_set;
*swig_PUCSORGBACK_get = *LibreDWGc::Dwg_Header_Variables_PUCSORGBACK_get;
*swig_PUCSORGBACK_set = *LibreDWGc::Dwg_Header_Variables_PUCSORGBACK_set;
*swig_INSBASE_get = *LibreDWGc::Dwg_Header_Variables_INSBASE_get;
*swig_INSBASE_set = *LibreDWGc::Dwg_Header_Variables_INSBASE_set;
*swig_EXTMIN_get = *LibreDWGc::Dwg_Header_Variables_EXTMIN_get;
*swig_EXTMIN_set = *LibreDWGc::Dwg_Header_Variables_EXTMIN_set;
*swig_EXTMAX_get = *LibreDWGc::Dwg_Header_Variables_EXTMAX_get;
*swig_EXTMAX_set = *LibreDWGc::Dwg_Header_Variables_EXTMAX_set;
*swig_VIEWDIR_get = *LibreDWGc::Dwg_Header_Variables_VIEWDIR_get;
*swig_VIEWDIR_set = *LibreDWGc::Dwg_Header_Variables_VIEWDIR_set;
*swig_TARGET_get = *LibreDWGc::Dwg_Header_Variables_TARGET_get;
*swig_TARGET_set = *LibreDWGc::Dwg_Header_Variables_TARGET_set;
*swig_LIMMIN_get = *LibreDWGc::Dwg_Header_Variables_LIMMIN_get;
*swig_LIMMIN_set = *LibreDWGc::Dwg_Header_Variables_LIMMIN_set;
*swig_LIMMAX_get = *LibreDWGc::Dwg_Header_Variables_LIMMAX_get;
*swig_LIMMAX_set = *LibreDWGc::Dwg_Header_Variables_LIMMAX_set;
*swig_VIEWCTR_get = *LibreDWGc::Dwg_Header_Variables_VIEWCTR_get;
*swig_VIEWCTR_set = *LibreDWGc::Dwg_Header_Variables_VIEWCTR_set;
*swig_ELEVATION_get = *LibreDWGc::Dwg_Header_Variables_ELEVATION_get;
*swig_ELEVATION_set = *LibreDWGc::Dwg_Header_Variables_ELEVATION_set;
*swig_VIEWSIZE_get = *LibreDWGc::Dwg_Header_Variables_VIEWSIZE_get;
*swig_VIEWSIZE_set = *LibreDWGc::Dwg_Header_Variables_VIEWSIZE_set;
*swig_SNAPMODE_get = *LibreDWGc::Dwg_Header_Variables_SNAPMODE_get;
*swig_SNAPMODE_set = *LibreDWGc::Dwg_Header_Variables_SNAPMODE_set;
*swig_SNAPUNIT_get = *LibreDWGc::Dwg_Header_Variables_SNAPUNIT_get;
*swig_SNAPUNIT_set = *LibreDWGc::Dwg_Header_Variables_SNAPUNIT_set;
*swig_SNAPBASE_get = *LibreDWGc::Dwg_Header_Variables_SNAPBASE_get;
*swig_SNAPBASE_set = *LibreDWGc::Dwg_Header_Variables_SNAPBASE_set;
*swig_SNAPANG_get = *LibreDWGc::Dwg_Header_Variables_SNAPANG_get;
*swig_SNAPANG_set = *LibreDWGc::Dwg_Header_Variables_SNAPANG_set;
*swig_SNAPSTYLE_get = *LibreDWGc::Dwg_Header_Variables_SNAPSTYLE_get;
*swig_SNAPSTYLE_set = *LibreDWGc::Dwg_Header_Variables_SNAPSTYLE_set;
*swig_SNAPISOPAIR_get = *LibreDWGc::Dwg_Header_Variables_SNAPISOPAIR_get;
*swig_SNAPISOPAIR_set = *LibreDWGc::Dwg_Header_Variables_SNAPISOPAIR_set;
*swig_GRIDMODE_get = *LibreDWGc::Dwg_Header_Variables_GRIDMODE_get;
*swig_GRIDMODE_set = *LibreDWGc::Dwg_Header_Variables_GRIDMODE_set;
*swig_GRIDUNIT_get = *LibreDWGc::Dwg_Header_Variables_GRIDUNIT_get;
*swig_GRIDUNIT_set = *LibreDWGc::Dwg_Header_Variables_GRIDUNIT_set;
*swig_AXISMODE_get = *LibreDWGc::Dwg_Header_Variables_AXISMODE_get;
*swig_AXISMODE_set = *LibreDWGc::Dwg_Header_Variables_AXISMODE_set;
*swig_AXISUNIT_get = *LibreDWGc::Dwg_Header_Variables_AXISUNIT_get;
*swig_AXISUNIT_set = *LibreDWGc::Dwg_Header_Variables_AXISUNIT_set;
*swig_VPOINTX_get = *LibreDWGc::Dwg_Header_Variables_VPOINTX_get;
*swig_VPOINTX_set = *LibreDWGc::Dwg_Header_Variables_VPOINTX_set;
*swig_VPOINTY_get = *LibreDWGc::Dwg_Header_Variables_VPOINTY_get;
*swig_VPOINTY_set = *LibreDWGc::Dwg_Header_Variables_VPOINTY_set;
*swig_VPOINTZ_get = *LibreDWGc::Dwg_Header_Variables_VPOINTZ_get;
*swig_VPOINTZ_set = *LibreDWGc::Dwg_Header_Variables_VPOINTZ_set;
*swig_VPOINTXALT_get = *LibreDWGc::Dwg_Header_Variables_VPOINTXALT_get;
*swig_VPOINTXALT_set = *LibreDWGc::Dwg_Header_Variables_VPOINTXALT_set;
*swig_VPOINTYALT_get = *LibreDWGc::Dwg_Header_Variables_VPOINTYALT_get;
*swig_VPOINTYALT_set = *LibreDWGc::Dwg_Header_Variables_VPOINTYALT_set;
*swig_VPOINTZALT_get = *LibreDWGc::Dwg_Header_Variables_VPOINTZALT_get;
*swig_VPOINTZALT_set = *LibreDWGc::Dwg_Header_Variables_VPOINTZALT_set;
*swig_flag_3d_get = *LibreDWGc::Dwg_Header_Variables_flag_3d_get;
*swig_flag_3d_set = *LibreDWGc::Dwg_Header_Variables_flag_3d_set;
*swig_UCSORG_get = *LibreDWGc::Dwg_Header_Variables_UCSORG_get;
*swig_UCSORG_set = *LibreDWGc::Dwg_Header_Variables_UCSORG_set;
*swig_UCSXDIR_get = *LibreDWGc::Dwg_Header_Variables_UCSXDIR_get;
*swig_UCSXDIR_set = *LibreDWGc::Dwg_Header_Variables_UCSXDIR_set;
*swig_UCSYDIR_get = *LibreDWGc::Dwg_Header_Variables_UCSYDIR_get;
*swig_UCSYDIR_set = *LibreDWGc::Dwg_Header_Variables_UCSYDIR_set;
*swig_UCSNAME_get = *LibreDWGc::Dwg_Header_Variables_UCSNAME_get;
*swig_UCSNAME_set = *LibreDWGc::Dwg_Header_Variables_UCSNAME_set;
*swig_UCSBASE_get = *LibreDWGc::Dwg_Header_Variables_UCSBASE_get;
*swig_UCSBASE_set = *LibreDWGc::Dwg_Header_Variables_UCSBASE_set;
*swig_UCSORTHOVIEW_get = *LibreDWGc::Dwg_Header_Variables_UCSORTHOVIEW_get;
*swig_UCSORTHOVIEW_set = *LibreDWGc::Dwg_Header_Variables_UCSORTHOVIEW_set;
*swig_UCSORTHOREF_get = *LibreDWGc::Dwg_Header_Variables_UCSORTHOREF_get;
*swig_UCSORTHOREF_set = *LibreDWGc::Dwg_Header_Variables_UCSORTHOREF_set;
*swig_UCSORGTOP_get = *LibreDWGc::Dwg_Header_Variables_UCSORGTOP_get;
*swig_UCSORGTOP_set = *LibreDWGc::Dwg_Header_Variables_UCSORGTOP_set;
*swig_UCSORGBOTTOM_get = *LibreDWGc::Dwg_Header_Variables_UCSORGBOTTOM_get;
*swig_UCSORGBOTTOM_set = *LibreDWGc::Dwg_Header_Variables_UCSORGBOTTOM_set;
*swig_UCSORGLEFT_get = *LibreDWGc::Dwg_Header_Variables_UCSORGLEFT_get;
*swig_UCSORGLEFT_set = *LibreDWGc::Dwg_Header_Variables_UCSORGLEFT_set;
*swig_UCSORGRIGHT_get = *LibreDWGc::Dwg_Header_Variables_UCSORGRIGHT_get;
*swig_UCSORGRIGHT_set = *LibreDWGc::Dwg_Header_Variables_UCSORGRIGHT_set;
*swig_UCSORGFRONT_get = *LibreDWGc::Dwg_Header_Variables_UCSORGFRONT_get;
*swig_UCSORGFRONT_set = *LibreDWGc::Dwg_Header_Variables_UCSORGFRONT_set;
*swig_UCSORGBACK_get = *LibreDWGc::Dwg_Header_Variables_UCSORGBACK_get;
*swig_UCSORGBACK_set = *LibreDWGc::Dwg_Header_Variables_UCSORGBACK_set;
*swig_DIMPOST_get = *LibreDWGc::Dwg_Header_Variables_DIMPOST_get;
*swig_DIMPOST_set = *LibreDWGc::Dwg_Header_Variables_DIMPOST_set;
*swig_DIMAPOST_get = *LibreDWGc::Dwg_Header_Variables_DIMAPOST_get;
*swig_DIMAPOST_set = *LibreDWGc::Dwg_Header_Variables_DIMAPOST_set;
*swig_DIMTOL_get = *LibreDWGc::Dwg_Header_Variables_DIMTOL_get;
*swig_DIMTOL_set = *LibreDWGc::Dwg_Header_Variables_DIMTOL_set;
*swig_DIMLIM_get = *LibreDWGc::Dwg_Header_Variables_DIMLIM_get;
*swig_DIMLIM_set = *LibreDWGc::Dwg_Header_Variables_DIMLIM_set;
*swig_DIMTIH_get = *LibreDWGc::Dwg_Header_Variables_DIMTIH_get;
*swig_DIMTIH_set = *LibreDWGc::Dwg_Header_Variables_DIMTIH_set;
*swig_DIMTOH_get = *LibreDWGc::Dwg_Header_Variables_DIMTOH_get;
*swig_DIMTOH_set = *LibreDWGc::Dwg_Header_Variables_DIMTOH_set;
*swig_DIMSE1_get = *LibreDWGc::Dwg_Header_Variables_DIMSE1_get;
*swig_DIMSE1_set = *LibreDWGc::Dwg_Header_Variables_DIMSE1_set;
*swig_DIMSE2_get = *LibreDWGc::Dwg_Header_Variables_DIMSE2_get;
*swig_DIMSE2_set = *LibreDWGc::Dwg_Header_Variables_DIMSE2_set;
*swig_DIMALT_get = *LibreDWGc::Dwg_Header_Variables_DIMALT_get;
*swig_DIMALT_set = *LibreDWGc::Dwg_Header_Variables_DIMALT_set;
*swig_DIMTOFL_get = *LibreDWGc::Dwg_Header_Variables_DIMTOFL_get;
*swig_DIMTOFL_set = *LibreDWGc::Dwg_Header_Variables_DIMTOFL_set;
*swig_DIMSAH_get = *LibreDWGc::Dwg_Header_Variables_DIMSAH_get;
*swig_DIMSAH_set = *LibreDWGc::Dwg_Header_Variables_DIMSAH_set;
*swig_DIMTIX_get = *LibreDWGc::Dwg_Header_Variables_DIMTIX_get;
*swig_DIMTIX_set = *LibreDWGc::Dwg_Header_Variables_DIMTIX_set;
*swig_DIMSOXD_get = *LibreDWGc::Dwg_Header_Variables_DIMSOXD_get;
*swig_DIMSOXD_set = *LibreDWGc::Dwg_Header_Variables_DIMSOXD_set;
*swig_DIMALTD_get = *LibreDWGc::Dwg_Header_Variables_DIMALTD_get;
*swig_DIMALTD_set = *LibreDWGc::Dwg_Header_Variables_DIMALTD_set;
*swig_DIMZIN_get = *LibreDWGc::Dwg_Header_Variables_DIMZIN_get;
*swig_DIMZIN_set = *LibreDWGc::Dwg_Header_Variables_DIMZIN_set;
*swig_DIMSD1_get = *LibreDWGc::Dwg_Header_Variables_DIMSD1_get;
*swig_DIMSD1_set = *LibreDWGc::Dwg_Header_Variables_DIMSD1_set;
*swig_DIMSD2_get = *LibreDWGc::Dwg_Header_Variables_DIMSD2_get;
*swig_DIMSD2_set = *LibreDWGc::Dwg_Header_Variables_DIMSD2_set;
*swig_DIMTOLJ_get = *LibreDWGc::Dwg_Header_Variables_DIMTOLJ_get;
*swig_DIMTOLJ_set = *LibreDWGc::Dwg_Header_Variables_DIMTOLJ_set;
*swig_DIMJUST_get = *LibreDWGc::Dwg_Header_Variables_DIMJUST_get;
*swig_DIMJUST_set = *LibreDWGc::Dwg_Header_Variables_DIMJUST_set;
*swig_DIMFIT_get = *LibreDWGc::Dwg_Header_Variables_DIMFIT_get;
*swig_DIMFIT_set = *LibreDWGc::Dwg_Header_Variables_DIMFIT_set;
*swig_DIMUPT_get = *LibreDWGc::Dwg_Header_Variables_DIMUPT_get;
*swig_DIMUPT_set = *LibreDWGc::Dwg_Header_Variables_DIMUPT_set;
*swig_DIMTZIN_get = *LibreDWGc::Dwg_Header_Variables_DIMTZIN_get;
*swig_DIMTZIN_set = *LibreDWGc::Dwg_Header_Variables_DIMTZIN_set;
*swig_DIMTAD_get = *LibreDWGc::Dwg_Header_Variables_DIMTAD_get;
*swig_DIMTAD_set = *LibreDWGc::Dwg_Header_Variables_DIMTAD_set;
*swig_DIMUNIT_get = *LibreDWGc::Dwg_Header_Variables_DIMUNIT_get;
*swig_DIMUNIT_set = *LibreDWGc::Dwg_Header_Variables_DIMUNIT_set;
*swig_DIMAUNIT_get = *LibreDWGc::Dwg_Header_Variables_DIMAUNIT_get;
*swig_DIMAUNIT_set = *LibreDWGc::Dwg_Header_Variables_DIMAUNIT_set;
*swig_DIMDEC_get = *LibreDWGc::Dwg_Header_Variables_DIMDEC_get;
*swig_DIMDEC_set = *LibreDWGc::Dwg_Header_Variables_DIMDEC_set;
*swig_DIMTDEC_get = *LibreDWGc::Dwg_Header_Variables_DIMTDEC_get;
*swig_DIMTDEC_set = *LibreDWGc::Dwg_Header_Variables_DIMTDEC_set;
*swig_DIMALTU_get = *LibreDWGc::Dwg_Header_Variables_DIMALTU_get;
*swig_DIMALTU_set = *LibreDWGc::Dwg_Header_Variables_DIMALTU_set;
*swig_DIMALTTD_get = *LibreDWGc::Dwg_Header_Variables_DIMALTTD_get;
*swig_DIMALTTD_set = *LibreDWGc::Dwg_Header_Variables_DIMALTTD_set;
*swig_DIMTXSTY_get = *LibreDWGc::Dwg_Header_Variables_DIMTXSTY_get;
*swig_DIMTXSTY_set = *LibreDWGc::Dwg_Header_Variables_DIMTXSTY_set;
*swig_DIMSCALE_get = *LibreDWGc::Dwg_Header_Variables_DIMSCALE_get;
*swig_DIMSCALE_set = *LibreDWGc::Dwg_Header_Variables_DIMSCALE_set;
*swig_DIMARROW_get = *LibreDWGc::Dwg_Header_Variables_DIMARROW_get;
*swig_DIMARROW_set = *LibreDWGc::Dwg_Header_Variables_DIMARROW_set;
*swig_DIMASZ_get = *LibreDWGc::Dwg_Header_Variables_DIMASZ_get;
*swig_DIMASZ_set = *LibreDWGc::Dwg_Header_Variables_DIMASZ_set;
*swig_DIMEXO_get = *LibreDWGc::Dwg_Header_Variables_DIMEXO_get;
*swig_DIMEXO_set = *LibreDWGc::Dwg_Header_Variables_DIMEXO_set;
*swig_DIMDLI_get = *LibreDWGc::Dwg_Header_Variables_DIMDLI_get;
*swig_DIMDLI_set = *LibreDWGc::Dwg_Header_Variables_DIMDLI_set;
*swig_DIMEXE_get = *LibreDWGc::Dwg_Header_Variables_DIMEXE_get;
*swig_DIMEXE_set = *LibreDWGc::Dwg_Header_Variables_DIMEXE_set;
*swig_DIMRND_get = *LibreDWGc::Dwg_Header_Variables_DIMRND_get;
*swig_DIMRND_set = *LibreDWGc::Dwg_Header_Variables_DIMRND_set;
*swig_DIMDLE_get = *LibreDWGc::Dwg_Header_Variables_DIMDLE_get;
*swig_DIMDLE_set = *LibreDWGc::Dwg_Header_Variables_DIMDLE_set;
*swig_DIMTP_get = *LibreDWGc::Dwg_Header_Variables_DIMTP_get;
*swig_DIMTP_set = *LibreDWGc::Dwg_Header_Variables_DIMTP_set;
*swig_DIMTM_get = *LibreDWGc::Dwg_Header_Variables_DIMTM_get;
*swig_DIMTM_set = *LibreDWGc::Dwg_Header_Variables_DIMTM_set;
*swig_DIMFXL_get = *LibreDWGc::Dwg_Header_Variables_DIMFXL_get;
*swig_DIMFXL_set = *LibreDWGc::Dwg_Header_Variables_DIMFXL_set;
*swig_DIMJOGANG_get = *LibreDWGc::Dwg_Header_Variables_DIMJOGANG_get;
*swig_DIMJOGANG_set = *LibreDWGc::Dwg_Header_Variables_DIMJOGANG_set;
*swig_DIMTFILL_get = *LibreDWGc::Dwg_Header_Variables_DIMTFILL_get;
*swig_DIMTFILL_set = *LibreDWGc::Dwg_Header_Variables_DIMTFILL_set;
*swig_DIMTFILLCLR_get = *LibreDWGc::Dwg_Header_Variables_DIMTFILLCLR_get;
*swig_DIMTFILLCLR_set = *LibreDWGc::Dwg_Header_Variables_DIMTFILLCLR_set;
*swig_DIMAZIN_get = *LibreDWGc::Dwg_Header_Variables_DIMAZIN_get;
*swig_DIMAZIN_set = *LibreDWGc::Dwg_Header_Variables_DIMAZIN_set;
*swig_DIMARCSYM_get = *LibreDWGc::Dwg_Header_Variables_DIMARCSYM_get;
*swig_DIMARCSYM_set = *LibreDWGc::Dwg_Header_Variables_DIMARCSYM_set;
*swig_DIMTXT_get = *LibreDWGc::Dwg_Header_Variables_DIMTXT_get;
*swig_DIMTXT_set = *LibreDWGc::Dwg_Header_Variables_DIMTXT_set;
*swig_DIMCEN_get = *LibreDWGc::Dwg_Header_Variables_DIMCEN_get;
*swig_DIMCEN_set = *LibreDWGc::Dwg_Header_Variables_DIMCEN_set;
*swig_DIMTSZ_get = *LibreDWGc::Dwg_Header_Variables_DIMTSZ_get;
*swig_DIMTSZ_set = *LibreDWGc::Dwg_Header_Variables_DIMTSZ_set;
*swig_DIMALTF_get = *LibreDWGc::Dwg_Header_Variables_DIMALTF_get;
*swig_DIMALTF_set = *LibreDWGc::Dwg_Header_Variables_DIMALTF_set;
*swig_DIMLFAC_get = *LibreDWGc::Dwg_Header_Variables_DIMLFAC_get;
*swig_DIMLFAC_set = *LibreDWGc::Dwg_Header_Variables_DIMLFAC_set;
*swig_DIMTVP_get = *LibreDWGc::Dwg_Header_Variables_DIMTVP_get;
*swig_DIMTVP_set = *LibreDWGc::Dwg_Header_Variables_DIMTVP_set;
*swig_DIMTFAC_get = *LibreDWGc::Dwg_Header_Variables_DIMTFAC_get;
*swig_DIMTFAC_set = *LibreDWGc::Dwg_Header_Variables_DIMTFAC_set;
*swig_DIMGAP_get = *LibreDWGc::Dwg_Header_Variables_DIMGAP_get;
*swig_DIMGAP_set = *LibreDWGc::Dwg_Header_Variables_DIMGAP_set;
*swig_DIMPOST_T_get = *LibreDWGc::Dwg_Header_Variables_DIMPOST_T_get;
*swig_DIMPOST_T_set = *LibreDWGc::Dwg_Header_Variables_DIMPOST_T_set;
*swig_DIMAPOST_T_get = *LibreDWGc::Dwg_Header_Variables_DIMAPOST_T_get;
*swig_DIMAPOST_T_set = *LibreDWGc::Dwg_Header_Variables_DIMAPOST_T_set;
*swig_DIMBLK_T_get = *LibreDWGc::Dwg_Header_Variables_DIMBLK_T_get;
*swig_DIMBLK_T_set = *LibreDWGc::Dwg_Header_Variables_DIMBLK_T_set;
*swig_DIMBLK1_T_get = *LibreDWGc::Dwg_Header_Variables_DIMBLK1_T_get;
*swig_DIMBLK1_T_set = *LibreDWGc::Dwg_Header_Variables_DIMBLK1_T_set;
*swig_DIMBLK2_T_get = *LibreDWGc::Dwg_Header_Variables_DIMBLK2_T_get;
*swig_DIMBLK2_T_set = *LibreDWGc::Dwg_Header_Variables_DIMBLK2_T_set;
*swig_unknown_string_get = *LibreDWGc::Dwg_Header_Variables_unknown_string_get;
*swig_unknown_string_set = *LibreDWGc::Dwg_Header_Variables_unknown_string_set;
*swig_DIMALTRND_get = *LibreDWGc::Dwg_Header_Variables_DIMALTRND_get;
*swig_DIMALTRND_set = *LibreDWGc::Dwg_Header_Variables_DIMALTRND_set;
*swig_DIMCLRD_C_get = *LibreDWGc::Dwg_Header_Variables_DIMCLRD_C_get;
*swig_DIMCLRD_C_set = *LibreDWGc::Dwg_Header_Variables_DIMCLRD_C_set;
*swig_DIMCLRE_C_get = *LibreDWGc::Dwg_Header_Variables_DIMCLRE_C_get;
*swig_DIMCLRE_C_set = *LibreDWGc::Dwg_Header_Variables_DIMCLRE_C_set;
*swig_DIMCLRT_C_get = *LibreDWGc::Dwg_Header_Variables_DIMCLRT_C_get;
*swig_DIMCLRT_C_set = *LibreDWGc::Dwg_Header_Variables_DIMCLRT_C_set;
*swig_DIMCLRD_get = *LibreDWGc::Dwg_Header_Variables_DIMCLRD_get;
*swig_DIMCLRD_set = *LibreDWGc::Dwg_Header_Variables_DIMCLRD_set;
*swig_DIMCLRE_get = *LibreDWGc::Dwg_Header_Variables_DIMCLRE_get;
*swig_DIMCLRE_set = *LibreDWGc::Dwg_Header_Variables_DIMCLRE_set;
*swig_DIMCLRT_get = *LibreDWGc::Dwg_Header_Variables_DIMCLRT_get;
*swig_DIMCLRT_set = *LibreDWGc::Dwg_Header_Variables_DIMCLRT_set;
*swig_DIMADEC_get = *LibreDWGc::Dwg_Header_Variables_DIMADEC_get;
*swig_DIMADEC_set = *LibreDWGc::Dwg_Header_Variables_DIMADEC_set;
*swig_DIMFRAC_get = *LibreDWGc::Dwg_Header_Variables_DIMFRAC_get;
*swig_DIMFRAC_set = *LibreDWGc::Dwg_Header_Variables_DIMFRAC_set;
*swig_DIMLUNIT_get = *LibreDWGc::Dwg_Header_Variables_DIMLUNIT_get;
*swig_DIMLUNIT_set = *LibreDWGc::Dwg_Header_Variables_DIMLUNIT_set;
*swig_DIMDSEP_get = *LibreDWGc::Dwg_Header_Variables_DIMDSEP_get;
*swig_DIMDSEP_set = *LibreDWGc::Dwg_Header_Variables_DIMDSEP_set;
*swig_DIMTMOVE_get = *LibreDWGc::Dwg_Header_Variables_DIMTMOVE_get;
*swig_DIMTMOVE_set = *LibreDWGc::Dwg_Header_Variables_DIMTMOVE_set;
*swig_DIMALTZ_get = *LibreDWGc::Dwg_Header_Variables_DIMALTZ_get;
*swig_DIMALTZ_set = *LibreDWGc::Dwg_Header_Variables_DIMALTZ_set;
*swig_DIMALTTZ_get = *LibreDWGc::Dwg_Header_Variables_DIMALTTZ_get;
*swig_DIMALTTZ_set = *LibreDWGc::Dwg_Header_Variables_DIMALTTZ_set;
*swig_DIMATFIT_get = *LibreDWGc::Dwg_Header_Variables_DIMATFIT_get;
*swig_DIMATFIT_set = *LibreDWGc::Dwg_Header_Variables_DIMATFIT_set;
*swig_DIMFXLON_get = *LibreDWGc::Dwg_Header_Variables_DIMFXLON_get;
*swig_DIMFXLON_set = *LibreDWGc::Dwg_Header_Variables_DIMFXLON_set;
*swig_DIMTXTDIRECTION_get = *LibreDWGc::Dwg_Header_Variables_DIMTXTDIRECTION_get;
*swig_DIMTXTDIRECTION_set = *LibreDWGc::Dwg_Header_Variables_DIMTXTDIRECTION_set;
*swig_DIMALTMZF_get = *LibreDWGc::Dwg_Header_Variables_DIMALTMZF_get;
*swig_DIMALTMZF_set = *LibreDWGc::Dwg_Header_Variables_DIMALTMZF_set;
*swig_DIMALTMZS_get = *LibreDWGc::Dwg_Header_Variables_DIMALTMZS_get;
*swig_DIMALTMZS_set = *LibreDWGc::Dwg_Header_Variables_DIMALTMZS_set;
*swig_DIMMZF_get = *LibreDWGc::Dwg_Header_Variables_DIMMZF_get;
*swig_DIMMZF_set = *LibreDWGc::Dwg_Header_Variables_DIMMZF_set;
*swig_DIMMZS_get = *LibreDWGc::Dwg_Header_Variables_DIMMZS_get;
*swig_DIMMZS_set = *LibreDWGc::Dwg_Header_Variables_DIMMZS_set;
*swig_DIMLDRBLK_get = *LibreDWGc::Dwg_Header_Variables_DIMLDRBLK_get;
*swig_DIMLDRBLK_set = *LibreDWGc::Dwg_Header_Variables_DIMLDRBLK_set;
*swig_DIMBLK_get = *LibreDWGc::Dwg_Header_Variables_DIMBLK_get;
*swig_DIMBLK_set = *LibreDWGc::Dwg_Header_Variables_DIMBLK_set;
*swig_DIMBLK1_get = *LibreDWGc::Dwg_Header_Variables_DIMBLK1_get;
*swig_DIMBLK1_set = *LibreDWGc::Dwg_Header_Variables_DIMBLK1_set;
*swig_DIMBLK2_get = *LibreDWGc::Dwg_Header_Variables_DIMBLK2_get;
*swig_DIMBLK2_set = *LibreDWGc::Dwg_Header_Variables_DIMBLK2_set;
*swig_DIMLTYPE_get = *LibreDWGc::Dwg_Header_Variables_DIMLTYPE_get;
*swig_DIMLTYPE_set = *LibreDWGc::Dwg_Header_Variables_DIMLTYPE_set;
*swig_DIMLTEX1_get = *LibreDWGc::Dwg_Header_Variables_DIMLTEX1_get;
*swig_DIMLTEX1_set = *LibreDWGc::Dwg_Header_Variables_DIMLTEX1_set;
*swig_DIMLTEX2_get = *LibreDWGc::Dwg_Header_Variables_DIMLTEX2_get;
*swig_DIMLTEX2_set = *LibreDWGc::Dwg_Header_Variables_DIMLTEX2_set;
*swig_DIMLWD_get = *LibreDWGc::Dwg_Header_Variables_DIMLWD_get;
*swig_DIMLWD_set = *LibreDWGc::Dwg_Header_Variables_DIMLWD_set;
*swig_DIMLWE_get = *LibreDWGc::Dwg_Header_Variables_DIMLWE_get;
*swig_DIMLWE_set = *LibreDWGc::Dwg_Header_Variables_DIMLWE_set;
*swig_BLOCK_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_get;
*swig_BLOCK_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_BLOCK_CONTROL_OBJECT_set;
*swig_LAYER_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_LAYER_CONTROL_OBJECT_get;
*swig_LAYER_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_LAYER_CONTROL_OBJECT_set;
*swig_STYLE_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_STYLE_CONTROL_OBJECT_get;
*swig_STYLE_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_STYLE_CONTROL_OBJECT_set;
*swig_LTYPE_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_get;
*swig_LTYPE_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_LTYPE_CONTROL_OBJECT_set;
*swig_VIEW_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_VIEW_CONTROL_OBJECT_get;
*swig_VIEW_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_VIEW_CONTROL_OBJECT_set;
*swig_UCS_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_UCS_CONTROL_OBJECT_get;
*swig_UCS_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_UCS_CONTROL_OBJECT_set;
*swig_VPORT_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_VPORT_CONTROL_OBJECT_get;
*swig_VPORT_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_VPORT_CONTROL_OBJECT_set;
*swig_APPID_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_APPID_CONTROL_OBJECT_get;
*swig_APPID_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_APPID_CONTROL_OBJECT_set;
*swig_DIMSTYLE_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_get;
*swig_DIMSTYLE_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_DIMSTYLE_CONTROL_OBJECT_set;
*swig_VX_CONTROL_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_VX_CONTROL_OBJECT_get;
*swig_VX_CONTROL_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_VX_CONTROL_OBJECT_set;
*swig_DICTIONARY_ACAD_GROUP_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_get;
*swig_DICTIONARY_ACAD_GROUP_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_GROUP_set;
*swig_DICTIONARY_ACAD_MLINESTYLE_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_get;
*swig_DICTIONARY_ACAD_MLINESTYLE_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_ACAD_MLINESTYLE_set;
*swig_DICTIONARY_NAMED_OBJECT_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_get;
*swig_DICTIONARY_NAMED_OBJECT_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_NAMED_OBJECT_set;
*swig_TSTACKALIGN_get = *LibreDWGc::Dwg_Header_Variables_TSTACKALIGN_get;
*swig_TSTACKALIGN_set = *LibreDWGc::Dwg_Header_Variables_TSTACKALIGN_set;
*swig_TSTACKSIZE_get = *LibreDWGc::Dwg_Header_Variables_TSTACKSIZE_get;
*swig_TSTACKSIZE_set = *LibreDWGc::Dwg_Header_Variables_TSTACKSIZE_set;
*swig_HYPERLINKBASE_get = *LibreDWGc::Dwg_Header_Variables_HYPERLINKBASE_get;
*swig_HYPERLINKBASE_set = *LibreDWGc::Dwg_Header_Variables_HYPERLINKBASE_set;
*swig_STYLESHEET_get = *LibreDWGc::Dwg_Header_Variables_STYLESHEET_get;
*swig_STYLESHEET_set = *LibreDWGc::Dwg_Header_Variables_STYLESHEET_set;
*swig_DICTIONARY_LAYOUT_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_LAYOUT_get;
*swig_DICTIONARY_LAYOUT_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_LAYOUT_set;
*swig_DICTIONARY_PLOTSETTINGS_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_get;
*swig_DICTIONARY_PLOTSETTINGS_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSETTINGS_set;
*swig_DICTIONARY_PLOTSTYLENAME_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_get;
*swig_DICTIONARY_PLOTSTYLENAME_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_PLOTSTYLENAME_set;
*swig_DICTIONARY_MATERIAL_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_MATERIAL_get;
*swig_DICTIONARY_MATERIAL_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_MATERIAL_set;
*swig_DICTIONARY_COLOR_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_COLOR_get;
*swig_DICTIONARY_COLOR_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_COLOR_set;
*swig_DICTIONARY_VISUALSTYLE_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_get;
*swig_DICTIONARY_VISUALSTYLE_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_VISUALSTYLE_set;
*swig_DICTIONARY_LIGHTLIST_get = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_LIGHTLIST_get;
*swig_DICTIONARY_LIGHTLIST_set = *LibreDWGc::Dwg_Header_Variables_DICTIONARY_LIGHTLIST_set;
*swig_unknown_20_get = *LibreDWGc::Dwg_Header_Variables_unknown_20_get;
*swig_unknown_20_set = *LibreDWGc::Dwg_Header_Variables_unknown_20_set;
*swig_FLAGS_get = *LibreDWGc::Dwg_Header_Variables_FLAGS_get;
*swig_FLAGS_set = *LibreDWGc::Dwg_Header_Variables_FLAGS_set;
*swig_CELWEIGHT_get = *LibreDWGc::Dwg_Header_Variables_CELWEIGHT_get;
*swig_CELWEIGHT_set = *LibreDWGc::Dwg_Header_Variables_CELWEIGHT_set;
*swig_ENDCAPS_get = *LibreDWGc::Dwg_Header_Variables_ENDCAPS_get;
*swig_ENDCAPS_set = *LibreDWGc::Dwg_Header_Variables_ENDCAPS_set;
*swig_JOINSTYLE_get = *LibreDWGc::Dwg_Header_Variables_JOINSTYLE_get;
*swig_JOINSTYLE_set = *LibreDWGc::Dwg_Header_Variables_JOINSTYLE_set;
*swig_LWDISPLAY_get = *LibreDWGc::Dwg_Header_Variables_LWDISPLAY_get;
*swig_LWDISPLAY_set = *LibreDWGc::Dwg_Header_Variables_LWDISPLAY_set;
*swig_XEDIT_get = *LibreDWGc::Dwg_Header_Variables_XEDIT_get;
*swig_XEDIT_set = *LibreDWGc::Dwg_Header_Variables_XEDIT_set;
*swig_EXTNAMES_get = *LibreDWGc::Dwg_Header_Variables_EXTNAMES_get;
*swig_EXTNAMES_set = *LibreDWGc::Dwg_Header_Variables_EXTNAMES_set;
*swig_PSTYLEMODE_get = *LibreDWGc::Dwg_Header_Variables_PSTYLEMODE_get;
*swig_PSTYLEMODE_set = *LibreDWGc::Dwg_Header_Variables_PSTYLEMODE_set;
*swig_OLESTARTUP_get = *LibreDWGc::Dwg_Header_Variables_OLESTARTUP_get;
*swig_OLESTARTUP_set = *LibreDWGc::Dwg_Header_Variables_OLESTARTUP_set;
*swig_INSUNITS_get = *LibreDWGc::Dwg_Header_Variables_INSUNITS_get;
*swig_INSUNITS_set = *LibreDWGc::Dwg_Header_Variables_INSUNITS_set;
*swig_CEPSNTYPE_get = *LibreDWGc::Dwg_Header_Variables_CEPSNTYPE_get;
*swig_CEPSNTYPE_set = *LibreDWGc::Dwg_Header_Variables_CEPSNTYPE_set;
*swig_CPSNID_get = *LibreDWGc::Dwg_Header_Variables_CPSNID_get;
*swig_CPSNID_set = *LibreDWGc::Dwg_Header_Variables_CPSNID_set;
*swig_FINGERPRINTGUID_get = *LibreDWGc::Dwg_Header_Variables_FINGERPRINTGUID_get;
*swig_FINGERPRINTGUID_set = *LibreDWGc::Dwg_Header_Variables_FINGERPRINTGUID_set;
*swig_VERSIONGUID_get = *LibreDWGc::Dwg_Header_Variables_VERSIONGUID_get;
*swig_VERSIONGUID_set = *LibreDWGc::Dwg_Header_Variables_VERSIONGUID_set;
*swig_SORTENTS_get = *LibreDWGc::Dwg_Header_Variables_SORTENTS_get;
*swig_SORTENTS_set = *LibreDWGc::Dwg_Header_Variables_SORTENTS_set;
*swig_INDEXCTL_get = *LibreDWGc::Dwg_Header_Variables_INDEXCTL_get;
*swig_INDEXCTL_set = *LibreDWGc::Dwg_Header_Variables_INDEXCTL_set;
*swig_HIDETEXT_get = *LibreDWGc::Dwg_Header_Variables_HIDETEXT_get;
*swig_HIDETEXT_set = *LibreDWGc::Dwg_Header_Variables_HIDETEXT_set;
*swig_XCLIPFRAME_get = *LibreDWGc::Dwg_Header_Variables_XCLIPFRAME_get;
*swig_XCLIPFRAME_set = *LibreDWGc::Dwg_Header_Variables_XCLIPFRAME_set;
*swig_DIMASSOC_get = *LibreDWGc::Dwg_Header_Variables_DIMASSOC_get;
*swig_DIMASSOC_set = *LibreDWGc::Dwg_Header_Variables_DIMASSOC_set;
*swig_HALOGAP_get = *LibreDWGc::Dwg_Header_Variables_HALOGAP_get;
*swig_HALOGAP_set = *LibreDWGc::Dwg_Header_Variables_HALOGAP_set;
*swig_OBSCOLOR_get = *LibreDWGc::Dwg_Header_Variables_OBSCOLOR_get;
*swig_OBSCOLOR_set = *LibreDWGc::Dwg_Header_Variables_OBSCOLOR_set;
*swig_INTERSECTIONCOLOR_get = *LibreDWGc::Dwg_Header_Variables_INTERSECTIONCOLOR_get;
*swig_INTERSECTIONCOLOR_set = *LibreDWGc::Dwg_Header_Variables_INTERSECTIONCOLOR_set;
*swig_OBSLTYPE_get = *LibreDWGc::Dwg_Header_Variables_OBSLTYPE_get;
*swig_OBSLTYPE_set = *LibreDWGc::Dwg_Header_Variables_OBSLTYPE_set;
*swig_INTERSECTIONDISPLAY_get = *LibreDWGc::Dwg_Header_Variables_INTERSECTIONDISPLAY_get;
*swig_INTERSECTIONDISPLAY_set = *LibreDWGc::Dwg_Header_Variables_INTERSECTIONDISPLAY_set;
*swig_PROJECTNAME_get = *LibreDWGc::Dwg_Header_Variables_PROJECTNAME_get;
*swig_PROJECTNAME_set = *LibreDWGc::Dwg_Header_Variables_PROJECTNAME_set;
*swig_BLOCK_RECORD_PSPACE_get = *LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_PSPACE_get;
*swig_BLOCK_RECORD_PSPACE_set = *LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_PSPACE_set;
*swig_BLOCK_RECORD_MSPACE_get = *LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_MSPACE_get;
*swig_BLOCK_RECORD_MSPACE_set = *LibreDWGc::Dwg_Header_Variables_BLOCK_RECORD_MSPACE_set;
*swig_LTYPE_BYLAYER_get = *LibreDWGc::Dwg_Header_Variables_LTYPE_BYLAYER_get;
*swig_LTYPE_BYLAYER_set = *LibreDWGc::Dwg_Header_Variables_LTYPE_BYLAYER_set;
*swig_LTYPE_BYBLOCK_get = *LibreDWGc::Dwg_Header_Variables_LTYPE_BYBLOCK_get;
*swig_LTYPE_BYBLOCK_set = *LibreDWGc::Dwg_Header_Variables_LTYPE_BYBLOCK_set;
*swig_LTYPE_CONTINUOUS_get = *LibreDWGc::Dwg_Header_Variables_LTYPE_CONTINUOUS_get;
*swig_LTYPE_CONTINUOUS_set = *LibreDWGc::Dwg_Header_Variables_LTYPE_CONTINUOUS_set;
*swig_CAMERADISPLAY_get = *LibreDWGc::Dwg_Header_Variables_CAMERADISPLAY_get;
*swig_CAMERADISPLAY_set = *LibreDWGc::Dwg_Header_Variables_CAMERADISPLAY_set;
*swig_unknown_21_get = *LibreDWGc::Dwg_Header_Variables_unknown_21_get;
*swig_unknown_21_set = *LibreDWGc::Dwg_Header_Variables_unknown_21_set;
*swig_unknown_22_get = *LibreDWGc::Dwg_Header_Variables_unknown_22_get;
*swig_unknown_22_set = *LibreDWGc::Dwg_Header_Variables_unknown_22_set;
*swig_unknown_23_get = *LibreDWGc::Dwg_Header_Variables_unknown_23_get;
*swig_unknown_23_set = *LibreDWGc::Dwg_Header_Variables_unknown_23_set;
*swig_STEPSPERSEC_get = *LibreDWGc::Dwg_Header_Variables_STEPSPERSEC_get;
*swig_STEPSPERSEC_set = *LibreDWGc::Dwg_Header_Variables_STEPSPERSEC_set;
*swig_STEPSIZE_get = *LibreDWGc::Dwg_Header_Variables_STEPSIZE_get;
*swig_STEPSIZE_set = *LibreDWGc::Dwg_Header_Variables_STEPSIZE_set;
*swig__3DDWFPREC_get = *LibreDWGc::Dwg_Header_Variables__3DDWFPREC_get;
*swig__3DDWFPREC_set = *LibreDWGc::Dwg_Header_Variables__3DDWFPREC_set;
*swig_LENSLENGTH_get = *LibreDWGc::Dwg_Header_Variables_LENSLENGTH_get;
*swig_LENSLENGTH_set = *LibreDWGc::Dwg_Header_Variables_LENSLENGTH_set;
*swig_CAMERAHEIGHT_get = *LibreDWGc::Dwg_Header_Variables_CAMERAHEIGHT_get;
*swig_CAMERAHEIGHT_set = *LibreDWGc::Dwg_Header_Variables_CAMERAHEIGHT_set;
*swig_SOLIDHIST_get = *LibreDWGc::Dwg_Header_Variables_SOLIDHIST_get;
*swig_SOLIDHIST_set = *LibreDWGc::Dwg_Header_Variables_SOLIDHIST_set;
*swig_SHOWHIST_get = *LibreDWGc::Dwg_Header_Variables_SHOWHIST_get;
*swig_SHOWHIST_set = *LibreDWGc::Dwg_Header_Variables_SHOWHIST_set;
*swig_PSOLWIDTH_get = *LibreDWGc::Dwg_Header_Variables_PSOLWIDTH_get;
*swig_PSOLWIDTH_set = *LibreDWGc::Dwg_Header_Variables_PSOLWIDTH_set;
*swig_PSOLHEIGHT_get = *LibreDWGc::Dwg_Header_Variables_PSOLHEIGHT_get;
*swig_PSOLHEIGHT_set = *LibreDWGc::Dwg_Header_Variables_PSOLHEIGHT_set;
*swig_LOFTANG1_get = *LibreDWGc::Dwg_Header_Variables_LOFTANG1_get;
*swig_LOFTANG1_set = *LibreDWGc::Dwg_Header_Variables_LOFTANG1_set;
*swig_LOFTANG2_get = *LibreDWGc::Dwg_Header_Variables_LOFTANG2_get;
*swig_LOFTANG2_set = *LibreDWGc::Dwg_Header_Variables_LOFTANG2_set;
*swig_LOFTMAG1_get = *LibreDWGc::Dwg_Header_Variables_LOFTMAG1_get;
*swig_LOFTMAG1_set = *LibreDWGc::Dwg_Header_Variables_LOFTMAG1_set;
*swig_LOFTMAG2_get = *LibreDWGc::Dwg_Header_Variables_LOFTMAG2_get;
*swig_LOFTMAG2_set = *LibreDWGc::Dwg_Header_Variables_LOFTMAG2_set;
*swig_LOFTPARAM_get = *LibreDWGc::Dwg_Header_Variables_LOFTPARAM_get;
*swig_LOFTPARAM_set = *LibreDWGc::Dwg_Header_Variables_LOFTPARAM_set;
*swig_LOFTNORMALS_get = *LibreDWGc::Dwg_Header_Variables_LOFTNORMALS_get;
*swig_LOFTNORMALS_set = *LibreDWGc::Dwg_Header_Variables_LOFTNORMALS_set;
*swig_LATITUDE_get = *LibreDWGc::Dwg_Header_Variables_LATITUDE_get;
*swig_LATITUDE_set = *LibreDWGc::Dwg_Header_Variables_LATITUDE_set;
*swig_LONGITUDE_get = *LibreDWGc::Dwg_Header_Variables_LONGITUDE_get;
*swig_LONGITUDE_set = *LibreDWGc::Dwg_Header_Variables_LONGITUDE_set;
*swig_NORTHDIRECTION_get = *LibreDWGc::Dwg_Header_Variables_NORTHDIRECTION_get;
*swig_NORTHDIRECTION_set = *LibreDWGc::Dwg_Header_Variables_NORTHDIRECTION_set;
*swig_TIMEZONE_get = *LibreDWGc::Dwg_Header_Variables_TIMEZONE_get;
*swig_TIMEZONE_set = *LibreDWGc::Dwg_Header_Variables_TIMEZONE_set;
*swig_LIGHTGLYPHDISPLAY_get = *LibreDWGc::Dwg_Header_Variables_LIGHTGLYPHDISPLAY_get;
*swig_LIGHTGLYPHDISPLAY_set = *LibreDWGc::Dwg_Header_Variables_LIGHTGLYPHDISPLAY_set;
*swig_TILEMODELIGHTSYNCH_get = *LibreDWGc::Dwg_Header_Variables_TILEMODELIGHTSYNCH_get;
*swig_TILEMODELIGHTSYNCH_set = *LibreDWGc::Dwg_Header_Variables_TILEMODELIGHTSYNCH_set;
*swig_DWFFRAME_get = *LibreDWGc::Dwg_Header_Variables_DWFFRAME_get;
*swig_DWFFRAME_set = *LibreDWGc::Dwg_Header_Variables_DWFFRAME_set;
*swig_DGNFRAME_get = *LibreDWGc::Dwg_Header_Variables_DGNFRAME_get;
*swig_DGNFRAME_set = *LibreDWGc::Dwg_Header_Variables_DGNFRAME_set;
*swig_REALWORLDSCALE_get = *LibreDWGc::Dwg_Header_Variables_REALWORLDSCALE_get;
*swig_REALWORLDSCALE_set = *LibreDWGc::Dwg_Header_Variables_REALWORLDSCALE_set;
*swig_INTERFERECOLOR_get = *LibreDWGc::Dwg_Header_Variables_INTERFERECOLOR_get;
*swig_INTERFERECOLOR_set = *LibreDWGc::Dwg_Header_Variables_INTERFERECOLOR_set;
*swig_INTERFEREOBJVS_get = *LibreDWGc::Dwg_Header_Variables_INTERFEREOBJVS_get;
*swig_INTERFEREOBJVS_set = *LibreDWGc::Dwg_Header_Variables_INTERFEREOBJVS_set;
*swig_INTERFEREVPVS_get = *LibreDWGc::Dwg_Header_Variables_INTERFEREVPVS_get;
*swig_INTERFEREVPVS_set = *LibreDWGc::Dwg_Header_Variables_INTERFEREVPVS_set;
*swig_DRAGVS_get = *LibreDWGc::Dwg_Header_Variables_DRAGVS_get;
*swig_DRAGVS_set = *LibreDWGc::Dwg_Header_Variables_DRAGVS_set;
*swig_CSHADOW_get = *LibreDWGc::Dwg_Header_Variables_CSHADOW_get;
*swig_CSHADOW_set = *LibreDWGc::Dwg_Header_Variables_CSHADOW_set;
*swig_SHADOWPLANELOCATION_get = *LibreDWGc::Dwg_Header_Variables_SHADOWPLANELOCATION_get;
*swig_SHADOWPLANELOCATION_set = *LibreDWGc::Dwg_Header_Variables_SHADOWPLANELOCATION_set;
*swig_unknown_54_get = *LibreDWGc::Dwg_Header_Variables_unknown_54_get;
*swig_unknown_54_set = *LibreDWGc::Dwg_Header_Variables_unknown_54_set;
*swig_unknown_55_get = *LibreDWGc::Dwg_Header_Variables_unknown_55_get;
*swig_unknown_55_set = *LibreDWGc::Dwg_Header_Variables_unknown_55_set;
*swig_unknown_56_get = *LibreDWGc::Dwg_Header_Variables_unknown_56_get;
*swig_unknown_56_set = *LibreDWGc::Dwg_Header_Variables_unknown_56_set;
*swig_unknown_57_get = *LibreDWGc::Dwg_Header_Variables_unknown_57_get;
*swig_unknown_57_set = *LibreDWGc::Dwg_Header_Variables_unknown_57_set;
*swig_dwg_size_get = *LibreDWGc::Dwg_Header_Variables_dwg_size_get;
*swig_dwg_size_set = *LibreDWGc::Dwg_Header_Variables_dwg_size_set;
*swig_numentities_get = *LibreDWGc::Dwg_Header_Variables_numentities_get;
*swig_numentities_set = *LibreDWGc::Dwg_Header_Variables_numentities_set;
*swig_circle_zoom_percent_get = *LibreDWGc::Dwg_Header_Variables_circle_zoom_percent_get;
*swig_circle_zoom_percent_set = *LibreDWGc::Dwg_Header_Variables_circle_zoom_percent_set;
*swig_FRONTZ_get = *LibreDWGc::Dwg_Header_Variables_FRONTZ_get;
*swig_FRONTZ_set = *LibreDWGc::Dwg_Header_Variables_FRONTZ_set;
*swig_BACKZ_get = *LibreDWGc::Dwg_Header_Variables_BACKZ_get;
*swig_BACKZ_set = *LibreDWGc::Dwg_Header_Variables_BACKZ_set;
*swig_UCSICON_get = *LibreDWGc::Dwg_Header_Variables_UCSICON_get;
*swig_UCSICON_set = *LibreDWGc::Dwg_Header_Variables_UCSICON_set;
*swig_oldCECOLOR_hi_get = *LibreDWGc::Dwg_Header_Variables_oldCECOLOR_hi_get;
*swig_oldCECOLOR_hi_set = *LibreDWGc::Dwg_Header_Variables_oldCECOLOR_hi_set;
*swig_oldCECOLOR_lo_get = *LibreDWGc::Dwg_Header_Variables_oldCECOLOR_lo_get;
*swig_oldCECOLOR_lo_set = *LibreDWGc::Dwg_Header_Variables_oldCECOLOR_lo_set;
*swig_layer_colors_get = *LibreDWGc::Dwg_Header_Variables_layer_colors_get;
*swig_layer_colors_set = *LibreDWGc::Dwg_Header_Variables_layer_colors_set;
*swig_unknown_51e_get = *LibreDWGc::Dwg_Header_Variables_unknown_51e_get;
*swig_unknown_51e_set = *LibreDWGc::Dwg_Header_Variables_unknown_51e_set;
*swig_unknown_520_get = *LibreDWGc::Dwg_Header_Variables_unknown_520_get;
*swig_unknown_520_set = *LibreDWGc::Dwg_Header_Variables_unknown_520_set;
*swig_unknown_52c_get = *LibreDWGc::Dwg_Header_Variables_unknown_52c_get;
*swig_unknown_52c_set = *LibreDWGc::Dwg_Header_Variables_unknown_52c_set;
*swig_unknown_52e_get = *LibreDWGc::Dwg_Header_Variables_unknown_52e_get;
*swig_unknown_52e_set = *LibreDWGc::Dwg_Header_Variables_unknown_52e_set;
*swig_unknown_530_get = *LibreDWGc::Dwg_Header_Variables_unknown_530_get;
*swig_unknown_530_set = *LibreDWGc::Dwg_Header_Variables_unknown_530_set;
*swig_unknown_59_get = *LibreDWGc::Dwg_Header_Variables_unknown_59_get;
*swig_unknown_59_set = *LibreDWGc::Dwg_Header_Variables_unknown_59_set;
*swig_aspect_ratio_get = *LibreDWGc::Dwg_Header_Variables_aspect_ratio_get;
*swig_aspect_ratio_set = *LibreDWGc::Dwg_Header_Variables_aspect_ratio_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Header_Variables(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Header_Variables($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_TEXT ##############

package LibreDWG::Dwg_Entity_TEXT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_TEXT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_TEXT_parent_set;
*swig_dataflags_get = *LibreDWGc::Dwg_Entity_TEXT_dataflags_get;
*swig_dataflags_set = *LibreDWGc::Dwg_Entity_TEXT_dataflags_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_TEXT_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_TEXT_elevation_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_TEXT_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_TEXT_ins_pt_set;
*swig_alignment_pt_get = *LibreDWGc::Dwg_Entity_TEXT_alignment_pt_get;
*swig_alignment_pt_set = *LibreDWGc::Dwg_Entity_TEXT_alignment_pt_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_TEXT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_TEXT_extrusion_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_TEXT_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_TEXT_thickness_set;
*swig_oblique_angle_get = *LibreDWGc::Dwg_Entity_TEXT_oblique_angle_get;
*swig_oblique_angle_set = *LibreDWGc::Dwg_Entity_TEXT_oblique_angle_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_TEXT_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_TEXT_rotation_set;
*swig_height_get = *LibreDWGc::Dwg_Entity_TEXT_height_get;
*swig_height_set = *LibreDWGc::Dwg_Entity_TEXT_height_set;
*swig_width_factor_get = *LibreDWGc::Dwg_Entity_TEXT_width_factor_get;
*swig_width_factor_set = *LibreDWGc::Dwg_Entity_TEXT_width_factor_set;
*swig_text_value_get = *LibreDWGc::Dwg_Entity_TEXT_text_value_get;
*swig_text_value_set = *LibreDWGc::Dwg_Entity_TEXT_text_value_set;
*swig_generation_get = *LibreDWGc::Dwg_Entity_TEXT_generation_get;
*swig_generation_set = *LibreDWGc::Dwg_Entity_TEXT_generation_set;
*swig_horiz_alignment_get = *LibreDWGc::Dwg_Entity_TEXT_horiz_alignment_get;
*swig_horiz_alignment_set = *LibreDWGc::Dwg_Entity_TEXT_horiz_alignment_set;
*swig_vert_alignment_get = *LibreDWGc::Dwg_Entity_TEXT_vert_alignment_get;
*swig_vert_alignment_set = *LibreDWGc::Dwg_Entity_TEXT_vert_alignment_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_TEXT_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_TEXT_style_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_TEXT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_TEXT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDbMTextObjectEmbedded ##############

package LibreDWG::Dwg_AcDbMTextObjectEmbedded;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_attachment_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_attachment_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_ins_pt_set;
*swig_x_axis_dir_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_x_axis_dir_get;
*swig_x_axis_dir_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_x_axis_dir_set;
*swig_rect_height_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_height_get;
*swig_rect_height_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_height_set;
*swig_rect_width_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_width_get;
*swig_rect_width_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_rect_width_set;
*swig_extents_width_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_width_get;
*swig_extents_width_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_width_set;
*swig_extents_height_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_height_get;
*swig_extents_height_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_extents_height_set;
*swig_column_type_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_type_get;
*swig_column_type_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_type_set;
*swig_column_width_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_width_get;
*swig_column_width_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_width_set;
*swig_gutter_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_gutter_get;
*swig_gutter_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_gutter_set;
*swig_auto_height_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_auto_height_get;
*swig_auto_height_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_auto_height_set;
*swig_flow_reversed_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_flow_reversed_get;
*swig_flow_reversed_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_flow_reversed_set;
*swig_num_column_heights_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_num_column_heights_get;
*swig_num_column_heights_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_num_column_heights_set;
*swig_column_heights_get = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_heights_get;
*swig_column_heights_set = *LibreDWGc::Dwg_AcDbMTextObjectEmbedded_column_heights_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDbMTextObjectEmbedded(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDbMTextObjectEmbedded($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ATTRIB ##############

package LibreDWG::Dwg_Entity_ATTRIB;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ATTRIB_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ATTRIB_parent_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_ATTRIB_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_ATTRIB_elevation_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_ATTRIB_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_ATTRIB_ins_pt_set;
*swig_alignment_pt_get = *LibreDWGc::Dwg_Entity_ATTRIB_alignment_pt_get;
*swig_alignment_pt_set = *LibreDWGc::Dwg_Entity_ATTRIB_alignment_pt_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_ATTRIB_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_ATTRIB_extrusion_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_ATTRIB_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_ATTRIB_thickness_set;
*swig_oblique_angle_get = *LibreDWGc::Dwg_Entity_ATTRIB_oblique_angle_get;
*swig_oblique_angle_set = *LibreDWGc::Dwg_Entity_ATTRIB_oblique_angle_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_ATTRIB_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_ATTRIB_rotation_set;
*swig_height_get = *LibreDWGc::Dwg_Entity_ATTRIB_height_get;
*swig_height_set = *LibreDWGc::Dwg_Entity_ATTRIB_height_set;
*swig_width_factor_get = *LibreDWGc::Dwg_Entity_ATTRIB_width_factor_get;
*swig_width_factor_set = *LibreDWGc::Dwg_Entity_ATTRIB_width_factor_set;
*swig_text_value_get = *LibreDWGc::Dwg_Entity_ATTRIB_text_value_get;
*swig_text_value_set = *LibreDWGc::Dwg_Entity_ATTRIB_text_value_set;
*swig_generation_get = *LibreDWGc::Dwg_Entity_ATTRIB_generation_get;
*swig_generation_set = *LibreDWGc::Dwg_Entity_ATTRIB_generation_set;
*swig_horiz_alignment_get = *LibreDWGc::Dwg_Entity_ATTRIB_horiz_alignment_get;
*swig_horiz_alignment_set = *LibreDWGc::Dwg_Entity_ATTRIB_horiz_alignment_set;
*swig_vert_alignment_get = *LibreDWGc::Dwg_Entity_ATTRIB_vert_alignment_get;
*swig_vert_alignment_set = *LibreDWGc::Dwg_Entity_ATTRIB_vert_alignment_set;
*swig_dataflags_get = *LibreDWGc::Dwg_Entity_ATTRIB_dataflags_get;
*swig_dataflags_set = *LibreDWGc::Dwg_Entity_ATTRIB_dataflags_set;
*swig_is_locked_in_block_get = *LibreDWGc::Dwg_Entity_ATTRIB_is_locked_in_block_get;
*swig_is_locked_in_block_set = *LibreDWGc::Dwg_Entity_ATTRIB_is_locked_in_block_set;
*swig_keep_duplicate_records_get = *LibreDWGc::Dwg_Entity_ATTRIB_keep_duplicate_records_get;
*swig_keep_duplicate_records_set = *LibreDWGc::Dwg_Entity_ATTRIB_keep_duplicate_records_set;
*swig_tag_get = *LibreDWGc::Dwg_Entity_ATTRIB_tag_get;
*swig_tag_set = *LibreDWGc::Dwg_Entity_ATTRIB_tag_set;
*swig_field_length_get = *LibreDWGc::Dwg_Entity_ATTRIB_field_length_get;
*swig_field_length_set = *LibreDWGc::Dwg_Entity_ATTRIB_field_length_set;
*swig_flags_get = *LibreDWGc::Dwg_Entity_ATTRIB_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Entity_ATTRIB_flags_set;
*swig_lock_position_flag_get = *LibreDWGc::Dwg_Entity_ATTRIB_lock_position_flag_get;
*swig_lock_position_flag_set = *LibreDWGc::Dwg_Entity_ATTRIB_lock_position_flag_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_ATTRIB_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_ATTRIB_style_set;
*swig_mtext_type_get = *LibreDWGc::Dwg_Entity_ATTRIB_mtext_type_get;
*swig_mtext_type_set = *LibreDWGc::Dwg_Entity_ATTRIB_mtext_type_set;
*swig_mtext_get = *LibreDWGc::Dwg_Entity_ATTRIB_mtext_get;
*swig_mtext_set = *LibreDWGc::Dwg_Entity_ATTRIB_mtext_set;
*swig_is_really_locked_get = *LibreDWGc::Dwg_Entity_ATTRIB_is_really_locked_get;
*swig_is_really_locked_set = *LibreDWGc::Dwg_Entity_ATTRIB_is_really_locked_set;
*swig_annotative_data_size_get = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_size_get;
*swig_annotative_data_size_set = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_size_set;
*swig_annotative_data_get = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_get;
*swig_annotative_data_set = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_data_set;
*swig_annotative_flag_get = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_flag_get;
*swig_annotative_flag_set = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_flag_set;
*swig_annotative_style_get = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_style_get;
*swig_annotative_style_set = *LibreDWGc::Dwg_Entity_ATTRIB_annotative_style_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ATTRIB(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ATTRIB($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ATTDEF ##############

package LibreDWG::Dwg_Entity_ATTDEF;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ATTDEF_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ATTDEF_parent_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_ATTDEF_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_ATTDEF_elevation_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_ATTDEF_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_ATTDEF_ins_pt_set;
*swig_alignment_pt_get = *LibreDWGc::Dwg_Entity_ATTDEF_alignment_pt_get;
*swig_alignment_pt_set = *LibreDWGc::Dwg_Entity_ATTDEF_alignment_pt_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_ATTDEF_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_ATTDEF_extrusion_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_ATTDEF_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_ATTDEF_thickness_set;
*swig_oblique_angle_get = *LibreDWGc::Dwg_Entity_ATTDEF_oblique_angle_get;
*swig_oblique_angle_set = *LibreDWGc::Dwg_Entity_ATTDEF_oblique_angle_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_ATTDEF_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_ATTDEF_rotation_set;
*swig_height_get = *LibreDWGc::Dwg_Entity_ATTDEF_height_get;
*swig_height_set = *LibreDWGc::Dwg_Entity_ATTDEF_height_set;
*swig_width_factor_get = *LibreDWGc::Dwg_Entity_ATTDEF_width_factor_get;
*swig_width_factor_set = *LibreDWGc::Dwg_Entity_ATTDEF_width_factor_set;
*swig_default_value_get = *LibreDWGc::Dwg_Entity_ATTDEF_default_value_get;
*swig_default_value_set = *LibreDWGc::Dwg_Entity_ATTDEF_default_value_set;
*swig_generation_get = *LibreDWGc::Dwg_Entity_ATTDEF_generation_get;
*swig_generation_set = *LibreDWGc::Dwg_Entity_ATTDEF_generation_set;
*swig_horiz_alignment_get = *LibreDWGc::Dwg_Entity_ATTDEF_horiz_alignment_get;
*swig_horiz_alignment_set = *LibreDWGc::Dwg_Entity_ATTDEF_horiz_alignment_set;
*swig_vert_alignment_get = *LibreDWGc::Dwg_Entity_ATTDEF_vert_alignment_get;
*swig_vert_alignment_set = *LibreDWGc::Dwg_Entity_ATTDEF_vert_alignment_set;
*swig_dataflags_get = *LibreDWGc::Dwg_Entity_ATTDEF_dataflags_get;
*swig_dataflags_set = *LibreDWGc::Dwg_Entity_ATTDEF_dataflags_set;
*swig_is_locked_in_block_get = *LibreDWGc::Dwg_Entity_ATTDEF_is_locked_in_block_get;
*swig_is_locked_in_block_set = *LibreDWGc::Dwg_Entity_ATTDEF_is_locked_in_block_set;
*swig_keep_duplicate_records_get = *LibreDWGc::Dwg_Entity_ATTDEF_keep_duplicate_records_get;
*swig_keep_duplicate_records_set = *LibreDWGc::Dwg_Entity_ATTDEF_keep_duplicate_records_set;
*swig_tag_get = *LibreDWGc::Dwg_Entity_ATTDEF_tag_get;
*swig_tag_set = *LibreDWGc::Dwg_Entity_ATTDEF_tag_set;
*swig_field_length_get = *LibreDWGc::Dwg_Entity_ATTDEF_field_length_get;
*swig_field_length_set = *LibreDWGc::Dwg_Entity_ATTDEF_field_length_set;
*swig_flags_get = *LibreDWGc::Dwg_Entity_ATTDEF_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Entity_ATTDEF_flags_set;
*swig_lock_position_flag_get = *LibreDWGc::Dwg_Entity_ATTDEF_lock_position_flag_get;
*swig_lock_position_flag_set = *LibreDWGc::Dwg_Entity_ATTDEF_lock_position_flag_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_ATTDEF_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_ATTDEF_style_set;
*swig_mtext_type_get = *LibreDWGc::Dwg_Entity_ATTDEF_mtext_type_get;
*swig_mtext_type_set = *LibreDWGc::Dwg_Entity_ATTDEF_mtext_type_set;
*swig_mtext_get = *LibreDWGc::Dwg_Entity_ATTDEF_mtext_get;
*swig_mtext_set = *LibreDWGc::Dwg_Entity_ATTDEF_mtext_set;
*swig_is_really_locked_get = *LibreDWGc::Dwg_Entity_ATTDEF_is_really_locked_get;
*swig_is_really_locked_set = *LibreDWGc::Dwg_Entity_ATTDEF_is_really_locked_set;
*swig_annotative_data_size_get = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_size_get;
*swig_annotative_data_size_set = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_size_set;
*swig_annotative_data_get = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_get;
*swig_annotative_data_set = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_data_set;
*swig_annotative_flag_get = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_flag_get;
*swig_annotative_flag_set = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_flag_set;
*swig_annotative_style_get = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_style_get;
*swig_annotative_style_set = *LibreDWGc::Dwg_Entity_ATTDEF_annotative_style_set;
*swig_prompt_get = *LibreDWGc::Dwg_Entity_ATTDEF_prompt_get;
*swig_prompt_set = *LibreDWGc::Dwg_Entity_ATTDEF_prompt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ATTDEF(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ATTDEF($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_BLOCK ##############

package LibreDWG::Dwg_Entity_BLOCK;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_BLOCK_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_BLOCK_parent_set;
*swig_name_get = *LibreDWGc::Dwg_Entity_BLOCK_name_get;
*swig_name_set = *LibreDWGc::Dwg_Entity_BLOCK_name_set;
*swig_xref_pname_get = *LibreDWGc::Dwg_Entity_BLOCK_xref_pname_get;
*swig_xref_pname_set = *LibreDWGc::Dwg_Entity_BLOCK_xref_pname_set;
*swig_base_pt_get = *LibreDWGc::Dwg_Entity_BLOCK_base_pt_get;
*swig_base_pt_set = *LibreDWGc::Dwg_Entity_BLOCK_base_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_BLOCK(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_BLOCK($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ENDBLK ##############

package LibreDWG::Dwg_Entity_ENDBLK;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ENDBLK_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ENDBLK_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ENDBLK(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ENDBLK($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_SEQEND ##############

package LibreDWG::Dwg_Entity_SEQEND;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_SEQEND_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_SEQEND_parent_set;
*swig_begin_addr_r11_get = *LibreDWGc::Dwg_Entity_SEQEND_begin_addr_r11_get;
*swig_begin_addr_r11_set = *LibreDWGc::Dwg_Entity_SEQEND_begin_addr_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_SEQEND(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_SEQEND($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_INSERT ##############

package LibreDWG::Dwg_Entity_INSERT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_INSERT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_INSERT_parent_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_INSERT_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_INSERT_ins_pt_set;
*swig_scale_flag_get = *LibreDWGc::Dwg_Entity_INSERT_scale_flag_get;
*swig_scale_flag_set = *LibreDWGc::Dwg_Entity_INSERT_scale_flag_set;
*swig_scale_get = *LibreDWGc::Dwg_Entity_INSERT_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Entity_INSERT_scale_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_INSERT_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_INSERT_rotation_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_INSERT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_INSERT_extrusion_set;
*swig_has_attribs_get = *LibreDWGc::Dwg_Entity_INSERT_has_attribs_get;
*swig_has_attribs_set = *LibreDWGc::Dwg_Entity_INSERT_has_attribs_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Entity_INSERT_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Entity_INSERT_num_owned_set;
*swig_block_header_get = *LibreDWGc::Dwg_Entity_INSERT_block_header_get;
*swig_block_header_set = *LibreDWGc::Dwg_Entity_INSERT_block_header_set;
*swig_first_attrib_get = *LibreDWGc::Dwg_Entity_INSERT_first_attrib_get;
*swig_first_attrib_set = *LibreDWGc::Dwg_Entity_INSERT_first_attrib_set;
*swig_last_attrib_get = *LibreDWGc::Dwg_Entity_INSERT_last_attrib_get;
*swig_last_attrib_set = *LibreDWGc::Dwg_Entity_INSERT_last_attrib_set;
*swig_attribs_get = *LibreDWGc::Dwg_Entity_INSERT_attribs_get;
*swig_attribs_set = *LibreDWGc::Dwg_Entity_INSERT_attribs_set;
*swig_seqend_get = *LibreDWGc::Dwg_Entity_INSERT_seqend_get;
*swig_seqend_set = *LibreDWGc::Dwg_Entity_INSERT_seqend_set;
*swig_num_cols_get = *LibreDWGc::Dwg_Entity_INSERT_num_cols_get;
*swig_num_cols_set = *LibreDWGc::Dwg_Entity_INSERT_num_cols_set;
*swig_num_rows_get = *LibreDWGc::Dwg_Entity_INSERT_num_rows_get;
*swig_num_rows_set = *LibreDWGc::Dwg_Entity_INSERT_num_rows_set;
*swig_col_spacing_get = *LibreDWGc::Dwg_Entity_INSERT_col_spacing_get;
*swig_col_spacing_set = *LibreDWGc::Dwg_Entity_INSERT_col_spacing_set;
*swig_row_spacing_get = *LibreDWGc::Dwg_Entity_INSERT_row_spacing_get;
*swig_row_spacing_set = *LibreDWGc::Dwg_Entity_INSERT_row_spacing_set;
*swig_block_name_get = *LibreDWGc::Dwg_Entity_INSERT_block_name_get;
*swig_block_name_set = *LibreDWGc::Dwg_Entity_INSERT_block_name_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_INSERT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_INSERT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_MINSERT ##############

package LibreDWG::Dwg_Entity_MINSERT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_MINSERT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_MINSERT_parent_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_MINSERT_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_MINSERT_ins_pt_set;
*swig_scale_flag_get = *LibreDWGc::Dwg_Entity_MINSERT_scale_flag_get;
*swig_scale_flag_set = *LibreDWGc::Dwg_Entity_MINSERT_scale_flag_set;
*swig_scale_get = *LibreDWGc::Dwg_Entity_MINSERT_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Entity_MINSERT_scale_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_MINSERT_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_MINSERT_rotation_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_MINSERT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_MINSERT_extrusion_set;
*swig_has_attribs_get = *LibreDWGc::Dwg_Entity_MINSERT_has_attribs_get;
*swig_has_attribs_set = *LibreDWGc::Dwg_Entity_MINSERT_has_attribs_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Entity_MINSERT_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Entity_MINSERT_num_owned_set;
*swig_num_cols_get = *LibreDWGc::Dwg_Entity_MINSERT_num_cols_get;
*swig_num_cols_set = *LibreDWGc::Dwg_Entity_MINSERT_num_cols_set;
*swig_num_rows_get = *LibreDWGc::Dwg_Entity_MINSERT_num_rows_get;
*swig_num_rows_set = *LibreDWGc::Dwg_Entity_MINSERT_num_rows_set;
*swig_col_spacing_get = *LibreDWGc::Dwg_Entity_MINSERT_col_spacing_get;
*swig_col_spacing_set = *LibreDWGc::Dwg_Entity_MINSERT_col_spacing_set;
*swig_row_spacing_get = *LibreDWGc::Dwg_Entity_MINSERT_row_spacing_get;
*swig_row_spacing_set = *LibreDWGc::Dwg_Entity_MINSERT_row_spacing_set;
*swig_block_header_get = *LibreDWGc::Dwg_Entity_MINSERT_block_header_get;
*swig_block_header_set = *LibreDWGc::Dwg_Entity_MINSERT_block_header_set;
*swig_first_attrib_get = *LibreDWGc::Dwg_Entity_MINSERT_first_attrib_get;
*swig_first_attrib_set = *LibreDWGc::Dwg_Entity_MINSERT_first_attrib_set;
*swig_last_attrib_get = *LibreDWGc::Dwg_Entity_MINSERT_last_attrib_get;
*swig_last_attrib_set = *LibreDWGc::Dwg_Entity_MINSERT_last_attrib_set;
*swig_attribs_get = *LibreDWGc::Dwg_Entity_MINSERT_attribs_get;
*swig_attribs_set = *LibreDWGc::Dwg_Entity_MINSERT_attribs_set;
*swig_seqend_get = *LibreDWGc::Dwg_Entity_MINSERT_seqend_get;
*swig_seqend_set = *LibreDWGc::Dwg_Entity_MINSERT_seqend_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_MINSERT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_MINSERT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_VERTEX_2D ##############

package LibreDWG::Dwg_Entity_VERTEX_2D;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_flag_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_point_set;
*swig_start_width_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_start_width_get;
*swig_start_width_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_start_width_set;
*swig_end_width_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_end_width_get;
*swig_end_width_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_end_width_set;
*swig_id_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_id_get;
*swig_id_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_id_set;
*swig_bulge_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_bulge_get;
*swig_bulge_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_bulge_set;
*swig_tangent_dir_get = *LibreDWGc::Dwg_Entity_VERTEX_2D_tangent_dir_get;
*swig_tangent_dir_set = *LibreDWGc::Dwg_Entity_VERTEX_2D_tangent_dir_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_VERTEX_2D(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_VERTEX_2D($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_VERTEX_3D ##############

package LibreDWG::Dwg_Entity_VERTEX_3D;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_VERTEX_3D_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_VERTEX_3D_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_VERTEX_3D_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_VERTEX_3D_flag_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_VERTEX_3D_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_VERTEX_3D_point_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_VERTEX_3D(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_VERTEX_3D($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_VERTEX_PFACE_FACE ##############

package LibreDWG::Dwg_Entity_VERTEX_PFACE_FACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_flag_set;
*swig_vertind_get = *LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_vertind_get;
*swig_vertind_set = *LibreDWGc::Dwg_Entity_VERTEX_PFACE_FACE_vertind_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_VERTEX_PFACE_FACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_VERTEX_PFACE_FACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POLYLINE_2D ##############

package LibreDWG::Dwg_Entity_POLYLINE_2D;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_parent_set;
*swig_has_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_has_vertex_get;
*swig_has_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_has_vertex_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_num_owned_set;
*swig_first_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_first_vertex_get;
*swig_first_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_first_vertex_set;
*swig_last_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_last_vertex_get;
*swig_last_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_last_vertex_set;
*swig_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_vertex_get;
*swig_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_vertex_set;
*swig_seqend_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_seqend_get;
*swig_seqend_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_seqend_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_flag_set;
*swig_curve_type_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_curve_type_get;
*swig_curve_type_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_curve_type_set;
*swig_start_width_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_start_width_get;
*swig_start_width_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_start_width_set;
*swig_end_width_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_end_width_get;
*swig_end_width_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_end_width_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_thickness_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_elevation_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_extrusion_set;
*swig_extra_r11_size_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_size_get;
*swig_extra_r11_size_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_size_set;
*swig_extra_r11_text_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_text_get;
*swig_extra_r11_text_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_extra_r11_text_set;
*swig_num_m_verts_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_num_m_verts_get;
*swig_num_m_verts_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_num_m_verts_set;
*swig_num_n_verts_get = *LibreDWGc::Dwg_Entity_POLYLINE_2D_num_n_verts_get;
*swig_num_n_verts_set = *LibreDWGc::Dwg_Entity_POLYLINE_2D_num_n_verts_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POLYLINE_2D(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POLYLINE_2D($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POLYLINE_3D ##############

package LibreDWG::Dwg_Entity_POLYLINE_3D;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_parent_set;
*swig_has_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_has_vertex_get;
*swig_has_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_has_vertex_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_num_owned_set;
*swig_first_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_first_vertex_get;
*swig_first_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_first_vertex_set;
*swig_last_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_last_vertex_get;
*swig_last_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_last_vertex_set;
*swig_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_vertex_get;
*swig_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_vertex_set;
*swig_seqend_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_seqend_get;
*swig_seqend_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_seqend_set;
*swig_curve_type_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_curve_type_get;
*swig_curve_type_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_curve_type_set;
*swig_start_width_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_start_width_get;
*swig_start_width_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_start_width_set;
*swig_end_width_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_end_width_get;
*swig_end_width_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_end_width_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_flag_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_POLYLINE_3D_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_POLYLINE_3D_extrusion_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POLYLINE_3D(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POLYLINE_3D($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ARC ##############

package LibreDWG::Dwg_Entity_ARC;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ARC_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ARC_parent_set;
*swig_center_get = *LibreDWGc::Dwg_Entity_ARC_center_get;
*swig_center_set = *LibreDWGc::Dwg_Entity_ARC_center_set;
*swig_radius_get = *LibreDWGc::Dwg_Entity_ARC_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_Entity_ARC_radius_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_ARC_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_ARC_thickness_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_ARC_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_ARC_extrusion_set;
*swig_start_angle_get = *LibreDWGc::Dwg_Entity_ARC_start_angle_get;
*swig_start_angle_set = *LibreDWGc::Dwg_Entity_ARC_start_angle_set;
*swig_end_angle_get = *LibreDWGc::Dwg_Entity_ARC_end_angle_get;
*swig_end_angle_set = *LibreDWGc::Dwg_Entity_ARC_end_angle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ARC(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ARC($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_CIRCLE ##############

package LibreDWG::Dwg_Entity_CIRCLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_CIRCLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_CIRCLE_parent_set;
*swig_center_get = *LibreDWGc::Dwg_Entity_CIRCLE_center_get;
*swig_center_set = *LibreDWGc::Dwg_Entity_CIRCLE_center_set;
*swig_radius_get = *LibreDWGc::Dwg_Entity_CIRCLE_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_Entity_CIRCLE_radius_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_CIRCLE_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_CIRCLE_thickness_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_CIRCLE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_CIRCLE_extrusion_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_CIRCLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_CIRCLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LINE ##############

package LibreDWG::Dwg_Entity_LINE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LINE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LINE_parent_set;
*swig_z_is_zero_get = *LibreDWGc::Dwg_Entity_LINE_z_is_zero_get;
*swig_z_is_zero_set = *LibreDWGc::Dwg_Entity_LINE_z_is_zero_set;
*swig_start_get = *LibreDWGc::Dwg_Entity_LINE_start_get;
*swig_start_set = *LibreDWGc::Dwg_Entity_LINE_start_set;
*swig_end_get = *LibreDWGc::Dwg_Entity_LINE_end_get;
*swig_end_set = *LibreDWGc::Dwg_Entity_LINE_end_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_LINE_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_LINE_thickness_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_LINE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_LINE_extrusion_set;
*swig_unknown_r11_get = *LibreDWGc::Dwg_Entity_LINE_unknown_r11_get;
*swig_unknown_r11_set = *LibreDWGc::Dwg_Entity_LINE_unknown_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LINE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LINE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_DIMENSION_common ##############

package LibreDWG::Dwg_DIMENSION_common;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_DIMENSION_common_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_DIMENSION_common_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_DIMENSION_common_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_DIMENSION_common_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_DIMENSION_common_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_DIMENSION_common_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_DIMENSION_common_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_DIMENSION_common_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_DIMENSION_common_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_DIMENSION_common_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_DIMENSION_common_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_DIMENSION_common_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_DIMENSION_common_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_DIMENSION_common_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_DIMENSION_common_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_DIMENSION_common_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_DIMENSION_common_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_DIMENSION_common_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_DIMENSION_common_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_DIMENSION_common_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_DIMENSION_common_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_DIMENSION_common_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_DIMENSION_common_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_DIMENSION_common_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_DIMENSION_common_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_DIMENSION_common_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_DIMENSION_common_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_DIMENSION_common_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_DIMENSION_common_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_DIMENSION_common_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_DIMENSION_common_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_DIMENSION_common_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_DIMENSION_common_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_DIMENSION_common_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_DIMENSION_common_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_DIMENSION_common_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_DIMENSION_common_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_DIMENSION_common_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_DIMENSION_common_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_DIMENSION_common_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_DIMENSION_common_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_DIMENSION_common_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_DIMENSION_common_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_DIMENSION_common_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_DIMENSION_common_block_get;
*swig_block_set = *LibreDWGc::Dwg_DIMENSION_common_block_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_DIMENSION_common(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_DIMENSION_common($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_DIMENSION_ORDINATE ##############

package LibreDWG::Dwg_Entity_DIMENSION_ORDINATE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_block_set;
*swig_feature_location_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_get;
*swig_feature_location_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_feature_location_pt_set;
*swig_leader_endpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_get;
*swig_leader_endpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_leader_endpt_set;
*swig_flag2_get = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag2_get;
*swig_flag2_set = *LibreDWGc::Dwg_Entity_DIMENSION_ORDINATE_flag2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_DIMENSION_ORDINATE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_DIMENSION_ORDINATE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_DIMENSION_LINEAR ##############

package LibreDWG::Dwg_Entity_DIMENSION_LINEAR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_block_set;
*swig_xline1_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline1_pt_get;
*swig_xline1_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline1_pt_set;
*swig_xline2_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline2_pt_get;
*swig_xline2_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_xline2_pt_set;
*swig_oblique_angle_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_oblique_angle_get;
*swig_oblique_angle_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_oblique_angle_set;
*swig_dim_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dim_rotation_get;
*swig_dim_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_LINEAR_dim_rotation_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_DIMENSION_LINEAR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_DIMENSION_LINEAR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_DIMENSION_ALIGNED ##############

package LibreDWG::Dwg_Entity_DIMENSION_ALIGNED;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_block_set;
*swig_xline1_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_get;
*swig_xline1_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline1_pt_set;
*swig_xline2_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_get;
*swig_xline2_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_xline2_pt_set;
*swig_oblique_angle_get = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_get;
*swig_oblique_angle_set = *LibreDWGc::Dwg_Entity_DIMENSION_ALIGNED_oblique_angle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_DIMENSION_ALIGNED(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_DIMENSION_ALIGNED($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_DIMENSION_ANG3PT ##############

package LibreDWG::Dwg_Entity_DIMENSION_ANG3PT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_block_set;
*swig_xline1_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_get;
*swig_xline1_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline1_pt_set;
*swig_xline2_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_get;
*swig_xline2_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2_pt_set;
*swig_center_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_center_pt_get;
*swig_center_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_center_pt_set;
*swig_xline2end_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_get;
*swig_xline2end_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG3PT_xline2end_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_DIMENSION_ANG3PT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_DIMENSION_ANG3PT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_DIMENSION_ANG2LN ##############

package LibreDWG::Dwg_Entity_DIMENSION_ANG2LN;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_block_set;
*swig_xline1start_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_get;
*swig_xline1start_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1start_pt_set;
*swig_xline1end_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_get;
*swig_xline1end_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline1end_pt_set;
*swig_xline2start_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_get;
*swig_xline2start_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2start_pt_set;
*swig_xline2end_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_get;
*swig_xline2end_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_ANG2LN_xline2end_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_DIMENSION_ANG2LN(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_DIMENSION_ANG2LN($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_DIMENSION_RADIUS ##############

package LibreDWG::Dwg_Entity_DIMENSION_RADIUS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_block_set;
*swig_first_arc_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_get;
*swig_first_arc_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_first_arc_pt_set;
*swig_leader_len_get = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_leader_len_get;
*swig_leader_len_set = *LibreDWGc::Dwg_Entity_DIMENSION_RADIUS_leader_len_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_DIMENSION_RADIUS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_DIMENSION_RADIUS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_DIMENSION_DIAMETER ##############

package LibreDWG::Dwg_Entity_DIMENSION_DIAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_block_set;
*swig_first_arc_pt_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_get;
*swig_first_arc_pt_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_first_arc_pt_set;
*swig_leader_len_get = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_leader_len_get;
*swig_leader_len_set = *LibreDWGc::Dwg_Entity_DIMENSION_DIAMETER_leader_len_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_DIMENSION_DIAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_DIMENSION_DIAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ARC_DIMENSION ##############

package LibreDWG::Dwg_Entity_ARC_DIMENSION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_block_set;
*swig_xline1_pt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline1_pt_get;
*swig_xline1_pt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline1_pt_set;
*swig_xline2_pt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline2_pt_get;
*swig_xline2_pt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_xline2_pt_set;
*swig_center_pt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_center_pt_get;
*swig_center_pt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_center_pt_set;
*swig_is_partial_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_is_partial_get;
*swig_is_partial_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_is_partial_set;
*swig_arc_start_param_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_start_param_get;
*swig_arc_start_param_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_start_param_set;
*swig_arc_end_param_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_end_param_get;
*swig_arc_end_param_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_arc_end_param_set;
*swig_has_leader_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_has_leader_get;
*swig_has_leader_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_has_leader_set;
*swig_leader1_pt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader1_pt_get;
*swig_leader1_pt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader1_pt_set;
*swig_leader2_pt_get = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader2_pt_get;
*swig_leader2_pt_set = *LibreDWGc::Dwg_Entity_ARC_DIMENSION_leader2_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ARC_DIMENSION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ARC_DIMENSION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LARGE_RADIAL_DIMENSION ##############

package LibreDWG::Dwg_Entity_LARGE_RADIAL_DIMENSION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_class_version_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_extrusion_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_def_pt_set;
*swig_text_midpt_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_get;
*swig_text_midpt_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_midpt_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_elevation_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag_set;
*swig_flag1_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_get;
*swig_flag1_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flag1_set;
*swig_user_text_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_get;
*swig_user_text_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_user_text_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_text_rotation_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_horiz_dir_set;
*swig_ins_scale_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_get;
*swig_ins_scale_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_scale_set;
*swig_ins_rotation_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_get;
*swig_ins_rotation_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ins_rotation_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_attachment_set;
*swig_lspace_style_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_get;
*swig_lspace_style_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_style_set;
*swig_lspace_factor_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_get;
*swig_lspace_factor_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_lspace_factor_set;
*swig_act_measurement_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_get;
*swig_act_measurement_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_act_measurement_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_unknown_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow1_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_flip_arrow2_set;
*swig_clone_ins_pt_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_get;
*swig_clone_ins_pt_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_clone_ins_pt_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_dimstyle_set;
*swig_block_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_block_get;
*swig_block_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_block_set;
*swig_jog_angle_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_get;
*swig_jog_angle_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_angle_set;
*swig_jog_pt_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_get;
*swig_jog_pt_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_jog_pt_set;
*swig_ovr_center_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_get;
*swig_ovr_center_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_ovr_center_set;
*swig_chord_pt_get = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_get;
*swig_chord_pt_set = *LibreDWGc::Dwg_Entity_LARGE_RADIAL_DIMENSION_chord_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LARGE_RADIAL_DIMENSION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LARGE_RADIAL_DIMENSION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POINT ##############

package LibreDWG::Dwg_Entity_POINT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POINT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POINT_parent_set;
*swig_x_get = *LibreDWGc::Dwg_Entity_POINT_x_get;
*swig_x_set = *LibreDWGc::Dwg_Entity_POINT_x_set;
*swig_y_get = *LibreDWGc::Dwg_Entity_POINT_y_get;
*swig_y_set = *LibreDWGc::Dwg_Entity_POINT_y_set;
*swig_z_get = *LibreDWGc::Dwg_Entity_POINT_z_get;
*swig_z_set = *LibreDWGc::Dwg_Entity_POINT_z_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_POINT_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_POINT_thickness_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_POINT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_POINT_extrusion_set;
*swig_x_ang_get = *LibreDWGc::Dwg_Entity_POINT_x_ang_get;
*swig_x_ang_set = *LibreDWGc::Dwg_Entity_POINT_x_ang_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POINT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POINT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity__3DFACE ##############

package LibreDWG::Dwg_Entity__3DFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity__3DFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity__3DFACE_parent_set;
*swig_has_no_flags_get = *LibreDWGc::Dwg_Entity__3DFACE_has_no_flags_get;
*swig_has_no_flags_set = *LibreDWGc::Dwg_Entity__3DFACE_has_no_flags_set;
*swig_z_is_zero_get = *LibreDWGc::Dwg_Entity__3DFACE_z_is_zero_get;
*swig_z_is_zero_set = *LibreDWGc::Dwg_Entity__3DFACE_z_is_zero_set;
*swig_corner1_get = *LibreDWGc::Dwg_Entity__3DFACE_corner1_get;
*swig_corner1_set = *LibreDWGc::Dwg_Entity__3DFACE_corner1_set;
*swig_corner2_get = *LibreDWGc::Dwg_Entity__3DFACE_corner2_get;
*swig_corner2_set = *LibreDWGc::Dwg_Entity__3DFACE_corner2_set;
*swig_corner3_get = *LibreDWGc::Dwg_Entity__3DFACE_corner3_get;
*swig_corner3_set = *LibreDWGc::Dwg_Entity__3DFACE_corner3_set;
*swig_corner4_get = *LibreDWGc::Dwg_Entity__3DFACE_corner4_get;
*swig_corner4_set = *LibreDWGc::Dwg_Entity__3DFACE_corner4_set;
*swig_invis_flags_get = *LibreDWGc::Dwg_Entity__3DFACE_invis_flags_get;
*swig_invis_flags_set = *LibreDWGc::Dwg_Entity__3DFACE_invis_flags_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity__3DFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity__3DFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POLYLINE_PFACE ##############

package LibreDWG::Dwg_Entity_POLYLINE_PFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_parent_set;
*swig_has_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_has_vertex_get;
*swig_has_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_has_vertex_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_num_owned_set;
*swig_first_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_first_vertex_get;
*swig_first_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_first_vertex_set;
*swig_last_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_last_vertex_get;
*swig_last_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_last_vertex_set;
*swig_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_vertex_get;
*swig_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_vertex_set;
*swig_seqend_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_seqend_get;
*swig_seqend_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_seqend_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_flag_set;
*swig_numverts_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numverts_get;
*swig_numverts_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numverts_set;
*swig_numfaces_get = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numfaces_get;
*swig_numfaces_set = *LibreDWGc::Dwg_Entity_POLYLINE_PFACE_numfaces_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POLYLINE_PFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POLYLINE_PFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POLYLINE_MESH ##############

package LibreDWG::Dwg_Entity_POLYLINE_MESH;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_parent_set;
*swig_has_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_has_vertex_get;
*swig_has_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_has_vertex_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_owned_set;
*swig_first_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_first_vertex_get;
*swig_first_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_first_vertex_set;
*swig_last_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_last_vertex_get;
*swig_last_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_last_vertex_set;
*swig_vertex_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_vertex_get;
*swig_vertex_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_vertex_set;
*swig_seqend_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_seqend_get;
*swig_seqend_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_seqend_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_flag_set;
*swig_curve_type_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_curve_type_get;
*swig_curve_type_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_curve_type_set;
*swig_num_m_verts_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_m_verts_get;
*swig_num_m_verts_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_m_verts_set;
*swig_num_n_verts_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_n_verts_get;
*swig_num_n_verts_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_num_n_verts_set;
*swig_m_density_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_m_density_get;
*swig_m_density_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_m_density_set;
*swig_n_density_get = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_n_density_get;
*swig_n_density_set = *LibreDWGc::Dwg_Entity_POLYLINE_MESH_n_density_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POLYLINE_MESH(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POLYLINE_MESH($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_SOLID ##############

package LibreDWG::Dwg_Entity_SOLID;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_SOLID_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_SOLID_parent_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_SOLID_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_SOLID_thickness_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_SOLID_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_SOLID_elevation_set;
*swig_corner1_get = *LibreDWGc::Dwg_Entity_SOLID_corner1_get;
*swig_corner1_set = *LibreDWGc::Dwg_Entity_SOLID_corner1_set;
*swig_corner2_get = *LibreDWGc::Dwg_Entity_SOLID_corner2_get;
*swig_corner2_set = *LibreDWGc::Dwg_Entity_SOLID_corner2_set;
*swig_corner3_get = *LibreDWGc::Dwg_Entity_SOLID_corner3_get;
*swig_corner3_set = *LibreDWGc::Dwg_Entity_SOLID_corner3_set;
*swig_corner4_get = *LibreDWGc::Dwg_Entity_SOLID_corner4_get;
*swig_corner4_set = *LibreDWGc::Dwg_Entity_SOLID_corner4_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_SOLID_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_SOLID_extrusion_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_SOLID(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_SOLID($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_TRACE ##############

package LibreDWG::Dwg_Entity_TRACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_TRACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_TRACE_parent_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_TRACE_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_TRACE_thickness_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_TRACE_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_TRACE_elevation_set;
*swig_corner1_get = *LibreDWGc::Dwg_Entity_TRACE_corner1_get;
*swig_corner1_set = *LibreDWGc::Dwg_Entity_TRACE_corner1_set;
*swig_corner2_get = *LibreDWGc::Dwg_Entity_TRACE_corner2_get;
*swig_corner2_set = *LibreDWGc::Dwg_Entity_TRACE_corner2_set;
*swig_corner3_get = *LibreDWGc::Dwg_Entity_TRACE_corner3_get;
*swig_corner3_set = *LibreDWGc::Dwg_Entity_TRACE_corner3_set;
*swig_corner4_get = *LibreDWGc::Dwg_Entity_TRACE_corner4_get;
*swig_corner4_set = *LibreDWGc::Dwg_Entity_TRACE_corner4_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_TRACE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_TRACE_extrusion_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_TRACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_TRACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_SHAPE ##############

package LibreDWG::Dwg_Entity_SHAPE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_SHAPE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_SHAPE_parent_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_SHAPE_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_SHAPE_ins_pt_set;
*swig_scale_get = *LibreDWGc::Dwg_Entity_SHAPE_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Entity_SHAPE_scale_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_SHAPE_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_SHAPE_rotation_set;
*swig_width_factor_get = *LibreDWGc::Dwg_Entity_SHAPE_width_factor_get;
*swig_width_factor_set = *LibreDWGc::Dwg_Entity_SHAPE_width_factor_set;
*swig_oblique_angle_get = *LibreDWGc::Dwg_Entity_SHAPE_oblique_angle_get;
*swig_oblique_angle_set = *LibreDWGc::Dwg_Entity_SHAPE_oblique_angle_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_SHAPE_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_SHAPE_thickness_set;
*swig_style_id_get = *LibreDWGc::Dwg_Entity_SHAPE_style_id_get;
*swig_style_id_set = *LibreDWGc::Dwg_Entity_SHAPE_style_id_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_SHAPE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_SHAPE_extrusion_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_SHAPE_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_SHAPE_style_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_SHAPE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_SHAPE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_VIEWPORT ##############

package LibreDWG::Dwg_Entity_VIEWPORT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_VIEWPORT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_VIEWPORT_parent_set;
*swig_center_get = *LibreDWGc::Dwg_Entity_VIEWPORT_center_get;
*swig_center_set = *LibreDWGc::Dwg_Entity_VIEWPORT_center_set;
*swig_width_get = *LibreDWGc::Dwg_Entity_VIEWPORT_width_get;
*swig_width_set = *LibreDWGc::Dwg_Entity_VIEWPORT_width_set;
*swig_height_get = *LibreDWGc::Dwg_Entity_VIEWPORT_height_get;
*swig_height_set = *LibreDWGc::Dwg_Entity_VIEWPORT_height_set;
*swig_on_off_get = *LibreDWGc::Dwg_Entity_VIEWPORT_on_off_get;
*swig_on_off_set = *LibreDWGc::Dwg_Entity_VIEWPORT_on_off_set;
*swig_id_get = *LibreDWGc::Dwg_Entity_VIEWPORT_id_get;
*swig_id_set = *LibreDWGc::Dwg_Entity_VIEWPORT_id_set;
*swig_view_target_get = *LibreDWGc::Dwg_Entity_VIEWPORT_view_target_get;
*swig_view_target_set = *LibreDWGc::Dwg_Entity_VIEWPORT_view_target_set;
*swig_VIEWDIR_get = *LibreDWGc::Dwg_Entity_VIEWPORT_VIEWDIR_get;
*swig_VIEWDIR_set = *LibreDWGc::Dwg_Entity_VIEWPORT_VIEWDIR_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Entity_VIEWPORT_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Entity_VIEWPORT_twist_angle_set;
*swig_VIEWSIZE_get = *LibreDWGc::Dwg_Entity_VIEWPORT_VIEWSIZE_get;
*swig_VIEWSIZE_set = *LibreDWGc::Dwg_Entity_VIEWPORT_VIEWSIZE_set;
*swig_lens_length_get = *LibreDWGc::Dwg_Entity_VIEWPORT_lens_length_get;
*swig_lens_length_set = *LibreDWGc::Dwg_Entity_VIEWPORT_lens_length_set;
*swig_front_clip_z_get = *LibreDWGc::Dwg_Entity_VIEWPORT_front_clip_z_get;
*swig_front_clip_z_set = *LibreDWGc::Dwg_Entity_VIEWPORT_front_clip_z_set;
*swig_back_clip_z_get = *LibreDWGc::Dwg_Entity_VIEWPORT_back_clip_z_get;
*swig_back_clip_z_set = *LibreDWGc::Dwg_Entity_VIEWPORT_back_clip_z_set;
*swig_SNAPANG_get = *LibreDWGc::Dwg_Entity_VIEWPORT_SNAPANG_get;
*swig_SNAPANG_set = *LibreDWGc::Dwg_Entity_VIEWPORT_SNAPANG_set;
*swig_VIEWCTR_get = *LibreDWGc::Dwg_Entity_VIEWPORT_VIEWCTR_get;
*swig_VIEWCTR_set = *LibreDWGc::Dwg_Entity_VIEWPORT_VIEWCTR_set;
*swig_SNAPBASE_get = *LibreDWGc::Dwg_Entity_VIEWPORT_SNAPBASE_get;
*swig_SNAPBASE_set = *LibreDWGc::Dwg_Entity_VIEWPORT_SNAPBASE_set;
*swig_SNAPUNIT_get = *LibreDWGc::Dwg_Entity_VIEWPORT_SNAPUNIT_get;
*swig_SNAPUNIT_set = *LibreDWGc::Dwg_Entity_VIEWPORT_SNAPUNIT_set;
*swig_GRIDUNIT_get = *LibreDWGc::Dwg_Entity_VIEWPORT_GRIDUNIT_get;
*swig_GRIDUNIT_set = *LibreDWGc::Dwg_Entity_VIEWPORT_GRIDUNIT_set;
*swig_circle_zoom_get = *LibreDWGc::Dwg_Entity_VIEWPORT_circle_zoom_get;
*swig_circle_zoom_set = *LibreDWGc::Dwg_Entity_VIEWPORT_circle_zoom_set;
*swig_grid_major_get = *LibreDWGc::Dwg_Entity_VIEWPORT_grid_major_get;
*swig_grid_major_set = *LibreDWGc::Dwg_Entity_VIEWPORT_grid_major_set;
*swig_num_frozen_layers_get = *LibreDWGc::Dwg_Entity_VIEWPORT_num_frozen_layers_get;
*swig_num_frozen_layers_set = *LibreDWGc::Dwg_Entity_VIEWPORT_num_frozen_layers_set;
*swig_status_flag_get = *LibreDWGc::Dwg_Entity_VIEWPORT_status_flag_get;
*swig_status_flag_set = *LibreDWGc::Dwg_Entity_VIEWPORT_status_flag_set;
*swig_style_sheet_get = *LibreDWGc::Dwg_Entity_VIEWPORT_style_sheet_get;
*swig_style_sheet_set = *LibreDWGc::Dwg_Entity_VIEWPORT_style_sheet_set;
*swig_render_mode_get = *LibreDWGc::Dwg_Entity_VIEWPORT_render_mode_get;
*swig_render_mode_set = *LibreDWGc::Dwg_Entity_VIEWPORT_render_mode_set;
*swig_ucs_at_origin_get = *LibreDWGc::Dwg_Entity_VIEWPORT_ucs_at_origin_get;
*swig_ucs_at_origin_set = *LibreDWGc::Dwg_Entity_VIEWPORT_ucs_at_origin_set;
*swig_UCSVP_get = *LibreDWGc::Dwg_Entity_VIEWPORT_UCSVP_get;
*swig_UCSVP_set = *LibreDWGc::Dwg_Entity_VIEWPORT_UCSVP_set;
*swig_ucsorg_get = *LibreDWGc::Dwg_Entity_VIEWPORT_ucsorg_get;
*swig_ucsorg_set = *LibreDWGc::Dwg_Entity_VIEWPORT_ucsorg_set;
*swig_ucsxdir_get = *LibreDWGc::Dwg_Entity_VIEWPORT_ucsxdir_get;
*swig_ucsxdir_set = *LibreDWGc::Dwg_Entity_VIEWPORT_ucsxdir_set;
*swig_ucsydir_get = *LibreDWGc::Dwg_Entity_VIEWPORT_ucsydir_get;
*swig_ucsydir_set = *LibreDWGc::Dwg_Entity_VIEWPORT_ucsydir_set;
*swig_ucs_elevation_get = *LibreDWGc::Dwg_Entity_VIEWPORT_ucs_elevation_get;
*swig_ucs_elevation_set = *LibreDWGc::Dwg_Entity_VIEWPORT_ucs_elevation_set;
*swig_UCSORTHOVIEW_get = *LibreDWGc::Dwg_Entity_VIEWPORT_UCSORTHOVIEW_get;
*swig_UCSORTHOVIEW_set = *LibreDWGc::Dwg_Entity_VIEWPORT_UCSORTHOVIEW_set;
*swig_shadeplot_mode_get = *LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_mode_get;
*swig_shadeplot_mode_set = *LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_mode_set;
*swig_use_default_lights_get = *LibreDWGc::Dwg_Entity_VIEWPORT_use_default_lights_get;
*swig_use_default_lights_set = *LibreDWGc::Dwg_Entity_VIEWPORT_use_default_lights_set;
*swig_default_lighting_type_get = *LibreDWGc::Dwg_Entity_VIEWPORT_default_lighting_type_get;
*swig_default_lighting_type_set = *LibreDWGc::Dwg_Entity_VIEWPORT_default_lighting_type_set;
*swig_brightness_get = *LibreDWGc::Dwg_Entity_VIEWPORT_brightness_get;
*swig_brightness_set = *LibreDWGc::Dwg_Entity_VIEWPORT_brightness_set;
*swig_contrast_get = *LibreDWGc::Dwg_Entity_VIEWPORT_contrast_get;
*swig_contrast_set = *LibreDWGc::Dwg_Entity_VIEWPORT_contrast_set;
*swig_ambient_color_get = *LibreDWGc::Dwg_Entity_VIEWPORT_ambient_color_get;
*swig_ambient_color_set = *LibreDWGc::Dwg_Entity_VIEWPORT_ambient_color_set;
*swig_vport_entity_header_get = *LibreDWGc::Dwg_Entity_VIEWPORT_vport_entity_header_get;
*swig_vport_entity_header_set = *LibreDWGc::Dwg_Entity_VIEWPORT_vport_entity_header_set;
*swig_frozen_layers_get = *LibreDWGc::Dwg_Entity_VIEWPORT_frozen_layers_get;
*swig_frozen_layers_set = *LibreDWGc::Dwg_Entity_VIEWPORT_frozen_layers_set;
*swig_clip_boundary_get = *LibreDWGc::Dwg_Entity_VIEWPORT_clip_boundary_get;
*swig_clip_boundary_set = *LibreDWGc::Dwg_Entity_VIEWPORT_clip_boundary_set;
*swig_named_ucs_get = *LibreDWGc::Dwg_Entity_VIEWPORT_named_ucs_get;
*swig_named_ucs_set = *LibreDWGc::Dwg_Entity_VIEWPORT_named_ucs_set;
*swig_base_ucs_get = *LibreDWGc::Dwg_Entity_VIEWPORT_base_ucs_get;
*swig_base_ucs_set = *LibreDWGc::Dwg_Entity_VIEWPORT_base_ucs_set;
*swig_background_get = *LibreDWGc::Dwg_Entity_VIEWPORT_background_get;
*swig_background_set = *LibreDWGc::Dwg_Entity_VIEWPORT_background_set;
*swig_visualstyle_get = *LibreDWGc::Dwg_Entity_VIEWPORT_visualstyle_get;
*swig_visualstyle_set = *LibreDWGc::Dwg_Entity_VIEWPORT_visualstyle_set;
*swig_shadeplot_get = *LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_get;
*swig_shadeplot_set = *LibreDWGc::Dwg_Entity_VIEWPORT_shadeplot_set;
*swig_sun_get = *LibreDWGc::Dwg_Entity_VIEWPORT_sun_get;
*swig_sun_set = *LibreDWGc::Dwg_Entity_VIEWPORT_sun_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_VIEWPORT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_VIEWPORT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ELLIPSE ##############

package LibreDWG::Dwg_Entity_ELLIPSE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ELLIPSE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ELLIPSE_parent_set;
*swig_center_get = *LibreDWGc::Dwg_Entity_ELLIPSE_center_get;
*swig_center_set = *LibreDWGc::Dwg_Entity_ELLIPSE_center_set;
*swig_sm_axis_get = *LibreDWGc::Dwg_Entity_ELLIPSE_sm_axis_get;
*swig_sm_axis_set = *LibreDWGc::Dwg_Entity_ELLIPSE_sm_axis_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_ELLIPSE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_ELLIPSE_extrusion_set;
*swig_axis_ratio_get = *LibreDWGc::Dwg_Entity_ELLIPSE_axis_ratio_get;
*swig_axis_ratio_set = *LibreDWGc::Dwg_Entity_ELLIPSE_axis_ratio_set;
*swig_start_angle_get = *LibreDWGc::Dwg_Entity_ELLIPSE_start_angle_get;
*swig_start_angle_set = *LibreDWGc::Dwg_Entity_ELLIPSE_start_angle_set;
*swig_end_angle_get = *LibreDWGc::Dwg_Entity_ELLIPSE_end_angle_get;
*swig_end_angle_set = *LibreDWGc::Dwg_Entity_ELLIPSE_end_angle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ELLIPSE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ELLIPSE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SPLINE_control_point ##############

package LibreDWG::Dwg_SPLINE_control_point;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_SPLINE_control_point_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_SPLINE_control_point_parent_set;
*swig_x_get = *LibreDWGc::Dwg_SPLINE_control_point_x_get;
*swig_x_set = *LibreDWGc::Dwg_SPLINE_control_point_x_set;
*swig_y_get = *LibreDWGc::Dwg_SPLINE_control_point_y_get;
*swig_y_set = *LibreDWGc::Dwg_SPLINE_control_point_y_set;
*swig_z_get = *LibreDWGc::Dwg_SPLINE_control_point_z_get;
*swig_z_set = *LibreDWGc::Dwg_SPLINE_control_point_z_set;
*swig_w_get = *LibreDWGc::Dwg_SPLINE_control_point_w_get;
*swig_w_set = *LibreDWGc::Dwg_SPLINE_control_point_w_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SPLINE_control_point(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SPLINE_control_point($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_SPLINE ##############

package LibreDWG::Dwg_Entity_SPLINE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_SPLINE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_SPLINE_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_SPLINE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_SPLINE_flag_set;
*swig_scenario_get = *LibreDWGc::Dwg_Entity_SPLINE_scenario_get;
*swig_scenario_set = *LibreDWGc::Dwg_Entity_SPLINE_scenario_set;
*swig_degree_get = *LibreDWGc::Dwg_Entity_SPLINE_degree_get;
*swig_degree_set = *LibreDWGc::Dwg_Entity_SPLINE_degree_set;
*swig_splineflags_get = *LibreDWGc::Dwg_Entity_SPLINE_splineflags_get;
*swig_splineflags_set = *LibreDWGc::Dwg_Entity_SPLINE_splineflags_set;
*swig_knotparam_get = *LibreDWGc::Dwg_Entity_SPLINE_knotparam_get;
*swig_knotparam_set = *LibreDWGc::Dwg_Entity_SPLINE_knotparam_set;
*swig_fit_tol_get = *LibreDWGc::Dwg_Entity_SPLINE_fit_tol_get;
*swig_fit_tol_set = *LibreDWGc::Dwg_Entity_SPLINE_fit_tol_set;
*swig_beg_tan_vec_get = *LibreDWGc::Dwg_Entity_SPLINE_beg_tan_vec_get;
*swig_beg_tan_vec_set = *LibreDWGc::Dwg_Entity_SPLINE_beg_tan_vec_set;
*swig_end_tan_vec_get = *LibreDWGc::Dwg_Entity_SPLINE_end_tan_vec_get;
*swig_end_tan_vec_set = *LibreDWGc::Dwg_Entity_SPLINE_end_tan_vec_set;
*swig_closed_b_get = *LibreDWGc::Dwg_Entity_SPLINE_closed_b_get;
*swig_closed_b_set = *LibreDWGc::Dwg_Entity_SPLINE_closed_b_set;
*swig_periodic_get = *LibreDWGc::Dwg_Entity_SPLINE_periodic_get;
*swig_periodic_set = *LibreDWGc::Dwg_Entity_SPLINE_periodic_set;
*swig_rational_get = *LibreDWGc::Dwg_Entity_SPLINE_rational_get;
*swig_rational_set = *LibreDWGc::Dwg_Entity_SPLINE_rational_set;
*swig_weighted_get = *LibreDWGc::Dwg_Entity_SPLINE_weighted_get;
*swig_weighted_set = *LibreDWGc::Dwg_Entity_SPLINE_weighted_set;
*swig_knot_tol_get = *LibreDWGc::Dwg_Entity_SPLINE_knot_tol_get;
*swig_knot_tol_set = *LibreDWGc::Dwg_Entity_SPLINE_knot_tol_set;
*swig_ctrl_tol_get = *LibreDWGc::Dwg_Entity_SPLINE_ctrl_tol_get;
*swig_ctrl_tol_set = *LibreDWGc::Dwg_Entity_SPLINE_ctrl_tol_set;
*swig_num_fit_pts_get = *LibreDWGc::Dwg_Entity_SPLINE_num_fit_pts_get;
*swig_num_fit_pts_set = *LibreDWGc::Dwg_Entity_SPLINE_num_fit_pts_set;
*swig_fit_pts_get = *LibreDWGc::Dwg_Entity_SPLINE_fit_pts_get;
*swig_fit_pts_set = *LibreDWGc::Dwg_Entity_SPLINE_fit_pts_set;
*swig_num_knots_get = *LibreDWGc::Dwg_Entity_SPLINE_num_knots_get;
*swig_num_knots_set = *LibreDWGc::Dwg_Entity_SPLINE_num_knots_set;
*swig_knots_get = *LibreDWGc::Dwg_Entity_SPLINE_knots_get;
*swig_knots_set = *LibreDWGc::Dwg_Entity_SPLINE_knots_set;
*swig_num_ctrl_pts_get = *LibreDWGc::Dwg_Entity_SPLINE_num_ctrl_pts_get;
*swig_num_ctrl_pts_set = *LibreDWGc::Dwg_Entity_SPLINE_num_ctrl_pts_set;
*swig_ctrl_pts_get = *LibreDWGc::Dwg_Entity_SPLINE_ctrl_pts_get;
*swig_ctrl_pts_set = *LibreDWGc::Dwg_Entity_SPLINE_ctrl_pts_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_SPLINE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_SPLINE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_3DSOLID_wire ##############

package LibreDWG::Dwg_3DSOLID_wire;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_3DSOLID_wire_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_3DSOLID_wire_parent_set;
*swig_type_get = *LibreDWGc::Dwg_3DSOLID_wire_type_get;
*swig_type_set = *LibreDWGc::Dwg_3DSOLID_wire_type_set;
*swig_selection_marker_get = *LibreDWGc::Dwg_3DSOLID_wire_selection_marker_get;
*swig_selection_marker_set = *LibreDWGc::Dwg_3DSOLID_wire_selection_marker_set;
*swig_color_get = *LibreDWGc::Dwg_3DSOLID_wire_color_get;
*swig_color_set = *LibreDWGc::Dwg_3DSOLID_wire_color_set;
*swig_acis_index_get = *LibreDWGc::Dwg_3DSOLID_wire_acis_index_get;
*swig_acis_index_set = *LibreDWGc::Dwg_3DSOLID_wire_acis_index_set;
*swig_num_points_get = *LibreDWGc::Dwg_3DSOLID_wire_num_points_get;
*swig_num_points_set = *LibreDWGc::Dwg_3DSOLID_wire_num_points_set;
*swig_points_get = *LibreDWGc::Dwg_3DSOLID_wire_points_get;
*swig_points_set = *LibreDWGc::Dwg_3DSOLID_wire_points_set;
*swig_transform_present_get = *LibreDWGc::Dwg_3DSOLID_wire_transform_present_get;
*swig_transform_present_set = *LibreDWGc::Dwg_3DSOLID_wire_transform_present_set;
*swig_axis_x_get = *LibreDWGc::Dwg_3DSOLID_wire_axis_x_get;
*swig_axis_x_set = *LibreDWGc::Dwg_3DSOLID_wire_axis_x_set;
*swig_axis_y_get = *LibreDWGc::Dwg_3DSOLID_wire_axis_y_get;
*swig_axis_y_set = *LibreDWGc::Dwg_3DSOLID_wire_axis_y_set;
*swig_axis_z_get = *LibreDWGc::Dwg_3DSOLID_wire_axis_z_get;
*swig_axis_z_set = *LibreDWGc::Dwg_3DSOLID_wire_axis_z_set;
*swig_translation_get = *LibreDWGc::Dwg_3DSOLID_wire_translation_get;
*swig_translation_set = *LibreDWGc::Dwg_3DSOLID_wire_translation_set;
*swig_scale_get = *LibreDWGc::Dwg_3DSOLID_wire_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_3DSOLID_wire_scale_set;
*swig_has_rotation_get = *LibreDWGc::Dwg_3DSOLID_wire_has_rotation_get;
*swig_has_rotation_set = *LibreDWGc::Dwg_3DSOLID_wire_has_rotation_set;
*swig_has_reflection_get = *LibreDWGc::Dwg_3DSOLID_wire_has_reflection_get;
*swig_has_reflection_set = *LibreDWGc::Dwg_3DSOLID_wire_has_reflection_set;
*swig_has_shear_get = *LibreDWGc::Dwg_3DSOLID_wire_has_shear_get;
*swig_has_shear_set = *LibreDWGc::Dwg_3DSOLID_wire_has_shear_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_3DSOLID_wire(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_3DSOLID_wire($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_3DSOLID_silhouette ##############

package LibreDWG::Dwg_3DSOLID_silhouette;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_3DSOLID_silhouette_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_3DSOLID_silhouette_parent_set;
*swig_vp_id_get = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_id_get;
*swig_vp_id_set = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_id_set;
*swig_vp_target_get = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_target_get;
*swig_vp_target_set = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_target_set;
*swig_vp_dir_from_target_get = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_dir_from_target_get;
*swig_vp_dir_from_target_set = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_dir_from_target_set;
*swig_vp_up_dir_get = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_up_dir_get;
*swig_vp_up_dir_set = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_up_dir_set;
*swig_vp_perspective_get = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_perspective_get;
*swig_vp_perspective_set = *LibreDWGc::Dwg_3DSOLID_silhouette_vp_perspective_set;
*swig_has_wires_get = *LibreDWGc::Dwg_3DSOLID_silhouette_has_wires_get;
*swig_has_wires_set = *LibreDWGc::Dwg_3DSOLID_silhouette_has_wires_set;
*swig_num_wires_get = *LibreDWGc::Dwg_3DSOLID_silhouette_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_3DSOLID_silhouette_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_3DSOLID_silhouette_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_3DSOLID_silhouette_wires_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_3DSOLID_silhouette(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_3DSOLID_silhouette($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_3DSOLID_material ##############

package LibreDWG::Dwg_3DSOLID_material;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_3DSOLID_material_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_3DSOLID_material_parent_set;
*swig_array_index_get = *LibreDWGc::Dwg_3DSOLID_material_array_index_get;
*swig_array_index_set = *LibreDWGc::Dwg_3DSOLID_material_array_index_set;
*swig_mat_absref_get = *LibreDWGc::Dwg_3DSOLID_material_mat_absref_get;
*swig_mat_absref_set = *LibreDWGc::Dwg_3DSOLID_material_mat_absref_set;
*swig_material_handle_get = *LibreDWGc::Dwg_3DSOLID_material_material_handle_get;
*swig_material_handle_set = *LibreDWGc::Dwg_3DSOLID_material_material_handle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_3DSOLID_material(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_3DSOLID_material($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity__3DSOLID ##############

package LibreDWG::Dwg_Entity__3DSOLID;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity__3DSOLID_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity__3DSOLID_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity__3DSOLID_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity__3DSOLID_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Entity__3DSOLID_version_get;
*swig_version_set = *LibreDWGc::Dwg_Entity__3DSOLID_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Entity__3DSOLID_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Entity__3DSOLID_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Entity__3DSOLID_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Entity__3DSOLID_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Entity__3DSOLID_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Entity__3DSOLID_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Entity__3DSOLID_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Entity__3DSOLID_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Entity__3DSOLID_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Entity__3DSOLID_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Entity__3DSOLID_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Entity__3DSOLID_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Entity__3DSOLID_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Entity__3DSOLID_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Entity__3DSOLID_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity__3DSOLID_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Entity__3DSOLID_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Entity__3DSOLID_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Entity__3DSOLID_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Entity__3DSOLID_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Entity__3DSOLID_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Entity__3DSOLID_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Entity__3DSOLID_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Entity__3DSOLID_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Entity__3DSOLID_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Entity__3DSOLID_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Entity__3DSOLID_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Entity__3DSOLID_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Entity__3DSOLID__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Entity__3DSOLID__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Entity__3DSOLID_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Entity__3DSOLID_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Entity__3DSOLID_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Entity__3DSOLID_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Entity__3DSOLID_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Entity__3DSOLID_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Entity__3DSOLID_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Entity__3DSOLID_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Entity__3DSOLID_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Entity__3DSOLID_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Entity__3DSOLID_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Entity__3DSOLID_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Entity__3DSOLID_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Entity__3DSOLID_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Entity__3DSOLID_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Entity__3DSOLID_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Entity__3DSOLID_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Entity__3DSOLID_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Entity__3DSOLID_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Entity__3DSOLID_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Entity__3DSOLID_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Entity__3DSOLID_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Entity__3DSOLID_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Entity__3DSOLID_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Entity__3DSOLID_acis_empty_bit_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity__3DSOLID(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity__3DSOLID($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_RAY ##############

package LibreDWG::Dwg_Entity_RAY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_RAY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_RAY_parent_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_RAY_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_RAY_point_set;
*swig_vector_get = *LibreDWGc::Dwg_Entity_RAY_vector_get;
*swig_vector_set = *LibreDWGc::Dwg_Entity_RAY_vector_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_RAY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_RAY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DICTIONARY ##############

package LibreDWG::Dwg_Object_DICTIONARY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DICTIONARY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DICTIONARY_parent_set;
*swig_numitems_get = *LibreDWGc::Dwg_Object_DICTIONARY_numitems_get;
*swig_numitems_set = *LibreDWGc::Dwg_Object_DICTIONARY_numitems_set;
*swig_is_hardowner_get = *LibreDWGc::Dwg_Object_DICTIONARY_is_hardowner_get;
*swig_is_hardowner_set = *LibreDWGc::Dwg_Object_DICTIONARY_is_hardowner_set;
*swig_cloning_get = *LibreDWGc::Dwg_Object_DICTIONARY_cloning_get;
*swig_cloning_set = *LibreDWGc::Dwg_Object_DICTIONARY_cloning_set;
*swig_texts_get = *LibreDWGc::Dwg_Object_DICTIONARY_texts_get;
*swig_texts_set = *LibreDWGc::Dwg_Object_DICTIONARY_texts_set;
*swig_itemhandles_get = *LibreDWGc::Dwg_Object_DICTIONARY_itemhandles_get;
*swig_itemhandles_set = *LibreDWGc::Dwg_Object_DICTIONARY_itemhandles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DICTIONARY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DICTIONARY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DICTIONARYWDFLT ##############

package LibreDWG::Dwg_Object_DICTIONARYWDFLT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_parent_set;
*swig_numitems_get = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_numitems_get;
*swig_numitems_set = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_numitems_set;
*swig_is_hardowner_get = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_is_hardowner_get;
*swig_is_hardowner_set = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_is_hardowner_set;
*swig_cloning_get = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_cloning_get;
*swig_cloning_set = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_cloning_set;
*swig_texts_get = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_texts_get;
*swig_texts_set = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_texts_set;
*swig_itemhandles_get = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_itemhandles_get;
*swig_itemhandles_set = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_itemhandles_set;
*swig_defaultid_get = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_defaultid_get;
*swig_defaultid_set = *LibreDWGc::Dwg_Object_DICTIONARYWDFLT_defaultid_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DICTIONARYWDFLT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DICTIONARYWDFLT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_OLEFRAME ##############

package LibreDWG::Dwg_Entity_OLEFRAME;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_OLEFRAME_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_OLEFRAME_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_OLEFRAME_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_OLEFRAME_flag_set;
*swig_mode_get = *LibreDWGc::Dwg_Entity_OLEFRAME_mode_get;
*swig_mode_set = *LibreDWGc::Dwg_Entity_OLEFRAME_mode_set;
*swig_data_size_get = *LibreDWGc::Dwg_Entity_OLEFRAME_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_Entity_OLEFRAME_data_size_set;
*swig_data_get = *LibreDWGc::Dwg_Entity_OLEFRAME_data_get;
*swig_data_set = *LibreDWGc::Dwg_Entity_OLEFRAME_data_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_OLEFRAME(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_OLEFRAME($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_MTEXT ##############

package LibreDWG::Dwg_Entity_MTEXT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_MTEXT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_MTEXT_parent_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_MTEXT_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_MTEXT_ins_pt_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_MTEXT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_MTEXT_extrusion_set;
*swig_x_axis_dir_get = *LibreDWGc::Dwg_Entity_MTEXT_x_axis_dir_get;
*swig_x_axis_dir_set = *LibreDWGc::Dwg_Entity_MTEXT_x_axis_dir_set;
*swig_rect_height_get = *LibreDWGc::Dwg_Entity_MTEXT_rect_height_get;
*swig_rect_height_set = *LibreDWGc::Dwg_Entity_MTEXT_rect_height_set;
*swig_rect_width_get = *LibreDWGc::Dwg_Entity_MTEXT_rect_width_get;
*swig_rect_width_set = *LibreDWGc::Dwg_Entity_MTEXT_rect_width_set;
*swig_text_height_get = *LibreDWGc::Dwg_Entity_MTEXT_text_height_get;
*swig_text_height_set = *LibreDWGc::Dwg_Entity_MTEXT_text_height_set;
*swig_attachment_get = *LibreDWGc::Dwg_Entity_MTEXT_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Entity_MTEXT_attachment_set;
*swig_flow_dir_get = *LibreDWGc::Dwg_Entity_MTEXT_flow_dir_get;
*swig_flow_dir_set = *LibreDWGc::Dwg_Entity_MTEXT_flow_dir_set;
*swig_extents_width_get = *LibreDWGc::Dwg_Entity_MTEXT_extents_width_get;
*swig_extents_width_set = *LibreDWGc::Dwg_Entity_MTEXT_extents_width_set;
*swig_extents_height_get = *LibreDWGc::Dwg_Entity_MTEXT_extents_height_get;
*swig_extents_height_set = *LibreDWGc::Dwg_Entity_MTEXT_extents_height_set;
*swig_text_get = *LibreDWGc::Dwg_Entity_MTEXT_text_get;
*swig_text_set = *LibreDWGc::Dwg_Entity_MTEXT_text_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_MTEXT_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_MTEXT_style_set;
*swig_linespace_style_get = *LibreDWGc::Dwg_Entity_MTEXT_linespace_style_get;
*swig_linespace_style_set = *LibreDWGc::Dwg_Entity_MTEXT_linespace_style_set;
*swig_linespace_factor_get = *LibreDWGc::Dwg_Entity_MTEXT_linespace_factor_get;
*swig_linespace_factor_set = *LibreDWGc::Dwg_Entity_MTEXT_linespace_factor_set;
*swig_unknown_b0_get = *LibreDWGc::Dwg_Entity_MTEXT_unknown_b0_get;
*swig_unknown_b0_set = *LibreDWGc::Dwg_Entity_MTEXT_unknown_b0_set;
*swig_bg_fill_flag_get = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_flag_get;
*swig_bg_fill_flag_set = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_flag_set;
*swig_bg_fill_scale_get = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_scale_get;
*swig_bg_fill_scale_set = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_scale_set;
*swig_bg_fill_color_get = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_color_get;
*swig_bg_fill_color_set = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_color_set;
*swig_bg_fill_trans_get = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_trans_get;
*swig_bg_fill_trans_set = *LibreDWGc::Dwg_Entity_MTEXT_bg_fill_trans_set;
*swig_is_not_annotative_get = *LibreDWGc::Dwg_Entity_MTEXT_is_not_annotative_get;
*swig_is_not_annotative_set = *LibreDWGc::Dwg_Entity_MTEXT_is_not_annotative_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_MTEXT_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_MTEXT_class_version_set;
*swig_default_flag_get = *LibreDWGc::Dwg_Entity_MTEXT_default_flag_get;
*swig_default_flag_set = *LibreDWGc::Dwg_Entity_MTEXT_default_flag_set;
*swig_appid_get = *LibreDWGc::Dwg_Entity_MTEXT_appid_get;
*swig_appid_set = *LibreDWGc::Dwg_Entity_MTEXT_appid_set;
*swig_ignore_attachment_get = *LibreDWGc::Dwg_Entity_MTEXT_ignore_attachment_get;
*swig_ignore_attachment_set = *LibreDWGc::Dwg_Entity_MTEXT_ignore_attachment_set;
*swig_column_type_get = *LibreDWGc::Dwg_Entity_MTEXT_column_type_get;
*swig_column_type_set = *LibreDWGc::Dwg_Entity_MTEXT_column_type_set;
*swig_numfragments_get = *LibreDWGc::Dwg_Entity_MTEXT_numfragments_get;
*swig_numfragments_set = *LibreDWGc::Dwg_Entity_MTEXT_numfragments_set;
*swig_column_width_get = *LibreDWGc::Dwg_Entity_MTEXT_column_width_get;
*swig_column_width_set = *LibreDWGc::Dwg_Entity_MTEXT_column_width_set;
*swig_gutter_get = *LibreDWGc::Dwg_Entity_MTEXT_gutter_get;
*swig_gutter_set = *LibreDWGc::Dwg_Entity_MTEXT_gutter_set;
*swig_auto_height_get = *LibreDWGc::Dwg_Entity_MTEXT_auto_height_get;
*swig_auto_height_set = *LibreDWGc::Dwg_Entity_MTEXT_auto_height_set;
*swig_flow_reversed_get = *LibreDWGc::Dwg_Entity_MTEXT_flow_reversed_get;
*swig_flow_reversed_set = *LibreDWGc::Dwg_Entity_MTEXT_flow_reversed_set;
*swig_num_column_heights_get = *LibreDWGc::Dwg_Entity_MTEXT_num_column_heights_get;
*swig_num_column_heights_set = *LibreDWGc::Dwg_Entity_MTEXT_num_column_heights_set;
*swig_column_heights_get = *LibreDWGc::Dwg_Entity_MTEXT_column_heights_get;
*swig_column_heights_set = *LibreDWGc::Dwg_Entity_MTEXT_column_heights_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_MTEXT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_MTEXT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LEADER ##############

package LibreDWG::Dwg_Entity_LEADER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LEADER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LEADER_parent_set;
*swig_unknown_bit_1_get = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_1_get;
*swig_unknown_bit_1_set = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_1_set;
*swig_path_type_get = *LibreDWGc::Dwg_Entity_LEADER_path_type_get;
*swig_path_type_set = *LibreDWGc::Dwg_Entity_LEADER_path_type_set;
*swig_annot_type_get = *LibreDWGc::Dwg_Entity_LEADER_annot_type_get;
*swig_annot_type_set = *LibreDWGc::Dwg_Entity_LEADER_annot_type_set;
*swig_num_points_get = *LibreDWGc::Dwg_Entity_LEADER_num_points_get;
*swig_num_points_set = *LibreDWGc::Dwg_Entity_LEADER_num_points_set;
*swig_points_get = *LibreDWGc::Dwg_Entity_LEADER_points_get;
*swig_points_set = *LibreDWGc::Dwg_Entity_LEADER_points_set;
*swig_origin_get = *LibreDWGc::Dwg_Entity_LEADER_origin_get;
*swig_origin_set = *LibreDWGc::Dwg_Entity_LEADER_origin_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_LEADER_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_LEADER_extrusion_set;
*swig_x_direction_get = *LibreDWGc::Dwg_Entity_LEADER_x_direction_get;
*swig_x_direction_set = *LibreDWGc::Dwg_Entity_LEADER_x_direction_set;
*swig_inspt_offset_get = *LibreDWGc::Dwg_Entity_LEADER_inspt_offset_get;
*swig_inspt_offset_set = *LibreDWGc::Dwg_Entity_LEADER_inspt_offset_set;
*swig_endptproj_get = *LibreDWGc::Dwg_Entity_LEADER_endptproj_get;
*swig_endptproj_set = *LibreDWGc::Dwg_Entity_LEADER_endptproj_set;
*swig_dimgap_get = *LibreDWGc::Dwg_Entity_LEADER_dimgap_get;
*swig_dimgap_set = *LibreDWGc::Dwg_Entity_LEADER_dimgap_set;
*swig_box_height_get = *LibreDWGc::Dwg_Entity_LEADER_box_height_get;
*swig_box_height_set = *LibreDWGc::Dwg_Entity_LEADER_box_height_set;
*swig_box_width_get = *LibreDWGc::Dwg_Entity_LEADER_box_width_get;
*swig_box_width_set = *LibreDWGc::Dwg_Entity_LEADER_box_width_set;
*swig_hookline_dir_get = *LibreDWGc::Dwg_Entity_LEADER_hookline_dir_get;
*swig_hookline_dir_set = *LibreDWGc::Dwg_Entity_LEADER_hookline_dir_set;
*swig_hookline_on_get = *LibreDWGc::Dwg_Entity_LEADER_hookline_on_get;
*swig_hookline_on_set = *LibreDWGc::Dwg_Entity_LEADER_hookline_on_set;
*swig_arrowhead_on_get = *LibreDWGc::Dwg_Entity_LEADER_arrowhead_on_get;
*swig_arrowhead_on_set = *LibreDWGc::Dwg_Entity_LEADER_arrowhead_on_set;
*swig_arrowhead_type_get = *LibreDWGc::Dwg_Entity_LEADER_arrowhead_type_get;
*swig_arrowhead_type_set = *LibreDWGc::Dwg_Entity_LEADER_arrowhead_type_set;
*swig_dimasz_get = *LibreDWGc::Dwg_Entity_LEADER_dimasz_get;
*swig_dimasz_set = *LibreDWGc::Dwg_Entity_LEADER_dimasz_set;
*swig_unknown_bit_2_get = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_2_get;
*swig_unknown_bit_2_set = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_2_set;
*swig_unknown_bit_3_get = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_3_get;
*swig_unknown_bit_3_set = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_3_set;
*swig_unknown_short_1_get = *LibreDWGc::Dwg_Entity_LEADER_unknown_short_1_get;
*swig_unknown_short_1_set = *LibreDWGc::Dwg_Entity_LEADER_unknown_short_1_set;
*swig_byblock_color_get = *LibreDWGc::Dwg_Entity_LEADER_byblock_color_get;
*swig_byblock_color_set = *LibreDWGc::Dwg_Entity_LEADER_byblock_color_set;
*swig_unknown_bit_4_get = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_4_get;
*swig_unknown_bit_4_set = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_4_set;
*swig_unknown_bit_5_get = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_5_get;
*swig_unknown_bit_5_set = *LibreDWGc::Dwg_Entity_LEADER_unknown_bit_5_set;
*swig_associated_annotation_get = *LibreDWGc::Dwg_Entity_LEADER_associated_annotation_get;
*swig_associated_annotation_set = *LibreDWGc::Dwg_Entity_LEADER_associated_annotation_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_LEADER_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_LEADER_dimstyle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LEADER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LEADER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_TOLERANCE ##############

package LibreDWG::Dwg_Entity_TOLERANCE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_TOLERANCE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_TOLERANCE_parent_set;
*swig_unknown_short_get = *LibreDWGc::Dwg_Entity_TOLERANCE_unknown_short_get;
*swig_unknown_short_set = *LibreDWGc::Dwg_Entity_TOLERANCE_unknown_short_set;
*swig_height_get = *LibreDWGc::Dwg_Entity_TOLERANCE_height_get;
*swig_height_set = *LibreDWGc::Dwg_Entity_TOLERANCE_height_set;
*swig_dimgap_get = *LibreDWGc::Dwg_Entity_TOLERANCE_dimgap_get;
*swig_dimgap_set = *LibreDWGc::Dwg_Entity_TOLERANCE_dimgap_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_TOLERANCE_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_TOLERANCE_ins_pt_set;
*swig_x_direction_get = *LibreDWGc::Dwg_Entity_TOLERANCE_x_direction_get;
*swig_x_direction_set = *LibreDWGc::Dwg_Entity_TOLERANCE_x_direction_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_TOLERANCE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_TOLERANCE_extrusion_set;
*swig_text_value_get = *LibreDWGc::Dwg_Entity_TOLERANCE_text_value_get;
*swig_text_value_set = *LibreDWGc::Dwg_Entity_TOLERANCE_text_value_set;
*swig_dimstyle_get = *LibreDWGc::Dwg_Entity_TOLERANCE_dimstyle_get;
*swig_dimstyle_set = *LibreDWGc::Dwg_Entity_TOLERANCE_dimstyle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_TOLERANCE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_TOLERANCE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MLINE_line ##############

package LibreDWG::Dwg_MLINE_line;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_MLINE_line_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_MLINE_line_parent_set;
*swig_num_segparms_get = *LibreDWGc::Dwg_MLINE_line_num_segparms_get;
*swig_num_segparms_set = *LibreDWGc::Dwg_MLINE_line_num_segparms_set;
*swig_segparms_get = *LibreDWGc::Dwg_MLINE_line_segparms_get;
*swig_segparms_set = *LibreDWGc::Dwg_MLINE_line_segparms_set;
*swig_num_areafillparms_get = *LibreDWGc::Dwg_MLINE_line_num_areafillparms_get;
*swig_num_areafillparms_set = *LibreDWGc::Dwg_MLINE_line_num_areafillparms_set;
*swig_areafillparms_get = *LibreDWGc::Dwg_MLINE_line_areafillparms_get;
*swig_areafillparms_set = *LibreDWGc::Dwg_MLINE_line_areafillparms_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MLINE_line(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MLINE_line($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MLINE_vertex ##############

package LibreDWG::Dwg_MLINE_vertex;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_MLINE_vertex_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_MLINE_vertex_parent_set;
*swig_vertex_get = *LibreDWGc::Dwg_MLINE_vertex_vertex_get;
*swig_vertex_set = *LibreDWGc::Dwg_MLINE_vertex_vertex_set;
*swig_vertex_direction_get = *LibreDWGc::Dwg_MLINE_vertex_vertex_direction_get;
*swig_vertex_direction_set = *LibreDWGc::Dwg_MLINE_vertex_vertex_direction_set;
*swig_miter_direction_get = *LibreDWGc::Dwg_MLINE_vertex_miter_direction_get;
*swig_miter_direction_set = *LibreDWGc::Dwg_MLINE_vertex_miter_direction_set;
*swig_num_lines_get = *LibreDWGc::Dwg_MLINE_vertex_num_lines_get;
*swig_num_lines_set = *LibreDWGc::Dwg_MLINE_vertex_num_lines_set;
*swig_lines_get = *LibreDWGc::Dwg_MLINE_vertex_lines_get;
*swig_lines_set = *LibreDWGc::Dwg_MLINE_vertex_lines_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MLINE_vertex(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MLINE_vertex($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_MLINE ##############

package LibreDWG::Dwg_Entity_MLINE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_MLINE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_MLINE_parent_set;
*swig_scale_get = *LibreDWGc::Dwg_Entity_MLINE_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Entity_MLINE_scale_set;
*swig_justification_get = *LibreDWGc::Dwg_Entity_MLINE_justification_get;
*swig_justification_set = *LibreDWGc::Dwg_Entity_MLINE_justification_set;
*swig_base_point_get = *LibreDWGc::Dwg_Entity_MLINE_base_point_get;
*swig_base_point_set = *LibreDWGc::Dwg_Entity_MLINE_base_point_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_MLINE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_MLINE_extrusion_set;
*swig_flags_get = *LibreDWGc::Dwg_Entity_MLINE_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Entity_MLINE_flags_set;
*swig_num_lines_get = *LibreDWGc::Dwg_Entity_MLINE_num_lines_get;
*swig_num_lines_set = *LibreDWGc::Dwg_Entity_MLINE_num_lines_set;
*swig_num_verts_get = *LibreDWGc::Dwg_Entity_MLINE_num_verts_get;
*swig_num_verts_set = *LibreDWGc::Dwg_Entity_MLINE_num_verts_set;
*swig_verts_get = *LibreDWGc::Dwg_Entity_MLINE_verts_get;
*swig_verts_set = *LibreDWGc::Dwg_Entity_MLINE_verts_set;
*swig_mlinestyle_get = *LibreDWGc::Dwg_Entity_MLINE_mlinestyle_get;
*swig_mlinestyle_set = *LibreDWGc::Dwg_Entity_MLINE_mlinestyle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_MLINE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_MLINE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCK_CONTROL ##############

package LibreDWG::Dwg_Object_BLOCK_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_flags_r11_set;
*swig_model_space_get = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_model_space_get;
*swig_model_space_set = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_model_space_set;
*swig_paper_space_get = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_paper_space_get;
*swig_paper_space_set = *LibreDWGc::Dwg_Object_BLOCK_CONTROL_paper_space_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCK_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCK_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCK_HEADER ##############

package LibreDWG::Dwg_Object_BLOCK_HEADER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_set;
*swig___iterator_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER___iterator_get;
*swig___iterator_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER___iterator_set;
*swig_anonymous_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_anonymous_get;
*swig_anonymous_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_anonymous_set;
*swig_hasattrs_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_hasattrs_get;
*swig_hasattrs_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_hasattrs_set;
*swig_blkisxref_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_blkisxref_get;
*swig_blkisxref_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_blkisxref_set;
*swig_xrefoverlaid_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xrefoverlaid_get;
*swig_xrefoverlaid_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xrefoverlaid_set;
*swig_xref_loaded_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_loaded_get;
*swig_xref_loaded_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_loaded_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_num_owned_set;
*swig_base_pt_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_base_pt_get;
*swig_base_pt_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_base_pt_set;
*swig_xref_pname_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_pname_get;
*swig_xref_pname_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_xref_pname_set;
*swig_num_inserts_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_num_inserts_get;
*swig_num_inserts_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_num_inserts_set;
*swig_description_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_description_set;
*swig_preview_size_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_size_get;
*swig_preview_size_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_size_set;
*swig_preview_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_get;
*swig_preview_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_preview_set;
*swig_insert_units_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_insert_units_get;
*swig_insert_units_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_insert_units_set;
*swig_explodable_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_explodable_get;
*swig_explodable_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_explodable_set;
*swig_block_scaling_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_block_scaling_get;
*swig_block_scaling_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_block_scaling_set;
*swig_block_entity_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_block_entity_get;
*swig_block_entity_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_block_entity_set;
*swig_first_entity_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_first_entity_get;
*swig_first_entity_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_first_entity_set;
*swig_last_entity_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_last_entity_get;
*swig_last_entity_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_last_entity_set;
*swig_entities_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_entities_get;
*swig_entities_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_entities_set;
*swig_endblk_entity_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_endblk_entity_get;
*swig_endblk_entity_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_endblk_entity_set;
*swig_inserts_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_inserts_get;
*swig_inserts_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_inserts_set;
*swig_layout_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_layout_get;
*swig_layout_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_layout_set;
*swig_flag2_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_flag2_get;
*swig_flag2_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_flag2_set;
*swig_unknown_r11_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_unknown_r11_get;
*swig_unknown_r11_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_unknown_r11_set;
*swig_block_offset_r11_get = *LibreDWGc::Dwg_Object_BLOCK_HEADER_block_offset_r11_get;
*swig_block_offset_r11_set = *LibreDWGc::Dwg_Object_BLOCK_HEADER_block_offset_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCK_HEADER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCK_HEADER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LAYER_CONTROL ##############

package LibreDWG::Dwg_Object_LAYER_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LAYER_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LAYER_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_LAYER_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_LAYER_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_LAYER_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_LAYER_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_LAYER_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_LAYER_CONTROL_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LAYER_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LAYER_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LAYER ##############

package LibreDWG::Dwg_Object_LAYER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LAYER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LAYER_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_LAYER_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_LAYER_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_LAYER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_LAYER_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_LAYER_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_LAYER_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_LAYER_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_LAYER_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_LAYER_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_LAYER_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_LAYER_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_LAYER_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_LAYER_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_LAYER_xref_set;
*swig_flag0_get = *LibreDWGc::Dwg_Object_LAYER_flag0_get;
*swig_flag0_set = *LibreDWGc::Dwg_Object_LAYER_flag0_set;
*swig_frozen_get = *LibreDWGc::Dwg_Object_LAYER_frozen_get;
*swig_frozen_set = *LibreDWGc::Dwg_Object_LAYER_frozen_set;
*swig_off_get = *LibreDWGc::Dwg_Object_LAYER_off_get;
*swig_off_set = *LibreDWGc::Dwg_Object_LAYER_off_set;
*swig_frozen_in_new_get = *LibreDWGc::Dwg_Object_LAYER_frozen_in_new_get;
*swig_frozen_in_new_set = *LibreDWGc::Dwg_Object_LAYER_frozen_in_new_set;
*swig_locked_get = *LibreDWGc::Dwg_Object_LAYER_locked_get;
*swig_locked_set = *LibreDWGc::Dwg_Object_LAYER_locked_set;
*swig_plotflag_get = *LibreDWGc::Dwg_Object_LAYER_plotflag_get;
*swig_plotflag_set = *LibreDWGc::Dwg_Object_LAYER_plotflag_set;
*swig_linewt_get = *LibreDWGc::Dwg_Object_LAYER_linewt_get;
*swig_linewt_set = *LibreDWGc::Dwg_Object_LAYER_linewt_set;
*swig_color_get = *LibreDWGc::Dwg_Object_LAYER_color_get;
*swig_color_set = *LibreDWGc::Dwg_Object_LAYER_color_set;
*swig_plotstyle_get = *LibreDWGc::Dwg_Object_LAYER_plotstyle_get;
*swig_plotstyle_set = *LibreDWGc::Dwg_Object_LAYER_plotstyle_set;
*swig_material_get = *LibreDWGc::Dwg_Object_LAYER_material_get;
*swig_material_set = *LibreDWGc::Dwg_Object_LAYER_material_set;
*swig_ltype_get = *LibreDWGc::Dwg_Object_LAYER_ltype_get;
*swig_ltype_set = *LibreDWGc::Dwg_Object_LAYER_ltype_set;
*swig_visualstyle_get = *LibreDWGc::Dwg_Object_LAYER_visualstyle_get;
*swig_visualstyle_set = *LibreDWGc::Dwg_Object_LAYER_visualstyle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LAYER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LAYER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_STYLE_CONTROL ##############

package LibreDWG::Dwg_Object_STYLE_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_STYLE_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_STYLE_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_STYLE_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_STYLE_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_STYLE_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_STYLE_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_STYLE_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_STYLE_CONTROL_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_STYLE_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_STYLE_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_STYLE ##############

package LibreDWG::Dwg_Object_STYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_STYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_STYLE_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_STYLE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_STYLE_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_STYLE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_STYLE_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_STYLE_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_STYLE_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_STYLE_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_STYLE_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_STYLE_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_STYLE_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_STYLE_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_STYLE_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_STYLE_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_STYLE_xref_set;
*swig_is_shape_get = *LibreDWGc::Dwg_Object_STYLE_is_shape_get;
*swig_is_shape_set = *LibreDWGc::Dwg_Object_STYLE_is_shape_set;
*swig_is_vertical_get = *LibreDWGc::Dwg_Object_STYLE_is_vertical_get;
*swig_is_vertical_set = *LibreDWGc::Dwg_Object_STYLE_is_vertical_set;
*swig_text_size_get = *LibreDWGc::Dwg_Object_STYLE_text_size_get;
*swig_text_size_set = *LibreDWGc::Dwg_Object_STYLE_text_size_set;
*swig_width_factor_get = *LibreDWGc::Dwg_Object_STYLE_width_factor_get;
*swig_width_factor_set = *LibreDWGc::Dwg_Object_STYLE_width_factor_set;
*swig_oblique_angle_get = *LibreDWGc::Dwg_Object_STYLE_oblique_angle_get;
*swig_oblique_angle_set = *LibreDWGc::Dwg_Object_STYLE_oblique_angle_set;
*swig_generation_get = *LibreDWGc::Dwg_Object_STYLE_generation_get;
*swig_generation_set = *LibreDWGc::Dwg_Object_STYLE_generation_set;
*swig_last_height_get = *LibreDWGc::Dwg_Object_STYLE_last_height_get;
*swig_last_height_set = *LibreDWGc::Dwg_Object_STYLE_last_height_set;
*swig_font_file_get = *LibreDWGc::Dwg_Object_STYLE_font_file_get;
*swig_font_file_set = *LibreDWGc::Dwg_Object_STYLE_font_file_set;
*swig_bigfont_file_get = *LibreDWGc::Dwg_Object_STYLE_bigfont_file_get;
*swig_bigfont_file_set = *LibreDWGc::Dwg_Object_STYLE_bigfont_file_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_STYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_STYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LTYPE_CONTROL ##############

package LibreDWG::Dwg_Object_LTYPE_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_flags_r11_set;
*swig_bylayer_get = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_bylayer_get;
*swig_bylayer_set = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_bylayer_set;
*swig_byblock_get = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_byblock_get;
*swig_byblock_set = *LibreDWGc::Dwg_Object_LTYPE_CONTROL_byblock_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LTYPE_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LTYPE_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LTYPE_dash ##############

package LibreDWG::Dwg_LTYPE_dash;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LTYPE_dash_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LTYPE_dash_parent_set;
*swig_length_get = *LibreDWGc::Dwg_LTYPE_dash_length_get;
*swig_length_set = *LibreDWGc::Dwg_LTYPE_dash_length_set;
*swig_complex_shapecode_get = *LibreDWGc::Dwg_LTYPE_dash_complex_shapecode_get;
*swig_complex_shapecode_set = *LibreDWGc::Dwg_LTYPE_dash_complex_shapecode_set;
*swig_style_get = *LibreDWGc::Dwg_LTYPE_dash_style_get;
*swig_style_set = *LibreDWGc::Dwg_LTYPE_dash_style_set;
*swig_x_offset_get = *LibreDWGc::Dwg_LTYPE_dash_x_offset_get;
*swig_x_offset_set = *LibreDWGc::Dwg_LTYPE_dash_x_offset_set;
*swig_y_offset_get = *LibreDWGc::Dwg_LTYPE_dash_y_offset_get;
*swig_y_offset_set = *LibreDWGc::Dwg_LTYPE_dash_y_offset_set;
*swig_scale_get = *LibreDWGc::Dwg_LTYPE_dash_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_LTYPE_dash_scale_set;
*swig_rotation_get = *LibreDWGc::Dwg_LTYPE_dash_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_LTYPE_dash_rotation_set;
*swig_shape_flag_get = *LibreDWGc::Dwg_LTYPE_dash_shape_flag_get;
*swig_shape_flag_set = *LibreDWGc::Dwg_LTYPE_dash_shape_flag_set;
*swig_text_get = *LibreDWGc::Dwg_LTYPE_dash_text_get;
*swig_text_set = *LibreDWGc::Dwg_LTYPE_dash_text_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LTYPE_dash(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LTYPE_dash($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LTYPE ##############

package LibreDWG::Dwg_Object_LTYPE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LTYPE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LTYPE_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_LTYPE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_LTYPE_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_LTYPE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_LTYPE_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_LTYPE_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_LTYPE_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_LTYPE_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_LTYPE_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_LTYPE_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_LTYPE_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_LTYPE_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_LTYPE_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_LTYPE_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_LTYPE_xref_set;
*swig_description_get = *LibreDWGc::Dwg_Object_LTYPE_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_LTYPE_description_set;
*swig_pattern_len_get = *LibreDWGc::Dwg_Object_LTYPE_pattern_len_get;
*swig_pattern_len_set = *LibreDWGc::Dwg_Object_LTYPE_pattern_len_set;
*swig_alignment_get = *LibreDWGc::Dwg_Object_LTYPE_alignment_get;
*swig_alignment_set = *LibreDWGc::Dwg_Object_LTYPE_alignment_set;
*swig_numdashes_get = *LibreDWGc::Dwg_Object_LTYPE_numdashes_get;
*swig_numdashes_set = *LibreDWGc::Dwg_Object_LTYPE_numdashes_set;
*swig_dashes_get = *LibreDWGc::Dwg_Object_LTYPE_dashes_get;
*swig_dashes_set = *LibreDWGc::Dwg_Object_LTYPE_dashes_set;
*swig_dashes_r11_get = *LibreDWGc::Dwg_Object_LTYPE_dashes_r11_get;
*swig_dashes_r11_set = *LibreDWGc::Dwg_Object_LTYPE_dashes_r11_set;
*swig_has_strings_area_get = *LibreDWGc::Dwg_Object_LTYPE_has_strings_area_get;
*swig_has_strings_area_set = *LibreDWGc::Dwg_Object_LTYPE_has_strings_area_set;
*swig_strings_area_get = *LibreDWGc::Dwg_Object_LTYPE_strings_area_get;
*swig_strings_area_set = *LibreDWGc::Dwg_Object_LTYPE_strings_area_set;
*swig_unknown_r11_get = *LibreDWGc::Dwg_Object_LTYPE_unknown_r11_get;
*swig_unknown_r11_set = *LibreDWGc::Dwg_Object_LTYPE_unknown_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LTYPE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LTYPE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VIEW_CONTROL ##############

package LibreDWG::Dwg_Object_VIEW_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VIEW_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VIEW_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_VIEW_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_VIEW_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_VIEW_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_VIEW_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_VIEW_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_VIEW_CONTROL_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VIEW_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VIEW_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VIEW ##############

package LibreDWG::Dwg_Object_VIEW;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VIEW_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VIEW_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_VIEW_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_VIEW_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_VIEW_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_VIEW_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_VIEW_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_VIEW_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_VIEW_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_VIEW_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_VIEW_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_VIEW_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_VIEW_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_VIEW_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_VIEW_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_VIEW_xref_set;
*swig_VIEWSIZE_get = *LibreDWGc::Dwg_Object_VIEW_VIEWSIZE_get;
*swig_VIEWSIZE_set = *LibreDWGc::Dwg_Object_VIEW_VIEWSIZE_set;
*swig_view_width_get = *LibreDWGc::Dwg_Object_VIEW_view_width_get;
*swig_view_width_set = *LibreDWGc::Dwg_Object_VIEW_view_width_set;
*swig_aspect_ratio_get = *LibreDWGc::Dwg_Object_VIEW_aspect_ratio_get;
*swig_aspect_ratio_set = *LibreDWGc::Dwg_Object_VIEW_aspect_ratio_set;
*swig_VIEWCTR_get = *LibreDWGc::Dwg_Object_VIEW_VIEWCTR_get;
*swig_VIEWCTR_set = *LibreDWGc::Dwg_Object_VIEW_VIEWCTR_set;
*swig_view_target_get = *LibreDWGc::Dwg_Object_VIEW_view_target_get;
*swig_view_target_set = *LibreDWGc::Dwg_Object_VIEW_view_target_set;
*swig_VIEWDIR_get = *LibreDWGc::Dwg_Object_VIEW_VIEWDIR_get;
*swig_VIEWDIR_set = *LibreDWGc::Dwg_Object_VIEW_VIEWDIR_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Object_VIEW_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Object_VIEW_twist_angle_set;
*swig_lens_length_get = *LibreDWGc::Dwg_Object_VIEW_lens_length_get;
*swig_lens_length_set = *LibreDWGc::Dwg_Object_VIEW_lens_length_set;
*swig_front_clip_z_get = *LibreDWGc::Dwg_Object_VIEW_front_clip_z_get;
*swig_front_clip_z_set = *LibreDWGc::Dwg_Object_VIEW_front_clip_z_set;
*swig_back_clip_z_get = *LibreDWGc::Dwg_Object_VIEW_back_clip_z_get;
*swig_back_clip_z_set = *LibreDWGc::Dwg_Object_VIEW_back_clip_z_set;
*swig_VIEWMODE_get = *LibreDWGc::Dwg_Object_VIEW_VIEWMODE_get;
*swig_VIEWMODE_set = *LibreDWGc::Dwg_Object_VIEW_VIEWMODE_set;
*swig_render_mode_get = *LibreDWGc::Dwg_Object_VIEW_render_mode_get;
*swig_render_mode_set = *LibreDWGc::Dwg_Object_VIEW_render_mode_set;
*swig_use_default_lights_get = *LibreDWGc::Dwg_Object_VIEW_use_default_lights_get;
*swig_use_default_lights_set = *LibreDWGc::Dwg_Object_VIEW_use_default_lights_set;
*swig_default_lightning_type_get = *LibreDWGc::Dwg_Object_VIEW_default_lightning_type_get;
*swig_default_lightning_type_set = *LibreDWGc::Dwg_Object_VIEW_default_lightning_type_set;
*swig_brightness_get = *LibreDWGc::Dwg_Object_VIEW_brightness_get;
*swig_brightness_set = *LibreDWGc::Dwg_Object_VIEW_brightness_set;
*swig_contrast_get = *LibreDWGc::Dwg_Object_VIEW_contrast_get;
*swig_contrast_set = *LibreDWGc::Dwg_Object_VIEW_contrast_set;
*swig_ambient_color_get = *LibreDWGc::Dwg_Object_VIEW_ambient_color_get;
*swig_ambient_color_set = *LibreDWGc::Dwg_Object_VIEW_ambient_color_set;
*swig_is_pspace_get = *LibreDWGc::Dwg_Object_VIEW_is_pspace_get;
*swig_is_pspace_set = *LibreDWGc::Dwg_Object_VIEW_is_pspace_set;
*swig_associated_ucs_get = *LibreDWGc::Dwg_Object_VIEW_associated_ucs_get;
*swig_associated_ucs_set = *LibreDWGc::Dwg_Object_VIEW_associated_ucs_set;
*swig_ucsorg_get = *LibreDWGc::Dwg_Object_VIEW_ucsorg_get;
*swig_ucsorg_set = *LibreDWGc::Dwg_Object_VIEW_ucsorg_set;
*swig_ucsxdir_get = *LibreDWGc::Dwg_Object_VIEW_ucsxdir_get;
*swig_ucsxdir_set = *LibreDWGc::Dwg_Object_VIEW_ucsxdir_set;
*swig_ucsydir_get = *LibreDWGc::Dwg_Object_VIEW_ucsydir_get;
*swig_ucsydir_set = *LibreDWGc::Dwg_Object_VIEW_ucsydir_set;
*swig_ucs_elevation_get = *LibreDWGc::Dwg_Object_VIEW_ucs_elevation_get;
*swig_ucs_elevation_set = *LibreDWGc::Dwg_Object_VIEW_ucs_elevation_set;
*swig_UCSORTHOVIEW_get = *LibreDWGc::Dwg_Object_VIEW_UCSORTHOVIEW_get;
*swig_UCSORTHOVIEW_set = *LibreDWGc::Dwg_Object_VIEW_UCSORTHOVIEW_set;
*swig_is_camera_plottable_get = *LibreDWGc::Dwg_Object_VIEW_is_camera_plottable_get;
*swig_is_camera_plottable_set = *LibreDWGc::Dwg_Object_VIEW_is_camera_plottable_set;
*swig_background_get = *LibreDWGc::Dwg_Object_VIEW_background_get;
*swig_background_set = *LibreDWGc::Dwg_Object_VIEW_background_set;
*swig_visualstyle_get = *LibreDWGc::Dwg_Object_VIEW_visualstyle_get;
*swig_visualstyle_set = *LibreDWGc::Dwg_Object_VIEW_visualstyle_set;
*swig_sun_get = *LibreDWGc::Dwg_Object_VIEW_sun_get;
*swig_sun_set = *LibreDWGc::Dwg_Object_VIEW_sun_set;
*swig_base_ucs_get = *LibreDWGc::Dwg_Object_VIEW_base_ucs_get;
*swig_base_ucs_set = *LibreDWGc::Dwg_Object_VIEW_base_ucs_set;
*swig_named_ucs_get = *LibreDWGc::Dwg_Object_VIEW_named_ucs_get;
*swig_named_ucs_set = *LibreDWGc::Dwg_Object_VIEW_named_ucs_set;
*swig_livesection_get = *LibreDWGc::Dwg_Object_VIEW_livesection_get;
*swig_livesection_set = *LibreDWGc::Dwg_Object_VIEW_livesection_set;
*swig_flag_3d_get = *LibreDWGc::Dwg_Object_VIEW_flag_3d_get;
*swig_flag_3d_set = *LibreDWGc::Dwg_Object_VIEW_flag_3d_set;
*swig_unknown_r2_get = *LibreDWGc::Dwg_Object_VIEW_unknown_r2_get;
*swig_unknown_r2_set = *LibreDWGc::Dwg_Object_VIEW_unknown_r2_set;
*swig_unknown_r11_get = *LibreDWGc::Dwg_Object_VIEW_unknown_r11_get;
*swig_unknown_r11_set = *LibreDWGc::Dwg_Object_VIEW_unknown_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VIEW(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VIEW($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_UCS_CONTROL ##############

package LibreDWG::Dwg_Object_UCS_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_UCS_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_UCS_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_UCS_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_UCS_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_UCS_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_UCS_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_UCS_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_UCS_CONTROL_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_UCS_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_UCS_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_UCS_orthopts ##############

package LibreDWG::Dwg_UCS_orthopts;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_UCS_orthopts_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_UCS_orthopts_parent_set;
*swig_type_get = *LibreDWGc::Dwg_UCS_orthopts_type_get;
*swig_type_set = *LibreDWGc::Dwg_UCS_orthopts_type_set;
*swig_pt_get = *LibreDWGc::Dwg_UCS_orthopts_pt_get;
*swig_pt_set = *LibreDWGc::Dwg_UCS_orthopts_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_UCS_orthopts(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_UCS_orthopts($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_UCS ##############

package LibreDWG::Dwg_Object_UCS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_UCS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_UCS_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_UCS_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_UCS_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_UCS_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_UCS_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_UCS_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_UCS_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_UCS_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_UCS_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_UCS_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_UCS_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_UCS_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_UCS_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_UCS_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_UCS_xref_set;
*swig_ucsorg_get = *LibreDWGc::Dwg_Object_UCS_ucsorg_get;
*swig_ucsorg_set = *LibreDWGc::Dwg_Object_UCS_ucsorg_set;
*swig_ucsxdir_get = *LibreDWGc::Dwg_Object_UCS_ucsxdir_get;
*swig_ucsxdir_set = *LibreDWGc::Dwg_Object_UCS_ucsxdir_set;
*swig_ucsydir_get = *LibreDWGc::Dwg_Object_UCS_ucsydir_get;
*swig_ucsydir_set = *LibreDWGc::Dwg_Object_UCS_ucsydir_set;
*swig_ucs_elevation_get = *LibreDWGc::Dwg_Object_UCS_ucs_elevation_get;
*swig_ucs_elevation_set = *LibreDWGc::Dwg_Object_UCS_ucs_elevation_set;
*swig_UCSORTHOVIEW_get = *LibreDWGc::Dwg_Object_UCS_UCSORTHOVIEW_get;
*swig_UCSORTHOVIEW_set = *LibreDWGc::Dwg_Object_UCS_UCSORTHOVIEW_set;
*swig_base_ucs_get = *LibreDWGc::Dwg_Object_UCS_base_ucs_get;
*swig_base_ucs_set = *LibreDWGc::Dwg_Object_UCS_base_ucs_set;
*swig_named_ucs_get = *LibreDWGc::Dwg_Object_UCS_named_ucs_get;
*swig_named_ucs_set = *LibreDWGc::Dwg_Object_UCS_named_ucs_set;
*swig_num_orthopts_get = *LibreDWGc::Dwg_Object_UCS_num_orthopts_get;
*swig_num_orthopts_set = *LibreDWGc::Dwg_Object_UCS_num_orthopts_set;
*swig_orthopts_get = *LibreDWGc::Dwg_Object_UCS_orthopts_get;
*swig_orthopts_set = *LibreDWGc::Dwg_Object_UCS_orthopts_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_UCS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_UCS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VPORT_CONTROL ##############

package LibreDWG::Dwg_Object_VPORT_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VPORT_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VPORT_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_VPORT_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_VPORT_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_VPORT_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_VPORT_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_VPORT_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_VPORT_CONTROL_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VPORT_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VPORT_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VPORT ##############

package LibreDWG::Dwg_Object_VPORT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VPORT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VPORT_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_VPORT_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_VPORT_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_VPORT_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_VPORT_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_VPORT_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_VPORT_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_VPORT_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_VPORT_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_VPORT_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_VPORT_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_VPORT_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_VPORT_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_VPORT_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_VPORT_xref_set;
*swig_VIEWSIZE_get = *LibreDWGc::Dwg_Object_VPORT_VIEWSIZE_get;
*swig_VIEWSIZE_set = *LibreDWGc::Dwg_Object_VPORT_VIEWSIZE_set;
*swig_view_width_get = *LibreDWGc::Dwg_Object_VPORT_view_width_get;
*swig_view_width_set = *LibreDWGc::Dwg_Object_VPORT_view_width_set;
*swig_aspect_ratio_get = *LibreDWGc::Dwg_Object_VPORT_aspect_ratio_get;
*swig_aspect_ratio_set = *LibreDWGc::Dwg_Object_VPORT_aspect_ratio_set;
*swig_VIEWCTR_get = *LibreDWGc::Dwg_Object_VPORT_VIEWCTR_get;
*swig_VIEWCTR_set = *LibreDWGc::Dwg_Object_VPORT_VIEWCTR_set;
*swig_view_target_get = *LibreDWGc::Dwg_Object_VPORT_view_target_get;
*swig_view_target_set = *LibreDWGc::Dwg_Object_VPORT_view_target_set;
*swig_VIEWDIR_get = *LibreDWGc::Dwg_Object_VPORT_VIEWDIR_get;
*swig_VIEWDIR_set = *LibreDWGc::Dwg_Object_VPORT_VIEWDIR_set;
*swig_view_twist_get = *LibreDWGc::Dwg_Object_VPORT_view_twist_get;
*swig_view_twist_set = *LibreDWGc::Dwg_Object_VPORT_view_twist_set;
*swig_lens_length_get = *LibreDWGc::Dwg_Object_VPORT_lens_length_get;
*swig_lens_length_set = *LibreDWGc::Dwg_Object_VPORT_lens_length_set;
*swig_front_clip_z_get = *LibreDWGc::Dwg_Object_VPORT_front_clip_z_get;
*swig_front_clip_z_set = *LibreDWGc::Dwg_Object_VPORT_front_clip_z_set;
*swig_back_clip_z_get = *LibreDWGc::Dwg_Object_VPORT_back_clip_z_get;
*swig_back_clip_z_set = *LibreDWGc::Dwg_Object_VPORT_back_clip_z_set;
*swig_VIEWMODE_get = *LibreDWGc::Dwg_Object_VPORT_VIEWMODE_get;
*swig_VIEWMODE_set = *LibreDWGc::Dwg_Object_VPORT_VIEWMODE_set;
*swig_render_mode_get = *LibreDWGc::Dwg_Object_VPORT_render_mode_get;
*swig_render_mode_set = *LibreDWGc::Dwg_Object_VPORT_render_mode_set;
*swig_use_default_lights_get = *LibreDWGc::Dwg_Object_VPORT_use_default_lights_get;
*swig_use_default_lights_set = *LibreDWGc::Dwg_Object_VPORT_use_default_lights_set;
*swig_default_lightning_type_get = *LibreDWGc::Dwg_Object_VPORT_default_lightning_type_get;
*swig_default_lightning_type_set = *LibreDWGc::Dwg_Object_VPORT_default_lightning_type_set;
*swig_brightness_get = *LibreDWGc::Dwg_Object_VPORT_brightness_get;
*swig_brightness_set = *LibreDWGc::Dwg_Object_VPORT_brightness_set;
*swig_contrast_get = *LibreDWGc::Dwg_Object_VPORT_contrast_get;
*swig_contrast_set = *LibreDWGc::Dwg_Object_VPORT_contrast_set;
*swig_ambient_color_get = *LibreDWGc::Dwg_Object_VPORT_ambient_color_get;
*swig_ambient_color_set = *LibreDWGc::Dwg_Object_VPORT_ambient_color_set;
*swig_lower_left_get = *LibreDWGc::Dwg_Object_VPORT_lower_left_get;
*swig_lower_left_set = *LibreDWGc::Dwg_Object_VPORT_lower_left_set;
*swig_upper_right_get = *LibreDWGc::Dwg_Object_VPORT_upper_right_get;
*swig_upper_right_set = *LibreDWGc::Dwg_Object_VPORT_upper_right_set;
*swig_UCSFOLLOW_get = *LibreDWGc::Dwg_Object_VPORT_UCSFOLLOW_get;
*swig_UCSFOLLOW_set = *LibreDWGc::Dwg_Object_VPORT_UCSFOLLOW_set;
*swig_circle_zoom_get = *LibreDWGc::Dwg_Object_VPORT_circle_zoom_get;
*swig_circle_zoom_set = *LibreDWGc::Dwg_Object_VPORT_circle_zoom_set;
*swig_FASTZOOM_get = *LibreDWGc::Dwg_Object_VPORT_FASTZOOM_get;
*swig_FASTZOOM_set = *LibreDWGc::Dwg_Object_VPORT_FASTZOOM_set;
*swig_UCSICON_get = *LibreDWGc::Dwg_Object_VPORT_UCSICON_get;
*swig_UCSICON_set = *LibreDWGc::Dwg_Object_VPORT_UCSICON_set;
*swig_GRIDMODE_get = *LibreDWGc::Dwg_Object_VPORT_GRIDMODE_get;
*swig_GRIDMODE_set = *LibreDWGc::Dwg_Object_VPORT_GRIDMODE_set;
*swig_GRIDUNIT_get = *LibreDWGc::Dwg_Object_VPORT_GRIDUNIT_get;
*swig_GRIDUNIT_set = *LibreDWGc::Dwg_Object_VPORT_GRIDUNIT_set;
*swig_SNAPMODE_get = *LibreDWGc::Dwg_Object_VPORT_SNAPMODE_get;
*swig_SNAPMODE_set = *LibreDWGc::Dwg_Object_VPORT_SNAPMODE_set;
*swig_SNAPSTYLE_get = *LibreDWGc::Dwg_Object_VPORT_SNAPSTYLE_get;
*swig_SNAPSTYLE_set = *LibreDWGc::Dwg_Object_VPORT_SNAPSTYLE_set;
*swig_SNAPISOPAIR_get = *LibreDWGc::Dwg_Object_VPORT_SNAPISOPAIR_get;
*swig_SNAPISOPAIR_set = *LibreDWGc::Dwg_Object_VPORT_SNAPISOPAIR_set;
*swig_SNAPANG_get = *LibreDWGc::Dwg_Object_VPORT_SNAPANG_get;
*swig_SNAPANG_set = *LibreDWGc::Dwg_Object_VPORT_SNAPANG_set;
*swig_SNAPBASE_get = *LibreDWGc::Dwg_Object_VPORT_SNAPBASE_get;
*swig_SNAPBASE_set = *LibreDWGc::Dwg_Object_VPORT_SNAPBASE_set;
*swig_SNAPUNIT_get = *LibreDWGc::Dwg_Object_VPORT_SNAPUNIT_get;
*swig_SNAPUNIT_set = *LibreDWGc::Dwg_Object_VPORT_SNAPUNIT_set;
*swig_ucs_at_origin_get = *LibreDWGc::Dwg_Object_VPORT_ucs_at_origin_get;
*swig_ucs_at_origin_set = *LibreDWGc::Dwg_Object_VPORT_ucs_at_origin_set;
*swig_UCSVP_get = *LibreDWGc::Dwg_Object_VPORT_UCSVP_get;
*swig_UCSVP_set = *LibreDWGc::Dwg_Object_VPORT_UCSVP_set;
*swig_ucsorg_get = *LibreDWGc::Dwg_Object_VPORT_ucsorg_get;
*swig_ucsorg_set = *LibreDWGc::Dwg_Object_VPORT_ucsorg_set;
*swig_ucsxdir_get = *LibreDWGc::Dwg_Object_VPORT_ucsxdir_get;
*swig_ucsxdir_set = *LibreDWGc::Dwg_Object_VPORT_ucsxdir_set;
*swig_ucsydir_get = *LibreDWGc::Dwg_Object_VPORT_ucsydir_get;
*swig_ucsydir_set = *LibreDWGc::Dwg_Object_VPORT_ucsydir_set;
*swig_ucs_elevation_get = *LibreDWGc::Dwg_Object_VPORT_ucs_elevation_get;
*swig_ucs_elevation_set = *LibreDWGc::Dwg_Object_VPORT_ucs_elevation_set;
*swig_UCSORTHOVIEW_get = *LibreDWGc::Dwg_Object_VPORT_UCSORTHOVIEW_get;
*swig_UCSORTHOVIEW_set = *LibreDWGc::Dwg_Object_VPORT_UCSORTHOVIEW_set;
*swig_grid_flags_get = *LibreDWGc::Dwg_Object_VPORT_grid_flags_get;
*swig_grid_flags_set = *LibreDWGc::Dwg_Object_VPORT_grid_flags_set;
*swig_grid_major_get = *LibreDWGc::Dwg_Object_VPORT_grid_major_get;
*swig_grid_major_set = *LibreDWGc::Dwg_Object_VPORT_grid_major_set;
*swig_background_get = *LibreDWGc::Dwg_Object_VPORT_background_get;
*swig_background_set = *LibreDWGc::Dwg_Object_VPORT_background_set;
*swig_visualstyle_get = *LibreDWGc::Dwg_Object_VPORT_visualstyle_get;
*swig_visualstyle_set = *LibreDWGc::Dwg_Object_VPORT_visualstyle_set;
*swig_sun_get = *LibreDWGc::Dwg_Object_VPORT_sun_get;
*swig_sun_set = *LibreDWGc::Dwg_Object_VPORT_sun_set;
*swig_named_ucs_get = *LibreDWGc::Dwg_Object_VPORT_named_ucs_get;
*swig_named_ucs_set = *LibreDWGc::Dwg_Object_VPORT_named_ucs_set;
*swig_base_ucs_get = *LibreDWGc::Dwg_Object_VPORT_base_ucs_get;
*swig_base_ucs_set = *LibreDWGc::Dwg_Object_VPORT_base_ucs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VPORT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VPORT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_APPID_CONTROL ##############

package LibreDWG::Dwg_Object_APPID_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_APPID_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_APPID_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_APPID_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_APPID_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_APPID_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_APPID_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_APPID_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_APPID_CONTROL_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_APPID_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_APPID_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_APPID ##############

package LibreDWG::Dwg_Object_APPID;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_APPID_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_APPID_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_APPID_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_APPID_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_APPID_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_APPID_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_APPID_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_APPID_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_APPID_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_APPID_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_APPID_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_APPID_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_APPID_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_APPID_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_APPID_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_APPID_xref_set;
*swig_unknown_get = *LibreDWGc::Dwg_Object_APPID_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Object_APPID_unknown_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_APPID(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_APPID($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DIMSTYLE_CONTROL ##############

package LibreDWG::Dwg_Object_DIMSTYLE_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_flags_r11_set;
*swig_num_morehandles_get = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_get;
*swig_num_morehandles_set = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_num_morehandles_set;
*swig_morehandles_get = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_morehandles_get;
*swig_morehandles_set = *LibreDWGc::Dwg_Object_DIMSTYLE_CONTROL_morehandles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DIMSTYLE_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DIMSTYLE_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DIMSTYLE ##############

package LibreDWG::Dwg_Object_DIMSTYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DIMSTYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DIMSTYLE_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_DIMSTYLE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_DIMSTYLE_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_DIMSTYLE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_DIMSTYLE_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_DIMSTYLE_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_DIMSTYLE_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_DIMSTYLE_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_DIMSTYLE_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_DIMSTYLE_xref_set;
*swig_DIMTOL_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOL_get;
*swig_DIMTOL_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOL_set;
*swig_DIMLIM_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLIM_get;
*swig_DIMLIM_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLIM_set;
*swig_DIMTIH_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIH_get;
*swig_DIMTIH_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIH_set;
*swig_DIMTOH_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOH_get;
*swig_DIMTOH_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOH_set;
*swig_DIMSE1_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE1_get;
*swig_DIMSE1_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE1_set;
*swig_DIMSE2_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE2_get;
*swig_DIMSE2_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSE2_set;
*swig_DIMALT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALT_get;
*swig_DIMALT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALT_set;
*swig_DIMTOFL_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOFL_get;
*swig_DIMTOFL_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOFL_set;
*swig_DIMSAH_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSAH_get;
*swig_DIMSAH_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSAH_set;
*swig_DIMTIX_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIX_get;
*swig_DIMTIX_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTIX_set;
*swig_DIMSOXD_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSOXD_get;
*swig_DIMSOXD_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSOXD_set;
*swig_DIMALTD_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTD_get;
*swig_DIMALTD_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTD_set;
*swig_DIMZIN_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMZIN_get;
*swig_DIMZIN_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMZIN_set;
*swig_DIMSD1_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD1_get;
*swig_DIMSD1_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD1_set;
*swig_DIMSD2_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD2_get;
*swig_DIMSD2_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSD2_set;
*swig_DIMTOLJ_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOLJ_get;
*swig_DIMTOLJ_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTOLJ_set;
*swig_DIMJUST_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMJUST_get;
*swig_DIMJUST_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMJUST_set;
*swig_DIMFIT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFIT_get;
*swig_DIMFIT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFIT_set;
*swig_DIMUPT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMUPT_get;
*swig_DIMUPT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMUPT_set;
*swig_DIMTZIN_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTZIN_get;
*swig_DIMTZIN_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTZIN_set;
*swig_DIMALTZ_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTZ_get;
*swig_DIMALTZ_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTZ_set;
*swig_DIMALTTZ_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTZ_get;
*swig_DIMALTTZ_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTZ_set;
*swig_DIMTAD_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTAD_get;
*swig_DIMTAD_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTAD_set;
*swig_DIMUNIT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMUNIT_get;
*swig_DIMUNIT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMUNIT_set;
*swig_DIMAUNIT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMAUNIT_get;
*swig_DIMAUNIT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMAUNIT_set;
*swig_DIMDEC_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDEC_get;
*swig_DIMDEC_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDEC_set;
*swig_DIMTDEC_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTDEC_get;
*swig_DIMTDEC_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTDEC_set;
*swig_DIMALTU_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTU_get;
*swig_DIMALTU_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTU_set;
*swig_DIMALTTD_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTD_get;
*swig_DIMALTTD_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTTD_set;
*swig_DIMSCALE_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSCALE_get;
*swig_DIMSCALE_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMSCALE_set;
*swig_DIMASZ_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMASZ_get;
*swig_DIMASZ_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMASZ_set;
*swig_DIMEXO_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXO_get;
*swig_DIMEXO_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXO_set;
*swig_DIMDLI_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLI_get;
*swig_DIMDLI_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLI_set;
*swig_DIMEXE_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXE_get;
*swig_DIMEXE_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMEXE_set;
*swig_DIMRND_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMRND_get;
*swig_DIMRND_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMRND_set;
*swig_DIMDLE_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLE_get;
*swig_DIMDLE_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDLE_set;
*swig_DIMTP_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTP_get;
*swig_DIMTP_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTP_set;
*swig_DIMTM_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTM_get;
*swig_DIMTM_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTM_set;
*swig_DIMFXL_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXL_get;
*swig_DIMFXL_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXL_set;
*swig_DIMJOGANG_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMJOGANG_get;
*swig_DIMJOGANG_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMJOGANG_set;
*swig_DIMTFILL_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILL_get;
*swig_DIMTFILL_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILL_set;
*swig_DIMTFILLCLR_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILLCLR_get;
*swig_DIMTFILLCLR_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFILLCLR_set;
*swig_DIMAZIN_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMAZIN_get;
*swig_DIMAZIN_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMAZIN_set;
*swig_DIMARCSYM_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMARCSYM_get;
*swig_DIMARCSYM_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMARCSYM_set;
*swig_DIMTXT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXT_get;
*swig_DIMTXT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXT_set;
*swig_DIMCEN_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCEN_get;
*swig_DIMCEN_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCEN_set;
*swig_DIMTSZ_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTSZ_get;
*swig_DIMTSZ_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTSZ_set;
*swig_DIMALTF_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTF_get;
*swig_DIMALTF_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTF_set;
*swig_DIMLFAC_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLFAC_get;
*swig_DIMLFAC_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLFAC_set;
*swig_DIMTVP_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTVP_get;
*swig_DIMTVP_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTVP_set;
*swig_DIMTFAC_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFAC_get;
*swig_DIMTFAC_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTFAC_set;
*swig_DIMGAP_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMGAP_get;
*swig_DIMGAP_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMGAP_set;
*swig_DIMPOST_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMPOST_get;
*swig_DIMPOST_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMPOST_set;
*swig_DIMAPOST_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMAPOST_get;
*swig_DIMAPOST_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMAPOST_set;
*swig_DIMBLK_T_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_T_get;
*swig_DIMBLK_T_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_T_set;
*swig_DIMBLK1_T_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_T_get;
*swig_DIMBLK1_T_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_T_set;
*swig_DIMBLK2_T_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_T_get;
*swig_DIMBLK2_T_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_T_set;
*swig_DIMALTRND_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTRND_get;
*swig_DIMALTRND_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTRND_set;
*swig_DIMCLRD_N_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_N_get;
*swig_DIMCLRD_N_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_N_set;
*swig_DIMCLRE_N_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_N_get;
*swig_DIMCLRE_N_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_N_set;
*swig_DIMCLRT_N_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_N_get;
*swig_DIMCLRT_N_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_N_set;
*swig_DIMCLRD_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_get;
*swig_DIMCLRD_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRD_set;
*swig_DIMCLRE_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_get;
*swig_DIMCLRE_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRE_set;
*swig_DIMCLRT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_get;
*swig_DIMCLRT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMCLRT_set;
*swig_DIMADEC_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMADEC_get;
*swig_DIMADEC_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMADEC_set;
*swig_DIMFRAC_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFRAC_get;
*swig_DIMFRAC_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFRAC_set;
*swig_DIMLUNIT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLUNIT_get;
*swig_DIMLUNIT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLUNIT_set;
*swig_DIMDSEP_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDSEP_get;
*swig_DIMDSEP_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMDSEP_set;
*swig_DIMTMOVE_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTMOVE_get;
*swig_DIMTMOVE_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTMOVE_set;
*swig_DIMATFIT_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMATFIT_get;
*swig_DIMATFIT_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMATFIT_set;
*swig_DIMFXLON_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXLON_get;
*swig_DIMFXLON_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMFXLON_set;
*swig_DIMTXTDIRECTION_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_get;
*swig_DIMTXTDIRECTION_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXTDIRECTION_set;
*swig_DIMALTMZF_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZF_get;
*swig_DIMALTMZF_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZF_set;
*swig_DIMALTMZS_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZS_get;
*swig_DIMALTMZS_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMALTMZS_set;
*swig_DIMMZF_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZF_get;
*swig_DIMMZF_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZF_set;
*swig_DIMMZS_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZS_get;
*swig_DIMMZS_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMMZS_set;
*swig_DIMLWD_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWD_get;
*swig_DIMLWD_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWD_set;
*swig_DIMLWE_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWE_get;
*swig_DIMLWE_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLWE_set;
*swig_flag0_get = *LibreDWGc::Dwg_Object_DIMSTYLE_flag0_get;
*swig_flag0_set = *LibreDWGc::Dwg_Object_DIMSTYLE_flag0_set;
*swig_DIMTXSTY_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXSTY_get;
*swig_DIMTXSTY_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMTXSTY_set;
*swig_DIMLDRBLK_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLDRBLK_get;
*swig_DIMLDRBLK_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLDRBLK_set;
*swig_DIMBLK_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_get;
*swig_DIMBLK_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK_set;
*swig_DIMBLK1_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_get;
*swig_DIMBLK1_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK1_set;
*swig_DIMBLK2_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_get;
*swig_DIMBLK2_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMBLK2_set;
*swig_DIMLTYPE_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTYPE_get;
*swig_DIMLTYPE_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTYPE_set;
*swig_DIMLTEX1_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX1_get;
*swig_DIMLTEX1_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX1_set;
*swig_DIMLTEX2_get = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX2_get;
*swig_DIMLTEX2_set = *LibreDWGc::Dwg_Object_DIMSTYLE_DIMLTEX2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DIMSTYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DIMSTYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VX_CONTROL ##############

package LibreDWG::Dwg_Object_VX_CONTROL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VX_CONTROL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VX_CONTROL_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_VX_CONTROL_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_VX_CONTROL_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_VX_CONTROL_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_VX_CONTROL_entries_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Object_VX_CONTROL_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Object_VX_CONTROL_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VX_CONTROL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VX_CONTROL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VX_TABLE_RECORD ##############

package LibreDWG::Dwg_Object_VX_TABLE_RECORD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_name_set;
*swig_used_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_used_get;
*swig_used_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_used_set;
*swig_is_xref_ref_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_ref_get;
*swig_is_xref_ref_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_ref_set;
*swig_is_xref_resolved_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_get;
*swig_is_xref_resolved_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_resolved_set;
*swig_is_xref_dep_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_dep_get;
*swig_is_xref_dep_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_xref_dep_set;
*swig_xref_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_xref_get;
*swig_xref_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_xref_set;
*swig_is_on_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_on_get;
*swig_is_on_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_is_on_set;
*swig_viewport_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_viewport_get;
*swig_viewport_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_viewport_set;
*swig_prev_entry_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_prev_entry_get;
*swig_prev_entry_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_prev_entry_set;
*swig_vport_entity_address_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_vport_entity_address_get;
*swig_vport_entity_address_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_vport_entity_address_set;
*swig_r11_viewport_index_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_get;
*swig_r11_viewport_index_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_viewport_index_set;
*swig_r11_prev_entry_index_get = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_get;
*swig_r11_prev_entry_index_set = *LibreDWGc::Dwg_Object_VX_TABLE_RECORD_r11_prev_entry_index_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VX_TABLE_RECORD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VX_TABLE_RECORD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_GROUP ##############

package LibreDWG::Dwg_Object_GROUP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_GROUP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_GROUP_parent_set;
*swig_name_get = *LibreDWGc::Dwg_Object_GROUP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_GROUP_name_set;
*swig_unnamed_get = *LibreDWGc::Dwg_Object_GROUP_unnamed_get;
*swig_unnamed_set = *LibreDWGc::Dwg_Object_GROUP_unnamed_set;
*swig_selectable_get = *LibreDWGc::Dwg_Object_GROUP_selectable_get;
*swig_selectable_set = *LibreDWGc::Dwg_Object_GROUP_selectable_set;
*swig_num_groups_get = *LibreDWGc::Dwg_Object_GROUP_num_groups_get;
*swig_num_groups_set = *LibreDWGc::Dwg_Object_GROUP_num_groups_set;
*swig_groups_get = *LibreDWGc::Dwg_Object_GROUP_groups_get;
*swig_groups_set = *LibreDWGc::Dwg_Object_GROUP_groups_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_GROUP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_GROUP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MLINESTYLE_line ##############

package LibreDWG::Dwg_MLINESTYLE_line;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_MLINESTYLE_line_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_MLINESTYLE_line_parent_set;
*swig_offset_get = *LibreDWGc::Dwg_MLINESTYLE_line_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_MLINESTYLE_line_offset_set;
*swig_color_get = *LibreDWGc::Dwg_MLINESTYLE_line_color_get;
*swig_color_set = *LibreDWGc::Dwg_MLINESTYLE_line_color_set;
*swig_lt_get = *LibreDWGc::Dwg_MLINESTYLE_line_lt_get;
*swig_lt_set = *LibreDWGc::Dwg_MLINESTYLE_line_lt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MLINESTYLE_line(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MLINESTYLE_line($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_MLINESTYLE_line_lt ##############

package LibreDWG::_dwg_MLINESTYLE_line_lt;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_index_get = *LibreDWGc::_dwg_MLINESTYLE_line_lt_index_get;
*swig_index_set = *LibreDWGc::_dwg_MLINESTYLE_line_lt_index_set;
*swig_ltype_get = *LibreDWGc::_dwg_MLINESTYLE_line_lt_ltype_get;
*swig_ltype_set = *LibreDWGc::_dwg_MLINESTYLE_line_lt_ltype_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_MLINESTYLE_line_lt(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_MLINESTYLE_line_lt($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MLINESTYLE ##############

package LibreDWG::Dwg_Object_MLINESTYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MLINESTYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MLINESTYLE_parent_set;
*swig_name_get = *LibreDWGc::Dwg_Object_MLINESTYLE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_MLINESTYLE_name_set;
*swig_description_get = *LibreDWGc::Dwg_Object_MLINESTYLE_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_MLINESTYLE_description_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_MLINESTYLE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_MLINESTYLE_flag_set;
*swig_fill_color_get = *LibreDWGc::Dwg_Object_MLINESTYLE_fill_color_get;
*swig_fill_color_set = *LibreDWGc::Dwg_Object_MLINESTYLE_fill_color_set;
*swig_start_angle_get = *LibreDWGc::Dwg_Object_MLINESTYLE_start_angle_get;
*swig_start_angle_set = *LibreDWGc::Dwg_Object_MLINESTYLE_start_angle_set;
*swig_end_angle_get = *LibreDWGc::Dwg_Object_MLINESTYLE_end_angle_get;
*swig_end_angle_set = *LibreDWGc::Dwg_Object_MLINESTYLE_end_angle_set;
*swig_num_lines_get = *LibreDWGc::Dwg_Object_MLINESTYLE_num_lines_get;
*swig_num_lines_set = *LibreDWGc::Dwg_Object_MLINESTYLE_num_lines_set;
*swig_lines_get = *LibreDWGc::Dwg_Object_MLINESTYLE_lines_get;
*swig_lines_set = *LibreDWGc::Dwg_Object_MLINESTYLE_lines_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MLINESTYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MLINESTYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_OLE2FRAME ##############

package LibreDWG::Dwg_Entity_OLE2FRAME;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_parent_set;
*swig_type_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_type_get;
*swig_type_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_type_set;
*swig_mode_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_mode_get;
*swig_mode_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_mode_set;
*swig_lock_aspect_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_lock_aspect_get;
*swig_lock_aspect_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_lock_aspect_set;
*swig_data_size_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_data_size_set;
*swig_data_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_data_get;
*swig_data_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_data_set;
*swig_oleversion_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_oleversion_get;
*swig_oleversion_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_oleversion_set;
*swig_oleclient_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_oleclient_get;
*swig_oleclient_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_oleclient_set;
*swig_pt1_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_pt1_get;
*swig_pt1_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_pt1_set;
*swig_pt2_get = *LibreDWGc::Dwg_Entity_OLE2FRAME_pt2_get;
*swig_pt2_set = *LibreDWGc::Dwg_Entity_OLE2FRAME_pt2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_OLE2FRAME(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_OLE2FRAME($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DUMMY ##############

package LibreDWG::Dwg_Object_DUMMY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DUMMY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DUMMY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DUMMY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DUMMY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LONG_TRANSACTION ##############

package LibreDWG::Dwg_Object_LONG_TRANSACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LONG_TRANSACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LONG_TRANSACTION_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LONG_TRANSACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LONG_TRANSACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LWPOLYLINE_width ##############

package LibreDWG::Dwg_LWPOLYLINE_width;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_start_get = *LibreDWGc::Dwg_LWPOLYLINE_width_start_get;
*swig_start_set = *LibreDWGc::Dwg_LWPOLYLINE_width_start_set;
*swig_end_get = *LibreDWGc::Dwg_LWPOLYLINE_width_end_get;
*swig_end_set = *LibreDWGc::Dwg_LWPOLYLINE_width_end_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LWPOLYLINE_width(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LWPOLYLINE_width($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_PROXY_LWPOLYLINE ##############

package LibreDWG::Dwg_PROXY_LWPOLYLINE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_parent_set;
*swig_data_size_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_data_size_set;
*swig_flags_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_flags_set;
*swig_const_width_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_const_width_get;
*swig_const_width_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_const_width_set;
*swig_elevation_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_elevation_set;
*swig_thickness_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_thickness_set;
*swig_extrusion_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_extrusion_set;
*swig_num_points_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_points_get;
*swig_num_points_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_points_set;
*swig_points_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_points_get;
*swig_points_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_points_set;
*swig_num_bulges_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_bulges_get;
*swig_num_bulges_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_bulges_set;
*swig_bulges_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_bulges_get;
*swig_bulges_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_bulges_set;
*swig_num_widths_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_widths_get;
*swig_num_widths_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_num_widths_set;
*swig_widths_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_widths_get;
*swig_widths_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_widths_set;
*swig_unknown_1_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_1_get;
*swig_unknown_1_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_1_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_2_set;
*swig_unknown_3_get = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_3_get;
*swig_unknown_3_set = *LibreDWGc::Dwg_PROXY_LWPOLYLINE_unknown_3_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_PROXY_LWPOLYLINE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_PROXY_LWPOLYLINE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_PROXY_ENTITY ##############

package LibreDWG::Dwg_Entity_PROXY_ENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_parent_set;
*swig_proxy_id_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_id_get;
*swig_proxy_id_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_id_set;
*swig_class_id_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_class_id_get;
*swig_class_id_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_class_id_set;
*swig_dwg_versions_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_versions_get;
*swig_dwg_versions_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_versions_set;
*swig_maint_version_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_maint_version_get;
*swig_maint_version_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_maint_version_set;
*swig_dwg_version_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_version_get;
*swig_dwg_version_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_dwg_version_set;
*swig_from_dxf_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_from_dxf_get;
*swig_from_dxf_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_from_dxf_set;
*swig_proxy_data_size_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_size_get;
*swig_proxy_data_size_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_size_set;
*swig_proxy_data_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_get;
*swig_proxy_data_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_proxy_data_set;
*swig_data_numbits_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_numbits_get;
*swig_data_numbits_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_numbits_set;
*swig_data_size_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_size_set;
*swig_data_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_get;
*swig_data_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_data_set;
*swig_num_objids_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_num_objids_get;
*swig_num_objids_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_num_objids_set;
*swig_objids_get = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_objids_get;
*swig_objids_set = *LibreDWGc::Dwg_Entity_PROXY_ENTITY_objids_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_PROXY_ENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_PROXY_ENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_PROXY_OBJECT ##############

package LibreDWG::Dwg_Object_PROXY_OBJECT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_parent_set;
*swig_proxy_id_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_proxy_id_get;
*swig_proxy_id_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_proxy_id_set;
*swig_class_id_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_class_id_get;
*swig_class_id_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_class_id_set;
*swig_dwg_versions_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_versions_get;
*swig_dwg_versions_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_versions_set;
*swig_maint_version_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_maint_version_get;
*swig_maint_version_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_maint_version_set;
*swig_dwg_version_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_version_get;
*swig_dwg_version_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_dwg_version_set;
*swig_from_dxf_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_from_dxf_get;
*swig_from_dxf_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_from_dxf_set;
*swig_data_numbits_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_data_numbits_get;
*swig_data_numbits_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_data_numbits_set;
*swig_data_size_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_data_size_set;
*swig_data_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_data_get;
*swig_data_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_data_set;
*swig_num_objids_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_num_objids_get;
*swig_num_objids_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_num_objids_set;
*swig_objids_get = *LibreDWGc::Dwg_Object_PROXY_OBJECT_objids_get;
*swig_objids_set = *LibreDWGc::Dwg_Object_PROXY_OBJECT_objids_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_PROXY_OBJECT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_PROXY_OBJECT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_HATCH_Color ##############

package LibreDWG::Dwg_HATCH_Color;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_HATCH_Color_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_HATCH_Color_parent_set;
*swig_shift_value_get = *LibreDWGc::Dwg_HATCH_Color_shift_value_get;
*swig_shift_value_set = *LibreDWGc::Dwg_HATCH_Color_shift_value_set;
*swig_color_get = *LibreDWGc::Dwg_HATCH_Color_color_get;
*swig_color_set = *LibreDWGc::Dwg_HATCH_Color_color_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_HATCH_Color(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_HATCH_Color($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_HATCH_ControlPoint ##############

package LibreDWG::Dwg_HATCH_ControlPoint;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_HATCH_ControlPoint_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_HATCH_ControlPoint_parent_set;
*swig_point_get = *LibreDWGc::Dwg_HATCH_ControlPoint_point_get;
*swig_point_set = *LibreDWGc::Dwg_HATCH_ControlPoint_point_set;
*swig_weight_get = *LibreDWGc::Dwg_HATCH_ControlPoint_weight_get;
*swig_weight_set = *LibreDWGc::Dwg_HATCH_ControlPoint_weight_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_HATCH_ControlPoint(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_HATCH_ControlPoint($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_HATCH_PathSeg ##############

package LibreDWG::Dwg_HATCH_PathSeg;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_HATCH_PathSeg_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_HATCH_PathSeg_parent_set;
*swig_curve_type_get = *LibreDWGc::Dwg_HATCH_PathSeg_curve_type_get;
*swig_curve_type_set = *LibreDWGc::Dwg_HATCH_PathSeg_curve_type_set;
*swig_first_endpoint_get = *LibreDWGc::Dwg_HATCH_PathSeg_first_endpoint_get;
*swig_first_endpoint_set = *LibreDWGc::Dwg_HATCH_PathSeg_first_endpoint_set;
*swig_second_endpoint_get = *LibreDWGc::Dwg_HATCH_PathSeg_second_endpoint_get;
*swig_second_endpoint_set = *LibreDWGc::Dwg_HATCH_PathSeg_second_endpoint_set;
*swig_center_get = *LibreDWGc::Dwg_HATCH_PathSeg_center_get;
*swig_center_set = *LibreDWGc::Dwg_HATCH_PathSeg_center_set;
*swig_radius_get = *LibreDWGc::Dwg_HATCH_PathSeg_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_HATCH_PathSeg_radius_set;
*swig_start_angle_get = *LibreDWGc::Dwg_HATCH_PathSeg_start_angle_get;
*swig_start_angle_set = *LibreDWGc::Dwg_HATCH_PathSeg_start_angle_set;
*swig_end_angle_get = *LibreDWGc::Dwg_HATCH_PathSeg_end_angle_get;
*swig_end_angle_set = *LibreDWGc::Dwg_HATCH_PathSeg_end_angle_set;
*swig_is_ccw_get = *LibreDWGc::Dwg_HATCH_PathSeg_is_ccw_get;
*swig_is_ccw_set = *LibreDWGc::Dwg_HATCH_PathSeg_is_ccw_set;
*swig_endpoint_get = *LibreDWGc::Dwg_HATCH_PathSeg_endpoint_get;
*swig_endpoint_set = *LibreDWGc::Dwg_HATCH_PathSeg_endpoint_set;
*swig_minor_major_ratio_get = *LibreDWGc::Dwg_HATCH_PathSeg_minor_major_ratio_get;
*swig_minor_major_ratio_set = *LibreDWGc::Dwg_HATCH_PathSeg_minor_major_ratio_set;
*swig_degree_get = *LibreDWGc::Dwg_HATCH_PathSeg_degree_get;
*swig_degree_set = *LibreDWGc::Dwg_HATCH_PathSeg_degree_set;
*swig_is_rational_get = *LibreDWGc::Dwg_HATCH_PathSeg_is_rational_get;
*swig_is_rational_set = *LibreDWGc::Dwg_HATCH_PathSeg_is_rational_set;
*swig_is_periodic_get = *LibreDWGc::Dwg_HATCH_PathSeg_is_periodic_get;
*swig_is_periodic_set = *LibreDWGc::Dwg_HATCH_PathSeg_is_periodic_set;
*swig_num_knots_get = *LibreDWGc::Dwg_HATCH_PathSeg_num_knots_get;
*swig_num_knots_set = *LibreDWGc::Dwg_HATCH_PathSeg_num_knots_set;
*swig_num_control_points_get = *LibreDWGc::Dwg_HATCH_PathSeg_num_control_points_get;
*swig_num_control_points_set = *LibreDWGc::Dwg_HATCH_PathSeg_num_control_points_set;
*swig_knots_get = *LibreDWGc::Dwg_HATCH_PathSeg_knots_get;
*swig_knots_set = *LibreDWGc::Dwg_HATCH_PathSeg_knots_set;
*swig_control_points_get = *LibreDWGc::Dwg_HATCH_PathSeg_control_points_get;
*swig_control_points_set = *LibreDWGc::Dwg_HATCH_PathSeg_control_points_set;
*swig_num_fitpts_get = *LibreDWGc::Dwg_HATCH_PathSeg_num_fitpts_get;
*swig_num_fitpts_set = *LibreDWGc::Dwg_HATCH_PathSeg_num_fitpts_set;
*swig_fitpts_get = *LibreDWGc::Dwg_HATCH_PathSeg_fitpts_get;
*swig_fitpts_set = *LibreDWGc::Dwg_HATCH_PathSeg_fitpts_set;
*swig_start_tangent_get = *LibreDWGc::Dwg_HATCH_PathSeg_start_tangent_get;
*swig_start_tangent_set = *LibreDWGc::Dwg_HATCH_PathSeg_start_tangent_set;
*swig_end_tangent_get = *LibreDWGc::Dwg_HATCH_PathSeg_end_tangent_get;
*swig_end_tangent_set = *LibreDWGc::Dwg_HATCH_PathSeg_end_tangent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_HATCH_PathSeg(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_HATCH_PathSeg($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_HATCH_PolylinePath ##############

package LibreDWG::Dwg_HATCH_PolylinePath;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_HATCH_PolylinePath_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_HATCH_PolylinePath_parent_set;
*swig_point_get = *LibreDWGc::Dwg_HATCH_PolylinePath_point_get;
*swig_point_set = *LibreDWGc::Dwg_HATCH_PolylinePath_point_set;
*swig_bulge_get = *LibreDWGc::Dwg_HATCH_PolylinePath_bulge_get;
*swig_bulge_set = *LibreDWGc::Dwg_HATCH_PolylinePath_bulge_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_HATCH_PolylinePath(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_HATCH_PolylinePath($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_HATCH_Path ##############

package LibreDWG::Dwg_HATCH_Path;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_HATCH_Path_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_HATCH_Path_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_HATCH_Path_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_HATCH_Path_flag_set;
*swig_num_segs_or_paths_get = *LibreDWGc::Dwg_HATCH_Path_num_segs_or_paths_get;
*swig_num_segs_or_paths_set = *LibreDWGc::Dwg_HATCH_Path_num_segs_or_paths_set;
*swig_segs_get = *LibreDWGc::Dwg_HATCH_Path_segs_get;
*swig_segs_set = *LibreDWGc::Dwg_HATCH_Path_segs_set;
*swig_bulges_present_get = *LibreDWGc::Dwg_HATCH_Path_bulges_present_get;
*swig_bulges_present_set = *LibreDWGc::Dwg_HATCH_Path_bulges_present_set;
*swig_closed_get = *LibreDWGc::Dwg_HATCH_Path_closed_get;
*swig_closed_set = *LibreDWGc::Dwg_HATCH_Path_closed_set;
*swig_polyline_paths_get = *LibreDWGc::Dwg_HATCH_Path_polyline_paths_get;
*swig_polyline_paths_set = *LibreDWGc::Dwg_HATCH_Path_polyline_paths_set;
*swig_num_boundary_handles_get = *LibreDWGc::Dwg_HATCH_Path_num_boundary_handles_get;
*swig_num_boundary_handles_set = *LibreDWGc::Dwg_HATCH_Path_num_boundary_handles_set;
*swig_boundary_handles_get = *LibreDWGc::Dwg_HATCH_Path_boundary_handles_get;
*swig_boundary_handles_set = *LibreDWGc::Dwg_HATCH_Path_boundary_handles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_HATCH_Path(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_HATCH_Path($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_HATCH_DefLine ##############

package LibreDWG::Dwg_HATCH_DefLine;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_HATCH_DefLine_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_HATCH_DefLine_parent_set;
*swig_angle_get = *LibreDWGc::Dwg_HATCH_DefLine_angle_get;
*swig_angle_set = *LibreDWGc::Dwg_HATCH_DefLine_angle_set;
*swig_pt0_get = *LibreDWGc::Dwg_HATCH_DefLine_pt0_get;
*swig_pt0_set = *LibreDWGc::Dwg_HATCH_DefLine_pt0_set;
*swig_offset_get = *LibreDWGc::Dwg_HATCH_DefLine_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_HATCH_DefLine_offset_set;
*swig_num_dashes_get = *LibreDWGc::Dwg_HATCH_DefLine_num_dashes_get;
*swig_num_dashes_set = *LibreDWGc::Dwg_HATCH_DefLine_num_dashes_set;
*swig_dashes_get = *LibreDWGc::Dwg_HATCH_DefLine_dashes_get;
*swig_dashes_set = *LibreDWGc::Dwg_HATCH_DefLine_dashes_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_HATCH_DefLine(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_HATCH_DefLine($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_HATCH ##############

package LibreDWG::Dwg_Entity_HATCH;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_HATCH_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_HATCH_parent_set;
*swig_is_gradient_fill_get = *LibreDWGc::Dwg_Entity_HATCH_is_gradient_fill_get;
*swig_is_gradient_fill_set = *LibreDWGc::Dwg_Entity_HATCH_is_gradient_fill_set;
*swig_reserved_get = *LibreDWGc::Dwg_Entity_HATCH_reserved_get;
*swig_reserved_set = *LibreDWGc::Dwg_Entity_HATCH_reserved_set;
*swig_gradient_angle_get = *LibreDWGc::Dwg_Entity_HATCH_gradient_angle_get;
*swig_gradient_angle_set = *LibreDWGc::Dwg_Entity_HATCH_gradient_angle_set;
*swig_gradient_shift_get = *LibreDWGc::Dwg_Entity_HATCH_gradient_shift_get;
*swig_gradient_shift_set = *LibreDWGc::Dwg_Entity_HATCH_gradient_shift_set;
*swig_single_color_gradient_get = *LibreDWGc::Dwg_Entity_HATCH_single_color_gradient_get;
*swig_single_color_gradient_set = *LibreDWGc::Dwg_Entity_HATCH_single_color_gradient_set;
*swig_gradient_tint_get = *LibreDWGc::Dwg_Entity_HATCH_gradient_tint_get;
*swig_gradient_tint_set = *LibreDWGc::Dwg_Entity_HATCH_gradient_tint_set;
*swig_num_colors_get = *LibreDWGc::Dwg_Entity_HATCH_num_colors_get;
*swig_num_colors_set = *LibreDWGc::Dwg_Entity_HATCH_num_colors_set;
*swig_colors_get = *LibreDWGc::Dwg_Entity_HATCH_colors_get;
*swig_colors_set = *LibreDWGc::Dwg_Entity_HATCH_colors_set;
*swig_gradient_name_get = *LibreDWGc::Dwg_Entity_HATCH_gradient_name_get;
*swig_gradient_name_set = *LibreDWGc::Dwg_Entity_HATCH_gradient_name_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_HATCH_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_HATCH_elevation_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_HATCH_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_HATCH_extrusion_set;
*swig_name_get = *LibreDWGc::Dwg_Entity_HATCH_name_get;
*swig_name_set = *LibreDWGc::Dwg_Entity_HATCH_name_set;
*swig_is_solid_fill_get = *LibreDWGc::Dwg_Entity_HATCH_is_solid_fill_get;
*swig_is_solid_fill_set = *LibreDWGc::Dwg_Entity_HATCH_is_solid_fill_set;
*swig_is_associative_get = *LibreDWGc::Dwg_Entity_HATCH_is_associative_get;
*swig_is_associative_set = *LibreDWGc::Dwg_Entity_HATCH_is_associative_set;
*swig_num_paths_get = *LibreDWGc::Dwg_Entity_HATCH_num_paths_get;
*swig_num_paths_set = *LibreDWGc::Dwg_Entity_HATCH_num_paths_set;
*swig_paths_get = *LibreDWGc::Dwg_Entity_HATCH_paths_get;
*swig_paths_set = *LibreDWGc::Dwg_Entity_HATCH_paths_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_HATCH_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_HATCH_style_set;
*swig_pattern_type_get = *LibreDWGc::Dwg_Entity_HATCH_pattern_type_get;
*swig_pattern_type_set = *LibreDWGc::Dwg_Entity_HATCH_pattern_type_set;
*swig_angle_get = *LibreDWGc::Dwg_Entity_HATCH_angle_get;
*swig_angle_set = *LibreDWGc::Dwg_Entity_HATCH_angle_set;
*swig_scale_spacing_get = *LibreDWGc::Dwg_Entity_HATCH_scale_spacing_get;
*swig_scale_spacing_set = *LibreDWGc::Dwg_Entity_HATCH_scale_spacing_set;
*swig_double_flag_get = *LibreDWGc::Dwg_Entity_HATCH_double_flag_get;
*swig_double_flag_set = *LibreDWGc::Dwg_Entity_HATCH_double_flag_set;
*swig_num_deflines_get = *LibreDWGc::Dwg_Entity_HATCH_num_deflines_get;
*swig_num_deflines_set = *LibreDWGc::Dwg_Entity_HATCH_num_deflines_set;
*swig_deflines_get = *LibreDWGc::Dwg_Entity_HATCH_deflines_get;
*swig_deflines_set = *LibreDWGc::Dwg_Entity_HATCH_deflines_set;
*swig_has_derived_get = *LibreDWGc::Dwg_Entity_HATCH_has_derived_get;
*swig_has_derived_set = *LibreDWGc::Dwg_Entity_HATCH_has_derived_set;
*swig_pixel_size_get = *LibreDWGc::Dwg_Entity_HATCH_pixel_size_get;
*swig_pixel_size_set = *LibreDWGc::Dwg_Entity_HATCH_pixel_size_set;
*swig_num_seeds_get = *LibreDWGc::Dwg_Entity_HATCH_num_seeds_get;
*swig_num_seeds_set = *LibreDWGc::Dwg_Entity_HATCH_num_seeds_set;
*swig_seeds_get = *LibreDWGc::Dwg_Entity_HATCH_seeds_get;
*swig_seeds_set = *LibreDWGc::Dwg_Entity_HATCH_seeds_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_HATCH(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_HATCH($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_MPOLYGON ##############

package LibreDWG::Dwg_Entity_MPOLYGON;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_MPOLYGON_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_MPOLYGON_parent_set;
*swig_is_gradient_fill_get = *LibreDWGc::Dwg_Entity_MPOLYGON_is_gradient_fill_get;
*swig_is_gradient_fill_set = *LibreDWGc::Dwg_Entity_MPOLYGON_is_gradient_fill_set;
*swig_reserved_get = *LibreDWGc::Dwg_Entity_MPOLYGON_reserved_get;
*swig_reserved_set = *LibreDWGc::Dwg_Entity_MPOLYGON_reserved_set;
*swig_gradient_angle_get = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_angle_get;
*swig_gradient_angle_set = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_angle_set;
*swig_gradient_shift_get = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_shift_get;
*swig_gradient_shift_set = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_shift_set;
*swig_single_color_gradient_get = *LibreDWGc::Dwg_Entity_MPOLYGON_single_color_gradient_get;
*swig_single_color_gradient_set = *LibreDWGc::Dwg_Entity_MPOLYGON_single_color_gradient_set;
*swig_gradient_tint_get = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_tint_get;
*swig_gradient_tint_set = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_tint_set;
*swig_num_colors_get = *LibreDWGc::Dwg_Entity_MPOLYGON_num_colors_get;
*swig_num_colors_set = *LibreDWGc::Dwg_Entity_MPOLYGON_num_colors_set;
*swig_colors_get = *LibreDWGc::Dwg_Entity_MPOLYGON_colors_get;
*swig_colors_set = *LibreDWGc::Dwg_Entity_MPOLYGON_colors_set;
*swig_gradient_name_get = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_name_get;
*swig_gradient_name_set = *LibreDWGc::Dwg_Entity_MPOLYGON_gradient_name_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_MPOLYGON_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_MPOLYGON_elevation_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_MPOLYGON_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_MPOLYGON_extrusion_set;
*swig_name_get = *LibreDWGc::Dwg_Entity_MPOLYGON_name_get;
*swig_name_set = *LibreDWGc::Dwg_Entity_MPOLYGON_name_set;
*swig_is_solid_fill_get = *LibreDWGc::Dwg_Entity_MPOLYGON_is_solid_fill_get;
*swig_is_solid_fill_set = *LibreDWGc::Dwg_Entity_MPOLYGON_is_solid_fill_set;
*swig_is_associative_get = *LibreDWGc::Dwg_Entity_MPOLYGON_is_associative_get;
*swig_is_associative_set = *LibreDWGc::Dwg_Entity_MPOLYGON_is_associative_set;
*swig_num_paths_get = *LibreDWGc::Dwg_Entity_MPOLYGON_num_paths_get;
*swig_num_paths_set = *LibreDWGc::Dwg_Entity_MPOLYGON_num_paths_set;
*swig_paths_get = *LibreDWGc::Dwg_Entity_MPOLYGON_paths_get;
*swig_paths_set = *LibreDWGc::Dwg_Entity_MPOLYGON_paths_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_MPOLYGON_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_MPOLYGON_style_set;
*swig_pattern_type_get = *LibreDWGc::Dwg_Entity_MPOLYGON_pattern_type_get;
*swig_pattern_type_set = *LibreDWGc::Dwg_Entity_MPOLYGON_pattern_type_set;
*swig_angle_get = *LibreDWGc::Dwg_Entity_MPOLYGON_angle_get;
*swig_angle_set = *LibreDWGc::Dwg_Entity_MPOLYGON_angle_set;
*swig_scale_spacing_get = *LibreDWGc::Dwg_Entity_MPOLYGON_scale_spacing_get;
*swig_scale_spacing_set = *LibreDWGc::Dwg_Entity_MPOLYGON_scale_spacing_set;
*swig_double_flag_get = *LibreDWGc::Dwg_Entity_MPOLYGON_double_flag_get;
*swig_double_flag_set = *LibreDWGc::Dwg_Entity_MPOLYGON_double_flag_set;
*swig_num_deflines_get = *LibreDWGc::Dwg_Entity_MPOLYGON_num_deflines_get;
*swig_num_deflines_set = *LibreDWGc::Dwg_Entity_MPOLYGON_num_deflines_set;
*swig_deflines_get = *LibreDWGc::Dwg_Entity_MPOLYGON_deflines_get;
*swig_deflines_set = *LibreDWGc::Dwg_Entity_MPOLYGON_deflines_set;
*swig_hatch_color_get = *LibreDWGc::Dwg_Entity_MPOLYGON_hatch_color_get;
*swig_hatch_color_set = *LibreDWGc::Dwg_Entity_MPOLYGON_hatch_color_set;
*swig_x_dir_get = *LibreDWGc::Dwg_Entity_MPOLYGON_x_dir_get;
*swig_x_dir_set = *LibreDWGc::Dwg_Entity_MPOLYGON_x_dir_set;
*swig_num_boundary_handles_get = *LibreDWGc::Dwg_Entity_MPOLYGON_num_boundary_handles_get;
*swig_num_boundary_handles_set = *LibreDWGc::Dwg_Entity_MPOLYGON_num_boundary_handles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_MPOLYGON(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_MPOLYGON($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_XRECORD ##############

package LibreDWG::Dwg_Object_XRECORD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_XRECORD_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_XRECORD_parent_set;
*swig_cloning_get = *LibreDWGc::Dwg_Object_XRECORD_cloning_get;
*swig_cloning_set = *LibreDWGc::Dwg_Object_XRECORD_cloning_set;
*swig_xdata_size_get = *LibreDWGc::Dwg_Object_XRECORD_xdata_size_get;
*swig_xdata_size_set = *LibreDWGc::Dwg_Object_XRECORD_xdata_size_set;
*swig_num_xdata_get = *LibreDWGc::Dwg_Object_XRECORD_num_xdata_get;
*swig_num_xdata_set = *LibreDWGc::Dwg_Object_XRECORD_num_xdata_set;
*swig_xdata_get = *LibreDWGc::Dwg_Object_XRECORD_xdata_get;
*swig_xdata_set = *LibreDWGc::Dwg_Object_XRECORD_xdata_set;
*swig_num_objid_handles_get = *LibreDWGc::Dwg_Object_XRECORD_num_objid_handles_get;
*swig_num_objid_handles_set = *LibreDWGc::Dwg_Object_XRECORD_num_objid_handles_set;
*swig_objid_handles_get = *LibreDWGc::Dwg_Object_XRECORD_objid_handles_get;
*swig_objid_handles_set = *LibreDWGc::Dwg_Object_XRECORD_objid_handles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_XRECORD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_XRECORD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_PLACEHOLDER ##############

package LibreDWG::Dwg_Object_PLACEHOLDER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_PLACEHOLDER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_PLACEHOLDER_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_PLACEHOLDER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_PLACEHOLDER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LEADER_Break ##############

package LibreDWG::Dwg_LEADER_Break;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LEADER_Break_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LEADER_Break_parent_set;
*swig_start_get = *LibreDWGc::Dwg_LEADER_Break_start_get;
*swig_start_set = *LibreDWGc::Dwg_LEADER_Break_start_set;
*swig_end_get = *LibreDWGc::Dwg_LEADER_Break_end_get;
*swig_end_set = *LibreDWGc::Dwg_LEADER_Break_end_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LEADER_Break(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LEADER_Break($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LEADER_Line ##############

package LibreDWG::Dwg_LEADER_Line;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LEADER_Line_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LEADER_Line_parent_set;
*swig_num_points_get = *LibreDWGc::Dwg_LEADER_Line_num_points_get;
*swig_num_points_set = *LibreDWGc::Dwg_LEADER_Line_num_points_set;
*swig_points_get = *LibreDWGc::Dwg_LEADER_Line_points_get;
*swig_points_set = *LibreDWGc::Dwg_LEADER_Line_points_set;
*swig_num_breaks_get = *LibreDWGc::Dwg_LEADER_Line_num_breaks_get;
*swig_num_breaks_set = *LibreDWGc::Dwg_LEADER_Line_num_breaks_set;
*swig_breaks_get = *LibreDWGc::Dwg_LEADER_Line_breaks_get;
*swig_breaks_set = *LibreDWGc::Dwg_LEADER_Line_breaks_set;
*swig_line_index_get = *LibreDWGc::Dwg_LEADER_Line_line_index_get;
*swig_line_index_set = *LibreDWGc::Dwg_LEADER_Line_line_index_set;
*swig_type_get = *LibreDWGc::Dwg_LEADER_Line_type_get;
*swig_type_set = *LibreDWGc::Dwg_LEADER_Line_type_set;
*swig_color_get = *LibreDWGc::Dwg_LEADER_Line_color_get;
*swig_color_set = *LibreDWGc::Dwg_LEADER_Line_color_set;
*swig_ltype_get = *LibreDWGc::Dwg_LEADER_Line_ltype_get;
*swig_ltype_set = *LibreDWGc::Dwg_LEADER_Line_ltype_set;
*swig_linewt_get = *LibreDWGc::Dwg_LEADER_Line_linewt_get;
*swig_linewt_set = *LibreDWGc::Dwg_LEADER_Line_linewt_set;
*swig_arrow_size_get = *LibreDWGc::Dwg_LEADER_Line_arrow_size_get;
*swig_arrow_size_set = *LibreDWGc::Dwg_LEADER_Line_arrow_size_set;
*swig_arrow_handle_get = *LibreDWGc::Dwg_LEADER_Line_arrow_handle_get;
*swig_arrow_handle_set = *LibreDWGc::Dwg_LEADER_Line_arrow_handle_set;
*swig_flags_get = *LibreDWGc::Dwg_LEADER_Line_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_LEADER_Line_flags_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LEADER_Line(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LEADER_Line($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LEADER_ArrowHead ##############

package LibreDWG::Dwg_LEADER_ArrowHead;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LEADER_ArrowHead_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LEADER_ArrowHead_parent_set;
*swig_is_default_get = *LibreDWGc::Dwg_LEADER_ArrowHead_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_LEADER_ArrowHead_is_default_set;
*swig_arrowhead_get = *LibreDWGc::Dwg_LEADER_ArrowHead_arrowhead_get;
*swig_arrowhead_set = *LibreDWGc::Dwg_LEADER_ArrowHead_arrowhead_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LEADER_ArrowHead(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LEADER_ArrowHead($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LEADER_BlockLabel ##############

package LibreDWG::Dwg_LEADER_BlockLabel;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LEADER_BlockLabel_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LEADER_BlockLabel_parent_set;
*swig_attdef_get = *LibreDWGc::Dwg_LEADER_BlockLabel_attdef_get;
*swig_attdef_set = *LibreDWGc::Dwg_LEADER_BlockLabel_attdef_set;
*swig_label_text_get = *LibreDWGc::Dwg_LEADER_BlockLabel_label_text_get;
*swig_label_text_set = *LibreDWGc::Dwg_LEADER_BlockLabel_label_text_set;
*swig_ui_index_get = *LibreDWGc::Dwg_LEADER_BlockLabel_ui_index_get;
*swig_ui_index_set = *LibreDWGc::Dwg_LEADER_BlockLabel_ui_index_set;
*swig_width_get = *LibreDWGc::Dwg_LEADER_BlockLabel_width_get;
*swig_width_set = *LibreDWGc::Dwg_LEADER_BlockLabel_width_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LEADER_BlockLabel(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LEADER_BlockLabel($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LEADER_Node ##############

package LibreDWG::Dwg_LEADER_Node;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LEADER_Node_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LEADER_Node_parent_set;
*swig_has_lastleaderlinepoint_get = *LibreDWGc::Dwg_LEADER_Node_has_lastleaderlinepoint_get;
*swig_has_lastleaderlinepoint_set = *LibreDWGc::Dwg_LEADER_Node_has_lastleaderlinepoint_set;
*swig_has_dogleg_get = *LibreDWGc::Dwg_LEADER_Node_has_dogleg_get;
*swig_has_dogleg_set = *LibreDWGc::Dwg_LEADER_Node_has_dogleg_set;
*swig_lastleaderlinepoint_get = *LibreDWGc::Dwg_LEADER_Node_lastleaderlinepoint_get;
*swig_lastleaderlinepoint_set = *LibreDWGc::Dwg_LEADER_Node_lastleaderlinepoint_set;
*swig_dogleg_vector_get = *LibreDWGc::Dwg_LEADER_Node_dogleg_vector_get;
*swig_dogleg_vector_set = *LibreDWGc::Dwg_LEADER_Node_dogleg_vector_set;
*swig_branch_index_get = *LibreDWGc::Dwg_LEADER_Node_branch_index_get;
*swig_branch_index_set = *LibreDWGc::Dwg_LEADER_Node_branch_index_set;
*swig_dogleg_length_get = *LibreDWGc::Dwg_LEADER_Node_dogleg_length_get;
*swig_dogleg_length_set = *LibreDWGc::Dwg_LEADER_Node_dogleg_length_set;
*swig_num_lines_get = *LibreDWGc::Dwg_LEADER_Node_num_lines_get;
*swig_num_lines_set = *LibreDWGc::Dwg_LEADER_Node_num_lines_set;
*swig_lines_get = *LibreDWGc::Dwg_LEADER_Node_lines_get;
*swig_lines_set = *LibreDWGc::Dwg_LEADER_Node_lines_set;
*swig_num_breaks_get = *LibreDWGc::Dwg_LEADER_Node_num_breaks_get;
*swig_num_breaks_set = *LibreDWGc::Dwg_LEADER_Node_num_breaks_set;
*swig_breaks_get = *LibreDWGc::Dwg_LEADER_Node_breaks_get;
*swig_breaks_set = *LibreDWGc::Dwg_LEADER_Node_breaks_set;
*swig_attach_dir_get = *LibreDWGc::Dwg_LEADER_Node_attach_dir_get;
*swig_attach_dir_set = *LibreDWGc::Dwg_LEADER_Node_attach_dir_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LEADER_Node(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LEADER_Node($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MLEADER_Content_MText ##############

package LibreDWG::Dwg_MLEADER_Content_MText;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_type_get = *LibreDWGc::Dwg_MLEADER_Content_MText_type_get;
*swig_type_set = *LibreDWGc::Dwg_MLEADER_Content_MText_type_set;
*swig_normal_get = *LibreDWGc::Dwg_MLEADER_Content_MText_normal_get;
*swig_normal_set = *LibreDWGc::Dwg_MLEADER_Content_MText_normal_set;
*swig_location_get = *LibreDWGc::Dwg_MLEADER_Content_MText_location_get;
*swig_location_set = *LibreDWGc::Dwg_MLEADER_Content_MText_location_set;
*swig_rotation_get = *LibreDWGc::Dwg_MLEADER_Content_MText_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_MLEADER_Content_MText_rotation_set;
*swig_style_get = *LibreDWGc::Dwg_MLEADER_Content_MText_style_get;
*swig_style_set = *LibreDWGc::Dwg_MLEADER_Content_MText_style_set;
*swig_direction_get = *LibreDWGc::Dwg_MLEADER_Content_MText_direction_get;
*swig_direction_set = *LibreDWGc::Dwg_MLEADER_Content_MText_direction_set;
*swig_color_get = *LibreDWGc::Dwg_MLEADER_Content_MText_color_get;
*swig_color_set = *LibreDWGc::Dwg_MLEADER_Content_MText_color_set;
*swig_width_get = *LibreDWGc::Dwg_MLEADER_Content_MText_width_get;
*swig_width_set = *LibreDWGc::Dwg_MLEADER_Content_MText_width_set;
*swig_height_get = *LibreDWGc::Dwg_MLEADER_Content_MText_height_get;
*swig_height_set = *LibreDWGc::Dwg_MLEADER_Content_MText_height_set;
*swig_line_spacing_factor_get = *LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_factor_get;
*swig_line_spacing_factor_set = *LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_factor_set;
*swig_default_text_get = *LibreDWGc::Dwg_MLEADER_Content_MText_default_text_get;
*swig_default_text_set = *LibreDWGc::Dwg_MLEADER_Content_MText_default_text_set;
*swig_line_spacing_style_get = *LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_style_get;
*swig_line_spacing_style_set = *LibreDWGc::Dwg_MLEADER_Content_MText_line_spacing_style_set;
*swig_alignment_get = *LibreDWGc::Dwg_MLEADER_Content_MText_alignment_get;
*swig_alignment_set = *LibreDWGc::Dwg_MLEADER_Content_MText_alignment_set;
*swig_flow_get = *LibreDWGc::Dwg_MLEADER_Content_MText_flow_get;
*swig_flow_set = *LibreDWGc::Dwg_MLEADER_Content_MText_flow_set;
*swig_bg_color_get = *LibreDWGc::Dwg_MLEADER_Content_MText_bg_color_get;
*swig_bg_color_set = *LibreDWGc::Dwg_MLEADER_Content_MText_bg_color_set;
*swig_bg_scale_get = *LibreDWGc::Dwg_MLEADER_Content_MText_bg_scale_get;
*swig_bg_scale_set = *LibreDWGc::Dwg_MLEADER_Content_MText_bg_scale_set;
*swig_bg_transparency_get = *LibreDWGc::Dwg_MLEADER_Content_MText_bg_transparency_get;
*swig_bg_transparency_set = *LibreDWGc::Dwg_MLEADER_Content_MText_bg_transparency_set;
*swig_is_bg_fill_get = *LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_fill_get;
*swig_is_bg_fill_set = *LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_fill_set;
*swig_is_bg_mask_fill_get = *LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_mask_fill_get;
*swig_is_bg_mask_fill_set = *LibreDWGc::Dwg_MLEADER_Content_MText_is_bg_mask_fill_set;
*swig_col_type_get = *LibreDWGc::Dwg_MLEADER_Content_MText_col_type_get;
*swig_col_type_set = *LibreDWGc::Dwg_MLEADER_Content_MText_col_type_set;
*swig_is_height_auto_get = *LibreDWGc::Dwg_MLEADER_Content_MText_is_height_auto_get;
*swig_is_height_auto_set = *LibreDWGc::Dwg_MLEADER_Content_MText_is_height_auto_set;
*swig_col_width_get = *LibreDWGc::Dwg_MLEADER_Content_MText_col_width_get;
*swig_col_width_set = *LibreDWGc::Dwg_MLEADER_Content_MText_col_width_set;
*swig_col_gutter_get = *LibreDWGc::Dwg_MLEADER_Content_MText_col_gutter_get;
*swig_col_gutter_set = *LibreDWGc::Dwg_MLEADER_Content_MText_col_gutter_set;
*swig_is_col_flow_reversed_get = *LibreDWGc::Dwg_MLEADER_Content_MText_is_col_flow_reversed_get;
*swig_is_col_flow_reversed_set = *LibreDWGc::Dwg_MLEADER_Content_MText_is_col_flow_reversed_set;
*swig_num_col_sizes_get = *LibreDWGc::Dwg_MLEADER_Content_MText_num_col_sizes_get;
*swig_num_col_sizes_set = *LibreDWGc::Dwg_MLEADER_Content_MText_num_col_sizes_set;
*swig_col_sizes_get = *LibreDWGc::Dwg_MLEADER_Content_MText_col_sizes_get;
*swig_col_sizes_set = *LibreDWGc::Dwg_MLEADER_Content_MText_col_sizes_set;
*swig_word_break_get = *LibreDWGc::Dwg_MLEADER_Content_MText_word_break_get;
*swig_word_break_set = *LibreDWGc::Dwg_MLEADER_Content_MText_word_break_set;
*swig_unknown_get = *LibreDWGc::Dwg_MLEADER_Content_MText_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_MLEADER_Content_MText_unknown_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MLEADER_Content_MText(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MLEADER_Content_MText($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MLEADER_Content_Block ##############

package LibreDWG::Dwg_MLEADER_Content_Block;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_type_get = *LibreDWGc::Dwg_MLEADER_Content_Block_type_get;
*swig_type_set = *LibreDWGc::Dwg_MLEADER_Content_Block_type_set;
*swig_normal_get = *LibreDWGc::Dwg_MLEADER_Content_Block_normal_get;
*swig_normal_set = *LibreDWGc::Dwg_MLEADER_Content_Block_normal_set;
*swig_location_get = *LibreDWGc::Dwg_MLEADER_Content_Block_location_get;
*swig_location_set = *LibreDWGc::Dwg_MLEADER_Content_Block_location_set;
*swig_rotation_get = *LibreDWGc::Dwg_MLEADER_Content_Block_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_MLEADER_Content_Block_rotation_set;
*swig_block_table_get = *LibreDWGc::Dwg_MLEADER_Content_Block_block_table_get;
*swig_block_table_set = *LibreDWGc::Dwg_MLEADER_Content_Block_block_table_set;
*swig_scale_get = *LibreDWGc::Dwg_MLEADER_Content_Block_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_MLEADER_Content_Block_scale_set;
*swig_color_get = *LibreDWGc::Dwg_MLEADER_Content_Block_color_get;
*swig_color_set = *LibreDWGc::Dwg_MLEADER_Content_Block_color_set;
*swig_transform_get = *LibreDWGc::Dwg_MLEADER_Content_Block_transform_get;
*swig_transform_set = *LibreDWGc::Dwg_MLEADER_Content_Block_transform_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MLEADER_Content_Block(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MLEADER_Content_Block($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MLEADER_Content ##############

package LibreDWG::Dwg_MLEADER_Content;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_txt_get = *LibreDWGc::Dwg_MLEADER_Content_txt_get;
*swig_txt_set = *LibreDWGc::Dwg_MLEADER_Content_txt_set;
*swig_blk_get = *LibreDWGc::Dwg_MLEADER_Content_blk_get;
*swig_blk_set = *LibreDWGc::Dwg_MLEADER_Content_blk_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MLEADER_Content(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MLEADER_Content($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MLEADER_AnnotContext ##############

package LibreDWG::Dwg_MLEADER_AnnotContext;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_leaders_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_num_leaders_get;
*swig_num_leaders_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_num_leaders_set;
*swig_leaders_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_leaders_get;
*swig_leaders_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_leaders_set;
*swig_attach_dir_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_attach_dir_get;
*swig_attach_dir_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_attach_dir_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_scale_factor_set;
*swig_content_base_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_content_base_get;
*swig_content_base_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_content_base_set;
*swig_text_height_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_height_get;
*swig_text_height_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_height_set;
*swig_arrow_size_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_arrow_size_get;
*swig_arrow_size_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_arrow_size_set;
*swig_landing_gap_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_landing_gap_get;
*swig_landing_gap_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_landing_gap_set;
*swig_text_left_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_left_get;
*swig_text_left_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_left_set;
*swig_text_right_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_right_get;
*swig_text_right_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_right_set;
*swig_text_angletype_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_angletype_get;
*swig_text_angletype_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_angletype_set;
*swig_text_alignment_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_alignment_get;
*swig_text_alignment_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_alignment_set;
*swig_has_content_txt_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_txt_get;
*swig_has_content_txt_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_txt_set;
*swig_has_content_blk_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_blk_get;
*swig_has_content_blk_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_has_content_blk_set;
*swig_content_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_content_get;
*swig_content_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_content_set;
*swig_base_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_base_get;
*swig_base_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_base_set;
*swig_base_dir_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_base_dir_get;
*swig_base_dir_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_base_dir_set;
*swig_base_vert_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_base_vert_get;
*swig_base_vert_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_base_vert_set;
*swig_is_normal_reversed_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_is_normal_reversed_get;
*swig_is_normal_reversed_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_is_normal_reversed_set;
*swig_text_top_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_top_get;
*swig_text_top_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_top_set;
*swig_text_bottom_get = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_bottom_get;
*swig_text_bottom_set = *LibreDWGc::Dwg_MLEADER_AnnotContext_text_bottom_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MLEADER_AnnotContext(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MLEADER_AnnotContext($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_MULTILEADER ##############

package LibreDWG::Dwg_Entity_MULTILEADER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_MULTILEADER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_MULTILEADER_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_MULTILEADER_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_MULTILEADER_class_version_set;
*swig_ctx_get = *LibreDWGc::Dwg_Entity_MULTILEADER_ctx_get;
*swig_ctx_set = *LibreDWGc::Dwg_Entity_MULTILEADER_ctx_set;
*swig_mleaderstyle_get = *LibreDWGc::Dwg_Entity_MULTILEADER_mleaderstyle_get;
*swig_mleaderstyle_set = *LibreDWGc::Dwg_Entity_MULTILEADER_mleaderstyle_set;
*swig_flags_get = *LibreDWGc::Dwg_Entity_MULTILEADER_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Entity_MULTILEADER_flags_set;
*swig_type_get = *LibreDWGc::Dwg_Entity_MULTILEADER_type_get;
*swig_type_set = *LibreDWGc::Dwg_Entity_MULTILEADER_type_set;
*swig_line_color_get = *LibreDWGc::Dwg_Entity_MULTILEADER_line_color_get;
*swig_line_color_set = *LibreDWGc::Dwg_Entity_MULTILEADER_line_color_set;
*swig_line_ltype_get = *LibreDWGc::Dwg_Entity_MULTILEADER_line_ltype_get;
*swig_line_ltype_set = *LibreDWGc::Dwg_Entity_MULTILEADER_line_ltype_set;
*swig_line_linewt_get = *LibreDWGc::Dwg_Entity_MULTILEADER_line_linewt_get;
*swig_line_linewt_set = *LibreDWGc::Dwg_Entity_MULTILEADER_line_linewt_set;
*swig_has_landing_get = *LibreDWGc::Dwg_Entity_MULTILEADER_has_landing_get;
*swig_has_landing_set = *LibreDWGc::Dwg_Entity_MULTILEADER_has_landing_set;
*swig_has_dogleg_get = *LibreDWGc::Dwg_Entity_MULTILEADER_has_dogleg_get;
*swig_has_dogleg_set = *LibreDWGc::Dwg_Entity_MULTILEADER_has_dogleg_set;
*swig_landing_dist_get = *LibreDWGc::Dwg_Entity_MULTILEADER_landing_dist_get;
*swig_landing_dist_set = *LibreDWGc::Dwg_Entity_MULTILEADER_landing_dist_set;
*swig_arrow_handle_get = *LibreDWGc::Dwg_Entity_MULTILEADER_arrow_handle_get;
*swig_arrow_handle_set = *LibreDWGc::Dwg_Entity_MULTILEADER_arrow_handle_set;
*swig_arrow_size_get = *LibreDWGc::Dwg_Entity_MULTILEADER_arrow_size_get;
*swig_arrow_size_set = *LibreDWGc::Dwg_Entity_MULTILEADER_arrow_size_set;
*swig_style_content_get = *LibreDWGc::Dwg_Entity_MULTILEADER_style_content_get;
*swig_style_content_set = *LibreDWGc::Dwg_Entity_MULTILEADER_style_content_set;
*swig_text_style_get = *LibreDWGc::Dwg_Entity_MULTILEADER_text_style_get;
*swig_text_style_set = *LibreDWGc::Dwg_Entity_MULTILEADER_text_style_set;
*swig_text_left_get = *LibreDWGc::Dwg_Entity_MULTILEADER_text_left_get;
*swig_text_left_set = *LibreDWGc::Dwg_Entity_MULTILEADER_text_left_set;
*swig_text_right_get = *LibreDWGc::Dwg_Entity_MULTILEADER_text_right_get;
*swig_text_right_set = *LibreDWGc::Dwg_Entity_MULTILEADER_text_right_set;
*swig_text_angletype_get = *LibreDWGc::Dwg_Entity_MULTILEADER_text_angletype_get;
*swig_text_angletype_set = *LibreDWGc::Dwg_Entity_MULTILEADER_text_angletype_set;
*swig_text_alignment_get = *LibreDWGc::Dwg_Entity_MULTILEADER_text_alignment_get;
*swig_text_alignment_set = *LibreDWGc::Dwg_Entity_MULTILEADER_text_alignment_set;
*swig_text_color_get = *LibreDWGc::Dwg_Entity_MULTILEADER_text_color_get;
*swig_text_color_set = *LibreDWGc::Dwg_Entity_MULTILEADER_text_color_set;
*swig_has_text_frame_get = *LibreDWGc::Dwg_Entity_MULTILEADER_has_text_frame_get;
*swig_has_text_frame_set = *LibreDWGc::Dwg_Entity_MULTILEADER_has_text_frame_set;
*swig_block_style_get = *LibreDWGc::Dwg_Entity_MULTILEADER_block_style_get;
*swig_block_style_set = *LibreDWGc::Dwg_Entity_MULTILEADER_block_style_set;
*swig_block_color_get = *LibreDWGc::Dwg_Entity_MULTILEADER_block_color_get;
*swig_block_color_set = *LibreDWGc::Dwg_Entity_MULTILEADER_block_color_set;
*swig_block_scale_get = *LibreDWGc::Dwg_Entity_MULTILEADER_block_scale_get;
*swig_block_scale_set = *LibreDWGc::Dwg_Entity_MULTILEADER_block_scale_set;
*swig_block_rotation_get = *LibreDWGc::Dwg_Entity_MULTILEADER_block_rotation_get;
*swig_block_rotation_set = *LibreDWGc::Dwg_Entity_MULTILEADER_block_rotation_set;
*swig_style_attachment_get = *LibreDWGc::Dwg_Entity_MULTILEADER_style_attachment_get;
*swig_style_attachment_set = *LibreDWGc::Dwg_Entity_MULTILEADER_style_attachment_set;
*swig_is_annotative_get = *LibreDWGc::Dwg_Entity_MULTILEADER_is_annotative_get;
*swig_is_annotative_set = *LibreDWGc::Dwg_Entity_MULTILEADER_is_annotative_set;
*swig_num_arrowheads_get = *LibreDWGc::Dwg_Entity_MULTILEADER_num_arrowheads_get;
*swig_num_arrowheads_set = *LibreDWGc::Dwg_Entity_MULTILEADER_num_arrowheads_set;
*swig_arrowheads_get = *LibreDWGc::Dwg_Entity_MULTILEADER_arrowheads_get;
*swig_arrowheads_set = *LibreDWGc::Dwg_Entity_MULTILEADER_arrowheads_set;
*swig_num_blocklabels_get = *LibreDWGc::Dwg_Entity_MULTILEADER_num_blocklabels_get;
*swig_num_blocklabels_set = *LibreDWGc::Dwg_Entity_MULTILEADER_num_blocklabels_set;
*swig_blocklabels_get = *LibreDWGc::Dwg_Entity_MULTILEADER_blocklabels_get;
*swig_blocklabels_set = *LibreDWGc::Dwg_Entity_MULTILEADER_blocklabels_set;
*swig_is_neg_textdir_get = *LibreDWGc::Dwg_Entity_MULTILEADER_is_neg_textdir_get;
*swig_is_neg_textdir_set = *LibreDWGc::Dwg_Entity_MULTILEADER_is_neg_textdir_set;
*swig_ipe_alignment_get = *LibreDWGc::Dwg_Entity_MULTILEADER_ipe_alignment_get;
*swig_ipe_alignment_set = *LibreDWGc::Dwg_Entity_MULTILEADER_ipe_alignment_set;
*swig_justification_get = *LibreDWGc::Dwg_Entity_MULTILEADER_justification_get;
*swig_justification_set = *LibreDWGc::Dwg_Entity_MULTILEADER_justification_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_Entity_MULTILEADER_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_Entity_MULTILEADER_scale_factor_set;
*swig_attach_dir_get = *LibreDWGc::Dwg_Entity_MULTILEADER_attach_dir_get;
*swig_attach_dir_set = *LibreDWGc::Dwg_Entity_MULTILEADER_attach_dir_set;
*swig_attach_top_get = *LibreDWGc::Dwg_Entity_MULTILEADER_attach_top_get;
*swig_attach_top_set = *LibreDWGc::Dwg_Entity_MULTILEADER_attach_top_set;
*swig_attach_bottom_get = *LibreDWGc::Dwg_Entity_MULTILEADER_attach_bottom_get;
*swig_attach_bottom_set = *LibreDWGc::Dwg_Entity_MULTILEADER_attach_bottom_set;
*swig_is_text_extended_get = *LibreDWGc::Dwg_Entity_MULTILEADER_is_text_extended_get;
*swig_is_text_extended_set = *LibreDWGc::Dwg_Entity_MULTILEADER_is_text_extended_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_MULTILEADER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_MULTILEADER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MLEADERSTYLE ##############

package LibreDWG::Dwg_Object_MLEADERSTYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_class_version_set;
*swig_content_type_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_content_type_get;
*swig_content_type_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_content_type_set;
*swig_mleader_order_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_mleader_order_get;
*swig_mleader_order_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_mleader_order_set;
*swig_leader_order_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_leader_order_get;
*swig_leader_order_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_leader_order_set;
*swig_max_points_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_max_points_get;
*swig_max_points_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_max_points_set;
*swig_first_seg_angle_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_first_seg_angle_get;
*swig_first_seg_angle_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_first_seg_angle_set;
*swig_second_seg_angle_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_second_seg_angle_get;
*swig_second_seg_angle_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_second_seg_angle_set;
*swig_type_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_type_get;
*swig_type_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_type_set;
*swig_line_color_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_line_color_get;
*swig_line_color_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_line_color_set;
*swig_line_type_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_line_type_get;
*swig_line_type_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_line_type_set;
*swig_linewt_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_linewt_get;
*swig_linewt_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_linewt_set;
*swig_has_landing_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_has_landing_get;
*swig_has_landing_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_has_landing_set;
*swig_has_dogleg_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_has_dogleg_get;
*swig_has_dogleg_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_has_dogleg_set;
*swig_landing_gap_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_gap_get;
*swig_landing_gap_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_gap_set;
*swig_landing_dist_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_dist_get;
*swig_landing_dist_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_landing_dist_set;
*swig_description_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_description_set;
*swig_arrow_head_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_get;
*swig_arrow_head_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_set;
*swig_arrow_head_size_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_size_get;
*swig_arrow_head_size_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_arrow_head_size_set;
*swig_text_default_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_default_get;
*swig_text_default_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_default_set;
*swig_text_style_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_style_get;
*swig_text_style_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_style_set;
*swig_attach_left_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_left_get;
*swig_attach_left_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_left_set;
*swig_attach_right_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_right_get;
*swig_attach_right_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_right_set;
*swig_text_angle_type_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_angle_type_get;
*swig_text_angle_type_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_angle_type_set;
*swig_text_align_type_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_align_type_get;
*swig_text_align_type_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_align_type_set;
*swig_text_color_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_color_get;
*swig_text_color_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_color_set;
*swig_text_height_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_height_get;
*swig_text_height_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_height_set;
*swig_has_text_frame_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_has_text_frame_get;
*swig_has_text_frame_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_has_text_frame_set;
*swig_text_always_left_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_always_left_get;
*swig_text_always_left_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_always_left_set;
*swig_align_space_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_align_space_get;
*swig_align_space_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_align_space_set;
*swig_block_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_get;
*swig_block_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_set;
*swig_block_color_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_color_get;
*swig_block_color_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_color_set;
*swig_block_scale_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_scale_get;
*swig_block_scale_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_scale_set;
*swig_use_block_scale_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_scale_get;
*swig_use_block_scale_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_scale_set;
*swig_block_rotation_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_rotation_get;
*swig_block_rotation_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_rotation_set;
*swig_use_block_rotation_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_rotation_get;
*swig_use_block_rotation_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_use_block_rotation_set;
*swig_block_connection_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_connection_get;
*swig_block_connection_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_block_connection_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_scale_set;
*swig_is_changed_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_is_changed_get;
*swig_is_changed_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_is_changed_set;
*swig_is_annotative_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_is_annotative_get;
*swig_is_annotative_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_is_annotative_set;
*swig_break_size_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_break_size_get;
*swig_break_size_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_break_size_set;
*swig_attach_dir_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_dir_get;
*swig_attach_dir_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_dir_set;
*swig_attach_top_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_top_get;
*swig_attach_top_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_top_set;
*swig_attach_bottom_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_bottom_get;
*swig_attach_bottom_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_attach_bottom_set;
*swig_text_extended_get = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_extended_get;
*swig_text_extended_set = *LibreDWGc::Dwg_Object_MLEADERSTYLE_text_extended_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MLEADERSTYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MLEADERSTYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VBA_PROJECT ##############

package LibreDWG::Dwg_Object_VBA_PROJECT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VBA_PROJECT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VBA_PROJECT_parent_set;
*swig_data_size_get = *LibreDWGc::Dwg_Object_VBA_PROJECT_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_Object_VBA_PROJECT_data_size_set;
*swig_data_get = *LibreDWGc::Dwg_Object_VBA_PROJECT_data_get;
*swig_data_set = *LibreDWGc::Dwg_Object_VBA_PROJECT_data_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VBA_PROJECT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VBA_PROJECT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_PLOTSETTINGS ##############

package LibreDWG::Dwg_Object_PLOTSETTINGS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_parent_set;
*swig_printer_cfg_file_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_printer_cfg_file_get;
*swig_printer_cfg_file_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_printer_cfg_file_set;
*swig_paper_size_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_size_get;
*swig_paper_size_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_size_set;
*swig_canonical_media_name_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_canonical_media_name_get;
*swig_canonical_media_name_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_canonical_media_name_set;
*swig_plot_flags_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_flags_get;
*swig_plot_flags_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_flags_set;
*swig_plotview_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_get;
*swig_plotview_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_set;
*swig_plotview_name_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_name_get;
*swig_plotview_name_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plotview_name_set;
*swig_left_margin_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_left_margin_get;
*swig_left_margin_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_left_margin_set;
*swig_bottom_margin_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_bottom_margin_get;
*swig_bottom_margin_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_bottom_margin_set;
*swig_right_margin_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_right_margin_get;
*swig_right_margin_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_right_margin_set;
*swig_top_margin_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_top_margin_get;
*swig_top_margin_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_top_margin_set;
*swig_paper_width_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_width_get;
*swig_paper_width_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_width_set;
*swig_paper_height_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_height_get;
*swig_paper_height_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_height_set;
*swig_plot_origin_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_origin_get;
*swig_plot_origin_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_origin_set;
*swig_plot_window_ll_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ll_get;
*swig_plot_window_ll_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ll_set;
*swig_plot_window_ur_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ur_get;
*swig_plot_window_ur_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_window_ur_set;
*swig_plot_paper_unit_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_paper_unit_get;
*swig_plot_paper_unit_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_paper_unit_set;
*swig_plot_rotation_mode_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_rotation_mode_get;
*swig_plot_rotation_mode_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_rotation_mode_set;
*swig_plot_type_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_type_get;
*swig_plot_type_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_plot_type_set;
*swig_paper_units_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_units_get;
*swig_paper_units_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_units_set;
*swig_drawing_units_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_drawing_units_get;
*swig_drawing_units_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_drawing_units_set;
*swig_stylesheet_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_stylesheet_get;
*swig_stylesheet_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_stylesheet_set;
*swig_std_scale_type_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_type_get;
*swig_std_scale_type_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_type_set;
*swig_std_scale_factor_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_factor_get;
*swig_std_scale_factor_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_std_scale_factor_set;
*swig_paper_image_origin_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_image_origin_get;
*swig_paper_image_origin_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_paper_image_origin_set;
*swig_shadeplot_type_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_type_get;
*swig_shadeplot_type_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_type_set;
*swig_shadeplot_reslevel_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_get;
*swig_shadeplot_reslevel_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_reslevel_set;
*swig_shadeplot_customdpi_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_get;
*swig_shadeplot_customdpi_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_customdpi_set;
*swig_shadeplot_get = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_get;
*swig_shadeplot_set = *LibreDWGc::Dwg_Object_PLOTSETTINGS_shadeplot_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_PLOTSETTINGS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_PLOTSETTINGS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LAYOUT ##############

package LibreDWG::Dwg_Object_LAYOUT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LAYOUT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LAYOUT_parent_set;
*swig_plotsettings_get = *LibreDWGc::Dwg_Object_LAYOUT_plotsettings_get;
*swig_plotsettings_set = *LibreDWGc::Dwg_Object_LAYOUT_plotsettings_set;
*swig_layout_name_get = *LibreDWGc::Dwg_Object_LAYOUT_layout_name_get;
*swig_layout_name_set = *LibreDWGc::Dwg_Object_LAYOUT_layout_name_set;
*swig_tab_order_get = *LibreDWGc::Dwg_Object_LAYOUT_tab_order_get;
*swig_tab_order_set = *LibreDWGc::Dwg_Object_LAYOUT_tab_order_set;
*swig_layout_flags_get = *LibreDWGc::Dwg_Object_LAYOUT_layout_flags_get;
*swig_layout_flags_set = *LibreDWGc::Dwg_Object_LAYOUT_layout_flags_set;
*swig_INSBASE_get = *LibreDWGc::Dwg_Object_LAYOUT_INSBASE_get;
*swig_INSBASE_set = *LibreDWGc::Dwg_Object_LAYOUT_INSBASE_set;
*swig_LIMMIN_get = *LibreDWGc::Dwg_Object_LAYOUT_LIMMIN_get;
*swig_LIMMIN_set = *LibreDWGc::Dwg_Object_LAYOUT_LIMMIN_set;
*swig_LIMMAX_get = *LibreDWGc::Dwg_Object_LAYOUT_LIMMAX_get;
*swig_LIMMAX_set = *LibreDWGc::Dwg_Object_LAYOUT_LIMMAX_set;
*swig_UCSORG_get = *LibreDWGc::Dwg_Object_LAYOUT_UCSORG_get;
*swig_UCSORG_set = *LibreDWGc::Dwg_Object_LAYOUT_UCSORG_set;
*swig_UCSXDIR_get = *LibreDWGc::Dwg_Object_LAYOUT_UCSXDIR_get;
*swig_UCSXDIR_set = *LibreDWGc::Dwg_Object_LAYOUT_UCSXDIR_set;
*swig_UCSYDIR_get = *LibreDWGc::Dwg_Object_LAYOUT_UCSYDIR_get;
*swig_UCSYDIR_set = *LibreDWGc::Dwg_Object_LAYOUT_UCSYDIR_set;
*swig_ucs_elevation_get = *LibreDWGc::Dwg_Object_LAYOUT_ucs_elevation_get;
*swig_ucs_elevation_set = *LibreDWGc::Dwg_Object_LAYOUT_ucs_elevation_set;
*swig_UCSORTHOVIEW_get = *LibreDWGc::Dwg_Object_LAYOUT_UCSORTHOVIEW_get;
*swig_UCSORTHOVIEW_set = *LibreDWGc::Dwg_Object_LAYOUT_UCSORTHOVIEW_set;
*swig_EXTMIN_get = *LibreDWGc::Dwg_Object_LAYOUT_EXTMIN_get;
*swig_EXTMIN_set = *LibreDWGc::Dwg_Object_LAYOUT_EXTMIN_set;
*swig_EXTMAX_get = *LibreDWGc::Dwg_Object_LAYOUT_EXTMAX_get;
*swig_EXTMAX_set = *LibreDWGc::Dwg_Object_LAYOUT_EXTMAX_set;
*swig_block_header_get = *LibreDWGc::Dwg_Object_LAYOUT_block_header_get;
*swig_block_header_set = *LibreDWGc::Dwg_Object_LAYOUT_block_header_set;
*swig_active_viewport_get = *LibreDWGc::Dwg_Object_LAYOUT_active_viewport_get;
*swig_active_viewport_set = *LibreDWGc::Dwg_Object_LAYOUT_active_viewport_set;
*swig_base_ucs_get = *LibreDWGc::Dwg_Object_LAYOUT_base_ucs_get;
*swig_base_ucs_set = *LibreDWGc::Dwg_Object_LAYOUT_base_ucs_set;
*swig_named_ucs_get = *LibreDWGc::Dwg_Object_LAYOUT_named_ucs_get;
*swig_named_ucs_set = *LibreDWGc::Dwg_Object_LAYOUT_named_ucs_set;
*swig_num_viewports_get = *LibreDWGc::Dwg_Object_LAYOUT_num_viewports_get;
*swig_num_viewports_set = *LibreDWGc::Dwg_Object_LAYOUT_num_viewports_set;
*swig_viewports_get = *LibreDWGc::Dwg_Object_LAYOUT_viewports_get;
*swig_viewports_set = *LibreDWGc::Dwg_Object_LAYOUT_viewports_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LAYOUT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LAYOUT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DICTIONARYVAR ##############

package LibreDWG::Dwg_Object_DICTIONARYVAR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DICTIONARYVAR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DICTIONARYVAR_parent_set;
*swig_schema_get = *LibreDWGc::Dwg_Object_DICTIONARYVAR_schema_get;
*swig_schema_set = *LibreDWGc::Dwg_Object_DICTIONARYVAR_schema_set;
*swig_strvalue_get = *LibreDWGc::Dwg_Object_DICTIONARYVAR_strvalue_get;
*swig_strvalue_set = *LibreDWGc::Dwg_Object_DICTIONARYVAR_strvalue_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DICTIONARYVAR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DICTIONARYVAR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLE_value ##############

package LibreDWG::Dwg_TABLE_value;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_flags_get = *LibreDWGc::Dwg_TABLE_value_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_TABLE_value_flags_set;
*swig_format_flags_get = *LibreDWGc::Dwg_TABLE_value_format_flags_get;
*swig_format_flags_set = *LibreDWGc::Dwg_TABLE_value_format_flags_set;
*swig_data_type_get = *LibreDWGc::Dwg_TABLE_value_data_type_get;
*swig_data_type_set = *LibreDWGc::Dwg_TABLE_value_data_type_set;
*swig_data_size_get = *LibreDWGc::Dwg_TABLE_value_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_TABLE_value_data_size_set;
*swig_data_long_get = *LibreDWGc::Dwg_TABLE_value_data_long_get;
*swig_data_long_set = *LibreDWGc::Dwg_TABLE_value_data_long_set;
*swig_data_double_get = *LibreDWGc::Dwg_TABLE_value_data_double_get;
*swig_data_double_set = *LibreDWGc::Dwg_TABLE_value_data_double_set;
*swig_data_string_get = *LibreDWGc::Dwg_TABLE_value_data_string_get;
*swig_data_string_set = *LibreDWGc::Dwg_TABLE_value_data_string_set;
*swig_data_date_get = *LibreDWGc::Dwg_TABLE_value_data_date_get;
*swig_data_date_set = *LibreDWGc::Dwg_TABLE_value_data_date_set;
*swig_data_point_get = *LibreDWGc::Dwg_TABLE_value_data_point_get;
*swig_data_point_set = *LibreDWGc::Dwg_TABLE_value_data_point_set;
*swig_data_3dpoint_get = *LibreDWGc::Dwg_TABLE_value_data_3dpoint_get;
*swig_data_3dpoint_set = *LibreDWGc::Dwg_TABLE_value_data_3dpoint_set;
*swig_data_handle_get = *LibreDWGc::Dwg_TABLE_value_data_handle_get;
*swig_data_handle_set = *LibreDWGc::Dwg_TABLE_value_data_handle_set;
*swig_unit_type_get = *LibreDWGc::Dwg_TABLE_value_unit_type_get;
*swig_unit_type_set = *LibreDWGc::Dwg_TABLE_value_unit_type_set;
*swig_format_string_get = *LibreDWGc::Dwg_TABLE_value_format_string_get;
*swig_format_string_set = *LibreDWGc::Dwg_TABLE_value_format_string_set;
*swig_value_string_get = *LibreDWGc::Dwg_TABLE_value_value_string_get;
*swig_value_string_set = *LibreDWGc::Dwg_TABLE_value_value_string_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLE_value(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLE_value($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLE_CustomDataItem ##############

package LibreDWG::Dwg_TABLE_CustomDataItem;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_name_get = *LibreDWGc::Dwg_TABLE_CustomDataItem_name_get;
*swig_name_set = *LibreDWGc::Dwg_TABLE_CustomDataItem_name_set;
*swig_value_get = *LibreDWGc::Dwg_TABLE_CustomDataItem_value_get;
*swig_value_set = *LibreDWGc::Dwg_TABLE_CustomDataItem_value_set;
*swig_cell_parent_get = *LibreDWGc::Dwg_TABLE_CustomDataItem_cell_parent_get;
*swig_cell_parent_set = *LibreDWGc::Dwg_TABLE_CustomDataItem_cell_parent_set;
*swig_row_parent_get = *LibreDWGc::Dwg_TABLE_CustomDataItem_row_parent_get;
*swig_row_parent_set = *LibreDWGc::Dwg_TABLE_CustomDataItem_row_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLE_CustomDataItem(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLE_CustomDataItem($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLE_AttrDef ##############

package LibreDWG::Dwg_TABLE_AttrDef;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TABLE_AttrDef_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TABLE_AttrDef_parent_set;
*swig_attdef_get = *LibreDWGc::Dwg_TABLE_AttrDef_attdef_get;
*swig_attdef_set = *LibreDWGc::Dwg_TABLE_AttrDef_attdef_set;
*swig_index_get = *LibreDWGc::Dwg_TABLE_AttrDef_index_get;
*swig_index_set = *LibreDWGc::Dwg_TABLE_AttrDef_index_set;
*swig_text_get = *LibreDWGc::Dwg_TABLE_AttrDef_text_get;
*swig_text_set = *LibreDWGc::Dwg_TABLE_AttrDef_text_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLE_AttrDef(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLE_AttrDef($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLE_Cell ##############

package LibreDWG::Dwg_TABLE_Cell;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TABLE_Cell_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TABLE_Cell_parent_set;
*swig_type_get = *LibreDWGc::Dwg_TABLE_Cell_type_get;
*swig_type_set = *LibreDWGc::Dwg_TABLE_Cell_type_set;
*swig_flags_get = *LibreDWGc::Dwg_TABLE_Cell_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_TABLE_Cell_flags_set;
*swig_is_merged_value_get = *LibreDWGc::Dwg_TABLE_Cell_is_merged_value_get;
*swig_is_merged_value_set = *LibreDWGc::Dwg_TABLE_Cell_is_merged_value_set;
*swig_is_autofit_flag_get = *LibreDWGc::Dwg_TABLE_Cell_is_autofit_flag_get;
*swig_is_autofit_flag_set = *LibreDWGc::Dwg_TABLE_Cell_is_autofit_flag_set;
*swig_merged_width_flag_get = *LibreDWGc::Dwg_TABLE_Cell_merged_width_flag_get;
*swig_merged_width_flag_set = *LibreDWGc::Dwg_TABLE_Cell_merged_width_flag_set;
*swig_merged_height_flag_get = *LibreDWGc::Dwg_TABLE_Cell_merged_height_flag_get;
*swig_merged_height_flag_set = *LibreDWGc::Dwg_TABLE_Cell_merged_height_flag_set;
*swig_rotation_get = *LibreDWGc::Dwg_TABLE_Cell_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_TABLE_Cell_rotation_set;
*swig_text_value_get = *LibreDWGc::Dwg_TABLE_Cell_text_value_get;
*swig_text_value_set = *LibreDWGc::Dwg_TABLE_Cell_text_value_set;
*swig_text_style_get = *LibreDWGc::Dwg_TABLE_Cell_text_style_get;
*swig_text_style_set = *LibreDWGc::Dwg_TABLE_Cell_text_style_set;
*swig_block_handle_get = *LibreDWGc::Dwg_TABLE_Cell_block_handle_get;
*swig_block_handle_set = *LibreDWGc::Dwg_TABLE_Cell_block_handle_set;
*swig_block_scale_get = *LibreDWGc::Dwg_TABLE_Cell_block_scale_get;
*swig_block_scale_set = *LibreDWGc::Dwg_TABLE_Cell_block_scale_set;
*swig_additional_data_flag_get = *LibreDWGc::Dwg_TABLE_Cell_additional_data_flag_get;
*swig_additional_data_flag_set = *LibreDWGc::Dwg_TABLE_Cell_additional_data_flag_set;
*swig_cell_flag_override_get = *LibreDWGc::Dwg_TABLE_Cell_cell_flag_override_get;
*swig_cell_flag_override_set = *LibreDWGc::Dwg_TABLE_Cell_cell_flag_override_set;
*swig_virtual_edge_flag_get = *LibreDWGc::Dwg_TABLE_Cell_virtual_edge_flag_get;
*swig_virtual_edge_flag_set = *LibreDWGc::Dwg_TABLE_Cell_virtual_edge_flag_set;
*swig_cell_alignment_get = *LibreDWGc::Dwg_TABLE_Cell_cell_alignment_get;
*swig_cell_alignment_set = *LibreDWGc::Dwg_TABLE_Cell_cell_alignment_set;
*swig_bg_fill_none_get = *LibreDWGc::Dwg_TABLE_Cell_bg_fill_none_get;
*swig_bg_fill_none_set = *LibreDWGc::Dwg_TABLE_Cell_bg_fill_none_set;
*swig_bg_color_get = *LibreDWGc::Dwg_TABLE_Cell_bg_color_get;
*swig_bg_color_set = *LibreDWGc::Dwg_TABLE_Cell_bg_color_set;
*swig_content_color_get = *LibreDWGc::Dwg_TABLE_Cell_content_color_get;
*swig_content_color_set = *LibreDWGc::Dwg_TABLE_Cell_content_color_set;
*swig_text_height_get = *LibreDWGc::Dwg_TABLE_Cell_text_height_get;
*swig_text_height_set = *LibreDWGc::Dwg_TABLE_Cell_text_height_set;
*swig_top_grid_color_get = *LibreDWGc::Dwg_TABLE_Cell_top_grid_color_get;
*swig_top_grid_color_set = *LibreDWGc::Dwg_TABLE_Cell_top_grid_color_set;
*swig_top_grid_linewt_get = *LibreDWGc::Dwg_TABLE_Cell_top_grid_linewt_get;
*swig_top_grid_linewt_set = *LibreDWGc::Dwg_TABLE_Cell_top_grid_linewt_set;
*swig_top_visibility_get = *LibreDWGc::Dwg_TABLE_Cell_top_visibility_get;
*swig_top_visibility_set = *LibreDWGc::Dwg_TABLE_Cell_top_visibility_set;
*swig_right_grid_color_get = *LibreDWGc::Dwg_TABLE_Cell_right_grid_color_get;
*swig_right_grid_color_set = *LibreDWGc::Dwg_TABLE_Cell_right_grid_color_set;
*swig_right_grid_linewt_get = *LibreDWGc::Dwg_TABLE_Cell_right_grid_linewt_get;
*swig_right_grid_linewt_set = *LibreDWGc::Dwg_TABLE_Cell_right_grid_linewt_set;
*swig_right_visibility_get = *LibreDWGc::Dwg_TABLE_Cell_right_visibility_get;
*swig_right_visibility_set = *LibreDWGc::Dwg_TABLE_Cell_right_visibility_set;
*swig_bottom_grid_color_get = *LibreDWGc::Dwg_TABLE_Cell_bottom_grid_color_get;
*swig_bottom_grid_color_set = *LibreDWGc::Dwg_TABLE_Cell_bottom_grid_color_set;
*swig_bottom_grid_linewt_get = *LibreDWGc::Dwg_TABLE_Cell_bottom_grid_linewt_get;
*swig_bottom_grid_linewt_set = *LibreDWGc::Dwg_TABLE_Cell_bottom_grid_linewt_set;
*swig_bottom_visibility_get = *LibreDWGc::Dwg_TABLE_Cell_bottom_visibility_get;
*swig_bottom_visibility_set = *LibreDWGc::Dwg_TABLE_Cell_bottom_visibility_set;
*swig_left_grid_color_get = *LibreDWGc::Dwg_TABLE_Cell_left_grid_color_get;
*swig_left_grid_color_set = *LibreDWGc::Dwg_TABLE_Cell_left_grid_color_set;
*swig_left_grid_linewt_get = *LibreDWGc::Dwg_TABLE_Cell_left_grid_linewt_get;
*swig_left_grid_linewt_set = *LibreDWGc::Dwg_TABLE_Cell_left_grid_linewt_set;
*swig_left_visibility_get = *LibreDWGc::Dwg_TABLE_Cell_left_visibility_get;
*swig_left_visibility_set = *LibreDWGc::Dwg_TABLE_Cell_left_visibility_set;
*swig_unknown_get = *LibreDWGc::Dwg_TABLE_Cell_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_TABLE_Cell_unknown_set;
*swig_value_get = *LibreDWGc::Dwg_TABLE_Cell_value_get;
*swig_value_set = *LibreDWGc::Dwg_TABLE_Cell_value_set;
*swig_num_attr_defs_get = *LibreDWGc::Dwg_TABLE_Cell_num_attr_defs_get;
*swig_num_attr_defs_set = *LibreDWGc::Dwg_TABLE_Cell_num_attr_defs_set;
*swig_attr_defs_get = *LibreDWGc::Dwg_TABLE_Cell_attr_defs_get;
*swig_attr_defs_set = *LibreDWGc::Dwg_TABLE_Cell_attr_defs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLE_Cell(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLE_Cell($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLE_BreakHeight ##############

package LibreDWG::Dwg_TABLE_BreakHeight;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TABLE_BreakHeight_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TABLE_BreakHeight_parent_set;
*swig_position_get = *LibreDWGc::Dwg_TABLE_BreakHeight_position_get;
*swig_position_set = *LibreDWGc::Dwg_TABLE_BreakHeight_position_set;
*swig_height_get = *LibreDWGc::Dwg_TABLE_BreakHeight_height_get;
*swig_height_set = *LibreDWGc::Dwg_TABLE_BreakHeight_height_set;
*swig_flag_get = *LibreDWGc::Dwg_TABLE_BreakHeight_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_TABLE_BreakHeight_flag_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLE_BreakHeight(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLE_BreakHeight($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLE_BreakRow ##############

package LibreDWG::Dwg_TABLE_BreakRow;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TABLE_BreakRow_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TABLE_BreakRow_parent_set;
*swig_position_get = *LibreDWGc::Dwg_TABLE_BreakRow_position_get;
*swig_position_set = *LibreDWGc::Dwg_TABLE_BreakRow_position_set;
*swig_start_get = *LibreDWGc::Dwg_TABLE_BreakRow_start_get;
*swig_start_set = *LibreDWGc::Dwg_TABLE_BreakRow_start_set;
*swig_end_get = *LibreDWGc::Dwg_TABLE_BreakRow_end_get;
*swig_end_set = *LibreDWGc::Dwg_TABLE_BreakRow_end_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLE_BreakRow(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLE_BreakRow($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LinkedData ##############

package LibreDWG::Dwg_LinkedData;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_name_get = *LibreDWGc::Dwg_LinkedData_name_get;
*swig_name_set = *LibreDWGc::Dwg_LinkedData_name_set;
*swig_description_get = *LibreDWGc::Dwg_LinkedData_description_get;
*swig_description_set = *LibreDWGc::Dwg_LinkedData_description_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LinkedData(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LinkedData($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TableCellContent_Attr ##############

package LibreDWG::Dwg_TableCellContent_Attr;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TableCellContent_Attr_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TableCellContent_Attr_parent_set;
*swig_attdef_get = *LibreDWGc::Dwg_TableCellContent_Attr_attdef_get;
*swig_attdef_set = *LibreDWGc::Dwg_TableCellContent_Attr_attdef_set;
*swig_value_get = *LibreDWGc::Dwg_TableCellContent_Attr_value_get;
*swig_value_set = *LibreDWGc::Dwg_TableCellContent_Attr_value_set;
*swig_index_get = *LibreDWGc::Dwg_TableCellContent_Attr_index_get;
*swig_index_set = *LibreDWGc::Dwg_TableCellContent_Attr_index_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TableCellContent_Attr(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TableCellContent_Attr($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ContentFormat ##############

package LibreDWG::Dwg_ContentFormat;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_property_override_flags_get = *LibreDWGc::Dwg_ContentFormat_property_override_flags_get;
*swig_property_override_flags_set = *LibreDWGc::Dwg_ContentFormat_property_override_flags_set;
*swig_property_flags_get = *LibreDWGc::Dwg_ContentFormat_property_flags_get;
*swig_property_flags_set = *LibreDWGc::Dwg_ContentFormat_property_flags_set;
*swig_value_data_type_get = *LibreDWGc::Dwg_ContentFormat_value_data_type_get;
*swig_value_data_type_set = *LibreDWGc::Dwg_ContentFormat_value_data_type_set;
*swig_value_unit_type_get = *LibreDWGc::Dwg_ContentFormat_value_unit_type_get;
*swig_value_unit_type_set = *LibreDWGc::Dwg_ContentFormat_value_unit_type_set;
*swig_value_format_string_get = *LibreDWGc::Dwg_ContentFormat_value_format_string_get;
*swig_value_format_string_set = *LibreDWGc::Dwg_ContentFormat_value_format_string_set;
*swig_rotation_get = *LibreDWGc::Dwg_ContentFormat_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_ContentFormat_rotation_set;
*swig_block_scale_get = *LibreDWGc::Dwg_ContentFormat_block_scale_get;
*swig_block_scale_set = *LibreDWGc::Dwg_ContentFormat_block_scale_set;
*swig_cell_alignment_get = *LibreDWGc::Dwg_ContentFormat_cell_alignment_get;
*swig_cell_alignment_set = *LibreDWGc::Dwg_ContentFormat_cell_alignment_set;
*swig_content_color_get = *LibreDWGc::Dwg_ContentFormat_content_color_get;
*swig_content_color_set = *LibreDWGc::Dwg_ContentFormat_content_color_set;
*swig_text_style_get = *LibreDWGc::Dwg_ContentFormat_text_style_get;
*swig_text_style_set = *LibreDWGc::Dwg_ContentFormat_text_style_set;
*swig_text_height_get = *LibreDWGc::Dwg_ContentFormat_text_height_get;
*swig_text_height_set = *LibreDWGc::Dwg_ContentFormat_text_height_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ContentFormat(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ContentFormat($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TableCellContent ##############

package LibreDWG::Dwg_TableCellContent;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TableCellContent_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TableCellContent_parent_set;
*swig_type_get = *LibreDWGc::Dwg_TableCellContent_type_get;
*swig_type_set = *LibreDWGc::Dwg_TableCellContent_type_set;
*swig_value_get = *LibreDWGc::Dwg_TableCellContent_value_get;
*swig_value_set = *LibreDWGc::Dwg_TableCellContent_value_set;
*swig_handle_get = *LibreDWGc::Dwg_TableCellContent_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_TableCellContent_handle_set;
*swig_num_attrs_get = *LibreDWGc::Dwg_TableCellContent_num_attrs_get;
*swig_num_attrs_set = *LibreDWGc::Dwg_TableCellContent_num_attrs_set;
*swig_attrs_get = *LibreDWGc::Dwg_TableCellContent_attrs_get;
*swig_attrs_set = *LibreDWGc::Dwg_TableCellContent_attrs_set;
*swig_has_content_format_overrides_get = *LibreDWGc::Dwg_TableCellContent_has_content_format_overrides_get;
*swig_has_content_format_overrides_set = *LibreDWGc::Dwg_TableCellContent_has_content_format_overrides_set;
*swig_content_format_get = *LibreDWGc::Dwg_TableCellContent_content_format_get;
*swig_content_format_set = *LibreDWGc::Dwg_TableCellContent_content_format_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TableCellContent(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TableCellContent($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_CellContentGeometry ##############

package LibreDWG::Dwg_CellContentGeometry;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_dist_top_left_get = *LibreDWGc::Dwg_CellContentGeometry_dist_top_left_get;
*swig_dist_top_left_set = *LibreDWGc::Dwg_CellContentGeometry_dist_top_left_set;
*swig_dist_center_get = *LibreDWGc::Dwg_CellContentGeometry_dist_center_get;
*swig_dist_center_set = *LibreDWGc::Dwg_CellContentGeometry_dist_center_set;
*swig_content_width_get = *LibreDWGc::Dwg_CellContentGeometry_content_width_get;
*swig_content_width_set = *LibreDWGc::Dwg_CellContentGeometry_content_width_set;
*swig_content_height_get = *LibreDWGc::Dwg_CellContentGeometry_content_height_get;
*swig_content_height_set = *LibreDWGc::Dwg_CellContentGeometry_content_height_set;
*swig_width_get = *LibreDWGc::Dwg_CellContentGeometry_width_get;
*swig_width_set = *LibreDWGc::Dwg_CellContentGeometry_width_set;
*swig_height_get = *LibreDWGc::Dwg_CellContentGeometry_height_get;
*swig_height_set = *LibreDWGc::Dwg_CellContentGeometry_height_set;
*swig_unknown_get = *LibreDWGc::Dwg_CellContentGeometry_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_CellContentGeometry_unknown_set;
*swig_cell_parent_get = *LibreDWGc::Dwg_CellContentGeometry_cell_parent_get;
*swig_cell_parent_set = *LibreDWGc::Dwg_CellContentGeometry_cell_parent_set;
*swig_geom_parent_get = *LibreDWGc::Dwg_CellContentGeometry_geom_parent_get;
*swig_geom_parent_set = *LibreDWGc::Dwg_CellContentGeometry_geom_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_CellContentGeometry(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_CellContentGeometry($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TableCell ##############

package LibreDWG::Dwg_TableCell;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_flag_get = *LibreDWGc::Dwg_TableCell_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_TableCell_flag_set;
*swig_tooltip_get = *LibreDWGc::Dwg_TableCell_tooltip_get;
*swig_tooltip_set = *LibreDWGc::Dwg_TableCell_tooltip_set;
*swig_customdata_get = *LibreDWGc::Dwg_TableCell_customdata_get;
*swig_customdata_set = *LibreDWGc::Dwg_TableCell_customdata_set;
*swig_num_customdata_items_get = *LibreDWGc::Dwg_TableCell_num_customdata_items_get;
*swig_num_customdata_items_set = *LibreDWGc::Dwg_TableCell_num_customdata_items_set;
*swig_customdata_items_get = *LibreDWGc::Dwg_TableCell_customdata_items_get;
*swig_customdata_items_set = *LibreDWGc::Dwg_TableCell_customdata_items_set;
*swig_has_linked_data_get = *LibreDWGc::Dwg_TableCell_has_linked_data_get;
*swig_has_linked_data_set = *LibreDWGc::Dwg_TableCell_has_linked_data_set;
*swig_data_link_get = *LibreDWGc::Dwg_TableCell_data_link_get;
*swig_data_link_set = *LibreDWGc::Dwg_TableCell_data_link_set;
*swig_num_rows_get = *LibreDWGc::Dwg_TableCell_num_rows_get;
*swig_num_rows_set = *LibreDWGc::Dwg_TableCell_num_rows_set;
*swig_num_cols_get = *LibreDWGc::Dwg_TableCell_num_cols_get;
*swig_num_cols_set = *LibreDWGc::Dwg_TableCell_num_cols_set;
*swig_unknown_get = *LibreDWGc::Dwg_TableCell_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_TableCell_unknown_set;
*swig_num_cell_contents_get = *LibreDWGc::Dwg_TableCell_num_cell_contents_get;
*swig_num_cell_contents_set = *LibreDWGc::Dwg_TableCell_num_cell_contents_set;
*swig_cell_contents_get = *LibreDWGc::Dwg_TableCell_cell_contents_get;
*swig_cell_contents_set = *LibreDWGc::Dwg_TableCell_cell_contents_set;
*swig_style_id_get = *LibreDWGc::Dwg_TableCell_style_id_get;
*swig_style_id_set = *LibreDWGc::Dwg_TableCell_style_id_set;
*swig_has_geom_data_get = *LibreDWGc::Dwg_TableCell_has_geom_data_get;
*swig_has_geom_data_set = *LibreDWGc::Dwg_TableCell_has_geom_data_set;
*swig_geom_data_flag_get = *LibreDWGc::Dwg_TableCell_geom_data_flag_get;
*swig_geom_data_flag_set = *LibreDWGc::Dwg_TableCell_geom_data_flag_set;
*swig_width_w_gap_get = *LibreDWGc::Dwg_TableCell_width_w_gap_get;
*swig_width_w_gap_set = *LibreDWGc::Dwg_TableCell_width_w_gap_set;
*swig_height_w_gap_get = *LibreDWGc::Dwg_TableCell_height_w_gap_get;
*swig_height_w_gap_set = *LibreDWGc::Dwg_TableCell_height_w_gap_set;
*swig_tablegeometry_get = *LibreDWGc::Dwg_TableCell_tablegeometry_get;
*swig_tablegeometry_set = *LibreDWGc::Dwg_TableCell_tablegeometry_set;
*swig_num_geometry_get = *LibreDWGc::Dwg_TableCell_num_geometry_get;
*swig_num_geometry_set = *LibreDWGc::Dwg_TableCell_num_geometry_set;
*swig_geometry_get = *LibreDWGc::Dwg_TableCell_geometry_get;
*swig_geometry_set = *LibreDWGc::Dwg_TableCell_geometry_set;
*swig_style_parent_get = *LibreDWGc::Dwg_TableCell_style_parent_get;
*swig_style_parent_set = *LibreDWGc::Dwg_TableCell_style_parent_set;
*swig_row_parent_get = *LibreDWGc::Dwg_TableCell_row_parent_get;
*swig_row_parent_set = *LibreDWGc::Dwg_TableCell_row_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TableCell(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TableCell($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_GridFormat ##############

package LibreDWG::Dwg_GridFormat;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_GridFormat_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_GridFormat_parent_set;
*swig_index_mask_get = *LibreDWGc::Dwg_GridFormat_index_mask_get;
*swig_index_mask_set = *LibreDWGc::Dwg_GridFormat_index_mask_set;
*swig_border_overrides_get = *LibreDWGc::Dwg_GridFormat_border_overrides_get;
*swig_border_overrides_set = *LibreDWGc::Dwg_GridFormat_border_overrides_set;
*swig_border_type_get = *LibreDWGc::Dwg_GridFormat_border_type_get;
*swig_border_type_set = *LibreDWGc::Dwg_GridFormat_border_type_set;
*swig_color_get = *LibreDWGc::Dwg_GridFormat_color_get;
*swig_color_set = *LibreDWGc::Dwg_GridFormat_color_set;
*swig_linewt_get = *LibreDWGc::Dwg_GridFormat_linewt_get;
*swig_linewt_set = *LibreDWGc::Dwg_GridFormat_linewt_set;
*swig_ltype_get = *LibreDWGc::Dwg_GridFormat_ltype_get;
*swig_ltype_set = *LibreDWGc::Dwg_GridFormat_ltype_set;
*swig_visible_get = *LibreDWGc::Dwg_GridFormat_visible_get;
*swig_visible_set = *LibreDWGc::Dwg_GridFormat_visible_set;
*swig_double_line_spacing_get = *LibreDWGc::Dwg_GridFormat_double_line_spacing_get;
*swig_double_line_spacing_set = *LibreDWGc::Dwg_GridFormat_double_line_spacing_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_GridFormat(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_GridFormat($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_CellStyle ##############

package LibreDWG::Dwg_CellStyle;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_type_get = *LibreDWGc::Dwg_CellStyle_type_get;
*swig_type_set = *LibreDWGc::Dwg_CellStyle_type_set;
*swig_data_flags_get = *LibreDWGc::Dwg_CellStyle_data_flags_get;
*swig_data_flags_set = *LibreDWGc::Dwg_CellStyle_data_flags_set;
*swig_property_override_flags_get = *LibreDWGc::Dwg_CellStyle_property_override_flags_get;
*swig_property_override_flags_set = *LibreDWGc::Dwg_CellStyle_property_override_flags_set;
*swig_merge_flags_get = *LibreDWGc::Dwg_CellStyle_merge_flags_get;
*swig_merge_flags_set = *LibreDWGc::Dwg_CellStyle_merge_flags_set;
*swig_bg_color_get = *LibreDWGc::Dwg_CellStyle_bg_color_get;
*swig_bg_color_set = *LibreDWGc::Dwg_CellStyle_bg_color_set;
*swig_content_layout_get = *LibreDWGc::Dwg_CellStyle_content_layout_get;
*swig_content_layout_set = *LibreDWGc::Dwg_CellStyle_content_layout_set;
*swig_content_format_get = *LibreDWGc::Dwg_CellStyle_content_format_get;
*swig_content_format_set = *LibreDWGc::Dwg_CellStyle_content_format_set;
*swig_margin_override_flags_get = *LibreDWGc::Dwg_CellStyle_margin_override_flags_get;
*swig_margin_override_flags_set = *LibreDWGc::Dwg_CellStyle_margin_override_flags_set;
*swig_vert_margin_get = *LibreDWGc::Dwg_CellStyle_vert_margin_get;
*swig_vert_margin_set = *LibreDWGc::Dwg_CellStyle_vert_margin_set;
*swig_horiz_margin_get = *LibreDWGc::Dwg_CellStyle_horiz_margin_get;
*swig_horiz_margin_set = *LibreDWGc::Dwg_CellStyle_horiz_margin_set;
*swig_bottom_margin_get = *LibreDWGc::Dwg_CellStyle_bottom_margin_get;
*swig_bottom_margin_set = *LibreDWGc::Dwg_CellStyle_bottom_margin_set;
*swig_right_margin_get = *LibreDWGc::Dwg_CellStyle_right_margin_get;
*swig_right_margin_set = *LibreDWGc::Dwg_CellStyle_right_margin_set;
*swig_margin_horiz_spacing_get = *LibreDWGc::Dwg_CellStyle_margin_horiz_spacing_get;
*swig_margin_horiz_spacing_set = *LibreDWGc::Dwg_CellStyle_margin_horiz_spacing_set;
*swig_margin_vert_spacing_get = *LibreDWGc::Dwg_CellStyle_margin_vert_spacing_get;
*swig_margin_vert_spacing_set = *LibreDWGc::Dwg_CellStyle_margin_vert_spacing_set;
*swig_num_borders_get = *LibreDWGc::Dwg_CellStyle_num_borders_get;
*swig_num_borders_set = *LibreDWGc::Dwg_CellStyle_num_borders_set;
*swig_borders_get = *LibreDWGc::Dwg_CellStyle_borders_get;
*swig_borders_set = *LibreDWGc::Dwg_CellStyle_borders_set;
*swig_tablerow_parent_get = *LibreDWGc::Dwg_CellStyle_tablerow_parent_get;
*swig_tablerow_parent_set = *LibreDWGc::Dwg_CellStyle_tablerow_parent_set;
*swig_tabledatacolumn_parent_get = *LibreDWGc::Dwg_CellStyle_tabledatacolumn_parent_get;
*swig_tabledatacolumn_parent_set = *LibreDWGc::Dwg_CellStyle_tabledatacolumn_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_CellStyle(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_CellStyle($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TableRow ##############

package LibreDWG::Dwg_TableRow;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TableRow_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TableRow_parent_set;
*swig_num_cells_get = *LibreDWGc::Dwg_TableRow_num_cells_get;
*swig_num_cells_set = *LibreDWGc::Dwg_TableRow_num_cells_set;
*swig_cells_get = *LibreDWGc::Dwg_TableRow_cells_get;
*swig_cells_set = *LibreDWGc::Dwg_TableRow_cells_set;
*swig_custom_data_get = *LibreDWGc::Dwg_TableRow_custom_data_get;
*swig_custom_data_set = *LibreDWGc::Dwg_TableRow_custom_data_set;
*swig_num_customdata_items_get = *LibreDWGc::Dwg_TableRow_num_customdata_items_get;
*swig_num_customdata_items_set = *LibreDWGc::Dwg_TableRow_num_customdata_items_set;
*swig_customdata_items_get = *LibreDWGc::Dwg_TableRow_customdata_items_get;
*swig_customdata_items_set = *LibreDWGc::Dwg_TableRow_customdata_items_set;
*swig_cellstyle_get = *LibreDWGc::Dwg_TableRow_cellstyle_get;
*swig_cellstyle_set = *LibreDWGc::Dwg_TableRow_cellstyle_set;
*swig_style_id_get = *LibreDWGc::Dwg_TableRow_style_id_get;
*swig_style_id_set = *LibreDWGc::Dwg_TableRow_style_id_set;
*swig_height_get = *LibreDWGc::Dwg_TableRow_height_get;
*swig_height_set = *LibreDWGc::Dwg_TableRow_height_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TableRow(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TableRow($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TableDataColumn ##############

package LibreDWG::Dwg_TableDataColumn;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TableDataColumn_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TableDataColumn_parent_set;
*swig_name_get = *LibreDWGc::Dwg_TableDataColumn_name_get;
*swig_name_set = *LibreDWGc::Dwg_TableDataColumn_name_set;
*swig_custom_data_get = *LibreDWGc::Dwg_TableDataColumn_custom_data_get;
*swig_custom_data_set = *LibreDWGc::Dwg_TableDataColumn_custom_data_set;
*swig_cellstyle_get = *LibreDWGc::Dwg_TableDataColumn_cellstyle_get;
*swig_cellstyle_set = *LibreDWGc::Dwg_TableDataColumn_cellstyle_set;
*swig_cellstyle_id_get = *LibreDWGc::Dwg_TableDataColumn_cellstyle_id_get;
*swig_cellstyle_id_set = *LibreDWGc::Dwg_TableDataColumn_cellstyle_id_set;
*swig_width_get = *LibreDWGc::Dwg_TableDataColumn_width_get;
*swig_width_set = *LibreDWGc::Dwg_TableDataColumn_width_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TableDataColumn(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TableDataColumn($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LinkedTableData ##############

package LibreDWG::Dwg_LinkedTableData;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_cols_get = *LibreDWGc::Dwg_LinkedTableData_num_cols_get;
*swig_num_cols_set = *LibreDWGc::Dwg_LinkedTableData_num_cols_set;
*swig_cols_get = *LibreDWGc::Dwg_LinkedTableData_cols_get;
*swig_cols_set = *LibreDWGc::Dwg_LinkedTableData_cols_set;
*swig_num_rows_get = *LibreDWGc::Dwg_LinkedTableData_num_rows_get;
*swig_num_rows_set = *LibreDWGc::Dwg_LinkedTableData_num_rows_set;
*swig_rows_get = *LibreDWGc::Dwg_LinkedTableData_rows_get;
*swig_rows_set = *LibreDWGc::Dwg_LinkedTableData_rows_set;
*swig_num_field_refs_get = *LibreDWGc::Dwg_LinkedTableData_num_field_refs_get;
*swig_num_field_refs_set = *LibreDWGc::Dwg_LinkedTableData_num_field_refs_set;
*swig_field_refs_get = *LibreDWGc::Dwg_LinkedTableData_field_refs_get;
*swig_field_refs_set = *LibreDWGc::Dwg_LinkedTableData_field_refs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LinkedTableData(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LinkedTableData($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_FormattedTableMerged ##############

package LibreDWG::Dwg_FormattedTableMerged;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_FormattedTableMerged_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_FormattedTableMerged_parent_set;
*swig_top_row_get = *LibreDWGc::Dwg_FormattedTableMerged_top_row_get;
*swig_top_row_set = *LibreDWGc::Dwg_FormattedTableMerged_top_row_set;
*swig_left_col_get = *LibreDWGc::Dwg_FormattedTableMerged_left_col_get;
*swig_left_col_set = *LibreDWGc::Dwg_FormattedTableMerged_left_col_set;
*swig_bottom_row_get = *LibreDWGc::Dwg_FormattedTableMerged_bottom_row_get;
*swig_bottom_row_set = *LibreDWGc::Dwg_FormattedTableMerged_bottom_row_set;
*swig_right_col_get = *LibreDWGc::Dwg_FormattedTableMerged_right_col_get;
*swig_right_col_set = *LibreDWGc::Dwg_FormattedTableMerged_right_col_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_FormattedTableMerged(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_FormattedTableMerged($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_FormattedTableData ##############

package LibreDWG::Dwg_FormattedTableData;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_FormattedTableData_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_FormattedTableData_parent_set;
*swig_cellstyle_get = *LibreDWGc::Dwg_FormattedTableData_cellstyle_get;
*swig_cellstyle_set = *LibreDWGc::Dwg_FormattedTableData_cellstyle_set;
*swig_num_merged_cells_get = *LibreDWGc::Dwg_FormattedTableData_num_merged_cells_get;
*swig_num_merged_cells_set = *LibreDWGc::Dwg_FormattedTableData_num_merged_cells_set;
*swig_merged_cells_get = *LibreDWGc::Dwg_FormattedTableData_merged_cells_get;
*swig_merged_cells_set = *LibreDWGc::Dwg_FormattedTableData_merged_cells_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_FormattedTableData(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_FormattedTableData($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_TABLECONTENT ##############

package LibreDWG::Dwg_Object_TABLECONTENT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_TABLECONTENT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_TABLECONTENT_parent_set;
*swig_ldata_get = *LibreDWGc::Dwg_Object_TABLECONTENT_ldata_get;
*swig_ldata_set = *LibreDWGc::Dwg_Object_TABLECONTENT_ldata_set;
*swig_tdata_get = *LibreDWGc::Dwg_Object_TABLECONTENT_tdata_get;
*swig_tdata_set = *LibreDWGc::Dwg_Object_TABLECONTENT_tdata_set;
*swig_fdata_get = *LibreDWGc::Dwg_Object_TABLECONTENT_fdata_get;
*swig_fdata_set = *LibreDWGc::Dwg_Object_TABLECONTENT_fdata_set;
*swig_tablestyle_get = *LibreDWGc::Dwg_Object_TABLECONTENT_tablestyle_get;
*swig_tablestyle_set = *LibreDWGc::Dwg_Object_TABLECONTENT_tablestyle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_TABLECONTENT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_TABLECONTENT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_TABLE ##############

package LibreDWG::Dwg_Entity_TABLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_TABLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_TABLE_parent_set;
*swig_ldata_get = *LibreDWGc::Dwg_Entity_TABLE_ldata_get;
*swig_ldata_set = *LibreDWGc::Dwg_Entity_TABLE_ldata_set;
*swig_tdata_get = *LibreDWGc::Dwg_Entity_TABLE_tdata_get;
*swig_tdata_set = *LibreDWGc::Dwg_Entity_TABLE_tdata_set;
*swig_fdata_get = *LibreDWGc::Dwg_Entity_TABLE_fdata_get;
*swig_fdata_set = *LibreDWGc::Dwg_Entity_TABLE_fdata_set;
*swig_tablestyle_get = *LibreDWGc::Dwg_Entity_TABLE_tablestyle_get;
*swig_tablestyle_set = *LibreDWGc::Dwg_Entity_TABLE_tablestyle_set;
*swig_unknown_rc_get = *LibreDWGc::Dwg_Entity_TABLE_unknown_rc_get;
*swig_unknown_rc_set = *LibreDWGc::Dwg_Entity_TABLE_unknown_rc_set;
*swig_unknown_h_get = *LibreDWGc::Dwg_Entity_TABLE_unknown_h_get;
*swig_unknown_h_set = *LibreDWGc::Dwg_Entity_TABLE_unknown_h_set;
*swig_unknown_bl_get = *LibreDWGc::Dwg_Entity_TABLE_unknown_bl_get;
*swig_unknown_bl_set = *LibreDWGc::Dwg_Entity_TABLE_unknown_bl_set;
*swig_unknown_b_get = *LibreDWGc::Dwg_Entity_TABLE_unknown_b_get;
*swig_unknown_b_set = *LibreDWGc::Dwg_Entity_TABLE_unknown_b_set;
*swig_unknown_bl1_get = *LibreDWGc::Dwg_Entity_TABLE_unknown_bl1_get;
*swig_unknown_bl1_set = *LibreDWGc::Dwg_Entity_TABLE_unknown_bl1_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_TABLE_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_TABLE_ins_pt_set;
*swig_scale_get = *LibreDWGc::Dwg_Entity_TABLE_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Entity_TABLE_scale_set;
*swig_scale_flag_get = *LibreDWGc::Dwg_Entity_TABLE_scale_flag_get;
*swig_scale_flag_set = *LibreDWGc::Dwg_Entity_TABLE_scale_flag_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_TABLE_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_TABLE_rotation_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_TABLE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_TABLE_extrusion_set;
*swig_has_attribs_get = *LibreDWGc::Dwg_Entity_TABLE_has_attribs_get;
*swig_has_attribs_set = *LibreDWGc::Dwg_Entity_TABLE_has_attribs_set;
*swig_num_owned_get = *LibreDWGc::Dwg_Entity_TABLE_num_owned_get;
*swig_num_owned_set = *LibreDWGc::Dwg_Entity_TABLE_num_owned_set;
*swig_flag_for_table_value_get = *LibreDWGc::Dwg_Entity_TABLE_flag_for_table_value_get;
*swig_flag_for_table_value_set = *LibreDWGc::Dwg_Entity_TABLE_flag_for_table_value_set;
*swig_horiz_direction_get = *LibreDWGc::Dwg_Entity_TABLE_horiz_direction_get;
*swig_horiz_direction_set = *LibreDWGc::Dwg_Entity_TABLE_horiz_direction_set;
*swig_num_cols_get = *LibreDWGc::Dwg_Entity_TABLE_num_cols_get;
*swig_num_cols_set = *LibreDWGc::Dwg_Entity_TABLE_num_cols_set;
*swig_num_rows_get = *LibreDWGc::Dwg_Entity_TABLE_num_rows_get;
*swig_num_rows_set = *LibreDWGc::Dwg_Entity_TABLE_num_rows_set;
*swig_num_cells_get = *LibreDWGc::Dwg_Entity_TABLE_num_cells_get;
*swig_num_cells_set = *LibreDWGc::Dwg_Entity_TABLE_num_cells_set;
*swig_col_widths_get = *LibreDWGc::Dwg_Entity_TABLE_col_widths_get;
*swig_col_widths_set = *LibreDWGc::Dwg_Entity_TABLE_col_widths_set;
*swig_row_heights_get = *LibreDWGc::Dwg_Entity_TABLE_row_heights_get;
*swig_row_heights_set = *LibreDWGc::Dwg_Entity_TABLE_row_heights_set;
*swig_cells_get = *LibreDWGc::Dwg_Entity_TABLE_cells_get;
*swig_cells_set = *LibreDWGc::Dwg_Entity_TABLE_cells_set;
*swig_has_table_overrides_get = *LibreDWGc::Dwg_Entity_TABLE_has_table_overrides_get;
*swig_has_table_overrides_set = *LibreDWGc::Dwg_Entity_TABLE_has_table_overrides_set;
*swig_table_flag_override_get = *LibreDWGc::Dwg_Entity_TABLE_table_flag_override_get;
*swig_table_flag_override_set = *LibreDWGc::Dwg_Entity_TABLE_table_flag_override_set;
*swig_title_suppressed_get = *LibreDWGc::Dwg_Entity_TABLE_title_suppressed_get;
*swig_title_suppressed_set = *LibreDWGc::Dwg_Entity_TABLE_title_suppressed_set;
*swig_header_suppressed_get = *LibreDWGc::Dwg_Entity_TABLE_header_suppressed_get;
*swig_header_suppressed_set = *LibreDWGc::Dwg_Entity_TABLE_header_suppressed_set;
*swig_flow_direction_get = *LibreDWGc::Dwg_Entity_TABLE_flow_direction_get;
*swig_flow_direction_set = *LibreDWGc::Dwg_Entity_TABLE_flow_direction_set;
*swig_horiz_cell_margin_get = *LibreDWGc::Dwg_Entity_TABLE_horiz_cell_margin_get;
*swig_horiz_cell_margin_set = *LibreDWGc::Dwg_Entity_TABLE_horiz_cell_margin_set;
*swig_vert_cell_margin_get = *LibreDWGc::Dwg_Entity_TABLE_vert_cell_margin_get;
*swig_vert_cell_margin_set = *LibreDWGc::Dwg_Entity_TABLE_vert_cell_margin_set;
*swig_title_row_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_row_color_get;
*swig_title_row_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_row_color_set;
*swig_header_row_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_row_color_get;
*swig_header_row_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_row_color_set;
*swig_data_row_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_row_color_get;
*swig_data_row_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_row_color_set;
*swig_title_row_fill_none_get = *LibreDWGc::Dwg_Entity_TABLE_title_row_fill_none_get;
*swig_title_row_fill_none_set = *LibreDWGc::Dwg_Entity_TABLE_title_row_fill_none_set;
*swig_header_row_fill_none_get = *LibreDWGc::Dwg_Entity_TABLE_header_row_fill_none_get;
*swig_header_row_fill_none_set = *LibreDWGc::Dwg_Entity_TABLE_header_row_fill_none_set;
*swig_data_row_fill_none_get = *LibreDWGc::Dwg_Entity_TABLE_data_row_fill_none_get;
*swig_data_row_fill_none_set = *LibreDWGc::Dwg_Entity_TABLE_data_row_fill_none_set;
*swig_title_row_fill_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_row_fill_color_get;
*swig_title_row_fill_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_row_fill_color_set;
*swig_header_row_fill_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_row_fill_color_get;
*swig_header_row_fill_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_row_fill_color_set;
*swig_data_row_fill_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_row_fill_color_get;
*swig_data_row_fill_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_row_fill_color_set;
*swig_title_row_alignment_get = *LibreDWGc::Dwg_Entity_TABLE_title_row_alignment_get;
*swig_title_row_alignment_set = *LibreDWGc::Dwg_Entity_TABLE_title_row_alignment_set;
*swig_header_row_alignment_get = *LibreDWGc::Dwg_Entity_TABLE_header_row_alignment_get;
*swig_header_row_alignment_set = *LibreDWGc::Dwg_Entity_TABLE_header_row_alignment_set;
*swig_data_row_alignment_get = *LibreDWGc::Dwg_Entity_TABLE_data_row_alignment_get;
*swig_data_row_alignment_set = *LibreDWGc::Dwg_Entity_TABLE_data_row_alignment_set;
*swig_title_text_style_get = *LibreDWGc::Dwg_Entity_TABLE_title_text_style_get;
*swig_title_text_style_set = *LibreDWGc::Dwg_Entity_TABLE_title_text_style_set;
*swig_header_text_style_get = *LibreDWGc::Dwg_Entity_TABLE_header_text_style_get;
*swig_header_text_style_set = *LibreDWGc::Dwg_Entity_TABLE_header_text_style_set;
*swig_data_text_style_get = *LibreDWGc::Dwg_Entity_TABLE_data_text_style_get;
*swig_data_text_style_set = *LibreDWGc::Dwg_Entity_TABLE_data_text_style_set;
*swig_title_row_height_get = *LibreDWGc::Dwg_Entity_TABLE_title_row_height_get;
*swig_title_row_height_set = *LibreDWGc::Dwg_Entity_TABLE_title_row_height_set;
*swig_header_row_height_get = *LibreDWGc::Dwg_Entity_TABLE_header_row_height_get;
*swig_header_row_height_set = *LibreDWGc::Dwg_Entity_TABLE_header_row_height_set;
*swig_data_row_height_get = *LibreDWGc::Dwg_Entity_TABLE_data_row_height_get;
*swig_data_row_height_set = *LibreDWGc::Dwg_Entity_TABLE_data_row_height_set;
*swig_has_border_color_overrides_get = *LibreDWGc::Dwg_Entity_TABLE_has_border_color_overrides_get;
*swig_has_border_color_overrides_set = *LibreDWGc::Dwg_Entity_TABLE_has_border_color_overrides_set;
*swig_border_color_overrides_flag_get = *LibreDWGc::Dwg_Entity_TABLE_border_color_overrides_flag_get;
*swig_border_color_overrides_flag_set = *LibreDWGc::Dwg_Entity_TABLE_border_color_overrides_flag_set;
*swig_title_horiz_top_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_color_get;
*swig_title_horiz_top_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_color_set;
*swig_title_horiz_ins_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_color_get;
*swig_title_horiz_ins_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_color_set;
*swig_title_horiz_bottom_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_color_get;
*swig_title_horiz_bottom_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_color_set;
*swig_title_vert_left_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_left_color_get;
*swig_title_vert_left_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_left_color_set;
*swig_title_vert_ins_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_color_get;
*swig_title_vert_ins_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_color_set;
*swig_title_vert_right_color_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_right_color_get;
*swig_title_vert_right_color_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_right_color_set;
*swig_header_horiz_top_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_color_get;
*swig_header_horiz_top_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_color_set;
*swig_header_horiz_ins_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_color_get;
*swig_header_horiz_ins_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_color_set;
*swig_header_horiz_bottom_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_color_get;
*swig_header_horiz_bottom_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_color_set;
*swig_header_vert_left_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_left_color_get;
*swig_header_vert_left_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_left_color_set;
*swig_header_vert_ins_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_color_get;
*swig_header_vert_ins_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_color_set;
*swig_header_vert_right_color_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_right_color_get;
*swig_header_vert_right_color_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_right_color_set;
*swig_data_horiz_top_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_color_get;
*swig_data_horiz_top_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_color_set;
*swig_data_horiz_ins_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_color_get;
*swig_data_horiz_ins_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_color_set;
*swig_data_horiz_bottom_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_color_get;
*swig_data_horiz_bottom_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_color_set;
*swig_data_vert_left_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_left_color_get;
*swig_data_vert_left_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_left_color_set;
*swig_data_vert_ins_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_color_get;
*swig_data_vert_ins_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_color_set;
*swig_data_vert_right_color_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_right_color_get;
*swig_data_vert_right_color_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_right_color_set;
*swig_has_border_lineweight_overrides_get = *LibreDWGc::Dwg_Entity_TABLE_has_border_lineweight_overrides_get;
*swig_has_border_lineweight_overrides_set = *LibreDWGc::Dwg_Entity_TABLE_has_border_lineweight_overrides_set;
*swig_border_lineweight_overrides_flag_get = *LibreDWGc::Dwg_Entity_TABLE_border_lineweight_overrides_flag_get;
*swig_border_lineweight_overrides_flag_set = *LibreDWGc::Dwg_Entity_TABLE_border_lineweight_overrides_flag_set;
*swig_title_horiz_top_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_linewt_get;
*swig_title_horiz_top_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_linewt_set;
*swig_title_horiz_ins_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_linewt_get;
*swig_title_horiz_ins_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_linewt_set;
*swig_title_horiz_bottom_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_linewt_get;
*swig_title_horiz_bottom_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_linewt_set;
*swig_title_vert_left_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_left_linewt_get;
*swig_title_vert_left_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_left_linewt_set;
*swig_title_vert_ins_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_linewt_get;
*swig_title_vert_ins_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_linewt_set;
*swig_title_vert_right_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_right_linewt_get;
*swig_title_vert_right_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_right_linewt_set;
*swig_header_horiz_top_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_linewt_get;
*swig_header_horiz_top_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_linewt_set;
*swig_header_horiz_ins_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_linewt_get;
*swig_header_horiz_ins_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_linewt_set;
*swig_header_horiz_bottom_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_linewt_get;
*swig_header_horiz_bottom_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_linewt_set;
*swig_header_vert_left_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_left_linewt_get;
*swig_header_vert_left_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_left_linewt_set;
*swig_header_vert_ins_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_linewt_get;
*swig_header_vert_ins_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_linewt_set;
*swig_header_vert_right_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_right_linewt_get;
*swig_header_vert_right_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_right_linewt_set;
*swig_data_horiz_top_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_linewt_get;
*swig_data_horiz_top_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_linewt_set;
*swig_data_horiz_ins_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_linewt_get;
*swig_data_horiz_ins_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_linewt_set;
*swig_data_horiz_bottom_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_linewt_get;
*swig_data_horiz_bottom_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_linewt_set;
*swig_data_vert_left_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_left_linewt_get;
*swig_data_vert_left_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_left_linewt_set;
*swig_data_vert_ins_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_linewt_get;
*swig_data_vert_ins_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_linewt_set;
*swig_data_vert_right_linewt_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_right_linewt_get;
*swig_data_vert_right_linewt_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_right_linewt_set;
*swig_has_border_visibility_overrides_get = *LibreDWGc::Dwg_Entity_TABLE_has_border_visibility_overrides_get;
*swig_has_border_visibility_overrides_set = *LibreDWGc::Dwg_Entity_TABLE_has_border_visibility_overrides_set;
*swig_border_visibility_overrides_flag_get = *LibreDWGc::Dwg_Entity_TABLE_border_visibility_overrides_flag_get;
*swig_border_visibility_overrides_flag_set = *LibreDWGc::Dwg_Entity_TABLE_border_visibility_overrides_flag_set;
*swig_title_horiz_top_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_visibility_get;
*swig_title_horiz_top_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_top_visibility_set;
*swig_title_horiz_ins_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_visibility_get;
*swig_title_horiz_ins_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_ins_visibility_set;
*swig_title_horiz_bottom_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_visibility_get;
*swig_title_horiz_bottom_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_title_horiz_bottom_visibility_set;
*swig_title_vert_left_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_left_visibility_get;
*swig_title_vert_left_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_left_visibility_set;
*swig_title_vert_ins_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_visibility_get;
*swig_title_vert_ins_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_ins_visibility_set;
*swig_title_vert_right_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_title_vert_right_visibility_get;
*swig_title_vert_right_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_title_vert_right_visibility_set;
*swig_header_horiz_top_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_visibility_get;
*swig_header_horiz_top_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_top_visibility_set;
*swig_header_horiz_ins_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_visibility_get;
*swig_header_horiz_ins_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_ins_visibility_set;
*swig_header_horiz_bottom_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_visibility_get;
*swig_header_horiz_bottom_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_header_horiz_bottom_visibility_set;
*swig_header_vert_left_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_left_visibility_get;
*swig_header_vert_left_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_left_visibility_set;
*swig_header_vert_ins_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_visibility_get;
*swig_header_vert_ins_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_ins_visibility_set;
*swig_header_vert_right_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_header_vert_right_visibility_get;
*swig_header_vert_right_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_header_vert_right_visibility_set;
*swig_data_horiz_top_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_visibility_get;
*swig_data_horiz_top_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_top_visibility_set;
*swig_data_horiz_ins_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_visibility_get;
*swig_data_horiz_ins_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_ins_visibility_set;
*swig_data_horiz_bottom_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_visibility_get;
*swig_data_horiz_bottom_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_data_horiz_bottom_visibility_set;
*swig_data_vert_left_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_left_visibility_get;
*swig_data_vert_left_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_left_visibility_set;
*swig_data_vert_ins_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_visibility_get;
*swig_data_vert_ins_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_ins_visibility_set;
*swig_data_vert_right_visibility_get = *LibreDWGc::Dwg_Entity_TABLE_data_vert_right_visibility_get;
*swig_data_vert_right_visibility_set = *LibreDWGc::Dwg_Entity_TABLE_data_vert_right_visibility_set;
*swig_block_header_get = *LibreDWGc::Dwg_Entity_TABLE_block_header_get;
*swig_block_header_set = *LibreDWGc::Dwg_Entity_TABLE_block_header_set;
*swig_first_attrib_get = *LibreDWGc::Dwg_Entity_TABLE_first_attrib_get;
*swig_first_attrib_set = *LibreDWGc::Dwg_Entity_TABLE_first_attrib_set;
*swig_last_attrib_get = *LibreDWGc::Dwg_Entity_TABLE_last_attrib_get;
*swig_last_attrib_set = *LibreDWGc::Dwg_Entity_TABLE_last_attrib_set;
*swig_attribs_get = *LibreDWGc::Dwg_Entity_TABLE_attribs_get;
*swig_attribs_set = *LibreDWGc::Dwg_Entity_TABLE_attribs_set;
*swig_seqend_get = *LibreDWGc::Dwg_Entity_TABLE_seqend_get;
*swig_seqend_set = *LibreDWGc::Dwg_Entity_TABLE_seqend_set;
*swig_title_row_style_override_get = *LibreDWGc::Dwg_Entity_TABLE_title_row_style_override_get;
*swig_title_row_style_override_set = *LibreDWGc::Dwg_Entity_TABLE_title_row_style_override_set;
*swig_header_row_style_override_get = *LibreDWGc::Dwg_Entity_TABLE_header_row_style_override_get;
*swig_header_row_style_override_set = *LibreDWGc::Dwg_Entity_TABLE_header_row_style_override_set;
*swig_data_row_style_override_get = *LibreDWGc::Dwg_Entity_TABLE_data_row_style_override_get;
*swig_data_row_style_override_set = *LibreDWGc::Dwg_Entity_TABLE_data_row_style_override_set;
*swig_unknown_bs_get = *LibreDWGc::Dwg_Entity_TABLE_unknown_bs_get;
*swig_unknown_bs_set = *LibreDWGc::Dwg_Entity_TABLE_unknown_bs_set;
*swig_hor_dir_get = *LibreDWGc::Dwg_Entity_TABLE_hor_dir_get;
*swig_hor_dir_set = *LibreDWGc::Dwg_Entity_TABLE_hor_dir_set;
*swig_has_break_data_get = *LibreDWGc::Dwg_Entity_TABLE_has_break_data_get;
*swig_has_break_data_set = *LibreDWGc::Dwg_Entity_TABLE_has_break_data_set;
*swig_break_flag_get = *LibreDWGc::Dwg_Entity_TABLE_break_flag_get;
*swig_break_flag_set = *LibreDWGc::Dwg_Entity_TABLE_break_flag_set;
*swig_break_flow_direction_get = *LibreDWGc::Dwg_Entity_TABLE_break_flow_direction_get;
*swig_break_flow_direction_set = *LibreDWGc::Dwg_Entity_TABLE_break_flow_direction_set;
*swig_break_spacing_get = *LibreDWGc::Dwg_Entity_TABLE_break_spacing_get;
*swig_break_spacing_set = *LibreDWGc::Dwg_Entity_TABLE_break_spacing_set;
*swig_break_unknown1_get = *LibreDWGc::Dwg_Entity_TABLE_break_unknown1_get;
*swig_break_unknown1_set = *LibreDWGc::Dwg_Entity_TABLE_break_unknown1_set;
*swig_break_unknown2_get = *LibreDWGc::Dwg_Entity_TABLE_break_unknown2_get;
*swig_break_unknown2_set = *LibreDWGc::Dwg_Entity_TABLE_break_unknown2_set;
*swig_num_break_heights_get = *LibreDWGc::Dwg_Entity_TABLE_num_break_heights_get;
*swig_num_break_heights_set = *LibreDWGc::Dwg_Entity_TABLE_num_break_heights_set;
*swig_break_heights_get = *LibreDWGc::Dwg_Entity_TABLE_break_heights_get;
*swig_break_heights_set = *LibreDWGc::Dwg_Entity_TABLE_break_heights_set;
*swig_num_break_rows_get = *LibreDWGc::Dwg_Entity_TABLE_num_break_rows_get;
*swig_num_break_rows_set = *LibreDWGc::Dwg_Entity_TABLE_num_break_rows_set;
*swig_break_rows_get = *LibreDWGc::Dwg_Entity_TABLE_break_rows_get;
*swig_break_rows_set = *LibreDWGc::Dwg_Entity_TABLE_break_rows_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_TABLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_TABLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLESTYLE_CellStyle ##############

package LibreDWG::Dwg_TABLESTYLE_CellStyle;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_parent_set;
*swig_id_get = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_id_get;
*swig_id_set = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_id_set;
*swig_type_get = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_type_get;
*swig_type_set = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_type_set;
*swig_name_get = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_name_get;
*swig_name_set = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_name_set;
*swig_cellstyle_get = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_cellstyle_get;
*swig_cellstyle_set = *LibreDWGc::Dwg_TABLESTYLE_CellStyle_cellstyle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLESTYLE_CellStyle(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLESTYLE_CellStyle($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLESTYLE_border ##############

package LibreDWG::Dwg_TABLESTYLE_border;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_linewt_get = *LibreDWGc::Dwg_TABLESTYLE_border_linewt_get;
*swig_linewt_set = *LibreDWGc::Dwg_TABLESTYLE_border_linewt_set;
*swig_visible_get = *LibreDWGc::Dwg_TABLESTYLE_border_visible_get;
*swig_visible_set = *LibreDWGc::Dwg_TABLESTYLE_border_visible_set;
*swig_color_get = *LibreDWGc::Dwg_TABLESTYLE_border_color_get;
*swig_color_set = *LibreDWGc::Dwg_TABLESTYLE_border_color_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLESTYLE_border(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLESTYLE_border($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLESTYLE_rowstyles ##############

package LibreDWG::Dwg_TABLESTYLE_rowstyles;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_parent_set;
*swig_text_style_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_style_get;
*swig_text_style_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_style_set;
*swig_text_height_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_height_get;
*swig_text_height_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_height_set;
*swig_text_alignment_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_alignment_get;
*swig_text_alignment_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_alignment_set;
*swig_text_color_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_color_get;
*swig_text_color_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_text_color_set;
*swig_fill_color_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_fill_color_get;
*swig_fill_color_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_fill_color_set;
*swig_has_bgcolor_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_has_bgcolor_get;
*swig_has_bgcolor_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_has_bgcolor_set;
*swig_num_borders_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_num_borders_get;
*swig_num_borders_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_num_borders_set;
*swig_borders_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_borders_get;
*swig_borders_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_borders_set;
*swig_data_type_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_data_type_get;
*swig_data_type_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_data_type_set;
*swig_unit_type_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_unit_type_get;
*swig_unit_type_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_unit_type_set;
*swig_format_string_get = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_format_string_get;
*swig_format_string_set = *LibreDWGc::Dwg_TABLESTYLE_rowstyles_format_string_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLESTYLE_rowstyles(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLESTYLE_rowstyles($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_TABLESTYLE ##############

package LibreDWG::Dwg_Object_TABLESTYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_TABLESTYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_TABLESTYLE_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_TABLESTYLE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_TABLESTYLE_class_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_TABLESTYLE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_TABLESTYLE_name_set;
*swig_flags_get = *LibreDWGc::Dwg_Object_TABLESTYLE_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Object_TABLESTYLE_flags_set;
*swig_flow_direction_get = *LibreDWGc::Dwg_Object_TABLESTYLE_flow_direction_get;
*swig_flow_direction_set = *LibreDWGc::Dwg_Object_TABLESTYLE_flow_direction_set;
*swig_horiz_cell_margin_get = *LibreDWGc::Dwg_Object_TABLESTYLE_horiz_cell_margin_get;
*swig_horiz_cell_margin_set = *LibreDWGc::Dwg_Object_TABLESTYLE_horiz_cell_margin_set;
*swig_vert_cell_margin_get = *LibreDWGc::Dwg_Object_TABLESTYLE_vert_cell_margin_get;
*swig_vert_cell_margin_set = *LibreDWGc::Dwg_Object_TABLESTYLE_vert_cell_margin_set;
*swig_is_title_suppressed_get = *LibreDWGc::Dwg_Object_TABLESTYLE_is_title_suppressed_get;
*swig_is_title_suppressed_set = *LibreDWGc::Dwg_Object_TABLESTYLE_is_title_suppressed_set;
*swig_is_header_suppressed_get = *LibreDWGc::Dwg_Object_TABLESTYLE_is_header_suppressed_get;
*swig_is_header_suppressed_set = *LibreDWGc::Dwg_Object_TABLESTYLE_is_header_suppressed_set;
*swig_unknown_rc_get = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_rc_get;
*swig_unknown_rc_set = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_rc_set;
*swig_unknown_bl1_get = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl1_get;
*swig_unknown_bl1_set = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl1_set;
*swig_unknown_bl2_get = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl2_get;
*swig_unknown_bl2_set = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl2_set;
*swig_cellstyle_get = *LibreDWGc::Dwg_Object_TABLESTYLE_cellstyle_get;
*swig_cellstyle_set = *LibreDWGc::Dwg_Object_TABLESTYLE_cellstyle_set;
*swig_sty_get = *LibreDWGc::Dwg_Object_TABLESTYLE_sty_get;
*swig_sty_set = *LibreDWGc::Dwg_Object_TABLESTYLE_sty_set;
*swig_numoverrides_get = *LibreDWGc::Dwg_Object_TABLESTYLE_numoverrides_get;
*swig_numoverrides_set = *LibreDWGc::Dwg_Object_TABLESTYLE_numoverrides_set;
*swig_unknown_bl3_get = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl3_get;
*swig_unknown_bl3_set = *LibreDWGc::Dwg_Object_TABLESTYLE_unknown_bl3_set;
*swig_ovr_get = *LibreDWGc::Dwg_Object_TABLESTYLE_ovr_get;
*swig_ovr_set = *LibreDWGc::Dwg_Object_TABLESTYLE_ovr_set;
*swig_num_rowstyles_get = *LibreDWGc::Dwg_Object_TABLESTYLE_num_rowstyles_get;
*swig_num_rowstyles_set = *LibreDWGc::Dwg_Object_TABLESTYLE_num_rowstyles_set;
*swig_rowstyles_get = *LibreDWGc::Dwg_Object_TABLESTYLE_rowstyles_get;
*swig_rowstyles_set = *LibreDWGc::Dwg_Object_TABLESTYLE_rowstyles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_TABLESTYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_TABLESTYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_CELLSTYLEMAP ##############

package LibreDWG::Dwg_Object_CELLSTYLEMAP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_CELLSTYLEMAP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_CELLSTYLEMAP_parent_set;
*swig_num_cells_get = *LibreDWGc::Dwg_Object_CELLSTYLEMAP_num_cells_get;
*swig_num_cells_set = *LibreDWGc::Dwg_Object_CELLSTYLEMAP_num_cells_set;
*swig_cells_get = *LibreDWGc::Dwg_Object_CELLSTYLEMAP_cells_get;
*swig_cells_set = *LibreDWGc::Dwg_Object_CELLSTYLEMAP_cells_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_CELLSTYLEMAP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_CELLSTYLEMAP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_TABLEGEOMETRY_Cell ##############

package LibreDWG::Dwg_TABLEGEOMETRY_Cell;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_parent_set;
*swig_geom_data_flag_get = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geom_data_flag_get;
*swig_geom_data_flag_set = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geom_data_flag_set;
*swig_width_w_gap_get = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_width_w_gap_get;
*swig_width_w_gap_set = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_width_w_gap_set;
*swig_height_w_gap_get = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_height_w_gap_get;
*swig_height_w_gap_set = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_height_w_gap_set;
*swig_tablegeometry_get = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_tablegeometry_get;
*swig_tablegeometry_set = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_tablegeometry_set;
*swig_num_geometry_get = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_num_geometry_get;
*swig_num_geometry_set = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_num_geometry_set;
*swig_geometry_get = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geometry_get;
*swig_geometry_set = *LibreDWGc::Dwg_TABLEGEOMETRY_Cell_geometry_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_TABLEGEOMETRY_Cell(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_TABLEGEOMETRY_Cell($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_TABLEGEOMETRY ##############

package LibreDWG::Dwg_Object_TABLEGEOMETRY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_parent_set;
*swig_numrows_get = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_numrows_get;
*swig_numrows_set = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_numrows_set;
*swig_numcols_get = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_numcols_get;
*swig_numcols_set = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_numcols_set;
*swig_num_cells_get = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_num_cells_get;
*swig_num_cells_set = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_num_cells_set;
*swig_cells_get = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_cells_get;
*swig_cells_set = *LibreDWGc::Dwg_Object_TABLEGEOMETRY_cells_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_TABLEGEOMETRY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_TABLEGEOMETRY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_UNDERLAYDEFINITION ##############

package LibreDWG::Dwg_Object_UNDERLAYDEFINITION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_parent_set;
*swig_filename_get = *LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_filename_get;
*swig_filename_set = *LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_filename_set;
*swig_name_get = *LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_UNDERLAYDEFINITION_name_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_UNDERLAYDEFINITION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_UNDERLAYDEFINITION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_UNDERLAY ##############

package LibreDWG::Dwg_Entity_UNDERLAY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_UNDERLAY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_UNDERLAY_parent_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_UNDERLAY_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_UNDERLAY_extrusion_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_UNDERLAY_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_UNDERLAY_ins_pt_set;
*swig_scale_get = *LibreDWGc::Dwg_Entity_UNDERLAY_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Entity_UNDERLAY_scale_set;
*swig_angle_get = *LibreDWGc::Dwg_Entity_UNDERLAY_angle_get;
*swig_angle_set = *LibreDWGc::Dwg_Entity_UNDERLAY_angle_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_UNDERLAY_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_UNDERLAY_flag_set;
*swig_contrast_get = *LibreDWGc::Dwg_Entity_UNDERLAY_contrast_get;
*swig_contrast_set = *LibreDWGc::Dwg_Entity_UNDERLAY_contrast_set;
*swig_fade_get = *LibreDWGc::Dwg_Entity_UNDERLAY_fade_get;
*swig_fade_set = *LibreDWGc::Dwg_Entity_UNDERLAY_fade_set;
*swig_num_clip_verts_get = *LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_verts_get;
*swig_num_clip_verts_set = *LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_verts_set;
*swig_clip_verts_get = *LibreDWGc::Dwg_Entity_UNDERLAY_clip_verts_get;
*swig_clip_verts_set = *LibreDWGc::Dwg_Entity_UNDERLAY_clip_verts_set;
*swig_num_clip_inverts_get = *LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_inverts_get;
*swig_num_clip_inverts_set = *LibreDWGc::Dwg_Entity_UNDERLAY_num_clip_inverts_set;
*swig_clip_inverts_get = *LibreDWGc::Dwg_Entity_UNDERLAY_clip_inverts_get;
*swig_clip_inverts_set = *LibreDWGc::Dwg_Entity_UNDERLAY_clip_inverts_set;
*swig_definition_id_get = *LibreDWGc::Dwg_Entity_UNDERLAY_definition_id_get;
*swig_definition_id_set = *LibreDWGc::Dwg_Entity_UNDERLAY_definition_id_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_UNDERLAY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_UNDERLAY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DBCOLOR ##############

package LibreDWG::Dwg_Object_DBCOLOR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DBCOLOR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DBCOLOR_parent_set;
*swig_color_get = *LibreDWGc::Dwg_Object_DBCOLOR_color_get;
*swig_color_set = *LibreDWGc::Dwg_Object_DBCOLOR_color_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DBCOLOR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DBCOLOR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_FIELD_ChildValue ##############

package LibreDWG::Dwg_FIELD_ChildValue;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_FIELD_ChildValue_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_FIELD_ChildValue_parent_set;
*swig_key_get = *LibreDWGc::Dwg_FIELD_ChildValue_key_get;
*swig_key_set = *LibreDWGc::Dwg_FIELD_ChildValue_key_set;
*swig_value_get = *LibreDWGc::Dwg_FIELD_ChildValue_value_get;
*swig_value_set = *LibreDWGc::Dwg_FIELD_ChildValue_value_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_FIELD_ChildValue(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_FIELD_ChildValue($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_FIELD ##############

package LibreDWG::Dwg_Object_FIELD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_FIELD_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_FIELD_parent_set;
*swig_id_get = *LibreDWGc::Dwg_Object_FIELD_id_get;
*swig_id_set = *LibreDWGc::Dwg_Object_FIELD_id_set;
*swig_code_get = *LibreDWGc::Dwg_Object_FIELD_code_get;
*swig_code_set = *LibreDWGc::Dwg_Object_FIELD_code_set;
*swig_num_childs_get = *LibreDWGc::Dwg_Object_FIELD_num_childs_get;
*swig_num_childs_set = *LibreDWGc::Dwg_Object_FIELD_num_childs_set;
*swig_childs_get = *LibreDWGc::Dwg_Object_FIELD_childs_get;
*swig_childs_set = *LibreDWGc::Dwg_Object_FIELD_childs_set;
*swig_num_objects_get = *LibreDWGc::Dwg_Object_FIELD_num_objects_get;
*swig_num_objects_set = *LibreDWGc::Dwg_Object_FIELD_num_objects_set;
*swig_objects_get = *LibreDWGc::Dwg_Object_FIELD_objects_get;
*swig_objects_set = *LibreDWGc::Dwg_Object_FIELD_objects_set;
*swig_format_get = *LibreDWGc::Dwg_Object_FIELD_format_get;
*swig_format_set = *LibreDWGc::Dwg_Object_FIELD_format_set;
*swig_evaluation_option_get = *LibreDWGc::Dwg_Object_FIELD_evaluation_option_get;
*swig_evaluation_option_set = *LibreDWGc::Dwg_Object_FIELD_evaluation_option_set;
*swig_filing_option_get = *LibreDWGc::Dwg_Object_FIELD_filing_option_get;
*swig_filing_option_set = *LibreDWGc::Dwg_Object_FIELD_filing_option_set;
*swig_field_state_get = *LibreDWGc::Dwg_Object_FIELD_field_state_get;
*swig_field_state_set = *LibreDWGc::Dwg_Object_FIELD_field_state_set;
*swig_evaluation_status_get = *LibreDWGc::Dwg_Object_FIELD_evaluation_status_get;
*swig_evaluation_status_set = *LibreDWGc::Dwg_Object_FIELD_evaluation_status_set;
*swig_evaluation_error_code_get = *LibreDWGc::Dwg_Object_FIELD_evaluation_error_code_get;
*swig_evaluation_error_code_set = *LibreDWGc::Dwg_Object_FIELD_evaluation_error_code_set;
*swig_evaluation_error_msg_get = *LibreDWGc::Dwg_Object_FIELD_evaluation_error_msg_get;
*swig_evaluation_error_msg_set = *LibreDWGc::Dwg_Object_FIELD_evaluation_error_msg_set;
*swig_value_get = *LibreDWGc::Dwg_Object_FIELD_value_get;
*swig_value_set = *LibreDWGc::Dwg_Object_FIELD_value_set;
*swig_value_string_get = *LibreDWGc::Dwg_Object_FIELD_value_string_get;
*swig_value_string_set = *LibreDWGc::Dwg_Object_FIELD_value_string_set;
*swig_value_string_length_get = *LibreDWGc::Dwg_Object_FIELD_value_string_length_get;
*swig_value_string_length_set = *LibreDWGc::Dwg_Object_FIELD_value_string_length_set;
*swig_num_childval_get = *LibreDWGc::Dwg_Object_FIELD_num_childval_get;
*swig_num_childval_set = *LibreDWGc::Dwg_Object_FIELD_num_childval_set;
*swig_childval_get = *LibreDWGc::Dwg_Object_FIELD_childval_get;
*swig_childval_set = *LibreDWGc::Dwg_Object_FIELD_childval_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_FIELD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_FIELD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_FIELDLIST ##############

package LibreDWG::Dwg_Object_FIELDLIST;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_FIELDLIST_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_FIELDLIST_parent_set;
*swig_num_fields_get = *LibreDWGc::Dwg_Object_FIELDLIST_num_fields_get;
*swig_num_fields_set = *LibreDWGc::Dwg_Object_FIELDLIST_num_fields_set;
*swig_unknown_get = *LibreDWGc::Dwg_Object_FIELDLIST_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Object_FIELDLIST_unknown_set;
*swig_fields_get = *LibreDWGc::Dwg_Object_FIELDLIST_fields_get;
*swig_fields_set = *LibreDWGc::Dwg_Object_FIELDLIST_fields_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_FIELDLIST(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_FIELDLIST($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_GEODATA_meshpt ##############

package LibreDWG::Dwg_GEODATA_meshpt;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_source_pt_get = *LibreDWGc::Dwg_GEODATA_meshpt_source_pt_get;
*swig_source_pt_set = *LibreDWGc::Dwg_GEODATA_meshpt_source_pt_set;
*swig_dest_pt_get = *LibreDWGc::Dwg_GEODATA_meshpt_dest_pt_get;
*swig_dest_pt_set = *LibreDWGc::Dwg_GEODATA_meshpt_dest_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_GEODATA_meshpt(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_GEODATA_meshpt($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_GEODATA_meshface ##############

package LibreDWG::Dwg_GEODATA_meshface;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_face1_get = *LibreDWGc::Dwg_GEODATA_meshface_face1_get;
*swig_face1_set = *LibreDWGc::Dwg_GEODATA_meshface_face1_set;
*swig_face2_get = *LibreDWGc::Dwg_GEODATA_meshface_face2_get;
*swig_face2_set = *LibreDWGc::Dwg_GEODATA_meshface_face2_set;
*swig_face3_get = *LibreDWGc::Dwg_GEODATA_meshface_face3_get;
*swig_face3_set = *LibreDWGc::Dwg_GEODATA_meshface_face3_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_GEODATA_meshface(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_GEODATA_meshface($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_GEODATA ##############

package LibreDWG::Dwg_Object_GEODATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_GEODATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_GEODATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_GEODATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_GEODATA_class_version_set;
*swig_host_block_get = *LibreDWGc::Dwg_Object_GEODATA_host_block_get;
*swig_host_block_set = *LibreDWGc::Dwg_Object_GEODATA_host_block_set;
*swig_coord_type_get = *LibreDWGc::Dwg_Object_GEODATA_coord_type_get;
*swig_coord_type_set = *LibreDWGc::Dwg_Object_GEODATA_coord_type_set;
*swig_design_pt_get = *LibreDWGc::Dwg_Object_GEODATA_design_pt_get;
*swig_design_pt_set = *LibreDWGc::Dwg_Object_GEODATA_design_pt_set;
*swig_ref_pt_get = *LibreDWGc::Dwg_Object_GEODATA_ref_pt_get;
*swig_ref_pt_set = *LibreDWGc::Dwg_Object_GEODATA_ref_pt_set;
*swig_obs_pt_get = *LibreDWGc::Dwg_Object_GEODATA_obs_pt_get;
*swig_obs_pt_set = *LibreDWGc::Dwg_Object_GEODATA_obs_pt_set;
*swig_scale_vec_get = *LibreDWGc::Dwg_Object_GEODATA_scale_vec_get;
*swig_scale_vec_set = *LibreDWGc::Dwg_Object_GEODATA_scale_vec_set;
*swig_unit_scale_horiz_get = *LibreDWGc::Dwg_Object_GEODATA_unit_scale_horiz_get;
*swig_unit_scale_horiz_set = *LibreDWGc::Dwg_Object_GEODATA_unit_scale_horiz_set;
*swig_units_value_horiz_get = *LibreDWGc::Dwg_Object_GEODATA_units_value_horiz_get;
*swig_units_value_horiz_set = *LibreDWGc::Dwg_Object_GEODATA_units_value_horiz_set;
*swig_unit_scale_vert_get = *LibreDWGc::Dwg_Object_GEODATA_unit_scale_vert_get;
*swig_unit_scale_vert_set = *LibreDWGc::Dwg_Object_GEODATA_unit_scale_vert_set;
*swig_units_value_vert_get = *LibreDWGc::Dwg_Object_GEODATA_units_value_vert_get;
*swig_units_value_vert_set = *LibreDWGc::Dwg_Object_GEODATA_units_value_vert_set;
*swig_up_dir_get = *LibreDWGc::Dwg_Object_GEODATA_up_dir_get;
*swig_up_dir_set = *LibreDWGc::Dwg_Object_GEODATA_up_dir_set;
*swig_north_dir_get = *LibreDWGc::Dwg_Object_GEODATA_north_dir_get;
*swig_north_dir_set = *LibreDWGc::Dwg_Object_GEODATA_north_dir_set;
*swig_scale_est_get = *LibreDWGc::Dwg_Object_GEODATA_scale_est_get;
*swig_scale_est_set = *LibreDWGc::Dwg_Object_GEODATA_scale_est_set;
*swig_user_scale_factor_get = *LibreDWGc::Dwg_Object_GEODATA_user_scale_factor_get;
*swig_user_scale_factor_set = *LibreDWGc::Dwg_Object_GEODATA_user_scale_factor_set;
*swig_do_sea_level_corr_get = *LibreDWGc::Dwg_Object_GEODATA_do_sea_level_corr_get;
*swig_do_sea_level_corr_set = *LibreDWGc::Dwg_Object_GEODATA_do_sea_level_corr_set;
*swig_sea_level_elev_get = *LibreDWGc::Dwg_Object_GEODATA_sea_level_elev_get;
*swig_sea_level_elev_set = *LibreDWGc::Dwg_Object_GEODATA_sea_level_elev_set;
*swig_coord_proj_radius_get = *LibreDWGc::Dwg_Object_GEODATA_coord_proj_radius_get;
*swig_coord_proj_radius_set = *LibreDWGc::Dwg_Object_GEODATA_coord_proj_radius_set;
*swig_coord_system_def_get = *LibreDWGc::Dwg_Object_GEODATA_coord_system_def_get;
*swig_coord_system_def_set = *LibreDWGc::Dwg_Object_GEODATA_coord_system_def_set;
*swig_geo_rss_tag_get = *LibreDWGc::Dwg_Object_GEODATA_geo_rss_tag_get;
*swig_geo_rss_tag_set = *LibreDWGc::Dwg_Object_GEODATA_geo_rss_tag_set;
*swig_coord_system_datum_get = *LibreDWGc::Dwg_Object_GEODATA_coord_system_datum_get;
*swig_coord_system_datum_set = *LibreDWGc::Dwg_Object_GEODATA_coord_system_datum_set;
*swig_coord_system_wkt_get = *LibreDWGc::Dwg_Object_GEODATA_coord_system_wkt_get;
*swig_coord_system_wkt_set = *LibreDWGc::Dwg_Object_GEODATA_coord_system_wkt_set;
*swig_observation_from_tag_get = *LibreDWGc::Dwg_Object_GEODATA_observation_from_tag_get;
*swig_observation_from_tag_set = *LibreDWGc::Dwg_Object_GEODATA_observation_from_tag_set;
*swig_observation_to_tag_get = *LibreDWGc::Dwg_Object_GEODATA_observation_to_tag_get;
*swig_observation_to_tag_set = *LibreDWGc::Dwg_Object_GEODATA_observation_to_tag_set;
*swig_observation_coverage_tag_get = *LibreDWGc::Dwg_Object_GEODATA_observation_coverage_tag_get;
*swig_observation_coverage_tag_set = *LibreDWGc::Dwg_Object_GEODATA_observation_coverage_tag_set;
*swig_num_geomesh_pts_get = *LibreDWGc::Dwg_Object_GEODATA_num_geomesh_pts_get;
*swig_num_geomesh_pts_set = *LibreDWGc::Dwg_Object_GEODATA_num_geomesh_pts_set;
*swig_geomesh_pts_get = *LibreDWGc::Dwg_Object_GEODATA_geomesh_pts_get;
*swig_geomesh_pts_set = *LibreDWGc::Dwg_Object_GEODATA_geomesh_pts_set;
*swig_num_geomesh_faces_get = *LibreDWGc::Dwg_Object_GEODATA_num_geomesh_faces_get;
*swig_num_geomesh_faces_set = *LibreDWGc::Dwg_Object_GEODATA_num_geomesh_faces_set;
*swig_geomesh_faces_get = *LibreDWGc::Dwg_Object_GEODATA_geomesh_faces_get;
*swig_geomesh_faces_set = *LibreDWGc::Dwg_Object_GEODATA_geomesh_faces_set;
*swig_has_civil_data_get = *LibreDWGc::Dwg_Object_GEODATA_has_civil_data_get;
*swig_has_civil_data_set = *LibreDWGc::Dwg_Object_GEODATA_has_civil_data_set;
*swig_obsolete_false_get = *LibreDWGc::Dwg_Object_GEODATA_obsolete_false_get;
*swig_obsolete_false_set = *LibreDWGc::Dwg_Object_GEODATA_obsolete_false_set;
*swig_ref_pt2d_get = *LibreDWGc::Dwg_Object_GEODATA_ref_pt2d_get;
*swig_ref_pt2d_set = *LibreDWGc::Dwg_Object_GEODATA_ref_pt2d_set;
*swig_zero1_get = *LibreDWGc::Dwg_Object_GEODATA_zero1_get;
*swig_zero1_set = *LibreDWGc::Dwg_Object_GEODATA_zero1_set;
*swig_zero2_get = *LibreDWGc::Dwg_Object_GEODATA_zero2_get;
*swig_zero2_set = *LibreDWGc::Dwg_Object_GEODATA_zero2_set;
*swig_unknown1_get = *LibreDWGc::Dwg_Object_GEODATA_unknown1_get;
*swig_unknown1_set = *LibreDWGc::Dwg_Object_GEODATA_unknown1_set;
*swig_unknown2_get = *LibreDWGc::Dwg_Object_GEODATA_unknown2_get;
*swig_unknown2_set = *LibreDWGc::Dwg_Object_GEODATA_unknown2_set;
*swig_unknown_b_get = *LibreDWGc::Dwg_Object_GEODATA_unknown_b_get;
*swig_unknown_b_set = *LibreDWGc::Dwg_Object_GEODATA_unknown_b_set;
*swig_north_dir_angle_deg_get = *LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_deg_get;
*swig_north_dir_angle_deg_set = *LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_deg_set;
*swig_north_dir_angle_rad_get = *LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_rad_get;
*swig_north_dir_angle_rad_set = *LibreDWGc::Dwg_Object_GEODATA_north_dir_angle_rad_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_GEODATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_GEODATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_IDBUFFER ##############

package LibreDWG::Dwg_Object_IDBUFFER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_IDBUFFER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_IDBUFFER_parent_set;
*swig_unknown_get = *LibreDWGc::Dwg_Object_IDBUFFER_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Object_IDBUFFER_unknown_set;
*swig_num_obj_ids_get = *LibreDWGc::Dwg_Object_IDBUFFER_num_obj_ids_get;
*swig_num_obj_ids_set = *LibreDWGc::Dwg_Object_IDBUFFER_num_obj_ids_set;
*swig_obj_ids_get = *LibreDWGc::Dwg_Object_IDBUFFER_obj_ids_get;
*swig_obj_ids_set = *LibreDWGc::Dwg_Object_IDBUFFER_obj_ids_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_IDBUFFER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_IDBUFFER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_IMAGE ##############

package LibreDWG::Dwg_Entity_IMAGE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_IMAGE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_IMAGE_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_IMAGE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_IMAGE_class_version_set;
*swig_pt0_get = *LibreDWGc::Dwg_Entity_IMAGE_pt0_get;
*swig_pt0_set = *LibreDWGc::Dwg_Entity_IMAGE_pt0_set;
*swig_uvec_get = *LibreDWGc::Dwg_Entity_IMAGE_uvec_get;
*swig_uvec_set = *LibreDWGc::Dwg_Entity_IMAGE_uvec_set;
*swig_vvec_get = *LibreDWGc::Dwg_Entity_IMAGE_vvec_get;
*swig_vvec_set = *LibreDWGc::Dwg_Entity_IMAGE_vvec_set;
*swig_image_size_get = *LibreDWGc::Dwg_Entity_IMAGE_image_size_get;
*swig_image_size_set = *LibreDWGc::Dwg_Entity_IMAGE_image_size_set;
*swig_display_props_get = *LibreDWGc::Dwg_Entity_IMAGE_display_props_get;
*swig_display_props_set = *LibreDWGc::Dwg_Entity_IMAGE_display_props_set;
*swig_clipping_get = *LibreDWGc::Dwg_Entity_IMAGE_clipping_get;
*swig_clipping_set = *LibreDWGc::Dwg_Entity_IMAGE_clipping_set;
*swig_brightness_get = *LibreDWGc::Dwg_Entity_IMAGE_brightness_get;
*swig_brightness_set = *LibreDWGc::Dwg_Entity_IMAGE_brightness_set;
*swig_contrast_get = *LibreDWGc::Dwg_Entity_IMAGE_contrast_get;
*swig_contrast_set = *LibreDWGc::Dwg_Entity_IMAGE_contrast_set;
*swig_fade_get = *LibreDWGc::Dwg_Entity_IMAGE_fade_get;
*swig_fade_set = *LibreDWGc::Dwg_Entity_IMAGE_fade_set;
*swig_clip_mode_get = *LibreDWGc::Dwg_Entity_IMAGE_clip_mode_get;
*swig_clip_mode_set = *LibreDWGc::Dwg_Entity_IMAGE_clip_mode_set;
*swig_clip_boundary_type_get = *LibreDWGc::Dwg_Entity_IMAGE_clip_boundary_type_get;
*swig_clip_boundary_type_set = *LibreDWGc::Dwg_Entity_IMAGE_clip_boundary_type_set;
*swig_num_clip_verts_get = *LibreDWGc::Dwg_Entity_IMAGE_num_clip_verts_get;
*swig_num_clip_verts_set = *LibreDWGc::Dwg_Entity_IMAGE_num_clip_verts_set;
*swig_clip_verts_get = *LibreDWGc::Dwg_Entity_IMAGE_clip_verts_get;
*swig_clip_verts_set = *LibreDWGc::Dwg_Entity_IMAGE_clip_verts_set;
*swig_imagedef_get = *LibreDWGc::Dwg_Entity_IMAGE_imagedef_get;
*swig_imagedef_set = *LibreDWGc::Dwg_Entity_IMAGE_imagedef_set;
*swig_imagedefreactor_get = *LibreDWGc::Dwg_Entity_IMAGE_imagedefreactor_get;
*swig_imagedefreactor_set = *LibreDWGc::Dwg_Entity_IMAGE_imagedefreactor_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_IMAGE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_IMAGE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_IMAGEDEF ##############

package LibreDWG::Dwg_Object_IMAGEDEF;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_IMAGEDEF_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_IMAGEDEF_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_IMAGEDEF_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_IMAGEDEF_class_version_set;
*swig_image_size_get = *LibreDWGc::Dwg_Object_IMAGEDEF_image_size_get;
*swig_image_size_set = *LibreDWGc::Dwg_Object_IMAGEDEF_image_size_set;
*swig_file_path_get = *LibreDWGc::Dwg_Object_IMAGEDEF_file_path_get;
*swig_file_path_set = *LibreDWGc::Dwg_Object_IMAGEDEF_file_path_set;
*swig_is_loaded_get = *LibreDWGc::Dwg_Object_IMAGEDEF_is_loaded_get;
*swig_is_loaded_set = *LibreDWGc::Dwg_Object_IMAGEDEF_is_loaded_set;
*swig_resunits_get = *LibreDWGc::Dwg_Object_IMAGEDEF_resunits_get;
*swig_resunits_set = *LibreDWGc::Dwg_Object_IMAGEDEF_resunits_set;
*swig_pixel_size_get = *LibreDWGc::Dwg_Object_IMAGEDEF_pixel_size_get;
*swig_pixel_size_set = *LibreDWGc::Dwg_Object_IMAGEDEF_pixel_size_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_IMAGEDEF(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_IMAGEDEF($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_IMAGEDEF_REACTOR ##############

package LibreDWG::Dwg_Object_IMAGEDEF_REACTOR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_IMAGEDEF_REACTOR_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_IMAGEDEF_REACTOR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_IMAGEDEF_REACTOR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_INDEX ##############

package LibreDWG::Dwg_Object_INDEX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_INDEX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_INDEX_parent_set;
*swig_last_updated_get = *LibreDWGc::Dwg_Object_INDEX_last_updated_get;
*swig_last_updated_set = *LibreDWGc::Dwg_Object_INDEX_last_updated_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_INDEX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_INDEX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LAYER_entry ##############

package LibreDWG::Dwg_LAYER_entry;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LAYER_entry_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LAYER_entry_parent_set;
*swig_numlayers_get = *LibreDWGc::Dwg_LAYER_entry_numlayers_get;
*swig_numlayers_set = *LibreDWGc::Dwg_LAYER_entry_numlayers_set;
*swig_name_get = *LibreDWGc::Dwg_LAYER_entry_name_get;
*swig_name_set = *LibreDWGc::Dwg_LAYER_entry_name_set;
*swig_handle_get = *LibreDWGc::Dwg_LAYER_entry_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_LAYER_entry_handle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LAYER_entry(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LAYER_entry($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LAYER_INDEX ##############

package LibreDWG::Dwg_Object_LAYER_INDEX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LAYER_INDEX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LAYER_INDEX_parent_set;
*swig_last_updated_get = *LibreDWGc::Dwg_Object_LAYER_INDEX_last_updated_get;
*swig_last_updated_set = *LibreDWGc::Dwg_Object_LAYER_INDEX_last_updated_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_LAYER_INDEX_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_LAYER_INDEX_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_LAYER_INDEX_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_LAYER_INDEX_entries_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LAYER_INDEX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LAYER_INDEX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LWPOLYLINE ##############

package LibreDWG::Dwg_Entity_LWPOLYLINE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_flag_set;
*swig_const_width_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_const_width_get;
*swig_const_width_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_const_width_set;
*swig_elevation_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_elevation_get;
*swig_elevation_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_elevation_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_thickness_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_extrusion_set;
*swig_num_points_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_points_get;
*swig_num_points_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_points_set;
*swig_points_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_points_get;
*swig_points_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_points_set;
*swig_num_bulges_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_bulges_get;
*swig_num_bulges_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_bulges_set;
*swig_bulges_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_bulges_get;
*swig_bulges_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_bulges_set;
*swig_num_vertexids_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_vertexids_get;
*swig_num_vertexids_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_vertexids_set;
*swig_vertexids_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_vertexids_get;
*swig_vertexids_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_vertexids_set;
*swig_num_widths_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_widths_get;
*swig_num_widths_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_num_widths_set;
*swig_widths_get = *LibreDWGc::Dwg_Entity_LWPOLYLINE_widths_get;
*swig_widths_set = *LibreDWGc::Dwg_Entity_LWPOLYLINE_widths_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LWPOLYLINE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LWPOLYLINE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RASTERVARIABLES ##############

package LibreDWG::Dwg_Object_RASTERVARIABLES;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RASTERVARIABLES_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RASTERVARIABLES_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RASTERVARIABLES_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RASTERVARIABLES_class_version_set;
*swig_image_frame_get = *LibreDWGc::Dwg_Object_RASTERVARIABLES_image_frame_get;
*swig_image_frame_set = *LibreDWGc::Dwg_Object_RASTERVARIABLES_image_frame_set;
*swig_image_quality_get = *LibreDWGc::Dwg_Object_RASTERVARIABLES_image_quality_get;
*swig_image_quality_set = *LibreDWGc::Dwg_Object_RASTERVARIABLES_image_quality_set;
*swig_units_get = *LibreDWGc::Dwg_Object_RASTERVARIABLES_units_get;
*swig_units_set = *LibreDWGc::Dwg_Object_RASTERVARIABLES_units_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RASTERVARIABLES(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RASTERVARIABLES($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SCALE ##############

package LibreDWG::Dwg_Object_SCALE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SCALE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SCALE_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_SCALE_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_SCALE_flag_set;
*swig_name_get = *LibreDWGc::Dwg_Object_SCALE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_SCALE_name_set;
*swig_paper_units_get = *LibreDWGc::Dwg_Object_SCALE_paper_units_get;
*swig_paper_units_set = *LibreDWGc::Dwg_Object_SCALE_paper_units_set;
*swig_drawing_units_get = *LibreDWGc::Dwg_Object_SCALE_drawing_units_get;
*swig_drawing_units_set = *LibreDWGc::Dwg_Object_SCALE_drawing_units_set;
*swig_is_unit_scale_get = *LibreDWGc::Dwg_Object_SCALE_is_unit_scale_get;
*swig_is_unit_scale_set = *LibreDWGc::Dwg_Object_SCALE_is_unit_scale_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SCALE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SCALE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SORTENTSTABLE ##############

package LibreDWG::Dwg_Object_SORTENTSTABLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SORTENTSTABLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SORTENTSTABLE_parent_set;
*swig_num_ents_get = *LibreDWGc::Dwg_Object_SORTENTSTABLE_num_ents_get;
*swig_num_ents_set = *LibreDWGc::Dwg_Object_SORTENTSTABLE_num_ents_set;
*swig_sort_ents_get = *LibreDWGc::Dwg_Object_SORTENTSTABLE_sort_ents_get;
*swig_sort_ents_set = *LibreDWGc::Dwg_Object_SORTENTSTABLE_sort_ents_set;
*swig_block_owner_get = *LibreDWGc::Dwg_Object_SORTENTSTABLE_block_owner_get;
*swig_block_owner_set = *LibreDWGc::Dwg_Object_SORTENTSTABLE_block_owner_set;
*swig_ents_get = *LibreDWGc::Dwg_Object_SORTENTSTABLE_ents_get;
*swig_ents_set = *LibreDWGc::Dwg_Object_SORTENTSTABLE_ents_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SORTENTSTABLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SORTENTSTABLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SPATIAL_FILTER ##############

package LibreDWG::Dwg_Object_SPATIAL_FILTER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_parent_set;
*swig_num_clip_verts_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_num_clip_verts_get;
*swig_num_clip_verts_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_num_clip_verts_set;
*swig_clip_verts_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_clip_verts_get;
*swig_clip_verts_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_clip_verts_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_extrusion_set;
*swig_origin_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_origin_get;
*swig_origin_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_origin_set;
*swig_display_boundary_on_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_display_boundary_on_get;
*swig_display_boundary_on_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_display_boundary_on_set;
*swig_front_clip_on_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_on_get;
*swig_front_clip_on_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_on_set;
*swig_front_clip_z_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_z_get;
*swig_front_clip_z_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_front_clip_z_set;
*swig_back_clip_on_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_on_get;
*swig_back_clip_on_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_on_set;
*swig_back_clip_z_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_z_get;
*swig_back_clip_z_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_back_clip_z_set;
*swig_inverse_transform_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_inverse_transform_get;
*swig_inverse_transform_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_inverse_transform_set;
*swig_transform_get = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_transform_get;
*swig_transform_set = *LibreDWGc::Dwg_Object_SPATIAL_FILTER_transform_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SPATIAL_FILTER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SPATIAL_FILTER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SPATIAL_INDEX ##############

package LibreDWG::Dwg_Object_SPATIAL_INDEX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_parent_set;
*swig_last_updated_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_last_updated_get;
*swig_last_updated_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_last_updated_set;
*swig_num1_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num1_get;
*swig_num1_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num1_set;
*swig_num2_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num2_get;
*swig_num2_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num2_set;
*swig_num3_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num3_get;
*swig_num3_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num3_set;
*swig_num4_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num4_get;
*swig_num4_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num4_set;
*swig_num5_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num5_get;
*swig_num5_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num5_set;
*swig_num6_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num6_get;
*swig_num6_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num6_set;
*swig_num_hdls_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num_hdls_get;
*swig_num_hdls_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_num_hdls_set;
*swig_hdls_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_hdls_get;
*swig_hdls_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_hdls_set;
*swig_bindata_size_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_size_get;
*swig_bindata_size_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_size_set;
*swig_bindata_get = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_get;
*swig_bindata_set = *LibreDWGc::Dwg_Object_SPATIAL_INDEX_bindata_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SPATIAL_INDEX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SPATIAL_INDEX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_WIPEOUT ##############

package LibreDWG::Dwg_Entity_WIPEOUT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_WIPEOUT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_WIPEOUT_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_WIPEOUT_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_WIPEOUT_class_version_set;
*swig_pt0_get = *LibreDWGc::Dwg_Entity_WIPEOUT_pt0_get;
*swig_pt0_set = *LibreDWGc::Dwg_Entity_WIPEOUT_pt0_set;
*swig_uvec_get = *LibreDWGc::Dwg_Entity_WIPEOUT_uvec_get;
*swig_uvec_set = *LibreDWGc::Dwg_Entity_WIPEOUT_uvec_set;
*swig_vvec_get = *LibreDWGc::Dwg_Entity_WIPEOUT_vvec_get;
*swig_vvec_set = *LibreDWGc::Dwg_Entity_WIPEOUT_vvec_set;
*swig_image_size_get = *LibreDWGc::Dwg_Entity_WIPEOUT_image_size_get;
*swig_image_size_set = *LibreDWGc::Dwg_Entity_WIPEOUT_image_size_set;
*swig_display_props_get = *LibreDWGc::Dwg_Entity_WIPEOUT_display_props_get;
*swig_display_props_set = *LibreDWGc::Dwg_Entity_WIPEOUT_display_props_set;
*swig_clipping_get = *LibreDWGc::Dwg_Entity_WIPEOUT_clipping_get;
*swig_clipping_set = *LibreDWGc::Dwg_Entity_WIPEOUT_clipping_set;
*swig_brightness_get = *LibreDWGc::Dwg_Entity_WIPEOUT_brightness_get;
*swig_brightness_set = *LibreDWGc::Dwg_Entity_WIPEOUT_brightness_set;
*swig_contrast_get = *LibreDWGc::Dwg_Entity_WIPEOUT_contrast_get;
*swig_contrast_set = *LibreDWGc::Dwg_Entity_WIPEOUT_contrast_set;
*swig_fade_get = *LibreDWGc::Dwg_Entity_WIPEOUT_fade_get;
*swig_fade_set = *LibreDWGc::Dwg_Entity_WIPEOUT_fade_set;
*swig_clip_mode_get = *LibreDWGc::Dwg_Entity_WIPEOUT_clip_mode_get;
*swig_clip_mode_set = *LibreDWGc::Dwg_Entity_WIPEOUT_clip_mode_set;
*swig_clip_boundary_type_get = *LibreDWGc::Dwg_Entity_WIPEOUT_clip_boundary_type_get;
*swig_clip_boundary_type_set = *LibreDWGc::Dwg_Entity_WIPEOUT_clip_boundary_type_set;
*swig_num_clip_verts_get = *LibreDWGc::Dwg_Entity_WIPEOUT_num_clip_verts_get;
*swig_num_clip_verts_set = *LibreDWGc::Dwg_Entity_WIPEOUT_num_clip_verts_set;
*swig_clip_verts_get = *LibreDWGc::Dwg_Entity_WIPEOUT_clip_verts_get;
*swig_clip_verts_set = *LibreDWGc::Dwg_Entity_WIPEOUT_clip_verts_set;
*swig_imagedef_get = *LibreDWGc::Dwg_Entity_WIPEOUT_imagedef_get;
*swig_imagedef_set = *LibreDWGc::Dwg_Entity_WIPEOUT_imagedef_set;
*swig_imagedefreactor_get = *LibreDWGc::Dwg_Entity_WIPEOUT_imagedefreactor_get;
*swig_imagedefreactor_set = *LibreDWGc::Dwg_Entity_WIPEOUT_imagedefreactor_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_WIPEOUT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_WIPEOUT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_WIPEOUTVARIABLES ##############

package LibreDWG::Dwg_Object_WIPEOUTVARIABLES;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_parent_set;
*swig_display_frame_get = *LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_display_frame_get;
*swig_display_frame_set = *LibreDWGc::Dwg_Object_WIPEOUTVARIABLES_display_frame_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_WIPEOUTVARIABLES(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_WIPEOUTVARIABLES($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_SECTIONOBJECT ##############

package LibreDWG::Dwg_Entity_SECTIONOBJECT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_parent_set;
*swig_state_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_state_get;
*swig_state_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_state_set;
*swig_flags_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_flags_set;
*swig_name_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_name_get;
*swig_name_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_name_set;
*swig_vert_dir_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_vert_dir_get;
*swig_vert_dir_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_vert_dir_set;
*swig_top_height_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_top_height_get;
*swig_top_height_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_top_height_set;
*swig_bottom_height_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_bottom_height_get;
*swig_bottom_height_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_bottom_height_set;
*swig_indicator_alpha_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_alpha_get;
*swig_indicator_alpha_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_alpha_set;
*swig_indicator_color_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_color_get;
*swig_indicator_color_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_indicator_color_set;
*swig_num_verts_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_verts_get;
*swig_num_verts_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_verts_set;
*swig_verts_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_verts_get;
*swig_verts_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_verts_set;
*swig_num_blverts_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_blverts_get;
*swig_num_blverts_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_num_blverts_set;
*swig_blverts_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_blverts_get;
*swig_blverts_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_blverts_set;
*swig_section_settings_get = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_section_settings_get;
*swig_section_settings_set = *LibreDWGc::Dwg_Entity_SECTIONOBJECT_section_settings_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_SECTIONOBJECT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_SECTIONOBJECT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_VISUALSTYLE ##############

package LibreDWG::Dwg_Object_VISUALSTYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_parent_set;
*swig_description_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_description_set;
*swig_style_type_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_style_type_get;
*swig_style_type_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_style_type_set;
*swig_ext_lighting_model_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_ext_lighting_model_get;
*swig_ext_lighting_model_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_ext_lighting_model_set;
*swig_internal_only_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_internal_only_get;
*swig_internal_only_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_internal_only_set;
*swig_face_lighting_model_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_get;
*swig_face_lighting_model_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_set;
*swig_face_lighting_model_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_int_get;
*swig_face_lighting_model_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_model_int_set;
*swig_face_lighting_quality_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_get;
*swig_face_lighting_quality_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_set;
*swig_face_lighting_quality_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_int_get;
*swig_face_lighting_quality_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_lighting_quality_int_set;
*swig_face_color_mode_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_get;
*swig_face_color_mode_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_set;
*swig_face_color_mode_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_int_get;
*swig_face_color_mode_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_color_mode_int_set;
*swig_face_opacity_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_get;
*swig_face_opacity_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_set;
*swig_face_opacity_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_int_get;
*swig_face_opacity_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_opacity_int_set;
*swig_face_specular_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_get;
*swig_face_specular_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_set;
*swig_face_specular_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_int_get;
*swig_face_specular_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_specular_int_set;
*swig_face_modifier_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_get;
*swig_face_modifier_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_set;
*swig_face_modifier_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_int_get;
*swig_face_modifier_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_modifier_int_set;
*swig_face_mono_color_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_get;
*swig_face_mono_color_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_set;
*swig_face_mono_color_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_int_get;
*swig_face_mono_color_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_face_mono_color_int_set;
*swig_edge_model_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_get;
*swig_edge_model_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_set;
*swig_edge_model_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_int_get;
*swig_edge_model_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_model_int_set;
*swig_edge_style_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_get;
*swig_edge_style_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_set;
*swig_edge_style_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_int_get;
*swig_edge_style_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_int_set;
*swig_edge_intersection_color_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_get;
*swig_edge_intersection_color_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_set;
*swig_edge_intersection_color_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_int_get;
*swig_edge_intersection_color_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_color_int_set;
*swig_edge_obscured_color_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_get;
*swig_edge_obscured_color_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_set;
*swig_edge_obscured_color_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_int_get;
*swig_edge_obscured_color_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_color_int_set;
*swig_edge_obscured_ltype_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_get;
*swig_edge_obscured_ltype_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_set;
*swig_edge_obscured_ltype_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_get;
*swig_edge_obscured_ltype_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_obscured_ltype_int_set;
*swig_edge_intersection_ltype_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_get;
*swig_edge_intersection_ltype_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_set;
*swig_edge_intersection_ltype_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_get;
*swig_edge_intersection_ltype_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_intersection_ltype_int_set;
*swig_edge_crease_angle_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_get;
*swig_edge_crease_angle_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_set;
*swig_edge_crease_angle_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_int_get;
*swig_edge_crease_angle_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_crease_angle_int_set;
*swig_edge_modifier_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_get;
*swig_edge_modifier_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_set;
*swig_edge_modifier_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_int_get;
*swig_edge_modifier_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_modifier_int_set;
*swig_edge_color_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_get;
*swig_edge_color_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_set;
*swig_edge_color_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_int_get;
*swig_edge_color_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_color_int_set;
*swig_edge_opacity_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_get;
*swig_edge_opacity_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_set;
*swig_edge_opacity_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_int_get;
*swig_edge_opacity_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_opacity_int_set;
*swig_edge_width_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_get;
*swig_edge_width_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_set;
*swig_edge_width_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_int_get;
*swig_edge_width_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_width_int_set;
*swig_edge_overhang_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_get;
*swig_edge_overhang_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_set;
*swig_edge_overhang_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_int_get;
*swig_edge_overhang_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_overhang_int_set;
*swig_edge_jitter_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_get;
*swig_edge_jitter_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_set;
*swig_edge_jitter_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_int_get;
*swig_edge_jitter_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_jitter_int_set;
*swig_edge_silhouette_color_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_get;
*swig_edge_silhouette_color_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_set;
*swig_edge_silhouette_color_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_get;
*swig_edge_silhouette_color_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_color_int_set;
*swig_edge_silhouette_width_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_get;
*swig_edge_silhouette_width_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_set;
*swig_edge_silhouette_width_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_get;
*swig_edge_silhouette_width_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_silhouette_width_int_set;
*swig_edge_halo_gap_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_get;
*swig_edge_halo_gap_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_set;
*swig_edge_halo_gap_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_int_get;
*swig_edge_halo_gap_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_halo_gap_int_set;
*swig_edge_isolines_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_get;
*swig_edge_isolines_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_set;
*swig_edge_isolines_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_int_get;
*swig_edge_isolines_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_isolines_int_set;
*swig_edge_do_hide_precision_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_get;
*swig_edge_do_hide_precision_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_set;
*swig_edge_do_hide_precision_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_get;
*swig_edge_do_hide_precision_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_do_hide_precision_int_set;
*swig_edge_style_apply_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_get;
*swig_edge_style_apply_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_set;
*swig_edge_style_apply_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_int_get;
*swig_edge_style_apply_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_style_apply_int_set;
*swig_display_settings_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_get;
*swig_display_settings_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_set;
*swig_display_settings_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_int_get;
*swig_display_settings_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_settings_int_set;
*swig_display_brightness_bl_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_bl_get;
*swig_display_brightness_bl_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_bl_set;
*swig_display_brightness_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_get;
*swig_display_brightness_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_set;
*swig_display_brightness_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_int_get;
*swig_display_brightness_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_brightness_int_set;
*swig_display_shadow_type_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_get;
*swig_display_shadow_type_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_set;
*swig_display_shadow_type_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_int_get;
*swig_display_shadow_type_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_display_shadow_type_int_set;
*swig_bd2007_45_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd2007_45_get;
*swig_bd2007_45_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd2007_45_set;
*swig_num_props_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_num_props_get;
*swig_num_props_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_num_props_set;
*swig_b_prop1c_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_get;
*swig_b_prop1c_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_set;
*swig_b_prop1c_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_int_get;
*swig_b_prop1c_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1c_int_set;
*swig_b_prop1d_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_get;
*swig_b_prop1d_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_set;
*swig_b_prop1d_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_int_get;
*swig_b_prop1d_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1d_int_set;
*swig_b_prop1e_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_get;
*swig_b_prop1e_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_set;
*swig_b_prop1e_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_int_get;
*swig_b_prop1e_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1e_int_set;
*swig_b_prop1f_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_get;
*swig_b_prop1f_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_set;
*swig_b_prop1f_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_int_get;
*swig_b_prop1f_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop1f_int_set;
*swig_b_prop20_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_get;
*swig_b_prop20_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_set;
*swig_b_prop20_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_int_get;
*swig_b_prop20_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop20_int_set;
*swig_b_prop21_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_get;
*swig_b_prop21_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_set;
*swig_b_prop21_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_int_get;
*swig_b_prop21_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop21_int_set;
*swig_b_prop22_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_get;
*swig_b_prop22_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_set;
*swig_b_prop22_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_int_get;
*swig_b_prop22_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop22_int_set;
*swig_b_prop23_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_get;
*swig_b_prop23_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_set;
*swig_b_prop23_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_int_get;
*swig_b_prop23_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop23_int_set;
*swig_b_prop24_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_get;
*swig_b_prop24_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_set;
*swig_b_prop24_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_int_get;
*swig_b_prop24_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop24_int_set;
*swig_bl_prop25_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_get;
*swig_bl_prop25_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_set;
*swig_bl_prop25_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_int_get;
*swig_bl_prop25_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop25_int_set;
*swig_bd_prop26_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_get;
*swig_bd_prop26_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_set;
*swig_bd_prop26_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_int_get;
*swig_bd_prop26_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop26_int_set;
*swig_bd_prop27_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_get;
*swig_bd_prop27_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_set;
*swig_bd_prop27_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_int_get;
*swig_bd_prop27_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop27_int_set;
*swig_bl_prop28_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_get;
*swig_bl_prop28_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_set;
*swig_bl_prop28_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_int_get;
*swig_bl_prop28_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop28_int_set;
*swig_c_prop29_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_get;
*swig_c_prop29_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_set;
*swig_c_prop29_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_int_get;
*swig_c_prop29_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop29_int_set;
*swig_bl_prop2a_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_get;
*swig_bl_prop2a_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_set;
*swig_bl_prop2a_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_int_get;
*swig_bl_prop2a_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2a_int_set;
*swig_bl_prop2b_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_get;
*swig_bl_prop2b_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_set;
*swig_bl_prop2b_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_int_get;
*swig_bl_prop2b_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2b_int_set;
*swig_c_prop2c_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_get;
*swig_c_prop2c_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_set;
*swig_c_prop2c_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_int_get;
*swig_c_prop2c_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop2c_int_set;
*swig_b_prop2d_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_get;
*swig_b_prop2d_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_set;
*swig_b_prop2d_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_int_get;
*swig_b_prop2d_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop2d_int_set;
*swig_bl_prop2e_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_get;
*swig_bl_prop2e_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_set;
*swig_bl_prop2e_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_int_get;
*swig_bl_prop2e_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2e_int_set;
*swig_bl_prop2f_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_get;
*swig_bl_prop2f_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_set;
*swig_bl_prop2f_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_int_get;
*swig_bl_prop2f_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop2f_int_set;
*swig_bl_prop30_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_get;
*swig_bl_prop30_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_set;
*swig_bl_prop30_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_int_get;
*swig_bl_prop30_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop30_int_set;
*swig_b_prop31_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_get;
*swig_b_prop31_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_set;
*swig_b_prop31_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_int_get;
*swig_b_prop31_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop31_int_set;
*swig_bl_prop32_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_get;
*swig_bl_prop32_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_set;
*swig_bl_prop32_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_int_get;
*swig_bl_prop32_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bl_prop32_int_set;
*swig_c_prop33_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_get;
*swig_c_prop33_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_set;
*swig_c_prop33_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_int_get;
*swig_c_prop33_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_c_prop33_int_set;
*swig_bd_prop34_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_get;
*swig_bd_prop34_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_set;
*swig_bd_prop34_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_int_get;
*swig_bd_prop34_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop34_int_set;
*swig_edge_wiggle_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_get;
*swig_edge_wiggle_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_set;
*swig_edge_wiggle_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_int_get;
*swig_edge_wiggle_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_edge_wiggle_int_set;
*swig_strokes_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_get;
*swig_strokes_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_set;
*swig_strokes_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_int_get;
*swig_strokes_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_strokes_int_set;
*swig_b_prop37_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_get;
*swig_b_prop37_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_set;
*swig_b_prop37_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_int_get;
*swig_b_prop37_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_b_prop37_int_set;
*swig_bd_prop38_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_get;
*swig_bd_prop38_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_set;
*swig_bd_prop38_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_int_get;
*swig_bd_prop38_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop38_int_set;
*swig_bd_prop39_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_get;
*swig_bd_prop39_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_set;
*swig_bd_prop39_int_get = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_int_get;
*swig_bd_prop39_int_set = *LibreDWGc::Dwg_Object_VISUALSTYLE_bd_prop39_int_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_VISUALSTYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_VISUALSTYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_LIGHTLIST_light ##############

package LibreDWG::Dwg_LIGHTLIST_light;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_LIGHTLIST_light_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_LIGHTLIST_light_parent_set;
*swig_name_get = *LibreDWGc::Dwg_LIGHTLIST_light_name_get;
*swig_name_set = *LibreDWGc::Dwg_LIGHTLIST_light_name_set;
*swig_handle_get = *LibreDWGc::Dwg_LIGHTLIST_light_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_LIGHTLIST_light_handle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_LIGHTLIST_light(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_LIGHTLIST_light($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LIGHTLIST ##############

package LibreDWG::Dwg_Object_LIGHTLIST;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LIGHTLIST_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LIGHTLIST_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_LIGHTLIST_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_LIGHTLIST_class_version_set;
*swig_num_lights_get = *LibreDWGc::Dwg_Object_LIGHTLIST_num_lights_get;
*swig_num_lights_set = *LibreDWGc::Dwg_Object_LIGHTLIST_num_lights_set;
*swig_lights_get = *LibreDWGc::Dwg_Object_LIGHTLIST_lights_get;
*swig_lights_set = *LibreDWGc::Dwg_Object_LIGHTLIST_lights_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LIGHTLIST(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LIGHTLIST($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MATERIAL_color ##############

package LibreDWG::Dwg_MATERIAL_color;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_MATERIAL_color_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_MATERIAL_color_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_MATERIAL_color_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_MATERIAL_color_flag_set;
*swig_factor_get = *LibreDWGc::Dwg_MATERIAL_color_factor_get;
*swig_factor_set = *LibreDWGc::Dwg_MATERIAL_color_factor_set;
*swig_rgb_get = *LibreDWGc::Dwg_MATERIAL_color_rgb_get;
*swig_rgb_set = *LibreDWGc::Dwg_MATERIAL_color_rgb_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MATERIAL_color(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MATERIAL_color($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MATERIAL_mapper ##############

package LibreDWG::Dwg_MATERIAL_mapper;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_MATERIAL_mapper_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_MATERIAL_mapper_parent_set;
*swig_blendfactor_get = *LibreDWGc::Dwg_MATERIAL_mapper_blendfactor_get;
*swig_blendfactor_set = *LibreDWGc::Dwg_MATERIAL_mapper_blendfactor_set;
*swig_transmatrix_get = *LibreDWGc::Dwg_MATERIAL_mapper_transmatrix_get;
*swig_transmatrix_set = *LibreDWGc::Dwg_MATERIAL_mapper_transmatrix_set;
*swig_filename_get = *LibreDWGc::Dwg_MATERIAL_mapper_filename_get;
*swig_filename_set = *LibreDWGc::Dwg_MATERIAL_mapper_filename_set;
*swig_color1_get = *LibreDWGc::Dwg_MATERIAL_mapper_color1_get;
*swig_color1_set = *LibreDWGc::Dwg_MATERIAL_mapper_color1_set;
*swig_color2_get = *LibreDWGc::Dwg_MATERIAL_mapper_color2_get;
*swig_color2_set = *LibreDWGc::Dwg_MATERIAL_mapper_color2_set;
*swig_source_get = *LibreDWGc::Dwg_MATERIAL_mapper_source_get;
*swig_source_set = *LibreDWGc::Dwg_MATERIAL_mapper_source_set;
*swig_projection_get = *LibreDWGc::Dwg_MATERIAL_mapper_projection_get;
*swig_projection_set = *LibreDWGc::Dwg_MATERIAL_mapper_projection_set;
*swig_tiling_get = *LibreDWGc::Dwg_MATERIAL_mapper_tiling_get;
*swig_tiling_set = *LibreDWGc::Dwg_MATERIAL_mapper_tiling_set;
*swig_autotransform_get = *LibreDWGc::Dwg_MATERIAL_mapper_autotransform_get;
*swig_autotransform_set = *LibreDWGc::Dwg_MATERIAL_mapper_autotransform_set;
*swig_texturemode_get = *LibreDWGc::Dwg_MATERIAL_mapper_texturemode_get;
*swig_texturemode_set = *LibreDWGc::Dwg_MATERIAL_mapper_texturemode_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MATERIAL_mapper(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MATERIAL_mapper($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MATERIAL_gentexture ##############

package LibreDWG::Dwg_MATERIAL_gentexture;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_MATERIAL_gentexture_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_MATERIAL_gentexture_parent_set;
*swig_genprocname_get = *LibreDWGc::Dwg_MATERIAL_gentexture_genprocname_get;
*swig_genprocname_set = *LibreDWGc::Dwg_MATERIAL_gentexture_genprocname_set;
*swig_material_get = *LibreDWGc::Dwg_MATERIAL_gentexture_material_get;
*swig_material_set = *LibreDWGc::Dwg_MATERIAL_gentexture_material_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MATERIAL_gentexture(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MATERIAL_gentexture($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MATERIAL ##############

package LibreDWG::Dwg_Object_MATERIAL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MATERIAL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MATERIAL_parent_set;
*swig_name_get = *LibreDWGc::Dwg_Object_MATERIAL_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_MATERIAL_name_set;
*swig_description_get = *LibreDWGc::Dwg_Object_MATERIAL_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_MATERIAL_description_set;
*swig_ambient_color_get = *LibreDWGc::Dwg_Object_MATERIAL_ambient_color_get;
*swig_ambient_color_set = *LibreDWGc::Dwg_Object_MATERIAL_ambient_color_set;
*swig_diffuse_color_get = *LibreDWGc::Dwg_Object_MATERIAL_diffuse_color_get;
*swig_diffuse_color_set = *LibreDWGc::Dwg_Object_MATERIAL_diffuse_color_set;
*swig_diffusemap_get = *LibreDWGc::Dwg_Object_MATERIAL_diffusemap_get;
*swig_diffusemap_set = *LibreDWGc::Dwg_Object_MATERIAL_diffusemap_set;
*swig_specular_gloss_factor_get = *LibreDWGc::Dwg_Object_MATERIAL_specular_gloss_factor_get;
*swig_specular_gloss_factor_set = *LibreDWGc::Dwg_Object_MATERIAL_specular_gloss_factor_set;
*swig_specular_color_get = *LibreDWGc::Dwg_Object_MATERIAL_specular_color_get;
*swig_specular_color_set = *LibreDWGc::Dwg_Object_MATERIAL_specular_color_set;
*swig_specularmap_get = *LibreDWGc::Dwg_Object_MATERIAL_specularmap_get;
*swig_specularmap_set = *LibreDWGc::Dwg_Object_MATERIAL_specularmap_set;
*swig_reflectionmap_get = *LibreDWGc::Dwg_Object_MATERIAL_reflectionmap_get;
*swig_reflectionmap_set = *LibreDWGc::Dwg_Object_MATERIAL_reflectionmap_set;
*swig_opacity_percent_get = *LibreDWGc::Dwg_Object_MATERIAL_opacity_percent_get;
*swig_opacity_percent_set = *LibreDWGc::Dwg_Object_MATERIAL_opacity_percent_set;
*swig_opacitymap_get = *LibreDWGc::Dwg_Object_MATERIAL_opacitymap_get;
*swig_opacitymap_set = *LibreDWGc::Dwg_Object_MATERIAL_opacitymap_set;
*swig_bumpmap_get = *LibreDWGc::Dwg_Object_MATERIAL_bumpmap_get;
*swig_bumpmap_set = *LibreDWGc::Dwg_Object_MATERIAL_bumpmap_set;
*swig_refraction_index_get = *LibreDWGc::Dwg_Object_MATERIAL_refraction_index_get;
*swig_refraction_index_set = *LibreDWGc::Dwg_Object_MATERIAL_refraction_index_set;
*swig_refractionmap_get = *LibreDWGc::Dwg_Object_MATERIAL_refractionmap_get;
*swig_refractionmap_set = *LibreDWGc::Dwg_Object_MATERIAL_refractionmap_set;
*swig_color_bleed_scale_get = *LibreDWGc::Dwg_Object_MATERIAL_color_bleed_scale_get;
*swig_color_bleed_scale_set = *LibreDWGc::Dwg_Object_MATERIAL_color_bleed_scale_set;
*swig_indirect_bump_scale_get = *LibreDWGc::Dwg_Object_MATERIAL_indirect_bump_scale_get;
*swig_indirect_bump_scale_set = *LibreDWGc::Dwg_Object_MATERIAL_indirect_bump_scale_set;
*swig_reflectance_scale_get = *LibreDWGc::Dwg_Object_MATERIAL_reflectance_scale_get;
*swig_reflectance_scale_set = *LibreDWGc::Dwg_Object_MATERIAL_reflectance_scale_set;
*swig_transmittance_scale_get = *LibreDWGc::Dwg_Object_MATERIAL_transmittance_scale_get;
*swig_transmittance_scale_set = *LibreDWGc::Dwg_Object_MATERIAL_transmittance_scale_set;
*swig_two_sided_material_get = *LibreDWGc::Dwg_Object_MATERIAL_two_sided_material_get;
*swig_two_sided_material_set = *LibreDWGc::Dwg_Object_MATERIAL_two_sided_material_set;
*swig_luminance_get = *LibreDWGc::Dwg_Object_MATERIAL_luminance_get;
*swig_luminance_set = *LibreDWGc::Dwg_Object_MATERIAL_luminance_set;
*swig_luminance_mode_get = *LibreDWGc::Dwg_Object_MATERIAL_luminance_mode_get;
*swig_luminance_mode_set = *LibreDWGc::Dwg_Object_MATERIAL_luminance_mode_set;
*swig_translucence_get = *LibreDWGc::Dwg_Object_MATERIAL_translucence_get;
*swig_translucence_set = *LibreDWGc::Dwg_Object_MATERIAL_translucence_set;
*swig_self_illumination_get = *LibreDWGc::Dwg_Object_MATERIAL_self_illumination_get;
*swig_self_illumination_set = *LibreDWGc::Dwg_Object_MATERIAL_self_illumination_set;
*swig_reflectivity_get = *LibreDWGc::Dwg_Object_MATERIAL_reflectivity_get;
*swig_reflectivity_set = *LibreDWGc::Dwg_Object_MATERIAL_reflectivity_set;
*swig_illumination_model_get = *LibreDWGc::Dwg_Object_MATERIAL_illumination_model_get;
*swig_illumination_model_set = *LibreDWGc::Dwg_Object_MATERIAL_illumination_model_set;
*swig_channel_flags_get = *LibreDWGc::Dwg_Object_MATERIAL_channel_flags_get;
*swig_channel_flags_set = *LibreDWGc::Dwg_Object_MATERIAL_channel_flags_set;
*swig_mode_get = *LibreDWGc::Dwg_Object_MATERIAL_mode_get;
*swig_mode_set = *LibreDWGc::Dwg_Object_MATERIAL_mode_set;
*swig_genprocname_get = *LibreDWGc::Dwg_Object_MATERIAL_genprocname_get;
*swig_genprocname_set = *LibreDWGc::Dwg_Object_MATERIAL_genprocname_set;
*swig_genproctype_get = *LibreDWGc::Dwg_Object_MATERIAL_genproctype_get;
*swig_genproctype_set = *LibreDWGc::Dwg_Object_MATERIAL_genproctype_set;
*swig_genprocvalbool_get = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalbool_get;
*swig_genprocvalbool_set = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalbool_set;
*swig_genprocvalint_get = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalint_get;
*swig_genprocvalint_set = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalint_set;
*swig_genprocvalreal_get = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalreal_get;
*swig_genprocvalreal_set = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalreal_set;
*swig_genprocvaltext_get = *LibreDWGc::Dwg_Object_MATERIAL_genprocvaltext_get;
*swig_genprocvaltext_set = *LibreDWGc::Dwg_Object_MATERIAL_genprocvaltext_set;
*swig_genprocvalcolor_get = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalcolor_get;
*swig_genprocvalcolor_set = *LibreDWGc::Dwg_Object_MATERIAL_genprocvalcolor_set;
*swig_genproctableend_get = *LibreDWGc::Dwg_Object_MATERIAL_genproctableend_get;
*swig_genproctableend_set = *LibreDWGc::Dwg_Object_MATERIAL_genproctableend_set;
*swig_num_gentextures_get = *LibreDWGc::Dwg_Object_MATERIAL_num_gentextures_get;
*swig_num_gentextures_set = *LibreDWGc::Dwg_Object_MATERIAL_num_gentextures_set;
*swig_gentextures_get = *LibreDWGc::Dwg_Object_MATERIAL_gentextures_get;
*swig_gentextures_set = *LibreDWGc::Dwg_Object_MATERIAL_gentextures_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MATERIAL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MATERIAL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_OBJECT_PTR ##############

package LibreDWG::Dwg_Object_OBJECT_PTR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_OBJECT_PTR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_OBJECT_PTR_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_OBJECT_PTR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_OBJECT_PTR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LIGHT ##############

package LibreDWG::Dwg_Entity_LIGHT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LIGHT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LIGHT_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_LIGHT_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_LIGHT_class_version_set;
*swig_name_get = *LibreDWGc::Dwg_Entity_LIGHT_name_get;
*swig_name_set = *LibreDWGc::Dwg_Entity_LIGHT_name_set;
*swig_type_get = *LibreDWGc::Dwg_Entity_LIGHT_type_get;
*swig_type_set = *LibreDWGc::Dwg_Entity_LIGHT_type_set;
*swig_status_get = *LibreDWGc::Dwg_Entity_LIGHT_status_get;
*swig_status_set = *LibreDWGc::Dwg_Entity_LIGHT_status_set;
*swig_light_color_get = *LibreDWGc::Dwg_Entity_LIGHT_light_color_get;
*swig_light_color_set = *LibreDWGc::Dwg_Entity_LIGHT_light_color_set;
*swig_plot_glyph_get = *LibreDWGc::Dwg_Entity_LIGHT_plot_glyph_get;
*swig_plot_glyph_set = *LibreDWGc::Dwg_Entity_LIGHT_plot_glyph_set;
*swig_intensity_get = *LibreDWGc::Dwg_Entity_LIGHT_intensity_get;
*swig_intensity_set = *LibreDWGc::Dwg_Entity_LIGHT_intensity_set;
*swig_position_get = *LibreDWGc::Dwg_Entity_LIGHT_position_get;
*swig_position_set = *LibreDWGc::Dwg_Entity_LIGHT_position_set;
*swig_target_get = *LibreDWGc::Dwg_Entity_LIGHT_target_get;
*swig_target_set = *LibreDWGc::Dwg_Entity_LIGHT_target_set;
*swig_attenuation_type_get = *LibreDWGc::Dwg_Entity_LIGHT_attenuation_type_get;
*swig_attenuation_type_set = *LibreDWGc::Dwg_Entity_LIGHT_attenuation_type_set;
*swig_use_attenuation_limits_get = *LibreDWGc::Dwg_Entity_LIGHT_use_attenuation_limits_get;
*swig_use_attenuation_limits_set = *LibreDWGc::Dwg_Entity_LIGHT_use_attenuation_limits_set;
*swig_attenuation_start_limit_get = *LibreDWGc::Dwg_Entity_LIGHT_attenuation_start_limit_get;
*swig_attenuation_start_limit_set = *LibreDWGc::Dwg_Entity_LIGHT_attenuation_start_limit_set;
*swig_attenuation_end_limit_get = *LibreDWGc::Dwg_Entity_LIGHT_attenuation_end_limit_get;
*swig_attenuation_end_limit_set = *LibreDWGc::Dwg_Entity_LIGHT_attenuation_end_limit_set;
*swig_hotspot_angle_get = *LibreDWGc::Dwg_Entity_LIGHT_hotspot_angle_get;
*swig_hotspot_angle_set = *LibreDWGc::Dwg_Entity_LIGHT_hotspot_angle_set;
*swig_falloff_angle_get = *LibreDWGc::Dwg_Entity_LIGHT_falloff_angle_get;
*swig_falloff_angle_set = *LibreDWGc::Dwg_Entity_LIGHT_falloff_angle_set;
*swig_cast_shadows_get = *LibreDWGc::Dwg_Entity_LIGHT_cast_shadows_get;
*swig_cast_shadows_set = *LibreDWGc::Dwg_Entity_LIGHT_cast_shadows_set;
*swig_shadow_type_get = *LibreDWGc::Dwg_Entity_LIGHT_shadow_type_get;
*swig_shadow_type_set = *LibreDWGc::Dwg_Entity_LIGHT_shadow_type_set;
*swig_shadow_map_size_get = *LibreDWGc::Dwg_Entity_LIGHT_shadow_map_size_get;
*swig_shadow_map_size_set = *LibreDWGc::Dwg_Entity_LIGHT_shadow_map_size_set;
*swig_shadow_map_softness_get = *LibreDWGc::Dwg_Entity_LIGHT_shadow_map_softness_get;
*swig_shadow_map_softness_set = *LibreDWGc::Dwg_Entity_LIGHT_shadow_map_softness_set;
*swig_is_photometric_get = *LibreDWGc::Dwg_Entity_LIGHT_is_photometric_get;
*swig_is_photometric_set = *LibreDWGc::Dwg_Entity_LIGHT_is_photometric_set;
*swig_has_photometric_data_get = *LibreDWGc::Dwg_Entity_LIGHT_has_photometric_data_get;
*swig_has_photometric_data_set = *LibreDWGc::Dwg_Entity_LIGHT_has_photometric_data_set;
*swig_has_webfile_get = *LibreDWGc::Dwg_Entity_LIGHT_has_webfile_get;
*swig_has_webfile_set = *LibreDWGc::Dwg_Entity_LIGHT_has_webfile_set;
*swig_webfile_get = *LibreDWGc::Dwg_Entity_LIGHT_webfile_get;
*swig_webfile_set = *LibreDWGc::Dwg_Entity_LIGHT_webfile_set;
*swig_physical_intensity_method_get = *LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_method_get;
*swig_physical_intensity_method_set = *LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_method_set;
*swig_physical_intensity_get = *LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_get;
*swig_physical_intensity_set = *LibreDWGc::Dwg_Entity_LIGHT_physical_intensity_set;
*swig_illuminance_dist_get = *LibreDWGc::Dwg_Entity_LIGHT_illuminance_dist_get;
*swig_illuminance_dist_set = *LibreDWGc::Dwg_Entity_LIGHT_illuminance_dist_set;
*swig_lamp_color_type_get = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_type_get;
*swig_lamp_color_type_set = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_type_set;
*swig_lamp_color_temp_get = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_temp_get;
*swig_lamp_color_temp_set = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_temp_set;
*swig_lamp_color_preset_get = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_preset_get;
*swig_lamp_color_preset_set = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_preset_set;
*swig_lamp_color_rgb_get = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_rgb_get;
*swig_lamp_color_rgb_set = *LibreDWGc::Dwg_Entity_LIGHT_lamp_color_rgb_set;
*swig_web_rotation_get = *LibreDWGc::Dwg_Entity_LIGHT_web_rotation_get;
*swig_web_rotation_set = *LibreDWGc::Dwg_Entity_LIGHT_web_rotation_set;
*swig_extlight_shape_get = *LibreDWGc::Dwg_Entity_LIGHT_extlight_shape_get;
*swig_extlight_shape_set = *LibreDWGc::Dwg_Entity_LIGHT_extlight_shape_set;
*swig_extlight_length_get = *LibreDWGc::Dwg_Entity_LIGHT_extlight_length_get;
*swig_extlight_length_set = *LibreDWGc::Dwg_Entity_LIGHT_extlight_length_set;
*swig_extlight_width_get = *LibreDWGc::Dwg_Entity_LIGHT_extlight_width_get;
*swig_extlight_width_set = *LibreDWGc::Dwg_Entity_LIGHT_extlight_width_set;
*swig_extlight_radius_get = *LibreDWGc::Dwg_Entity_LIGHT_extlight_radius_get;
*swig_extlight_radius_set = *LibreDWGc::Dwg_Entity_LIGHT_extlight_radius_set;
*swig_webfile_type_get = *LibreDWGc::Dwg_Entity_LIGHT_webfile_type_get;
*swig_webfile_type_set = *LibreDWGc::Dwg_Entity_LIGHT_webfile_type_set;
*swig_web_symetry_get = *LibreDWGc::Dwg_Entity_LIGHT_web_symetry_get;
*swig_web_symetry_set = *LibreDWGc::Dwg_Entity_LIGHT_web_symetry_set;
*swig_has_target_grip_get = *LibreDWGc::Dwg_Entity_LIGHT_has_target_grip_get;
*swig_has_target_grip_set = *LibreDWGc::Dwg_Entity_LIGHT_has_target_grip_set;
*swig_web_flux_get = *LibreDWGc::Dwg_Entity_LIGHT_web_flux_get;
*swig_web_flux_set = *LibreDWGc::Dwg_Entity_LIGHT_web_flux_set;
*swig_web_angle1_get = *LibreDWGc::Dwg_Entity_LIGHT_web_angle1_get;
*swig_web_angle1_set = *LibreDWGc::Dwg_Entity_LIGHT_web_angle1_set;
*swig_web_angle2_get = *LibreDWGc::Dwg_Entity_LIGHT_web_angle2_get;
*swig_web_angle2_set = *LibreDWGc::Dwg_Entity_LIGHT_web_angle2_set;
*swig_web_angle3_get = *LibreDWGc::Dwg_Entity_LIGHT_web_angle3_get;
*swig_web_angle3_set = *LibreDWGc::Dwg_Entity_LIGHT_web_angle3_set;
*swig_web_angle4_get = *LibreDWGc::Dwg_Entity_LIGHT_web_angle4_get;
*swig_web_angle4_set = *LibreDWGc::Dwg_Entity_LIGHT_web_angle4_set;
*swig_web_angle5_get = *LibreDWGc::Dwg_Entity_LIGHT_web_angle5_get;
*swig_web_angle5_set = *LibreDWGc::Dwg_Entity_LIGHT_web_angle5_set;
*swig_glyph_display_type_get = *LibreDWGc::Dwg_Entity_LIGHT_glyph_display_type_get;
*swig_glyph_display_type_set = *LibreDWGc::Dwg_Entity_LIGHT_glyph_display_type_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LIGHT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LIGHT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_CAMERA ##############

package LibreDWG::Dwg_Entity_CAMERA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_CAMERA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_CAMERA_parent_set;
*swig_view_get = *LibreDWGc::Dwg_Entity_CAMERA_view_get;
*swig_view_set = *LibreDWGc::Dwg_Entity_CAMERA_view_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_CAMERA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_CAMERA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_GEOPOSITIONMARKER ##############

package LibreDWG::Dwg_Entity_GEOPOSITIONMARKER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_class_version_set;
*swig_position_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_position_get;
*swig_position_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_position_set;
*swig_radius_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_radius_set;
*swig_landing_gap_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_landing_gap_get;
*swig_landing_gap_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_landing_gap_set;
*swig_notes_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_notes_get;
*swig_notes_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_notes_set;
*swig_text_alignment_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_text_alignment_get;
*swig_text_alignment_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_text_alignment_set;
*swig_mtext_visible_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_get;
*swig_mtext_visible_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_visible_set;
*swig_enable_frame_text_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_get;
*swig_enable_frame_text_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_enable_frame_text_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_ins_pt_set;
*swig_alignment_pt_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_get;
*swig_alignment_pt_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_alignment_pt_set;
*swig_mtext_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_get;
*swig_mtext_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_mtext_set;
*swig_is_really_locked_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_get;
*swig_is_really_locked_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_is_really_locked_set;
*swig_annotative_data_size_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_get;
*swig_annotative_data_size_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_size_set;
*swig_annotative_data_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_get;
*swig_annotative_data_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_data_set;
*swig_annotative_flag_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_get;
*swig_annotative_flag_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_flag_set;
*swig_annotative_style_get = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_style_get;
*swig_annotative_style_set = *LibreDWGc::Dwg_Entity_GEOPOSITIONMARKER_annotative_style_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_GEOPOSITIONMARKER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_GEOPOSITIONMARKER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_GEOMAPIMAGE ##############

package LibreDWG::Dwg_Object_GEOMAPIMAGE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_class_version_set;
*swig_pt0_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_pt0_get;
*swig_pt0_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_pt0_set;
*swig_image_size_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_size_get;
*swig_image_size_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_size_set;
*swig_display_props_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_display_props_get;
*swig_display_props_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_display_props_set;
*swig_clipping_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_clipping_get;
*swig_clipping_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_clipping_set;
*swig_brightness_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_brightness_get;
*swig_brightness_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_brightness_set;
*swig_contrast_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_contrast_get;
*swig_contrast_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_contrast_set;
*swig_fade_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_fade_get;
*swig_fade_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_fade_set;
*swig_rotation_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_rotation_set;
*swig_image_width_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_width_get;
*swig_image_width_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_width_set;
*swig_image_height_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_height_get;
*swig_image_height_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_height_set;
*swig_name_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_name_set;
*swig_image_file_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_file_get;
*swig_image_file_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_file_set;
*swig_image_visibility_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_visibility_get;
*swig_image_visibility_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_image_visibility_set;
*swig_transparency_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_transparency_get;
*swig_transparency_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_transparency_set;
*swig_height_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_height_get;
*swig_height_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_height_set;
*swig_width_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_width_get;
*swig_width_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_width_set;
*swig_show_rotation_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_show_rotation_get;
*swig_show_rotation_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_show_rotation_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_scale_factor_set;
*swig_geoimage_brightness_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_brightness_get;
*swig_geoimage_brightness_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_brightness_set;
*swig_geoimage_contrast_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_contrast_get;
*swig_geoimage_contrast_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_contrast_set;
*swig_geoimage_fade_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_fade_get;
*swig_geoimage_fade_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_fade_set;
*swig_geoimage_position_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_position_get;
*swig_geoimage_position_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_position_set;
*swig_geoimage_width_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_width_get;
*swig_geoimage_width_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_width_set;
*swig_geoimage_height_get = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_height_get;
*swig_geoimage_height_set = *LibreDWGc::Dwg_Object_GEOMAPIMAGE_geoimage_height_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_GEOMAPIMAGE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_GEOMAPIMAGE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_HELIX ##############

package LibreDWG::Dwg_Entity_HELIX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_HELIX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_HELIX_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_HELIX_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_HELIX_flag_set;
*swig_scenario_get = *LibreDWGc::Dwg_Entity_HELIX_scenario_get;
*swig_scenario_set = *LibreDWGc::Dwg_Entity_HELIX_scenario_set;
*swig_degree_get = *LibreDWGc::Dwg_Entity_HELIX_degree_get;
*swig_degree_set = *LibreDWGc::Dwg_Entity_HELIX_degree_set;
*swig_splineflags_get = *LibreDWGc::Dwg_Entity_HELIX_splineflags_get;
*swig_splineflags_set = *LibreDWGc::Dwg_Entity_HELIX_splineflags_set;
*swig_knotparam_get = *LibreDWGc::Dwg_Entity_HELIX_knotparam_get;
*swig_knotparam_set = *LibreDWGc::Dwg_Entity_HELIX_knotparam_set;
*swig_fit_tol_get = *LibreDWGc::Dwg_Entity_HELIX_fit_tol_get;
*swig_fit_tol_set = *LibreDWGc::Dwg_Entity_HELIX_fit_tol_set;
*swig_beg_tan_vec_get = *LibreDWGc::Dwg_Entity_HELIX_beg_tan_vec_get;
*swig_beg_tan_vec_set = *LibreDWGc::Dwg_Entity_HELIX_beg_tan_vec_set;
*swig_end_tan_vec_get = *LibreDWGc::Dwg_Entity_HELIX_end_tan_vec_get;
*swig_end_tan_vec_set = *LibreDWGc::Dwg_Entity_HELIX_end_tan_vec_set;
*swig_rational_get = *LibreDWGc::Dwg_Entity_HELIX_rational_get;
*swig_rational_set = *LibreDWGc::Dwg_Entity_HELIX_rational_set;
*swig_closed_b_get = *LibreDWGc::Dwg_Entity_HELIX_closed_b_get;
*swig_closed_b_set = *LibreDWGc::Dwg_Entity_HELIX_closed_b_set;
*swig_periodic_get = *LibreDWGc::Dwg_Entity_HELIX_periodic_get;
*swig_periodic_set = *LibreDWGc::Dwg_Entity_HELIX_periodic_set;
*swig_weighted_get = *LibreDWGc::Dwg_Entity_HELIX_weighted_get;
*swig_weighted_set = *LibreDWGc::Dwg_Entity_HELIX_weighted_set;
*swig_knot_tol_get = *LibreDWGc::Dwg_Entity_HELIX_knot_tol_get;
*swig_knot_tol_set = *LibreDWGc::Dwg_Entity_HELIX_knot_tol_set;
*swig_ctrl_tol_get = *LibreDWGc::Dwg_Entity_HELIX_ctrl_tol_get;
*swig_ctrl_tol_set = *LibreDWGc::Dwg_Entity_HELIX_ctrl_tol_set;
*swig_num_fit_pts_get = *LibreDWGc::Dwg_Entity_HELIX_num_fit_pts_get;
*swig_num_fit_pts_set = *LibreDWGc::Dwg_Entity_HELIX_num_fit_pts_set;
*swig_fit_pts_get = *LibreDWGc::Dwg_Entity_HELIX_fit_pts_get;
*swig_fit_pts_set = *LibreDWGc::Dwg_Entity_HELIX_fit_pts_set;
*swig_num_knots_get = *LibreDWGc::Dwg_Entity_HELIX_num_knots_get;
*swig_num_knots_set = *LibreDWGc::Dwg_Entity_HELIX_num_knots_set;
*swig_knots_get = *LibreDWGc::Dwg_Entity_HELIX_knots_get;
*swig_knots_set = *LibreDWGc::Dwg_Entity_HELIX_knots_set;
*swig_num_ctrl_pts_get = *LibreDWGc::Dwg_Entity_HELIX_num_ctrl_pts_get;
*swig_num_ctrl_pts_set = *LibreDWGc::Dwg_Entity_HELIX_num_ctrl_pts_set;
*swig_ctrl_pts_get = *LibreDWGc::Dwg_Entity_HELIX_ctrl_pts_get;
*swig_ctrl_pts_set = *LibreDWGc::Dwg_Entity_HELIX_ctrl_pts_set;
*swig_major_version_get = *LibreDWGc::Dwg_Entity_HELIX_major_version_get;
*swig_major_version_set = *LibreDWGc::Dwg_Entity_HELIX_major_version_set;
*swig_maint_version_get = *LibreDWGc::Dwg_Entity_HELIX_maint_version_get;
*swig_maint_version_set = *LibreDWGc::Dwg_Entity_HELIX_maint_version_set;
*swig_axis_base_pt_get = *LibreDWGc::Dwg_Entity_HELIX_axis_base_pt_get;
*swig_axis_base_pt_set = *LibreDWGc::Dwg_Entity_HELIX_axis_base_pt_set;
*swig_start_pt_get = *LibreDWGc::Dwg_Entity_HELIX_start_pt_get;
*swig_start_pt_set = *LibreDWGc::Dwg_Entity_HELIX_start_pt_set;
*swig_axis_vector_get = *LibreDWGc::Dwg_Entity_HELIX_axis_vector_get;
*swig_axis_vector_set = *LibreDWGc::Dwg_Entity_HELIX_axis_vector_set;
*swig_radius_get = *LibreDWGc::Dwg_Entity_HELIX_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_Entity_HELIX_radius_set;
*swig_turns_get = *LibreDWGc::Dwg_Entity_HELIX_turns_get;
*swig_turns_set = *LibreDWGc::Dwg_Entity_HELIX_turns_set;
*swig_turn_height_get = *LibreDWGc::Dwg_Entity_HELIX_turn_height_get;
*swig_turn_height_set = *LibreDWGc::Dwg_Entity_HELIX_turn_height_set;
*swig_handedness_get = *LibreDWGc::Dwg_Entity_HELIX_handedness_get;
*swig_handedness_set = *LibreDWGc::Dwg_Entity_HELIX_handedness_set;
*swig_constraint_type_get = *LibreDWGc::Dwg_Entity_HELIX_constraint_type_get;
*swig_constraint_type_set = *LibreDWGc::Dwg_Entity_HELIX_constraint_type_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_HELIX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_HELIX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_EXTRUDEDSURFACE ##############

package LibreDWG::Dwg_Entity_EXTRUDEDSURFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_version_get;
*swig_version_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_acis_empty_bit_set;
*swig_modeler_format_version_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_get;
*swig_modeler_format_version_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_modeler_format_version_set;
*swig_bindata_size_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_size_get;
*swig_bindata_size_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_size_set;
*swig_bindata_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_get;
*swig_bindata_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bindata_set;
*swig_u_isolines_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_u_isolines_get;
*swig_u_isolines_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_u_isolines_set;
*swig_v_isolines_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_v_isolines_get;
*swig_v_isolines_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_v_isolines_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_class_version_set;
*swig_draft_angle_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_angle_get;
*swig_draft_angle_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_angle_set;
*swig_draft_start_distance_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_get;
*swig_draft_start_distance_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_start_distance_set;
*swig_draft_end_distance_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_get;
*swig_draft_end_distance_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_draft_end_distance_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_twist_angle_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_scale_factor_set;
*swig_align_angle_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_angle_get;
*swig_align_angle_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_angle_set;
*swig_sweep_entity_transmatrix_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_get;
*swig_sweep_entity_transmatrix_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transmatrix_set;
*swig_path_entity_transmatrix_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_get;
*swig_path_entity_transmatrix_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transmatrix_set;
*swig_is_solid_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_is_solid_get;
*swig_is_solid_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_is_solid_set;
*swig_sweep_alignment_flags_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_get;
*swig_sweep_alignment_flags_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_alignment_flags_set;
*swig_path_flags_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_flags_get;
*swig_path_flags_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_flags_set;
*swig_align_start_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_start_get;
*swig_align_start_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_align_start_set;
*swig_bank_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bank_get;
*swig_bank_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_bank_set;
*swig_base_point_set_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_base_point_set_get;
*swig_base_point_set_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_base_point_set_set;
*swig_sweep_entity_transform_computed_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_get;
*swig_sweep_entity_transform_computed_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_transform_computed_set;
*swig_path_entity_transform_computed_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_get;
*swig_path_entity_transform_computed_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_transform_computed_set;
*swig_reference_vector_for_controlling_twist_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_get;
*swig_reference_vector_for_controlling_twist_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_reference_vector_for_controlling_twist_set;
*swig_sweep_entity_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_get;
*swig_sweep_entity_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_entity_set;
*swig_path_entity_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_get;
*swig_path_entity_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_path_entity_set;
*swig_sweep_vector_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_get;
*swig_sweep_vector_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_vector_set;
*swig_sweep_transmatrix_get = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_get;
*swig_sweep_transmatrix_set = *LibreDWGc::Dwg_Entity_EXTRUDEDSURFACE_sweep_transmatrix_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_EXTRUDEDSURFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_EXTRUDEDSURFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_SWEPTSURFACE ##############

package LibreDWG::Dwg_Entity_SWEPTSURFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_version_get;
*swig_version_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_acis_empty_bit_set;
*swig_modeler_format_version_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_modeler_format_version_get;
*swig_modeler_format_version_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_modeler_format_version_set;
*swig_u_isolines_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_u_isolines_get;
*swig_u_isolines_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_u_isolines_set;
*swig_v_isolines_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_v_isolines_get;
*swig_v_isolines_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_v_isolines_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_class_version_set;
*swig_sweep_entity_id_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_id_get;
*swig_sweep_entity_id_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_id_set;
*swig_sweepdata_size_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_size_get;
*swig_sweepdata_size_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_size_set;
*swig_sweepdata_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_get;
*swig_sweepdata_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweepdata_set;
*swig_path_entity_id_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_id_get;
*swig_path_entity_id_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_id_set;
*swig_pathdata_size_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_size_get;
*swig_pathdata_size_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_size_set;
*swig_pathdata_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_get;
*swig_pathdata_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_pathdata_set;
*swig_draft_angle_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_angle_get;
*swig_draft_angle_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_angle_set;
*swig_draft_start_distance_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_start_distance_get;
*swig_draft_start_distance_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_start_distance_set;
*swig_draft_end_distance_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_end_distance_get;
*swig_draft_end_distance_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_draft_end_distance_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_twist_angle_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_scale_factor_set;
*swig_align_angle_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_angle_get;
*swig_align_angle_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_angle_set;
*swig_sweep_entity_transmatrix_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_get;
*swig_sweep_entity_transmatrix_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transmatrix_set;
*swig_path_entity_transmatrix_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_get;
*swig_path_entity_transmatrix_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transmatrix_set;
*swig_is_solid_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_is_solid_get;
*swig_is_solid_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_is_solid_set;
*swig_sweep_alignment_flags_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_get;
*swig_sweep_alignment_flags_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_alignment_flags_set;
*swig_path_flags_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_flags_get;
*swig_path_flags_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_flags_set;
*swig_align_start_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_start_get;
*swig_align_start_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_align_start_set;
*swig_bank_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_bank_get;
*swig_bank_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_bank_set;
*swig_base_point_set_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_base_point_set_get;
*swig_base_point_set_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_base_point_set_set;
*swig_sweep_entity_transform_computed_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_get;
*swig_sweep_entity_transform_computed_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_transform_computed_set;
*swig_path_entity_transform_computed_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_get;
*swig_path_entity_transform_computed_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_transform_computed_set;
*swig_reference_vector_for_controlling_twist_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_get;
*swig_reference_vector_for_controlling_twist_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_reference_vector_for_controlling_twist_set;
*swig_sweep_entity_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_get;
*swig_sweep_entity_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_sweep_entity_set;
*swig_path_entity_get = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_get;
*swig_path_entity_set = *LibreDWGc::Dwg_Entity_SWEPTSURFACE_path_entity_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_SWEPTSURFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_SWEPTSURFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LOFTEDSURFACE ##############

package LibreDWG::Dwg_Entity_LOFTEDSURFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_version_get;
*swig_version_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_acis_empty_bit_set;
*swig_modeler_format_version_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_modeler_format_version_get;
*swig_modeler_format_version_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_modeler_format_version_set;
*swig_u_isolines_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_u_isolines_get;
*swig_u_isolines_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_u_isolines_set;
*swig_v_isolines_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_v_isolines_get;
*swig_v_isolines_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_v_isolines_set;
*swig_loft_entity_transmatrix_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_get;
*swig_loft_entity_transmatrix_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_loft_entity_transmatrix_set;
*swig_plane_normal_lofting_type_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_get;
*swig_plane_normal_lofting_type_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_plane_normal_lofting_type_set;
*swig_start_draft_angle_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_angle_get;
*swig_start_draft_angle_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_angle_set;
*swig_end_draft_angle_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_angle_get;
*swig_end_draft_angle_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_angle_set;
*swig_start_draft_magnitude_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_get;
*swig_start_draft_magnitude_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_start_draft_magnitude_set;
*swig_end_draft_magnitude_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_get;
*swig_end_draft_magnitude_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_end_draft_magnitude_set;
*swig_arc_length_parameterization_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_get;
*swig_arc_length_parameterization_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_arc_length_parameterization_set;
*swig_no_twist_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_no_twist_get;
*swig_no_twist_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_no_twist_set;
*swig_align_direction_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_align_direction_get;
*swig_align_direction_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_align_direction_set;
*swig_simple_surfaces_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_simple_surfaces_get;
*swig_simple_surfaces_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_simple_surfaces_set;
*swig_closed_surfaces_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_closed_surfaces_get;
*swig_closed_surfaces_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_closed_surfaces_set;
*swig_solid_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_solid_get;
*swig_solid_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_solid_set;
*swig_ruled_surface_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_ruled_surface_get;
*swig_ruled_surface_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_ruled_surface_set;
*swig_virtual_guide_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_virtual_guide_get;
*swig_virtual_guide_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_virtual_guide_set;
*swig_num_cross_sections_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_cross_sections_get;
*swig_num_cross_sections_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_cross_sections_set;
*swig_num_guide_curves_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_guide_curves_get;
*swig_num_guide_curves_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_num_guide_curves_set;
*swig_cross_sections_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_cross_sections_get;
*swig_cross_sections_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_cross_sections_set;
*swig_guide_curves_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_guide_curves_get;
*swig_guide_curves_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_guide_curves_set;
*swig_path_curve_get = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_path_curve_get;
*swig_path_curve_set = *LibreDWGc::Dwg_Entity_LOFTEDSURFACE_path_curve_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LOFTEDSURFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LOFTEDSURFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_NURBSURFACE ##############

package LibreDWG::Dwg_Entity_NURBSURFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_version_get;
*swig_version_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Entity_NURBSURFACE__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Entity_NURBSURFACE__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_acis_empty_bit_set;
*swig_u_isolines_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_u_isolines_get;
*swig_u_isolines_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_u_isolines_set;
*swig_v_isolines_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_v_isolines_get;
*swig_v_isolines_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_v_isolines_set;
*swig_short170_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_short170_get;
*swig_short170_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_short170_set;
*swig_cv_hull_display_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_cv_hull_display_get;
*swig_cv_hull_display_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_cv_hull_display_set;
*swig_uvec1_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_uvec1_get;
*swig_uvec1_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_uvec1_set;
*swig_vvec1_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_vvec1_get;
*swig_vvec1_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_vvec1_set;
*swig_uvec2_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_uvec2_get;
*swig_uvec2_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_uvec2_set;
*swig_vvec2_get = *LibreDWGc::Dwg_Entity_NURBSURFACE_vvec2_get;
*swig_vvec2_set = *LibreDWGc::Dwg_Entity_NURBSURFACE_vvec2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_NURBSURFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_NURBSURFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_PLANESURFACE ##############

package LibreDWG::Dwg_Entity_PLANESURFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_version_get;
*swig_version_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Entity_PLANESURFACE__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Entity_PLANESURFACE__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_acis_empty_bit_set;
*swig_modeler_format_version_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_modeler_format_version_get;
*swig_modeler_format_version_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_modeler_format_version_set;
*swig_u_isolines_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_u_isolines_get;
*swig_u_isolines_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_u_isolines_set;
*swig_v_isolines_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_v_isolines_get;
*swig_v_isolines_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_v_isolines_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_PLANESURFACE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_PLANESURFACE_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_PLANESURFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_PLANESURFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_REVOLVEDSURFACE ##############

package LibreDWG::Dwg_Entity_REVOLVEDSURFACE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_version_get;
*swig_version_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_get;
*swig_point_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_acis_empty_bit_set;
*swig_modeler_format_version_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_get;
*swig_modeler_format_version_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_modeler_format_version_set;
*swig_u_isolines_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_u_isolines_get;
*swig_u_isolines_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_u_isolines_set;
*swig_v_isolines_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_v_isolines_get;
*swig_v_isolines_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_v_isolines_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_class_version_set;
*swig_id_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_id_get;
*swig_id_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_id_set;
*swig_axis_point_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_point_get;
*swig_axis_point_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_point_set;
*swig_axis_vector_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_vector_get;
*swig_axis_vector_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_axis_vector_set;
*swig_revolve_angle_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolve_angle_get;
*swig_revolve_angle_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolve_angle_set;
*swig_start_angle_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_start_angle_get;
*swig_start_angle_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_start_angle_set;
*swig_revolved_entity_transmatrix_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_get;
*swig_revolved_entity_transmatrix_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_revolved_entity_transmatrix_set;
*swig_draft_angle_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_angle_get;
*swig_draft_angle_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_angle_set;
*swig_draft_start_distance_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_get;
*swig_draft_start_distance_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_start_distance_set;
*swig_draft_end_distance_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_get;
*swig_draft_end_distance_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_draft_end_distance_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_twist_angle_set;
*swig_solid_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_solid_get;
*swig_solid_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_solid_set;
*swig_close_to_axis_get = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_close_to_axis_get;
*swig_close_to_axis_set = *LibreDWGc::Dwg_Entity_REVOLVEDSURFACE_close_to_axis_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_REVOLVEDSURFACE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_REVOLVEDSURFACE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_MESH_edge ##############

package LibreDWG::Dwg_MESH_edge;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_MESH_edge_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_MESH_edge_parent_set;
*swig_idxfrom_get = *LibreDWGc::Dwg_MESH_edge_idxfrom_get;
*swig_idxfrom_set = *LibreDWGc::Dwg_MESH_edge_idxfrom_set;
*swig_idxto_get = *LibreDWGc::Dwg_MESH_edge_idxto_get;
*swig_idxto_set = *LibreDWGc::Dwg_MESH_edge_idxto_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_MESH_edge(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_MESH_edge($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_MESH ##############

package LibreDWG::Dwg_Entity_MESH;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_MESH_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_MESH_parent_set;
*swig_dlevel_get = *LibreDWGc::Dwg_Entity_MESH_dlevel_get;
*swig_dlevel_set = *LibreDWGc::Dwg_Entity_MESH_dlevel_set;
*swig_is_watertight_get = *LibreDWGc::Dwg_Entity_MESH_is_watertight_get;
*swig_is_watertight_set = *LibreDWGc::Dwg_Entity_MESH_is_watertight_set;
*swig_num_subdiv_vertex_get = *LibreDWGc::Dwg_Entity_MESH_num_subdiv_vertex_get;
*swig_num_subdiv_vertex_set = *LibreDWGc::Dwg_Entity_MESH_num_subdiv_vertex_set;
*swig_subdiv_vertex_get = *LibreDWGc::Dwg_Entity_MESH_subdiv_vertex_get;
*swig_subdiv_vertex_set = *LibreDWGc::Dwg_Entity_MESH_subdiv_vertex_set;
*swig_num_vertex_get = *LibreDWGc::Dwg_Entity_MESH_num_vertex_get;
*swig_num_vertex_set = *LibreDWGc::Dwg_Entity_MESH_num_vertex_set;
*swig_vertex_get = *LibreDWGc::Dwg_Entity_MESH_vertex_get;
*swig_vertex_set = *LibreDWGc::Dwg_Entity_MESH_vertex_set;
*swig_num_faces_get = *LibreDWGc::Dwg_Entity_MESH_num_faces_get;
*swig_num_faces_set = *LibreDWGc::Dwg_Entity_MESH_num_faces_set;
*swig_faces_get = *LibreDWGc::Dwg_Entity_MESH_faces_get;
*swig_faces_set = *LibreDWGc::Dwg_Entity_MESH_faces_set;
*swig_num_edges_get = *LibreDWGc::Dwg_Entity_MESH_num_edges_get;
*swig_num_edges_set = *LibreDWGc::Dwg_Entity_MESH_num_edges_set;
*swig_edges_get = *LibreDWGc::Dwg_Entity_MESH_edges_get;
*swig_edges_set = *LibreDWGc::Dwg_Entity_MESH_edges_set;
*swig_num_crease_get = *LibreDWGc::Dwg_Entity_MESH_num_crease_get;
*swig_num_crease_set = *LibreDWGc::Dwg_Entity_MESH_num_crease_set;
*swig_crease_get = *LibreDWGc::Dwg_Entity_MESH_crease_get;
*swig_crease_set = *LibreDWGc::Dwg_Entity_MESH_crease_set;
*swig_unknown_b1_get = *LibreDWGc::Dwg_Entity_MESH_unknown_b1_get;
*swig_unknown_b1_set = *LibreDWGc::Dwg_Entity_MESH_unknown_b1_set;
*swig_unknown_b2_get = *LibreDWGc::Dwg_Entity_MESH_unknown_b2_get;
*swig_unknown_b2_set = *LibreDWGc::Dwg_Entity_MESH_unknown_b2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_MESH(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_MESH($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SUN ##############

package LibreDWG::Dwg_Object_SUN;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SUN_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SUN_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_SUN_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_SUN_class_version_set;
*swig_is_on_get = *LibreDWGc::Dwg_Object_SUN_is_on_get;
*swig_is_on_set = *LibreDWGc::Dwg_Object_SUN_is_on_set;
*swig_color_get = *LibreDWGc::Dwg_Object_SUN_color_get;
*swig_color_set = *LibreDWGc::Dwg_Object_SUN_color_set;
*swig_intensity_get = *LibreDWGc::Dwg_Object_SUN_intensity_get;
*swig_intensity_set = *LibreDWGc::Dwg_Object_SUN_intensity_set;
*swig_has_shadow_get = *LibreDWGc::Dwg_Object_SUN_has_shadow_get;
*swig_has_shadow_set = *LibreDWGc::Dwg_Object_SUN_has_shadow_set;
*swig_julian_day_get = *LibreDWGc::Dwg_Object_SUN_julian_day_get;
*swig_julian_day_set = *LibreDWGc::Dwg_Object_SUN_julian_day_set;
*swig_msecs_get = *LibreDWGc::Dwg_Object_SUN_msecs_get;
*swig_msecs_set = *LibreDWGc::Dwg_Object_SUN_msecs_set;
*swig_is_dst_get = *LibreDWGc::Dwg_Object_SUN_is_dst_get;
*swig_is_dst_set = *LibreDWGc::Dwg_Object_SUN_is_dst_set;
*swig_shadow_type_get = *LibreDWGc::Dwg_Object_SUN_shadow_type_get;
*swig_shadow_type_set = *LibreDWGc::Dwg_Object_SUN_shadow_type_set;
*swig_shadow_mapsize_get = *LibreDWGc::Dwg_Object_SUN_shadow_mapsize_get;
*swig_shadow_mapsize_set = *LibreDWGc::Dwg_Object_SUN_shadow_mapsize_set;
*swig_shadow_softness_get = *LibreDWGc::Dwg_Object_SUN_shadow_softness_get;
*swig_shadow_softness_set = *LibreDWGc::Dwg_Object_SUN_shadow_softness_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SUN(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SUN($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SUNSTUDY_Dates ##############

package LibreDWG::Dwg_SUNSTUDY_Dates;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_julian_day_get = *LibreDWGc::Dwg_SUNSTUDY_Dates_julian_day_get;
*swig_julian_day_set = *LibreDWGc::Dwg_SUNSTUDY_Dates_julian_day_set;
*swig_msecs_get = *LibreDWGc::Dwg_SUNSTUDY_Dates_msecs_get;
*swig_msecs_set = *LibreDWGc::Dwg_SUNSTUDY_Dates_msecs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SUNSTUDY_Dates(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SUNSTUDY_Dates($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SUNSTUDY ##############

package LibreDWG::Dwg_Object_SUNSTUDY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SUNSTUDY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SUNSTUDY_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_SUNSTUDY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_SUNSTUDY_class_version_set;
*swig_setup_name_get = *LibreDWGc::Dwg_Object_SUNSTUDY_setup_name_get;
*swig_setup_name_set = *LibreDWGc::Dwg_Object_SUNSTUDY_setup_name_set;
*swig_description_get = *LibreDWGc::Dwg_Object_SUNSTUDY_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_SUNSTUDY_description_set;
*swig_output_type_get = *LibreDWGc::Dwg_Object_SUNSTUDY_output_type_get;
*swig_output_type_set = *LibreDWGc::Dwg_Object_SUNSTUDY_output_type_set;
*swig_sheet_set_name_get = *LibreDWGc::Dwg_Object_SUNSTUDY_sheet_set_name_get;
*swig_sheet_set_name_set = *LibreDWGc::Dwg_Object_SUNSTUDY_sheet_set_name_set;
*swig_use_subset_get = *LibreDWGc::Dwg_Object_SUNSTUDY_use_subset_get;
*swig_use_subset_set = *LibreDWGc::Dwg_Object_SUNSTUDY_use_subset_set;
*swig_sheet_subset_name_get = *LibreDWGc::Dwg_Object_SUNSTUDY_sheet_subset_name_get;
*swig_sheet_subset_name_set = *LibreDWGc::Dwg_Object_SUNSTUDY_sheet_subset_name_set;
*swig_select_dates_from_calendar_get = *LibreDWGc::Dwg_Object_SUNSTUDY_select_dates_from_calendar_get;
*swig_select_dates_from_calendar_set = *LibreDWGc::Dwg_Object_SUNSTUDY_select_dates_from_calendar_set;
*swig_num_dates_get = *LibreDWGc::Dwg_Object_SUNSTUDY_num_dates_get;
*swig_num_dates_set = *LibreDWGc::Dwg_Object_SUNSTUDY_num_dates_set;
*swig_dates_get = *LibreDWGc::Dwg_Object_SUNSTUDY_dates_get;
*swig_dates_set = *LibreDWGc::Dwg_Object_SUNSTUDY_dates_set;
*swig_select_range_of_dates_get = *LibreDWGc::Dwg_Object_SUNSTUDY_select_range_of_dates_get;
*swig_select_range_of_dates_set = *LibreDWGc::Dwg_Object_SUNSTUDY_select_range_of_dates_set;
*swig_start_time_get = *LibreDWGc::Dwg_Object_SUNSTUDY_start_time_get;
*swig_start_time_set = *LibreDWGc::Dwg_Object_SUNSTUDY_start_time_set;
*swig_end_time_get = *LibreDWGc::Dwg_Object_SUNSTUDY_end_time_get;
*swig_end_time_set = *LibreDWGc::Dwg_Object_SUNSTUDY_end_time_set;
*swig_interval_get = *LibreDWGc::Dwg_Object_SUNSTUDY_interval_get;
*swig_interval_set = *LibreDWGc::Dwg_Object_SUNSTUDY_interval_set;
*swig_num_hours_get = *LibreDWGc::Dwg_Object_SUNSTUDY_num_hours_get;
*swig_num_hours_set = *LibreDWGc::Dwg_Object_SUNSTUDY_num_hours_set;
*swig_hours_get = *LibreDWGc::Dwg_Object_SUNSTUDY_hours_get;
*swig_hours_set = *LibreDWGc::Dwg_Object_SUNSTUDY_hours_set;
*swig_shade_plot_type_get = *LibreDWGc::Dwg_Object_SUNSTUDY_shade_plot_type_get;
*swig_shade_plot_type_set = *LibreDWGc::Dwg_Object_SUNSTUDY_shade_plot_type_set;
*swig_numvports_get = *LibreDWGc::Dwg_Object_SUNSTUDY_numvports_get;
*swig_numvports_set = *LibreDWGc::Dwg_Object_SUNSTUDY_numvports_set;
*swig_numrows_get = *LibreDWGc::Dwg_Object_SUNSTUDY_numrows_get;
*swig_numrows_set = *LibreDWGc::Dwg_Object_SUNSTUDY_numrows_set;
*swig_numcols_get = *LibreDWGc::Dwg_Object_SUNSTUDY_numcols_get;
*swig_numcols_set = *LibreDWGc::Dwg_Object_SUNSTUDY_numcols_set;
*swig_spacing_get = *LibreDWGc::Dwg_Object_SUNSTUDY_spacing_get;
*swig_spacing_set = *LibreDWGc::Dwg_Object_SUNSTUDY_spacing_set;
*swig_lock_viewports_get = *LibreDWGc::Dwg_Object_SUNSTUDY_lock_viewports_get;
*swig_lock_viewports_set = *LibreDWGc::Dwg_Object_SUNSTUDY_lock_viewports_set;
*swig_label_viewports_get = *LibreDWGc::Dwg_Object_SUNSTUDY_label_viewports_get;
*swig_label_viewports_set = *LibreDWGc::Dwg_Object_SUNSTUDY_label_viewports_set;
*swig_page_setup_wizard_get = *LibreDWGc::Dwg_Object_SUNSTUDY_page_setup_wizard_get;
*swig_page_setup_wizard_set = *LibreDWGc::Dwg_Object_SUNSTUDY_page_setup_wizard_set;
*swig_view_get = *LibreDWGc::Dwg_Object_SUNSTUDY_view_get;
*swig_view_set = *LibreDWGc::Dwg_Object_SUNSTUDY_view_set;
*swig_visualstyle_get = *LibreDWGc::Dwg_Object_SUNSTUDY_visualstyle_get;
*swig_visualstyle_set = *LibreDWGc::Dwg_Object_SUNSTUDY_visualstyle_set;
*swig_text_style_get = *LibreDWGc::Dwg_Object_SUNSTUDY_text_style_get;
*swig_text_style_set = *LibreDWGc::Dwg_Object_SUNSTUDY_text_style_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SUNSTUDY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SUNSTUDY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_DATATABLE_row ##############

package LibreDWG::Dwg_DATATABLE_row;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_DATATABLE_row_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_DATATABLE_row_parent_set;
*swig_value_get = *LibreDWGc::Dwg_DATATABLE_row_value_get;
*swig_value_set = *LibreDWGc::Dwg_DATATABLE_row_value_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_DATATABLE_row(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_DATATABLE_row($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_DATATABLE_column ##############

package LibreDWG::Dwg_DATATABLE_column;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_DATATABLE_column_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_DATATABLE_column_parent_set;
*swig_type_get = *LibreDWGc::Dwg_DATATABLE_column_type_get;
*swig_type_set = *LibreDWGc::Dwg_DATATABLE_column_type_set;
*swig_text_get = *LibreDWGc::Dwg_DATATABLE_column_text_get;
*swig_text_set = *LibreDWGc::Dwg_DATATABLE_column_text_set;
*swig_rows_get = *LibreDWGc::Dwg_DATATABLE_column_rows_get;
*swig_rows_set = *LibreDWGc::Dwg_DATATABLE_column_rows_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_DATATABLE_column(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_DATATABLE_column($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DATATABLE ##############

package LibreDWG::Dwg_Object_DATATABLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DATATABLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DATATABLE_parent_set;
*swig_flags_get = *LibreDWGc::Dwg_Object_DATATABLE_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Object_DATATABLE_flags_set;
*swig_num_cols_get = *LibreDWGc::Dwg_Object_DATATABLE_num_cols_get;
*swig_num_cols_set = *LibreDWGc::Dwg_Object_DATATABLE_num_cols_set;
*swig_num_rows_get = *LibreDWGc::Dwg_Object_DATATABLE_num_rows_get;
*swig_num_rows_set = *LibreDWGc::Dwg_Object_DATATABLE_num_rows_set;
*swig_table_name_get = *LibreDWGc::Dwg_Object_DATATABLE_table_name_get;
*swig_table_name_set = *LibreDWGc::Dwg_Object_DATATABLE_table_name_set;
*swig_cols_get = *LibreDWGc::Dwg_Object_DATATABLE_cols_get;
*swig_cols_set = *LibreDWGc::Dwg_Object_DATATABLE_cols_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DATATABLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DATATABLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_DATALINK_customdata ##############

package LibreDWG::Dwg_DATALINK_customdata;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_DATALINK_customdata_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_DATALINK_customdata_parent_set;
*swig_target_get = *LibreDWGc::Dwg_DATALINK_customdata_target_get;
*swig_target_set = *LibreDWGc::Dwg_DATALINK_customdata_target_set;
*swig_text_get = *LibreDWGc::Dwg_DATALINK_customdata_text_get;
*swig_text_set = *LibreDWGc::Dwg_DATALINK_customdata_text_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_DATALINK_customdata(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_DATALINK_customdata($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DATALINK ##############

package LibreDWG::Dwg_Object_DATALINK;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DATALINK_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DATALINK_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_DATALINK_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_DATALINK_class_version_set;
*swig_data_adapter_get = *LibreDWGc::Dwg_Object_DATALINK_data_adapter_get;
*swig_data_adapter_set = *LibreDWGc::Dwg_Object_DATALINK_data_adapter_set;
*swig_description_get = *LibreDWGc::Dwg_Object_DATALINK_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_DATALINK_description_set;
*swig_tooltip_get = *LibreDWGc::Dwg_Object_DATALINK_tooltip_get;
*swig_tooltip_set = *LibreDWGc::Dwg_Object_DATALINK_tooltip_set;
*swig_connection_string_get = *LibreDWGc::Dwg_Object_DATALINK_connection_string_get;
*swig_connection_string_set = *LibreDWGc::Dwg_Object_DATALINK_connection_string_set;
*swig_option_get = *LibreDWGc::Dwg_Object_DATALINK_option_get;
*swig_option_set = *LibreDWGc::Dwg_Object_DATALINK_option_set;
*swig_update_option_get = *LibreDWGc::Dwg_Object_DATALINK_update_option_get;
*swig_update_option_set = *LibreDWGc::Dwg_Object_DATALINK_update_option_set;
*swig_bl92_get = *LibreDWGc::Dwg_Object_DATALINK_bl92_get;
*swig_bl92_set = *LibreDWGc::Dwg_Object_DATALINK_bl92_set;
*swig_year_get = *LibreDWGc::Dwg_Object_DATALINK_year_get;
*swig_year_set = *LibreDWGc::Dwg_Object_DATALINK_year_set;
*swig_month_get = *LibreDWGc::Dwg_Object_DATALINK_month_get;
*swig_month_set = *LibreDWGc::Dwg_Object_DATALINK_month_set;
*swig_day_get = *LibreDWGc::Dwg_Object_DATALINK_day_get;
*swig_day_set = *LibreDWGc::Dwg_Object_DATALINK_day_set;
*swig_hour_get = *LibreDWGc::Dwg_Object_DATALINK_hour_get;
*swig_hour_set = *LibreDWGc::Dwg_Object_DATALINK_hour_set;
*swig_minute_get = *LibreDWGc::Dwg_Object_DATALINK_minute_get;
*swig_minute_set = *LibreDWGc::Dwg_Object_DATALINK_minute_set;
*swig_seconds_get = *LibreDWGc::Dwg_Object_DATALINK_seconds_get;
*swig_seconds_set = *LibreDWGc::Dwg_Object_DATALINK_seconds_set;
*swig_msec_get = *LibreDWGc::Dwg_Object_DATALINK_msec_get;
*swig_msec_set = *LibreDWGc::Dwg_Object_DATALINK_msec_set;
*swig_path_option_get = *LibreDWGc::Dwg_Object_DATALINK_path_option_get;
*swig_path_option_set = *LibreDWGc::Dwg_Object_DATALINK_path_option_set;
*swig_bl93_get = *LibreDWGc::Dwg_Object_DATALINK_bl93_get;
*swig_bl93_set = *LibreDWGc::Dwg_Object_DATALINK_bl93_set;
*swig_update_status_get = *LibreDWGc::Dwg_Object_DATALINK_update_status_get;
*swig_update_status_set = *LibreDWGc::Dwg_Object_DATALINK_update_status_set;
*swig_num_customdata_get = *LibreDWGc::Dwg_Object_DATALINK_num_customdata_get;
*swig_num_customdata_set = *LibreDWGc::Dwg_Object_DATALINK_num_customdata_set;
*swig_customdata_get = *LibreDWGc::Dwg_Object_DATALINK_customdata_get;
*swig_customdata_set = *LibreDWGc::Dwg_Object_DATALINK_customdata_set;
*swig_hardowner_get = *LibreDWGc::Dwg_Object_DATALINK_hardowner_get;
*swig_hardowner_set = *LibreDWGc::Dwg_Object_DATALINK_hardowner_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DATALINK(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DATALINK($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_DIMASSOC_Ref ##############

package LibreDWG::Dwg_DIMASSOC_Ref;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_DIMASSOC_Ref_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_DIMASSOC_Ref_parent_set;
*swig_classname_get = *LibreDWGc::Dwg_DIMASSOC_Ref_classname_get;
*swig_classname_set = *LibreDWGc::Dwg_DIMASSOC_Ref_classname_set;
*swig_osnap_type_get = *LibreDWGc::Dwg_DIMASSOC_Ref_osnap_type_get;
*swig_osnap_type_set = *LibreDWGc::Dwg_DIMASSOC_Ref_osnap_type_set;
*swig_osnap_dist_get = *LibreDWGc::Dwg_DIMASSOC_Ref_osnap_dist_get;
*swig_osnap_dist_set = *LibreDWGc::Dwg_DIMASSOC_Ref_osnap_dist_set;
*swig_osnap_pt_get = *LibreDWGc::Dwg_DIMASSOC_Ref_osnap_pt_get;
*swig_osnap_pt_set = *LibreDWGc::Dwg_DIMASSOC_Ref_osnap_pt_set;
*swig_num_xrefs_get = *LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefs_get;
*swig_num_xrefs_set = *LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefs_set;
*swig_xrefs_get = *LibreDWGc::Dwg_DIMASSOC_Ref_xrefs_get;
*swig_xrefs_set = *LibreDWGc::Dwg_DIMASSOC_Ref_xrefs_set;
*swig_main_subent_type_get = *LibreDWGc::Dwg_DIMASSOC_Ref_main_subent_type_get;
*swig_main_subent_type_set = *LibreDWGc::Dwg_DIMASSOC_Ref_main_subent_type_set;
*swig_main_gsmarker_get = *LibreDWGc::Dwg_DIMASSOC_Ref_main_gsmarker_get;
*swig_main_gsmarker_set = *LibreDWGc::Dwg_DIMASSOC_Ref_main_gsmarker_set;
*swig_num_xrefpaths_get = *LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefpaths_get;
*swig_num_xrefpaths_set = *LibreDWGc::Dwg_DIMASSOC_Ref_num_xrefpaths_set;
*swig_xrefpaths_get = *LibreDWGc::Dwg_DIMASSOC_Ref_xrefpaths_get;
*swig_xrefpaths_set = *LibreDWGc::Dwg_DIMASSOC_Ref_xrefpaths_set;
*swig_has_lastpt_ref_get = *LibreDWGc::Dwg_DIMASSOC_Ref_has_lastpt_ref_get;
*swig_has_lastpt_ref_set = *LibreDWGc::Dwg_DIMASSOC_Ref_has_lastpt_ref_set;
*swig_lastpt_ref_get = *LibreDWGc::Dwg_DIMASSOC_Ref_lastpt_ref_get;
*swig_lastpt_ref_set = *LibreDWGc::Dwg_DIMASSOC_Ref_lastpt_ref_set;
*swig_num_intsectobj_get = *LibreDWGc::Dwg_DIMASSOC_Ref_num_intsectobj_get;
*swig_num_intsectobj_set = *LibreDWGc::Dwg_DIMASSOC_Ref_num_intsectobj_set;
*swig_intsectobj_get = *LibreDWGc::Dwg_DIMASSOC_Ref_intsectobj_get;
*swig_intsectobj_set = *LibreDWGc::Dwg_DIMASSOC_Ref_intsectobj_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_DIMASSOC_Ref(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_DIMASSOC_Ref($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DIMASSOC ##############

package LibreDWG::Dwg_Object_DIMASSOC;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DIMASSOC_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DIMASSOC_parent_set;
*swig_dimensionobj_get = *LibreDWGc::Dwg_Object_DIMASSOC_dimensionobj_get;
*swig_dimensionobj_set = *LibreDWGc::Dwg_Object_DIMASSOC_dimensionobj_set;
*swig_associativity_get = *LibreDWGc::Dwg_Object_DIMASSOC_associativity_get;
*swig_associativity_set = *LibreDWGc::Dwg_Object_DIMASSOC_associativity_set;
*swig_trans_space_flag_get = *LibreDWGc::Dwg_Object_DIMASSOC_trans_space_flag_get;
*swig_trans_space_flag_set = *LibreDWGc::Dwg_Object_DIMASSOC_trans_space_flag_set;
*swig_rotated_type_get = *LibreDWGc::Dwg_Object_DIMASSOC_rotated_type_get;
*swig_rotated_type_set = *LibreDWGc::Dwg_Object_DIMASSOC_rotated_type_set;
*swig_ref_get = *LibreDWGc::Dwg_Object_DIMASSOC_ref_get;
*swig_ref_set = *LibreDWGc::Dwg_Object_DIMASSOC_ref_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DIMASSOC(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DIMASSOC($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ACTIONBODY ##############

package LibreDWG::Dwg_ACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ACTIONBODY_parent_set;
*swig_evaluatorid_get = *LibreDWGc::Dwg_ACTIONBODY_evaluatorid_get;
*swig_evaluatorid_set = *LibreDWGc::Dwg_ACTIONBODY_evaluatorid_set;
*swig_expression_get = *LibreDWGc::Dwg_ACTIONBODY_expression_get;
*swig_expression_set = *LibreDWGc::Dwg_ACTIONBODY_expression_set;
*swig_value_get = *LibreDWGc::Dwg_ACTIONBODY_value_get;
*swig_value_set = *LibreDWGc::Dwg_ACTIONBODY_value_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_EvalVariant ##############

package LibreDWG::Dwg_EvalVariant;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_code_get = *LibreDWGc::Dwg_EvalVariant_code_get;
*swig_code_set = *LibreDWGc::Dwg_EvalVariant_code_set;
*swig_u_get = *LibreDWGc::Dwg_EvalVariant_u_get;
*swig_u_set = *LibreDWGc::Dwg_EvalVariant_u_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_EvalVariant(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_EvalVariant($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_EvalVariant_u ##############

package LibreDWG::_dwg_EvalVariant_u;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_bd_get = *LibreDWGc::_dwg_EvalVariant_u_bd_get;
*swig_bd_set = *LibreDWGc::_dwg_EvalVariant_u_bd_set;
*swig_bl_get = *LibreDWGc::_dwg_EvalVariant_u_bl_get;
*swig_bl_set = *LibreDWGc::_dwg_EvalVariant_u_bl_set;
*swig_bs_get = *LibreDWGc::_dwg_EvalVariant_u_bs_get;
*swig_bs_set = *LibreDWGc::_dwg_EvalVariant_u_bs_set;
*swig_rc_get = *LibreDWGc::_dwg_EvalVariant_u_rc_get;
*swig_rc_set = *LibreDWGc::_dwg_EvalVariant_u_rc_set;
*swig_text_get = *LibreDWGc::_dwg_EvalVariant_u_text_get;
*swig_text_set = *LibreDWGc::_dwg_EvalVariant_u_text_set;
*swig_handle_get = *LibreDWGc::_dwg_EvalVariant_u_handle_get;
*swig_handle_set = *LibreDWGc::_dwg_EvalVariant_u_handle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_EvalVariant_u(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_EvalVariant_u($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_VALUEPARAM_vars ##############

package LibreDWG::Dwg_VALUEPARAM_vars;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_value_get = *LibreDWGc::Dwg_VALUEPARAM_vars_value_get;
*swig_value_set = *LibreDWGc::Dwg_VALUEPARAM_vars_value_set;
*swig_handle_get = *LibreDWGc::Dwg_VALUEPARAM_vars_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_VALUEPARAM_vars_handle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_VALUEPARAM_vars(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_VALUEPARAM_vars($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_VALUEPARAM ##############

package LibreDWG::Dwg_VALUEPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_VALUEPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_VALUEPARAM_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_VALUEPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_VALUEPARAM_class_version_set;
*swig_name_get = *LibreDWGc::Dwg_VALUEPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_VALUEPARAM_name_set;
*swig_unit_type_get = *LibreDWGc::Dwg_VALUEPARAM_unit_type_get;
*swig_unit_type_set = *LibreDWGc::Dwg_VALUEPARAM_unit_type_set;
*swig_num_vars_get = *LibreDWGc::Dwg_VALUEPARAM_num_vars_get;
*swig_num_vars_set = *LibreDWGc::Dwg_VALUEPARAM_num_vars_set;
*swig_vars_get = *LibreDWGc::Dwg_VALUEPARAM_vars_get;
*swig_vars_set = *LibreDWGc::Dwg_VALUEPARAM_vars_set;
*swig_controlled_objdep_get = *LibreDWGc::Dwg_VALUEPARAM_controlled_objdep_get;
*swig_controlled_objdep_set = *LibreDWGc::Dwg_VALUEPARAM_controlled_objdep_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_VALUEPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_VALUEPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ASSOCPARAMBASEDACTIONBODY ##############

package LibreDWG::Dwg_ASSOCPARAMBASEDACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_parent_set;
*swig_version_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_version_get;
*swig_version_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_version_set;
*swig_minor_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_minor_set;
*swig_num_deps_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_deps_set;
*swig_l4_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l4_get;
*swig_l4_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l4_set;
*swig_l5_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l5_get;
*swig_l5_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_l5_set;
*swig_assocdep_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_get;
*swig_assocdep_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_assocdep_set;
*swig_num_values_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_get;
*swig_num_values_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_num_values_set;
*swig_values_get = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_values_get;
*swig_values_set = *LibreDWGc::Dwg_ASSOCPARAMBASEDACTIONBODY_values_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ASSOCPARAMBASEDACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ASSOCPARAMBASEDACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ASSOCACTION_Deps ##############

package LibreDWG::Dwg_ASSOCACTION_Deps;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ASSOCACTION_Deps_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ASSOCACTION_Deps_parent_set;
*swig_is_owned_get = *LibreDWGc::Dwg_ASSOCACTION_Deps_is_owned_get;
*swig_is_owned_set = *LibreDWGc::Dwg_ASSOCACTION_Deps_is_owned_set;
*swig_dep_get = *LibreDWGc::Dwg_ASSOCACTION_Deps_dep_get;
*swig_dep_set = *LibreDWGc::Dwg_ASSOCACTION_Deps_dep_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ASSOCACTION_Deps(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ASSOCACTION_Deps($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCDEPENDENCY ##############

package LibreDWG::Dwg_Object_ASSOCDEPENDENCY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_class_version_set;
*swig_status_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_status_get;
*swig_status_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_status_set;
*swig_is_read_dep_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_read_dep_get;
*swig_is_read_dep_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_read_dep_set;
*swig_is_write_dep_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_write_dep_get;
*swig_is_write_dep_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_write_dep_set;
*swig_is_attached_to_object_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_get;
*swig_is_attached_to_object_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_attached_to_object_set;
*swig_is_delegating_to_owning_action_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_get;
*swig_is_delegating_to_owning_action_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_is_delegating_to_owning_action_set;
*swig_order_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_order_get;
*swig_order_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_order_set;
*swig_dep_on_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_on_get;
*swig_dep_on_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_on_set;
*swig_has_name_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_has_name_get;
*swig_has_name_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_has_name_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_name_set;
*swig_depbodyid_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_depbodyid_get;
*swig_depbodyid_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_depbodyid_set;
*swig_readdep_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_readdep_get;
*swig_readdep_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_readdep_set;
*swig_dep_body_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_body_get;
*swig_dep_body_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_dep_body_set;
*swig_node_get = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_node_get;
*swig_node_set = *LibreDWGc::Dwg_Object_ASSOCDEPENDENCY_node_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCDEPENDENCY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCDEPENDENCY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCVALUEDEPENDENCY ##############

package LibreDWG::Dwg_Object_ASSOCVALUEDEPENDENCY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_parent_set;
*swig_assocdep_get = *LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_get;
*swig_assocdep_set = *LibreDWGc::Dwg_Object_ASSOCVALUEDEPENDENCY_assocdep_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCVALUEDEPENDENCY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCVALUEDEPENDENCY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCGEOMDEPENDENCY ##############

package LibreDWG::Dwg_Object_ASSOCGEOMDEPENDENCY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_parent_set;
*swig_assocdep_get = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_get;
*swig_assocdep_set = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_assocdep_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_class_version_set;
*swig_enabled_get = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_get;
*swig_enabled_set = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_enabled_set;
*swig_classname_get = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_classname_get;
*swig_classname_set = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_classname_set;
*swig_dependent_on_compound_object_get = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_get;
*swig_dependent_on_compound_object_set = *LibreDWGc::Dwg_Object_ASSOCGEOMDEPENDENCY_dependent_on_compound_object_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCGEOMDEPENDENCY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCGEOMDEPENDENCY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCACTION ##############

package LibreDWG::Dwg_Object_ASSOCACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCACTION_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCACTION_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCACTION_class_version_set;
*swig_geometry_status_get = *LibreDWGc::Dwg_Object_ASSOCACTION_geometry_status_get;
*swig_geometry_status_set = *LibreDWGc::Dwg_Object_ASSOCACTION_geometry_status_set;
*swig_owningnetwork_get = *LibreDWGc::Dwg_Object_ASSOCACTION_owningnetwork_get;
*swig_owningnetwork_set = *LibreDWGc::Dwg_Object_ASSOCACTION_owningnetwork_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOCACTION_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOCACTION_actionbody_set;
*swig_action_index_get = *LibreDWGc::Dwg_Object_ASSOCACTION_action_index_get;
*swig_action_index_set = *LibreDWGc::Dwg_Object_ASSOCACTION_action_index_set;
*swig_max_assoc_dep_index_get = *LibreDWGc::Dwg_Object_ASSOCACTION_max_assoc_dep_index_get;
*swig_max_assoc_dep_index_set = *LibreDWGc::Dwg_Object_ASSOCACTION_max_assoc_dep_index_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_ASSOCACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_ASSOCACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_ASSOCACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_ASSOCACTION_deps_set;
*swig_num_owned_params_get = *LibreDWGc::Dwg_Object_ASSOCACTION_num_owned_params_get;
*swig_num_owned_params_set = *LibreDWGc::Dwg_Object_ASSOCACTION_num_owned_params_set;
*swig_owned_params_get = *LibreDWGc::Dwg_Object_ASSOCACTION_owned_params_get;
*swig_owned_params_set = *LibreDWGc::Dwg_Object_ASSOCACTION_owned_params_set;
*swig_num_values_get = *LibreDWGc::Dwg_Object_ASSOCACTION_num_values_get;
*swig_num_values_set = *LibreDWGc::Dwg_Object_ASSOCACTION_num_values_set;
*swig_values_get = *LibreDWGc::Dwg_Object_ASSOCACTION_values_get;
*swig_values_set = *LibreDWGc::Dwg_Object_ASSOCACTION_values_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCNETWORK ##############

package LibreDWG::Dwg_Object_ASSOCNETWORK;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_class_version_set;
*swig_geometry_status_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_geometry_status_get;
*swig_geometry_status_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_geometry_status_set;
*swig_owningnetwork_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_owningnetwork_get;
*swig_owningnetwork_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_owningnetwork_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_actionbody_set;
*swig_action_index_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_action_index_get;
*swig_action_index_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_action_index_set;
*swig_max_assoc_dep_index_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_get;
*swig_max_assoc_dep_index_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_max_assoc_dep_index_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_deps_set;
*swig_num_owned_params_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_params_get;
*swig_num_owned_params_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_params_set;
*swig_owned_params_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_params_get;
*swig_owned_params_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_params_set;
*swig_num_values_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_values_get;
*swig_num_values_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_values_set;
*swig_values_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_values_get;
*swig_values_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_values_set;
*swig_network_version_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_network_version_get;
*swig_network_version_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_network_version_set;
*swig_network_action_index_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_network_action_index_get;
*swig_network_action_index_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_network_action_index_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_actions_set;
*swig_num_owned_actions_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_actions_get;
*swig_num_owned_actions_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_num_owned_actions_set;
*swig_owned_actions_get = *LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_actions_get;
*swig_owned_actions_set = *LibreDWGc::Dwg_Object_ASSOCNETWORK_owned_actions_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCNETWORK(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCNETWORK($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_CONSTRAINTGROUPNODE ##############

package LibreDWG::Dwg_CONSTRAINTGROUPNODE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_parent_set;
*swig_nodeid_get = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_nodeid_get;
*swig_nodeid_set = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_nodeid_set;
*swig_status_get = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_status_get;
*swig_status_set = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_status_set;
*swig_num_connections_get = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_num_connections_get;
*swig_num_connections_set = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_num_connections_set;
*swig_connections_get = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_connections_get;
*swig_connections_set = *LibreDWGc::Dwg_CONSTRAINTGROUPNODE_connections_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_CONSTRAINTGROUPNODE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_CONSTRAINTGROUPNODE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOC2DCONSTRAINTGROUP ##############

package LibreDWG::Dwg_Object_ASSOC2DCONSTRAINTGROUP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_class_version_set;
*swig_geometry_status_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_get;
*swig_geometry_status_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_geometry_status_set;
*swig_owningnetwork_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_get;
*swig_owningnetwork_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owningnetwork_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actionbody_set;
*swig_action_index_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_get;
*swig_action_index_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_action_index_set;
*swig_max_assoc_dep_index_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_get;
*swig_max_assoc_dep_index_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_max_assoc_dep_index_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_deps_set;
*swig_num_owned_params_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_get;
*swig_num_owned_params_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_owned_params_set;
*swig_owned_params_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_get;
*swig_owned_params_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_owned_params_set;
*swig_num_values_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_get;
*swig_num_values_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_values_set;
*swig_values_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_get;
*swig_values_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_values_set;
*swig_version_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_get;
*swig_version_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_version_set;
*swig_b1_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_get;
*swig_b1_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_b1_set;
*swig_workplane_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_get;
*swig_workplane_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_workplane_set;
*swig_h1_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_get;
*swig_h1_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_h1_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_actions_set;
*swig_num_nodes_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_get;
*swig_num_nodes_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_num_nodes_set;
*swig_nodes_get = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_get;
*swig_nodes_set = *LibreDWGc::Dwg_Object_ASSOC2DCONSTRAINTGROUP_nodes_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOC2DCONSTRAINTGROUP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOC2DCONSTRAINTGROUP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCVARIABLE ##############

package LibreDWG::Dwg_Object_ASSOCVARIABLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_parent_set;
*swig_av_class_version_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_av_class_version_get;
*swig_av_class_version_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_av_class_version_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_class_version_set;
*swig_geometry_status_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_geometry_status_get;
*swig_geometry_status_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_geometry_status_set;
*swig_owningnetwork_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_owningnetwork_get;
*swig_owningnetwork_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_owningnetwork_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_actionbody_set;
*swig_action_index_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_action_index_get;
*swig_action_index_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_action_index_set;
*swig_max_assoc_dep_index_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_get;
*swig_max_assoc_dep_index_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_max_assoc_dep_index_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_deps_set;
*swig_num_owned_params_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_owned_params_get;
*swig_num_owned_params_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_owned_params_set;
*swig_owned_params_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_owned_params_get;
*swig_owned_params_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_owned_params_set;
*swig_num_values_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_values_get;
*swig_num_values_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_num_values_set;
*swig_values_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_values_get;
*swig_values_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_values_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_name_set;
*swig_t58_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_t58_get;
*swig_t58_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_t58_set;
*swig_evaluator_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_evaluator_get;
*swig_evaluator_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_evaluator_set;
*swig_desc_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_desc_get;
*swig_desc_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_desc_set;
*swig_value_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_value_get;
*swig_value_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_value_set;
*swig_has_t78_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_has_t78_get;
*swig_has_t78_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_has_t78_set;
*swig_t78_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_t78_get;
*swig_t78_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_t78_set;
*swig_b290_get = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_b290_get;
*swig_b290_set = *LibreDWGc::Dwg_Object_ASSOCVARIABLE_b290_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCVARIABLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCVARIABLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_EVAL_Node ##############

package LibreDWG::Dwg_EVAL_Node;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_EVAL_Node_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_EVAL_Node_parent_set;
*swig_id_get = *LibreDWGc::Dwg_EVAL_Node_id_get;
*swig_id_set = *LibreDWGc::Dwg_EVAL_Node_id_set;
*swig_edge_flags_get = *LibreDWGc::Dwg_EVAL_Node_edge_flags_get;
*swig_edge_flags_set = *LibreDWGc::Dwg_EVAL_Node_edge_flags_set;
*swig_nextid_get = *LibreDWGc::Dwg_EVAL_Node_nextid_get;
*swig_nextid_set = *LibreDWGc::Dwg_EVAL_Node_nextid_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_EVAL_Node_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_EVAL_Node_evalexpr_set;
*swig_node_get = *LibreDWGc::Dwg_EVAL_Node_node_get;
*swig_node_set = *LibreDWGc::Dwg_EVAL_Node_node_set;
*swig_active_cycles_get = *LibreDWGc::Dwg_EVAL_Node_active_cycles_get;
*swig_active_cycles_set = *LibreDWGc::Dwg_EVAL_Node_active_cycles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_EVAL_Node(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_EVAL_Node($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_EVAL_Edge ##############

package LibreDWG::Dwg_EVAL_Edge;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_EVAL_Edge_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_EVAL_Edge_parent_set;
*swig_id_get = *LibreDWGc::Dwg_EVAL_Edge_id_get;
*swig_id_set = *LibreDWGc::Dwg_EVAL_Edge_id_set;
*swig_nextid_get = *LibreDWGc::Dwg_EVAL_Edge_nextid_get;
*swig_nextid_set = *LibreDWGc::Dwg_EVAL_Edge_nextid_set;
*swig_e1_get = *LibreDWGc::Dwg_EVAL_Edge_e1_get;
*swig_e1_set = *LibreDWGc::Dwg_EVAL_Edge_e1_set;
*swig_e2_get = *LibreDWGc::Dwg_EVAL_Edge_e2_get;
*swig_e2_set = *LibreDWGc::Dwg_EVAL_Edge_e2_set;
*swig_e3_get = *LibreDWGc::Dwg_EVAL_Edge_e3_get;
*swig_e3_set = *LibreDWGc::Dwg_EVAL_Edge_e3_set;
*swig_out_edge_get = *LibreDWGc::Dwg_EVAL_Edge_out_edge_get;
*swig_out_edge_set = *LibreDWGc::Dwg_EVAL_Edge_out_edge_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_EVAL_Edge(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_EVAL_Edge($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_EVALUATION_GRAPH ##############

package LibreDWG::Dwg_Object_EVALUATION_GRAPH;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_parent_set;
*swig_major_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_minor_set;
*swig_first_nodeid_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_get;
*swig_first_nodeid_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_set;
*swig_first_nodeid_copy_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_get;
*swig_first_nodeid_copy_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_first_nodeid_copy_set;
*swig_num_nodes_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_nodes_get;
*swig_num_nodes_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_nodes_set;
*swig_nodes_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_nodes_get;
*swig_nodes_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_nodes_set;
*swig_has_graph_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_has_graph_get;
*swig_has_graph_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_has_graph_set;
*swig_num_edges_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_edges_get;
*swig_num_edges_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_num_edges_set;
*swig_edges_get = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_edges_get;
*swig_edges_set = *LibreDWGc::Dwg_Object_EVALUATION_GRAPH_edges_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_EVALUATION_GRAPH(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_EVALUATION_GRAPH($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER ##############

package LibreDWG::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_flag_set;
*swig_block_get = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_get;
*swig_block_set = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPURGEPREVENTER_block_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DYNAMICBLOCKPURGEPREVENTER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_PERSUBENTMGR ##############

package LibreDWG::Dwg_Object_PERSUBENTMGR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_class_version_set;
*swig_unknown_0_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_0_get;
*swig_unknown_0_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_0_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_unknown_2_set;
*swig_numassocsteps_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsteps_get;
*swig_numassocsteps_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsteps_set;
*swig_numassocsubents_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsubents_get;
*swig_numassocsubents_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_numassocsubents_set;
*swig_num_steps_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_num_steps_get;
*swig_num_steps_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_num_steps_set;
*swig_steps_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_steps_get;
*swig_steps_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_steps_set;
*swig_num_subents_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_num_subents_get;
*swig_num_subents_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_num_subents_set;
*swig_subents_get = *LibreDWGc::Dwg_Object_PERSUBENTMGR_subents_get;
*swig_subents_set = *LibreDWGc::Dwg_Object_PERSUBENTMGR_subents_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_PERSUBENTMGR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_PERSUBENTMGR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCPERSSUBENTMANAGER ##############

package LibreDWG::Dwg_Object_ASSOCPERSSUBENTMANAGER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_class_version_set;
*swig_unknown_3_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_get;
*swig_unknown_3_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_3_set;
*swig_unknown_0_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_get;
*swig_unknown_0_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_0_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_2_set;
*swig_num_steps_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_get;
*swig_num_steps_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_steps_set;
*swig_num_subents_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_get;
*swig_num_subents_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_num_subents_set;
*swig_steps_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_get;
*swig_steps_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_steps_set;
*swig_subents_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_get;
*swig_subents_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_subents_set;
*swig_unknown_bl6_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_get;
*swig_unknown_bl6_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6_set;
*swig_unknown_bl6a_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_get;
*swig_unknown_bl6a_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl6a_set;
*swig_unknown_bl7a_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_get;
*swig_unknown_bl7a_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7a_set;
*swig_unknown_bl7_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_get;
*swig_unknown_bl7_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl7_set;
*swig_unknown_bl8_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_get;
*swig_unknown_bl8_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl8_set;
*swig_unknown_bl9_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_get;
*swig_unknown_bl9_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl9_set;
*swig_unknown_bl10_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_get;
*swig_unknown_bl10_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl10_set;
*swig_unknown_bl11_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_get;
*swig_unknown_bl11_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl11_set;
*swig_unknown_bl12_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_get;
*swig_unknown_bl12_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl12_set;
*swig_unknown_bl13_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_get;
*swig_unknown_bl13_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl13_set;
*swig_unknown_bl14_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_get;
*swig_unknown_bl14_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl14_set;
*swig_unknown_bl15_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_get;
*swig_unknown_bl15_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl15_set;
*swig_unknown_bl16_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_get;
*swig_unknown_bl16_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl16_set;
*swig_unknown_bl17_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_get;
*swig_unknown_bl17_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl17_set;
*swig_unknown_bl18_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_get;
*swig_unknown_bl18_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl18_set;
*swig_unknown_bl19_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_get;
*swig_unknown_bl19_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl19_set;
*swig_unknown_bl20_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_get;
*swig_unknown_bl20_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl20_set;
*swig_unknown_bl21_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_get;
*swig_unknown_bl21_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl21_set;
*swig_unknown_bl22_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_get;
*swig_unknown_bl22_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl22_set;
*swig_unknown_bl23_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_get;
*swig_unknown_bl23_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl23_set;
*swig_unknown_bl24_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_get;
*swig_unknown_bl24_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl24_set;
*swig_unknown_bl25_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_get;
*swig_unknown_bl25_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl25_set;
*swig_unknown_bl26_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_get;
*swig_unknown_bl26_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl26_set;
*swig_unknown_bl27_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_get;
*swig_unknown_bl27_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl27_set;
*swig_unknown_bl28_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_get;
*swig_unknown_bl28_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl28_set;
*swig_unknown_bl29_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_get;
*swig_unknown_bl29_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl29_set;
*swig_unknown_bl30_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_get;
*swig_unknown_bl30_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl30_set;
*swig_unknown_bl31_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_get;
*swig_unknown_bl31_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl31_set;
*swig_unknown_bl32_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_get;
*swig_unknown_bl32_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl32_set;
*swig_unknown_bl33_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_get;
*swig_unknown_bl33_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl33_set;
*swig_unknown_bl34_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_get;
*swig_unknown_bl34_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl34_set;
*swig_unknown_bl35_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_get;
*swig_unknown_bl35_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl35_set;
*swig_unknown_bl36_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_get;
*swig_unknown_bl36_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_bl36_set;
*swig_unknown_b37_get = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_get;
*swig_unknown_b37_set = *LibreDWGc::Dwg_Object_ASSOCPERSSUBENTMANAGER_unknown_b37_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCPERSSUBENTMANAGER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCPERSSUBENTMANAGER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCACTIONPARAM_name_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_name_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_class_version_set;
*swig_bs1_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_get;
*swig_bs1_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bs1_set;
*swig_num_params_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_get;
*swig_num_params_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_num_params_set;
*swig_params_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_get;
*swig_params_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_params_set;
*swig_has_child_param_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_get;
*swig_has_child_param_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_has_child_param_set;
*swig_child_status_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_get;
*swig_child_status_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_status_set;
*swig_child_id_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_get;
*swig_child_id_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_id_set;
*swig_child_param_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_get;
*swig_child_param_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_child_param_set;
*swig_h330_2_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_get;
*swig_h330_2_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_2_set;
*swig_bl2_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_get;
*swig_bl2_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_bl2_set;
*swig_h330_3_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_get;
*swig_h330_3_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_h330_3_set;
*swig_status_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_get;
*swig_status_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_status_set;
*swig_osnap_mode_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_get;
*swig_osnap_mode_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_osnap_mode_set;
*swig_param_get = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_get;
*swig_param_set = *LibreDWGc::Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM_param_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCOSNAPPOINTREFACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCPOINTREFACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCPOINTREFACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_name_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_class_version_set;
*swig_bs1_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_get;
*swig_bs1_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bs1_set;
*swig_num_params_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_get;
*swig_num_params_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_num_params_set;
*swig_params_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_get;
*swig_params_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_params_set;
*swig_has_child_param_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_get;
*swig_has_child_param_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_has_child_param_set;
*swig_child_status_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_get;
*swig_child_status_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_status_set;
*swig_child_id_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_get;
*swig_child_id_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_id_set;
*swig_child_param_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_get;
*swig_child_param_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_child_param_set;
*swig_h330_2_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_get;
*swig_h330_2_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_2_set;
*swig_bl2_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_get;
*swig_bl2_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_bl2_set;
*swig_h330_3_get = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_get;
*swig_h330_3_set = *LibreDWGc::Dwg_Object_ASSOCPOINTREFACTIONPARAM_h330_3_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCPOINTREFACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCPOINTREFACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCASMBODYACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCASMBODYACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_name_set;
*swig_asdap_class_version_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_get;
*swig_asdap_class_version_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_asdap_class_version_set;
*swig_dep_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_get;
*swig_dep_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_dep_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_class_version_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_version_get;
*swig_version_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_get;
*swig_point_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Object_ASSOCASMBODYACTIONPARAM_acis_empty_bit_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCASMBODYACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCASMBODYACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_name_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_class_version_set;
*swig_bs1_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_get;
*swig_bs1_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bs1_set;
*swig_num_params_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_get;
*swig_num_params_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_num_params_set;
*swig_params_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_get;
*swig_params_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_params_set;
*swig_has_child_param_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_get;
*swig_has_child_param_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_has_child_param_set;
*swig_child_status_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_get;
*swig_child_status_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_status_set;
*swig_child_id_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_get;
*swig_child_id_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_id_set;
*swig_child_param_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_get;
*swig_child_param_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_child_param_set;
*swig_h330_2_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_get;
*swig_h330_2_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_2_set;
*swig_bl2_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_get;
*swig_bl2_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_bl2_set;
*swig_h330_3_get = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_get;
*swig_h330_3_set = *LibreDWGc::Dwg_Object_ASSOCCOMPOUNDACTIONPARAM_h330_3_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCCOMPOUNDACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCOBJECTACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCOBJECTACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_name_set;
*swig_asdap_class_version_get = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_get;
*swig_asdap_class_version_set = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_asdap_class_version_set;
*swig_dep_get = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_get;
*swig_dep_set = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_dep_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCOBJECTACTIONPARAM_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCOBJECTACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCOBJECTACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCEDGEACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCEDGEACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_name_set;
*swig_asdap_class_version_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_get;
*swig_asdap_class_version_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_asdap_class_version_set;
*swig_dep_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_dep_get;
*swig_dep_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_dep_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_class_version_set;
*swig_param_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_param_get;
*swig_param_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_param_set;
*swig_has_action_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_get;
*swig_has_action_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_has_action_set;
*swig_action_type_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_get;
*swig_action_type_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_action_type_set;
*swig_subent_get = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_subent_get;
*swig_subent_set = *LibreDWGc::Dwg_Object_ASSOCEDGEACTIONPARAM_subent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCEDGEACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCEDGEACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCFACEACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCFACEACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_name_set;
*swig_asdap_class_version_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_get;
*swig_asdap_class_version_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_asdap_class_version_set;
*swig_dep_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_dep_get;
*swig_dep_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_dep_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_class_version_set;
*swig_index_get = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_index_get;
*swig_index_set = *LibreDWGc::Dwg_Object_ASSOCFACEACTIONPARAM_index_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCFACEACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCFACEACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCPATHACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCPATHACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_name_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_class_version_set;
*swig_bs1_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bs1_get;
*swig_bs1_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bs1_set;
*swig_num_params_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_num_params_get;
*swig_num_params_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_num_params_set;
*swig_params_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_params_get;
*swig_params_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_params_set;
*swig_has_child_param_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_get;
*swig_has_child_param_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_has_child_param_set;
*swig_child_status_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_status_get;
*swig_child_status_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_status_set;
*swig_child_id_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_id_get;
*swig_child_id_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_id_set;
*swig_child_param_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_param_get;
*swig_child_param_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_child_param_set;
*swig_h330_2_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_get;
*swig_h330_2_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_2_set;
*swig_bl2_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bl2_get;
*swig_bl2_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_bl2_set;
*swig_h330_3_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_get;
*swig_h330_3_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_h330_3_set;
*swig_version_get = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_version_get;
*swig_version_set = *LibreDWGc::Dwg_Object_ASSOCPATHACTIONPARAM_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCPATHACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCPATHACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCVERTEXACTIONPARAM ##############

package LibreDWG::Dwg_Object_ASSOCVERTEXACTIONPARAM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_parent_set;
*swig_is_r2013_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_get;
*swig_is_r2013_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_is_r2013_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_aap_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_name_set;
*swig_asdap_class_version_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_get;
*swig_asdap_class_version_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_asdap_class_version_set;
*swig_dep_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_get;
*swig_dep_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_dep_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_class_version_set;
*swig_pt_get = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_get;
*swig_pt_set = *LibreDWGc::Dwg_Object_ASSOCVERTEXACTIONPARAM_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCVERTEXACTIONPARAM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCVERTEXACTIONPARAM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ASSOCARRAYITEM ##############

package LibreDWG::Dwg_ASSOCARRAYITEM;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_class_version_set;
*swig_itemloc_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_itemloc_get;
*swig_itemloc_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_itemloc_set;
*swig_flags_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_flags_set;
*swig_is_default_transmatrix_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_is_default_transmatrix_get;
*swig_is_default_transmatrix_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_is_default_transmatrix_set;
*swig_x_dir_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_x_dir_get;
*swig_x_dir_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_x_dir_set;
*swig_transmatrix_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_transmatrix_get;
*swig_transmatrix_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_transmatrix_set;
*swig_rel_transform_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_rel_transform_get;
*swig_rel_transform_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_rel_transform_set;
*swig_has_h1_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_has_h1_get;
*swig_has_h1_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_has_h1_set;
*swig_h1_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_h1_get;
*swig_h1_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_h1_set;
*swig_h2_get = *LibreDWGc::Dwg_ASSOCARRAYITEM_h2_get;
*swig_h2_set = *LibreDWGc::Dwg_ASSOCARRAYITEM_h2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ASSOCARRAYITEM(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ASSOCARRAYITEM($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCARRAYPARAMETERS ##############

package LibreDWG::Dwg_Object_ASSOCARRAYPARAMETERS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_parent_set;
*swig_aap_version_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_get;
*swig_aap_version_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_aap_version_set;
*swig_num_items_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_num_items_get;
*swig_num_items_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_num_items_set;
*swig_classname_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_classname_get;
*swig_classname_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_classname_set;
*swig_items_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_items_get;
*swig_items_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_items_set;
*swig_numitems_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numitems_get;
*swig_numitems_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numitems_set;
*swig_numrows_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numrows_get;
*swig_numrows_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numrows_set;
*swig_numlevels_get = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_get;
*swig_numlevels_set = *LibreDWGc::Dwg_Object_ASSOCARRAYPARAMETERS_numlevels_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCARRAYPARAMETERS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCARRAYPARAMETERS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_aab_version_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_class_version_set;
*swig_entity_get = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_get;
*swig_entity_set = *LibreDWGc::Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY_entity_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCRESTOREENTITYSTATEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ASSOCSURFACEACTIONBODY ##############

package LibreDWG::Dwg_ASSOCSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_parent_set;
*swig_version_get = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_version_get;
*swig_version_set = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_version_set;
*swig_is_semi_assoc_get = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_get;
*swig_is_semi_assoc_set = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_assoc_set;
*swig_l2_get = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_l2_get;
*swig_l2_set = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_l2_set;
*swig_is_semi_ovr_get = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_get;
*swig_is_semi_ovr_set = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_is_semi_ovr_set;
*swig_grip_status_get = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_grip_status_get;
*swig_grip_status_set = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_grip_status_set;
*swig_assocdep_get = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_assocdep_get;
*swig_assocdep_set = *LibreDWGc::Dwg_ASSOCSURFACEACTIONBODY_assocdep_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ASSOCSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ASSOCSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_class_version_set;
*swig_option_get = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_get;
*swig_option_set = *LibreDWGc::Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY_option_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCEXTENDSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCEXTRUDEDSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCPLANESURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCPLANESURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCPLANESURFACEACTIONBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCPLANESURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCPLANESURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCLOFTEDSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCNETWORKSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_class_version_set;
*swig_b1_get = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_get;
*swig_b1_set = *LibreDWGc::Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY_b1_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCOFFSETSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCREVOLVEDSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCSWEPTSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCEDGECHAMFERACTIONBODY_pbsab_status_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCEDGECHAMFERACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCEDGEFILLETACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCEDGEFILLETACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCEDGEFILLETACTIONBODY_pbsab_status_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCEDGEFILLETACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCEDGEFILLETACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_class_version_set;
*swig_b1_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_get;
*swig_b1_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b1_set;
*swig_b2_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_get;
*swig_b2_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_b2_set;
*swig_distance_get = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_get;
*swig_distance_set = *LibreDWGc::Dwg_Object_ASSOCTRIMSURFACEACTIONBODY_distance_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCTRIMSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_class_version_set;
*swig_b1_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_get;
*swig_b1_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b1_set;
*swig_b2_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_get;
*swig_b2_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b2_set;
*swig_b3_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_get;
*swig_b3_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b3_set;
*swig_b4_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_get;
*swig_b4_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b4_set;
*swig_b5_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_get;
*swig_b5_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_b5_set;
*swig_blend_options_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_get;
*swig_blend_options_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_blend_options_set;
*swig_bs2_get = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_get;
*swig_bs2_set = *LibreDWGc::Dwg_Object_ASSOCBLENDSURFACEACTIONBODY_bs2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCBLENDSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_class_version_set;
*swig_status_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_get;
*swig_status_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_status_set;
*swig_pt1_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_get;
*swig_pt1_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt1_set;
*swig_pt2_get = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_get;
*swig_pt2_set = *LibreDWGc::Dwg_Object_ASSOCFILLETSURFACEACTIONBODY_pt2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCFILLETSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pab_set;
*swig_sab_get = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_get;
*swig_sab_set = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_sab_set;
*swig_pbsab_status_get = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_get;
*swig_pbsab_status_set = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_pbsab_status_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCPATCHSURFACEACTIONBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCPATCHSURFACEACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ASSOCACTIONBODY_action ##############

package LibreDWG::Dwg_ASSOCACTIONBODY_action;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ASSOCACTIONBODY_action_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ASSOCACTIONBODY_action_parent_set;
*swig_depid_get = *LibreDWGc::Dwg_ASSOCACTIONBODY_action_depid_get;
*swig_depid_set = *LibreDWGc::Dwg_ASSOCACTIONBODY_action_depid_set;
*swig_dep_get = *LibreDWGc::Dwg_ASSOCACTIONBODY_action_dep_get;
*swig_dep_set = *LibreDWGc::Dwg_ASSOCACTIONBODY_action_dep_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ASSOCACTIONBODY_action(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ASSOCACTIONBODY_action($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCMLEADERACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCMLEADERACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_parent_set;
*swig_aaab_version_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_get;
*swig_aaab_version_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aaab_version_set;
*swig_assoc_dep_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_get;
*swig_assoc_dep_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_assoc_dep_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_aab_version_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actionbody_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_pab_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_class_version_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_ASSOCMLEADERACTIONBODY_actions_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCMLEADERACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCMLEADERACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_parent_set;
*swig_aaab_version_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_get;
*swig_aaab_version_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aaab_version_set;
*swig_assoc_dep_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_get;
*swig_assoc_dep_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_assoc_dep_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_aab_version_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_actionbody_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_pab_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_class_version_set;
*swig_r_node_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_get;
*swig_r_node_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_r_node_set;
*swig_d_node_get = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_get;
*swig_d_node_set = *LibreDWGc::Dwg_Object_ASSOCALIGNEDDIMACTIONBODY_d_node_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCALIGNEDDIMACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_parent_set;
*swig_aaab_version_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_get;
*swig_aaab_version_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aaab_version_set;
*swig_assoc_dep_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_get;
*swig_assoc_dep_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assoc_dep_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_aab_version_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_actionbody_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_pab_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_class_version_set;
*swig_r_node_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_get;
*swig_r_node_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_r_node_set;
*swig_d_node_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_get;
*swig_d_node_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_d_node_set;
*swig_assocdep_get = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_get;
*swig_assocdep_set = *LibreDWGc::Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY_assocdep_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOC3POINTANGULARDIMACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCORDINATEDIMACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCORDINATEDIMACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_parent_set;
*swig_aaab_version_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_get;
*swig_aaab_version_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aaab_version_set;
*swig_assoc_dep_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_get;
*swig_assoc_dep_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_assoc_dep_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_aab_version_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_actionbody_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_pab_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_class_version_set;
*swig_r_node_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_get;
*swig_r_node_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_r_node_set;
*swig_d_node_get = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_get;
*swig_d_node_set = *LibreDWGc::Dwg_Object_ASSOCORDINATEDIMACTIONBODY_d_node_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCORDINATEDIMACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCORDINATEDIMACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCROTATEDDIMACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCROTATEDDIMACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_parent_set;
*swig_aaab_version_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_get;
*swig_aaab_version_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aaab_version_set;
*swig_assoc_dep_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_get;
*swig_assoc_dep_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_assoc_dep_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_aab_version_set;
*swig_actionbody_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_get;
*swig_actionbody_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_actionbody_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_pab_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_class_version_set;
*swig_r_node_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_get;
*swig_r_node_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_r_node_set;
*swig_d_node_get = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_get;
*swig_d_node_set = *LibreDWGc::Dwg_Object_ASSOCROTATEDDIMACTIONBODY_d_node_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCROTATEDDIMACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCROTATEDDIMACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCDIMDEPENDENCYBODY ##############

package LibreDWG::Dwg_Object_ASSOCDIMDEPENDENCYBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_parent_set;
*swig_adb_version_get = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_get;
*swig_adb_version_set = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_adb_version_set;
*swig_dimbase_version_get = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_get;
*swig_dimbase_version_set = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_dimbase_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_name_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ASSOCDIMDEPENDENCYBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCDIMDEPENDENCYBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCDIMDEPENDENCYBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKPARAMDEPENDENCYBODY ##############

package LibreDWG::Dwg_Object_BLOCKPARAMDEPENDENCYBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_parent_set;
*swig_adb_version_get = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_get;
*swig_adb_version_set = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_adb_version_set;
*swig_dimbase_version_get = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_get;
*swig_dimbase_version_set = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_dimbase_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_name_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_BLOCKPARAMDEPENDENCYBODY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKPARAMDEPENDENCYBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKPARAMDEPENDENCYBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ARRAYITEMLOCATOR ##############

package LibreDWG::Dwg_ARRAYITEMLOCATOR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ARRAYITEMLOCATOR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ARRAYITEMLOCATOR_parent_set;
*swig_itemloc_get = *LibreDWGc::Dwg_ARRAYITEMLOCATOR_itemloc_get;
*swig_itemloc_set = *LibreDWGc::Dwg_ARRAYITEMLOCATOR_itemloc_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ARRAYITEMLOCATOR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ARRAYITEMLOCATOR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCARRAYACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCARRAYACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_pab_set;
*swig_aaab_version_get = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_get;
*swig_aaab_version_set = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_aaab_version_set;
*swig_paramblock_get = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_get;
*swig_paramblock_set = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_paramblock_set;
*swig_transmatrix_get = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_get;
*swig_transmatrix_set = *LibreDWGc::Dwg_Object_ASSOCARRAYACTIONBODY_transmatrix_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCARRAYACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCARRAYACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY ##############

package LibreDWG::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_parent_set;
*swig_aab_version_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_get;
*swig_aab_version_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aab_version_set;
*swig_pab_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_get;
*swig_pab_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_pab_set;
*swig_aaab_version_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_get;
*swig_aaab_version_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_aaab_version_set;
*swig_paramblock_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_get;
*swig_paramblock_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_paramblock_set;
*swig_transmatrix_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_get;
*swig_transmatrix_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_transmatrix_set;
*swig_status_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_get;
*swig_status_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_status_set;
*swig_num_items_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_get;
*swig_num_items_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_num_items_set;
*swig_items_get = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_get;
*swig_items_set = *LibreDWGc::Dwg_Object_ASSOCARRAYMODIFYACTIONBODY_items_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ASSOCARRAYMODIFYACTIONBODY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_EvalExpr ##############

package LibreDWG::Dwg_EvalExpr;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parentid_get = *LibreDWGc::Dwg_EvalExpr_parentid_get;
*swig_parentid_set = *LibreDWGc::Dwg_EvalExpr_parentid_set;
*swig_major_get = *LibreDWGc::Dwg_EvalExpr_major_get;
*swig_major_set = *LibreDWGc::Dwg_EvalExpr_major_set;
*swig_minor_get = *LibreDWGc::Dwg_EvalExpr_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_EvalExpr_minor_set;
*swig_value_code_get = *LibreDWGc::Dwg_EvalExpr_value_code_get;
*swig_value_code_set = *LibreDWGc::Dwg_EvalExpr_value_code_set;
*swig_value_get = *LibreDWGc::Dwg_EvalExpr_value_get;
*swig_value_set = *LibreDWGc::Dwg_EvalExpr_value_set;
*swig_nodeid_get = *LibreDWGc::Dwg_EvalExpr_nodeid_get;
*swig_nodeid_set = *LibreDWGc::Dwg_EvalExpr_nodeid_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_EvalExpr(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_EvalExpr($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_EvalExpr_value ##############

package LibreDWG::_dwg_EvalExpr_value;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num40_get = *LibreDWGc::_dwg_EvalExpr_value_num40_get;
*swig_num40_set = *LibreDWGc::_dwg_EvalExpr_value_num40_set;
*swig_pt2d_get = *LibreDWGc::_dwg_EvalExpr_value_pt2d_get;
*swig_pt2d_set = *LibreDWGc::_dwg_EvalExpr_value_pt2d_set;
*swig_pt3d_get = *LibreDWGc::_dwg_EvalExpr_value_pt3d_get;
*swig_pt3d_set = *LibreDWGc::_dwg_EvalExpr_value_pt3d_set;
*swig_text1_get = *LibreDWGc::_dwg_EvalExpr_value_text1_get;
*swig_text1_set = *LibreDWGc::_dwg_EvalExpr_value_text1_set;
*swig_long90_get = *LibreDWGc::_dwg_EvalExpr_value_long90_get;
*swig_long90_set = *LibreDWGc::_dwg_EvalExpr_value_long90_set;
*swig_handle91_get = *LibreDWGc::_dwg_EvalExpr_value_handle91_get;
*swig_handle91_set = *LibreDWGc::_dwg_EvalExpr_value_handle91_set;
*swig_short70_get = *LibreDWGc::_dwg_EvalExpr_value_short70_get;
*swig_short70_set = *LibreDWGc::_dwg_EvalExpr_value_short70_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_EvalExpr_value(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_EvalExpr_value($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ACSH_SubentMaterial ##############

package LibreDWG::Dwg_ACSH_SubentMaterial;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_major_get = *LibreDWGc::Dwg_ACSH_SubentMaterial_major_get;
*swig_major_set = *LibreDWGc::Dwg_ACSH_SubentMaterial_major_set;
*swig_minor_get = *LibreDWGc::Dwg_ACSH_SubentMaterial_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_ACSH_SubentMaterial_minor_set;
*swig_reflectance_get = *LibreDWGc::Dwg_ACSH_SubentMaterial_reflectance_get;
*swig_reflectance_set = *LibreDWGc::Dwg_ACSH_SubentMaterial_reflectance_set;
*swig_displacement_get = *LibreDWGc::Dwg_ACSH_SubentMaterial_displacement_get;
*swig_displacement_set = *LibreDWGc::Dwg_ACSH_SubentMaterial_displacement_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ACSH_SubentMaterial(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ACSH_SubentMaterial($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ACSH_SubentColor ##############

package LibreDWG::Dwg_ACSH_SubentColor;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_major_get = *LibreDWGc::Dwg_ACSH_SubentColor_major_get;
*swig_major_set = *LibreDWGc::Dwg_ACSH_SubentColor_major_set;
*swig_minor_get = *LibreDWGc::Dwg_ACSH_SubentColor_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_ACSH_SubentColor_minor_set;
*swig_transparency_get = *LibreDWGc::Dwg_ACSH_SubentColor_transparency_get;
*swig_transparency_set = *LibreDWGc::Dwg_ACSH_SubentColor_transparency_set;
*swig_bl93_get = *LibreDWGc::Dwg_ACSH_SubentColor_bl93_get;
*swig_bl93_set = *LibreDWGc::Dwg_ACSH_SubentColor_bl93_set;
*swig_is_face_variable_get = *LibreDWGc::Dwg_ACSH_SubentColor_is_face_variable_get;
*swig_is_face_variable_set = *LibreDWGc::Dwg_ACSH_SubentColor_is_face_variable_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ACSH_SubentColor(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ACSH_SubentColor($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ACSH_HistoryNode ##############

package LibreDWG::Dwg_ACSH_HistoryNode;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_major_get = *LibreDWGc::Dwg_ACSH_HistoryNode_major_get;
*swig_major_set = *LibreDWGc::Dwg_ACSH_HistoryNode_major_set;
*swig_minor_get = *LibreDWGc::Dwg_ACSH_HistoryNode_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_ACSH_HistoryNode_minor_set;
*swig_trans_get = *LibreDWGc::Dwg_ACSH_HistoryNode_trans_get;
*swig_trans_set = *LibreDWGc::Dwg_ACSH_HistoryNode_trans_set;
*swig_color_get = *LibreDWGc::Dwg_ACSH_HistoryNode_color_get;
*swig_color_set = *LibreDWGc::Dwg_ACSH_HistoryNode_color_set;
*swig_step_id_get = *LibreDWGc::Dwg_ACSH_HistoryNode_step_id_get;
*swig_step_id_set = *LibreDWGc::Dwg_ACSH_HistoryNode_step_id_set;
*swig_material_get = *LibreDWGc::Dwg_ACSH_HistoryNode_material_get;
*swig_material_set = *LibreDWGc::Dwg_ACSH_HistoryNode_material_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ACSH_HistoryNode(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ACSH_HistoryNode($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_HISTORY_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_HISTORY_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_parent_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_minor_set;
*swig_owner_get = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_owner_get;
*swig_owner_set = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_owner_set;
*swig_h_nodeid_get = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_get;
*swig_h_nodeid_set = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_h_nodeid_set;
*swig_show_history_get = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_show_history_get;
*swig_show_history_set = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_show_history_set;
*swig_record_history_get = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_record_history_get;
*swig_record_history_set = *LibreDWGc::Dwg_Object_ACSH_HISTORY_CLASS_record_history_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_HISTORY_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_HISTORY_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_BOX_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_BOX_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_minor_set;
*swig_length_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_length_get;
*swig_length_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_length_set;
*swig_width_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_width_get;
*swig_width_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_width_set;
*swig_height_get = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_height_get;
*swig_height_set = *LibreDWGc::Dwg_Object_ACSH_BOX_CLASS_height_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_BOX_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_BOX_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_WEDGE_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_WEDGE_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_minor_set;
*swig_length_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_length_get;
*swig_length_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_length_set;
*swig_width_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_width_get;
*swig_width_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_width_set;
*swig_height_get = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_height_get;
*swig_height_set = *LibreDWGc::Dwg_Object_ACSH_WEDGE_CLASS_height_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_WEDGE_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_WEDGE_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_BOOLEAN_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_BOOLEAN_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_minor_set;
*swig_operation_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operation_get;
*swig_operation_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operation_set;
*swig_operand1_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_get;
*swig_operand1_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand1_set;
*swig_operand2_get = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_get;
*swig_operand2_set = *LibreDWGc::Dwg_Object_ACSH_BOOLEAN_CLASS_operand2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_BOOLEAN_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_BOOLEAN_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_BREP_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_BREP_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_parent_set;
*swig_acis_empty_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_get;
*swig_acis_empty_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_set;
*swig_unknown_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_unknown_set;
*swig_version_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_version_get;
*swig_version_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_version_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_blocks_set;
*swig_block_size_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_block_size_get;
*swig_block_size_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_block_size_set;
*swig_encr_sat_data_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_get;
*swig_encr_sat_data_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_encr_sat_data_set;
*swig_sab_size_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_sab_size_get;
*swig_sab_size_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_sab_size_set;
*swig_acis_data_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_data_get;
*swig_acis_data_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_data_set;
*swig_wireframe_data_present_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_get;
*swig_wireframe_data_present_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wireframe_data_present_set;
*swig_point_present_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_present_get;
*swig_point_present_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_present_set;
*swig_point_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_get;
*swig_point_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_point_set;
*swig_isolines_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isolines_get;
*swig_isolines_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isolines_set;
*swig_isoline_present_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isoline_present_get;
*swig_isoline_present_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_isoline_present_set;
*swig_num_wires_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_wires_get;
*swig_num_wires_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_wires_set;
*swig_wires_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wires_get;
*swig_wires_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_wires_set;
*swig_num_silhouettes_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_get;
*swig_num_silhouettes_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_silhouettes_set;
*swig_silhouettes_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_silhouettes_get;
*swig_silhouettes_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_silhouettes_set;
*swig__dxf_sab_converted_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_get;
*swig__dxf_sab_converted_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS__dxf_sab_converted_set;
*swig_acis_empty2_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty2_get;
*swig_acis_empty2_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty2_set;
*swig_extra_acis_data_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_get;
*swig_extra_acis_data_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_extra_acis_data_set;
*swig_num_materials_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_materials_get;
*swig_num_materials_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_num_materials_set;
*swig_materials_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_materials_get;
*swig_materials_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_materials_set;
*swig_revision_guid_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_guid_get;
*swig_revision_guid_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_guid_set;
*swig_revision_major_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_major_get;
*swig_revision_major_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_major_set;
*swig_revision_minor1_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor1_get;
*swig_revision_minor1_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor1_set;
*swig_revision_minor2_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor2_get;
*swig_revision_minor2_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_minor2_set;
*swig_revision_bytes_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_bytes_get;
*swig_revision_bytes_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_revision_bytes_set;
*swig_end_marker_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_end_marker_get;
*swig_end_marker_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_end_marker_set;
*swig_history_id_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_id_get;
*swig_history_id_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_id_set;
*swig_has_revision_guid_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_get;
*swig_has_revision_guid_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_has_revision_guid_set;
*swig_acis_empty_bit_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_get;
*swig_acis_empty_bit_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_acis_empty_bit_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_BREP_CLASS_minor_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_BREP_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_BREP_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_SWEEP_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_SWEEP_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_minor_set;
*swig_direction_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_direction_get;
*swig_direction_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_direction_set;
*swig_bl92_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bl92_get;
*swig_bl92_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bl92_set;
*swig_shsw_text_size_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_get;
*swig_shsw_text_size_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_size_set;
*swig_shsw_text_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_get;
*swig_shsw_text_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text_set;
*swig_shsw_bl93_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_get;
*swig_shsw_bl93_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_bl93_set;
*swig_shsw_text2_size_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_get;
*swig_shsw_text2_size_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_size_set;
*swig_shsw_text2_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_get;
*swig_shsw_text2_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_text2_set;
*swig_draft_angle_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_get;
*swig_draft_angle_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_draft_angle_set;
*swig_start_draft_dist_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_get;
*swig_start_draft_dist_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_start_draft_dist_set;
*swig_end_draft_dist_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_get;
*swig_end_draft_dist_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_end_draft_dist_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_scale_factor_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_twist_angle_set;
*swig_align_angle_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_angle_get;
*swig_align_angle_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_angle_set;
*swig_sweepentity_transform_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_get;
*swig_sweepentity_transform_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_sweepentity_transform_set;
*swig_pathentity_transform_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_get;
*swig_pathentity_transform_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pathentity_transform_set;
*swig_align_option_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_option_get;
*swig_align_option_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_align_option_set;
*swig_miter_option_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_miter_option_get;
*swig_miter_option_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_miter_option_set;
*swig_has_align_start_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_get;
*swig_has_align_start_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_has_align_start_set;
*swig_bank_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bank_get;
*swig_bank_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_bank_set;
*swig_check_intersections_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_get;
*swig_check_intersections_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_check_intersections_set;
*swig_shsw_b294_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_get;
*swig_shsw_b294_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b294_set;
*swig_shsw_b295_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_get;
*swig_shsw_b295_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b295_set;
*swig_shsw_b296_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_get;
*swig_shsw_b296_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_shsw_b296_set;
*swig_pt2_get = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pt2_get;
*swig_pt2_set = *LibreDWGc::Dwg_Object_ACSH_SWEEP_CLASS_pt2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_SWEEP_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_SWEEP_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_EXTRUSION_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_EXTRUSION_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_minor_set;
*swig_direction_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_direction_get;
*swig_direction_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_direction_set;
*swig_bl92_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_get;
*swig_bl92_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bl92_set;
*swig_shsw_text_size_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_get;
*swig_shsw_text_size_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_size_set;
*swig_shsw_text_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_get;
*swig_shsw_text_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text_set;
*swig_shsw_bl93_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_get;
*swig_shsw_bl93_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_bl93_set;
*swig_shsw_text2_size_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_get;
*swig_shsw_text2_size_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_size_set;
*swig_shsw_text2_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_get;
*swig_shsw_text2_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_text2_set;
*swig_draft_angle_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_get;
*swig_draft_angle_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_draft_angle_set;
*swig_start_draft_dist_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_get;
*swig_start_draft_dist_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_start_draft_dist_set;
*swig_end_draft_dist_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_get;
*swig_end_draft_dist_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_end_draft_dist_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_scale_factor_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_twist_angle_set;
*swig_align_angle_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_get;
*swig_align_angle_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_angle_set;
*swig_sweepentity_transform_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_get;
*swig_sweepentity_transform_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_sweepentity_transform_set;
*swig_pathentity_transform_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_get;
*swig_pathentity_transform_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pathentity_transform_set;
*swig_align_option_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_get;
*swig_align_option_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_align_option_set;
*swig_miter_option_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_get;
*swig_miter_option_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_miter_option_set;
*swig_has_align_start_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_get;
*swig_has_align_start_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_has_align_start_set;
*swig_bank_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bank_get;
*swig_bank_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_bank_set;
*swig_check_intersections_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_get;
*swig_check_intersections_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_check_intersections_set;
*swig_shsw_b294_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_get;
*swig_shsw_b294_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b294_set;
*swig_shsw_b295_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_get;
*swig_shsw_b295_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b295_set;
*swig_shsw_b296_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_get;
*swig_shsw_b296_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_shsw_b296_set;
*swig_pt2_get = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_get;
*swig_pt2_set = *LibreDWGc::Dwg_Object_ACSH_EXTRUSION_CLASS_pt2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_EXTRUSION_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_EXTRUSION_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_LOFT_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_LOFT_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_minor_set;
*swig_num_crosssects_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_get;
*swig_num_crosssects_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_crosssects_set;
*swig_crosssects_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_crosssects_get;
*swig_crosssects_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_crosssects_set;
*swig_num_guides_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_guides_get;
*swig_num_guides_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_num_guides_set;
*swig_guides_get = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_guides_get;
*swig_guides_set = *LibreDWGc::Dwg_Object_ACSH_LOFT_CLASS_guides_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_LOFT_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_LOFT_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_FILLET_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_FILLET_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_minor_set;
*swig_bl92_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_bl92_get;
*swig_bl92_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_bl92_set;
*swig_num_edges_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_edges_get;
*swig_num_edges_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_edges_set;
*swig_edges_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_edges_get;
*swig_edges_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_edges_set;
*swig_num_radiuses_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_get;
*swig_num_radiuses_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_radiuses_set;
*swig_num_startsetbacks_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_get;
*swig_num_startsetbacks_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_startsetbacks_set;
*swig_num_endsetbacks_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_get;
*swig_num_endsetbacks_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_num_endsetbacks_set;
*swig_radiuses_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_radiuses_get;
*swig_radiuses_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_radiuses_set;
*swig_startsetbacks_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_get;
*swig_startsetbacks_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_startsetbacks_set;
*swig_endsetbacks_get = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_get;
*swig_endsetbacks_set = *LibreDWGc::Dwg_Object_ACSH_FILLET_CLASS_endsetbacks_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_FILLET_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_FILLET_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_CHAMFER_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_CHAMFER_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_minor_set;
*swig_bl92_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl92_get;
*swig_bl92_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl92_set;
*swig_base_dist_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_get;
*swig_base_dist_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_base_dist_set;
*swig_other_dist_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_get;
*swig_other_dist_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_other_dist_set;
*swig_num_edges_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_get;
*swig_num_edges_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_num_edges_set;
*swig_edges_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_edges_get;
*swig_edges_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_edges_set;
*swig_bl95_get = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl95_get;
*swig_bl95_set = *LibreDWGc::Dwg_Object_ACSH_CHAMFER_CLASS_bl95_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_CHAMFER_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_CHAMFER_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_CYLINDER_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_CYLINDER_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_set;
*swig_height_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_height_get;
*swig_height_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_height_set;
*swig_major_radius_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_get;
*swig_major_radius_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_major_radius_set;
*swig_minor_radius_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_get;
*swig_minor_radius_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_minor_radius_set;
*swig_x_radius_get = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_get;
*swig_x_radius_set = *LibreDWGc::Dwg_Object_ACSH_CYLINDER_CLASS_x_radius_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_CYLINDER_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_CYLINDER_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_CONE_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_CONE_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_set;
*swig_height_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_height_get;
*swig_height_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_height_set;
*swig_major_radius_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_radius_get;
*swig_major_radius_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_major_radius_set;
*swig_minor_radius_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_radius_get;
*swig_minor_radius_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_minor_radius_set;
*swig_x_radius_get = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_x_radius_get;
*swig_x_radius_set = *LibreDWGc::Dwg_Object_ACSH_CONE_CLASS_x_radius_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_CONE_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_CONE_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_PYRAMID_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_PYRAMID_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_minor_set;
*swig_height_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_height_get;
*swig_height_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_height_set;
*swig_sides_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_sides_get;
*swig_sides_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_sides_set;
*swig_radius_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_radius_set;
*swig_topradius_get = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_topradius_get;
*swig_topradius_set = *LibreDWGc::Dwg_Object_ACSH_PYRAMID_CLASS_topradius_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_PYRAMID_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_PYRAMID_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_SPHERE_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_SPHERE_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_minor_set;
*swig_radius_get = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_Object_ACSH_SPHERE_CLASS_radius_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_SPHERE_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_SPHERE_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_TORUS_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_TORUS_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_set;
*swig_major_radius_get = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_radius_get;
*swig_major_radius_set = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_major_radius_set;
*swig_minor_radius_get = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_radius_get;
*swig_minor_radius_set = *LibreDWGc::Dwg_Object_ACSH_TORUS_CLASS_minor_radius_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_TORUS_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_TORUS_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACSH_REVOLVE_CLASS ##############

package LibreDWG::Dwg_Object_ACSH_REVOLVE_CLASS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_evalexpr_set;
*swig_history_node_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_history_node_get;
*swig_history_node_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_history_node_set;
*swig_major_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_major_get;
*swig_major_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_major_set;
*swig_minor_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_minor_get;
*swig_minor_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_minor_set;
*swig_axis_pt_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_get;
*swig_axis_pt_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_axis_pt_set;
*swig_direction_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_direction_get;
*swig_direction_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_direction_set;
*swig_revolve_angle_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_get;
*swig_revolve_angle_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_revolve_angle_set;
*swig_start_angle_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_get;
*swig_start_angle_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_start_angle_set;
*swig_draft_angle_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_get;
*swig_draft_angle_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_draft_angle_set;
*swig_bd44_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd44_get;
*swig_bd44_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd44_set;
*swig_bd45_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd45_get;
*swig_bd45_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_bd45_set;
*swig_twist_angle_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_get;
*swig_twist_angle_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_twist_angle_set;
*swig_b290_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_b290_get;
*swig_b290_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_b290_set;
*swig_is_close_to_axis_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_get;
*swig_is_close_to_axis_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_is_close_to_axis_set;
*swig_sweep_entity_get = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_get;
*swig_sweep_entity_set = *LibreDWGc::Dwg_Object_ACSH_REVOLVE_CLASS_sweep_entity_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACSH_REVOLVE_CLASS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACSH_REVOLVE_CLASS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_NAVISWORKSMODEL ##############

package LibreDWG::Dwg_Entity_NAVISWORKSMODEL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_parent_set;
*swig_flags_get = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_flags_set;
*swig_definition_get = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_definition_get;
*swig_definition_set = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_definition_set;
*swig_transmatrix_get = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_transmatrix_get;
*swig_transmatrix_set = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_transmatrix_set;
*swig_unitfactor_get = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_unitfactor_get;
*swig_unitfactor_set = *LibreDWGc::Dwg_Entity_NAVISWORKSMODEL_unitfactor_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_NAVISWORKSMODEL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_NAVISWORKSMODEL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_NAVISWORKSMODELDEF ##############

package LibreDWG::Dwg_Object_NAVISWORKSMODELDEF;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_parent_set;
*swig_flags_get = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_flags_set;
*swig_path_get = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_path_get;
*swig_path_set = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_path_set;
*swig_status_get = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_status_get;
*swig_status_set = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_status_set;
*swig_min_extent_get = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_min_extent_get;
*swig_min_extent_set = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_min_extent_set;
*swig_max_extent_get = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_max_extent_get;
*swig_max_extent_set = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_max_extent_set;
*swig_host_drawing_visibility_get = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_get;
*swig_host_drawing_visibility_set = *LibreDWGc::Dwg_Object_NAVISWORKSMODELDEF_host_drawing_visibility_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_NAVISWORKSMODELDEF(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_NAVISWORKSMODELDEF($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RENDERSETTINGS ##############

package LibreDWG::Dwg_Object_RENDERSETTINGS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_class_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_name_set;
*swig_fog_enabled_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_enabled_get;
*swig_fog_enabled_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_enabled_set;
*swig_fog_background_enabled_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_background_enabled_get;
*swig_fog_background_enabled_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_fog_background_enabled_set;
*swig_backfaces_enabled_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_backfaces_enabled_get;
*swig_backfaces_enabled_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_backfaces_enabled_set;
*swig_environ_image_enabled_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_enabled_get;
*swig_environ_image_enabled_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_enabled_set;
*swig_environ_image_filename_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_filename_get;
*swig_environ_image_filename_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_environ_image_filename_set;
*swig_description_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_description_set;
*swig_display_index_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_display_index_get;
*swig_display_index_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_display_index_set;
*swig_has_predefined_get = *LibreDWGc::Dwg_Object_RENDERSETTINGS_has_predefined_get;
*swig_has_predefined_set = *LibreDWGc::Dwg_Object_RENDERSETTINGS_has_predefined_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RENDERSETTINGS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RENDERSETTINGS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MENTALRAYRENDERSETTINGS ##############

package LibreDWG::Dwg_Object_MENTALRAYRENDERSETTINGS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_class_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_name_set;
*swig_fog_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_get;
*swig_fog_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_enabled_set;
*swig_fog_background_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_get;
*swig_fog_background_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fog_background_enabled_set;
*swig_backfaces_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_get;
*swig_backfaces_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_backfaces_enabled_set;
*swig_environ_image_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_get;
*swig_environ_image_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_enabled_set;
*swig_environ_image_filename_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_get;
*swig_environ_image_filename_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_environ_image_filename_set;
*swig_description_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_description_set;
*swig_display_index_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_get;
*swig_display_index_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_display_index_set;
*swig_has_predefined_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_get;
*swig_has_predefined_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_has_predefined_set;
*swig_mr_version_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_get;
*swig_mr_version_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_version_set;
*swig_sampling1_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_get;
*swig_sampling1_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling1_set;
*swig_sampling2_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_get;
*swig_sampling2_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling2_set;
*swig_sampling_mr_filter_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_get;
*swig_sampling_mr_filter_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_mr_filter_set;
*swig_sampling_filter1_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_get;
*swig_sampling_filter1_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter1_set;
*swig_sampling_filter2_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_get;
*swig_sampling_filter2_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_filter2_set;
*swig_sampling_contrast_color1_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_get;
*swig_sampling_contrast_color1_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color1_set;
*swig_sampling_contrast_color2_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_get;
*swig_sampling_contrast_color2_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color2_set;
*swig_sampling_contrast_color3_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_get;
*swig_sampling_contrast_color3_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color3_set;
*swig_sampling_contrast_color4_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_get;
*swig_sampling_contrast_color4_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_sampling_contrast_color4_set;
*swig_shadow_mode_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_get;
*swig_shadow_mode_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_mode_set;
*swig_shadow_maps_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_get;
*swig_shadow_maps_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_shadow_maps_enabled_set;
*swig_ray_tracing_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_get;
*swig_ray_tracing_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_tracing_enabled_set;
*swig_ray_trace_depth1_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_get;
*swig_ray_trace_depth1_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth1_set;
*swig_ray_trace_depth2_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_get;
*swig_ray_trace_depth2_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth2_set;
*swig_ray_trace_depth3_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_get;
*swig_ray_trace_depth3_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_ray_trace_depth3_set;
*swig_global_illumination_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_get;
*swig_global_illumination_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_global_illumination_enabled_set;
*swig_gi_sample_count_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_get;
*swig_gi_sample_count_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_count_set;
*swig_gi_sample_radius_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_get;
*swig_gi_sample_radius_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_enabled_set;
*swig_gi_sample_radius_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_get;
*swig_gi_sample_radius_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_sample_radius_set;
*swig_gi_photons_per_light_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_get;
*swig_gi_photons_per_light_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_gi_photons_per_light_set;
*swig_photon_trace_depth1_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_get;
*swig_photon_trace_depth1_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth1_set;
*swig_photon_trace_depth2_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_get;
*swig_photon_trace_depth2_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth2_set;
*swig_photon_trace_depth3_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_get;
*swig_photon_trace_depth3_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_photon_trace_depth3_set;
*swig_final_gathering_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_get;
*swig_final_gathering_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_final_gathering_enabled_set;
*swig_fg_ray_count_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_get;
*swig_fg_ray_count_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_ray_count_set;
*swig_fg_sample_radius_state1_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_get;
*swig_fg_sample_radius_state1_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state1_set;
*swig_fg_sample_radius_state2_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_get;
*swig_fg_sample_radius_state2_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state2_set;
*swig_fg_sample_radius_state3_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_get;
*swig_fg_sample_radius_state3_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius_state3_set;
*swig_fg_sample_radius1_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_get;
*swig_fg_sample_radius1_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius1_set;
*swig_fg_sample_radius2_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_get;
*swig_fg_sample_radius2_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_fg_sample_radius2_set;
*swig_light_luminance_scale_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_get;
*swig_light_luminance_scale_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_light_luminance_scale_set;
*swig_diagnostics_mode_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_get;
*swig_diagnostics_mode_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_mode_set;
*swig_diagnostics_grid_mode_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_get;
*swig_diagnostics_grid_mode_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_mode_set;
*swig_diagnostics_grid_float_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_get;
*swig_diagnostics_grid_float_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_grid_float_set;
*swig_diagnostics_photon_mode_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_get;
*swig_diagnostics_photon_mode_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_photon_mode_set;
*swig_diagnostics_bsp_mode_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_get;
*swig_diagnostics_bsp_mode_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_bsp_mode_set;
*swig_export_mi_enabled_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_get;
*swig_export_mi_enabled_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_export_mi_enabled_set;
*swig_mr_description_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_get;
*swig_mr_description_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_mr_description_set;
*swig_tile_size_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_get;
*swig_tile_size_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_size_set;
*swig_tile_order_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_get;
*swig_tile_order_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_tile_order_set;
*swig_memory_limit_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_get;
*swig_memory_limit_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_memory_limit_set;
*swig_diagnostics_samples_mode_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_get;
*swig_diagnostics_samples_mode_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_diagnostics_samples_mode_set;
*swig_energy_multiplier_get = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_get;
*swig_energy_multiplier_set = *LibreDWGc::Dwg_Object_MENTALRAYRENDERSETTINGS_energy_multiplier_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MENTALRAYRENDERSETTINGS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MENTALRAYRENDERSETTINGS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RAPIDRTRENDERSETTINGS ##############

package LibreDWG::Dwg_Object_RAPIDRTRENDERSETTINGS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_class_version_set;
*swig_name_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_name_set;
*swig_fog_enabled_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_get;
*swig_fog_enabled_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_enabled_set;
*swig_fog_background_enabled_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_get;
*swig_fog_background_enabled_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_fog_background_enabled_set;
*swig_backfaces_enabled_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_get;
*swig_backfaces_enabled_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_backfaces_enabled_set;
*swig_environ_image_enabled_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_get;
*swig_environ_image_enabled_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_enabled_set;
*swig_environ_image_filename_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_get;
*swig_environ_image_filename_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_environ_image_filename_set;
*swig_description_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_description_get;
*swig_description_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_description_set;
*swig_display_index_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_get;
*swig_display_index_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_display_index_set;
*swig_has_predefined_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_get;
*swig_has_predefined_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_has_predefined_set;
*swig_rapidrt_version_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_get;
*swig_rapidrt_version_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_rapidrt_version_set;
*swig_render_target_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_get;
*swig_render_target_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_target_set;
*swig_render_level_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_get;
*swig_render_level_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_level_set;
*swig_render_time_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_get;
*swig_render_time_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_render_time_set;
*swig_lighting_model_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_get;
*swig_lighting_model_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_lighting_model_set;
*swig_filter_type_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_get;
*swig_filter_type_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_type_set;
*swig_filter_width_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_get;
*swig_filter_width_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_width_set;
*swig_filter_height_get = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_get;
*swig_filter_height_set = *LibreDWGc::Dwg_Object_RAPIDRTRENDERSETTINGS_filter_height_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RAPIDRTRENDERSETTINGS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RAPIDRTRENDERSETTINGS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RENDERENVIRONMENT ##############

package LibreDWG::Dwg_Object_RENDERENVIRONMENT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_class_version_set;
*swig_fog_enabled_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_enabled_get;
*swig_fog_enabled_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_enabled_set;
*swig_fog_background_enabled_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_get;
*swig_fog_background_enabled_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_background_enabled_set;
*swig_fog_color_r_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_r_get;
*swig_fog_color_r_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_r_set;
*swig_fog_color_g_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_g_get;
*swig_fog_color_g_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_g_set;
*swig_fog_color_b_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_b_get;
*swig_fog_color_b_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_color_b_set;
*swig_fog_density_near_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_near_get;
*swig_fog_density_near_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_near_set;
*swig_fog_density_far_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_far_get;
*swig_fog_density_far_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_density_far_set;
*swig_fog_distance_near_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_near_get;
*swig_fog_distance_near_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_near_set;
*swig_fog_distance_far_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_far_get;
*swig_fog_distance_far_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_fog_distance_far_set;
*swig_environ_image_enabled_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_get;
*swig_environ_image_enabled_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_enabled_set;
*swig_environ_image_filename_get = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_filename_get;
*swig_environ_image_filename_set = *LibreDWGc::Dwg_Object_RENDERENVIRONMENT_environ_image_filename_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RENDERENVIRONMENT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RENDERENVIRONMENT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RENDERGLOBAL ##############

package LibreDWG::Dwg_Object_RENDERGLOBAL;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_class_version_set;
*swig_procedure_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_procedure_get;
*swig_procedure_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_procedure_set;
*swig_destination_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_destination_get;
*swig_destination_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_destination_set;
*swig_save_enabled_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_save_enabled_get;
*swig_save_enabled_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_save_enabled_set;
*swig_save_filename_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_save_filename_get;
*swig_save_filename_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_save_filename_set;
*swig_image_width_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_image_width_get;
*swig_image_width_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_image_width_set;
*swig_image_height_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_image_height_get;
*swig_image_height_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_image_height_set;
*swig_predef_presets_first_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_predef_presets_first_get;
*swig_predef_presets_first_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_predef_presets_first_set;
*swig_highlevel_info_get = *LibreDWGc::Dwg_Object_RENDERGLOBAL_highlevel_info_get;
*swig_highlevel_info_set = *LibreDWGc::Dwg_Object_RENDERGLOBAL_highlevel_info_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RENDERGLOBAL(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RENDERGLOBAL($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RENDERENTRY ##############

package LibreDWG::Dwg_Object_RENDERENTRY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RENDERENTRY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RENDERENTRY_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RENDERENTRY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RENDERENTRY_class_version_set;
*swig_image_file_name_get = *LibreDWGc::Dwg_Object_RENDERENTRY_image_file_name_get;
*swig_image_file_name_set = *LibreDWGc::Dwg_Object_RENDERENTRY_image_file_name_set;
*swig_preset_name_get = *LibreDWGc::Dwg_Object_RENDERENTRY_preset_name_get;
*swig_preset_name_set = *LibreDWGc::Dwg_Object_RENDERENTRY_preset_name_set;
*swig_view_name_get = *LibreDWGc::Dwg_Object_RENDERENTRY_view_name_get;
*swig_view_name_set = *LibreDWGc::Dwg_Object_RENDERENTRY_view_name_set;
*swig_dimension_x_get = *LibreDWGc::Dwg_Object_RENDERENTRY_dimension_x_get;
*swig_dimension_x_set = *LibreDWGc::Dwg_Object_RENDERENTRY_dimension_x_set;
*swig_dimension_y_get = *LibreDWGc::Dwg_Object_RENDERENTRY_dimension_y_get;
*swig_dimension_y_set = *LibreDWGc::Dwg_Object_RENDERENTRY_dimension_y_set;
*swig_start_year_get = *LibreDWGc::Dwg_Object_RENDERENTRY_start_year_get;
*swig_start_year_set = *LibreDWGc::Dwg_Object_RENDERENTRY_start_year_set;
*swig_start_month_get = *LibreDWGc::Dwg_Object_RENDERENTRY_start_month_get;
*swig_start_month_set = *LibreDWGc::Dwg_Object_RENDERENTRY_start_month_set;
*swig_start_day_get = *LibreDWGc::Dwg_Object_RENDERENTRY_start_day_get;
*swig_start_day_set = *LibreDWGc::Dwg_Object_RENDERENTRY_start_day_set;
*swig_start_minute_get = *LibreDWGc::Dwg_Object_RENDERENTRY_start_minute_get;
*swig_start_minute_set = *LibreDWGc::Dwg_Object_RENDERENTRY_start_minute_set;
*swig_start_second_get = *LibreDWGc::Dwg_Object_RENDERENTRY_start_second_get;
*swig_start_second_set = *LibreDWGc::Dwg_Object_RENDERENTRY_start_second_set;
*swig_start_msec_get = *LibreDWGc::Dwg_Object_RENDERENTRY_start_msec_get;
*swig_start_msec_set = *LibreDWGc::Dwg_Object_RENDERENTRY_start_msec_set;
*swig_render_time_get = *LibreDWGc::Dwg_Object_RENDERENTRY_render_time_get;
*swig_render_time_set = *LibreDWGc::Dwg_Object_RENDERENTRY_render_time_set;
*swig_memory_amount_get = *LibreDWGc::Dwg_Object_RENDERENTRY_memory_amount_get;
*swig_memory_amount_set = *LibreDWGc::Dwg_Object_RENDERENTRY_memory_amount_set;
*swig_material_count_get = *LibreDWGc::Dwg_Object_RENDERENTRY_material_count_get;
*swig_material_count_set = *LibreDWGc::Dwg_Object_RENDERENTRY_material_count_set;
*swig_light_count_get = *LibreDWGc::Dwg_Object_RENDERENTRY_light_count_get;
*swig_light_count_set = *LibreDWGc::Dwg_Object_RENDERENTRY_light_count_set;
*swig_triangle_count_get = *LibreDWGc::Dwg_Object_RENDERENTRY_triangle_count_get;
*swig_triangle_count_set = *LibreDWGc::Dwg_Object_RENDERENTRY_triangle_count_set;
*swig_display_index_get = *LibreDWGc::Dwg_Object_RENDERENTRY_display_index_get;
*swig_display_index_set = *LibreDWGc::Dwg_Object_RENDERENTRY_display_index_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RENDERENTRY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RENDERENTRY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MOTIONPATH ##############

package LibreDWG::Dwg_Object_MOTIONPATH;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MOTIONPATH_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MOTIONPATH_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_MOTIONPATH_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_MOTIONPATH_class_version_set;
*swig_camera_path_get = *LibreDWGc::Dwg_Object_MOTIONPATH_camera_path_get;
*swig_camera_path_set = *LibreDWGc::Dwg_Object_MOTIONPATH_camera_path_set;
*swig_target_path_get = *LibreDWGc::Dwg_Object_MOTIONPATH_target_path_get;
*swig_target_path_set = *LibreDWGc::Dwg_Object_MOTIONPATH_target_path_set;
*swig_viewtable_get = *LibreDWGc::Dwg_Object_MOTIONPATH_viewtable_get;
*swig_viewtable_set = *LibreDWGc::Dwg_Object_MOTIONPATH_viewtable_set;
*swig_frames_get = *LibreDWGc::Dwg_Object_MOTIONPATH_frames_get;
*swig_frames_set = *LibreDWGc::Dwg_Object_MOTIONPATH_frames_set;
*swig_frame_rate_get = *LibreDWGc::Dwg_Object_MOTIONPATH_frame_rate_get;
*swig_frame_rate_set = *LibreDWGc::Dwg_Object_MOTIONPATH_frame_rate_set;
*swig_corner_decel_get = *LibreDWGc::Dwg_Object_MOTIONPATH_corner_decel_get;
*swig_corner_decel_set = *LibreDWGc::Dwg_Object_MOTIONPATH_corner_decel_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MOTIONPATH(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MOTIONPATH($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_CURVEPATH ##############

package LibreDWG::Dwg_Object_CURVEPATH;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_CURVEPATH_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_CURVEPATH_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_CURVEPATH_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_CURVEPATH_class_version_set;
*swig_entity_get = *LibreDWGc::Dwg_Object_CURVEPATH_entity_get;
*swig_entity_set = *LibreDWGc::Dwg_Object_CURVEPATH_entity_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_CURVEPATH(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_CURVEPATH($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_POINTPATH ##############

package LibreDWG::Dwg_Object_POINTPATH;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_POINTPATH_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_POINTPATH_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_POINTPATH_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_POINTPATH_class_version_set;
*swig_point_get = *LibreDWGc::Dwg_Object_POINTPATH_point_get;
*swig_point_set = *LibreDWGc::Dwg_Object_POINTPATH_point_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_POINTPATH(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_POINTPATH($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_TVDEVICEPROPERTIES ##############

package LibreDWG::Dwg_Object_TVDEVICEPROPERTIES;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_parent_set;
*swig_flags_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_flags_set;
*swig_max_regen_threads_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_get;
*swig_max_regen_threads_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_max_regen_threads_set;
*swig_use_lut_palette_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_get;
*swig_use_lut_palette_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_use_lut_palette_set;
*swig_alt_hlt_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_get;
*swig_alt_hlt_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hlt_set;
*swig_alt_hltcolor_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_get;
*swig_alt_hltcolor_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_alt_hltcolor_set;
*swig_geom_shader_usage_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_get;
*swig_geom_shader_usage_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_geom_shader_usage_set;
*swig_blending_mode_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_blending_mode_get;
*swig_blending_mode_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_blending_mode_set;
*swig_antialiasing_level_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_get;
*swig_antialiasing_level_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_antialiasing_level_set;
*swig_bd2_get = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_bd2_get;
*swig_bd2_set = *LibreDWGc::Dwg_Object_TVDEVICEPROPERTIES_bd2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_TVDEVICEPROPERTIES(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_TVDEVICEPROPERTIES($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SKYLIGHT_BACKGROUND ##############

package LibreDWG::Dwg_Object_SKYLIGHT_BACKGROUND;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_class_version_set;
*swig_sunid_get = *LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_sunid_get;
*swig_sunid_set = *LibreDWGc::Dwg_Object_SKYLIGHT_BACKGROUND_sunid_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SKYLIGHT_BACKGROUND(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SKYLIGHT_BACKGROUND($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SOLID_BACKGROUND ##############

package LibreDWG::Dwg_Object_SOLID_BACKGROUND;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SOLID_BACKGROUND_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SOLID_BACKGROUND_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_SOLID_BACKGROUND_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_SOLID_BACKGROUND_class_version_set;
*swig_color_get = *LibreDWGc::Dwg_Object_SOLID_BACKGROUND_color_get;
*swig_color_set = *LibreDWGc::Dwg_Object_SOLID_BACKGROUND_color_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SOLID_BACKGROUND(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SOLID_BACKGROUND($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_IMAGE_BACKGROUND ##############

package LibreDWG::Dwg_Object_IMAGE_BACKGROUND;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_class_version_set;
*swig_filename_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_filename_get;
*swig_filename_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_filename_set;
*swig_fit_to_screen_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_get;
*swig_fit_to_screen_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_fit_to_screen_set;
*swig_maintain_aspect_ratio_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_get;
*swig_maintain_aspect_ratio_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_maintain_aspect_ratio_set;
*swig_use_tiling_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_use_tiling_get;
*swig_use_tiling_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_use_tiling_set;
*swig_offset_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_offset_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_IMAGE_BACKGROUND_scale_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_IMAGE_BACKGROUND(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_IMAGE_BACKGROUND($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_IBL_BACKGROUND ##############

package LibreDWG::Dwg_Object_IBL_BACKGROUND;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_class_version_set;
*swig_enable_get = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_enable_get;
*swig_enable_set = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_enable_set;
*swig_name_get = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_name_set;
*swig_rotation_get = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_rotation_set;
*swig_display_image_get = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_display_image_get;
*swig_display_image_set = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_display_image_set;
*swig_secondary_background_get = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_secondary_background_get;
*swig_secondary_background_set = *LibreDWGc::Dwg_Object_IBL_BACKGROUND_secondary_background_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_IBL_BACKGROUND(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_IBL_BACKGROUND($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_GRADIENT_BACKGROUND ##############

package LibreDWG::Dwg_Object_GRADIENT_BACKGROUND;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_class_version_set;
*swig_color_top_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_top_get;
*swig_color_top_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_top_set;
*swig_color_middle_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_middle_get;
*swig_color_middle_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_middle_set;
*swig_color_bottom_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_bottom_get;
*swig_color_bottom_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_color_bottom_set;
*swig_horizon_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_horizon_get;
*swig_horizon_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_horizon_set;
*swig_height_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_height_get;
*swig_height_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_height_set;
*swig_rotation_get = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Object_GRADIENT_BACKGROUND_rotation_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_GRADIENT_BACKGROUND(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_GRADIENT_BACKGROUND($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_GROUND_PLANE_BACKGROUND ##############

package LibreDWG::Dwg_Object_GROUND_PLANE_BACKGROUND;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_class_version_set;
*swig_color_sky_zenith_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_get;
*swig_color_sky_zenith_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_zenith_set;
*swig_color_sky_horizon_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_get;
*swig_color_sky_horizon_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_sky_horizon_set;
*swig_color_underground_horizon_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_get;
*swig_color_underground_horizon_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_horizon_set;
*swig_color_underground_azimuth_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_get;
*swig_color_underground_azimuth_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_underground_azimuth_set;
*swig_color_near_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_get;
*swig_color_near_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_near_set;
*swig_color_far_get = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_get;
*swig_color_far_set = *LibreDWGc::Dwg_Object_GROUND_PLANE_BACKGROUND_color_far_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_GROUND_PLANE_BACKGROUND(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_GROUND_PLANE_BACKGROUND($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA_scale_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ANNOTSCALEOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_CONTEXTDATA_dict ##############

package LibreDWG::Dwg_CONTEXTDATA_dict;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_CONTEXTDATA_dict_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_CONTEXTDATA_dict_parent_set;
*swig_text_get = *LibreDWGc::Dwg_CONTEXTDATA_dict_text_get;
*swig_text_set = *LibreDWGc::Dwg_CONTEXTDATA_dict_text_set;
*swig_itemhandle_get = *LibreDWGc::Dwg_CONTEXTDATA_dict_itemhandle_get;
*swig_itemhandle_set = *LibreDWGc::Dwg_CONTEXTDATA_dict_itemhandle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_CONTEXTDATA_dict(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_CONTEXTDATA_dict($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_CONTEXTDATA_submgr ##############

package LibreDWG::Dwg_CONTEXTDATA_submgr;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_CONTEXTDATA_submgr_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_CONTEXTDATA_submgr_parent_set;
*swig_handle_get = *LibreDWGc::Dwg_CONTEXTDATA_submgr_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_CONTEXTDATA_submgr_handle_set;
*swig_num_entries_get = *LibreDWGc::Dwg_CONTEXTDATA_submgr_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_CONTEXTDATA_submgr_num_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_CONTEXTDATA_submgr_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_CONTEXTDATA_submgr_entries_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_CONTEXTDATA_submgr(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_CONTEXTDATA_submgr($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_CONTEXTDATAMANAGER ##############

package LibreDWG::Dwg_Object_CONTEXTDATAMANAGER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_parent_set;
*swig_objectcontext_get = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_objectcontext_get;
*swig_objectcontext_set = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_objectcontext_set;
*swig_num_submgrs_get = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_get;
*swig_num_submgrs_set = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_num_submgrs_set;
*swig_submgrs_get = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_submgrs_get;
*swig_submgrs_set = *LibreDWGc::Dwg_Object_CONTEXTDATAMANAGER_submgrs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_CONTEXTDATAMANAGER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_CONTEXTDATAMANAGER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_TEXTOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_TEXTOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_scale_set;
*swig_horizontal_mode_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_get;
*swig_horizontal_mode_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_horizontal_mode_set;
*swig_rotation_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_rotation_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_ins_pt_set;
*swig_alignment_pt_get = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_get;
*swig_alignment_pt_set = *LibreDWGc::Dwg_Object_TEXTOBJECTCONTEXTDATA_alignment_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_TEXTOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_TEXTOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MTEXTOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_MTEXTOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_scale_set;
*swig_attachment_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_get;
*swig_attachment_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_attachment_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_ins_pt_set;
*swig_x_axis_dir_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_get;
*swig_x_axis_dir_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_x_axis_dir_set;
*swig_rect_height_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_get;
*swig_rect_height_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_height_set;
*swig_rect_width_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_get;
*swig_rect_width_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_rect_width_set;
*swig_extents_width_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_get;
*swig_extents_width_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_width_set;
*swig_extents_height_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_get;
*swig_extents_height_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_extents_height_set;
*swig_column_type_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_get;
*swig_column_type_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_type_set;
*swig_column_width_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_get;
*swig_column_width_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_width_set;
*swig_gutter_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_get;
*swig_gutter_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_gutter_set;
*swig_auto_height_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_get;
*swig_auto_height_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_auto_height_set;
*swig_flow_reversed_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_get;
*swig_flow_reversed_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_flow_reversed_set;
*swig_num_column_heights_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_get;
*swig_num_column_heights_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_num_column_heights_set;
*swig_column_heights_get = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_get;
*swig_column_heights_set = *LibreDWGc::Dwg_Object_MTEXTOBJECTCONTEXTDATA_column_heights_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MTEXTOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MTEXTOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_OCD_Dimension ##############

package LibreDWG::Dwg_OCD_Dimension;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_b293_get = *LibreDWGc::Dwg_OCD_Dimension_b293_get;
*swig_b293_set = *LibreDWGc::Dwg_OCD_Dimension_b293_set;
*swig_def_pt_get = *LibreDWGc::Dwg_OCD_Dimension_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_OCD_Dimension_def_pt_set;
*swig_is_def_textloc_get = *LibreDWGc::Dwg_OCD_Dimension_is_def_textloc_get;
*swig_is_def_textloc_set = *LibreDWGc::Dwg_OCD_Dimension_is_def_textloc_set;
*swig_text_rotation_get = *LibreDWGc::Dwg_OCD_Dimension_text_rotation_get;
*swig_text_rotation_set = *LibreDWGc::Dwg_OCD_Dimension_text_rotation_set;
*swig_block_get = *LibreDWGc::Dwg_OCD_Dimension_block_get;
*swig_block_set = *LibreDWGc::Dwg_OCD_Dimension_block_set;
*swig_dimtofl_get = *LibreDWGc::Dwg_OCD_Dimension_dimtofl_get;
*swig_dimtofl_set = *LibreDWGc::Dwg_OCD_Dimension_dimtofl_set;
*swig_dimosxd_get = *LibreDWGc::Dwg_OCD_Dimension_dimosxd_get;
*swig_dimosxd_set = *LibreDWGc::Dwg_OCD_Dimension_dimosxd_set;
*swig_dimatfit_get = *LibreDWGc::Dwg_OCD_Dimension_dimatfit_get;
*swig_dimatfit_set = *LibreDWGc::Dwg_OCD_Dimension_dimatfit_set;
*swig_dimtix_get = *LibreDWGc::Dwg_OCD_Dimension_dimtix_get;
*swig_dimtix_set = *LibreDWGc::Dwg_OCD_Dimension_dimtix_set;
*swig_dimtmove_get = *LibreDWGc::Dwg_OCD_Dimension_dimtmove_get;
*swig_dimtmove_set = *LibreDWGc::Dwg_OCD_Dimension_dimtmove_set;
*swig_override_code_get = *LibreDWGc::Dwg_OCD_Dimension_override_code_get;
*swig_override_code_set = *LibreDWGc::Dwg_OCD_Dimension_override_code_set;
*swig_has_arrow2_get = *LibreDWGc::Dwg_OCD_Dimension_has_arrow2_get;
*swig_has_arrow2_set = *LibreDWGc::Dwg_OCD_Dimension_has_arrow2_set;
*swig_flip_arrow2_get = *LibreDWGc::Dwg_OCD_Dimension_flip_arrow2_get;
*swig_flip_arrow2_set = *LibreDWGc::Dwg_OCD_Dimension_flip_arrow2_set;
*swig_flip_arrow1_get = *LibreDWGc::Dwg_OCD_Dimension_flip_arrow1_get;
*swig_flip_arrow1_set = *LibreDWGc::Dwg_OCD_Dimension_flip_arrow1_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_OCD_Dimension(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_OCD_Dimension($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ALDIMOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_ALDIMOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_scale_set;
*swig_dimension_get = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_get;
*swig_dimension_set = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimension_set;
*swig_dimline_pt_get = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_get;
*swig_dimline_pt_set = *LibreDWGc::Dwg_Object_ALDIMOBJECTCONTEXTDATA_dimline_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ALDIMOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ALDIMOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ANGDIMOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_ANGDIMOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_scale_set;
*swig_dimension_get = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_get;
*swig_dimension_set = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_dimension_set;
*swig_arc_pt_get = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_get;
*swig_arc_pt_set = *LibreDWGc::Dwg_Object_ANGDIMOBJECTCONTEXTDATA_arc_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ANGDIMOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ANGDIMOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DMDIMOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_DMDIMOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_scale_set;
*swig_dimension_get = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_get;
*swig_dimension_set = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_dimension_set;
*swig_first_arc_pt_get = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_get;
*swig_first_arc_pt_set = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_first_arc_pt_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Object_DMDIMOBJECTCONTEXTDATA_def_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DMDIMOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DMDIMOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ORDDIMOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_ORDDIMOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_scale_set;
*swig_dimension_get = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_get;
*swig_dimension_set = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_dimension_set;
*swig_feature_location_pt_get = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_get;
*swig_feature_location_pt_set = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_feature_location_pt_set;
*swig_leader_endpt_get = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_get;
*swig_leader_endpt_set = *LibreDWGc::Dwg_Object_ORDDIMOBJECTCONTEXTDATA_leader_endpt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ORDDIMOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ORDDIMOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RADIMOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_RADIMOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_scale_set;
*swig_dimension_get = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_get;
*swig_dimension_set = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_dimension_set;
*swig_first_arc_pt_get = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_get;
*swig_first_arc_pt_set = *LibreDWGc::Dwg_Object_RADIMOBJECTCONTEXTDATA_first_arc_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RADIMOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RADIMOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_RADIMLGOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_RADIMLGOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_scale_set;
*swig_dimension_get = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_get;
*swig_dimension_set = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_dimension_set;
*swig_ovr_center_get = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_get;
*swig_ovr_center_set = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_ovr_center_set;
*swig_jog_point_get = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_get;
*swig_jog_point_set = *LibreDWGc::Dwg_Object_RADIMLGOBJECTCONTEXTDATA_jog_point_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_RADIMLGOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_RADIMLGOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_scale_set;
*swig_horizontal_mode_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_get;
*swig_horizontal_mode_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_horizontal_mode_set;
*swig_rotation_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_rotation_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_ins_pt_set;
*swig_alignment_pt_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_get;
*swig_alignment_pt_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_alignment_pt_set;
*swig_enable_context_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_get;
*swig_enable_context_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_enable_context_set;
*swig_context_get = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_get;
*swig_context_set = *LibreDWGc::Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA_context_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MTEXTATTRIBUTEOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_MLEADEROBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_MLEADEROBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_MLEADEROBJECTCONTEXTDATA_scale_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_MLEADEROBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_MLEADEROBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LEADEROBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_LEADEROBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_scale_set;
*swig_num_points_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_get;
*swig_num_points_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_num_points_set;
*swig_points_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_points_get;
*swig_points_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_points_set;
*swig_b290_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_get;
*swig_b290_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_b290_set;
*swig_x_direction_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_get;
*swig_x_direction_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_x_direction_set;
*swig_inspt_offset_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_get;
*swig_inspt_offset_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_inspt_offset_set;
*swig_endptproj_get = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_get;
*swig_endptproj_set = *LibreDWGc::Dwg_Object_LEADEROBJECTCONTEXTDATA_endptproj_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LEADEROBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LEADEROBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLKREFOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_BLKREFOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_set;
*swig_rotation_get = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_rotation_set;
*swig_ins_pt_get = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_get;
*swig_ins_pt_set = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_ins_pt_set;
*swig_scale_factor_get = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_get;
*swig_scale_factor_set = *LibreDWGc::Dwg_Object_BLKREFOBJECTCONTEXTDATA_scale_factor_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLKREFOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLKREFOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_FCFOBJECTCONTEXTDATA ##############

package LibreDWG::Dwg_Object_FCFOBJECTCONTEXTDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_class_version_set;
*swig_is_default_get = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_get;
*swig_is_default_set = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_is_default_set;
*swig_scale_get = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_scale_get;
*swig_scale_set = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_scale_set;
*swig_location_get = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_location_get;
*swig_location_set = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_location_set;
*swig_horiz_dir_get = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_get;
*swig_horiz_dir_set = *LibreDWGc::Dwg_Object_FCFOBJECTCONTEXTDATA_horiz_dir_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_FCFOBJECTCONTEXTDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_FCFOBJECTCONTEXTDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DETAILVIEWSTYLE ##############

package LibreDWG::Dwg_Object_DETAILVIEWSTYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_parent_set;
*swig_mdoc_class_version_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_get;
*swig_mdoc_class_version_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_mdoc_class_version_set;
*swig_desc_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_desc_get;
*swig_desc_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_desc_set;
*swig_is_modified_for_recompute_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_get;
*swig_is_modified_for_recompute_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_is_modified_for_recompute_set;
*swig_display_name_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_display_name_get;
*swig_display_name_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_display_name_set;
*swig_viewstyle_flags_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_get;
*swig_viewstyle_flags_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewstyle_flags_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_class_version_set;
*swig_flags_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_flags_set;
*swig_identifier_style_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_style_get;
*swig_identifier_style_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_style_set;
*swig_identifier_color_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_color_get;
*swig_identifier_color_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_color_set;
*swig_identifier_height_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_height_get;
*swig_identifier_height_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_height_set;
*swig_identifier_exclude_characters_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_get;
*swig_identifier_exclude_characters_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_exclude_characters_set;
*swig_identifier_offset_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_offset_get;
*swig_identifier_offset_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_offset_set;
*swig_identifier_placement_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_placement_get;
*swig_identifier_placement_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_identifier_placement_set;
*swig_arrow_symbol_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_get;
*swig_arrow_symbol_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_set;
*swig_arrow_symbol_color_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_get;
*swig_arrow_symbol_color_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_color_set;
*swig_arrow_symbol_size_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_get;
*swig_arrow_symbol_size_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_arrow_symbol_size_set;
*swig_boundary_ltype_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_get;
*swig_boundary_ltype_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_ltype_set;
*swig_boundary_linewt_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_get;
*swig_boundary_linewt_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_linewt_set;
*swig_boundary_line_color_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_get;
*swig_boundary_line_color_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_boundary_line_color_set;
*swig_viewlabel_text_style_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_get;
*swig_viewlabel_text_style_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_style_set;
*swig_viewlabel_text_color_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_get;
*swig_viewlabel_text_color_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_color_set;
*swig_viewlabel_text_height_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_get;
*swig_viewlabel_text_height_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_text_height_set;
*swig_viewlabel_attachment_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_get;
*swig_viewlabel_attachment_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_attachment_set;
*swig_viewlabel_offset_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_get;
*swig_viewlabel_offset_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_offset_set;
*swig_viewlabel_alignment_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_get;
*swig_viewlabel_alignment_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_alignment_set;
*swig_viewlabel_pattern_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_get;
*swig_viewlabel_pattern_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_viewlabel_pattern_set;
*swig_connection_ltype_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_ltype_get;
*swig_connection_ltype_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_ltype_set;
*swig_connection_linewt_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_linewt_get;
*swig_connection_linewt_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_linewt_set;
*swig_connection_line_color_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_line_color_get;
*swig_connection_line_color_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_connection_line_color_set;
*swig_borderline_ltype_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_get;
*swig_borderline_ltype_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_ltype_set;
*swig_borderline_linewt_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_get;
*swig_borderline_linewt_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_linewt_set;
*swig_borderline_color_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_color_get;
*swig_borderline_color_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_borderline_color_set;
*swig_model_edge_get = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_model_edge_get;
*swig_model_edge_set = *LibreDWGc::Dwg_Object_DETAILVIEWSTYLE_model_edge_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DETAILVIEWSTYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DETAILVIEWSTYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SECTIONVIEWSTYLE ##############

package LibreDWG::Dwg_Object_SECTIONVIEWSTYLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_parent_set;
*swig_mdoc_class_version_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_get;
*swig_mdoc_class_version_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_mdoc_class_version_set;
*swig_desc_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_desc_get;
*swig_desc_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_desc_set;
*swig_is_modified_for_recompute_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_get;
*swig_is_modified_for_recompute_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_is_modified_for_recompute_set;
*swig_display_name_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_display_name_get;
*swig_display_name_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_display_name_set;
*swig_viewstyle_flags_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_get;
*swig_viewstyle_flags_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewstyle_flags_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_class_version_set;
*swig_flags_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_flags_set;
*swig_identifier_style_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_style_get;
*swig_identifier_style_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_style_set;
*swig_identifier_color_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_color_get;
*swig_identifier_color_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_color_set;
*swig_identifier_height_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_height_get;
*swig_identifier_height_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_height_set;
*swig_arrow_start_symbol_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_get;
*swig_arrow_start_symbol_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_start_symbol_set;
*swig_arrow_end_symbol_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_get;
*swig_arrow_end_symbol_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_end_symbol_set;
*swig_arrow_symbol_color_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_get;
*swig_arrow_symbol_color_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_color_set;
*swig_arrow_symbol_size_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_get;
*swig_arrow_symbol_size_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_size_set;
*swig_identifier_exclude_characters_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_get;
*swig_identifier_exclude_characters_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_exclude_characters_set;
*swig_identifier_position_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_position_get;
*swig_identifier_position_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_position_set;
*swig_identifier_offset_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_get;
*swig_identifier_offset_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_identifier_offset_set;
*swig_arrow_position_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_position_get;
*swig_arrow_position_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_position_set;
*swig_arrow_symbol_extension_length_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_get;
*swig_arrow_symbol_extension_length_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_arrow_symbol_extension_length_set;
*swig_plane_ltype_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_get;
*swig_plane_ltype_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_ltype_set;
*swig_plane_linewt_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_get;
*swig_plane_linewt_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_linewt_set;
*swig_plane_line_color_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_get;
*swig_plane_line_color_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_plane_line_color_set;
*swig_bend_ltype_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_get;
*swig_bend_ltype_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_ltype_set;
*swig_bend_linewt_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_get;
*swig_bend_linewt_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_linewt_set;
*swig_bend_line_color_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_get;
*swig_bend_line_color_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_color_set;
*swig_bend_line_length_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_get;
*swig_bend_line_length_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_bend_line_length_set;
*swig_end_line_overshoot_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_get;
*swig_end_line_overshoot_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_overshoot_set;
*swig_end_line_length_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_length_get;
*swig_end_line_length_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_end_line_length_set;
*swig_viewlabel_text_style_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_get;
*swig_viewlabel_text_style_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_style_set;
*swig_viewlabel_text_color_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_get;
*swig_viewlabel_text_color_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_color_set;
*swig_viewlabel_text_height_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_get;
*swig_viewlabel_text_height_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_text_height_set;
*swig_viewlabel_attachment_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_get;
*swig_viewlabel_attachment_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_attachment_set;
*swig_viewlabel_offset_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_get;
*swig_viewlabel_offset_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_offset_set;
*swig_viewlabel_alignment_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_get;
*swig_viewlabel_alignment_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_alignment_set;
*swig_viewlabel_pattern_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_get;
*swig_viewlabel_pattern_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_viewlabel_pattern_set;
*swig_hatch_color_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_color_get;
*swig_hatch_color_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_color_set;
*swig_hatch_bg_color_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_get;
*swig_hatch_bg_color_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_bg_color_set;
*swig_hatch_pattern_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_get;
*swig_hatch_pattern_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_pattern_set;
*swig_hatch_scale_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_get;
*swig_hatch_scale_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_scale_set;
*swig_hatch_transparency_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_get;
*swig_hatch_transparency_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_transparency_set;
*swig_unknown_b1_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_get;
*swig_unknown_b1_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b1_set;
*swig_unknown_b2_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_get;
*swig_unknown_b2_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_unknown_b2_set;
*swig_num_hatch_angles_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_get;
*swig_num_hatch_angles_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_num_hatch_angles_set;
*swig_hatch_angles_get = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_get;
*swig_hatch_angles_set = *LibreDWGc::Dwg_Object_SECTIONVIEWSTYLE_hatch_angles_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SECTIONVIEWSTYLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SECTIONVIEWSTYLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SECTION_MANAGER ##############

package LibreDWG::Dwg_Object_SECTION_MANAGER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SECTION_MANAGER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SECTION_MANAGER_parent_set;
*swig_is_live_get = *LibreDWGc::Dwg_Object_SECTION_MANAGER_is_live_get;
*swig_is_live_set = *LibreDWGc::Dwg_Object_SECTION_MANAGER_is_live_set;
*swig_num_sections_get = *LibreDWGc::Dwg_Object_SECTION_MANAGER_num_sections_get;
*swig_num_sections_set = *LibreDWGc::Dwg_Object_SECTION_MANAGER_num_sections_set;
*swig_sections_get = *LibreDWGc::Dwg_Object_SECTION_MANAGER_sections_get;
*swig_sections_set = *LibreDWGc::Dwg_Object_SECTION_MANAGER_sections_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SECTION_MANAGER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SECTION_MANAGER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SECTION_geometrysettings ##############

package LibreDWG::Dwg_SECTION_geometrysettings;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_SECTION_geometrysettings_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_SECTION_geometrysettings_parent_set;
*swig_num_geoms_get = *LibreDWGc::Dwg_SECTION_geometrysettings_num_geoms_get;
*swig_num_geoms_set = *LibreDWGc::Dwg_SECTION_geometrysettings_num_geoms_set;
*swig_hexindex_get = *LibreDWGc::Dwg_SECTION_geometrysettings_hexindex_get;
*swig_hexindex_set = *LibreDWGc::Dwg_SECTION_geometrysettings_hexindex_set;
*swig_flags_get = *LibreDWGc::Dwg_SECTION_geometrysettings_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_SECTION_geometrysettings_flags_set;
*swig_color_get = *LibreDWGc::Dwg_SECTION_geometrysettings_color_get;
*swig_color_set = *LibreDWGc::Dwg_SECTION_geometrysettings_color_set;
*swig_layer_get = *LibreDWGc::Dwg_SECTION_geometrysettings_layer_get;
*swig_layer_set = *LibreDWGc::Dwg_SECTION_geometrysettings_layer_set;
*swig_ltype_get = *LibreDWGc::Dwg_SECTION_geometrysettings_ltype_get;
*swig_ltype_set = *LibreDWGc::Dwg_SECTION_geometrysettings_ltype_set;
*swig_ltype_scale_get = *LibreDWGc::Dwg_SECTION_geometrysettings_ltype_scale_get;
*swig_ltype_scale_set = *LibreDWGc::Dwg_SECTION_geometrysettings_ltype_scale_set;
*swig_plotstyle_get = *LibreDWGc::Dwg_SECTION_geometrysettings_plotstyle_get;
*swig_plotstyle_set = *LibreDWGc::Dwg_SECTION_geometrysettings_plotstyle_set;
*swig_linewt_get = *LibreDWGc::Dwg_SECTION_geometrysettings_linewt_get;
*swig_linewt_set = *LibreDWGc::Dwg_SECTION_geometrysettings_linewt_set;
*swig_face_transparency_get = *LibreDWGc::Dwg_SECTION_geometrysettings_face_transparency_get;
*swig_face_transparency_set = *LibreDWGc::Dwg_SECTION_geometrysettings_face_transparency_set;
*swig_edge_transparency_get = *LibreDWGc::Dwg_SECTION_geometrysettings_edge_transparency_get;
*swig_edge_transparency_set = *LibreDWGc::Dwg_SECTION_geometrysettings_edge_transparency_set;
*swig_hatch_type_get = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_type_get;
*swig_hatch_type_set = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_type_set;
*swig_hatch_pattern_get = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_pattern_get;
*swig_hatch_pattern_set = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_pattern_set;
*swig_hatch_angle_get = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_angle_get;
*swig_hatch_angle_set = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_angle_set;
*swig_hatch_spacing_get = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_spacing_get;
*swig_hatch_spacing_set = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_spacing_set;
*swig_hatch_scale_get = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_scale_get;
*swig_hatch_scale_set = *LibreDWGc::Dwg_SECTION_geometrysettings_hatch_scale_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SECTION_geometrysettings(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SECTION_geometrysettings($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SECTION_typesettings ##############

package LibreDWG::Dwg_SECTION_typesettings;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_SECTION_typesettings_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_SECTION_typesettings_parent_set;
*swig_type_get = *LibreDWGc::Dwg_SECTION_typesettings_type_get;
*swig_type_set = *LibreDWGc::Dwg_SECTION_typesettings_type_set;
*swig_generation_get = *LibreDWGc::Dwg_SECTION_typesettings_generation_get;
*swig_generation_set = *LibreDWGc::Dwg_SECTION_typesettings_generation_set;
*swig_num_sources_get = *LibreDWGc::Dwg_SECTION_typesettings_num_sources_get;
*swig_num_sources_set = *LibreDWGc::Dwg_SECTION_typesettings_num_sources_set;
*swig_sources_get = *LibreDWGc::Dwg_SECTION_typesettings_sources_get;
*swig_sources_set = *LibreDWGc::Dwg_SECTION_typesettings_sources_set;
*swig_destblock_get = *LibreDWGc::Dwg_SECTION_typesettings_destblock_get;
*swig_destblock_set = *LibreDWGc::Dwg_SECTION_typesettings_destblock_set;
*swig_destfile_get = *LibreDWGc::Dwg_SECTION_typesettings_destfile_get;
*swig_destfile_set = *LibreDWGc::Dwg_SECTION_typesettings_destfile_set;
*swig_num_geom_get = *LibreDWGc::Dwg_SECTION_typesettings_num_geom_get;
*swig_num_geom_set = *LibreDWGc::Dwg_SECTION_typesettings_num_geom_set;
*swig_geom_get = *LibreDWGc::Dwg_SECTION_typesettings_geom_get;
*swig_geom_set = *LibreDWGc::Dwg_SECTION_typesettings_geom_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SECTION_typesettings(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SECTION_typesettings($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_SECTION_SETTINGS ##############

package LibreDWG::Dwg_Object_SECTION_SETTINGS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_parent_set;
*swig_curr_type_get = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_curr_type_get;
*swig_curr_type_set = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_curr_type_set;
*swig_num_types_get = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_num_types_get;
*swig_num_types_set = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_num_types_set;
*swig_types_get = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_types_get;
*swig_types_set = *LibreDWGc::Dwg_Object_SECTION_SETTINGS_types_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_SECTION_SETTINGS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_SECTION_SETTINGS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_LAYERFILTER ##############

package LibreDWG::Dwg_Object_LAYERFILTER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_LAYERFILTER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_LAYERFILTER_parent_set;
*swig_num_names_get = *LibreDWGc::Dwg_Object_LAYERFILTER_num_names_get;
*swig_num_names_set = *LibreDWGc::Dwg_Object_LAYERFILTER_num_names_set;
*swig_names_get = *LibreDWGc::Dwg_Object_LAYERFILTER_names_get;
*swig_names_set = *LibreDWGc::Dwg_Object_LAYERFILTER_names_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_LAYERFILTER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_LAYERFILTER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ARCALIGNEDTEXT ##############

package LibreDWG::Dwg_Entity_ARCALIGNEDTEXT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_parent_set;
*swig_text_size_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_size_get;
*swig_text_size_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_size_set;
*swig_xscale_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_xscale_get;
*swig_xscale_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_xscale_set;
*swig_char_spacing_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_char_spacing_get;
*swig_char_spacing_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_char_spacing_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_style_set;
*swig_text_value_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_value_get;
*swig_text_value_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_value_set;
*swig_font_name_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_font_name_get;
*swig_font_name_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_font_name_set;
*swig_bigfont_name_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_get;
*swig_bigfont_name_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_bigfont_name_set;
*swig_offset_from_arc_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_get;
*swig_offset_from_arc_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_offset_from_arc_set;
*swig_right_offset_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_right_offset_get;
*swig_right_offset_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_right_offset_set;
*swig_left_offset_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_left_offset_get;
*swig_left_offset_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_left_offset_set;
*swig_center_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_center_get;
*swig_center_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_center_set;
*swig_radius_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_radius_get;
*swig_radius_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_radius_set;
*swig_start_angle_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_start_angle_get;
*swig_start_angle_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_start_angle_set;
*swig_end_angle_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_end_angle_get;
*swig_end_angle_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_end_angle_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_extrusion_set;
*swig_color_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_color_get;
*swig_color_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_color_set;
*swig_is_reverse_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_reverse_get;
*swig_is_reverse_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_reverse_set;
*swig_text_direction_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_direction_get;
*swig_text_direction_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_direction_set;
*swig_alignment_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_alignment_get;
*swig_alignment_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_alignment_set;
*swig_text_position_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_position_get;
*swig_text_position_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_text_position_set;
*swig_is_bold_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_bold_get;
*swig_is_bold_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_bold_set;
*swig_is_italic_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_italic_get;
*swig_is_italic_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_italic_set;
*swig_is_underlined_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_underlined_get;
*swig_is_underlined_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_underlined_set;
*swig_character_set_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_character_set_get;
*swig_character_set_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_character_set_set;
*swig_pitch_and_family_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_get;
*swig_pitch_and_family_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_pitch_and_family_set;
*swig_is_shx_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_shx_get;
*swig_is_shx_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_is_shx_set;
*swig_wizard_flag_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_get;
*swig_wizard_flag_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_wizard_flag_set;
*swig_arc_handle_get = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_arc_handle_get;
*swig_arc_handle_set = *LibreDWGc::Dwg_Entity_ARCALIGNEDTEXT_arc_handle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ARCALIGNEDTEXT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ARCALIGNEDTEXT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_RTEXT ##############

package LibreDWG::Dwg_Entity_RTEXT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_RTEXT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_RTEXT_parent_set;
*swig_pt_get = *LibreDWGc::Dwg_Entity_RTEXT_pt_get;
*swig_pt_set = *LibreDWGc::Dwg_Entity_RTEXT_pt_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity_RTEXT_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity_RTEXT_extrusion_set;
*swig_rotation_get = *LibreDWGc::Dwg_Entity_RTEXT_rotation_get;
*swig_rotation_set = *LibreDWGc::Dwg_Entity_RTEXT_rotation_set;
*swig_height_get = *LibreDWGc::Dwg_Entity_RTEXT_height_get;
*swig_height_set = *LibreDWGc::Dwg_Entity_RTEXT_height_set;
*swig_flags_get = *LibreDWGc::Dwg_Entity_RTEXT_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_Entity_RTEXT_flags_set;
*swig_text_value_get = *LibreDWGc::Dwg_Entity_RTEXT_text_value_get;
*swig_text_value_set = *LibreDWGc::Dwg_Entity_RTEXT_text_value_set;
*swig_style_get = *LibreDWGc::Dwg_Entity_RTEXT_style_get;
*swig_style_set = *LibreDWGc::Dwg_Entity_RTEXT_style_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_RTEXT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_RTEXT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LAYOUTPRINTCONFIG ##############

package LibreDWG::Dwg_Entity_LAYOUTPRINTCONFIG;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_class_version_set;
*swig_flag_get = *LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Entity_LAYOUTPRINTCONFIG_flag_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LAYOUTPRINTCONFIG(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LAYOUTPRINTCONFIG($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACMECOMMANDHISTORY ##############

package LibreDWG::Dwg_Object_ACMECOMMANDHISTORY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ACMECOMMANDHISTORY_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACMECOMMANDHISTORY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACMECOMMANDHISTORY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACMESCOPE ##############

package LibreDWG::Dwg_Object_ACMESCOPE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACMESCOPE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACMESCOPE_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ACMESCOPE_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ACMESCOPE_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACMESCOPE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACMESCOPE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_ACMESTATEMGR ##############

package LibreDWG::Dwg_Object_ACMESTATEMGR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_ACMESTATEMGR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_ACMESTATEMGR_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_ACMESTATEMGR_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_ACMESTATEMGR_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_ACMESTATEMGR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_ACMESTATEMGR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_CSACDOCUMENTOPTIONS ##############

package LibreDWG::Dwg_Object_CSACDOCUMENTOPTIONS;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_CSACDOCUMENTOPTIONS_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_CSACDOCUMENTOPTIONS(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_CSACDOCUMENTOPTIONS($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKPARAMETER_connection ##############

package LibreDWG::Dwg_BLOCKPARAMETER_connection;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_code_get = *LibreDWGc::Dwg_BLOCKPARAMETER_connection_code_get;
*swig_code_set = *LibreDWGc::Dwg_BLOCKPARAMETER_connection_code_set;
*swig_name_get = *LibreDWGc::Dwg_BLOCKPARAMETER_connection_name_get;
*swig_name_set = *LibreDWGc::Dwg_BLOCKPARAMETER_connection_name_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKPARAMETER_connection(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKPARAMETER_connection($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKPARAMETER_PropInfo ##############

package LibreDWG::Dwg_BLOCKPARAMETER_PropInfo;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_connections_get = *LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_num_connections_get;
*swig_num_connections_set = *LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_num_connections_set;
*swig_connections_get = *LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_connections_get;
*swig_connections_set = *LibreDWGc::Dwg_BLOCKPARAMETER_PropInfo_connections_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKPARAMETER_PropInfo(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKPARAMETER_PropInfo($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKPARAMVALUESET ##############

package LibreDWG::Dwg_BLOCKPARAMVALUESET;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_desc_get = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_desc_get;
*swig_desc_set = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_desc_set;
*swig_flags_get = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_flags_set;
*swig_minimum_get = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_minimum_get;
*swig_minimum_set = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_minimum_set;
*swig_maximum_get = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_maximum_get;
*swig_maximum_set = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_maximum_set;
*swig_increment_get = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_increment_get;
*swig_increment_set = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_increment_set;
*swig_num_valuelist_get = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_num_valuelist_get;
*swig_num_valuelist_set = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_num_valuelist_set;
*swig_valuelist_get = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_valuelist_get;
*swig_valuelist_set = *LibreDWGc::Dwg_BLOCKPARAMVALUESET_valuelist_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKPARAMVALUESET(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKPARAMVALUESET($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKACTION_connectionpts ##############

package LibreDWG::Dwg_BLOCKACTION_connectionpts;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_code_get = *LibreDWGc::Dwg_BLOCKACTION_connectionpts_code_get;
*swig_code_set = *LibreDWGc::Dwg_BLOCKACTION_connectionpts_code_set;
*swig_name_get = *LibreDWGc::Dwg_BLOCKACTION_connectionpts_name_get;
*swig_name_set = *LibreDWGc::Dwg_BLOCKACTION_connectionpts_name_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKACTION_connectionpts(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKACTION_connectionpts($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKVISIBILITYPARAMETER_state ##############

package LibreDWG::Dwg_BLOCKVISIBILITYPARAMETER_state;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_parent_set;
*swig_name_get = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_name_get;
*swig_name_set = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_name_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_blocks_set;
*swig_blocks_get = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_get;
*swig_blocks_set = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_blocks_set;
*swig_num_params_get = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_get;
*swig_num_params_set = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_num_params_set;
*swig_params_get = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_params_get;
*swig_params_set = *LibreDWGc::Dwg_BLOCKVISIBILITYPARAMETER_state_params_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKVISIBILITYPARAMETER_state(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKVISIBILITYPARAMETER_state($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKVISIBILITYPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKVISIBILITYPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_chain_actions_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_def_pt_set;
*swig_num_propinfos_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_get;
*swig_num_propinfos_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_propinfos_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_prop2_set;
*swig_is_initialized_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_get;
*swig_is_initialized_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_is_initialized_set;
*swig_unknown_bool_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_get;
*swig_unknown_bool_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_unknown_bool_set;
*swig_blockvisi_name_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_get;
*swig_blockvisi_name_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_name_set;
*swig_blockvisi_desc_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_get;
*swig_blockvisi_desc_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blockvisi_desc_set;
*swig_num_blocks_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_get;
*swig_num_blocks_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_blocks_set;
*swig_blocks_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_get;
*swig_blocks_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_blocks_set;
*swig_num_states_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_get;
*swig_num_states_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_num_states_set;
*swig_states_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_states_get;
*swig_states_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYPARAMETER_states_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKVISIBILITYPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKVISIBILITYPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKVISIBILITYGRIP ##############

package LibreDWG::Dwg_Object_BLOCKVISIBILITYGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKVISIBILITYGRIP_bg_insert_cycling_weight_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKVISIBILITYGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKVISIBILITYGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT ##############

package LibreDWG::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_evalexpr_set;
*swig_grip_type_get = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_get;
*swig_grip_type_set = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_type_set;
*swig_grip_expr_get = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_get;
*swig_grip_expr_set = *LibreDWGc::Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT_grip_expr_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKGRIPLOCATIONCOMPONENT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BREAKDATA ##############

package LibreDWG::Dwg_Object_BREAKDATA;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BREAKDATA_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BREAKDATA_parent_set;
*swig_num_pointrefs_get = *LibreDWGc::Dwg_Object_BREAKDATA_num_pointrefs_get;
*swig_num_pointrefs_set = *LibreDWGc::Dwg_Object_BREAKDATA_num_pointrefs_set;
*swig_pointrefs_get = *LibreDWGc::Dwg_Object_BREAKDATA_pointrefs_get;
*swig_pointrefs_set = *LibreDWGc::Dwg_Object_BREAKDATA_pointrefs_set;
*swig_dimref_get = *LibreDWGc::Dwg_Object_BREAKDATA_dimref_get;
*swig_dimref_set = *LibreDWGc::Dwg_Object_BREAKDATA_dimref_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BREAKDATA(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BREAKDATA($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BREAKPOINTREF ##############

package LibreDWG::Dwg_Object_BREAKPOINTREF;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BREAKPOINTREF_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BREAKPOINTREF_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BREAKPOINTREF(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BREAKPOINTREF($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_FLIPGRIPENTITY ##############

package LibreDWG::Dwg_Entity_FLIPGRIPENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_FLIPGRIPENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_FLIPGRIPENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_FLIPGRIPENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_FLIPGRIPENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LINEARGRIPENTITY ##############

package LibreDWG::Dwg_Entity_LINEARGRIPENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LINEARGRIPENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LINEARGRIPENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LINEARGRIPENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LINEARGRIPENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POLARGRIPENTITY ##############

package LibreDWG::Dwg_Entity_POLARGRIPENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POLARGRIPENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POLARGRIPENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POLARGRIPENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POLARGRIPENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ROTATIONGRIPENTITY ##############

package LibreDWG::Dwg_Entity_ROTATIONGRIPENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ROTATIONGRIPENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ROTATIONGRIPENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ROTATIONGRIPENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ROTATIONGRIPENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_VISIBILITYGRIPENTITY ##############

package LibreDWG::Dwg_Entity_VISIBILITYGRIPENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_VISIBILITYGRIPENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_VISIBILITYGRIPENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_VISIBILITYGRIPENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_VISIBILITYGRIPENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_XYGRIPENTITY ##############

package LibreDWG::Dwg_Entity_XYGRIPENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_XYGRIPENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_XYGRIPENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_XYGRIPENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_XYGRIPENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ALIGNMENTPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_ALIGNMENTPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ALIGNMENTPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ALIGNMENTPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ALIGNMENTPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_BASEPOINTPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_BASEPOINTPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_BASEPOINTPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_BASEPOINTPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_BASEPOINTPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_FLIPPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_FLIPPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_FLIPPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_FLIPPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_FLIPPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_FLIPPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LINEARPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_LINEARPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LINEARPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LINEARPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LINEARPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LINEARPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POINTPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_POINTPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POINTPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POINTPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POINTPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POINTPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ROTATIONPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_ROTATIONPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ROTATIONPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ROTATIONPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ROTATIONPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ROTATIONPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_VISIBILITYPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_VISIBILITYPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_VISIBILITYPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_VISIBILITYPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_VISIBILITYPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_XYPARAMETERENTITY ##############

package LibreDWG::Dwg_Entity_XYPARAMETERENTITY;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_XYPARAMETERENTITY_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_XYPARAMETERENTITY_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_XYPARAMETERENTITY(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_XYPARAMETERENTITY($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKALIGNMENTGRIP ##############

package LibreDWG::Dwg_Object_BLOCKALIGNMENTGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_bg_insert_cycling_weight_set;
*swig_orientation_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_orientation_get;
*swig_orientation_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTGRIP_orientation_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKALIGNMENTGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKALIGNMENTGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKALIGNMENTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKALIGNMENTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_endpt_set;
*swig_align_perpendicular_get = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_get;
*swig_align_perpendicular_set = *LibreDWGc::Dwg_Object_BLOCKALIGNMENTPARAMETER_align_perpendicular_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKALIGNMENTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKALIGNMENTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_endpt_set;
*swig_dependency_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_get;
*swig_dependency_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_dependency_set;
*swig_center_pt_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_get;
*swig_center_pt_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_center_pt_set;
*swig_end_pt_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_get;
*swig_end_pt_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_end_pt_set;
*swig_expr_name_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_get;
*swig_expr_name_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_name_set;
*swig_expr_description_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_get;
*swig_expr_description_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_expr_description_set;
*swig_angle_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_get;
*swig_angle_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_angle_set;
*swig_orientation_on_both_grips_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_get;
*swig_orientation_on_both_grips_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_orientation_on_both_grips_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKANGULARCONSTRAINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_endpt_set;
*swig_dependency_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_get;
*swig_dependency_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_dependency_set;
*swig_expr_name_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_get;
*swig_expr_name_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_name_set;
*swig_expr_description_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_get;
*swig_expr_description_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_expr_description_set;
*swig_distance_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_get;
*swig_distance_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_distance_set;
*swig_orientation_on_both_grips_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_get;
*swig_orientation_on_both_grips_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_orientation_on_both_grips_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKDIAMETRICCONSTRAINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_endpt_set;
*swig_dependency_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_get;
*swig_dependency_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_dependency_set;
*swig_expr_name_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_get;
*swig_expr_name_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_name_set;
*swig_expr_description_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_get;
*swig_expr_description_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_expr_description_set;
*swig_distance_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_get;
*swig_distance_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_distance_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKRADIALCONSTRAINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKARRAYACTION ##############

package LibreDWG::Dwg_Object_BLOCKARRAYACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_deps_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_conn_pts_set;
*swig_column_offset_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_column_offset_get;
*swig_column_offset_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_column_offset_set;
*swig_row_offset_get = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_row_offset_get;
*swig_row_offset_set = *LibreDWGc::Dwg_Object_BLOCKARRAYACTION_row_offset_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKARRAYACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKARRAYACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKBASEPOINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKBASEPOINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_chain_actions_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_def_pt_set;
*swig_num_propinfos_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_get;
*swig_num_propinfos_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_num_propinfos_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_prop2_set;
*swig_pt_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_get;
*swig_pt_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_pt_set;
*swig_base_pt_get = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_get;
*swig_base_pt_set = *LibreDWGc::Dwg_Object_BLOCKBASEPOINTPARAMETER_base_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKBASEPOINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKBASEPOINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKFLIPACTION ##############

package LibreDWG::Dwg_Object_BLOCKFLIPACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_deps_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_conn_pts_set;
*swig_action_offset_x_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_x_get;
*swig_action_offset_x_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_x_set;
*swig_action_offset_y_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_y_get;
*swig_action_offset_y_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_action_offset_y_set;
*swig_angle_offset_get = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_angle_offset_get;
*swig_angle_offset_set = *LibreDWGc::Dwg_Object_BLOCKFLIPACTION_angle_offset_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKFLIPACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKFLIPACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKFLIPGRIP ##############

package LibreDWG::Dwg_Object_BLOCKFLIPGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_bg_insert_cycling_weight_set;
*swig_combined_state_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_combined_state_get;
*swig_combined_state_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_combined_state_set;
*swig_orientation_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_orientation_get;
*swig_orientation_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_orientation_set;
*swig_upd_state_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_upd_state_get;
*swig_upd_state_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_upd_state_set;
*swig_state_get = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_state_get;
*swig_state_set = *LibreDWGc::Dwg_Object_BLOCKFLIPGRIP_state_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKFLIPGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKFLIPGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKFLIPPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKFLIPPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_endpt_set;
*swig_flip_label_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_get;
*swig_flip_label_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_set;
*swig_flip_label_desc_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_get;
*swig_flip_label_desc_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flip_label_desc_set;
*swig_base_state_label_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_get;
*swig_base_state_label_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_base_state_label_set;
*swig_flipped_state_label_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_get;
*swig_flipped_state_label_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_flipped_state_label_set;
*swig_def_label_pt_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_get;
*swig_def_label_pt_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_def_label_pt_set;
*swig_bl96_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_bl96_get;
*swig_bl96_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_bl96_set;
*swig_tooltip_get = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_tooltip_get;
*swig_tooltip_set = *LibreDWGc::Dwg_Object_BLOCKFLIPPARAMETER_tooltip_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKFLIPPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKFLIPPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_endpt_set;
*swig_dependency_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_get;
*swig_dependency_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_dependency_set;
*swig_expr_name_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_get;
*swig_expr_name_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_name_set;
*swig_expr_description_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_get;
*swig_expr_description_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_expr_description_set;
*swig_value_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_get;
*swig_value_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKALIGNEDCONSTRAINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_endpt_set;
*swig_dependency_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_get;
*swig_dependency_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_dependency_set;
*swig_expr_name_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_get;
*swig_expr_name_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_name_set;
*swig_expr_description_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_get;
*swig_expr_description_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_expr_description_set;
*swig_value_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_get;
*swig_value_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKLINEARCONSTRAINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_endpt_set;
*swig_dependency_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_get;
*swig_dependency_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_dependency_set;
*swig_expr_name_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_get;
*swig_expr_name_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_name_set;
*swig_expr_description_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_get;
*swig_expr_description_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_expr_description_set;
*swig_value_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_get;
*swig_value_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKHORIZONTALCONSTRAINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_endpt_set;
*swig_dependency_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_get;
*swig_dependency_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_dependency_set;
*swig_expr_name_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_get;
*swig_expr_name_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_name_set;
*swig_expr_description_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_get;
*swig_expr_description_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_expr_description_set;
*swig_value_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_get;
*swig_value_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKVERTICALCONSTRAINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKLINEARGRIP ##############

package LibreDWG::Dwg_Object_BLOCKLINEARGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_bg_insert_cycling_weight_set;
*swig_orientation_get = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_orientation_get;
*swig_orientation_set = *LibreDWGc::Dwg_Object_BLOCKLINEARGRIP_orientation_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKLINEARGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKLINEARGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKLINEARPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKLINEARPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_endpt_set;
*swig_distance_name_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_name_get;
*swig_distance_name_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_name_set;
*swig_distance_desc_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_get;
*swig_distance_desc_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_desc_set;
*swig_distance_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_get;
*swig_distance_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_distance_set;
*swig_value_set_get = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_value_set_get;
*swig_value_set_set = *LibreDWGc::Dwg_Object_BLOCKLINEARPARAMETER_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKLINEARPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKLINEARPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKLOOKUPACTION_lut ##############

package LibreDWG::Dwg_BLOCKLOOKUPACTION_lut;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_parent_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_conn_pts_set;
*swig_b282_get = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b282_get;
*swig_b282_set = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b282_set;
*swig_b281_get = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b281_get;
*swig_b281_set = *LibreDWGc::Dwg_BLOCKLOOKUPACTION_lut_b281_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKLOOKUPACTION_lut(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKLOOKUPACTION_lut($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKLOOKUPACTION ##############

package LibreDWG::Dwg_Object_BLOCKLOOKUPACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_deps_set;
*swig_numelems_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numelems_get;
*swig_numelems_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numelems_set;
*swig_numrows_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numrows_get;
*swig_numrows_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numrows_set;
*swig_numcols_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numcols_get;
*swig_numcols_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_numcols_set;
*swig_lut_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_lut_get;
*swig_lut_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_lut_set;
*swig_exprs_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_exprs_get;
*swig_exprs_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_exprs_set;
*swig_b280_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_b280_get;
*swig_b280_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPACTION_b280_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKLOOKUPACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKLOOKUPACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKLOOKUPGRIP ##############

package LibreDWG::Dwg_Object_BLOCKLOOKUPGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPGRIP_bg_insert_cycling_weight_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKLOOKUPGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKLOOKUPGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKLOOKUPPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKLOOKUPPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_chain_actions_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_def_pt_set;
*swig_num_propinfos_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_get;
*swig_num_propinfos_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_num_propinfos_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_prop2_set;
*swig_lookup_name_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_get;
*swig_lookup_name_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_name_set;
*swig_lookup_desc_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_get;
*swig_lookup_desc_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_lookup_desc_set;
*swig_index_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_index_get;
*swig_index_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_index_set;
*swig_unknown_t_get = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_get;
*swig_unknown_t_set = *LibreDWGc::Dwg_Object_BLOCKLOOKUPPARAMETER_unknown_t_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKLOOKUPPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKLOOKUPPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKMOVEACTION ##############

package LibreDWG::Dwg_Object_BLOCKMOVEACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_deps_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_conn_pts_set;
*swig_action_offset_x_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_x_get;
*swig_action_offset_x_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_x_set;
*swig_action_offset_y_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_y_get;
*swig_action_offset_y_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_action_offset_y_set;
*swig_angle_offset_get = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_angle_offset_get;
*swig_angle_offset_set = *LibreDWGc::Dwg_Object_BLOCKMOVEACTION_angle_offset_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKMOVEACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKMOVEACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKPOINTPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKPOINTPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_chain_actions_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_pt_set;
*swig_num_propinfos_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_get;
*swig_num_propinfos_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_num_propinfos_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_prop2_set;
*swig_position_name_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_name_get;
*swig_position_name_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_name_set;
*swig_position_desc_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_desc_get;
*swig_position_desc_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_position_desc_set;
*swig_def_label_pt_get = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_get;
*swig_def_label_pt_set = *LibreDWGc::Dwg_Object_BLOCKPOINTPARAMETER_def_label_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKPOINTPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKPOINTPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKPOLARGRIP ##############

package LibreDWG::Dwg_Object_BLOCKPOLARGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKPOLARGRIP_bg_insert_cycling_weight_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKPOLARGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKPOLARGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKPOLARPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKPOLARPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_endpt_set;
*swig_angle_name_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_name_get;
*swig_angle_name_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_name_set;
*swig_angle_desc_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_get;
*swig_angle_desc_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_desc_set;
*swig_distance_name_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_name_get;
*swig_distance_name_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_name_set;
*swig_distance_desc_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_get;
*swig_distance_desc_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_desc_set;
*swig_offset_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_offset_set;
*swig_angle_value_set_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_get;
*swig_angle_value_set_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_angle_value_set_set;
*swig_distance_value_set_get = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_get;
*swig_distance_value_set_set = *LibreDWGc::Dwg_Object_BLOCKPOLARPARAMETER_distance_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKPOLARPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKPOLARPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKPOLARSTRETCHACTION ##############

package LibreDWG::Dwg_Object_BLOCKPOLARSTRETCHACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_deps_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_conn_pts_set;
*swig_num_pts_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_get;
*swig_num_pts_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_pts_set;
*swig_pts_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_get;
*swig_pts_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_pts_set;
*swig_num_hdls_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_get;
*swig_num_hdls_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_hdls_set;
*swig_hdls_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_get;
*swig_hdls_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_hdls_set;
*swig_shorts_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_get;
*swig_shorts_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_shorts_set;
*swig_num_codes_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_get;
*swig_num_codes_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_num_codes_set;
*swig_codes_get = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_get;
*swig_codes_set = *LibreDWGc::Dwg_Object_BLOCKPOLARSTRETCHACTION_codes_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKPOLARSTRETCHACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKPOLARSTRETCHACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLE ##############

package LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLE_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKPROPERTIESTABLE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKPROPERTIESTABLE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLEGRIP ##############

package LibreDWG::Dwg_Object_BLOCKPROPERTIESTABLEGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKPROPERTIESTABLEGRIP_bg_insert_cycling_weight_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKPROPERTIESTABLEGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKPROPERTIESTABLEGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKREPRESENTATION ##############

package LibreDWG::Dwg_Object_BLOCKREPRESENTATION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_parent_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_flag_set;
*swig_block_get = *LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_block_get;
*swig_block_set = *LibreDWGc::Dwg_Object_BLOCKREPRESENTATION_block_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKREPRESENTATION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKREPRESENTATION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKROTATEACTION ##############

package LibreDWG::Dwg_Object_BLOCKROTATEACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_deps_set;
*swig_offset_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_offset_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_conn_pts_set;
*swig_dependent_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_dependent_get;
*swig_dependent_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_dependent_set;
*swig_base_pt_get = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_base_pt_get;
*swig_base_pt_set = *LibreDWGc::Dwg_Object_BLOCKROTATEACTION_base_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKROTATEACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKROTATEACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKROTATIONGRIP ##############

package LibreDWG::Dwg_Object_BLOCKROTATIONGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONGRIP_bg_insert_cycling_weight_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKROTATIONGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKROTATIONGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKROTATIONPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKROTATIONPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_endpt_set;
*swig_def_base_angle_pt_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_get;
*swig_def_base_angle_pt_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_def_base_angle_pt_set;
*swig_angle_name_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_get;
*swig_angle_name_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_name_set;
*swig_angle_desc_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_get;
*swig_angle_desc_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_desc_set;
*swig_angle_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_get;
*swig_angle_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_set;
*swig_angle_value_set_get = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_get;
*swig_angle_value_set_set = *LibreDWGc::Dwg_Object_BLOCKROTATIONPARAMETER_angle_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKROTATIONPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKROTATIONPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKSCALEACTION ##############

package LibreDWG::Dwg_Object_BLOCKSCALEACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_deps_set;
*swig_offset_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_offset_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_conn_pts_set;
*swig_dependent_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_dependent_get;
*swig_dependent_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_dependent_set;
*swig_base_pt_get = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_base_pt_get;
*swig_base_pt_set = *LibreDWGc::Dwg_Object_BLOCKSCALEACTION_base_pt_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKSCALEACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKSCALEACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKSTRETCHACTION_handles ##############

package LibreDWG::Dwg_BLOCKSTRETCHACTION_handles;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_parent_set;
*swig_hdl_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_hdl_get;
*swig_hdl_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_hdl_set;
*swig_num_indexes_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_num_indexes_get;
*swig_num_indexes_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_num_indexes_set;
*swig_indexes_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_indexes_get;
*swig_indexes_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_handles_indexes_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKSTRETCHACTION_handles(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKSTRETCHACTION_handles($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_BLOCKSTRETCHACTION_codes ##############

package LibreDWG::Dwg_BLOCKSTRETCHACTION_codes;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_parent_set;
*swig_bl95_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_bl95_get;
*swig_bl95_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_bl95_set;
*swig_num_indexes_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_num_indexes_get;
*swig_num_indexes_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_num_indexes_set;
*swig_indexes_get = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_indexes_get;
*swig_indexes_set = *LibreDWGc::Dwg_BLOCKSTRETCHACTION_codes_indexes_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_BLOCKSTRETCHACTION_codes(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_BLOCKSTRETCHACTION_codes($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKSTRETCHACTION ##############

package LibreDWG::Dwg_Object_BLOCKSTRETCHACTION;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_eed1071_set;
*swig_display_location_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_display_location_get;
*swig_display_location_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_display_location_set;
*swig_num_actions_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_actions_get;
*swig_num_actions_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_actions_set;
*swig_actions_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_actions_get;
*swig_actions_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_actions_set;
*swig_num_deps_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_deps_get;
*swig_num_deps_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_deps_set;
*swig_deps_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_deps_get;
*swig_deps_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_deps_set;
*swig_conn_pts_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_conn_pts_get;
*swig_conn_pts_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_conn_pts_set;
*swig_num_pts_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_pts_get;
*swig_num_pts_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_pts_set;
*swig_pts_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_pts_get;
*swig_pts_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_pts_set;
*swig_num_hdls_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_hdls_get;
*swig_num_hdls_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_hdls_set;
*swig_hdls_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_hdls_get;
*swig_hdls_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_hdls_set;
*swig_num_codes_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_codes_get;
*swig_num_codes_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_num_codes_set;
*swig_codes_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_codes_get;
*swig_codes_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_codes_set;
*swig_action_offset_x_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_get;
*swig_action_offset_x_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_x_set;
*swig_action_offset_y_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_get;
*swig_action_offset_y_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_action_offset_y_set;
*swig_angle_offset_get = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_angle_offset_get;
*swig_angle_offset_set = *LibreDWGc::Dwg_Object_BLOCKSTRETCHACTION_angle_offset_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKSTRETCHACTION(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKSTRETCHACTION($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKUSERPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKUSERPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_chain_actions_set;
*swig_def_pt_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_def_pt_get;
*swig_def_pt_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_def_pt_set;
*swig_num_propinfos_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_get;
*swig_num_propinfos_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_num_propinfos_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_prop2_set;
*swig_flag_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_flag_get;
*swig_flag_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_flag_set;
*swig_assocvariable_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_assocvariable_get;
*swig_assocvariable_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_assocvariable_set;
*swig_expr_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_expr_get;
*swig_expr_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_expr_set;
*swig_value_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_value_get;
*swig_value_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_value_set;
*swig_type_get = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_type_get;
*swig_type_set = *LibreDWGc::Dwg_Object_BLOCKUSERPARAMETER_type_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKUSERPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKUSERPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKXYGRIP ##############

package LibreDWG::Dwg_Object_BLOCKXYGRIP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_eed1071_set;
*swig_bg_bl91_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl91_get;
*swig_bg_bl91_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl91_set;
*swig_bg_bl92_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl92_get;
*swig_bg_bl92_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_bl92_set;
*swig_bg_location_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_location_get;
*swig_bg_location_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_location_set;
*swig_bg_insert_cycling_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_get;
*swig_bg_insert_cycling_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_set;
*swig_bg_insert_cycling_weight_get = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_get;
*swig_bg_insert_cycling_weight_set = *LibreDWGc::Dwg_Object_BLOCKXYGRIP_bg_insert_cycling_weight_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKXYGRIP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKXYGRIP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_BLOCKXYPARAMETER ##############

package LibreDWG::Dwg_Object_BLOCKXYPARAMETER;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_evalexpr_set;
*swig_name_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_name_set;
*swig_be_major_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_major_get;
*swig_be_major_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_major_set;
*swig_be_minor_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_minor_get;
*swig_be_minor_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_be_minor_set;
*swig_eed1071_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_eed1071_get;
*swig_eed1071_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_eed1071_set;
*swig_show_properties_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_show_properties_get;
*swig_show_properties_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_show_properties_set;
*swig_chain_actions_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_chain_actions_get;
*swig_chain_actions_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_chain_actions_set;
*swig_def_basept_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_basept_get;
*swig_def_basept_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_basept_set;
*swig_def_endpt_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_endpt_get;
*swig_def_endpt_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_def_endpt_set;
*swig_prop1_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop1_get;
*swig_prop1_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop1_set;
*swig_prop2_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop2_get;
*swig_prop2_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop2_set;
*swig_prop3_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop3_get;
*swig_prop3_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop3_set;
*swig_prop4_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop4_get;
*swig_prop4_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop4_set;
*swig_prop_states_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop_states_get;
*swig_prop_states_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_prop_states_set;
*swig_parameter_base_location_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_get;
*swig_parameter_base_location_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_parameter_base_location_set;
*swig_upd_basept_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_basept_get;
*swig_upd_basept_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_basept_set;
*swig_basept_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_basept_get;
*swig_basept_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_basept_set;
*swig_upd_endpt_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_endpt_get;
*swig_upd_endpt_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_upd_endpt_set;
*swig_endpt_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_endpt_get;
*swig_endpt_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_endpt_set;
*swig_x_label_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_get;
*swig_x_label_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_set;
*swig_x_label_desc_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_desc_get;
*swig_x_label_desc_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_label_desc_set;
*swig_y_label_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_get;
*swig_y_label_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_set;
*swig_y_label_desc_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_desc_get;
*swig_y_label_desc_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_label_desc_set;
*swig_x_value_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_get;
*swig_x_value_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_set;
*swig_y_value_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_get;
*swig_y_value_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_set;
*swig_x_value_set_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_set_get;
*swig_x_value_set_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_x_value_set_set;
*swig_y_value_set_get = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_set_get;
*swig_y_value_set_set = *LibreDWGc::Dwg_Object_BLOCKXYPARAMETER_y_value_set_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_BLOCKXYPARAMETER(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_BLOCKXYPARAMETER($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_DYNAMICBLOCKPROXYNODE ##############

package LibreDWG::Dwg_Object_DYNAMICBLOCKPROXYNODE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_parent_set;
*swig_evalexpr_get = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_get;
*swig_evalexpr_set = *LibreDWGc::Dwg_Object_DYNAMICBLOCKPROXYNODE_evalexpr_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_DYNAMICBLOCKPROXYNODE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_DYNAMICBLOCKPROXYNODE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_POINTCLOUD_IntensityStyle ##############

package LibreDWG::Dwg_POINTCLOUD_IntensityStyle;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_parent_set;
*swig_min_intensity_get = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_min_intensity_get;
*swig_min_intensity_set = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_min_intensity_set;
*swig_max_intensity_get = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_max_intensity_get;
*swig_max_intensity_set = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_max_intensity_set;
*swig_intensity_low_treshold_get = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_get;
*swig_intensity_low_treshold_set = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_low_treshold_set;
*swig_intensity_high_treshold_get = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_get;
*swig_intensity_high_treshold_set = *LibreDWGc::Dwg_POINTCLOUD_IntensityStyle_intensity_high_treshold_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_POINTCLOUD_IntensityStyle(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_POINTCLOUD_IntensityStyle($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_POINTCLOUD_Clippings ##############

package LibreDWG::Dwg_POINTCLOUD_Clippings;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_POINTCLOUD_Clippings_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_POINTCLOUD_Clippings_parent_set;
*swig_is_inverted_get = *LibreDWGc::Dwg_POINTCLOUD_Clippings_is_inverted_get;
*swig_is_inverted_set = *LibreDWGc::Dwg_POINTCLOUD_Clippings_is_inverted_set;
*swig_type_get = *LibreDWGc::Dwg_POINTCLOUD_Clippings_type_get;
*swig_type_set = *LibreDWGc::Dwg_POINTCLOUD_Clippings_type_set;
*swig_num_vertices_get = *LibreDWGc::Dwg_POINTCLOUD_Clippings_num_vertices_get;
*swig_num_vertices_set = *LibreDWGc::Dwg_POINTCLOUD_Clippings_num_vertices_set;
*swig_vertices_get = *LibreDWGc::Dwg_POINTCLOUD_Clippings_vertices_get;
*swig_vertices_set = *LibreDWGc::Dwg_POINTCLOUD_Clippings_vertices_set;
*swig_z_min_get = *LibreDWGc::Dwg_POINTCLOUD_Clippings_z_min_get;
*swig_z_min_set = *LibreDWGc::Dwg_POINTCLOUD_Clippings_z_min_set;
*swig_z_max_get = *LibreDWGc::Dwg_POINTCLOUD_Clippings_z_max_get;
*swig_z_max_set = *LibreDWGc::Dwg_POINTCLOUD_Clippings_z_max_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_POINTCLOUD_Clippings(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_POINTCLOUD_Clippings($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_POINTCLOUDEX_Croppings ##############

package LibreDWG::Dwg_POINTCLOUDEX_Croppings;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_parent_set;
*swig_type_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_type_get;
*swig_type_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_type_set;
*swig_is_inside_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inside_get;
*swig_is_inside_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inside_set;
*swig_is_inverted_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inverted_get;
*swig_is_inverted_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_is_inverted_set;
*swig_crop_plane_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_plane_get;
*swig_crop_plane_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_plane_set;
*swig_crop_x_dir_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_x_dir_get;
*swig_crop_x_dir_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_x_dir_set;
*swig_crop_y_dir_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_y_dir_get;
*swig_crop_y_dir_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_crop_y_dir_set;
*swig_num_pts_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_num_pts_get;
*swig_num_pts_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_num_pts_set;
*swig_pts_get = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_pts_get;
*swig_pts_set = *LibreDWGc::Dwg_POINTCLOUDEX_Croppings_pts_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_POINTCLOUDEX_Croppings(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_POINTCLOUDEX_Croppings($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POINTCLOUD ##############

package LibreDWG::Dwg_Entity_POINTCLOUD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_class_version_set;
*swig_origin_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_origin_get;
*swig_origin_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_origin_set;
*swig_saved_filename_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_saved_filename_get;
*swig_saved_filename_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_saved_filename_set;
*swig_num_source_files_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_num_source_files_get;
*swig_num_source_files_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_num_source_files_set;
*swig_source_files_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_source_files_get;
*swig_source_files_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_source_files_set;
*swig_extents_min_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_extents_min_get;
*swig_extents_min_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_extents_min_set;
*swig_extents_max_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_extents_max_get;
*swig_extents_max_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_extents_max_set;
*swig_numpoints_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_numpoints_get;
*swig_numpoints_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_numpoints_set;
*swig_ucs_name_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_name_get;
*swig_ucs_name_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_name_set;
*swig_ucs_origin_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_origin_get;
*swig_ucs_origin_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_origin_set;
*swig_ucs_x_dir_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_x_dir_get;
*swig_ucs_x_dir_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_x_dir_set;
*swig_ucs_y_dir_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_y_dir_get;
*swig_ucs_y_dir_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_y_dir_set;
*swig_ucs_z_dir_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_z_dir_get;
*swig_ucs_z_dir_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_ucs_z_dir_set;
*swig_pointclouddef_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_pointclouddef_get;
*swig_pointclouddef_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_pointclouddef_set;
*swig_reactor_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_reactor_get;
*swig_reactor_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_reactor_set;
*swig_show_intensity_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_show_intensity_get;
*swig_show_intensity_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_show_intensity_set;
*swig_intensity_scheme_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_scheme_get;
*swig_intensity_scheme_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_scheme_set;
*swig_intensity_style_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_style_get;
*swig_intensity_style_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_intensity_style_set;
*swig_show_clipping_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_show_clipping_get;
*swig_show_clipping_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_show_clipping_set;
*swig_num_clippings_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_num_clippings_get;
*swig_num_clippings_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_num_clippings_set;
*swig_clippings_get = *LibreDWGc::Dwg_Entity_POINTCLOUD_clippings_get;
*swig_clippings_set = *LibreDWGc::Dwg_Entity_POINTCLOUD_clippings_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POINTCLOUD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POINTCLOUD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_POINTCLOUDEX ##############

package LibreDWG::Dwg_Entity_POINTCLOUDEX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_class_version_set;
*swig_extents_min_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_min_get;
*swig_extents_min_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_min_set;
*swig_extents_max_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_max_get;
*swig_extents_max_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_extents_max_set;
*swig_ucs_origin_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_origin_get;
*swig_ucs_origin_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_origin_set;
*swig_ucs_x_dir_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_x_dir_get;
*swig_ucs_x_dir_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_x_dir_set;
*swig_ucs_y_dir_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_y_dir_get;
*swig_ucs_y_dir_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_y_dir_set;
*swig_ucs_z_dir_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_z_dir_get;
*swig_ucs_z_dir_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_ucs_z_dir_set;
*swig_is_locked_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_is_locked_get;
*swig_is_locked_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_is_locked_set;
*swig_pointclouddefex_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_pointclouddefex_get;
*swig_pointclouddefex_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_pointclouddefex_set;
*swig_reactor_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_reactor_get;
*swig_reactor_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_reactor_set;
*swig_name_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_name_get;
*swig_name_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_name_set;
*swig_show_intensity_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_intensity_get;
*swig_show_intensity_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_intensity_set;
*swig_stylization_type_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_stylization_type_get;
*swig_stylization_type_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_stylization_type_set;
*swig_intensity_colorscheme_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_get;
*swig_intensity_colorscheme_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_colorscheme_set;
*swig_cur_colorscheme_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_cur_colorscheme_get;
*swig_cur_colorscheme_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_cur_colorscheme_set;
*swig_classification_colorscheme_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_classification_colorscheme_get;
*swig_classification_colorscheme_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_classification_colorscheme_set;
*swig_elevation_min_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_min_get;
*swig_elevation_min_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_min_set;
*swig_elevation_max_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_max_get;
*swig_elevation_max_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_max_set;
*swig_intensity_min_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_min_get;
*swig_intensity_min_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_min_set;
*swig_intensity_max_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_max_get;
*swig_intensity_max_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_max_set;
*swig_intensity_out_of_range_behavior_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_get;
*swig_intensity_out_of_range_behavior_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_out_of_range_behavior_set;
*swig_elevation_out_of_range_behavior_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_get;
*swig_elevation_out_of_range_behavior_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_out_of_range_behavior_set;
*swig_elevation_apply_to_fixed_range_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_get;
*swig_elevation_apply_to_fixed_range_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_apply_to_fixed_range_set;
*swig_intensity_as_gradient_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_get;
*swig_intensity_as_gradient_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_intensity_as_gradient_set;
*swig_elevation_as_gradient_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_get;
*swig_elevation_as_gradient_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_elevation_as_gradient_set;
*swig_show_cropping_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_cropping_get;
*swig_show_cropping_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_show_cropping_set;
*swig_unknown_bl0_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl0_get;
*swig_unknown_bl0_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl0_set;
*swig_unknown_bl1_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl1_get;
*swig_unknown_bl1_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_unknown_bl1_set;
*swig_num_croppings_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_num_croppings_get;
*swig_num_croppings_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_num_croppings_set;
*swig_croppings_get = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_croppings_get;
*swig_croppings_set = *LibreDWGc::Dwg_Entity_POINTCLOUDEX_croppings_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_POINTCLOUDEX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_POINTCLOUDEX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_POINTCLOUDDEF ##############

package LibreDWG::Dwg_Object_POINTCLOUDDEF;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_class_version_set;
*swig_source_filename_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_source_filename_get;
*swig_source_filename_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_source_filename_set;
*swig_is_loaded_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_is_loaded_get;
*swig_is_loaded_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_is_loaded_set;
*swig_numpoints_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_numpoints_get;
*swig_numpoints_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_numpoints_set;
*swig_extents_min_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_min_get;
*swig_extents_min_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_min_set;
*swig_extents_max_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_max_get;
*swig_extents_max_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_extents_max_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_POINTCLOUDDEF(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_POINTCLOUDDEF($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_POINTCLOUDDEFEX ##############

package LibreDWG::Dwg_Object_POINTCLOUDDEFEX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_class_version_set;
*swig_source_filename_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_source_filename_get;
*swig_source_filename_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_source_filename_set;
*swig_is_loaded_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_is_loaded_get;
*swig_is_loaded_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_is_loaded_set;
*swig_numpoints_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_numpoints_get;
*swig_numpoints_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_numpoints_set;
*swig_extents_min_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_min_get;
*swig_extents_min_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_min_set;
*swig_extents_max_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_max_get;
*swig_extents_max_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEFEX_extents_max_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_POINTCLOUDDEFEX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_POINTCLOUDDEFEX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR ##############

package LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_POINTCLOUDDEF_REACTOR(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_POINTCLOUDDEF_REACTOR($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR_EX ##############

package LibreDWG::Dwg_Object_POINTCLOUDDEF_REACTOR_EX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_POINTCLOUDDEF_REACTOR_EX_class_version_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_POINTCLOUDDEF_REACTOR_EX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_POINTCLOUDDEF_REACTOR_EX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ColorRamp ##############

package LibreDWG::Dwg_ColorRamp;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_ColorRamp_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_ColorRamp_parent_set;
*swig_colorscheme_get = *LibreDWGc::Dwg_ColorRamp_colorscheme_get;
*swig_colorscheme_set = *LibreDWGc::Dwg_ColorRamp_colorscheme_set;
*swig_unknown_bl_get = *LibreDWGc::Dwg_ColorRamp_unknown_bl_get;
*swig_unknown_bl_set = *LibreDWGc::Dwg_ColorRamp_unknown_bl_set;
*swig_unknown_b_get = *LibreDWGc::Dwg_ColorRamp_unknown_b_get;
*swig_unknown_b_set = *LibreDWGc::Dwg_ColorRamp_unknown_b_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ColorRamp(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ColorRamp($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_POINTCLOUDCOLORMAP_Ramp ##############

package LibreDWG::Dwg_POINTCLOUDCOLORMAP_Ramp;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_class_version_set;
*swig_num_ramps_get = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_get;
*swig_num_ramps_set = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_num_ramps_set;
*swig_ramps_get = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_get;
*swig_ramps_set = *LibreDWGc::Dwg_POINTCLOUDCOLORMAP_Ramp_ramps_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_POINTCLOUDCOLORMAP_Ramp(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_POINTCLOUDCOLORMAP_Ramp($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_POINTCLOUDCOLORMAP ##############

package LibreDWG::Dwg_Object_POINTCLOUDCOLORMAP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_parent_set;
*swig_class_version_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_class_version_set;
*swig_def_intensity_colorscheme_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_get;
*swig_def_intensity_colorscheme_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_intensity_colorscheme_set;
*swig_def_elevation_colorscheme_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_get;
*swig_def_elevation_colorscheme_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_elevation_colorscheme_set;
*swig_def_classification_colorscheme_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_get;
*swig_def_classification_colorscheme_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_def_classification_colorscheme_set;
*swig_num_colorramps_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_get;
*swig_num_colorramps_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_colorramps_set;
*swig_colorramps_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_colorramps_get;
*swig_colorramps_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_colorramps_set;
*swig_num_classification_colorramps_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_get;
*swig_num_classification_colorramps_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_num_classification_colorramps_set;
*swig_classification_colorramps_get = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_get;
*swig_classification_colorramps_set = *LibreDWGc::Dwg_Object_POINTCLOUDCOLORMAP_classification_colorramps_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_POINTCLOUDCOLORMAP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_POINTCLOUDCOLORMAP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_COMPOUNDOBJECTID ##############

package LibreDWG::Dwg_COMPOUNDOBJECTID;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_COMPOUNDOBJECTID_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_COMPOUNDOBJECTID_parent_set;
*swig_has_object_get = *LibreDWGc::Dwg_COMPOUNDOBJECTID_has_object_get;
*swig_has_object_set = *LibreDWGc::Dwg_COMPOUNDOBJECTID_has_object_set;
*swig_name_get = *LibreDWGc::Dwg_COMPOUNDOBJECTID_name_get;
*swig_name_set = *LibreDWGc::Dwg_COMPOUNDOBJECTID_name_set;
*swig_object_get = *LibreDWGc::Dwg_COMPOUNDOBJECTID_object_get;
*swig_object_set = *LibreDWGc::Dwg_COMPOUNDOBJECTID_object_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_COMPOUNDOBJECTID(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_COMPOUNDOBJECTID($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_PARTIAL_VIEWING_INDEX_Entry ##############

package LibreDWG::Dwg_PARTIAL_VIEWING_INDEX_Entry;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_parent_set;
*swig_extents_min_get = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_get;
*swig_extents_min_set = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_min_set;
*swig_extents_max_get = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_get;
*swig_extents_max_set = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_extents_max_set;
*swig_object_get = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_object_get;
*swig_object_set = *LibreDWGc::Dwg_PARTIAL_VIEWING_INDEX_Entry_object_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_PARTIAL_VIEWING_INDEX_Entry(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_PARTIAL_VIEWING_INDEX_Entry($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_PARTIAL_VIEWING_INDEX ##############

package LibreDWG::Dwg_Object_PARTIAL_VIEWING_INDEX;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_parent_set;
*swig_num_entries_get = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_num_entries_set;
*swig_has_entries_get = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_get;
*swig_has_entries_set = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_has_entries_set;
*swig_entries_get = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_Object_PARTIAL_VIEWING_INDEX_entries_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_PARTIAL_VIEWING_INDEX(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_PARTIAL_VIEWING_INDEX($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_UNKNOWN_ENT ##############

package LibreDWG::Dwg_Entity_UNKNOWN_ENT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_UNKNOWN_ENT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_UNKNOWN_ENT_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_UNKNOWN_ENT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_UNKNOWN_ENT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_UNKNOWN_OBJ ##############

package LibreDWG::Dwg_Object_UNKNOWN_OBJ;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Object_UNKNOWN_OBJ_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_UNKNOWN_OBJ_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_UNKNOWN_OBJ(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_UNKNOWN_OBJ($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_REPEAT ##############

package LibreDWG::Dwg_Entity_REPEAT;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_REPEAT_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_REPEAT_parent_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_REPEAT(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_REPEAT($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_ENDREP ##############

package LibreDWG::Dwg_Entity_ENDREP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_ENDREP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_ENDREP_parent_set;
*swig_numcols_get = *LibreDWGc::Dwg_Entity_ENDREP_numcols_get;
*swig_numcols_set = *LibreDWGc::Dwg_Entity_ENDREP_numcols_set;
*swig_numrows_get = *LibreDWGc::Dwg_Entity_ENDREP_numrows_get;
*swig_numrows_set = *LibreDWGc::Dwg_Entity_ENDREP_numrows_set;
*swig_colspacing_get = *LibreDWGc::Dwg_Entity_ENDREP_colspacing_get;
*swig_colspacing_set = *LibreDWGc::Dwg_Entity_ENDREP_colspacing_set;
*swig_rowspacing_get = *LibreDWGc::Dwg_Entity_ENDREP_rowspacing_get;
*swig_rowspacing_set = *LibreDWGc::Dwg_Entity_ENDREP_rowspacing_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_ENDREP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_ENDREP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_LOAD ##############

package LibreDWG::Dwg_Entity_LOAD;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_LOAD_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_LOAD_parent_set;
*swig_file_name_get = *LibreDWGc::Dwg_Entity_LOAD_file_name_get;
*swig_file_name_set = *LibreDWGc::Dwg_Entity_LOAD_file_name_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_LOAD(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_LOAD($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity_JUMP ##############

package LibreDWG::Dwg_Entity_JUMP;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity_JUMP_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity_JUMP_parent_set;
*swig_jump_address_raw_get = *LibreDWGc::Dwg_Entity_JUMP_jump_address_raw_get;
*swig_jump_address_raw_set = *LibreDWGc::Dwg_Entity_JUMP_jump_address_raw_set;
*swig_jump_address_get = *LibreDWGc::Dwg_Entity_JUMP_jump_address_get;
*swig_jump_address_set = *LibreDWGc::Dwg_Entity_JUMP_jump_address_set;
*swig_jump_entity_section_get = *LibreDWGc::Dwg_Entity_JUMP_jump_entity_section_get;
*swig_jump_entity_section_set = *LibreDWGc::Dwg_Entity_JUMP_jump_entity_section_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity_JUMP(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity_JUMP($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Entity__3DLINE ##############

package LibreDWG::Dwg_Entity__3DLINE;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_parent_get = *LibreDWGc::Dwg_Entity__3DLINE_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Entity__3DLINE_parent_set;
*swig_start_get = *LibreDWGc::Dwg_Entity__3DLINE_start_get;
*swig_start_set = *LibreDWGc::Dwg_Entity__3DLINE_start_set;
*swig_end_get = *LibreDWGc::Dwg_Entity__3DLINE_end_get;
*swig_end_set = *LibreDWGc::Dwg_Entity__3DLINE_end_set;
*swig_extrusion_get = *LibreDWGc::Dwg_Entity__3DLINE_extrusion_get;
*swig_extrusion_set = *LibreDWGc::Dwg_Entity__3DLINE_extrusion_set;
*swig_thickness_get = *LibreDWGc::Dwg_Entity__3DLINE_thickness_get;
*swig_thickness_set = *LibreDWGc::Dwg_Entity__3DLINE_thickness_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Entity__3DLINE(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Entity__3DLINE($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Eed_Data ##############

package LibreDWG::Dwg_Eed_Data;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_u_get = *LibreDWGc::Dwg_Eed_Data_u_get;
*swig_u_set = *LibreDWGc::Dwg_Eed_Data_u_set;
*swig_code_get = *LibreDWGc::Dwg_Eed_Data_code_get;
*swig_code_set = *LibreDWGc::Dwg_Eed_Data_code_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Eed_Data(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Eed_Data($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t ##############

package LibreDWG::eed_data_t;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_eed_0_get = *LibreDWGc::eed_data_t_eed_0_get;
*swig_eed_0_set = *LibreDWGc::eed_data_t_eed_0_set;
*swig_eed_0_r2007_get = *LibreDWGc::eed_data_t_eed_0_r2007_get;
*swig_eed_0_r2007_set = *LibreDWGc::eed_data_t_eed_0_r2007_set;
*swig_eed_1_get = *LibreDWGc::eed_data_t_eed_1_get;
*swig_eed_1_set = *LibreDWGc::eed_data_t_eed_1_set;
*swig_eed_2_get = *LibreDWGc::eed_data_t_eed_2_get;
*swig_eed_2_set = *LibreDWGc::eed_data_t_eed_2_set;
*swig_eed_3_get = *LibreDWGc::eed_data_t_eed_3_get;
*swig_eed_3_set = *LibreDWGc::eed_data_t_eed_3_set;
*swig_eed_4_get = *LibreDWGc::eed_data_t_eed_4_get;
*swig_eed_4_set = *LibreDWGc::eed_data_t_eed_4_set;
*swig_eed_5_get = *LibreDWGc::eed_data_t_eed_5_get;
*swig_eed_5_set = *LibreDWGc::eed_data_t_eed_5_set;
*swig_eed_10_get = *LibreDWGc::eed_data_t_eed_10_get;
*swig_eed_10_set = *LibreDWGc::eed_data_t_eed_10_set;
*swig_eed_40_get = *LibreDWGc::eed_data_t_eed_40_get;
*swig_eed_40_set = *LibreDWGc::eed_data_t_eed_40_set;
*swig_eed_70_get = *LibreDWGc::eed_data_t_eed_70_get;
*swig_eed_70_set = *LibreDWGc::eed_data_t_eed_70_set;
*swig_eed_71_get = *LibreDWGc::eed_data_t_eed_71_get;
*swig_eed_71_set = *LibreDWGc::eed_data_t_eed_71_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_71 ##############

package LibreDWG::eed_data_t_eed_71;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_rl_get = *LibreDWGc::eed_data_t_eed_71_rl_get;
*swig_rl_set = *LibreDWGc::eed_data_t_eed_71_rl_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_71(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_71($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_70 ##############

package LibreDWG::eed_data_t_eed_70;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_rs_get = *LibreDWGc::eed_data_t_eed_70_rs_get;
*swig_rs_set = *LibreDWGc::eed_data_t_eed_70_rs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_70(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_70($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_40 ##############

package LibreDWG::eed_data_t_eed_40;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_real_get = *LibreDWGc::eed_data_t_eed_40_real_get;
*swig_real_set = *LibreDWGc::eed_data_t_eed_40_real_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_40(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_40($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_10 ##############

package LibreDWG::eed_data_t_eed_10;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_point_get = *LibreDWGc::eed_data_t_eed_10_point_get;
*swig_point_set = *LibreDWGc::eed_data_t_eed_10_point_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_10(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_10($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_5 ##############

package LibreDWG::eed_data_t_eed_5;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_entity_get = *LibreDWGc::eed_data_t_eed_5_entity_get;
*swig_entity_set = *LibreDWGc::eed_data_t_eed_5_entity_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_5(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_5($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_4 ##############

package LibreDWG::eed_data_t_eed_4;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_length_get = *LibreDWGc::eed_data_t_eed_4_length_get;
*swig_length_set = *LibreDWGc::eed_data_t_eed_4_length_set;
*swig_data_get = *LibreDWGc::eed_data_t_eed_4_data_get;
*swig_data_set = *LibreDWGc::eed_data_t_eed_4_data_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_4(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_4($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_3 ##############

package LibreDWG::eed_data_t_eed_3;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_layer_get = *LibreDWGc::eed_data_t_eed_3_layer_get;
*swig_layer_set = *LibreDWGc::eed_data_t_eed_3_layer_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_3(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_3($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_2 ##############

package LibreDWG::eed_data_t_eed_2;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_close_get = *LibreDWGc::eed_data_t_eed_2_close_get;
*swig_close_set = *LibreDWGc::eed_data_t_eed_2_close_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_2(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_2($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_1 ##############

package LibreDWG::eed_data_t_eed_1;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_appid_index_get = *LibreDWGc::eed_data_t_eed_1_appid_index_get;
*swig_appid_index_set = *LibreDWGc::eed_data_t_eed_1_appid_index_set;
*swig_invalid_get = *LibreDWGc::eed_data_t_eed_1_invalid_get;
*swig_invalid_set = *LibreDWGc::eed_data_t_eed_1_invalid_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_1(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_1($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_0_r2007 ##############

package LibreDWG::eed_data_t_eed_0_r2007;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_length_get = *LibreDWGc::eed_data_t_eed_0_r2007_length_get;
*swig_length_set = *LibreDWGc::eed_data_t_eed_0_r2007_length_set;
*swig__padding_get = *LibreDWGc::eed_data_t_eed_0_r2007__padding_get;
*swig__padding_set = *LibreDWGc::eed_data_t_eed_0_r2007__padding_set;
*swig_is_tu_get = *LibreDWGc::eed_data_t_eed_0_r2007_is_tu_get;
*swig_is_tu_set = *LibreDWGc::eed_data_t_eed_0_r2007_is_tu_set;
*swig_string_get = *LibreDWGc::eed_data_t_eed_0_r2007_string_get;
*swig_string_set = *LibreDWGc::eed_data_t_eed_0_r2007_string_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_0_r2007(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_0_r2007($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::eed_data_t_eed_0 ##############

package LibreDWG::eed_data_t_eed_0;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_length_get = *LibreDWGc::eed_data_t_eed_0_length_get;
*swig_length_set = *LibreDWGc::eed_data_t_eed_0_length_set;
*swig_codepage_get = *LibreDWGc::eed_data_t_eed_0_codepage_get;
*swig_codepage_set = *LibreDWGc::eed_data_t_eed_0_codepage_set;
*swig_is_tu_get = *LibreDWGc::eed_data_t_eed_0_is_tu_get;
*swig_is_tu_set = *LibreDWGc::eed_data_t_eed_0_is_tu_set;
*swig_string_get = *LibreDWGc::eed_data_t_eed_0_string_get;
*swig_string_set = *LibreDWGc::eed_data_t_eed_0_string_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_eed_data_t_eed_0(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_eed_data_t_eed_0($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Eed ##############

package LibreDWG::Dwg_Eed;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_Eed_size_get;
*swig_size_set = *LibreDWGc::Dwg_Eed_size_set;
*swig_handle_get = *LibreDWGc::Dwg_Eed_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_Eed_handle_set;
*swig_data_get = *LibreDWGc::Dwg_Eed_data_get;
*swig_data_set = *LibreDWGc::Dwg_Eed_data_set;
*swig_raw_get = *LibreDWGc::Dwg_Eed_raw_get;
*swig_raw_set = *LibreDWGc::Dwg_Eed_raw_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Eed(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Eed($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_Entity ##############

package LibreDWG::Dwg_Object_Entity;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_objid_get = *LibreDWGc::Dwg_Object_Entity_objid_get;
*swig_objid_set = *LibreDWGc::Dwg_Object_Entity_objid_set;
*swig_tio_get = *LibreDWGc::Dwg_Object_Entity_tio_get;
*swig_tio_set = *LibreDWGc::Dwg_Object_Entity_tio_set;
*swig_dwg_get = *LibreDWGc::Dwg_Object_Entity_dwg_get;
*swig_dwg_set = *LibreDWGc::Dwg_Object_Entity_dwg_set;
*swig_num_eed_get = *LibreDWGc::Dwg_Object_Entity_num_eed_get;
*swig_num_eed_set = *LibreDWGc::Dwg_Object_Entity_num_eed_set;
*swig_eed_get = *LibreDWGc::Dwg_Object_Entity_eed_get;
*swig_eed_set = *LibreDWGc::Dwg_Object_Entity_eed_set;
*swig_preview_exists_get = *LibreDWGc::Dwg_Object_Entity_preview_exists_get;
*swig_preview_exists_set = *LibreDWGc::Dwg_Object_Entity_preview_exists_set;
*swig_preview_is_proxy_get = *LibreDWGc::Dwg_Object_Entity_preview_is_proxy_get;
*swig_preview_is_proxy_set = *LibreDWGc::Dwg_Object_Entity_preview_is_proxy_set;
*swig_preview_size_get = *LibreDWGc::Dwg_Object_Entity_preview_size_get;
*swig_preview_size_set = *LibreDWGc::Dwg_Object_Entity_preview_size_set;
*swig_preview_get = *LibreDWGc::Dwg_Object_Entity_preview_get;
*swig_preview_set = *LibreDWGc::Dwg_Object_Entity_preview_set;
*swig_entmode_get = *LibreDWGc::Dwg_Object_Entity_entmode_get;
*swig_entmode_set = *LibreDWGc::Dwg_Object_Entity_entmode_set;
*swig_num_reactors_get = *LibreDWGc::Dwg_Object_Entity_num_reactors_get;
*swig_num_reactors_set = *LibreDWGc::Dwg_Object_Entity_num_reactors_set;
*swig_is_xdic_missing_get = *LibreDWGc::Dwg_Object_Entity_is_xdic_missing_get;
*swig_is_xdic_missing_set = *LibreDWGc::Dwg_Object_Entity_is_xdic_missing_set;
*swig_isbylayerlt_get = *LibreDWGc::Dwg_Object_Entity_isbylayerlt_get;
*swig_isbylayerlt_set = *LibreDWGc::Dwg_Object_Entity_isbylayerlt_set;
*swig_nolinks_get = *LibreDWGc::Dwg_Object_Entity_nolinks_get;
*swig_nolinks_set = *LibreDWGc::Dwg_Object_Entity_nolinks_set;
*swig_has_ds_data_get = *LibreDWGc::Dwg_Object_Entity_has_ds_data_get;
*swig_has_ds_data_set = *LibreDWGc::Dwg_Object_Entity_has_ds_data_set;
*swig_color_get = *LibreDWGc::Dwg_Object_Entity_color_get;
*swig_color_set = *LibreDWGc::Dwg_Object_Entity_color_set;
*swig_ltype_scale_get = *LibreDWGc::Dwg_Object_Entity_ltype_scale_get;
*swig_ltype_scale_set = *LibreDWGc::Dwg_Object_Entity_ltype_scale_set;
*swig_ltype_flags_get = *LibreDWGc::Dwg_Object_Entity_ltype_flags_get;
*swig_ltype_flags_set = *LibreDWGc::Dwg_Object_Entity_ltype_flags_set;
*swig_plotstyle_flags_get = *LibreDWGc::Dwg_Object_Entity_plotstyle_flags_get;
*swig_plotstyle_flags_set = *LibreDWGc::Dwg_Object_Entity_plotstyle_flags_set;
*swig_material_flags_get = *LibreDWGc::Dwg_Object_Entity_material_flags_get;
*swig_material_flags_set = *LibreDWGc::Dwg_Object_Entity_material_flags_set;
*swig_shadow_flags_get = *LibreDWGc::Dwg_Object_Entity_shadow_flags_get;
*swig_shadow_flags_set = *LibreDWGc::Dwg_Object_Entity_shadow_flags_set;
*swig_has_full_visualstyle_get = *LibreDWGc::Dwg_Object_Entity_has_full_visualstyle_get;
*swig_has_full_visualstyle_set = *LibreDWGc::Dwg_Object_Entity_has_full_visualstyle_set;
*swig_has_face_visualstyle_get = *LibreDWGc::Dwg_Object_Entity_has_face_visualstyle_get;
*swig_has_face_visualstyle_set = *LibreDWGc::Dwg_Object_Entity_has_face_visualstyle_set;
*swig_has_edge_visualstyle_get = *LibreDWGc::Dwg_Object_Entity_has_edge_visualstyle_get;
*swig_has_edge_visualstyle_set = *LibreDWGc::Dwg_Object_Entity_has_edge_visualstyle_set;
*swig_invisible_get = *LibreDWGc::Dwg_Object_Entity_invisible_get;
*swig_invisible_set = *LibreDWGc::Dwg_Object_Entity_invisible_set;
*swig_linewt_get = *LibreDWGc::Dwg_Object_Entity_linewt_get;
*swig_linewt_set = *LibreDWGc::Dwg_Object_Entity_linewt_set;
*swig_flag_r11_get = *LibreDWGc::Dwg_Object_Entity_flag_r11_get;
*swig_flag_r11_set = *LibreDWGc::Dwg_Object_Entity_flag_r11_set;
*swig_opts_r11_get = *LibreDWGc::Dwg_Object_Entity_opts_r11_get;
*swig_opts_r11_set = *LibreDWGc::Dwg_Object_Entity_opts_r11_set;
*swig_extra_r11_get = *LibreDWGc::Dwg_Object_Entity_extra_r11_get;
*swig_extra_r11_set = *LibreDWGc::Dwg_Object_Entity_extra_r11_set;
*swig_color_r11_get = *LibreDWGc::Dwg_Object_Entity_color_r11_get;
*swig_color_r11_set = *LibreDWGc::Dwg_Object_Entity_color_r11_set;
*swig_elevation_r11_get = *LibreDWGc::Dwg_Object_Entity_elevation_r11_get;
*swig_elevation_r11_set = *LibreDWGc::Dwg_Object_Entity_elevation_r11_set;
*swig_thickness_r11_get = *LibreDWGc::Dwg_Object_Entity_thickness_r11_get;
*swig_thickness_r11_set = *LibreDWGc::Dwg_Object_Entity_thickness_r11_set;
*swig_viewport_get = *LibreDWGc::Dwg_Object_Entity_viewport_get;
*swig_viewport_set = *LibreDWGc::Dwg_Object_Entity_viewport_set;
*swig___iterator_get = *LibreDWGc::Dwg_Object_Entity___iterator_get;
*swig___iterator_set = *LibreDWGc::Dwg_Object_Entity___iterator_set;
*swig_ownerhandle_get = *LibreDWGc::Dwg_Object_Entity_ownerhandle_get;
*swig_ownerhandle_set = *LibreDWGc::Dwg_Object_Entity_ownerhandle_set;
*swig_reactors_get = *LibreDWGc::Dwg_Object_Entity_reactors_get;
*swig_reactors_set = *LibreDWGc::Dwg_Object_Entity_reactors_set;
*swig_xdicobjhandle_get = *LibreDWGc::Dwg_Object_Entity_xdicobjhandle_get;
*swig_xdicobjhandle_set = *LibreDWGc::Dwg_Object_Entity_xdicobjhandle_set;
*swig_prev_entity_get = *LibreDWGc::Dwg_Object_Entity_prev_entity_get;
*swig_prev_entity_set = *LibreDWGc::Dwg_Object_Entity_prev_entity_set;
*swig_next_entity_get = *LibreDWGc::Dwg_Object_Entity_next_entity_get;
*swig_next_entity_set = *LibreDWGc::Dwg_Object_Entity_next_entity_set;
*swig_layer_get = *LibreDWGc::Dwg_Object_Entity_layer_get;
*swig_layer_set = *LibreDWGc::Dwg_Object_Entity_layer_set;
*swig_ltype_get = *LibreDWGc::Dwg_Object_Entity_ltype_get;
*swig_ltype_set = *LibreDWGc::Dwg_Object_Entity_ltype_set;
*swig_material_get = *LibreDWGc::Dwg_Object_Entity_material_get;
*swig_material_set = *LibreDWGc::Dwg_Object_Entity_material_set;
*swig_shadow_get = *LibreDWGc::Dwg_Object_Entity_shadow_get;
*swig_shadow_set = *LibreDWGc::Dwg_Object_Entity_shadow_set;
*swig_plotstyle_get = *LibreDWGc::Dwg_Object_Entity_plotstyle_get;
*swig_plotstyle_set = *LibreDWGc::Dwg_Object_Entity_plotstyle_set;
*swig_full_visualstyle_get = *LibreDWGc::Dwg_Object_Entity_full_visualstyle_get;
*swig_full_visualstyle_set = *LibreDWGc::Dwg_Object_Entity_full_visualstyle_set;
*swig_face_visualstyle_get = *LibreDWGc::Dwg_Object_Entity_face_visualstyle_get;
*swig_face_visualstyle_set = *LibreDWGc::Dwg_Object_Entity_face_visualstyle_set;
*swig_edge_visualstyle_get = *LibreDWGc::Dwg_Object_Entity_edge_visualstyle_get;
*swig_edge_visualstyle_set = *LibreDWGc::Dwg_Object_Entity_edge_visualstyle_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_Entity(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_Entity($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_object_entity_tio ##############

package LibreDWG::_dwg_object_entity_tio;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_UNUSED_get = *LibreDWGc::_dwg_object_entity_tio_UNUSED_get;
*swig_UNUSED_set = *LibreDWGc::_dwg_object_entity_tio_UNUSED_set;
*swig_DIMENSION_common_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_common_get;
*swig_DIMENSION_common_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_common_set;
*swig__3DFACE_get = *LibreDWGc::_dwg_object_entity_tio__3DFACE_get;
*swig__3DFACE_set = *LibreDWGc::_dwg_object_entity_tio__3DFACE_set;
*swig__3DSOLID_get = *LibreDWGc::_dwg_object_entity_tio__3DSOLID_get;
*swig__3DSOLID_set = *LibreDWGc::_dwg_object_entity_tio__3DSOLID_set;
*swig_ARC_get = *LibreDWGc::_dwg_object_entity_tio_ARC_get;
*swig_ARC_set = *LibreDWGc::_dwg_object_entity_tio_ARC_set;
*swig_ATTDEF_get = *LibreDWGc::_dwg_object_entity_tio_ATTDEF_get;
*swig_ATTDEF_set = *LibreDWGc::_dwg_object_entity_tio_ATTDEF_set;
*swig_ATTRIB_get = *LibreDWGc::_dwg_object_entity_tio_ATTRIB_get;
*swig_ATTRIB_set = *LibreDWGc::_dwg_object_entity_tio_ATTRIB_set;
*swig_BLOCK_get = *LibreDWGc::_dwg_object_entity_tio_BLOCK_get;
*swig_BLOCK_set = *LibreDWGc::_dwg_object_entity_tio_BLOCK_set;
*swig_BODY_get = *LibreDWGc::_dwg_object_entity_tio_BODY_get;
*swig_BODY_set = *LibreDWGc::_dwg_object_entity_tio_BODY_set;
*swig_CIRCLE_get = *LibreDWGc::_dwg_object_entity_tio_CIRCLE_get;
*swig_CIRCLE_set = *LibreDWGc::_dwg_object_entity_tio_CIRCLE_set;
*swig_DIMENSION_ALIGNED_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ALIGNED_get;
*swig_DIMENSION_ALIGNED_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ALIGNED_set;
*swig_DIMENSION_ANG2LN_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG2LN_get;
*swig_DIMENSION_ANG2LN_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG2LN_set;
*swig_DIMENSION_ANG3PT_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG3PT_get;
*swig_DIMENSION_ANG3PT_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ANG3PT_set;
*swig_DIMENSION_DIAMETER_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_DIAMETER_get;
*swig_DIMENSION_DIAMETER_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_DIAMETER_set;
*swig_DIMENSION_LINEAR_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_LINEAR_get;
*swig_DIMENSION_LINEAR_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_LINEAR_set;
*swig_DIMENSION_ORDINATE_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ORDINATE_get;
*swig_DIMENSION_ORDINATE_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_ORDINATE_set;
*swig_DIMENSION_RADIUS_get = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_RADIUS_get;
*swig_DIMENSION_RADIUS_set = *LibreDWGc::_dwg_object_entity_tio_DIMENSION_RADIUS_set;
*swig_ELLIPSE_get = *LibreDWGc::_dwg_object_entity_tio_ELLIPSE_get;
*swig_ELLIPSE_set = *LibreDWGc::_dwg_object_entity_tio_ELLIPSE_set;
*swig_ENDBLK_get = *LibreDWGc::_dwg_object_entity_tio_ENDBLK_get;
*swig_ENDBLK_set = *LibreDWGc::_dwg_object_entity_tio_ENDBLK_set;
*swig_INSERT_get = *LibreDWGc::_dwg_object_entity_tio_INSERT_get;
*swig_INSERT_set = *LibreDWGc::_dwg_object_entity_tio_INSERT_set;
*swig_LEADER_get = *LibreDWGc::_dwg_object_entity_tio_LEADER_get;
*swig_LEADER_set = *LibreDWGc::_dwg_object_entity_tio_LEADER_set;
*swig_LINE_get = *LibreDWGc::_dwg_object_entity_tio_LINE_get;
*swig_LINE_set = *LibreDWGc::_dwg_object_entity_tio_LINE_set;
*swig_LOAD_get = *LibreDWGc::_dwg_object_entity_tio_LOAD_get;
*swig_LOAD_set = *LibreDWGc::_dwg_object_entity_tio_LOAD_set;
*swig_MINSERT_get = *LibreDWGc::_dwg_object_entity_tio_MINSERT_get;
*swig_MINSERT_set = *LibreDWGc::_dwg_object_entity_tio_MINSERT_set;
*swig_MLINE_get = *LibreDWGc::_dwg_object_entity_tio_MLINE_get;
*swig_MLINE_set = *LibreDWGc::_dwg_object_entity_tio_MLINE_set;
*swig_MTEXT_get = *LibreDWGc::_dwg_object_entity_tio_MTEXT_get;
*swig_MTEXT_set = *LibreDWGc::_dwg_object_entity_tio_MTEXT_set;
*swig_OLEFRAME_get = *LibreDWGc::_dwg_object_entity_tio_OLEFRAME_get;
*swig_OLEFRAME_set = *LibreDWGc::_dwg_object_entity_tio_OLEFRAME_set;
*swig_POINT_get = *LibreDWGc::_dwg_object_entity_tio_POINT_get;
*swig_POINT_set = *LibreDWGc::_dwg_object_entity_tio_POINT_set;
*swig_POLYLINE_2D_get = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_2D_get;
*swig_POLYLINE_2D_set = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_2D_set;
*swig_POLYLINE_3D_get = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_3D_get;
*swig_POLYLINE_3D_set = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_3D_set;
*swig_POLYLINE_MESH_get = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_MESH_get;
*swig_POLYLINE_MESH_set = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_MESH_set;
*swig_POLYLINE_PFACE_get = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_PFACE_get;
*swig_POLYLINE_PFACE_set = *LibreDWGc::_dwg_object_entity_tio_POLYLINE_PFACE_set;
*swig_PROXY_ENTITY_get = *LibreDWGc::_dwg_object_entity_tio_PROXY_ENTITY_get;
*swig_PROXY_ENTITY_set = *LibreDWGc::_dwg_object_entity_tio_PROXY_ENTITY_set;
*swig_RAY_get = *LibreDWGc::_dwg_object_entity_tio_RAY_get;
*swig_RAY_set = *LibreDWGc::_dwg_object_entity_tio_RAY_set;
*swig_REGION_get = *LibreDWGc::_dwg_object_entity_tio_REGION_get;
*swig_REGION_set = *LibreDWGc::_dwg_object_entity_tio_REGION_set;
*swig_SEQEND_get = *LibreDWGc::_dwg_object_entity_tio_SEQEND_get;
*swig_SEQEND_set = *LibreDWGc::_dwg_object_entity_tio_SEQEND_set;
*swig_SHAPE_get = *LibreDWGc::_dwg_object_entity_tio_SHAPE_get;
*swig_SHAPE_set = *LibreDWGc::_dwg_object_entity_tio_SHAPE_set;
*swig_SOLID_get = *LibreDWGc::_dwg_object_entity_tio_SOLID_get;
*swig_SOLID_set = *LibreDWGc::_dwg_object_entity_tio_SOLID_set;
*swig_SPLINE_get = *LibreDWGc::_dwg_object_entity_tio_SPLINE_get;
*swig_SPLINE_set = *LibreDWGc::_dwg_object_entity_tio_SPLINE_set;
*swig_TEXT_get = *LibreDWGc::_dwg_object_entity_tio_TEXT_get;
*swig_TEXT_set = *LibreDWGc::_dwg_object_entity_tio_TEXT_set;
*swig_TOLERANCE_get = *LibreDWGc::_dwg_object_entity_tio_TOLERANCE_get;
*swig_TOLERANCE_set = *LibreDWGc::_dwg_object_entity_tio_TOLERANCE_set;
*swig_TRACE_get = *LibreDWGc::_dwg_object_entity_tio_TRACE_get;
*swig_TRACE_set = *LibreDWGc::_dwg_object_entity_tio_TRACE_set;
*swig_UNKNOWN_ENT_get = *LibreDWGc::_dwg_object_entity_tio_UNKNOWN_ENT_get;
*swig_UNKNOWN_ENT_set = *LibreDWGc::_dwg_object_entity_tio_UNKNOWN_ENT_set;
*swig_VERTEX_2D_get = *LibreDWGc::_dwg_object_entity_tio_VERTEX_2D_get;
*swig_VERTEX_2D_set = *LibreDWGc::_dwg_object_entity_tio_VERTEX_2D_set;
*swig_VERTEX_3D_get = *LibreDWGc::_dwg_object_entity_tio_VERTEX_3D_get;
*swig_VERTEX_3D_set = *LibreDWGc::_dwg_object_entity_tio_VERTEX_3D_set;
*swig_VERTEX_MESH_get = *LibreDWGc::_dwg_object_entity_tio_VERTEX_MESH_get;
*swig_VERTEX_MESH_set = *LibreDWGc::_dwg_object_entity_tio_VERTEX_MESH_set;
*swig_VERTEX_PFACE_get = *LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_get;
*swig_VERTEX_PFACE_set = *LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_set;
*swig_VERTEX_PFACE_FACE_get = *LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_FACE_get;
*swig_VERTEX_PFACE_FACE_set = *LibreDWGc::_dwg_object_entity_tio_VERTEX_PFACE_FACE_set;
*swig_VIEWPORT_get = *LibreDWGc::_dwg_object_entity_tio_VIEWPORT_get;
*swig_VIEWPORT_set = *LibreDWGc::_dwg_object_entity_tio_VIEWPORT_set;
*swig_XLINE_get = *LibreDWGc::_dwg_object_entity_tio_XLINE_get;
*swig_XLINE_set = *LibreDWGc::_dwg_object_entity_tio_XLINE_set;
*swig__3DLINE_get = *LibreDWGc::_dwg_object_entity_tio__3DLINE_get;
*swig__3DLINE_set = *LibreDWGc::_dwg_object_entity_tio__3DLINE_set;
*swig_CAMERA_get = *LibreDWGc::_dwg_object_entity_tio_CAMERA_get;
*swig_CAMERA_set = *LibreDWGc::_dwg_object_entity_tio_CAMERA_set;
*swig_DGNUNDERLAY_get = *LibreDWGc::_dwg_object_entity_tio_DGNUNDERLAY_get;
*swig_DGNUNDERLAY_set = *LibreDWGc::_dwg_object_entity_tio_DGNUNDERLAY_set;
*swig_DWFUNDERLAY_get = *LibreDWGc::_dwg_object_entity_tio_DWFUNDERLAY_get;
*swig_DWFUNDERLAY_set = *LibreDWGc::_dwg_object_entity_tio_DWFUNDERLAY_set;
*swig_ENDREP_get = *LibreDWGc::_dwg_object_entity_tio_ENDREP_get;
*swig_ENDREP_set = *LibreDWGc::_dwg_object_entity_tio_ENDREP_set;
*swig_HATCH_get = *LibreDWGc::_dwg_object_entity_tio_HATCH_get;
*swig_HATCH_set = *LibreDWGc::_dwg_object_entity_tio_HATCH_set;
*swig_IMAGE_get = *LibreDWGc::_dwg_object_entity_tio_IMAGE_get;
*swig_IMAGE_set = *LibreDWGc::_dwg_object_entity_tio_IMAGE_set;
*swig_JUMP_get = *LibreDWGc::_dwg_object_entity_tio_JUMP_get;
*swig_JUMP_set = *LibreDWGc::_dwg_object_entity_tio_JUMP_set;
*swig_LIGHT_get = *LibreDWGc::_dwg_object_entity_tio_LIGHT_get;
*swig_LIGHT_set = *LibreDWGc::_dwg_object_entity_tio_LIGHT_set;
*swig_LWPOLYLINE_get = *LibreDWGc::_dwg_object_entity_tio_LWPOLYLINE_get;
*swig_LWPOLYLINE_set = *LibreDWGc::_dwg_object_entity_tio_LWPOLYLINE_set;
*swig_MESH_get = *LibreDWGc::_dwg_object_entity_tio_MESH_get;
*swig_MESH_set = *LibreDWGc::_dwg_object_entity_tio_MESH_set;
*swig_MULTILEADER_get = *LibreDWGc::_dwg_object_entity_tio_MULTILEADER_get;
*swig_MULTILEADER_set = *LibreDWGc::_dwg_object_entity_tio_MULTILEADER_set;
*swig_OLE2FRAME_get = *LibreDWGc::_dwg_object_entity_tio_OLE2FRAME_get;
*swig_OLE2FRAME_set = *LibreDWGc::_dwg_object_entity_tio_OLE2FRAME_set;
*swig_PDFUNDERLAY_get = *LibreDWGc::_dwg_object_entity_tio_PDFUNDERLAY_get;
*swig_PDFUNDERLAY_set = *LibreDWGc::_dwg_object_entity_tio_PDFUNDERLAY_set;
*swig_REPEAT_get = *LibreDWGc::_dwg_object_entity_tio_REPEAT_get;
*swig_REPEAT_set = *LibreDWGc::_dwg_object_entity_tio_REPEAT_set;
*swig_SECTIONOBJECT_get = *LibreDWGc::_dwg_object_entity_tio_SECTIONOBJECT_get;
*swig_SECTIONOBJECT_set = *LibreDWGc::_dwg_object_entity_tio_SECTIONOBJECT_set;
*swig_WIPEOUT_get = *LibreDWGc::_dwg_object_entity_tio_WIPEOUT_get;
*swig_WIPEOUT_set = *LibreDWGc::_dwg_object_entity_tio_WIPEOUT_set;
*swig_ARC_DIMENSION_get = *LibreDWGc::_dwg_object_entity_tio_ARC_DIMENSION_get;
*swig_ARC_DIMENSION_set = *LibreDWGc::_dwg_object_entity_tio_ARC_DIMENSION_set;
*swig_HELIX_get = *LibreDWGc::_dwg_object_entity_tio_HELIX_get;
*swig_HELIX_set = *LibreDWGc::_dwg_object_entity_tio_HELIX_set;
*swig_LARGE_RADIAL_DIMENSION_get = *LibreDWGc::_dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_get;
*swig_LARGE_RADIAL_DIMENSION_set = *LibreDWGc::_dwg_object_entity_tio_LARGE_RADIAL_DIMENSION_set;
*swig_LAYOUTPRINTCONFIG_get = *LibreDWGc::_dwg_object_entity_tio_LAYOUTPRINTCONFIG_get;
*swig_LAYOUTPRINTCONFIG_set = *LibreDWGc::_dwg_object_entity_tio_LAYOUTPRINTCONFIG_set;
*swig_PLANESURFACE_get = *LibreDWGc::_dwg_object_entity_tio_PLANESURFACE_get;
*swig_PLANESURFACE_set = *LibreDWGc::_dwg_object_entity_tio_PLANESURFACE_set;
*swig_POINTCLOUD_get = *LibreDWGc::_dwg_object_entity_tio_POINTCLOUD_get;
*swig_POINTCLOUD_set = *LibreDWGc::_dwg_object_entity_tio_POINTCLOUD_set;
*swig_POINTCLOUDEX_get = *LibreDWGc::_dwg_object_entity_tio_POINTCLOUDEX_get;
*swig_POINTCLOUDEX_set = *LibreDWGc::_dwg_object_entity_tio_POINTCLOUDEX_set;
*swig_ALIGNMENTPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_get;
*swig_ALIGNMENTPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_ALIGNMENTPARAMETERENTITY_set;
*swig_ARCALIGNEDTEXT_get = *LibreDWGc::_dwg_object_entity_tio_ARCALIGNEDTEXT_get;
*swig_ARCALIGNEDTEXT_set = *LibreDWGc::_dwg_object_entity_tio_ARCALIGNEDTEXT_set;
*swig_BASEPOINTPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_get;
*swig_BASEPOINTPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_BASEPOINTPARAMETERENTITY_set;
*swig_EXTRUDEDSURFACE_get = *LibreDWGc::_dwg_object_entity_tio_EXTRUDEDSURFACE_get;
*swig_EXTRUDEDSURFACE_set = *LibreDWGc::_dwg_object_entity_tio_EXTRUDEDSURFACE_set;
*swig_FLIPGRIPENTITY_get = *LibreDWGc::_dwg_object_entity_tio_FLIPGRIPENTITY_get;
*swig_FLIPGRIPENTITY_set = *LibreDWGc::_dwg_object_entity_tio_FLIPGRIPENTITY_set;
*swig_FLIPPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_FLIPPARAMETERENTITY_get;
*swig_FLIPPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_FLIPPARAMETERENTITY_set;
*swig_GEOPOSITIONMARKER_get = *LibreDWGc::_dwg_object_entity_tio_GEOPOSITIONMARKER_get;
*swig_GEOPOSITIONMARKER_set = *LibreDWGc::_dwg_object_entity_tio_GEOPOSITIONMARKER_set;
*swig_LINEARGRIPENTITY_get = *LibreDWGc::_dwg_object_entity_tio_LINEARGRIPENTITY_get;
*swig_LINEARGRIPENTITY_set = *LibreDWGc::_dwg_object_entity_tio_LINEARGRIPENTITY_set;
*swig_LINEARPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_LINEARPARAMETERENTITY_get;
*swig_LINEARPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_LINEARPARAMETERENTITY_set;
*swig_LOFTEDSURFACE_get = *LibreDWGc::_dwg_object_entity_tio_LOFTEDSURFACE_get;
*swig_LOFTEDSURFACE_set = *LibreDWGc::_dwg_object_entity_tio_LOFTEDSURFACE_set;
*swig_MPOLYGON_get = *LibreDWGc::_dwg_object_entity_tio_MPOLYGON_get;
*swig_MPOLYGON_set = *LibreDWGc::_dwg_object_entity_tio_MPOLYGON_set;
*swig_NAVISWORKSMODEL_get = *LibreDWGc::_dwg_object_entity_tio_NAVISWORKSMODEL_get;
*swig_NAVISWORKSMODEL_set = *LibreDWGc::_dwg_object_entity_tio_NAVISWORKSMODEL_set;
*swig_NURBSURFACE_get = *LibreDWGc::_dwg_object_entity_tio_NURBSURFACE_get;
*swig_NURBSURFACE_set = *LibreDWGc::_dwg_object_entity_tio_NURBSURFACE_set;
*swig_POINTPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_POINTPARAMETERENTITY_get;
*swig_POINTPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_POINTPARAMETERENTITY_set;
*swig_POLARGRIPENTITY_get = *LibreDWGc::_dwg_object_entity_tio_POLARGRIPENTITY_get;
*swig_POLARGRIPENTITY_set = *LibreDWGc::_dwg_object_entity_tio_POLARGRIPENTITY_set;
*swig_REVOLVEDSURFACE_get = *LibreDWGc::_dwg_object_entity_tio_REVOLVEDSURFACE_get;
*swig_REVOLVEDSURFACE_set = *LibreDWGc::_dwg_object_entity_tio_REVOLVEDSURFACE_set;
*swig_ROTATIONGRIPENTITY_get = *LibreDWGc::_dwg_object_entity_tio_ROTATIONGRIPENTITY_get;
*swig_ROTATIONGRIPENTITY_set = *LibreDWGc::_dwg_object_entity_tio_ROTATIONGRIPENTITY_set;
*swig_ROTATIONPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_ROTATIONPARAMETERENTITY_get;
*swig_ROTATIONPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_ROTATIONPARAMETERENTITY_set;
*swig_RTEXT_get = *LibreDWGc::_dwg_object_entity_tio_RTEXT_get;
*swig_RTEXT_set = *LibreDWGc::_dwg_object_entity_tio_RTEXT_set;
*swig_SWEPTSURFACE_get = *LibreDWGc::_dwg_object_entity_tio_SWEPTSURFACE_get;
*swig_SWEPTSURFACE_set = *LibreDWGc::_dwg_object_entity_tio_SWEPTSURFACE_set;
*swig_TABLE_get = *LibreDWGc::_dwg_object_entity_tio_TABLE_get;
*swig_TABLE_set = *LibreDWGc::_dwg_object_entity_tio_TABLE_set;
*swig_VISIBILITYGRIPENTITY_get = *LibreDWGc::_dwg_object_entity_tio_VISIBILITYGRIPENTITY_get;
*swig_VISIBILITYGRIPENTITY_set = *LibreDWGc::_dwg_object_entity_tio_VISIBILITYGRIPENTITY_set;
*swig_VISIBILITYPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_get;
*swig_VISIBILITYPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_VISIBILITYPARAMETERENTITY_set;
*swig_XYGRIPENTITY_get = *LibreDWGc::_dwg_object_entity_tio_XYGRIPENTITY_get;
*swig_XYGRIPENTITY_set = *LibreDWGc::_dwg_object_entity_tio_XYGRIPENTITY_set;
*swig_XYPARAMETERENTITY_get = *LibreDWGc::_dwg_object_entity_tio_XYPARAMETERENTITY_get;
*swig_XYPARAMETERENTITY_set = *LibreDWGc::_dwg_object_entity_tio_XYPARAMETERENTITY_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_object_entity_tio(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_object_entity_tio($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object_Object ##############

package LibreDWG::Dwg_Object_Object;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_objid_get = *LibreDWGc::Dwg_Object_Object_objid_get;
*swig_objid_set = *LibreDWGc::Dwg_Object_Object_objid_set;
*swig_tio_get = *LibreDWGc::Dwg_Object_Object_tio_get;
*swig_tio_set = *LibreDWGc::Dwg_Object_Object_tio_set;
*swig_dwg_get = *LibreDWGc::Dwg_Object_Object_dwg_get;
*swig_dwg_set = *LibreDWGc::Dwg_Object_Object_dwg_set;
*swig_num_eed_get = *LibreDWGc::Dwg_Object_Object_num_eed_get;
*swig_num_eed_set = *LibreDWGc::Dwg_Object_Object_num_eed_set;
*swig_eed_get = *LibreDWGc::Dwg_Object_Object_eed_get;
*swig_eed_set = *LibreDWGc::Dwg_Object_Object_eed_set;
*swig_ownerhandle_get = *LibreDWGc::Dwg_Object_Object_ownerhandle_get;
*swig_ownerhandle_set = *LibreDWGc::Dwg_Object_Object_ownerhandle_set;
*swig_num_reactors_get = *LibreDWGc::Dwg_Object_Object_num_reactors_get;
*swig_num_reactors_set = *LibreDWGc::Dwg_Object_Object_num_reactors_set;
*swig_reactors_get = *LibreDWGc::Dwg_Object_Object_reactors_get;
*swig_reactors_set = *LibreDWGc::Dwg_Object_Object_reactors_set;
*swig_xdicobjhandle_get = *LibreDWGc::Dwg_Object_Object_xdicobjhandle_get;
*swig_xdicobjhandle_set = *LibreDWGc::Dwg_Object_Object_xdicobjhandle_set;
*swig_is_xdic_missing_get = *LibreDWGc::Dwg_Object_Object_is_xdic_missing_get;
*swig_is_xdic_missing_set = *LibreDWGc::Dwg_Object_Object_is_xdic_missing_set;
*swig_has_ds_data_get = *LibreDWGc::Dwg_Object_Object_has_ds_data_get;
*swig_has_ds_data_set = *LibreDWGc::Dwg_Object_Object_has_ds_data_set;
*swig_handleref_get = *LibreDWGc::Dwg_Object_Object_handleref_get;
*swig_handleref_set = *LibreDWGc::Dwg_Object_Object_handleref_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object_Object(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object_Object($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_object_object_tio ##############

package LibreDWG::_dwg_object_object_tio;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_APPID_get = *LibreDWGc::_dwg_object_object_tio_APPID_get;
*swig_APPID_set = *LibreDWGc::_dwg_object_object_tio_APPID_set;
*swig_APPID_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_APPID_CONTROL_get;
*swig_APPID_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_APPID_CONTROL_set;
*swig_BLOCK_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_BLOCK_CONTROL_get;
*swig_BLOCK_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_BLOCK_CONTROL_set;
*swig_BLOCK_HEADER_get = *LibreDWGc::_dwg_object_object_tio_BLOCK_HEADER_get;
*swig_BLOCK_HEADER_set = *LibreDWGc::_dwg_object_object_tio_BLOCK_HEADER_set;
*swig_DICTIONARY_get = *LibreDWGc::_dwg_object_object_tio_DICTIONARY_get;
*swig_DICTIONARY_set = *LibreDWGc::_dwg_object_object_tio_DICTIONARY_set;
*swig_DIMSTYLE_get = *LibreDWGc::_dwg_object_object_tio_DIMSTYLE_get;
*swig_DIMSTYLE_set = *LibreDWGc::_dwg_object_object_tio_DIMSTYLE_set;
*swig_DIMSTYLE_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_DIMSTYLE_CONTROL_get;
*swig_DIMSTYLE_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_DIMSTYLE_CONTROL_set;
*swig_DUMMY_get = *LibreDWGc::_dwg_object_object_tio_DUMMY_get;
*swig_DUMMY_set = *LibreDWGc::_dwg_object_object_tio_DUMMY_set;
*swig_LAYER_get = *LibreDWGc::_dwg_object_object_tio_LAYER_get;
*swig_LAYER_set = *LibreDWGc::_dwg_object_object_tio_LAYER_set;
*swig_LAYER_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_LAYER_CONTROL_get;
*swig_LAYER_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_LAYER_CONTROL_set;
*swig_LONG_TRANSACTION_get = *LibreDWGc::_dwg_object_object_tio_LONG_TRANSACTION_get;
*swig_LONG_TRANSACTION_set = *LibreDWGc::_dwg_object_object_tio_LONG_TRANSACTION_set;
*swig_LTYPE_get = *LibreDWGc::_dwg_object_object_tio_LTYPE_get;
*swig_LTYPE_set = *LibreDWGc::_dwg_object_object_tio_LTYPE_set;
*swig_LTYPE_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_LTYPE_CONTROL_get;
*swig_LTYPE_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_LTYPE_CONTROL_set;
*swig_MLINESTYLE_get = *LibreDWGc::_dwg_object_object_tio_MLINESTYLE_get;
*swig_MLINESTYLE_set = *LibreDWGc::_dwg_object_object_tio_MLINESTYLE_set;
*swig_STYLE_get = *LibreDWGc::_dwg_object_object_tio_STYLE_get;
*swig_STYLE_set = *LibreDWGc::_dwg_object_object_tio_STYLE_set;
*swig_STYLE_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_STYLE_CONTROL_get;
*swig_STYLE_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_STYLE_CONTROL_set;
*swig_UCS_get = *LibreDWGc::_dwg_object_object_tio_UCS_get;
*swig_UCS_set = *LibreDWGc::_dwg_object_object_tio_UCS_set;
*swig_UCS_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_UCS_CONTROL_get;
*swig_UCS_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_UCS_CONTROL_set;
*swig_UNKNOWN_OBJ_get = *LibreDWGc::_dwg_object_object_tio_UNKNOWN_OBJ_get;
*swig_UNKNOWN_OBJ_set = *LibreDWGc::_dwg_object_object_tio_UNKNOWN_OBJ_set;
*swig_VIEW_get = *LibreDWGc::_dwg_object_object_tio_VIEW_get;
*swig_VIEW_set = *LibreDWGc::_dwg_object_object_tio_VIEW_set;
*swig_VIEW_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_VIEW_CONTROL_get;
*swig_VIEW_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_VIEW_CONTROL_set;
*swig_VPORT_get = *LibreDWGc::_dwg_object_object_tio_VPORT_get;
*swig_VPORT_set = *LibreDWGc::_dwg_object_object_tio_VPORT_set;
*swig_VPORT_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_VPORT_CONTROL_get;
*swig_VPORT_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_VPORT_CONTROL_set;
*swig_VX_CONTROL_get = *LibreDWGc::_dwg_object_object_tio_VX_CONTROL_get;
*swig_VX_CONTROL_set = *LibreDWGc::_dwg_object_object_tio_VX_CONTROL_set;
*swig_VX_TABLE_RECORD_get = *LibreDWGc::_dwg_object_object_tio_VX_TABLE_RECORD_get;
*swig_VX_TABLE_RECORD_set = *LibreDWGc::_dwg_object_object_tio_VX_TABLE_RECORD_set;
*swig_ACSH_BOOLEAN_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_BOOLEAN_CLASS_get;
*swig_ACSH_BOOLEAN_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_BOOLEAN_CLASS_set;
*swig_ACSH_BOX_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_BOX_CLASS_get;
*swig_ACSH_BOX_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_BOX_CLASS_set;
*swig_ACSH_CONE_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_CONE_CLASS_get;
*swig_ACSH_CONE_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_CONE_CLASS_set;
*swig_ACSH_CYLINDER_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_CYLINDER_CLASS_get;
*swig_ACSH_CYLINDER_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_CYLINDER_CLASS_set;
*swig_ACSH_FILLET_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_FILLET_CLASS_get;
*swig_ACSH_FILLET_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_FILLET_CLASS_set;
*swig_ACSH_HISTORY_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_HISTORY_CLASS_get;
*swig_ACSH_HISTORY_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_HISTORY_CLASS_set;
*swig_ACSH_SPHERE_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_SPHERE_CLASS_get;
*swig_ACSH_SPHERE_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_SPHERE_CLASS_set;
*swig_ACSH_TORUS_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_TORUS_CLASS_get;
*swig_ACSH_TORUS_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_TORUS_CLASS_set;
*swig_ACSH_WEDGE_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_WEDGE_CLASS_get;
*swig_ACSH_WEDGE_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_WEDGE_CLASS_set;
*swig_ASSOCGEOMDEPENDENCY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCGEOMDEPENDENCY_get;
*swig_ASSOCGEOMDEPENDENCY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCGEOMDEPENDENCY_set;
*swig_ASSOCNETWORK_get = *LibreDWGc::_dwg_object_object_tio_ASSOCNETWORK_get;
*swig_ASSOCNETWORK_set = *LibreDWGc::_dwg_object_object_tio_ASSOCNETWORK_set;
*swig_BLOCKALIGNMENTGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTGRIP_get;
*swig_BLOCKALIGNMENTGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTGRIP_set;
*swig_BLOCKALIGNMENTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_get;
*swig_BLOCKALIGNMENTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKALIGNMENTPARAMETER_set;
*swig_BLOCKBASEPOINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_get;
*swig_BLOCKBASEPOINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKBASEPOINTPARAMETER_set;
*swig_BLOCKFLIPACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKFLIPACTION_get;
*swig_BLOCKFLIPACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKFLIPACTION_set;
*swig_BLOCKFLIPGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKFLIPGRIP_get;
*swig_BLOCKFLIPGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKFLIPGRIP_set;
*swig_BLOCKFLIPPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKFLIPPARAMETER_get;
*swig_BLOCKFLIPPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKFLIPPARAMETER_set;
*swig_BLOCKGRIPLOCATIONCOMPONENT_get = *LibreDWGc::_dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_get;
*swig_BLOCKGRIPLOCATIONCOMPONENT_set = *LibreDWGc::_dwg_object_object_tio_BLOCKGRIPLOCATIONCOMPONENT_set;
*swig_BLOCKLINEARGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKLINEARGRIP_get;
*swig_BLOCKLINEARGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKLINEARGRIP_set;
*swig_BLOCKLOOKUPGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPGRIP_get;
*swig_BLOCKLOOKUPGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPGRIP_set;
*swig_BLOCKMOVEACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKMOVEACTION_get;
*swig_BLOCKMOVEACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKMOVEACTION_set;
*swig_BLOCKROTATEACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKROTATEACTION_get;
*swig_BLOCKROTATEACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKROTATEACTION_set;
*swig_BLOCKROTATIONGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONGRIP_get;
*swig_BLOCKROTATIONGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONGRIP_set;
*swig_BLOCKSCALEACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKSCALEACTION_get;
*swig_BLOCKSCALEACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKSCALEACTION_set;
*swig_BLOCKVISIBILITYGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYGRIP_get;
*swig_BLOCKVISIBILITYGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYGRIP_set;
*swig_CELLSTYLEMAP_get = *LibreDWGc::_dwg_object_object_tio_CELLSTYLEMAP_get;
*swig_CELLSTYLEMAP_set = *LibreDWGc::_dwg_object_object_tio_CELLSTYLEMAP_set;
*swig_DETAILVIEWSTYLE_get = *LibreDWGc::_dwg_object_object_tio_DETAILVIEWSTYLE_get;
*swig_DETAILVIEWSTYLE_set = *LibreDWGc::_dwg_object_object_tio_DETAILVIEWSTYLE_set;
*swig_DICTIONARYVAR_get = *LibreDWGc::_dwg_object_object_tio_DICTIONARYVAR_get;
*swig_DICTIONARYVAR_set = *LibreDWGc::_dwg_object_object_tio_DICTIONARYVAR_set;
*swig_DICTIONARYWDFLT_get = *LibreDWGc::_dwg_object_object_tio_DICTIONARYWDFLT_get;
*swig_DICTIONARYWDFLT_set = *LibreDWGc::_dwg_object_object_tio_DICTIONARYWDFLT_set;
*swig_DYNAMICBLOCKPURGEPREVENTER_get = *LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_get;
*swig_DYNAMICBLOCKPURGEPREVENTER_set = *LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPURGEPREVENTER_set;
*swig_FIELD_get = *LibreDWGc::_dwg_object_object_tio_FIELD_get;
*swig_FIELD_set = *LibreDWGc::_dwg_object_object_tio_FIELD_set;
*swig_FIELDLIST_get = *LibreDWGc::_dwg_object_object_tio_FIELDLIST_get;
*swig_FIELDLIST_set = *LibreDWGc::_dwg_object_object_tio_FIELDLIST_set;
*swig_GEODATA_get = *LibreDWGc::_dwg_object_object_tio_GEODATA_get;
*swig_GEODATA_set = *LibreDWGc::_dwg_object_object_tio_GEODATA_set;
*swig_GROUP_get = *LibreDWGc::_dwg_object_object_tio_GROUP_get;
*swig_GROUP_set = *LibreDWGc::_dwg_object_object_tio_GROUP_set;
*swig_IDBUFFER_get = *LibreDWGc::_dwg_object_object_tio_IDBUFFER_get;
*swig_IDBUFFER_set = *LibreDWGc::_dwg_object_object_tio_IDBUFFER_set;
*swig_IMAGEDEF_get = *LibreDWGc::_dwg_object_object_tio_IMAGEDEF_get;
*swig_IMAGEDEF_set = *LibreDWGc::_dwg_object_object_tio_IMAGEDEF_set;
*swig_IMAGEDEF_REACTOR_get = *LibreDWGc::_dwg_object_object_tio_IMAGEDEF_REACTOR_get;
*swig_IMAGEDEF_REACTOR_set = *LibreDWGc::_dwg_object_object_tio_IMAGEDEF_REACTOR_set;
*swig_INDEX_get = *LibreDWGc::_dwg_object_object_tio_INDEX_get;
*swig_INDEX_set = *LibreDWGc::_dwg_object_object_tio_INDEX_set;
*swig_LAYERFILTER_get = *LibreDWGc::_dwg_object_object_tio_LAYERFILTER_get;
*swig_LAYERFILTER_set = *LibreDWGc::_dwg_object_object_tio_LAYERFILTER_set;
*swig_LAYER_INDEX_get = *LibreDWGc::_dwg_object_object_tio_LAYER_INDEX_get;
*swig_LAYER_INDEX_set = *LibreDWGc::_dwg_object_object_tio_LAYER_INDEX_set;
*swig_LAYOUT_get = *LibreDWGc::_dwg_object_object_tio_LAYOUT_get;
*swig_LAYOUT_set = *LibreDWGc::_dwg_object_object_tio_LAYOUT_set;
*swig_PLACEHOLDER_get = *LibreDWGc::_dwg_object_object_tio_PLACEHOLDER_get;
*swig_PLACEHOLDER_set = *LibreDWGc::_dwg_object_object_tio_PLACEHOLDER_set;
*swig_PLOTSETTINGS_get = *LibreDWGc::_dwg_object_object_tio_PLOTSETTINGS_get;
*swig_PLOTSETTINGS_set = *LibreDWGc::_dwg_object_object_tio_PLOTSETTINGS_set;
*swig_RASTERVARIABLES_get = *LibreDWGc::_dwg_object_object_tio_RASTERVARIABLES_get;
*swig_RASTERVARIABLES_set = *LibreDWGc::_dwg_object_object_tio_RASTERVARIABLES_set;
*swig_RENDERENVIRONMENT_get = *LibreDWGc::_dwg_object_object_tio_RENDERENVIRONMENT_get;
*swig_RENDERENVIRONMENT_set = *LibreDWGc::_dwg_object_object_tio_RENDERENVIRONMENT_set;
*swig_SCALE_get = *LibreDWGc::_dwg_object_object_tio_SCALE_get;
*swig_SCALE_set = *LibreDWGc::_dwg_object_object_tio_SCALE_set;
*swig_SECTIONVIEWSTYLE_get = *LibreDWGc::_dwg_object_object_tio_SECTIONVIEWSTYLE_get;
*swig_SECTIONVIEWSTYLE_set = *LibreDWGc::_dwg_object_object_tio_SECTIONVIEWSTYLE_set;
*swig_SECTION_MANAGER_get = *LibreDWGc::_dwg_object_object_tio_SECTION_MANAGER_get;
*swig_SECTION_MANAGER_set = *LibreDWGc::_dwg_object_object_tio_SECTION_MANAGER_set;
*swig_SORTENTSTABLE_get = *LibreDWGc::_dwg_object_object_tio_SORTENTSTABLE_get;
*swig_SORTENTSTABLE_set = *LibreDWGc::_dwg_object_object_tio_SORTENTSTABLE_set;
*swig_SPATIAL_FILTER_get = *LibreDWGc::_dwg_object_object_tio_SPATIAL_FILTER_get;
*swig_SPATIAL_FILTER_set = *LibreDWGc::_dwg_object_object_tio_SPATIAL_FILTER_set;
*swig_SUN_get = *LibreDWGc::_dwg_object_object_tio_SUN_get;
*swig_SUN_set = *LibreDWGc::_dwg_object_object_tio_SUN_set;
*swig_TABLEGEOMETRY_get = *LibreDWGc::_dwg_object_object_tio_TABLEGEOMETRY_get;
*swig_TABLEGEOMETRY_set = *LibreDWGc::_dwg_object_object_tio_TABLEGEOMETRY_set;
*swig_VBA_PROJECT_get = *LibreDWGc::_dwg_object_object_tio_VBA_PROJECT_get;
*swig_VBA_PROJECT_set = *LibreDWGc::_dwg_object_object_tio_VBA_PROJECT_set;
*swig_VISUALSTYLE_get = *LibreDWGc::_dwg_object_object_tio_VISUALSTYLE_get;
*swig_VISUALSTYLE_set = *LibreDWGc::_dwg_object_object_tio_VISUALSTYLE_set;
*swig_WIPEOUTVARIABLES_get = *LibreDWGc::_dwg_object_object_tio_WIPEOUTVARIABLES_get;
*swig_WIPEOUTVARIABLES_set = *LibreDWGc::_dwg_object_object_tio_WIPEOUTVARIABLES_set;
*swig_XRECORD_get = *LibreDWGc::_dwg_object_object_tio_XRECORD_get;
*swig_XRECORD_set = *LibreDWGc::_dwg_object_object_tio_XRECORD_set;
*swig_PDFDEFINITION_get = *LibreDWGc::_dwg_object_object_tio_PDFDEFINITION_get;
*swig_PDFDEFINITION_set = *LibreDWGc::_dwg_object_object_tio_PDFDEFINITION_set;
*swig_DGNDEFINITION_get = *LibreDWGc::_dwg_object_object_tio_DGNDEFINITION_get;
*swig_DGNDEFINITION_set = *LibreDWGc::_dwg_object_object_tio_DGNDEFINITION_set;
*swig_DWFDEFINITION_get = *LibreDWGc::_dwg_object_object_tio_DWFDEFINITION_get;
*swig_DWFDEFINITION_set = *LibreDWGc::_dwg_object_object_tio_DWFDEFINITION_set;
*swig_ACSH_BREP_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_BREP_CLASS_get;
*swig_ACSH_BREP_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_BREP_CLASS_set;
*swig_ACSH_CHAMFER_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_CHAMFER_CLASS_get;
*swig_ACSH_CHAMFER_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_CHAMFER_CLASS_set;
*swig_ACSH_PYRAMID_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_PYRAMID_CLASS_get;
*swig_ACSH_PYRAMID_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_PYRAMID_CLASS_set;
*swig_ALDIMOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_get;
*swig_ALDIMOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_ALDIMOBJECTCONTEXTDATA_set;
*swig_ASSOC2DCONSTRAINTGROUP_get = *LibreDWGc::_dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_get;
*swig_ASSOC2DCONSTRAINTGROUP_set = *LibreDWGc::_dwg_object_object_tio_ASSOC2DCONSTRAINTGROUP_set;
*swig_ASSOCACTION_get = *LibreDWGc::_dwg_object_object_tio_ASSOCACTION_get;
*swig_ASSOCACTION_set = *LibreDWGc::_dwg_object_object_tio_ASSOCACTION_set;
*swig_ASSOCACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCACTIONPARAM_get;
*swig_ASSOCACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCACTIONPARAM_set;
*swig_ASSOCARRAYACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYACTIONBODY_get;
*swig_ASSOCARRAYACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYACTIONBODY_set;
*swig_ASSOCASMBODYACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_get;
*swig_ASSOCASMBODYACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCASMBODYACTIONPARAM_set;
*swig_ASSOCBLENDSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_get;
*swig_ASSOCBLENDSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCBLENDSURFACEACTIONBODY_set;
*swig_ASSOCCOMPOUNDACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_get;
*swig_ASSOCCOMPOUNDACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCCOMPOUNDACTIONPARAM_set;
*swig_ASSOCDEPENDENCY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCDEPENDENCY_get;
*swig_ASSOCDEPENDENCY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCDEPENDENCY_set;
*swig_ASSOCDIMDEPENDENCYBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_get;
*swig_ASSOCDIMDEPENDENCYBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCDIMDEPENDENCYBODY_set;
*swig_ASSOCEXTENDSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_get;
*swig_ASSOCEXTENDSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCEXTENDSURFACEACTIONBODY_set;
*swig_ASSOCEXTRUDEDSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_get;
*swig_ASSOCEXTRUDEDSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCEXTRUDEDSURFACEACTIONBODY_set;
*swig_ASSOCFACEACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCFACEACTIONPARAM_get;
*swig_ASSOCFACEACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCFACEACTIONPARAM_set;
*swig_ASSOCFILLETSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_get;
*swig_ASSOCFILLETSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCFILLETSURFACEACTIONBODY_set;
*swig_ASSOCLOFTEDSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_get;
*swig_ASSOCLOFTEDSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCLOFTEDSURFACEACTIONBODY_set;
*swig_ASSOCNETWORKSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_get;
*swig_ASSOCNETWORKSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCNETWORKSURFACEACTIONBODY_set;
*swig_ASSOCOBJECTACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_get;
*swig_ASSOCOBJECTACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCOBJECTACTIONPARAM_set;
*swig_ASSOCOFFSETSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_get;
*swig_ASSOCOFFSETSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCOFFSETSURFACEACTIONBODY_set;
*swig_ASSOCOSNAPPOINTREFACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_get;
*swig_ASSOCOSNAPPOINTREFACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCOSNAPPOINTREFACTIONPARAM_set;
*swig_ASSOCPATCHSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_get;
*swig_ASSOCPATCHSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCPATCHSURFACEACTIONBODY_set;
*swig_ASSOCPATHACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCPATHACTIONPARAM_get;
*swig_ASSOCPATHACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCPATHACTIONPARAM_set;
*swig_ASSOCPLANESURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_get;
*swig_ASSOCPLANESURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCPLANESURFACEACTIONBODY_set;
*swig_ASSOCPOINTREFACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_get;
*swig_ASSOCPOINTREFACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCPOINTREFACTIONPARAM_set;
*swig_ASSOCREVOLVEDSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_get;
*swig_ASSOCREVOLVEDSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCREVOLVEDSURFACEACTIONBODY_set;
*swig_ASSOCTRIMSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_get;
*swig_ASSOCTRIMSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCTRIMSURFACEACTIONBODY_set;
*swig_ASSOCVALUEDEPENDENCY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCVALUEDEPENDENCY_get;
*swig_ASSOCVALUEDEPENDENCY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCVALUEDEPENDENCY_set;
*swig_ASSOCVARIABLE_get = *LibreDWGc::_dwg_object_object_tio_ASSOCVARIABLE_get;
*swig_ASSOCVARIABLE_set = *LibreDWGc::_dwg_object_object_tio_ASSOCVARIABLE_set;
*swig_ASSOCVERTEXACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_get;
*swig_ASSOCVERTEXACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCVERTEXACTIONPARAM_set;
*swig_BLKREFOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_get;
*swig_BLKREFOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_BLKREFOBJECTCONTEXTDATA_set;
*swig_BLOCKALIGNEDCONSTRAINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_get;
*swig_BLOCKALIGNEDCONSTRAINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKALIGNEDCONSTRAINTPARAMETER_set;
*swig_BLOCKANGULARCONSTRAINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_get;
*swig_BLOCKANGULARCONSTRAINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKANGULARCONSTRAINTPARAMETER_set;
*swig_BLOCKARRAYACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKARRAYACTION_get;
*swig_BLOCKARRAYACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKARRAYACTION_set;
*swig_BLOCKDIAMETRICCONSTRAINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_get;
*swig_BLOCKDIAMETRICCONSTRAINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKDIAMETRICCONSTRAINTPARAMETER_set;
*swig_BLOCKHORIZONTALCONSTRAINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_get;
*swig_BLOCKHORIZONTALCONSTRAINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKHORIZONTALCONSTRAINTPARAMETER_set;
*swig_BLOCKLINEARCONSTRAINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_get;
*swig_BLOCKLINEARCONSTRAINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKLINEARCONSTRAINTPARAMETER_set;
*swig_BLOCKLINEARPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKLINEARPARAMETER_get;
*swig_BLOCKLINEARPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKLINEARPARAMETER_set;
*swig_BLOCKLOOKUPACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPACTION_get;
*swig_BLOCKLOOKUPACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPACTION_set;
*swig_BLOCKLOOKUPPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPPARAMETER_get;
*swig_BLOCKLOOKUPPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKLOOKUPPARAMETER_set;
*swig_BLOCKPARAMDEPENDENCYBODY_get = *LibreDWGc::_dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_get;
*swig_BLOCKPARAMDEPENDENCYBODY_set = *LibreDWGc::_dwg_object_object_tio_BLOCKPARAMDEPENDENCYBODY_set;
*swig_BLOCKPOINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKPOINTPARAMETER_get;
*swig_BLOCKPOINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKPOINTPARAMETER_set;
*swig_BLOCKPOLARGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKPOLARGRIP_get;
*swig_BLOCKPOLARGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKPOLARGRIP_set;
*swig_BLOCKPOLARPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKPOLARPARAMETER_get;
*swig_BLOCKPOLARPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKPOLARPARAMETER_set;
*swig_BLOCKPOLARSTRETCHACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_get;
*swig_BLOCKPOLARSTRETCHACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKPOLARSTRETCHACTION_set;
*swig_BLOCKRADIALCONSTRAINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_get;
*swig_BLOCKRADIALCONSTRAINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKRADIALCONSTRAINTPARAMETER_set;
*swig_BLOCKREPRESENTATION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKREPRESENTATION_get;
*swig_BLOCKREPRESENTATION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKREPRESENTATION_set;
*swig_BLOCKROTATIONPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONPARAMETER_get;
*swig_BLOCKROTATIONPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKROTATIONPARAMETER_set;
*swig_BLOCKSTRETCHACTION_get = *LibreDWGc::_dwg_object_object_tio_BLOCKSTRETCHACTION_get;
*swig_BLOCKSTRETCHACTION_set = *LibreDWGc::_dwg_object_object_tio_BLOCKSTRETCHACTION_set;
*swig_BLOCKUSERPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKUSERPARAMETER_get;
*swig_BLOCKUSERPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKUSERPARAMETER_set;
*swig_BLOCKVERTICALCONSTRAINTPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_get;
*swig_BLOCKVERTICALCONSTRAINTPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKVERTICALCONSTRAINTPARAMETER_set;
*swig_BLOCKVISIBILITYPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_get;
*swig_BLOCKVISIBILITYPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKVISIBILITYPARAMETER_set;
*swig_BLOCKXYGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKXYGRIP_get;
*swig_BLOCKXYGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKXYGRIP_set;
*swig_BLOCKXYPARAMETER_get = *LibreDWGc::_dwg_object_object_tio_BLOCKXYPARAMETER_get;
*swig_BLOCKXYPARAMETER_set = *LibreDWGc::_dwg_object_object_tio_BLOCKXYPARAMETER_set;
*swig_DATALINK_get = *LibreDWGc::_dwg_object_object_tio_DATALINK_get;
*swig_DATALINK_set = *LibreDWGc::_dwg_object_object_tio_DATALINK_set;
*swig_DBCOLOR_get = *LibreDWGc::_dwg_object_object_tio_DBCOLOR_get;
*swig_DBCOLOR_set = *LibreDWGc::_dwg_object_object_tio_DBCOLOR_set;
*swig_EVALUATION_GRAPH_get = *LibreDWGc::_dwg_object_object_tio_EVALUATION_GRAPH_get;
*swig_EVALUATION_GRAPH_set = *LibreDWGc::_dwg_object_object_tio_EVALUATION_GRAPH_set;
*swig_FCFOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_FCFOBJECTCONTEXTDATA_get;
*swig_FCFOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_FCFOBJECTCONTEXTDATA_set;
*swig_GRADIENT_BACKGROUND_get = *LibreDWGc::_dwg_object_object_tio_GRADIENT_BACKGROUND_get;
*swig_GRADIENT_BACKGROUND_set = *LibreDWGc::_dwg_object_object_tio_GRADIENT_BACKGROUND_set;
*swig_GROUND_PLANE_BACKGROUND_get = *LibreDWGc::_dwg_object_object_tio_GROUND_PLANE_BACKGROUND_get;
*swig_GROUND_PLANE_BACKGROUND_set = *LibreDWGc::_dwg_object_object_tio_GROUND_PLANE_BACKGROUND_set;
*swig_IBL_BACKGROUND_get = *LibreDWGc::_dwg_object_object_tio_IBL_BACKGROUND_get;
*swig_IBL_BACKGROUND_set = *LibreDWGc::_dwg_object_object_tio_IBL_BACKGROUND_set;
*swig_IMAGE_BACKGROUND_get = *LibreDWGc::_dwg_object_object_tio_IMAGE_BACKGROUND_get;
*swig_IMAGE_BACKGROUND_set = *LibreDWGc::_dwg_object_object_tio_IMAGE_BACKGROUND_set;
*swig_LEADEROBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_get;
*swig_LEADEROBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_LEADEROBJECTCONTEXTDATA_set;
*swig_LIGHTLIST_get = *LibreDWGc::_dwg_object_object_tio_LIGHTLIST_get;
*swig_LIGHTLIST_set = *LibreDWGc::_dwg_object_object_tio_LIGHTLIST_set;
*swig_MATERIAL_get = *LibreDWGc::_dwg_object_object_tio_MATERIAL_get;
*swig_MATERIAL_set = *LibreDWGc::_dwg_object_object_tio_MATERIAL_set;
*swig_MENTALRAYRENDERSETTINGS_get = *LibreDWGc::_dwg_object_object_tio_MENTALRAYRENDERSETTINGS_get;
*swig_MENTALRAYRENDERSETTINGS_set = *LibreDWGc::_dwg_object_object_tio_MENTALRAYRENDERSETTINGS_set;
*swig_MLEADERSTYLE_get = *LibreDWGc::_dwg_object_object_tio_MLEADERSTYLE_get;
*swig_MLEADERSTYLE_set = *LibreDWGc::_dwg_object_object_tio_MLEADERSTYLE_set;
*swig_MTEXTOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_get;
*swig_MTEXTOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_MTEXTOBJECTCONTEXTDATA_set;
*swig_OBJECT_PTR_get = *LibreDWGc::_dwg_object_object_tio_OBJECT_PTR_get;
*swig_OBJECT_PTR_set = *LibreDWGc::_dwg_object_object_tio_OBJECT_PTR_set;
*swig_PARTIAL_VIEWING_INDEX_get = *LibreDWGc::_dwg_object_object_tio_PARTIAL_VIEWING_INDEX_get;
*swig_PARTIAL_VIEWING_INDEX_set = *LibreDWGc::_dwg_object_object_tio_PARTIAL_VIEWING_INDEX_set;
*swig_POINTCLOUDCOLORMAP_get = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDCOLORMAP_get;
*swig_POINTCLOUDCOLORMAP_set = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDCOLORMAP_set;
*swig_POINTCLOUDDEF_get = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_get;
*swig_POINTCLOUDDEF_set = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_set;
*swig_POINTCLOUDDEFEX_get = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEFEX_get;
*swig_POINTCLOUDDEFEX_set = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEFEX_set;
*swig_POINTCLOUDDEF_REACTOR_get = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_get;
*swig_POINTCLOUDDEF_REACTOR_set = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_set;
*swig_POINTCLOUDDEF_REACTOR_EX_get = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_get;
*swig_POINTCLOUDDEF_REACTOR_EX_set = *LibreDWGc::_dwg_object_object_tio_POINTCLOUDDEF_REACTOR_EX_set;
*swig_PROXY_OBJECT_get = *LibreDWGc::_dwg_object_object_tio_PROXY_OBJECT_get;
*swig_PROXY_OBJECT_set = *LibreDWGc::_dwg_object_object_tio_PROXY_OBJECT_set;
*swig_RAPIDRTRENDERSETTINGS_get = *LibreDWGc::_dwg_object_object_tio_RAPIDRTRENDERSETTINGS_get;
*swig_RAPIDRTRENDERSETTINGS_set = *LibreDWGc::_dwg_object_object_tio_RAPIDRTRENDERSETTINGS_set;
*swig_RENDERENTRY_get = *LibreDWGc::_dwg_object_object_tio_RENDERENTRY_get;
*swig_RENDERENTRY_set = *LibreDWGc::_dwg_object_object_tio_RENDERENTRY_set;
*swig_RENDERGLOBAL_get = *LibreDWGc::_dwg_object_object_tio_RENDERGLOBAL_get;
*swig_RENDERGLOBAL_set = *LibreDWGc::_dwg_object_object_tio_RENDERGLOBAL_set;
*swig_RENDERSETTINGS_get = *LibreDWGc::_dwg_object_object_tio_RENDERSETTINGS_get;
*swig_RENDERSETTINGS_set = *LibreDWGc::_dwg_object_object_tio_RENDERSETTINGS_set;
*swig_SECTION_SETTINGS_get = *LibreDWGc::_dwg_object_object_tio_SECTION_SETTINGS_get;
*swig_SECTION_SETTINGS_set = *LibreDWGc::_dwg_object_object_tio_SECTION_SETTINGS_set;
*swig_SKYLIGHT_BACKGROUND_get = *LibreDWGc::_dwg_object_object_tio_SKYLIGHT_BACKGROUND_get;
*swig_SKYLIGHT_BACKGROUND_set = *LibreDWGc::_dwg_object_object_tio_SKYLIGHT_BACKGROUND_set;
*swig_SOLID_BACKGROUND_get = *LibreDWGc::_dwg_object_object_tio_SOLID_BACKGROUND_get;
*swig_SOLID_BACKGROUND_set = *LibreDWGc::_dwg_object_object_tio_SOLID_BACKGROUND_set;
*swig_SPATIAL_INDEX_get = *LibreDWGc::_dwg_object_object_tio_SPATIAL_INDEX_get;
*swig_SPATIAL_INDEX_set = *LibreDWGc::_dwg_object_object_tio_SPATIAL_INDEX_set;
*swig_TABLESTYLE_get = *LibreDWGc::_dwg_object_object_tio_TABLESTYLE_get;
*swig_TABLESTYLE_set = *LibreDWGc::_dwg_object_object_tio_TABLESTYLE_set;
*swig_TEXTOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_get;
*swig_TEXTOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_TEXTOBJECTCONTEXTDATA_set;
*swig_ASSOCARRAYMODIFYPARAMETERS_get = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_get;
*swig_ASSOCARRAYMODIFYPARAMETERS_set = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYPARAMETERS_set;
*swig_ASSOCARRAYPATHPARAMETERS_get = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_get;
*swig_ASSOCARRAYPATHPARAMETERS_set = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPATHPARAMETERS_set;
*swig_ASSOCARRAYPOLARPARAMETERS_get = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_get;
*swig_ASSOCARRAYPOLARPARAMETERS_set = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYPOLARPARAMETERS_set;
*swig_ASSOCARRAYRECTANGULARPARAMETERS_get = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_get;
*swig_ASSOCARRAYRECTANGULARPARAMETERS_set = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYRECTANGULARPARAMETERS_set;
*swig_ACMECOMMANDHISTORY_get = *LibreDWGc::_dwg_object_object_tio_ACMECOMMANDHISTORY_get;
*swig_ACMECOMMANDHISTORY_set = *LibreDWGc::_dwg_object_object_tio_ACMECOMMANDHISTORY_set;
*swig_ACMESCOPE_get = *LibreDWGc::_dwg_object_object_tio_ACMESCOPE_get;
*swig_ACMESCOPE_set = *LibreDWGc::_dwg_object_object_tio_ACMESCOPE_set;
*swig_ACMESTATEMGR_get = *LibreDWGc::_dwg_object_object_tio_ACMESTATEMGR_get;
*swig_ACMESTATEMGR_set = *LibreDWGc::_dwg_object_object_tio_ACMESTATEMGR_set;
*swig_ACSH_EXTRUSION_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_EXTRUSION_CLASS_get;
*swig_ACSH_EXTRUSION_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_EXTRUSION_CLASS_set;
*swig_ACSH_LOFT_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_LOFT_CLASS_get;
*swig_ACSH_LOFT_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_LOFT_CLASS_set;
*swig_ACSH_REVOLVE_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_REVOLVE_CLASS_get;
*swig_ACSH_REVOLVE_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_REVOLVE_CLASS_set;
*swig_ACSH_SWEEP_CLASS_get = *LibreDWGc::_dwg_object_object_tio_ACSH_SWEEP_CLASS_get;
*swig_ACSH_SWEEP_CLASS_set = *LibreDWGc::_dwg_object_object_tio_ACSH_SWEEP_CLASS_set;
*swig_ANGDIMOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_get;
*swig_ANGDIMOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_ANGDIMOBJECTCONTEXTDATA_set;
*swig_ANNOTSCALEOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_get;
*swig_ANNOTSCALEOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_ANNOTSCALEOBJECTCONTEXTDATA_set;
*swig_ASSOC3POINTANGULARDIMACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_get;
*swig_ASSOC3POINTANGULARDIMACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOC3POINTANGULARDIMACTIONBODY_set;
*swig_ASSOCALIGNEDDIMACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_get;
*swig_ASSOCALIGNEDDIMACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCALIGNEDDIMACTIONBODY_set;
*swig_ASSOCARRAYMODIFYACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_get;
*swig_ASSOCARRAYMODIFYACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCARRAYMODIFYACTIONBODY_set;
*swig_ASSOCEDGEACTIONPARAM_get = *LibreDWGc::_dwg_object_object_tio_ASSOCEDGEACTIONPARAM_get;
*swig_ASSOCEDGEACTIONPARAM_set = *LibreDWGc::_dwg_object_object_tio_ASSOCEDGEACTIONPARAM_set;
*swig_ASSOCEDGECHAMFERACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_get;
*swig_ASSOCEDGECHAMFERACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCEDGECHAMFERACTIONBODY_set;
*swig_ASSOCEDGEFILLETACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_get;
*swig_ASSOCEDGEFILLETACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCEDGEFILLETACTIONBODY_set;
*swig_ASSOCMLEADERACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCMLEADERACTIONBODY_get;
*swig_ASSOCMLEADERACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCMLEADERACTIONBODY_set;
*swig_ASSOCORDINATEDIMACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_get;
*swig_ASSOCORDINATEDIMACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCORDINATEDIMACTIONBODY_set;
*swig_ASSOCPERSSUBENTMANAGER_get = *LibreDWGc::_dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_get;
*swig_ASSOCPERSSUBENTMANAGER_set = *LibreDWGc::_dwg_object_object_tio_ASSOCPERSSUBENTMANAGER_set;
*swig_ASSOCRESTOREENTITYSTATEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_get;
*swig_ASSOCRESTOREENTITYSTATEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCRESTOREENTITYSTATEACTIONBODY_set;
*swig_ASSOCROTATEDDIMACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_get;
*swig_ASSOCROTATEDDIMACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCROTATEDDIMACTIONBODY_set;
*swig_ASSOCSWEPTSURFACEACTIONBODY_get = *LibreDWGc::_dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_get;
*swig_ASSOCSWEPTSURFACEACTIONBODY_set = *LibreDWGc::_dwg_object_object_tio_ASSOCSWEPTSURFACEACTIONBODY_set;
*swig_BLOCKPROPERTIESTABLE_get = *LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLE_get;
*swig_BLOCKPROPERTIESTABLE_set = *LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLE_set;
*swig_BLOCKPROPERTIESTABLEGRIP_get = *LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_get;
*swig_BLOCKPROPERTIESTABLEGRIP_set = *LibreDWGc::_dwg_object_object_tio_BLOCKPROPERTIESTABLEGRIP_set;
*swig_BREAKDATA_get = *LibreDWGc::_dwg_object_object_tio_BREAKDATA_get;
*swig_BREAKDATA_set = *LibreDWGc::_dwg_object_object_tio_BREAKDATA_set;
*swig_BREAKPOINTREF_get = *LibreDWGc::_dwg_object_object_tio_BREAKPOINTREF_get;
*swig_BREAKPOINTREF_set = *LibreDWGc::_dwg_object_object_tio_BREAKPOINTREF_set;
*swig_CONTEXTDATAMANAGER_get = *LibreDWGc::_dwg_object_object_tio_CONTEXTDATAMANAGER_get;
*swig_CONTEXTDATAMANAGER_set = *LibreDWGc::_dwg_object_object_tio_CONTEXTDATAMANAGER_set;
*swig_CSACDOCUMENTOPTIONS_get = *LibreDWGc::_dwg_object_object_tio_CSACDOCUMENTOPTIONS_get;
*swig_CSACDOCUMENTOPTIONS_set = *LibreDWGc::_dwg_object_object_tio_CSACDOCUMENTOPTIONS_set;
*swig_CURVEPATH_get = *LibreDWGc::_dwg_object_object_tio_CURVEPATH_get;
*swig_CURVEPATH_set = *LibreDWGc::_dwg_object_object_tio_CURVEPATH_set;
*swig_DATATABLE_get = *LibreDWGc::_dwg_object_object_tio_DATATABLE_get;
*swig_DATATABLE_set = *LibreDWGc::_dwg_object_object_tio_DATATABLE_set;
*swig_DIMASSOC_get = *LibreDWGc::_dwg_object_object_tio_DIMASSOC_get;
*swig_DIMASSOC_set = *LibreDWGc::_dwg_object_object_tio_DIMASSOC_set;
*swig_DMDIMOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_get;
*swig_DMDIMOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_DMDIMOBJECTCONTEXTDATA_set;
*swig_DYNAMICBLOCKPROXYNODE_get = *LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_get;
*swig_DYNAMICBLOCKPROXYNODE_set = *LibreDWGc::_dwg_object_object_tio_DYNAMICBLOCKPROXYNODE_set;
*swig_GEOMAPIMAGE_get = *LibreDWGc::_dwg_object_object_tio_GEOMAPIMAGE_get;
*swig_GEOMAPIMAGE_set = *LibreDWGc::_dwg_object_object_tio_GEOMAPIMAGE_set;
*swig_MLEADEROBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_get;
*swig_MLEADEROBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_MLEADEROBJECTCONTEXTDATA_set;
*swig_MOTIONPATH_get = *LibreDWGc::_dwg_object_object_tio_MOTIONPATH_get;
*swig_MOTIONPATH_set = *LibreDWGc::_dwg_object_object_tio_MOTIONPATH_set;
*swig_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_get;
*swig_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_MTEXTATTRIBUTEOBJECTCONTEXTDATA_set;
*swig_NAVISWORKSMODELDEF_get = *LibreDWGc::_dwg_object_object_tio_NAVISWORKSMODELDEF_get;
*swig_NAVISWORKSMODELDEF_set = *LibreDWGc::_dwg_object_object_tio_NAVISWORKSMODELDEF_set;
*swig_ORDDIMOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_get;
*swig_ORDDIMOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_ORDDIMOBJECTCONTEXTDATA_set;
*swig_PERSUBENTMGR_get = *LibreDWGc::_dwg_object_object_tio_PERSUBENTMGR_get;
*swig_PERSUBENTMGR_set = *LibreDWGc::_dwg_object_object_tio_PERSUBENTMGR_set;
*swig_POINTPATH_get = *LibreDWGc::_dwg_object_object_tio_POINTPATH_get;
*swig_POINTPATH_set = *LibreDWGc::_dwg_object_object_tio_POINTPATH_set;
*swig_RADIMLGOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_get;
*swig_RADIMLGOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_RADIMLGOBJECTCONTEXTDATA_set;
*swig_RADIMOBJECTCONTEXTDATA_get = *LibreDWGc::_dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_get;
*swig_RADIMOBJECTCONTEXTDATA_set = *LibreDWGc::_dwg_object_object_tio_RADIMOBJECTCONTEXTDATA_set;
*swig_SUNSTUDY_get = *LibreDWGc::_dwg_object_object_tio_SUNSTUDY_get;
*swig_SUNSTUDY_set = *LibreDWGc::_dwg_object_object_tio_SUNSTUDY_set;
*swig_TABLECONTENT_get = *LibreDWGc::_dwg_object_object_tio_TABLECONTENT_get;
*swig_TABLECONTENT_set = *LibreDWGc::_dwg_object_object_tio_TABLECONTENT_set;
*swig_TVDEVICEPROPERTIES_get = *LibreDWGc::_dwg_object_object_tio_TVDEVICEPROPERTIES_get;
*swig_TVDEVICEPROPERTIES_set = *LibreDWGc::_dwg_object_object_tio_TVDEVICEPROPERTIES_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_object_object_tio(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_object_object_tio($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Class ##############

package LibreDWG::Dwg_Class;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_number_get = *LibreDWGc::Dwg_Class_number_get;
*swig_number_set = *LibreDWGc::Dwg_Class_number_set;
*swig_proxyflag_get = *LibreDWGc::Dwg_Class_proxyflag_get;
*swig_proxyflag_set = *LibreDWGc::Dwg_Class_proxyflag_set;
*swig_appname_get = *LibreDWGc::Dwg_Class_appname_get;
*swig_appname_set = *LibreDWGc::Dwg_Class_appname_set;
*swig_cppname_get = *LibreDWGc::Dwg_Class_cppname_get;
*swig_cppname_set = *LibreDWGc::Dwg_Class_cppname_set;
*swig_dxfname_get = *LibreDWGc::Dwg_Class_dxfname_get;
*swig_dxfname_set = *LibreDWGc::Dwg_Class_dxfname_set;
*swig_dxfname_u_get = *LibreDWGc::Dwg_Class_dxfname_u_get;
*swig_dxfname_u_set = *LibreDWGc::Dwg_Class_dxfname_u_set;
*swig_is_zombie_get = *LibreDWGc::Dwg_Class_is_zombie_get;
*swig_is_zombie_set = *LibreDWGc::Dwg_Class_is_zombie_set;
*swig_item_class_id_get = *LibreDWGc::Dwg_Class_item_class_id_get;
*swig_item_class_id_set = *LibreDWGc::Dwg_Class_item_class_id_set;
*swig_num_instances_get = *LibreDWGc::Dwg_Class_num_instances_get;
*swig_num_instances_set = *LibreDWGc::Dwg_Class_num_instances_set;
*swig_dwg_version_get = *LibreDWGc::Dwg_Class_dwg_version_get;
*swig_dwg_version_set = *LibreDWGc::Dwg_Class_dwg_version_set;
*swig_maint_version_get = *LibreDWGc::Dwg_Class_maint_version_get;
*swig_maint_version_set = *LibreDWGc::Dwg_Class_maint_version_set;
*swig_unknown_1_get = *LibreDWGc::Dwg_Class_unknown_1_get;
*swig_unknown_1_set = *LibreDWGc::Dwg_Class_unknown_1_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_Class_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_Class_unknown_2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Class(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Class($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Object ##############

package LibreDWG::Dwg_Object;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_Object_size_get;
*swig_size_set = *LibreDWGc::Dwg_Object_size_set;
*swig_address_get = *LibreDWGc::Dwg_Object_address_get;
*swig_address_set = *LibreDWGc::Dwg_Object_address_set;
*swig_type_get = *LibreDWGc::Dwg_Object_type_get;
*swig_type_set = *LibreDWGc::Dwg_Object_type_set;
*swig_index_get = *LibreDWGc::Dwg_Object_index_get;
*swig_index_set = *LibreDWGc::Dwg_Object_index_set;
*swig_fixedtype_get = *LibreDWGc::Dwg_Object_fixedtype_get;
*swig_fixedtype_set = *LibreDWGc::Dwg_Object_fixedtype_set;
*swig_name_get = *LibreDWGc::Dwg_Object_name_get;
*swig_name_set = *LibreDWGc::Dwg_Object_name_set;
*swig_dxfname_get = *LibreDWGc::Dwg_Object_dxfname_get;
*swig_dxfname_set = *LibreDWGc::Dwg_Object_dxfname_set;
*swig_supertype_get = *LibreDWGc::Dwg_Object_supertype_get;
*swig_supertype_set = *LibreDWGc::Dwg_Object_supertype_set;
*swig_tio_get = *LibreDWGc::Dwg_Object_tio_get;
*swig_tio_set = *LibreDWGc::Dwg_Object_tio_set;
*swig_handle_get = *LibreDWGc::Dwg_Object_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_Object_handle_set;
*swig_parent_get = *LibreDWGc::Dwg_Object_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Object_parent_set;
*swig_klass_get = *LibreDWGc::Dwg_Object_klass_get;
*swig_klass_set = *LibreDWGc::Dwg_Object_klass_set;
*swig_bitsize_get = *LibreDWGc::Dwg_Object_bitsize_get;
*swig_bitsize_set = *LibreDWGc::Dwg_Object_bitsize_set;
*swig_bitsize_pos_get = *LibreDWGc::Dwg_Object_bitsize_pos_get;
*swig_bitsize_pos_set = *LibreDWGc::Dwg_Object_bitsize_pos_set;
*swig_hdlpos_get = *LibreDWGc::Dwg_Object_hdlpos_get;
*swig_hdlpos_set = *LibreDWGc::Dwg_Object_hdlpos_set;
*swig_was_bitsize_set_get = *LibreDWGc::Dwg_Object_was_bitsize_set_get;
*swig_was_bitsize_set_set = *LibreDWGc::Dwg_Object_was_bitsize_set_set;
*swig_has_strings_get = *LibreDWGc::Dwg_Object_has_strings_get;
*swig_has_strings_set = *LibreDWGc::Dwg_Object_has_strings_set;
*swig_stringstream_size_get = *LibreDWGc::Dwg_Object_stringstream_size_get;
*swig_stringstream_size_set = *LibreDWGc::Dwg_Object_stringstream_size_set;
*swig_handlestream_size_get = *LibreDWGc::Dwg_Object_handlestream_size_get;
*swig_handlestream_size_set = *LibreDWGc::Dwg_Object_handlestream_size_set;
*swig_common_size_get = *LibreDWGc::Dwg_Object_common_size_get;
*swig_common_size_set = *LibreDWGc::Dwg_Object_common_size_set;
*swig_num_unknown_bits_get = *LibreDWGc::Dwg_Object_num_unknown_bits_get;
*swig_num_unknown_bits_set = *LibreDWGc::Dwg_Object_num_unknown_bits_set;
*swig_unknown_bits_get = *LibreDWGc::Dwg_Object_unknown_bits_get;
*swig_unknown_bits_set = *LibreDWGc::Dwg_Object_unknown_bits_set;
*swig_num_unknown_rest_get = *LibreDWGc::Dwg_Object_num_unknown_rest_get;
*swig_num_unknown_rest_set = *LibreDWGc::Dwg_Object_num_unknown_rest_set;
*swig_unknown_rest_get = *LibreDWGc::Dwg_Object_unknown_rest_get;
*swig_unknown_rest_set = *LibreDWGc::Dwg_Object_unknown_rest_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Object(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Object($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_object_tio ##############

package LibreDWG::_dwg_object_tio;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_entity_get = *LibreDWGc::_dwg_object_tio_entity_get;
*swig_entity_set = *LibreDWGc::_dwg_object_tio_entity_set;
*swig_object_get = *LibreDWGc::_dwg_object_tio_object_get;
*swig_object_set = *LibreDWGc::_dwg_object_tio_object_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_object_tio(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_object_tio($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Chain ##############

package LibreDWG::Dwg_Chain;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_chain_get = *LibreDWGc::Dwg_Chain_chain_get;
*swig_chain_set = *LibreDWGc::Dwg_Chain_chain_set;
*swig_size_get = *LibreDWGc::Dwg_Chain_size_get;
*swig_size_set = *LibreDWGc::Dwg_Chain_size_set;
*swig_byte_get = *LibreDWGc::Dwg_Chain_byte_get;
*swig_byte_set = *LibreDWGc::Dwg_Chain_byte_set;
*swig_bit_get = *LibreDWGc::Dwg_Chain_bit_get;
*swig_bit_set = *LibreDWGc::Dwg_Chain_bit_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Chain(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Chain($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Section ##############

package LibreDWG::Dwg_Section;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_number_get = *LibreDWGc::Dwg_Section_number_get;
*swig_number_set = *LibreDWGc::Dwg_Section_number_set;
*swig_size_get = *LibreDWGc::Dwg_Section_size_get;
*swig_size_set = *LibreDWGc::Dwg_Section_size_set;
*swig_address_get = *LibreDWGc::Dwg_Section_address_get;
*swig_address_set = *LibreDWGc::Dwg_Section_address_set;
*swig_objid_r11_get = *LibreDWGc::Dwg_Section_objid_r11_get;
*swig_objid_r11_set = *LibreDWGc::Dwg_Section_objid_r11_set;
*swig_parent_get = *LibreDWGc::Dwg_Section_parent_get;
*swig_parent_set = *LibreDWGc::Dwg_Section_parent_set;
*swig_left_get = *LibreDWGc::Dwg_Section_left_get;
*swig_left_set = *LibreDWGc::Dwg_Section_left_set;
*swig_right_get = *LibreDWGc::Dwg_Section_right_get;
*swig_right_set = *LibreDWGc::Dwg_Section_right_set;
*swig_x00_get = *LibreDWGc::Dwg_Section_x00_get;
*swig_x00_set = *LibreDWGc::Dwg_Section_x00_set;
*swig_type_get = *LibreDWGc::Dwg_Section_type_get;
*swig_type_set = *LibreDWGc::Dwg_Section_type_set;
*swig_name_get = *LibreDWGc::Dwg_Section_name_get;
*swig_name_set = *LibreDWGc::Dwg_Section_name_set;
*swig_section_type_get = *LibreDWGc::Dwg_Section_section_type_get;
*swig_section_type_set = *LibreDWGc::Dwg_Section_section_type_set;
*swig_decomp_data_size_get = *LibreDWGc::Dwg_Section_decomp_data_size_get;
*swig_decomp_data_size_set = *LibreDWGc::Dwg_Section_decomp_data_size_set;
*swig_comp_data_size_get = *LibreDWGc::Dwg_Section_comp_data_size_get;
*swig_comp_data_size_set = *LibreDWGc::Dwg_Section_comp_data_size_set;
*swig_compression_type_get = *LibreDWGc::Dwg_Section_compression_type_get;
*swig_compression_type_set = *LibreDWGc::Dwg_Section_compression_type_set;
*swig_checksum_get = *LibreDWGc::Dwg_Section_checksum_get;
*swig_checksum_set = *LibreDWGc::Dwg_Section_checksum_set;
*swig_flags_r11_get = *LibreDWGc::Dwg_Section_flags_r11_get;
*swig_flags_r11_set = *LibreDWGc::Dwg_Section_flags_r11_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Section(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Section($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Section_InfoHdr ##############

package LibreDWG::Dwg_Section_InfoHdr;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_desc_get = *LibreDWGc::Dwg_Section_InfoHdr_num_desc_get;
*swig_num_desc_set = *LibreDWGc::Dwg_Section_InfoHdr_num_desc_set;
*swig_compressed_get = *LibreDWGc::Dwg_Section_InfoHdr_compressed_get;
*swig_compressed_set = *LibreDWGc::Dwg_Section_InfoHdr_compressed_set;
*swig_max_size_get = *LibreDWGc::Dwg_Section_InfoHdr_max_size_get;
*swig_max_size_set = *LibreDWGc::Dwg_Section_InfoHdr_max_size_set;
*swig_encrypted_get = *LibreDWGc::Dwg_Section_InfoHdr_encrypted_get;
*swig_encrypted_set = *LibreDWGc::Dwg_Section_InfoHdr_encrypted_set;
*swig_num_desc2_get = *LibreDWGc::Dwg_Section_InfoHdr_num_desc2_get;
*swig_num_desc2_set = *LibreDWGc::Dwg_Section_InfoHdr_num_desc2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Section_InfoHdr(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Section_InfoHdr($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Section_Info ##############

package LibreDWG::Dwg_Section_Info;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_Section_Info_size_get;
*swig_size_set = *LibreDWGc::Dwg_Section_Info_size_set;
*swig_num_sections_get = *LibreDWGc::Dwg_Section_Info_num_sections_get;
*swig_num_sections_set = *LibreDWGc::Dwg_Section_Info_num_sections_set;
*swig_max_decomp_size_get = *LibreDWGc::Dwg_Section_Info_max_decomp_size_get;
*swig_max_decomp_size_set = *LibreDWGc::Dwg_Section_Info_max_decomp_size_set;
*swig_unknown_get = *LibreDWGc::Dwg_Section_Info_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_Section_Info_unknown_set;
*swig_compressed_get = *LibreDWGc::Dwg_Section_Info_compressed_get;
*swig_compressed_set = *LibreDWGc::Dwg_Section_Info_compressed_set;
*swig_type_get = *LibreDWGc::Dwg_Section_Info_type_get;
*swig_type_set = *LibreDWGc::Dwg_Section_Info_type_set;
*swig_encrypted_get = *LibreDWGc::Dwg_Section_Info_encrypted_get;
*swig_encrypted_set = *LibreDWGc::Dwg_Section_Info_encrypted_set;
*swig_name_get = *LibreDWGc::Dwg_Section_Info_name_get;
*swig_name_set = *LibreDWGc::Dwg_Section_Info_name_set;
*swig_fixedtype_get = *LibreDWGc::Dwg_Section_Info_fixedtype_get;
*swig_fixedtype_set = *LibreDWGc::Dwg_Section_Info_fixedtype_set;
*swig_sections_get = *LibreDWGc::Dwg_Section_Info_sections_get;
*swig_sections_set = *LibreDWGc::Dwg_Section_Info_sections_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Section_Info(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Section_Info($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SummaryInfo_Property ##############

package LibreDWG::Dwg_SummaryInfo_Property;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_tag_get = *LibreDWGc::Dwg_SummaryInfo_Property_tag_get;
*swig_tag_set = *LibreDWGc::Dwg_SummaryInfo_Property_tag_set;
*swig_value_get = *LibreDWGc::Dwg_SummaryInfo_Property_value_get;
*swig_value_set = *LibreDWGc::Dwg_SummaryInfo_Property_value_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SummaryInfo_Property(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SummaryInfo_Property($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_FileDepList_Files ##############

package LibreDWG::Dwg_FileDepList_Files;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_filename_get = *LibreDWGc::Dwg_FileDepList_Files_filename_get;
*swig_filename_set = *LibreDWGc::Dwg_FileDepList_Files_filename_set;
*swig_filepath_get = *LibreDWGc::Dwg_FileDepList_Files_filepath_get;
*swig_filepath_set = *LibreDWGc::Dwg_FileDepList_Files_filepath_set;
*swig_fingerprint_get = *LibreDWGc::Dwg_FileDepList_Files_fingerprint_get;
*swig_fingerprint_set = *LibreDWGc::Dwg_FileDepList_Files_fingerprint_set;
*swig_version_get = *LibreDWGc::Dwg_FileDepList_Files_version_get;
*swig_version_set = *LibreDWGc::Dwg_FileDepList_Files_version_set;
*swig_feature_index_get = *LibreDWGc::Dwg_FileDepList_Files_feature_index_get;
*swig_feature_index_set = *LibreDWGc::Dwg_FileDepList_Files_feature_index_set;
*swig_timestamp_get = *LibreDWGc::Dwg_FileDepList_Files_timestamp_get;
*swig_timestamp_set = *LibreDWGc::Dwg_FileDepList_Files_timestamp_set;
*swig_filesize_get = *LibreDWGc::Dwg_FileDepList_Files_filesize_get;
*swig_filesize_set = *LibreDWGc::Dwg_FileDepList_Files_filesize_set;
*swig_affects_graphics_get = *LibreDWGc::Dwg_FileDepList_Files_affects_graphics_get;
*swig_affects_graphics_set = *LibreDWGc::Dwg_FileDepList_Files_affects_graphics_set;
*swig_refcount_get = *LibreDWGc::Dwg_FileDepList_Files_refcount_get;
*swig_refcount_set = *LibreDWGc::Dwg_FileDepList_Files_refcount_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_FileDepList_Files(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_FileDepList_Files($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_SegmentIndex ##############

package LibreDWG::Dwg_AcDs_SegmentIndex;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_offset_get = *LibreDWGc::Dwg_AcDs_SegmentIndex_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_AcDs_SegmentIndex_offset_set;
*swig_size_get = *LibreDWGc::Dwg_AcDs_SegmentIndex_size_get;
*swig_size_set = *LibreDWGc::Dwg_AcDs_SegmentIndex_size_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_SegmentIndex(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_SegmentIndex($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_DataIndex_Entry ##############

package LibreDWG::Dwg_AcDs_DataIndex_Entry;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_segidx_get = *LibreDWGc::Dwg_AcDs_DataIndex_Entry_segidx_get;
*swig_segidx_set = *LibreDWGc::Dwg_AcDs_DataIndex_Entry_segidx_set;
*swig_offset_get = *LibreDWGc::Dwg_AcDs_DataIndex_Entry_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_AcDs_DataIndex_Entry_offset_set;
*swig_schidx_get = *LibreDWGc::Dwg_AcDs_DataIndex_Entry_schidx_get;
*swig_schidx_set = *LibreDWGc::Dwg_AcDs_DataIndex_Entry_schidx_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_DataIndex_Entry(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_DataIndex_Entry($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_DataIndex ##############

package LibreDWG::Dwg_AcDs_DataIndex;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_entries_get = *LibreDWGc::Dwg_AcDs_DataIndex_num_entries_get;
*swig_num_entries_set = *LibreDWGc::Dwg_AcDs_DataIndex_num_entries_set;
*swig_di_unknown_get = *LibreDWGc::Dwg_AcDs_DataIndex_di_unknown_get;
*swig_di_unknown_set = *LibreDWGc::Dwg_AcDs_DataIndex_di_unknown_set;
*swig_entries_get = *LibreDWGc::Dwg_AcDs_DataIndex_entries_get;
*swig_entries_set = *LibreDWGc::Dwg_AcDs_DataIndex_entries_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_DataIndex(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_DataIndex($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Data_RecordHdr ##############

package LibreDWG::Dwg_AcDs_Data_RecordHdr;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_entry_size_get = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_entry_size_get;
*swig_entry_size_set = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_entry_size_set;
*swig_unknown_get = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_unknown_set;
*swig_handle_get = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_handle_set;
*swig_offset_get = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_AcDs_Data_RecordHdr_offset_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Data_RecordHdr(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Data_RecordHdr($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Data_Record ##############

package LibreDWG::Dwg_AcDs_Data_Record;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_data_size_get = *LibreDWGc::Dwg_AcDs_Data_Record_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_AcDs_Data_Record_data_size_set;
*swig_blob_get = *LibreDWGc::Dwg_AcDs_Data_Record_blob_get;
*swig_blob_set = *LibreDWGc::Dwg_AcDs_Data_Record_blob_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Data_Record(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Data_Record($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Data ##############

package LibreDWG::Dwg_AcDs_Data;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_record_hdrs_get = *LibreDWGc::Dwg_AcDs_Data_record_hdrs_get;
*swig_record_hdrs_set = *LibreDWGc::Dwg_AcDs_Data_record_hdrs_set;
*swig_records_get = *LibreDWGc::Dwg_AcDs_Data_records_get;
*swig_records_set = *LibreDWGc::Dwg_AcDs_Data_records_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Data(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Data($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_DataBlobRef_Page ##############

package LibreDWG::Dwg_AcDs_DataBlobRef_Page;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_segidx_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_Page_segidx_get;
*swig_segidx_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_Page_segidx_set;
*swig_size_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_Page_size_get;
*swig_size_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_Page_size_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_DataBlobRef_Page(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_DataBlobRef_Page($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_DataBlobRef ##############

package LibreDWG::Dwg_AcDs_DataBlobRef;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_total_data_size_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_total_data_size_get;
*swig_total_data_size_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_total_data_size_set;
*swig_num_pages_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_num_pages_get;
*swig_num_pages_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_num_pages_set;
*swig_record_size_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_record_size_get;
*swig_record_size_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_record_size_set;
*swig_page_size_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_page_size_get;
*swig_page_size_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_page_size_set;
*swig_unknown_1_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_1_get;
*swig_unknown_1_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_1_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_unknown_2_set;
*swig_pages_get = *LibreDWGc::Dwg_AcDs_DataBlobRef_pages_get;
*swig_pages_set = *LibreDWGc::Dwg_AcDs_DataBlobRef_pages_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_DataBlobRef(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_DataBlobRef($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_DataBlob ##############

package LibreDWG::Dwg_AcDs_DataBlob;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_data_size_get = *LibreDWGc::Dwg_AcDs_DataBlob_data_size_get;
*swig_data_size_set = *LibreDWGc::Dwg_AcDs_DataBlob_data_size_set;
*swig_page_count_get = *LibreDWGc::Dwg_AcDs_DataBlob_page_count_get;
*swig_page_count_set = *LibreDWGc::Dwg_AcDs_DataBlob_page_count_set;
*swig_record_size_get = *LibreDWGc::Dwg_AcDs_DataBlob_record_size_get;
*swig_record_size_set = *LibreDWGc::Dwg_AcDs_DataBlob_record_size_set;
*swig_page_size_get = *LibreDWGc::Dwg_AcDs_DataBlob_page_size_get;
*swig_page_size_set = *LibreDWGc::Dwg_AcDs_DataBlob_page_size_set;
*swig_unknown_1_get = *LibreDWGc::Dwg_AcDs_DataBlob_unknown_1_get;
*swig_unknown_1_set = *LibreDWGc::Dwg_AcDs_DataBlob_unknown_1_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_AcDs_DataBlob_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_AcDs_DataBlob_unknown_2_set;
*swig_ref_get = *LibreDWGc::Dwg_AcDs_DataBlob_ref_get;
*swig_ref_set = *LibreDWGc::Dwg_AcDs_DataBlob_ref_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_DataBlob(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_DataBlob($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_DataBlob01 ##############

package LibreDWG::Dwg_AcDs_DataBlob01;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_total_data_size_get = *LibreDWGc::Dwg_AcDs_DataBlob01_total_data_size_get;
*swig_total_data_size_set = *LibreDWGc::Dwg_AcDs_DataBlob01_total_data_size_set;
*swig_page_start_offset_get = *LibreDWGc::Dwg_AcDs_DataBlob01_page_start_offset_get;
*swig_page_start_offset_set = *LibreDWGc::Dwg_AcDs_DataBlob01_page_start_offset_set;
*swig_page_index_get = *LibreDWGc::Dwg_AcDs_DataBlob01_page_index_get;
*swig_page_index_set = *LibreDWGc::Dwg_AcDs_DataBlob01_page_index_set;
*swig_page_count_get = *LibreDWGc::Dwg_AcDs_DataBlob01_page_count_get;
*swig_page_count_set = *LibreDWGc::Dwg_AcDs_DataBlob01_page_count_set;
*swig_page_data_size_get = *LibreDWGc::Dwg_AcDs_DataBlob01_page_data_size_get;
*swig_page_data_size_set = *LibreDWGc::Dwg_AcDs_DataBlob01_page_data_size_set;
*swig_page_data_get = *LibreDWGc::Dwg_AcDs_DataBlob01_page_data_get;
*swig_page_data_set = *LibreDWGc::Dwg_AcDs_DataBlob01_page_data_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_DataBlob01(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_DataBlob01($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_SchemaIndex_Prop ##############

package LibreDWG::Dwg_AcDs_SchemaIndex_Prop;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_index_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_index_get;
*swig_index_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_index_set;
*swig_segidx_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_segidx_get;
*swig_segidx_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_segidx_set;
*swig_offset_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_Prop_offset_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_SchemaIndex_Prop(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_SchemaIndex_Prop($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_SchemaIndex ##############

package LibreDWG::Dwg_AcDs_SchemaIndex;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_props_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_num_props_get;
*swig_num_props_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_num_props_set;
*swig_si_unknown_1_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_1_get;
*swig_si_unknown_1_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_1_set;
*swig_props_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_props_get;
*swig_props_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_props_set;
*swig_si_tag_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_si_tag_get;
*swig_si_tag_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_si_tag_set;
*swig_num_prop_entries_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_num_prop_entries_get;
*swig_num_prop_entries_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_num_prop_entries_set;
*swig_si_unknown_2_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_2_get;
*swig_si_unknown_2_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_si_unknown_2_set;
*swig_prop_entries_get = *LibreDWGc::Dwg_AcDs_SchemaIndex_prop_entries_get;
*swig_prop_entries_set = *LibreDWGc::Dwg_AcDs_SchemaIndex_prop_entries_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_SchemaIndex(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_SchemaIndex($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Schema_Prop ##############

package LibreDWG::Dwg_AcDs_Schema_Prop;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_flags_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_flags_set;
*swig_namidx_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_namidx_get;
*swig_namidx_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_namidx_set;
*swig_type_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_type_get;
*swig_type_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_type_set;
*swig_type_size_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_type_size_get;
*swig_type_size_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_type_size_set;
*swig_unknown_1_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_1_get;
*swig_unknown_1_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_1_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_unknown_2_set;
*swig_num_values_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_num_values_get;
*swig_num_values_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_num_values_set;
*swig_values_get = *LibreDWGc::Dwg_AcDs_Schema_Prop_values_get;
*swig_values_set = *LibreDWGc::Dwg_AcDs_Schema_Prop_values_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Schema_Prop(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Schema_Prop($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Schema ##############

package LibreDWG::Dwg_AcDs_Schema;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_index_get = *LibreDWGc::Dwg_AcDs_Schema_num_index_get;
*swig_num_index_set = *LibreDWGc::Dwg_AcDs_Schema_num_index_set;
*swig_index_get = *LibreDWGc::Dwg_AcDs_Schema_index_get;
*swig_index_set = *LibreDWGc::Dwg_AcDs_Schema_index_set;
*swig_num_props_get = *LibreDWGc::Dwg_AcDs_Schema_num_props_get;
*swig_num_props_set = *LibreDWGc::Dwg_AcDs_Schema_num_props_set;
*swig_props_get = *LibreDWGc::Dwg_AcDs_Schema_props_get;
*swig_props_set = *LibreDWGc::Dwg_AcDs_Schema_props_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Schema(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Schema($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_SchemaData_UProp ##############

package LibreDWG::Dwg_AcDs_SchemaData_UProp;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_AcDs_SchemaData_UProp_size_get;
*swig_size_set = *LibreDWGc::Dwg_AcDs_SchemaData_UProp_size_set;
*swig_flags_get = *LibreDWGc::Dwg_AcDs_SchemaData_UProp_flags_get;
*swig_flags_set = *LibreDWGc::Dwg_AcDs_SchemaData_UProp_flags_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_SchemaData_UProp(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_SchemaData_UProp($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_SchemaData ##############

package LibreDWG::Dwg_AcDs_SchemaData;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_uprops_get = *LibreDWGc::Dwg_AcDs_SchemaData_num_uprops_get;
*swig_num_uprops_set = *LibreDWGc::Dwg_AcDs_SchemaData_num_uprops_set;
*swig_uprops_get = *LibreDWGc::Dwg_AcDs_SchemaData_uprops_get;
*swig_uprops_set = *LibreDWGc::Dwg_AcDs_SchemaData_uprops_set;
*swig_num_schemas_get = *LibreDWGc::Dwg_AcDs_SchemaData_num_schemas_get;
*swig_num_schemas_set = *LibreDWGc::Dwg_AcDs_SchemaData_num_schemas_set;
*swig_schemas_get = *LibreDWGc::Dwg_AcDs_SchemaData_schemas_get;
*swig_schemas_set = *LibreDWGc::Dwg_AcDs_SchemaData_schemas_set;
*swig_num_propnames_get = *LibreDWGc::Dwg_AcDs_SchemaData_num_propnames_get;
*swig_num_propnames_set = *LibreDWGc::Dwg_AcDs_SchemaData_num_propnames_set;
*swig_propnames_get = *LibreDWGc::Dwg_AcDs_SchemaData_propnames_get;
*swig_propnames_set = *LibreDWGc::Dwg_AcDs_SchemaData_propnames_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_SchemaData(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_SchemaData($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Search_IdIdx ##############

package LibreDWG::Dwg_AcDs_Search_IdIdx;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_handle_get = *LibreDWGc::Dwg_AcDs_Search_IdIdx_handle_get;
*swig_handle_set = *LibreDWGc::Dwg_AcDs_Search_IdIdx_handle_set;
*swig_num_ididx_get = *LibreDWGc::Dwg_AcDs_Search_IdIdx_num_ididx_get;
*swig_num_ididx_set = *LibreDWGc::Dwg_AcDs_Search_IdIdx_num_ididx_set;
*swig_ididx_get = *LibreDWGc::Dwg_AcDs_Search_IdIdx_ididx_get;
*swig_ididx_set = *LibreDWGc::Dwg_AcDs_Search_IdIdx_ididx_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Search_IdIdx(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Search_IdIdx($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Search_IdIdxs ##############

package LibreDWG::Dwg_AcDs_Search_IdIdxs;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_ididx_get = *LibreDWGc::Dwg_AcDs_Search_IdIdxs_num_ididx_get;
*swig_num_ididx_set = *LibreDWGc::Dwg_AcDs_Search_IdIdxs_num_ididx_set;
*swig_ididx_get = *LibreDWGc::Dwg_AcDs_Search_IdIdxs_ididx_get;
*swig_ididx_set = *LibreDWGc::Dwg_AcDs_Search_IdIdxs_ididx_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Search_IdIdxs(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Search_IdIdxs($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Search_Data ##############

package LibreDWG::Dwg_AcDs_Search_Data;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_schema_namidx_get = *LibreDWGc::Dwg_AcDs_Search_Data_schema_namidx_get;
*swig_schema_namidx_set = *LibreDWGc::Dwg_AcDs_Search_Data_schema_namidx_set;
*swig_num_sortedidx_get = *LibreDWGc::Dwg_AcDs_Search_Data_num_sortedidx_get;
*swig_num_sortedidx_set = *LibreDWGc::Dwg_AcDs_Search_Data_num_sortedidx_set;
*swig_sortedidx_get = *LibreDWGc::Dwg_AcDs_Search_Data_sortedidx_get;
*swig_sortedidx_set = *LibreDWGc::Dwg_AcDs_Search_Data_sortedidx_set;
*swig_num_ididxs_get = *LibreDWGc::Dwg_AcDs_Search_Data_num_ididxs_get;
*swig_num_ididxs_set = *LibreDWGc::Dwg_AcDs_Search_Data_num_ididxs_set;
*swig_unknown_get = *LibreDWGc::Dwg_AcDs_Search_Data_unknown_get;
*swig_unknown_set = *LibreDWGc::Dwg_AcDs_Search_Data_unknown_set;
*swig_ididxs_get = *LibreDWGc::Dwg_AcDs_Search_Data_ididxs_get;
*swig_ididxs_set = *LibreDWGc::Dwg_AcDs_Search_Data_ididxs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Search_Data(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Search_Data($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Search ##############

package LibreDWG::Dwg_AcDs_Search;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_search_get = *LibreDWGc::Dwg_AcDs_Search_num_search_get;
*swig_num_search_set = *LibreDWGc::Dwg_AcDs_Search_num_search_set;
*swig_search_get = *LibreDWGc::Dwg_AcDs_Search_search_get;
*swig_search_set = *LibreDWGc::Dwg_AcDs_Search_search_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Search(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Search($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs_Segment ##############

package LibreDWG::Dwg_AcDs_Segment;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_signature_get = *LibreDWGc::Dwg_AcDs_Segment_signature_get;
*swig_signature_set = *LibreDWGc::Dwg_AcDs_Segment_signature_set;
*swig_name_get = *LibreDWGc::Dwg_AcDs_Segment_name_get;
*swig_name_set = *LibreDWGc::Dwg_AcDs_Segment_name_set;
*swig_type_get = *LibreDWGc::Dwg_AcDs_Segment_type_get;
*swig_type_set = *LibreDWGc::Dwg_AcDs_Segment_type_set;
*swig_segment_idx_get = *LibreDWGc::Dwg_AcDs_Segment_segment_idx_get;
*swig_segment_idx_set = *LibreDWGc::Dwg_AcDs_Segment_segment_idx_set;
*swig_is_blob01_get = *LibreDWGc::Dwg_AcDs_Segment_is_blob01_get;
*swig_is_blob01_set = *LibreDWGc::Dwg_AcDs_Segment_is_blob01_set;
*swig_segsize_get = *LibreDWGc::Dwg_AcDs_Segment_segsize_get;
*swig_segsize_set = *LibreDWGc::Dwg_AcDs_Segment_segsize_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_AcDs_Segment_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_AcDs_Segment_unknown_2_set;
*swig_ds_version_get = *LibreDWGc::Dwg_AcDs_Segment_ds_version_get;
*swig_ds_version_set = *LibreDWGc::Dwg_AcDs_Segment_ds_version_set;
*swig_unknown_3_get = *LibreDWGc::Dwg_AcDs_Segment_unknown_3_get;
*swig_unknown_3_set = *LibreDWGc::Dwg_AcDs_Segment_unknown_3_set;
*swig_data_algn_offset_get = *LibreDWGc::Dwg_AcDs_Segment_data_algn_offset_get;
*swig_data_algn_offset_set = *LibreDWGc::Dwg_AcDs_Segment_data_algn_offset_set;
*swig_objdata_algn_offset_get = *LibreDWGc::Dwg_AcDs_Segment_objdata_algn_offset_get;
*swig_objdata_algn_offset_set = *LibreDWGc::Dwg_AcDs_Segment_objdata_algn_offset_set;
*swig_padding_get = *LibreDWGc::Dwg_AcDs_Segment_padding_get;
*swig_padding_set = *LibreDWGc::Dwg_AcDs_Segment_padding_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs_Segment(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs_Segment($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AcDs ##############

package LibreDWG::Dwg_AcDs;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_file_signature_get = *LibreDWGc::Dwg_AcDs_file_signature_get;
*swig_file_signature_set = *LibreDWGc::Dwg_AcDs_file_signature_set;
*swig_file_header_size_get = *LibreDWGc::Dwg_AcDs_file_header_size_get;
*swig_file_header_size_set = *LibreDWGc::Dwg_AcDs_file_header_size_set;
*swig_unknown_1_get = *LibreDWGc::Dwg_AcDs_unknown_1_get;
*swig_unknown_1_set = *LibreDWGc::Dwg_AcDs_unknown_1_set;
*swig_version_get = *LibreDWGc::Dwg_AcDs_version_get;
*swig_version_set = *LibreDWGc::Dwg_AcDs_version_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_AcDs_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_AcDs_unknown_2_set;
*swig_ds_version_get = *LibreDWGc::Dwg_AcDs_ds_version_get;
*swig_ds_version_set = *LibreDWGc::Dwg_AcDs_ds_version_set;
*swig_segidx_offset_get = *LibreDWGc::Dwg_AcDs_segidx_offset_get;
*swig_segidx_offset_set = *LibreDWGc::Dwg_AcDs_segidx_offset_set;
*swig_segidx_unknown_get = *LibreDWGc::Dwg_AcDs_segidx_unknown_get;
*swig_segidx_unknown_set = *LibreDWGc::Dwg_AcDs_segidx_unknown_set;
*swig_num_segidx_get = *LibreDWGc::Dwg_AcDs_num_segidx_get;
*swig_num_segidx_set = *LibreDWGc::Dwg_AcDs_num_segidx_set;
*swig_schidx_segidx_get = *LibreDWGc::Dwg_AcDs_schidx_segidx_get;
*swig_schidx_segidx_set = *LibreDWGc::Dwg_AcDs_schidx_segidx_set;
*swig_datidx_segidx_get = *LibreDWGc::Dwg_AcDs_datidx_segidx_get;
*swig_datidx_segidx_set = *LibreDWGc::Dwg_AcDs_datidx_segidx_set;
*swig_search_segidx_get = *LibreDWGc::Dwg_AcDs_search_segidx_get;
*swig_search_segidx_set = *LibreDWGc::Dwg_AcDs_search_segidx_set;
*swig_prvsav_segidx_get = *LibreDWGc::Dwg_AcDs_prvsav_segidx_get;
*swig_prvsav_segidx_set = *LibreDWGc::Dwg_AcDs_prvsav_segidx_set;
*swig_file_size_get = *LibreDWGc::Dwg_AcDs_file_size_get;
*swig_file_size_set = *LibreDWGc::Dwg_AcDs_file_size_set;
*swig_total_segments_get = *LibreDWGc::Dwg_AcDs_total_segments_get;
*swig_total_segments_set = *LibreDWGc::Dwg_AcDs_total_segments_set;
*swig_segidx_get = *LibreDWGc::Dwg_AcDs_segidx_get;
*swig_segidx_set = *LibreDWGc::Dwg_AcDs_segidx_set;
*swig_datidx_get = *LibreDWGc::Dwg_AcDs_datidx_get;
*swig_datidx_set = *LibreDWGc::Dwg_AcDs_datidx_set;
*swig_data_get = *LibreDWGc::Dwg_AcDs_data_get;
*swig_data_set = *LibreDWGc::Dwg_AcDs_data_set;
*swig_blob01_get = *LibreDWGc::Dwg_AcDs_blob01_get;
*swig_blob01_set = *LibreDWGc::Dwg_AcDs_blob01_set;
*swig_schidx_get = *LibreDWGc::Dwg_AcDs_schidx_get;
*swig_schidx_set = *LibreDWGc::Dwg_AcDs_schidx_set;
*swig_schdat_get = *LibreDWGc::Dwg_AcDs_schdat_get;
*swig_schdat_set = *LibreDWGc::Dwg_AcDs_schdat_set;
*swig_search_get = *LibreDWGc::Dwg_AcDs_search_get;
*swig_search_set = *LibreDWGc::Dwg_AcDs_search_set;
*swig_segments_get = *LibreDWGc::Dwg_AcDs_segments_get;
*swig_segments_set = *LibreDWGc::Dwg_AcDs_segments_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AcDs(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AcDs($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Header ##############

package LibreDWG::Dwg_Header;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_version_get = *LibreDWGc::Dwg_Header_version_get;
*swig_version_set = *LibreDWGc::Dwg_Header_version_set;
*swig_from_version_get = *LibreDWGc::Dwg_Header_from_version_get;
*swig_from_version_set = *LibreDWGc::Dwg_Header_from_version_set;
*swig_is_maint_get = *LibreDWGc::Dwg_Header_is_maint_get;
*swig_is_maint_set = *LibreDWGc::Dwg_Header_is_maint_set;
*swig_zero_one_or_three_get = *LibreDWGc::Dwg_Header_zero_one_or_three_get;
*swig_zero_one_or_three_set = *LibreDWGc::Dwg_Header_zero_one_or_three_set;
*swig_numentity_sections_get = *LibreDWGc::Dwg_Header_numentity_sections_get;
*swig_numentity_sections_set = *LibreDWGc::Dwg_Header_numentity_sections_set;
*swig_numheader_vars_get = *LibreDWGc::Dwg_Header_numheader_vars_get;
*swig_numheader_vars_set = *LibreDWGc::Dwg_Header_numheader_vars_set;
*swig_thumbnail_address_get = *LibreDWGc::Dwg_Header_thumbnail_address_get;
*swig_thumbnail_address_set = *LibreDWGc::Dwg_Header_thumbnail_address_set;
*swig_dwg_version_get = *LibreDWGc::Dwg_Header_dwg_version_get;
*swig_dwg_version_set = *LibreDWGc::Dwg_Header_dwg_version_set;
*swig_maint_version_get = *LibreDWGc::Dwg_Header_maint_version_get;
*swig_maint_version_set = *LibreDWGc::Dwg_Header_maint_version_set;
*swig_entities_start_get = *LibreDWGc::Dwg_Header_entities_start_get;
*swig_entities_start_set = *LibreDWGc::Dwg_Header_entities_start_set;
*swig_entities_end_get = *LibreDWGc::Dwg_Header_entities_end_get;
*swig_entities_end_set = *LibreDWGc::Dwg_Header_entities_end_set;
*swig_blocks_start_get = *LibreDWGc::Dwg_Header_blocks_start_get;
*swig_blocks_start_set = *LibreDWGc::Dwg_Header_blocks_start_set;
*swig_blocks_size_get = *LibreDWGc::Dwg_Header_blocks_size_get;
*swig_blocks_size_set = *LibreDWGc::Dwg_Header_blocks_size_set;
*swig_extras_start_get = *LibreDWGc::Dwg_Header_extras_start_get;
*swig_extras_start_set = *LibreDWGc::Dwg_Header_extras_start_set;
*swig_extras_size_get = *LibreDWGc::Dwg_Header_extras_size_get;
*swig_extras_size_set = *LibreDWGc::Dwg_Header_extras_size_set;
*swig_codepage_get = *LibreDWGc::Dwg_Header_codepage_get;
*swig_codepage_set = *LibreDWGc::Dwg_Header_codepage_set;
*swig_unknown_0_get = *LibreDWGc::Dwg_Header_unknown_0_get;
*swig_unknown_0_set = *LibreDWGc::Dwg_Header_unknown_0_set;
*swig_app_dwg_version_get = *LibreDWGc::Dwg_Header_app_dwg_version_get;
*swig_app_dwg_version_set = *LibreDWGc::Dwg_Header_app_dwg_version_set;
*swig_app_maint_version_get = *LibreDWGc::Dwg_Header_app_maint_version_get;
*swig_app_maint_version_set = *LibreDWGc::Dwg_Header_app_maint_version_set;
*swig_security_type_get = *LibreDWGc::Dwg_Header_security_type_get;
*swig_security_type_set = *LibreDWGc::Dwg_Header_security_type_set;
*swig_rl_1c_address_get = *LibreDWGc::Dwg_Header_rl_1c_address_get;
*swig_rl_1c_address_set = *LibreDWGc::Dwg_Header_rl_1c_address_set;
*swig_summaryinfo_address_get = *LibreDWGc::Dwg_Header_summaryinfo_address_get;
*swig_summaryinfo_address_set = *LibreDWGc::Dwg_Header_summaryinfo_address_set;
*swig_vbaproj_address_get = *LibreDWGc::Dwg_Header_vbaproj_address_get;
*swig_vbaproj_address_set = *LibreDWGc::Dwg_Header_vbaproj_address_set;
*swig_r2004_header_address_get = *LibreDWGc::Dwg_Header_r2004_header_address_get;
*swig_r2004_header_address_set = *LibreDWGc::Dwg_Header_r2004_header_address_set;
*swig_sections_get = *LibreDWGc::Dwg_Header_sections_get;
*swig_sections_set = *LibreDWGc::Dwg_Header_sections_set;
*swig_num_sections_get = *LibreDWGc::Dwg_Header_num_sections_get;
*swig_num_sections_set = *LibreDWGc::Dwg_Header_num_sections_set;
*swig_section_get = *LibreDWGc::Dwg_Header_section_get;
*swig_section_set = *LibreDWGc::Dwg_Header_section_set;
*swig_section_infohdr_get = *LibreDWGc::Dwg_Header_section_infohdr_get;
*swig_section_infohdr_set = *LibreDWGc::Dwg_Header_section_infohdr_set;
*swig_section_info_get = *LibreDWGc::Dwg_Header_section_info_get;
*swig_section_info_set = *LibreDWGc::Dwg_Header_section_info_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Header(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Header($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_R2007_Header ##############

package LibreDWG::Dwg_R2007_Header;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_header_size_get = *LibreDWGc::Dwg_R2007_Header_header_size_get;
*swig_header_size_set = *LibreDWGc::Dwg_R2007_Header_header_size_set;
*swig_file_size_get = *LibreDWGc::Dwg_R2007_Header_file_size_get;
*swig_file_size_set = *LibreDWGc::Dwg_R2007_Header_file_size_set;
*swig_pages_map_crc_compressed_get = *LibreDWGc::Dwg_R2007_Header_pages_map_crc_compressed_get;
*swig_pages_map_crc_compressed_set = *LibreDWGc::Dwg_R2007_Header_pages_map_crc_compressed_set;
*swig_pages_map_correction_get = *LibreDWGc::Dwg_R2007_Header_pages_map_correction_get;
*swig_pages_map_correction_set = *LibreDWGc::Dwg_R2007_Header_pages_map_correction_set;
*swig_pages_map_crc_seed_get = *LibreDWGc::Dwg_R2007_Header_pages_map_crc_seed_get;
*swig_pages_map_crc_seed_set = *LibreDWGc::Dwg_R2007_Header_pages_map_crc_seed_set;
*swig_pages_map2_offset_get = *LibreDWGc::Dwg_R2007_Header_pages_map2_offset_get;
*swig_pages_map2_offset_set = *LibreDWGc::Dwg_R2007_Header_pages_map2_offset_set;
*swig_pages_map2_id_get = *LibreDWGc::Dwg_R2007_Header_pages_map2_id_get;
*swig_pages_map2_id_set = *LibreDWGc::Dwg_R2007_Header_pages_map2_id_set;
*swig_pages_map_offset_get = *LibreDWGc::Dwg_R2007_Header_pages_map_offset_get;
*swig_pages_map_offset_set = *LibreDWGc::Dwg_R2007_Header_pages_map_offset_set;
*swig_pages_map_id_get = *LibreDWGc::Dwg_R2007_Header_pages_map_id_get;
*swig_pages_map_id_set = *LibreDWGc::Dwg_R2007_Header_pages_map_id_set;
*swig_header2_offset_get = *LibreDWGc::Dwg_R2007_Header_header2_offset_get;
*swig_header2_offset_set = *LibreDWGc::Dwg_R2007_Header_header2_offset_set;
*swig_pages_map_size_comp_get = *LibreDWGc::Dwg_R2007_Header_pages_map_size_comp_get;
*swig_pages_map_size_comp_set = *LibreDWGc::Dwg_R2007_Header_pages_map_size_comp_set;
*swig_pages_map_size_uncomp_get = *LibreDWGc::Dwg_R2007_Header_pages_map_size_uncomp_get;
*swig_pages_map_size_uncomp_set = *LibreDWGc::Dwg_R2007_Header_pages_map_size_uncomp_set;
*swig_pages_amount_get = *LibreDWGc::Dwg_R2007_Header_pages_amount_get;
*swig_pages_amount_set = *LibreDWGc::Dwg_R2007_Header_pages_amount_set;
*swig_pages_maxid_get = *LibreDWGc::Dwg_R2007_Header_pages_maxid_get;
*swig_pages_maxid_set = *LibreDWGc::Dwg_R2007_Header_pages_maxid_set;
*swig_unknown1_get = *LibreDWGc::Dwg_R2007_Header_unknown1_get;
*swig_unknown1_set = *LibreDWGc::Dwg_R2007_Header_unknown1_set;
*swig_unknown2_get = *LibreDWGc::Dwg_R2007_Header_unknown2_get;
*swig_unknown2_set = *LibreDWGc::Dwg_R2007_Header_unknown2_set;
*swig_pages_map_crc_uncomp_get = *LibreDWGc::Dwg_R2007_Header_pages_map_crc_uncomp_get;
*swig_pages_map_crc_uncomp_set = *LibreDWGc::Dwg_R2007_Header_pages_map_crc_uncomp_set;
*swig_unknown3_get = *LibreDWGc::Dwg_R2007_Header_unknown3_get;
*swig_unknown3_set = *LibreDWGc::Dwg_R2007_Header_unknown3_set;
*swig_unknown4_get = *LibreDWGc::Dwg_R2007_Header_unknown4_get;
*swig_unknown4_set = *LibreDWGc::Dwg_R2007_Header_unknown4_set;
*swig_unknown5_get = *LibreDWGc::Dwg_R2007_Header_unknown5_get;
*swig_unknown5_set = *LibreDWGc::Dwg_R2007_Header_unknown5_set;
*swig_num_sections_get = *LibreDWGc::Dwg_R2007_Header_num_sections_get;
*swig_num_sections_set = *LibreDWGc::Dwg_R2007_Header_num_sections_set;
*swig_sections_map_crc_uncomp_get = *LibreDWGc::Dwg_R2007_Header_sections_map_crc_uncomp_get;
*swig_sections_map_crc_uncomp_set = *LibreDWGc::Dwg_R2007_Header_sections_map_crc_uncomp_set;
*swig_sections_map_size_comp_get = *LibreDWGc::Dwg_R2007_Header_sections_map_size_comp_get;
*swig_sections_map_size_comp_set = *LibreDWGc::Dwg_R2007_Header_sections_map_size_comp_set;
*swig_sections_map2_id_get = *LibreDWGc::Dwg_R2007_Header_sections_map2_id_get;
*swig_sections_map2_id_set = *LibreDWGc::Dwg_R2007_Header_sections_map2_id_set;
*swig_sections_map_id_get = *LibreDWGc::Dwg_R2007_Header_sections_map_id_get;
*swig_sections_map_id_set = *LibreDWGc::Dwg_R2007_Header_sections_map_id_set;
*swig_sections_map_size_uncomp_get = *LibreDWGc::Dwg_R2007_Header_sections_map_size_uncomp_get;
*swig_sections_map_size_uncomp_set = *LibreDWGc::Dwg_R2007_Header_sections_map_size_uncomp_set;
*swig_sections_map_crc_comp_get = *LibreDWGc::Dwg_R2007_Header_sections_map_crc_comp_get;
*swig_sections_map_crc_comp_set = *LibreDWGc::Dwg_R2007_Header_sections_map_crc_comp_set;
*swig_sections_map_correction_get = *LibreDWGc::Dwg_R2007_Header_sections_map_correction_get;
*swig_sections_map_correction_set = *LibreDWGc::Dwg_R2007_Header_sections_map_correction_set;
*swig_sections_map_crc_seed_get = *LibreDWGc::Dwg_R2007_Header_sections_map_crc_seed_get;
*swig_sections_map_crc_seed_set = *LibreDWGc::Dwg_R2007_Header_sections_map_crc_seed_set;
*swig_stream_version_get = *LibreDWGc::Dwg_R2007_Header_stream_version_get;
*swig_stream_version_set = *LibreDWGc::Dwg_R2007_Header_stream_version_set;
*swig_crc_seed_get = *LibreDWGc::Dwg_R2007_Header_crc_seed_get;
*swig_crc_seed_set = *LibreDWGc::Dwg_R2007_Header_crc_seed_set;
*swig_crc_seed_encoded_get = *LibreDWGc::Dwg_R2007_Header_crc_seed_encoded_get;
*swig_crc_seed_encoded_set = *LibreDWGc::Dwg_R2007_Header_crc_seed_encoded_set;
*swig_random_seed_get = *LibreDWGc::Dwg_R2007_Header_random_seed_get;
*swig_random_seed_set = *LibreDWGc::Dwg_R2007_Header_random_seed_set;
*swig_header_crc_get = *LibreDWGc::Dwg_R2007_Header_header_crc_get;
*swig_header_crc_set = *LibreDWGc::Dwg_R2007_Header_header_crc_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_R2007_Header(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_R2007_Header($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_R2004_Header ##############

package LibreDWG::Dwg_R2004_Header;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_file_ID_string_get = *LibreDWGc::Dwg_R2004_Header_file_ID_string_get;
*swig_file_ID_string_set = *LibreDWGc::Dwg_R2004_Header_file_ID_string_set;
*swig_header_address_get = *LibreDWGc::Dwg_R2004_Header_header_address_get;
*swig_header_address_set = *LibreDWGc::Dwg_R2004_Header_header_address_set;
*swig_header_size_get = *LibreDWGc::Dwg_R2004_Header_header_size_get;
*swig_header_size_set = *LibreDWGc::Dwg_R2004_Header_header_size_set;
*swig_x04_get = *LibreDWGc::Dwg_R2004_Header_x04_get;
*swig_x04_set = *LibreDWGc::Dwg_R2004_Header_x04_set;
*swig_root_tree_node_gap_get = *LibreDWGc::Dwg_R2004_Header_root_tree_node_gap_get;
*swig_root_tree_node_gap_set = *LibreDWGc::Dwg_R2004_Header_root_tree_node_gap_set;
*swig_lowermost_left_tree_node_gap_get = *LibreDWGc::Dwg_R2004_Header_lowermost_left_tree_node_gap_get;
*swig_lowermost_left_tree_node_gap_set = *LibreDWGc::Dwg_R2004_Header_lowermost_left_tree_node_gap_set;
*swig_lowermost_right_tree_node_gap_get = *LibreDWGc::Dwg_R2004_Header_lowermost_right_tree_node_gap_get;
*swig_lowermost_right_tree_node_gap_set = *LibreDWGc::Dwg_R2004_Header_lowermost_right_tree_node_gap_set;
*swig_unknown_long_get = *LibreDWGc::Dwg_R2004_Header_unknown_long_get;
*swig_unknown_long_set = *LibreDWGc::Dwg_R2004_Header_unknown_long_set;
*swig_last_section_id_get = *LibreDWGc::Dwg_R2004_Header_last_section_id_get;
*swig_last_section_id_set = *LibreDWGc::Dwg_R2004_Header_last_section_id_set;
*swig_last_section_address_get = *LibreDWGc::Dwg_R2004_Header_last_section_address_get;
*swig_last_section_address_set = *LibreDWGc::Dwg_R2004_Header_last_section_address_set;
*swig_secondheader_address_get = *LibreDWGc::Dwg_R2004_Header_secondheader_address_get;
*swig_secondheader_address_set = *LibreDWGc::Dwg_R2004_Header_secondheader_address_set;
*swig_numgaps_get = *LibreDWGc::Dwg_R2004_Header_numgaps_get;
*swig_numgaps_set = *LibreDWGc::Dwg_R2004_Header_numgaps_set;
*swig_numsections_get = *LibreDWGc::Dwg_R2004_Header_numsections_get;
*swig_numsections_set = *LibreDWGc::Dwg_R2004_Header_numsections_set;
*swig_x20_get = *LibreDWGc::Dwg_R2004_Header_x20_get;
*swig_x20_set = *LibreDWGc::Dwg_R2004_Header_x20_set;
*swig_x80_get = *LibreDWGc::Dwg_R2004_Header_x80_get;
*swig_x80_set = *LibreDWGc::Dwg_R2004_Header_x80_set;
*swig_x40_get = *LibreDWGc::Dwg_R2004_Header_x40_get;
*swig_x40_set = *LibreDWGc::Dwg_R2004_Header_x40_set;
*swig_section_map_id_get = *LibreDWGc::Dwg_R2004_Header_section_map_id_get;
*swig_section_map_id_set = *LibreDWGc::Dwg_R2004_Header_section_map_id_set;
*swig_section_map_address_get = *LibreDWGc::Dwg_R2004_Header_section_map_address_get;
*swig_section_map_address_set = *LibreDWGc::Dwg_R2004_Header_section_map_address_set;
*swig_section_info_id_get = *LibreDWGc::Dwg_R2004_Header_section_info_id_get;
*swig_section_info_id_set = *LibreDWGc::Dwg_R2004_Header_section_info_id_set;
*swig_section_array_size_get = *LibreDWGc::Dwg_R2004_Header_section_array_size_get;
*swig_section_array_size_set = *LibreDWGc::Dwg_R2004_Header_section_array_size_set;
*swig_gap_array_size_get = *LibreDWGc::Dwg_R2004_Header_gap_array_size_get;
*swig_gap_array_size_set = *LibreDWGc::Dwg_R2004_Header_gap_array_size_set;
*swig_crc32_get = *LibreDWGc::Dwg_R2004_Header_crc32_get;
*swig_crc32_set = *LibreDWGc::Dwg_R2004_Header_crc32_set;
*swig_padding_get = *LibreDWGc::Dwg_R2004_Header_padding_get;
*swig_padding_set = *LibreDWGc::Dwg_R2004_Header_padding_set;
*swig_section_type_get = *LibreDWGc::Dwg_R2004_Header_section_type_get;
*swig_section_type_set = *LibreDWGc::Dwg_R2004_Header_section_type_set;
*swig_decomp_data_size_get = *LibreDWGc::Dwg_R2004_Header_decomp_data_size_get;
*swig_decomp_data_size_set = *LibreDWGc::Dwg_R2004_Header_decomp_data_size_set;
*swig_comp_data_size_get = *LibreDWGc::Dwg_R2004_Header_comp_data_size_get;
*swig_comp_data_size_set = *LibreDWGc::Dwg_R2004_Header_comp_data_size_set;
*swig_compression_type_get = *LibreDWGc::Dwg_R2004_Header_compression_type_get;
*swig_compression_type_set = *LibreDWGc::Dwg_R2004_Header_compression_type_set;
*swig_checksum_get = *LibreDWGc::Dwg_R2004_Header_checksum_get;
*swig_checksum_set = *LibreDWGc::Dwg_R2004_Header_checksum_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_R2004_Header(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_R2004_Header($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AuxHeader ##############

package LibreDWG::Dwg_AuxHeader;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_aux_intro_get = *LibreDWGc::Dwg_AuxHeader_aux_intro_get;
*swig_aux_intro_set = *LibreDWGc::Dwg_AuxHeader_aux_intro_set;
*swig_dwg_version_get = *LibreDWGc::Dwg_AuxHeader_dwg_version_get;
*swig_dwg_version_set = *LibreDWGc::Dwg_AuxHeader_dwg_version_set;
*swig_maint_version_get = *LibreDWGc::Dwg_AuxHeader_maint_version_get;
*swig_maint_version_set = *LibreDWGc::Dwg_AuxHeader_maint_version_set;
*swig_numsaves_get = *LibreDWGc::Dwg_AuxHeader_numsaves_get;
*swig_numsaves_set = *LibreDWGc::Dwg_AuxHeader_numsaves_set;
*swig_minus_1_get = *LibreDWGc::Dwg_AuxHeader_minus_1_get;
*swig_minus_1_set = *LibreDWGc::Dwg_AuxHeader_minus_1_set;
*swig_numsaves_1_get = *LibreDWGc::Dwg_AuxHeader_numsaves_1_get;
*swig_numsaves_1_set = *LibreDWGc::Dwg_AuxHeader_numsaves_1_set;
*swig_numsaves_2_get = *LibreDWGc::Dwg_AuxHeader_numsaves_2_get;
*swig_numsaves_2_set = *LibreDWGc::Dwg_AuxHeader_numsaves_2_set;
*swig_zero_get = *LibreDWGc::Dwg_AuxHeader_zero_get;
*swig_zero_set = *LibreDWGc::Dwg_AuxHeader_zero_set;
*swig_dwg_version_1_get = *LibreDWGc::Dwg_AuxHeader_dwg_version_1_get;
*swig_dwg_version_1_set = *LibreDWGc::Dwg_AuxHeader_dwg_version_1_set;
*swig_maint_version_1_get = *LibreDWGc::Dwg_AuxHeader_maint_version_1_get;
*swig_maint_version_1_set = *LibreDWGc::Dwg_AuxHeader_maint_version_1_set;
*swig_dwg_version_2_get = *LibreDWGc::Dwg_AuxHeader_dwg_version_2_get;
*swig_dwg_version_2_set = *LibreDWGc::Dwg_AuxHeader_dwg_version_2_set;
*swig_maint_version_2_get = *LibreDWGc::Dwg_AuxHeader_maint_version_2_get;
*swig_maint_version_2_set = *LibreDWGc::Dwg_AuxHeader_maint_version_2_set;
*swig_unknown_6rs_get = *LibreDWGc::Dwg_AuxHeader_unknown_6rs_get;
*swig_unknown_6rs_set = *LibreDWGc::Dwg_AuxHeader_unknown_6rs_set;
*swig_unknown_5rl_get = *LibreDWGc::Dwg_AuxHeader_unknown_5rl_get;
*swig_unknown_5rl_set = *LibreDWGc::Dwg_AuxHeader_unknown_5rl_set;
*swig_TDCREATE_get = *LibreDWGc::Dwg_AuxHeader_TDCREATE_get;
*swig_TDCREATE_set = *LibreDWGc::Dwg_AuxHeader_TDCREATE_set;
*swig_TDUPDATE_get = *LibreDWGc::Dwg_AuxHeader_TDUPDATE_get;
*swig_TDUPDATE_set = *LibreDWGc::Dwg_AuxHeader_TDUPDATE_set;
*swig_HANDSEED_get = *LibreDWGc::Dwg_AuxHeader_HANDSEED_get;
*swig_HANDSEED_set = *LibreDWGc::Dwg_AuxHeader_HANDSEED_set;
*swig_plot_stamp_get = *LibreDWGc::Dwg_AuxHeader_plot_stamp_get;
*swig_plot_stamp_set = *LibreDWGc::Dwg_AuxHeader_plot_stamp_set;
*swig_zero_1_get = *LibreDWGc::Dwg_AuxHeader_zero_1_get;
*swig_zero_1_set = *LibreDWGc::Dwg_AuxHeader_zero_1_set;
*swig_numsaves_3_get = *LibreDWGc::Dwg_AuxHeader_numsaves_3_get;
*swig_numsaves_3_set = *LibreDWGc::Dwg_AuxHeader_numsaves_3_set;
*swig_zero_2_get = *LibreDWGc::Dwg_AuxHeader_zero_2_get;
*swig_zero_2_set = *LibreDWGc::Dwg_AuxHeader_zero_2_set;
*swig_zero_3_get = *LibreDWGc::Dwg_AuxHeader_zero_3_get;
*swig_zero_3_set = *LibreDWGc::Dwg_AuxHeader_zero_3_set;
*swig_zero_4_get = *LibreDWGc::Dwg_AuxHeader_zero_4_get;
*swig_zero_4_set = *LibreDWGc::Dwg_AuxHeader_zero_4_set;
*swig_numsaves_4_get = *LibreDWGc::Dwg_AuxHeader_numsaves_4_get;
*swig_numsaves_4_set = *LibreDWGc::Dwg_AuxHeader_numsaves_4_set;
*swig_zero_5_get = *LibreDWGc::Dwg_AuxHeader_zero_5_get;
*swig_zero_5_set = *LibreDWGc::Dwg_AuxHeader_zero_5_set;
*swig_zero_6_get = *LibreDWGc::Dwg_AuxHeader_zero_6_get;
*swig_zero_6_set = *LibreDWGc::Dwg_AuxHeader_zero_6_set;
*swig_zero_7_get = *LibreDWGc::Dwg_AuxHeader_zero_7_get;
*swig_zero_7_set = *LibreDWGc::Dwg_AuxHeader_zero_7_set;
*swig_zero_8_get = *LibreDWGc::Dwg_AuxHeader_zero_8_get;
*swig_zero_8_set = *LibreDWGc::Dwg_AuxHeader_zero_8_set;
*swig_zero_18_get = *LibreDWGc::Dwg_AuxHeader_zero_18_get;
*swig_zero_18_set = *LibreDWGc::Dwg_AuxHeader_zero_18_set;
*swig_num_auxheader_variables_get = *LibreDWGc::Dwg_AuxHeader_num_auxheader_variables_get;
*swig_num_auxheader_variables_set = *LibreDWGc::Dwg_AuxHeader_num_auxheader_variables_set;
*swig_auxheader_size_get = *LibreDWGc::Dwg_AuxHeader_auxheader_size_get;
*swig_auxheader_size_set = *LibreDWGc::Dwg_AuxHeader_auxheader_size_set;
*swig_entities_start_get = *LibreDWGc::Dwg_AuxHeader_entities_start_get;
*swig_entities_start_set = *LibreDWGc::Dwg_AuxHeader_entities_start_set;
*swig_entities_end_get = *LibreDWGc::Dwg_AuxHeader_entities_end_get;
*swig_entities_end_set = *LibreDWGc::Dwg_AuxHeader_entities_end_set;
*swig_blocks_start_get = *LibreDWGc::Dwg_AuxHeader_blocks_start_get;
*swig_blocks_start_set = *LibreDWGc::Dwg_AuxHeader_blocks_start_set;
*swig_extras_start_get = *LibreDWGc::Dwg_AuxHeader_extras_start_get;
*swig_extras_start_set = *LibreDWGc::Dwg_AuxHeader_extras_start_set;
*swig_auxheader_address_get = *LibreDWGc::Dwg_AuxHeader_auxheader_address_get;
*swig_auxheader_address_set = *LibreDWGc::Dwg_AuxHeader_auxheader_address_set;
*swig_num_aux_tables_get = *LibreDWGc::Dwg_AuxHeader_num_aux_tables_get;
*swig_num_aux_tables_set = *LibreDWGc::Dwg_AuxHeader_num_aux_tables_set;
*swig_R11_HANDLING_get = *LibreDWGc::Dwg_AuxHeader_R11_HANDLING_get;
*swig_R11_HANDLING_set = *LibreDWGc::Dwg_AuxHeader_R11_HANDLING_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AuxHeader(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AuxHeader($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SummaryInfo ##############

package LibreDWG::Dwg_SummaryInfo;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_TITLE_get = *LibreDWGc::Dwg_SummaryInfo_TITLE_get;
*swig_TITLE_set = *LibreDWGc::Dwg_SummaryInfo_TITLE_set;
*swig_SUBJECT_get = *LibreDWGc::Dwg_SummaryInfo_SUBJECT_get;
*swig_SUBJECT_set = *LibreDWGc::Dwg_SummaryInfo_SUBJECT_set;
*swig_AUTHOR_get = *LibreDWGc::Dwg_SummaryInfo_AUTHOR_get;
*swig_AUTHOR_set = *LibreDWGc::Dwg_SummaryInfo_AUTHOR_set;
*swig_KEYWORDS_get = *LibreDWGc::Dwg_SummaryInfo_KEYWORDS_get;
*swig_KEYWORDS_set = *LibreDWGc::Dwg_SummaryInfo_KEYWORDS_set;
*swig_COMMENTS_get = *LibreDWGc::Dwg_SummaryInfo_COMMENTS_get;
*swig_COMMENTS_set = *LibreDWGc::Dwg_SummaryInfo_COMMENTS_set;
*swig_LASTSAVEDBY_get = *LibreDWGc::Dwg_SummaryInfo_LASTSAVEDBY_get;
*swig_LASTSAVEDBY_set = *LibreDWGc::Dwg_SummaryInfo_LASTSAVEDBY_set;
*swig_REVISIONNUMBER_get = *LibreDWGc::Dwg_SummaryInfo_REVISIONNUMBER_get;
*swig_REVISIONNUMBER_set = *LibreDWGc::Dwg_SummaryInfo_REVISIONNUMBER_set;
*swig_HYPERLINKBASE_get = *LibreDWGc::Dwg_SummaryInfo_HYPERLINKBASE_get;
*swig_HYPERLINKBASE_set = *LibreDWGc::Dwg_SummaryInfo_HYPERLINKBASE_set;
*swig_TDINDWG_get = *LibreDWGc::Dwg_SummaryInfo_TDINDWG_get;
*swig_TDINDWG_set = *LibreDWGc::Dwg_SummaryInfo_TDINDWG_set;
*swig_TDCREATE_get = *LibreDWGc::Dwg_SummaryInfo_TDCREATE_get;
*swig_TDCREATE_set = *LibreDWGc::Dwg_SummaryInfo_TDCREATE_set;
*swig_TDUPDATE_get = *LibreDWGc::Dwg_SummaryInfo_TDUPDATE_get;
*swig_TDUPDATE_set = *LibreDWGc::Dwg_SummaryInfo_TDUPDATE_set;
*swig_num_props_get = *LibreDWGc::Dwg_SummaryInfo_num_props_get;
*swig_num_props_set = *LibreDWGc::Dwg_SummaryInfo_num_props_set;
*swig_props_get = *LibreDWGc::Dwg_SummaryInfo_props_get;
*swig_props_set = *LibreDWGc::Dwg_SummaryInfo_props_set;
*swig_unknown1_get = *LibreDWGc::Dwg_SummaryInfo_unknown1_get;
*swig_unknown1_set = *LibreDWGc::Dwg_SummaryInfo_unknown1_set;
*swig_unknown2_get = *LibreDWGc::Dwg_SummaryInfo_unknown2_get;
*swig_unknown2_set = *LibreDWGc::Dwg_SummaryInfo_unknown2_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SummaryInfo(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SummaryInfo($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AppInfo ##############

package LibreDWG::Dwg_AppInfo;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_class_version_get = *LibreDWGc::Dwg_AppInfo_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_AppInfo_class_version_set;
*swig_num_strings_get = *LibreDWGc::Dwg_AppInfo_num_strings_get;
*swig_num_strings_set = *LibreDWGc::Dwg_AppInfo_num_strings_set;
*swig_appinfo_name_get = *LibreDWGc::Dwg_AppInfo_appinfo_name_get;
*swig_appinfo_name_set = *LibreDWGc::Dwg_AppInfo_appinfo_name_set;
*swig_version_checksum_get = *LibreDWGc::Dwg_AppInfo_version_checksum_get;
*swig_version_checksum_set = *LibreDWGc::Dwg_AppInfo_version_checksum_set;
*swig_comment_checksum_get = *LibreDWGc::Dwg_AppInfo_comment_checksum_get;
*swig_comment_checksum_set = *LibreDWGc::Dwg_AppInfo_comment_checksum_set;
*swig_product_checksum_get = *LibreDWGc::Dwg_AppInfo_product_checksum_get;
*swig_product_checksum_set = *LibreDWGc::Dwg_AppInfo_product_checksum_set;
*swig_version_get = *LibreDWGc::Dwg_AppInfo_version_get;
*swig_version_set = *LibreDWGc::Dwg_AppInfo_version_set;
*swig_comment_get = *LibreDWGc::Dwg_AppInfo_comment_get;
*swig_comment_set = *LibreDWGc::Dwg_AppInfo_comment_set;
*swig_product_info_get = *LibreDWGc::Dwg_AppInfo_product_info_get;
*swig_product_info_set = *LibreDWGc::Dwg_AppInfo_product_info_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AppInfo(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AppInfo($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_FileDepList ##############

package LibreDWG::Dwg_FileDepList;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_features_get = *LibreDWGc::Dwg_FileDepList_num_features_get;
*swig_num_features_set = *LibreDWGc::Dwg_FileDepList_num_features_set;
*swig_features_get = *LibreDWGc::Dwg_FileDepList_features_get;
*swig_features_set = *LibreDWGc::Dwg_FileDepList_features_set;
*swig_num_files_get = *LibreDWGc::Dwg_FileDepList_num_files_get;
*swig_num_files_set = *LibreDWGc::Dwg_FileDepList_num_files_set;
*swig_files_get = *LibreDWGc::Dwg_FileDepList_files_get;
*swig_files_set = *LibreDWGc::Dwg_FileDepList_files_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_FileDepList(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_FileDepList($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Security ##############

package LibreDWG::Dwg_Security;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_unknown_1_get = *LibreDWGc::Dwg_Security_unknown_1_get;
*swig_unknown_1_set = *LibreDWGc::Dwg_Security_unknown_1_set;
*swig_unknown_2_get = *LibreDWGc::Dwg_Security_unknown_2_get;
*swig_unknown_2_set = *LibreDWGc::Dwg_Security_unknown_2_set;
*swig_unknown_3_get = *LibreDWGc::Dwg_Security_unknown_3_get;
*swig_unknown_3_set = *LibreDWGc::Dwg_Security_unknown_3_set;
*swig_crypto_id_get = *LibreDWGc::Dwg_Security_crypto_id_get;
*swig_crypto_id_set = *LibreDWGc::Dwg_Security_crypto_id_set;
*swig_crypto_name_get = *LibreDWGc::Dwg_Security_crypto_name_get;
*swig_crypto_name_set = *LibreDWGc::Dwg_Security_crypto_name_set;
*swig_algo_id_get = *LibreDWGc::Dwg_Security_algo_id_get;
*swig_algo_id_set = *LibreDWGc::Dwg_Security_algo_id_set;
*swig_key_len_get = *LibreDWGc::Dwg_Security_key_len_get;
*swig_key_len_set = *LibreDWGc::Dwg_Security_key_len_set;
*swig_encr_size_get = *LibreDWGc::Dwg_Security_encr_size_get;
*swig_encr_size_set = *LibreDWGc::Dwg_Security_encr_size_set;
*swig_encr_buffer_get = *LibreDWGc::Dwg_Security_encr_buffer_get;
*swig_encr_buffer_set = *LibreDWGc::Dwg_Security_encr_buffer_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Security(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Security($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_VBAProject ##############

package LibreDWG::Dwg_VBAProject;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_VBAProject_size_get;
*swig_size_set = *LibreDWGc::Dwg_VBAProject_size_set;
*swig_unknown_bits_get = *LibreDWGc::Dwg_VBAProject_unknown_bits_get;
*swig_unknown_bits_set = *LibreDWGc::Dwg_VBAProject_unknown_bits_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_VBAProject(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_VBAProject($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_AppInfoHistory ##############

package LibreDWG::Dwg_AppInfoHistory;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_AppInfoHistory_size_get;
*swig_size_set = *LibreDWGc::Dwg_AppInfoHistory_size_set;
*swig_unknown_bits_get = *LibreDWGc::Dwg_AppInfoHistory_unknown_bits_get;
*swig_unknown_bits_set = *LibreDWGc::Dwg_AppInfoHistory_unknown_bits_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_AppInfoHistory(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_AppInfoHistory($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_RevHistory ##############

package LibreDWG::Dwg_RevHistory;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_class_version_get = *LibreDWGc::Dwg_RevHistory_class_version_get;
*swig_class_version_set = *LibreDWGc::Dwg_RevHistory_class_version_set;
*swig_class_minor_get = *LibreDWGc::Dwg_RevHistory_class_minor_get;
*swig_class_minor_set = *LibreDWGc::Dwg_RevHistory_class_minor_set;
*swig_num_histories_get = *LibreDWGc::Dwg_RevHistory_num_histories_get;
*swig_num_histories_set = *LibreDWGc::Dwg_RevHistory_num_histories_set;
*swig_histories_get = *LibreDWGc::Dwg_RevHistory_histories_get;
*swig_histories_set = *LibreDWGc::Dwg_RevHistory_histories_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_RevHistory(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_RevHistory($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_ObjFreeSpace ##############

package LibreDWG::Dwg_ObjFreeSpace;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_zero_get = *LibreDWGc::Dwg_ObjFreeSpace_zero_get;
*swig_zero_set = *LibreDWGc::Dwg_ObjFreeSpace_zero_set;
*swig_numhandles_get = *LibreDWGc::Dwg_ObjFreeSpace_numhandles_get;
*swig_numhandles_set = *LibreDWGc::Dwg_ObjFreeSpace_numhandles_set;
*swig_TDUPDATE_get = *LibreDWGc::Dwg_ObjFreeSpace_TDUPDATE_get;
*swig_TDUPDATE_set = *LibreDWGc::Dwg_ObjFreeSpace_TDUPDATE_set;
*swig_objects_address_get = *LibreDWGc::Dwg_ObjFreeSpace_objects_address_get;
*swig_objects_address_set = *LibreDWGc::Dwg_ObjFreeSpace_objects_address_set;
*swig_numnums_get = *LibreDWGc::Dwg_ObjFreeSpace_numnums_get;
*swig_numnums_set = *LibreDWGc::Dwg_ObjFreeSpace_numnums_set;
*swig_max32_get = *LibreDWGc::Dwg_ObjFreeSpace_max32_get;
*swig_max32_set = *LibreDWGc::Dwg_ObjFreeSpace_max32_set;
*swig_max64_get = *LibreDWGc::Dwg_ObjFreeSpace_max64_get;
*swig_max64_set = *LibreDWGc::Dwg_ObjFreeSpace_max64_set;
*swig_maxtbl_get = *LibreDWGc::Dwg_ObjFreeSpace_maxtbl_get;
*swig_maxtbl_set = *LibreDWGc::Dwg_ObjFreeSpace_maxtbl_set;
*swig_maxrl_get = *LibreDWGc::Dwg_ObjFreeSpace_maxrl_get;
*swig_maxrl_set = *LibreDWGc::Dwg_ObjFreeSpace_maxrl_set;
*swig_max32_hi_get = *LibreDWGc::Dwg_ObjFreeSpace_max32_hi_get;
*swig_max32_hi_set = *LibreDWGc::Dwg_ObjFreeSpace_max32_hi_set;
*swig_max64_hi_get = *LibreDWGc::Dwg_ObjFreeSpace_max64_hi_get;
*swig_max64_hi_set = *LibreDWGc::Dwg_ObjFreeSpace_max64_hi_set;
*swig_maxtbl_hi_get = *LibreDWGc::Dwg_ObjFreeSpace_maxtbl_hi_get;
*swig_maxtbl_hi_set = *LibreDWGc::Dwg_ObjFreeSpace_maxtbl_hi_set;
*swig_maxrl_hi_get = *LibreDWGc::Dwg_ObjFreeSpace_maxrl_hi_get;
*swig_maxrl_hi_set = *LibreDWGc::Dwg_ObjFreeSpace_maxrl_hi_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_ObjFreeSpace(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_ObjFreeSpace($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Template ##############

package LibreDWG::Dwg_Template;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_description_get = *LibreDWGc::Dwg_Template_description_get;
*swig_description_set = *LibreDWGc::Dwg_Template_description_set;
*swig_MEASUREMENT_get = *LibreDWGc::Dwg_Template_MEASUREMENT_get;
*swig_MEASUREMENT_set = *LibreDWGc::Dwg_Template_MEASUREMENT_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Template(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Template($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SecondHeader_Sections ##############

package LibreDWG::Dwg_SecondHeader_Sections;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_nr_get = *LibreDWGc::Dwg_SecondHeader_Sections_nr_get;
*swig_nr_set = *LibreDWGc::Dwg_SecondHeader_Sections_nr_set;
*swig_address_get = *LibreDWGc::Dwg_SecondHeader_Sections_address_get;
*swig_address_set = *LibreDWGc::Dwg_SecondHeader_Sections_address_set;
*swig_size_get = *LibreDWGc::Dwg_SecondHeader_Sections_size_get;
*swig_size_set = *LibreDWGc::Dwg_SecondHeader_Sections_size_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SecondHeader_Sections(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SecondHeader_Sections($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SecondHeader_Handles ##############

package LibreDWG::Dwg_SecondHeader_Handles;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_num_hdl_get = *LibreDWGc::Dwg_SecondHeader_Handles_num_hdl_get;
*swig_num_hdl_set = *LibreDWGc::Dwg_SecondHeader_Handles_num_hdl_set;
*swig_nr_get = *LibreDWGc::Dwg_SecondHeader_Handles_nr_get;
*swig_nr_set = *LibreDWGc::Dwg_SecondHeader_Handles_nr_set;
*swig_hdl_get = *LibreDWGc::Dwg_SecondHeader_Handles_hdl_get;
*swig_hdl_set = *LibreDWGc::Dwg_SecondHeader_Handles_hdl_set;
*swig_name_get = *LibreDWGc::Dwg_SecondHeader_Handles_name_get;
*swig_name_set = *LibreDWGc::Dwg_SecondHeader_Handles_name_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SecondHeader_Handles(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SecondHeader_Handles($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_SecondHeader ##############

package LibreDWG::Dwg_SecondHeader;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_size_get = *LibreDWGc::Dwg_SecondHeader_size_get;
*swig_size_set = *LibreDWGc::Dwg_SecondHeader_size_set;
*swig_address_get = *LibreDWGc::Dwg_SecondHeader_address_get;
*swig_address_set = *LibreDWGc::Dwg_SecondHeader_address_set;
*swig_version_get = *LibreDWGc::Dwg_SecondHeader_version_get;
*swig_version_set = *LibreDWGc::Dwg_SecondHeader_version_set;
*swig_is_maint_get = *LibreDWGc::Dwg_SecondHeader_is_maint_get;
*swig_is_maint_set = *LibreDWGc::Dwg_SecondHeader_is_maint_set;
*swig_zero_one_or_three_get = *LibreDWGc::Dwg_SecondHeader_zero_one_or_three_get;
*swig_zero_one_or_three_set = *LibreDWGc::Dwg_SecondHeader_zero_one_or_three_set;
*swig_dwg_versions_get = *LibreDWGc::Dwg_SecondHeader_dwg_versions_get;
*swig_dwg_versions_set = *LibreDWGc::Dwg_SecondHeader_dwg_versions_set;
*swig_codepage_get = *LibreDWGc::Dwg_SecondHeader_codepage_get;
*swig_codepage_set = *LibreDWGc::Dwg_SecondHeader_codepage_set;
*swig_num_sections_get = *LibreDWGc::Dwg_SecondHeader_num_sections_get;
*swig_num_sections_set = *LibreDWGc::Dwg_SecondHeader_num_sections_set;
*swig_sections_get = *LibreDWGc::Dwg_SecondHeader_sections_get;
*swig_sections_set = *LibreDWGc::Dwg_SecondHeader_sections_set;
*swig_num_handles_get = *LibreDWGc::Dwg_SecondHeader_num_handles_get;
*swig_num_handles_set = *LibreDWGc::Dwg_SecondHeader_num_handles_set;
*swig_handles_get = *LibreDWGc::Dwg_SecondHeader_handles_get;
*swig_handles_set = *LibreDWGc::Dwg_SecondHeader_handles_set;
*swig_crc_get = *LibreDWGc::Dwg_SecondHeader_crc_get;
*swig_crc_set = *LibreDWGc::Dwg_SecondHeader_crc_set;
*swig_junk_r14_get = *LibreDWGc::Dwg_SecondHeader_junk_r14_get;
*swig_junk_r14_set = *LibreDWGc::Dwg_SecondHeader_junk_r14_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_SecondHeader(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_SecondHeader($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_Data ##############

package LibreDWG::Dwg_Data;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_header_get = *LibreDWGc::Dwg_Data_header_get;
*swig_header_set = *LibreDWGc::Dwg_Data_header_set;
*swig_num_classes_get = *LibreDWGc::Dwg_Data_num_classes_get;
*swig_num_classes_set = *LibreDWGc::Dwg_Data_num_classes_set;
*swig_dwg_class_get = *LibreDWGc::Dwg_Data_dwg_class_get;
*swig_dwg_class_set = *LibreDWGc::Dwg_Data_dwg_class_set;
*swig_num_objects_get = *LibreDWGc::Dwg_Data_num_objects_get;
*swig_num_objects_set = *LibreDWGc::Dwg_Data_num_objects_set;
*swig_num_alloced_objects_get = *LibreDWGc::Dwg_Data_num_alloced_objects_get;
*swig_num_alloced_objects_set = *LibreDWGc::Dwg_Data_num_alloced_objects_set;
*swig_object_get = *LibreDWGc::Dwg_Data_object_get;
*swig_object_set = *LibreDWGc::Dwg_Data_object_set;
*swig_num_entities_get = *LibreDWGc::Dwg_Data_num_entities_get;
*swig_num_entities_set = *LibreDWGc::Dwg_Data_num_entities_set;
*swig_num_object_refs_get = *LibreDWGc::Dwg_Data_num_object_refs_get;
*swig_num_object_refs_set = *LibreDWGc::Dwg_Data_num_object_refs_set;
*swig_cur_index_get = *LibreDWGc::Dwg_Data_cur_index_get;
*swig_cur_index_set = *LibreDWGc::Dwg_Data_cur_index_set;
*swig_object_ref_get = *LibreDWGc::Dwg_Data_object_ref_get;
*swig_object_ref_set = *LibreDWGc::Dwg_Data_object_ref_set;
*swig_object_map_get = *LibreDWGc::Dwg_Data_object_map_get;
*swig_object_map_set = *LibreDWGc::Dwg_Data_object_map_set;
*swig_dirty_refs_get = *LibreDWGc::Dwg_Data_dirty_refs_get;
*swig_dirty_refs_set = *LibreDWGc::Dwg_Data_dirty_refs_set;
*swig_opts_get = *LibreDWGc::Dwg_Data_opts_get;
*swig_opts_set = *LibreDWGc::Dwg_Data_opts_set;
*swig_header_vars_get = *LibreDWGc::Dwg_Data_header_vars_get;
*swig_header_vars_set = *LibreDWGc::Dwg_Data_header_vars_set;
*swig_thumbnail_get = *LibreDWGc::Dwg_Data_thumbnail_get;
*swig_thumbnail_set = *LibreDWGc::Dwg_Data_thumbnail_set;
*swig_fhdr_get = *LibreDWGc::Dwg_Data_fhdr_get;
*swig_fhdr_set = *LibreDWGc::Dwg_Data_fhdr_set;
*swig_mspace_block_get = *LibreDWGc::Dwg_Data_mspace_block_get;
*swig_mspace_block_set = *LibreDWGc::Dwg_Data_mspace_block_set;
*swig_pspace_block_get = *LibreDWGc::Dwg_Data_pspace_block_get;
*swig_pspace_block_set = *LibreDWGc::Dwg_Data_pspace_block_set;
*swig_block_control_get = *LibreDWGc::Dwg_Data_block_control_get;
*swig_block_control_set = *LibreDWGc::Dwg_Data_block_control_set;
*swig_auxheader_get = *LibreDWGc::Dwg_Data_auxheader_get;
*swig_auxheader_set = *LibreDWGc::Dwg_Data_auxheader_set;
*swig_secondheader_get = *LibreDWGc::Dwg_Data_secondheader_get;
*swig_secondheader_set = *LibreDWGc::Dwg_Data_secondheader_set;
*swig_summaryinfo_get = *LibreDWGc::Dwg_Data_summaryinfo_get;
*swig_summaryinfo_set = *LibreDWGc::Dwg_Data_summaryinfo_set;
*swig_appinfo_get = *LibreDWGc::Dwg_Data_appinfo_get;
*swig_appinfo_set = *LibreDWGc::Dwg_Data_appinfo_set;
*swig_filedeplist_get = *LibreDWGc::Dwg_Data_filedeplist_get;
*swig_filedeplist_set = *LibreDWGc::Dwg_Data_filedeplist_set;
*swig_security_get = *LibreDWGc::Dwg_Data_security_get;
*swig_security_set = *LibreDWGc::Dwg_Data_security_set;
*swig_vbaproject_get = *LibreDWGc::Dwg_Data_vbaproject_get;
*swig_vbaproject_set = *LibreDWGc::Dwg_Data_vbaproject_set;
*swig_appinfohistory_get = *LibreDWGc::Dwg_Data_appinfohistory_get;
*swig_appinfohistory_set = *LibreDWGc::Dwg_Data_appinfohistory_set;
*swig_revhistory_get = *LibreDWGc::Dwg_Data_revhistory_get;
*swig_revhistory_set = *LibreDWGc::Dwg_Data_revhistory_set;
*swig_objfreespace_get = *LibreDWGc::Dwg_Data_objfreespace_get;
*swig_objfreespace_set = *LibreDWGc::Dwg_Data_objfreespace_set;
*swig_Template_get = *LibreDWGc::Dwg_Data_Template_get;
*swig_Template_set = *LibreDWGc::Dwg_Data_Template_set;
*swig_acds_get = *LibreDWGc::Dwg_Data_acds_get;
*swig_acds_set = *LibreDWGc::Dwg_Data_acds_set;
*swig_layout_type_get = *LibreDWGc::Dwg_Data_layout_type_get;
*swig_layout_type_set = *LibreDWGc::Dwg_Data_layout_type_set;
*swig_num_acis_sab_hdl_get = *LibreDWGc::Dwg_Data_num_acis_sab_hdl_get;
*swig_num_acis_sab_hdl_set = *LibreDWGc::Dwg_Data_num_acis_sab_hdl_set;
*swig_acis_sab_hdl_get = *LibreDWGc::Dwg_Data_acis_sab_hdl_get;
*swig_acis_sab_hdl_set = *LibreDWGc::Dwg_Data_acis_sab_hdl_set;
*swig_next_hdl_get = *LibreDWGc::Dwg_Data_next_hdl_get;
*swig_next_hdl_set = *LibreDWGc::Dwg_Data_next_hdl_set;
*swig_prev_entity_index_get = *LibreDWGc::Dwg_Data_prev_entity_index_get;
*swig_prev_entity_index_set = *LibreDWGc::Dwg_Data_prev_entity_index_set;
*swig_object_ordered_ref_get = *LibreDWGc::Dwg_Data_object_ordered_ref_get;
*swig_object_ordered_ref_set = *LibreDWGc::Dwg_Data_object_ordered_ref_set;
*swig_num_object_ordered_refs_get = *LibreDWGc::Dwg_Data_num_object_ordered_refs_get;
*swig_num_object_ordered_refs_set = *LibreDWGc::Dwg_Data_num_object_ordered_refs_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_Data(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_Data($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::_dwg_struct_fhdr ##############

package LibreDWG::_dwg_struct_fhdr;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_r2004_header_get = *LibreDWGc::_dwg_struct_fhdr_r2004_header_get;
*swig_r2004_header_set = *LibreDWGc::_dwg_struct_fhdr_r2004_header_set;
*swig_r2007_file_header_get = *LibreDWGc::_dwg_struct_fhdr_r2007_file_header_get;
*swig_r2007_file_header_set = *LibreDWGc::_dwg_struct_fhdr_r2007_file_header_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new__dwg_struct_fhdr(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete__dwg_struct_fhdr($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_RGB_Palette ##############

package LibreDWG::Dwg_RGB_Palette;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_r_get = *LibreDWGc::Dwg_RGB_Palette_r_get;
*swig_r_set = *LibreDWGc::Dwg_RGB_Palette_r_set;
*swig_g_get = *LibreDWGc::Dwg_RGB_Palette_g_get;
*swig_g_set = *LibreDWGc::Dwg_RGB_Palette_g_set;
*swig_b_get = *LibreDWGc::Dwg_RGB_Palette_b_get;
*swig_b_set = *LibreDWGc::Dwg_RGB_Palette_b_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_RGB_Palette(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_RGB_Palette($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


############# Class : LibreDWG::Dwg_DYNAPI_field ##############

package LibreDWG::Dwg_DYNAPI_field;
use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
@ISA = qw( LibreDWG );
%OWNER = ();
%ITERATORS = ();
*swig_name_get = *LibreDWGc::Dwg_DYNAPI_field_name_get;
*swig_name_set = *LibreDWGc::Dwg_DYNAPI_field_name_set;
*swig_type_get = *LibreDWGc::Dwg_DYNAPI_field_type_get;
*swig_type_set = *LibreDWGc::Dwg_DYNAPI_field_type_set;
*swig_size_get = *LibreDWGc::Dwg_DYNAPI_field_size_get;
*swig_size_set = *LibreDWGc::Dwg_DYNAPI_field_size_set;
*swig_offset_get = *LibreDWGc::Dwg_DYNAPI_field_offset_get;
*swig_offset_set = *LibreDWGc::Dwg_DYNAPI_field_offset_set;
*swig_is_indirect_get = *LibreDWGc::Dwg_DYNAPI_field_is_indirect_get;
*swig_is_indirect_set = *LibreDWGc::Dwg_DYNAPI_field_is_indirect_set;
*swig_is_malloc_get = *LibreDWGc::Dwg_DYNAPI_field_is_malloc_get;
*swig_is_malloc_set = *LibreDWGc::Dwg_DYNAPI_field_is_malloc_set;
*swig_is_string_get = *LibreDWGc::Dwg_DYNAPI_field_is_string_get;
*swig_is_string_set = *LibreDWGc::Dwg_DYNAPI_field_is_string_set;
*swig_dxf_get = *LibreDWGc::Dwg_DYNAPI_field_dxf_get;
*swig_dxf_set = *LibreDWGc::Dwg_DYNAPI_field_dxf_set;
sub new {
    my $pkg = shift;
    my $self = LibreDWGc::new_Dwg_DYNAPI_field(@_);
    bless $self, $pkg if defined($self);
}

sub DESTROY {
    return unless $_[0]->isa('HASH');
    my $self = tied(%{$_[0]});
    return unless defined $self;
    delete $ITERATORS{$self};
    if (exists $OWNER{$self}) {
        LibreDWGc::delete_Dwg_DYNAPI_field($self);
        delete $OWNER{$self};
    }
}

sub DISOWN {
    my $self = shift;
    my $ptr = tied(%$self);
    delete $OWNER{$ptr};
}

sub ACQUIRE {
    my $self = shift;
    my $ptr = tied(%$self);
    $OWNER{$ptr} = 1;
}


# ------- VARIABLE STUBS --------

package LibreDWG;

*LIBREDWG_VERSION_MAJOR = *LibreDWGc::LIBREDWG_VERSION_MAJOR;
*LIBREDWG_VERSION_MINOR = *LibreDWGc::LIBREDWG_VERSION_MINOR;
*LIBREDWG_VERSION = *LibreDWGc::LIBREDWG_VERSION;
*FORMAT_RC = *LibreDWGc::FORMAT_RC;
*SCANF_2X = *LibreDWGc::SCANF_2X;
*FORMAT_RCd = *LibreDWGc::FORMAT_RCd;
*FORMAT_RCu = *LibreDWGc::FORMAT_RCu;
*FORMAT_RCx = *LibreDWGc::FORMAT_RCx;
*FORMAT_B = *LibreDWGc::FORMAT_B;
*FORMAT_BB = *LibreDWGc::FORMAT_BB;
*FORMAT_3B = *LibreDWGc::FORMAT_3B;
*FORMAT_RD = *LibreDWGc::FORMAT_RD;
*FORMAT_TF = *LibreDWGc::FORMAT_TF;
*FORMAT_TV = *LibreDWGc::FORMAT_TV;
*FORMAT_T16 = *LibreDWGc::FORMAT_T16;
*FORMAT_T32 = *LibreDWGc::FORMAT_T32;
*FORMAT_TU32 = *LibreDWGc::FORMAT_TU32;
*FORMAT_BT = *LibreDWGc::FORMAT_BT;
*FORMAT_DD = *LibreDWGc::FORMAT_DD;
*FORMAT_BD = *LibreDWGc::FORMAT_BD;
*FORMAT_4BITS = *LibreDWGc::FORMAT_4BITS;
*FORMAT_D2T = *LibreDWGc::FORMAT_D2T;
*FORMAT_TU = *LibreDWGc::FORMAT_TU;
*R_INVALID = *LibreDWGc::R_INVALID;
*R_1_1 = *LibreDWGc::R_1_1;
*R_1_2 = *LibreDWGc::R_1_2;
*R_1_3 = *LibreDWGc::R_1_3;
*R_1_4 = *LibreDWGc::R_1_4;
*R_2_0b = *LibreDWGc::R_2_0b;
*R_2_0 = *LibreDWGc::R_2_0;
*R_2_10 = *LibreDWGc::R_2_10;
*R_2_21 = *LibreDWGc::R_2_21;
*R_2_22 = *LibreDWGc::R_2_22;
*R_2_4 = *LibreDWGc::R_2_4;
*R_2_5 = *LibreDWGc::R_2_5;
*R_2_6 = *LibreDWGc::R_2_6;
*R_9 = *LibreDWGc::R_9;
*R_9c1 = *LibreDWGc::R_9c1;
*R_10 = *LibreDWGc::R_10;
*R_11b1 = *LibreDWGc::R_11b1;
*R_11b2 = *LibreDWGc::R_11b2;
*R_11 = *LibreDWGc::R_11;
*R_12 = *LibreDWGc::R_12;
*R_13b1 = *LibreDWGc::R_13b1;
*R_13b2 = *LibreDWGc::R_13b2;
*R_13 = *LibreDWGc::R_13;
*R_13c3 = *LibreDWGc::R_13c3;
*R_14 = *LibreDWGc::R_14;
*R_2000b = *LibreDWGc::R_2000b;
*R_2000 = *LibreDWGc::R_2000;
*R_2000i = *LibreDWGc::R_2000i;
*R_2002 = *LibreDWGc::R_2002;
*R_2004a = *LibreDWGc::R_2004a;
*R_2004b = *LibreDWGc::R_2004b;
*R_2004c = *LibreDWGc::R_2004c;
*R_2004 = *LibreDWGc::R_2004;
*R_2007a = *LibreDWGc::R_2007a;
*R_2007b = *LibreDWGc::R_2007b;
*R_2007 = *LibreDWGc::R_2007;
*R_2010b = *LibreDWGc::R_2010b;
*R_2010 = *LibreDWGc::R_2010;
*R_2013b = *LibreDWGc::R_2013b;
*R_2013 = *LibreDWGc::R_2013;
*R_2018b = *LibreDWGc::R_2018b;
*R_2018 = *LibreDWGc::R_2018;
*R_2022b = *LibreDWGc::R_2022b;
*R_AFTER = *LibreDWGc::R_AFTER;
*DWG_CLASS_STABLE = *LibreDWGc::DWG_CLASS_STABLE;
*DWG_CLASS_UNSTABLE = *LibreDWGc::DWG_CLASS_UNSTABLE;
*DWG_CLASS_DEBUGGING = *LibreDWGc::DWG_CLASS_DEBUGGING;
*DWG_CLASS_UNHANDLED = *LibreDWGc::DWG_CLASS_UNHANDLED;
*DWG_ENTITY_SECTION = *LibreDWGc::DWG_ENTITY_SECTION;
*DWG_BLOCKS_SECTION = *LibreDWGc::DWG_BLOCKS_SECTION;
*DWG_EXTRA_SECTION = *LibreDWGc::DWG_EXTRA_SECTION;
*DWG_SUPERTYPE_ENTITY = *LibreDWGc::DWG_SUPERTYPE_ENTITY;
*DWG_SUPERTYPE_OBJECT = *LibreDWGc::DWG_SUPERTYPE_OBJECT;
*DWG_TYPE_UNUSED = *LibreDWGc::DWG_TYPE_UNUSED;
*DWG_TYPE_TEXT = *LibreDWGc::DWG_TYPE_TEXT;
*DWG_TYPE_ATTRIB = *LibreDWGc::DWG_TYPE_ATTRIB;
*DWG_TYPE_ATTDEF = *LibreDWGc::DWG_TYPE_ATTDEF;
*DWG_TYPE_BLOCK = *LibreDWGc::DWG_TYPE_BLOCK;
*DWG_TYPE_ENDBLK = *LibreDWGc::DWG_TYPE_ENDBLK;
*DWG_TYPE_SEQEND = *LibreDWGc::DWG_TYPE_SEQEND;
*DWG_TYPE_INSERT = *LibreDWGc::DWG_TYPE_INSERT;
*DWG_TYPE_MINSERT = *LibreDWGc::DWG_TYPE_MINSERT;
*DWG_TYPE_VERTEX_2D = *LibreDWGc::DWG_TYPE_VERTEX_2D;
*DWG_TYPE_VERTEX_3D = *LibreDWGc::DWG_TYPE_VERTEX_3D;
*DWG_TYPE_VERTEX_MESH = *LibreDWGc::DWG_TYPE_VERTEX_MESH;
*DWG_TYPE_VERTEX_PFACE = *LibreDWGc::DWG_TYPE_VERTEX_PFACE;
*DWG_TYPE_VERTEX_PFACE_FACE = *LibreDWGc::DWG_TYPE_VERTEX_PFACE_FACE;
*DWG_TYPE_POLYLINE_2D = *LibreDWGc::DWG_TYPE_POLYLINE_2D;
*DWG_TYPE_POLYLINE_3D = *LibreDWGc::DWG_TYPE_POLYLINE_3D;
*DWG_TYPE_ARC = *LibreDWGc::DWG_TYPE_ARC;
*DWG_TYPE_CIRCLE = *LibreDWGc::DWG_TYPE_CIRCLE;
*DWG_TYPE_LINE = *LibreDWGc::DWG_TYPE_LINE;
*DWG_TYPE_DIMENSION_ORDINATE = *LibreDWGc::DWG_TYPE_DIMENSION_ORDINATE;
*DWG_TYPE_DIMENSION_LINEAR = *LibreDWGc::DWG_TYPE_DIMENSION_LINEAR;
*DWG_TYPE_DIMENSION_ALIGNED = *LibreDWGc::DWG_TYPE_DIMENSION_ALIGNED;
*DWG_TYPE_DIMENSION_ANG3PT = *LibreDWGc::DWG_TYPE_DIMENSION_ANG3PT;
*DWG_TYPE_DIMENSION_ANG2LN = *LibreDWGc::DWG_TYPE_DIMENSION_ANG2LN;
*DWG_TYPE_DIMENSION_RADIUS = *LibreDWGc::DWG_TYPE_DIMENSION_RADIUS;
*DWG_TYPE_DIMENSION_DIAMETER = *LibreDWGc::DWG_TYPE_DIMENSION_DIAMETER;
*DWG_TYPE_POINT = *LibreDWGc::DWG_TYPE_POINT;
*DWG_TYPE__3DFACE = *LibreDWGc::DWG_TYPE__3DFACE;
*DWG_TYPE_POLYLINE_PFACE = *LibreDWGc::DWG_TYPE_POLYLINE_PFACE;
*DWG_TYPE_POLYLINE_MESH = *LibreDWGc::DWG_TYPE_POLYLINE_MESH;
*DWG_TYPE_SOLID = *LibreDWGc::DWG_TYPE_SOLID;
*DWG_TYPE_TRACE = *LibreDWGc::DWG_TYPE_TRACE;
*DWG_TYPE_SHAPE = *LibreDWGc::DWG_TYPE_SHAPE;
*DWG_TYPE_VIEWPORT = *LibreDWGc::DWG_TYPE_VIEWPORT;
*DWG_TYPE_ELLIPSE = *LibreDWGc::DWG_TYPE_ELLIPSE;
*DWG_TYPE_SPLINE = *LibreDWGc::DWG_TYPE_SPLINE;
*DWG_TYPE_REGION = *LibreDWGc::DWG_TYPE_REGION;
*DWG_TYPE__3DSOLID = *LibreDWGc::DWG_TYPE__3DSOLID;
*DWG_TYPE_BODY = *LibreDWGc::DWG_TYPE_BODY;
*DWG_TYPE_RAY = *LibreDWGc::DWG_TYPE_RAY;
*DWG_TYPE_XLINE = *LibreDWGc::DWG_TYPE_XLINE;
*DWG_TYPE_DICTIONARY = *LibreDWGc::DWG_TYPE_DICTIONARY;
*DWG_TYPE_OLEFRAME = *LibreDWGc::DWG_TYPE_OLEFRAME;
*DWG_TYPE_MTEXT = *LibreDWGc::DWG_TYPE_MTEXT;
*DWG_TYPE_LEADER = *LibreDWGc::DWG_TYPE_LEADER;
*DWG_TYPE_TOLERANCE = *LibreDWGc::DWG_TYPE_TOLERANCE;
*DWG_TYPE_MLINE = *LibreDWGc::DWG_TYPE_MLINE;
*DWG_TYPE_BLOCK_CONTROL = *LibreDWGc::DWG_TYPE_BLOCK_CONTROL;
*DWG_TYPE_BLOCK_HEADER = *LibreDWGc::DWG_TYPE_BLOCK_HEADER;
*DWG_TYPE_LAYER_CONTROL = *LibreDWGc::DWG_TYPE_LAYER_CONTROL;
*DWG_TYPE_LAYER = *LibreDWGc::DWG_TYPE_LAYER;
*DWG_TYPE_STYLE_CONTROL = *LibreDWGc::DWG_TYPE_STYLE_CONTROL;
*DWG_TYPE_STYLE = *LibreDWGc::DWG_TYPE_STYLE;
*DWG_TYPE_LTYPE_CONTROL = *LibreDWGc::DWG_TYPE_LTYPE_CONTROL;
*DWG_TYPE_LTYPE = *LibreDWGc::DWG_TYPE_LTYPE;
*DWG_TYPE_VIEW_CONTROL = *LibreDWGc::DWG_TYPE_VIEW_CONTROL;
*DWG_TYPE_VIEW = *LibreDWGc::DWG_TYPE_VIEW;
*DWG_TYPE_UCS_CONTROL = *LibreDWGc::DWG_TYPE_UCS_CONTROL;
*DWG_TYPE_UCS = *LibreDWGc::DWG_TYPE_UCS;
*DWG_TYPE_VPORT_CONTROL = *LibreDWGc::DWG_TYPE_VPORT_CONTROL;
*DWG_TYPE_VPORT = *LibreDWGc::DWG_TYPE_VPORT;
*DWG_TYPE_APPID_CONTROL = *LibreDWGc::DWG_TYPE_APPID_CONTROL;
*DWG_TYPE_APPID = *LibreDWGc::DWG_TYPE_APPID;
*DWG_TYPE_DIMSTYLE_CONTROL = *LibreDWGc::DWG_TYPE_DIMSTYLE_CONTROL;
*DWG_TYPE_DIMSTYLE = *LibreDWGc::DWG_TYPE_DIMSTYLE;
*DWG_TYPE_VX_CONTROL = *LibreDWGc::DWG_TYPE_VX_CONTROL;
*DWG_TYPE_VX_TABLE_RECORD = *LibreDWGc::DWG_TYPE_VX_TABLE_RECORD;
*DWG_TYPE_GROUP = *LibreDWGc::DWG_TYPE_GROUP;
*DWG_TYPE_MLINESTYLE = *LibreDWGc::DWG_TYPE_MLINESTYLE;
*DWG_TYPE_OLE2FRAME = *LibreDWGc::DWG_TYPE_OLE2FRAME;
*DWG_TYPE_DUMMY = *LibreDWGc::DWG_TYPE_DUMMY;
*DWG_TYPE_LONG_TRANSACTION = *LibreDWGc::DWG_TYPE_LONG_TRANSACTION;
*DWG_TYPE_LWPOLYLINE = *LibreDWGc::DWG_TYPE_LWPOLYLINE;
*DWG_TYPE_HATCH = *LibreDWGc::DWG_TYPE_HATCH;
*DWG_TYPE_XRECORD = *LibreDWGc::DWG_TYPE_XRECORD;
*DWG_TYPE_PLACEHOLDER = *LibreDWGc::DWG_TYPE_PLACEHOLDER;
*DWG_TYPE_VBA_PROJECT = *LibreDWGc::DWG_TYPE_VBA_PROJECT;
*DWG_TYPE_LAYOUT = *LibreDWGc::DWG_TYPE_LAYOUT;
*DWG_TYPE_PROXY_ENTITY = *LibreDWGc::DWG_TYPE_PROXY_ENTITY;
*DWG_TYPE_PROXY_OBJECT = *LibreDWGc::DWG_TYPE_PROXY_OBJECT;
*DWG_TYPE_ACDSRECORD = *LibreDWGc::DWG_TYPE_ACDSRECORD;
*DWG_TYPE_ACDSSCHEMA = *LibreDWGc::DWG_TYPE_ACDSSCHEMA;
*DWG_TYPE_ACMECOMMANDHISTORY = *LibreDWGc::DWG_TYPE_ACMECOMMANDHISTORY;
*DWG_TYPE_ACMESCOPE = *LibreDWGc::DWG_TYPE_ACMESCOPE;
*DWG_TYPE_ACMESTATEMGR = *LibreDWGc::DWG_TYPE_ACMESTATEMGR;
*DWG_TYPE_ACSH_BOOLEAN_CLASS = *LibreDWGc::DWG_TYPE_ACSH_BOOLEAN_CLASS;
*DWG_TYPE_ACSH_BOX_CLASS = *LibreDWGc::DWG_TYPE_ACSH_BOX_CLASS;
*DWG_TYPE_ACSH_BREP_CLASS = *LibreDWGc::DWG_TYPE_ACSH_BREP_CLASS;
*DWG_TYPE_ACSH_CHAMFER_CLASS = *LibreDWGc::DWG_TYPE_ACSH_CHAMFER_CLASS;
*DWG_TYPE_ACSH_CONE_CLASS = *LibreDWGc::DWG_TYPE_ACSH_CONE_CLASS;
*DWG_TYPE_ACSH_CYLINDER_CLASS = *LibreDWGc::DWG_TYPE_ACSH_CYLINDER_CLASS;
*DWG_TYPE_ACSH_EXTRUSION_CLASS = *LibreDWGc::DWG_TYPE_ACSH_EXTRUSION_CLASS;
*DWG_TYPE_ACSH_FILLET_CLASS = *LibreDWGc::DWG_TYPE_ACSH_FILLET_CLASS;
*DWG_TYPE_ACSH_HISTORY_CLASS = *LibreDWGc::DWG_TYPE_ACSH_HISTORY_CLASS;
*DWG_TYPE_ACSH_LOFT_CLASS = *LibreDWGc::DWG_TYPE_ACSH_LOFT_CLASS;
*DWG_TYPE_ACSH_PYRAMID_CLASS = *LibreDWGc::DWG_TYPE_ACSH_PYRAMID_CLASS;
*DWG_TYPE_ACSH_REVOLVE_CLASS = *LibreDWGc::DWG_TYPE_ACSH_REVOLVE_CLASS;
*DWG_TYPE_ACSH_SPHERE_CLASS = *LibreDWGc::DWG_TYPE_ACSH_SPHERE_CLASS;
*DWG_TYPE_ACSH_SWEEP_CLASS = *LibreDWGc::DWG_TYPE_ACSH_SWEEP_CLASS;
*DWG_TYPE_ACSH_TORUS_CLASS = *LibreDWGc::DWG_TYPE_ACSH_TORUS_CLASS;
*DWG_TYPE_ACSH_WEDGE_CLASS = *LibreDWGc::DWG_TYPE_ACSH_WEDGE_CLASS;
*DWG_TYPE_ALDIMOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_ALDIMOBJECTCONTEXTDATA;
*DWG_TYPE_ALIGNMENTPARAMETERENTITY = *LibreDWGc::DWG_TYPE_ALIGNMENTPARAMETERENTITY;
*DWG_TYPE_ANGDIMOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_ANGDIMOBJECTCONTEXTDATA;
*DWG_TYPE_ANNOTSCALEOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_ANNOTSCALEOBJECTCONTEXTDATA;
*DWG_TYPE_ARCALIGNEDTEXT = *LibreDWGc::DWG_TYPE_ARCALIGNEDTEXT;
*DWG_TYPE_ARC_DIMENSION = *LibreDWGc::DWG_TYPE_ARC_DIMENSION;
*DWG_TYPE_ASSOC2DCONSTRAINTGROUP = *LibreDWGc::DWG_TYPE_ASSOC2DCONSTRAINTGROUP;
*DWG_TYPE_ASSOC3POINTANGULARDIMACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOC3POINTANGULARDIMACTIONBODY;
*DWG_TYPE_ASSOCACTION = *LibreDWGc::DWG_TYPE_ASSOCACTION;
*DWG_TYPE_ASSOCACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCACTIONPARAM;
*DWG_TYPE_ASSOCALIGNEDDIMACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCALIGNEDDIMACTIONBODY;
*DWG_TYPE_ASSOCARRAYACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCARRAYACTIONBODY;
*DWG_TYPE_ASSOCARRAYMODIFYACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCARRAYMODIFYACTIONBODY;
*DWG_TYPE_ASSOCARRAYMODIFYPARAMETERS = *LibreDWGc::DWG_TYPE_ASSOCARRAYMODIFYPARAMETERS;
*DWG_TYPE_ASSOCARRAYPATHPARAMETERS = *LibreDWGc::DWG_TYPE_ASSOCARRAYPATHPARAMETERS;
*DWG_TYPE_ASSOCARRAYPOLARPARAMETERS = *LibreDWGc::DWG_TYPE_ASSOCARRAYPOLARPARAMETERS;
*DWG_TYPE_ASSOCARRAYRECTANGULARPARAMETERS = *LibreDWGc::DWG_TYPE_ASSOCARRAYRECTANGULARPARAMETERS;
*DWG_TYPE_ASSOCASMBODYACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCASMBODYACTIONPARAM;
*DWG_TYPE_ASSOCBLENDSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCBLENDSURFACEACTIONBODY;
*DWG_TYPE_ASSOCCOMPOUNDACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCCOMPOUNDACTIONPARAM;
*DWG_TYPE_ASSOCDEPENDENCY = *LibreDWGc::DWG_TYPE_ASSOCDEPENDENCY;
*DWG_TYPE_ASSOCDIMDEPENDENCYBODY = *LibreDWGc::DWG_TYPE_ASSOCDIMDEPENDENCYBODY;
*DWG_TYPE_ASSOCEDGEACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCEDGEACTIONPARAM;
*DWG_TYPE_ASSOCEDGECHAMFERACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCEDGECHAMFERACTIONBODY;
*DWG_TYPE_ASSOCEDGEFILLETACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCEDGEFILLETACTIONBODY;
*DWG_TYPE_ASSOCEXTENDSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCEXTENDSURFACEACTIONBODY;
*DWG_TYPE_ASSOCEXTRUDEDSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCEXTRUDEDSURFACEACTIONBODY;
*DWG_TYPE_ASSOCFACEACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCFACEACTIONPARAM;
*DWG_TYPE_ASSOCFILLETSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCFILLETSURFACEACTIONBODY;
*DWG_TYPE_ASSOCGEOMDEPENDENCY = *LibreDWGc::DWG_TYPE_ASSOCGEOMDEPENDENCY;
*DWG_TYPE_ASSOCLOFTEDSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCLOFTEDSURFACEACTIONBODY;
*DWG_TYPE_ASSOCMLEADERACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCMLEADERACTIONBODY;
*DWG_TYPE_ASSOCNETWORK = *LibreDWGc::DWG_TYPE_ASSOCNETWORK;
*DWG_TYPE_ASSOCNETWORKSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCNETWORKSURFACEACTIONBODY;
*DWG_TYPE_ASSOCOBJECTACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCOBJECTACTIONPARAM;
*DWG_TYPE_ASSOCOFFSETSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCOFFSETSURFACEACTIONBODY;
*DWG_TYPE_ASSOCORDINATEDIMACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCORDINATEDIMACTIONBODY;
*DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCOSNAPPOINTREFACTIONPARAM;
*DWG_TYPE_ASSOCPATCHSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCPATCHSURFACEACTIONBODY;
*DWG_TYPE_ASSOCPATHACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCPATHACTIONPARAM;
*DWG_TYPE_ASSOCPERSSUBENTMANAGER = *LibreDWGc::DWG_TYPE_ASSOCPERSSUBENTMANAGER;
*DWG_TYPE_ASSOCPLANESURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCPLANESURFACEACTIONBODY;
*DWG_TYPE_ASSOCPOINTREFACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCPOINTREFACTIONPARAM;
*DWG_TYPE_ASSOCRESTOREENTITYSTATEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCRESTOREENTITYSTATEACTIONBODY;
*DWG_TYPE_ASSOCREVOLVEDSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCREVOLVEDSURFACEACTIONBODY;
*DWG_TYPE_ASSOCROTATEDDIMACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCROTATEDDIMACTIONBODY;
*DWG_TYPE_ASSOCSWEPTSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCSWEPTSURFACEACTIONBODY;
*DWG_TYPE_ASSOCTRIMSURFACEACTIONBODY = *LibreDWGc::DWG_TYPE_ASSOCTRIMSURFACEACTIONBODY;
*DWG_TYPE_ASSOCVALUEDEPENDENCY = *LibreDWGc::DWG_TYPE_ASSOCVALUEDEPENDENCY;
*DWG_TYPE_ASSOCVARIABLE = *LibreDWGc::DWG_TYPE_ASSOCVARIABLE;
*DWG_TYPE_ASSOCVERTEXACTIONPARAM = *LibreDWGc::DWG_TYPE_ASSOCVERTEXACTIONPARAM;
*DWG_TYPE_BASEPOINTPARAMETERENTITY = *LibreDWGc::DWG_TYPE_BASEPOINTPARAMETERENTITY;
*DWG_TYPE_BLKREFOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_BLKREFOBJECTCONTEXTDATA;
*DWG_TYPE_BLOCKALIGNEDCONSTRAINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKALIGNEDCONSTRAINTPARAMETER;
*DWG_TYPE_BLOCKALIGNMENTGRIP = *LibreDWGc::DWG_TYPE_BLOCKALIGNMENTGRIP;
*DWG_TYPE_BLOCKALIGNMENTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKALIGNMENTPARAMETER;
*DWG_TYPE_BLOCKANGULARCONSTRAINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKANGULARCONSTRAINTPARAMETER;
*DWG_TYPE_BLOCKARRAYACTION = *LibreDWGc::DWG_TYPE_BLOCKARRAYACTION;
*DWG_TYPE_BLOCKBASEPOINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKBASEPOINTPARAMETER;
*DWG_TYPE_BLOCKDIAMETRICCONSTRAINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKDIAMETRICCONSTRAINTPARAMETER;
*DWG_TYPE_BLOCKFLIPACTION = *LibreDWGc::DWG_TYPE_BLOCKFLIPACTION;
*DWG_TYPE_BLOCKFLIPGRIP = *LibreDWGc::DWG_TYPE_BLOCKFLIPGRIP;
*DWG_TYPE_BLOCKFLIPPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKFLIPPARAMETER;
*DWG_TYPE_BLOCKGRIPLOCATIONCOMPONENT = *LibreDWGc::DWG_TYPE_BLOCKGRIPLOCATIONCOMPONENT;
*DWG_TYPE_BLOCKHORIZONTALCONSTRAINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKHORIZONTALCONSTRAINTPARAMETER;
*DWG_TYPE_BLOCKLINEARCONSTRAINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKLINEARCONSTRAINTPARAMETER;
*DWG_TYPE_BLOCKLINEARGRIP = *LibreDWGc::DWG_TYPE_BLOCKLINEARGRIP;
*DWG_TYPE_BLOCKLINEARPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKLINEARPARAMETER;
*DWG_TYPE_BLOCKLOOKUPACTION = *LibreDWGc::DWG_TYPE_BLOCKLOOKUPACTION;
*DWG_TYPE_BLOCKLOOKUPGRIP = *LibreDWGc::DWG_TYPE_BLOCKLOOKUPGRIP;
*DWG_TYPE_BLOCKLOOKUPPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKLOOKUPPARAMETER;
*DWG_TYPE_BLOCKMOVEACTION = *LibreDWGc::DWG_TYPE_BLOCKMOVEACTION;
*DWG_TYPE_BLOCKPARAMDEPENDENCYBODY = *LibreDWGc::DWG_TYPE_BLOCKPARAMDEPENDENCYBODY;
*DWG_TYPE_BLOCKPOINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKPOINTPARAMETER;
*DWG_TYPE_BLOCKPOLARGRIP = *LibreDWGc::DWG_TYPE_BLOCKPOLARGRIP;
*DWG_TYPE_BLOCKPOLARPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKPOLARPARAMETER;
*DWG_TYPE_BLOCKPOLARSTRETCHACTION = *LibreDWGc::DWG_TYPE_BLOCKPOLARSTRETCHACTION;
*DWG_TYPE_BLOCKPROPERTIESTABLE = *LibreDWGc::DWG_TYPE_BLOCKPROPERTIESTABLE;
*DWG_TYPE_BLOCKPROPERTIESTABLEGRIP = *LibreDWGc::DWG_TYPE_BLOCKPROPERTIESTABLEGRIP;
*DWG_TYPE_BLOCKRADIALCONSTRAINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKRADIALCONSTRAINTPARAMETER;
*DWG_TYPE_BLOCKREPRESENTATION = *LibreDWGc::DWG_TYPE_BLOCKREPRESENTATION;
*DWG_TYPE_BLOCKROTATEACTION = *LibreDWGc::DWG_TYPE_BLOCKROTATEACTION;
*DWG_TYPE_BLOCKROTATIONGRIP = *LibreDWGc::DWG_TYPE_BLOCKROTATIONGRIP;
*DWG_TYPE_BLOCKROTATIONPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKROTATIONPARAMETER;
*DWG_TYPE_BLOCKSCALEACTION = *LibreDWGc::DWG_TYPE_BLOCKSCALEACTION;
*DWG_TYPE_BLOCKSTRETCHACTION = *LibreDWGc::DWG_TYPE_BLOCKSTRETCHACTION;
*DWG_TYPE_BLOCKUSERPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKUSERPARAMETER;
*DWG_TYPE_BLOCKVERTICALCONSTRAINTPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKVERTICALCONSTRAINTPARAMETER;
*DWG_TYPE_BLOCKVISIBILITYGRIP = *LibreDWGc::DWG_TYPE_BLOCKVISIBILITYGRIP;
*DWG_TYPE_BLOCKVISIBILITYPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKVISIBILITYPARAMETER;
*DWG_TYPE_BLOCKXYGRIP = *LibreDWGc::DWG_TYPE_BLOCKXYGRIP;
*DWG_TYPE_BLOCKXYPARAMETER = *LibreDWGc::DWG_TYPE_BLOCKXYPARAMETER;
*DWG_TYPE_CAMERA = *LibreDWGc::DWG_TYPE_CAMERA;
*DWG_TYPE_CELLSTYLEMAP = *LibreDWGc::DWG_TYPE_CELLSTYLEMAP;
*DWG_TYPE_CONTEXTDATAMANAGER = *LibreDWGc::DWG_TYPE_CONTEXTDATAMANAGER;
*DWG_TYPE_CSACDOCUMENTOPTIONS = *LibreDWGc::DWG_TYPE_CSACDOCUMENTOPTIONS;
*DWG_TYPE_CURVEPATH = *LibreDWGc::DWG_TYPE_CURVEPATH;
*DWG_TYPE_DATALINK = *LibreDWGc::DWG_TYPE_DATALINK;
*DWG_TYPE_DATATABLE = *LibreDWGc::DWG_TYPE_DATATABLE;
*DWG_TYPE_DBCOLOR = *LibreDWGc::DWG_TYPE_DBCOLOR;
*DWG_TYPE_DETAILVIEWSTYLE = *LibreDWGc::DWG_TYPE_DETAILVIEWSTYLE;
*DWG_TYPE_DGNDEFINITION = *LibreDWGc::DWG_TYPE_DGNDEFINITION;
*DWG_TYPE_DGNUNDERLAY = *LibreDWGc::DWG_TYPE_DGNUNDERLAY;
*DWG_TYPE_DICTIONARYVAR = *LibreDWGc::DWG_TYPE_DICTIONARYVAR;
*DWG_TYPE_DICTIONARYWDFLT = *LibreDWGc::DWG_TYPE_DICTIONARYWDFLT;
*DWG_TYPE_DIMASSOC = *LibreDWGc::DWG_TYPE_DIMASSOC;
*DWG_TYPE_DMDIMOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_DMDIMOBJECTCONTEXTDATA;
*DWG_TYPE_DWFDEFINITION = *LibreDWGc::DWG_TYPE_DWFDEFINITION;
*DWG_TYPE_DWFUNDERLAY = *LibreDWGc::DWG_TYPE_DWFUNDERLAY;
*DWG_TYPE_DYNAMICBLOCKPROXYNODE = *LibreDWGc::DWG_TYPE_DYNAMICBLOCKPROXYNODE;
*DWG_TYPE_DYNAMICBLOCKPURGEPREVENTER = *LibreDWGc::DWG_TYPE_DYNAMICBLOCKPURGEPREVENTER;
*DWG_TYPE_EVALUATION_GRAPH = *LibreDWGc::DWG_TYPE_EVALUATION_GRAPH;
*DWG_TYPE_EXTRUDEDSURFACE = *LibreDWGc::DWG_TYPE_EXTRUDEDSURFACE;
*DWG_TYPE_FCFOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_FCFOBJECTCONTEXTDATA;
*DWG_TYPE_FIELD = *LibreDWGc::DWG_TYPE_FIELD;
*DWG_TYPE_FIELDLIST = *LibreDWGc::DWG_TYPE_FIELDLIST;
*DWG_TYPE_FLIPPARAMETERENTITY = *LibreDWGc::DWG_TYPE_FLIPPARAMETERENTITY;
*DWG_TYPE_FLIPACTIONENTITY = *LibreDWGc::DWG_TYPE_FLIPACTIONENTITY;
*DWG_TYPE_GEODATA = *LibreDWGc::DWG_TYPE_GEODATA;
*DWG_TYPE_GEOMAPIMAGE = *LibreDWGc::DWG_TYPE_GEOMAPIMAGE;
*DWG_TYPE_GEOPOSITIONMARKER = *LibreDWGc::DWG_TYPE_GEOPOSITIONMARKER;
*DWG_TYPE_GRADIENT_BACKGROUND = *LibreDWGc::DWG_TYPE_GRADIENT_BACKGROUND;
*DWG_TYPE_GROUND_PLANE_BACKGROUND = *LibreDWGc::DWG_TYPE_GROUND_PLANE_BACKGROUND;
*DWG_TYPE_HELIX = *LibreDWGc::DWG_TYPE_HELIX;
*DWG_TYPE_IBL_BACKGROUND = *LibreDWGc::DWG_TYPE_IBL_BACKGROUND;
*DWG_TYPE_IDBUFFER = *LibreDWGc::DWG_TYPE_IDBUFFER;
*DWG_TYPE_IMAGE = *LibreDWGc::DWG_TYPE_IMAGE;
*DWG_TYPE_IMAGEDEF = *LibreDWGc::DWG_TYPE_IMAGEDEF;
*DWG_TYPE_IMAGEDEF_REACTOR = *LibreDWGc::DWG_TYPE_IMAGEDEF_REACTOR;
*DWG_TYPE_IMAGE_BACKGROUND = *LibreDWGc::DWG_TYPE_IMAGE_BACKGROUND;
*DWG_TYPE_INDEX = *LibreDWGc::DWG_TYPE_INDEX;
*DWG_TYPE_LARGE_RADIAL_DIMENSION = *LibreDWGc::DWG_TYPE_LARGE_RADIAL_DIMENSION;
*DWG_TYPE_LAYERFILTER = *LibreDWGc::DWG_TYPE_LAYERFILTER;
*DWG_TYPE_LAYER_INDEX = *LibreDWGc::DWG_TYPE_LAYER_INDEX;
*DWG_TYPE_LAYOUTPRINTCONFIG = *LibreDWGc::DWG_TYPE_LAYOUTPRINTCONFIG;
*DWG_TYPE_LEADEROBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_LEADEROBJECTCONTEXTDATA;
*DWG_TYPE_LIGHT = *LibreDWGc::DWG_TYPE_LIGHT;
*DWG_TYPE_LIGHTLIST = *LibreDWGc::DWG_TYPE_LIGHTLIST;
*DWG_TYPE_LINEARPARAMETERENTITY = *LibreDWGc::DWG_TYPE_LINEARPARAMETERENTITY;
*DWG_TYPE_LOFTEDSURFACE = *LibreDWGc::DWG_TYPE_LOFTEDSURFACE;
*DWG_TYPE_MATERIAL = *LibreDWGc::DWG_TYPE_MATERIAL;
*DWG_TYPE_MENTALRAYRENDERSETTINGS = *LibreDWGc::DWG_TYPE_MENTALRAYRENDERSETTINGS;
*DWG_TYPE_MESH = *LibreDWGc::DWG_TYPE_MESH;
*DWG_TYPE_MLEADEROBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_MLEADEROBJECTCONTEXTDATA;
*DWG_TYPE_MLEADERSTYLE = *LibreDWGc::DWG_TYPE_MLEADERSTYLE;
*DWG_TYPE_MOVEACTIONENTITY = *LibreDWGc::DWG_TYPE_MOVEACTIONENTITY;
*DWG_TYPE_MOTIONPATH = *LibreDWGc::DWG_TYPE_MOTIONPATH;
*DWG_TYPE_MPOLYGON = *LibreDWGc::DWG_TYPE_MPOLYGON;
*DWG_TYPE_MTEXTATTRIBUTEOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_MTEXTATTRIBUTEOBJECTCONTEXTDATA;
*DWG_TYPE_MTEXTOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_MTEXTOBJECTCONTEXTDATA;
*DWG_TYPE_MULTILEADER = *LibreDWGc::DWG_TYPE_MULTILEADER;
*DWG_TYPE_NAVISWORKSMODEL = *LibreDWGc::DWG_TYPE_NAVISWORKSMODEL;
*DWG_TYPE_NAVISWORKSMODELDEF = *LibreDWGc::DWG_TYPE_NAVISWORKSMODELDEF;
*DWG_TYPE_NPOCOLLECTION = *LibreDWGc::DWG_TYPE_NPOCOLLECTION;
*DWG_TYPE_NURBSURFACE = *LibreDWGc::DWG_TYPE_NURBSURFACE;
*DWG_TYPE_OBJECT_PTR = *LibreDWGc::DWG_TYPE_OBJECT_PTR;
*DWG_TYPE_ORDDIMOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_ORDDIMOBJECTCONTEXTDATA;
*DWG_TYPE_PARTIAL_VIEWING_INDEX = *LibreDWGc::DWG_TYPE_PARTIAL_VIEWING_INDEX;
*DWG_TYPE_PDFDEFINITION = *LibreDWGc::DWG_TYPE_PDFDEFINITION;
*DWG_TYPE_PDFUNDERLAY = *LibreDWGc::DWG_TYPE_PDFUNDERLAY;
*DWG_TYPE_PERSUBENTMGR = *LibreDWGc::DWG_TYPE_PERSUBENTMGR;
*DWG_TYPE_PLANESURFACE = *LibreDWGc::DWG_TYPE_PLANESURFACE;
*DWG_TYPE_PLOTSETTINGS = *LibreDWGc::DWG_TYPE_PLOTSETTINGS;
*DWG_TYPE_POINTCLOUD = *LibreDWGc::DWG_TYPE_POINTCLOUD;
*DWG_TYPE_POINTCLOUDCOLORMAP = *LibreDWGc::DWG_TYPE_POINTCLOUDCOLORMAP;
*DWG_TYPE_POINTCLOUDDEF = *LibreDWGc::DWG_TYPE_POINTCLOUDDEF;
*DWG_TYPE_POINTCLOUDDEFEX = *LibreDWGc::DWG_TYPE_POINTCLOUDDEFEX;
*DWG_TYPE_POINTCLOUDDEF_REACTOR = *LibreDWGc::DWG_TYPE_POINTCLOUDDEF_REACTOR;
*DWG_TYPE_POINTCLOUDDEF_REACTOR_EX = *LibreDWGc::DWG_TYPE_POINTCLOUDDEF_REACTOR_EX;
*DWG_TYPE_POINTCLOUDEX = *LibreDWGc::DWG_TYPE_POINTCLOUDEX;
*DWG_TYPE_POINTPARAMETERENTITY = *LibreDWGc::DWG_TYPE_POINTPARAMETERENTITY;
*DWG_TYPE_POINTPATH = *LibreDWGc::DWG_TYPE_POINTPATH;
*DWG_TYPE_POLARGRIPENTITY = *LibreDWGc::DWG_TYPE_POLARGRIPENTITY;
*DWG_TYPE_RADIMLGOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_RADIMLGOBJECTCONTEXTDATA;
*DWG_TYPE_RADIMOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_RADIMOBJECTCONTEXTDATA;
*DWG_TYPE_RAPIDRTRENDERSETTINGS = *LibreDWGc::DWG_TYPE_RAPIDRTRENDERSETTINGS;
*DWG_TYPE_RASTERVARIABLES = *LibreDWGc::DWG_TYPE_RASTERVARIABLES;
*DWG_TYPE_RENDERENTRY = *LibreDWGc::DWG_TYPE_RENDERENTRY;
*DWG_TYPE_RENDERENVIRONMENT = *LibreDWGc::DWG_TYPE_RENDERENVIRONMENT;
*DWG_TYPE_RENDERGLOBAL = *LibreDWGc::DWG_TYPE_RENDERGLOBAL;
*DWG_TYPE_RENDERSETTINGS = *LibreDWGc::DWG_TYPE_RENDERSETTINGS;
*DWG_TYPE_REVOLVEDSURFACE = *LibreDWGc::DWG_TYPE_REVOLVEDSURFACE;
*DWG_TYPE_ROTATIONPARAMETERENTITY = *LibreDWGc::DWG_TYPE_ROTATIONPARAMETERENTITY;
*DWG_TYPE_ROTATEACTIONENTITY = *LibreDWGc::DWG_TYPE_ROTATEACTIONENTITY;
*DWG_TYPE_RTEXT = *LibreDWGc::DWG_TYPE_RTEXT;
*DWG_TYPE_SCALE = *LibreDWGc::DWG_TYPE_SCALE;
*DWG_TYPE_SCALEACTIONENTITY = *LibreDWGc::DWG_TYPE_SCALEACTIONENTITY;
*DWG_TYPE_SECTIONOBJECT = *LibreDWGc::DWG_TYPE_SECTIONOBJECT;
*DWG_TYPE_SECTIONVIEWSTYLE = *LibreDWGc::DWG_TYPE_SECTIONVIEWSTYLE;
*DWG_TYPE_SECTION_MANAGER = *LibreDWGc::DWG_TYPE_SECTION_MANAGER;
*DWG_TYPE_SECTION_SETTINGS = *LibreDWGc::DWG_TYPE_SECTION_SETTINGS;
*DWG_TYPE_SKYLIGHT_BACKGROUND = *LibreDWGc::DWG_TYPE_SKYLIGHT_BACKGROUND;
*DWG_TYPE_SOLID_BACKGROUND = *LibreDWGc::DWG_TYPE_SOLID_BACKGROUND;
*DWG_TYPE_SORTENTSTABLE = *LibreDWGc::DWG_TYPE_SORTENTSTABLE;
*DWG_TYPE_SPATIAL_FILTER = *LibreDWGc::DWG_TYPE_SPATIAL_FILTER;
*DWG_TYPE_SPATIAL_INDEX = *LibreDWGc::DWG_TYPE_SPATIAL_INDEX;
*DWG_TYPE_STRETCHACTIONENTITY = *LibreDWGc::DWG_TYPE_STRETCHACTIONENTITY;
*DWG_TYPE_SUN = *LibreDWGc::DWG_TYPE_SUN;
*DWG_TYPE_SUNSTUDY = *LibreDWGc::DWG_TYPE_SUNSTUDY;
*DWG_TYPE_SWEPTSURFACE = *LibreDWGc::DWG_TYPE_SWEPTSURFACE;
*DWG_TYPE_TABLE = *LibreDWGc::DWG_TYPE_TABLE;
*DWG_TYPE_TABLECONTENT = *LibreDWGc::DWG_TYPE_TABLECONTENT;
*DWG_TYPE_TABLEGEOMETRY = *LibreDWGc::DWG_TYPE_TABLEGEOMETRY;
*DWG_TYPE_TABLESTYLE = *LibreDWGc::DWG_TYPE_TABLESTYLE;
*DWG_TYPE_TEXTOBJECTCONTEXTDATA = *LibreDWGc::DWG_TYPE_TEXTOBJECTCONTEXTDATA;
*DWG_TYPE_TVDEVICEPROPERTIES = *LibreDWGc::DWG_TYPE_TVDEVICEPROPERTIES;
*DWG_TYPE_VISIBILITYGRIPENTITY = *LibreDWGc::DWG_TYPE_VISIBILITYGRIPENTITY;
*DWG_TYPE_VISIBILITYPARAMETERENTITY = *LibreDWGc::DWG_TYPE_VISIBILITYPARAMETERENTITY;
*DWG_TYPE_VISUALSTYLE = *LibreDWGc::DWG_TYPE_VISUALSTYLE;
*DWG_TYPE_WIPEOUT = *LibreDWGc::DWG_TYPE_WIPEOUT;
*DWG_TYPE_WIPEOUTVARIABLES = *LibreDWGc::DWG_TYPE_WIPEOUTVARIABLES;
*DWG_TYPE_XREFPANELOBJECT = *LibreDWGc::DWG_TYPE_XREFPANELOBJECT;
*DWG_TYPE_XYPARAMETERENTITY = *LibreDWGc::DWG_TYPE_XYPARAMETERENTITY;
*DWG_TYPE_BREAKDATA = *LibreDWGc::DWG_TYPE_BREAKDATA;
*DWG_TYPE_BREAKPOINTREF = *LibreDWGc::DWG_TYPE_BREAKPOINTREF;
*DWG_TYPE_FLIPGRIPENTITY = *LibreDWGc::DWG_TYPE_FLIPGRIPENTITY;
*DWG_TYPE_LINEARGRIPENTITY = *LibreDWGc::DWG_TYPE_LINEARGRIPENTITY;
*DWG_TYPE_ROTATIONGRIPENTITY = *LibreDWGc::DWG_TYPE_ROTATIONGRIPENTITY;
*DWG_TYPE_XYGRIPENTITY = *LibreDWGc::DWG_TYPE_XYGRIPENTITY;
*DWG_TYPE__3DLINE = *LibreDWGc::DWG_TYPE__3DLINE;
*DWG_TYPE_REPEAT = *LibreDWGc::DWG_TYPE_REPEAT;
*DWG_TYPE_ENDREP = *LibreDWGc::DWG_TYPE_ENDREP;
*DWG_TYPE_JUMP = *LibreDWGc::DWG_TYPE_JUMP;
*DWG_TYPE_LOAD = *LibreDWGc::DWG_TYPE_LOAD;
*DWG_TYPE_ABSHDRAWINGSETTINGS = *LibreDWGc::DWG_TYPE_ABSHDRAWINGSETTINGS;
*DWG_TYPE_ACAECUSTOBJ = *LibreDWGc::DWG_TYPE_ACAECUSTOBJ;
*DWG_TYPE_ACAEEEMGROBJ = *LibreDWGc::DWG_TYPE_ACAEEEMGROBJ;
*DWG_TYPE_ACAMCOMP = *LibreDWGc::DWG_TYPE_ACAMCOMP;
*DWG_TYPE_ACAMCOMPDEF = *LibreDWGc::DWG_TYPE_ACAMCOMPDEF;
*DWG_TYPE_ACAMCOMPDEFMGR = *LibreDWGc::DWG_TYPE_ACAMCOMPDEFMGR;
*DWG_TYPE_ACAMCONTEXTMODELER = *LibreDWGc::DWG_TYPE_ACAMCONTEXTMODELER;
*DWG_TYPE_ACAMGDIMSTD = *LibreDWGc::DWG_TYPE_ACAMGDIMSTD;
*DWG_TYPE_ACAMGFILTERDAT = *LibreDWGc::DWG_TYPE_ACAMGFILTERDAT;
*DWG_TYPE_ACAMGHOLECHARTSTDCSN = *LibreDWGc::DWG_TYPE_ACAMGHOLECHARTSTDCSN;
*DWG_TYPE_ACAMGHOLECHARTSTDDIN = *LibreDWGc::DWG_TYPE_ACAMGHOLECHARTSTDDIN;
*DWG_TYPE_ACAMGHOLECHARTSTDISO = *LibreDWGc::DWG_TYPE_ACAMGHOLECHARTSTDISO;
*DWG_TYPE_ACAMGLAYSTD = *LibreDWGc::DWG_TYPE_ACAMGLAYSTD;
*DWG_TYPE_ACAMGRCOMPDEF = *LibreDWGc::DWG_TYPE_ACAMGRCOMPDEF;
*DWG_TYPE_ACAMGRCOMPDEFSET = *LibreDWGc::DWG_TYPE_ACAMGRCOMPDEFSET;
*DWG_TYPE_ACAMGTITLESTD = *LibreDWGc::DWG_TYPE_ACAMGTITLESTD;
*DWG_TYPE_ACAMMVDBACKUPOBJECT = *LibreDWGc::DWG_TYPE_ACAMMVDBACKUPOBJECT;
*DWG_TYPE_ACAMPROJECT = *LibreDWGc::DWG_TYPE_ACAMPROJECT;
*DWG_TYPE_ACAMSHAFTCOMPDEF = *LibreDWGc::DWG_TYPE_ACAMSHAFTCOMPDEF;
*DWG_TYPE_ACAMSTDPCOMPDEF = *LibreDWGc::DWG_TYPE_ACAMSTDPCOMPDEF;
*DWG_TYPE_ACAMWBLOCKTEMPENTS = *LibreDWGc::DWG_TYPE_ACAMWBLOCKTEMPENTS;
*DWG_TYPE_ACARRAYJIGENTITY = *LibreDWGc::DWG_TYPE_ACARRAYJIGENTITY;
*DWG_TYPE_ACCMCONTEXT = *LibreDWGc::DWG_TYPE_ACCMCONTEXT;
*DWG_TYPE_ACDBCIRCARCRES = *LibreDWGc::DWG_TYPE_ACDBCIRCARCRES;
*DWG_TYPE_ACDBDIMENSIONRES = *LibreDWGc::DWG_TYPE_ACDBDIMENSIONRES;
*DWG_TYPE_ACDBENTITYCACHE = *LibreDWGc::DWG_TYPE_ACDBENTITYCACHE;
*DWG_TYPE_ACDBLINERES = *LibreDWGc::DWG_TYPE_ACDBLINERES;
*DWG_TYPE_ACDBSTDPARTRES_ARC = *LibreDWGc::DWG_TYPE_ACDBSTDPARTRES_ARC;
*DWG_TYPE_ACDBSTDPARTRES_LINE = *LibreDWGc::DWG_TYPE_ACDBSTDPARTRES_LINE;
*DWG_TYPE_ACDB_HATCHSCALECONTEXTDATA_CLASS = *LibreDWGc::DWG_TYPE_ACDB_HATCHSCALECONTEXTDATA_CLASS;
*DWG_TYPE_ACDB_HATCHVIEWCONTEXTDATA_CLASS = *LibreDWGc::DWG_TYPE_ACDB_HATCHVIEWCONTEXTDATA_CLASS;
*DWG_TYPE_ACDB_PROXY_ENTITY_DATA = *LibreDWGc::DWG_TYPE_ACDB_PROXY_ENTITY_DATA;
*DWG_TYPE_ACGREFACADMASTER = *LibreDWGc::DWG_TYPE_ACGREFACADMASTER;
*DWG_TYPE_ACGREFMASTER = *LibreDWGc::DWG_TYPE_ACGREFMASTER;
*DWG_TYPE_ACIMINTSYSVAR = *LibreDWGc::DWG_TYPE_ACIMINTSYSVAR;
*DWG_TYPE_ACIMREALSYSVAR = *LibreDWGc::DWG_TYPE_ACIMREALSYSVAR;
*DWG_TYPE_ACIMSTRSYSVAR = *LibreDWGc::DWG_TYPE_ACIMSTRSYSVAR;
*DWG_TYPE_ACIMSYSVARMAN = *LibreDWGc::DWG_TYPE_ACIMSYSVARMAN;
*DWG_TYPE_ACMANOOTATIONVIEWSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMANOOTATIONVIEWSTANDARDANSI;
*DWG_TYPE_ACMANOOTATIONVIEWSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMANOOTATIONVIEWSTANDARDCSN;
*DWG_TYPE_ACMANOOTATIONVIEWSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMANOOTATIONVIEWSTANDARDDIN;
*DWG_TYPE_ACMANOOTATIONVIEWSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMANOOTATIONVIEWSTANDARDISO;
*DWG_TYPE_ACMAPLEGENDDBOBJECT = *LibreDWGc::DWG_TYPE_ACMAPLEGENDDBOBJECT;
*DWG_TYPE_ACMAPLEGENDITEMDBOBJECT = *LibreDWGc::DWG_TYPE_ACMAPLEGENDITEMDBOBJECT;
*DWG_TYPE_ACMAPMAPVIEWPORTDBOBJECT = *LibreDWGc::DWG_TYPE_ACMAPMAPVIEWPORTDBOBJECT;
*DWG_TYPE_ACMAPPRINTLAYOUTELEMENTDBOBJECTCONTAINER = *LibreDWGc::DWG_TYPE_ACMAPPRINTLAYOUTELEMENTDBOBJECTCONTAINER;
*DWG_TYPE_ACMBALLOON = *LibreDWGc::DWG_TYPE_ACMBALLOON;
*DWG_TYPE_ACMBOM = *LibreDWGc::DWG_TYPE_ACMBOM;
*DWG_TYPE_ACMBOMROW = *LibreDWGc::DWG_TYPE_ACMBOMROW;
*DWG_TYPE_ACMBOMROWSTRUCT = *LibreDWGc::DWG_TYPE_ACMBOMROWSTRUCT;
*DWG_TYPE_ACMBOMSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMBOMSTANDARDANSI;
*DWG_TYPE_ACMBOMSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMBOMSTANDARDCSN;
*DWG_TYPE_ACMBOMSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMBOMSTANDARDDIN;
*DWG_TYPE_ACMBOMSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMBOMSTANDARDISO;
*DWG_TYPE_ACMCENTERLINESTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMCENTERLINESTANDARDANSI;
*DWG_TYPE_ACMCENTERLINESTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMCENTERLINESTANDARDCSN;
*DWG_TYPE_ACMCENTERLINESTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMCENTERLINESTANDARDDIN;
*DWG_TYPE_ACMCENTERLINESTANDARDISO = *LibreDWGc::DWG_TYPE_ACMCENTERLINESTANDARDISO;
*DWG_TYPE_ACMDATADICTIONARY = *LibreDWGc::DWG_TYPE_ACMDATADICTIONARY;
*DWG_TYPE_ACMDATAENTRY = *LibreDWGc::DWG_TYPE_ACMDATAENTRY;
*DWG_TYPE_ACMDATAENTRYBLOCK = *LibreDWGc::DWG_TYPE_ACMDATAENTRYBLOCK;
*DWG_TYPE_ACMDATUMID = *LibreDWGc::DWG_TYPE_ACMDATUMID;
*DWG_TYPE_ACMDATUMSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMDATUMSTANDARDANSI;
*DWG_TYPE_ACMDATUMSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMDATUMSTANDARDCSN;
*DWG_TYPE_ACMDATUMSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMDATUMSTANDARDDIN;
*DWG_TYPE_ACMDATUMSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMDATUMSTANDARDISO;
*DWG_TYPE_ACMDATUMSTANDARDISO2012 = *LibreDWGc::DWG_TYPE_ACMDATUMSTANDARDISO2012;
*DWG_TYPE_ACMDETAILSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMDETAILSTANDARDANSI;
*DWG_TYPE_ACMDETAILSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMDETAILSTANDARDCSN;
*DWG_TYPE_ACMDETAILSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMDETAILSTANDARDDIN;
*DWG_TYPE_ACMDETAILSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMDETAILSTANDARDISO;
*DWG_TYPE_ACMDETAILTANDARDCUSTOM = *LibreDWGc::DWG_TYPE_ACMDETAILTANDARDCUSTOM;
*DWG_TYPE_ACMDIMBREAKPERSREACTOR = *LibreDWGc::DWG_TYPE_ACMDIMBREAKPERSREACTOR;
*DWG_TYPE_ACMEDRAWINGMAN = *LibreDWGc::DWG_TYPE_ACMEDRAWINGMAN;
*DWG_TYPE_ACMEVIEW = *LibreDWGc::DWG_TYPE_ACMEVIEW;
*DWG_TYPE_ACME_DATABASE = *LibreDWGc::DWG_TYPE_ACME_DATABASE;
*DWG_TYPE_ACME_DOCUMENT = *LibreDWGc::DWG_TYPE_ACME_DOCUMENT;
*DWG_TYPE_ACMFCFRAME = *LibreDWGc::DWG_TYPE_ACMFCFRAME;
*DWG_TYPE_ACMFCFSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMFCFSTANDARDANSI;
*DWG_TYPE_ACMFCFSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMFCFSTANDARDCSN;
*DWG_TYPE_ACMFCFSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMFCFSTANDARDDIN;
*DWG_TYPE_ACMFCFSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMFCFSTANDARDISO;
*DWG_TYPE_ACMFCFSTANDARDISO2004 = *LibreDWGc::DWG_TYPE_ACMFCFSTANDARDISO2004;
*DWG_TYPE_ACMFCFSTANDARDISO2012 = *LibreDWGc::DWG_TYPE_ACMFCFSTANDARDISO2012;
*DWG_TYPE_ACMIDSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMIDSTANDARDANSI;
*DWG_TYPE_ACMIDSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMIDSTANDARDCSN;
*DWG_TYPE_ACMIDSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMIDSTANDARDDIN;
*DWG_TYPE_ACMIDSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMIDSTANDARDISO;
*DWG_TYPE_ACMIDSTANDARDISO2004 = *LibreDWGc::DWG_TYPE_ACMIDSTANDARDISO2004;
*DWG_TYPE_ACMIDSTANDARDISO2012 = *LibreDWGc::DWG_TYPE_ACMIDSTANDARDISO2012;
*DWG_TYPE_ACMNOTESTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMNOTESTANDARDANSI;
*DWG_TYPE_ACMNOTESTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMNOTESTANDARDCSN;
*DWG_TYPE_ACMNOTESTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMNOTESTANDARDDIN;
*DWG_TYPE_ACMNOTESTANDARDISO = *LibreDWGc::DWG_TYPE_ACMNOTESTANDARDISO;
*DWG_TYPE_ACMPARTLIST = *LibreDWGc::DWG_TYPE_ACMPARTLIST;
*DWG_TYPE_ACMPICKOBJ = *LibreDWGc::DWG_TYPE_ACMPICKOBJ;
*DWG_TYPE_ACMSECTIONSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMSECTIONSTANDARDANSI;
*DWG_TYPE_ACMSECTIONSTANDARDCSN2002 = *LibreDWGc::DWG_TYPE_ACMSECTIONSTANDARDCSN2002;
*DWG_TYPE_ACMSECTIONSTANDARDCUSTOM = *LibreDWGc::DWG_TYPE_ACMSECTIONSTANDARDCUSTOM;
*DWG_TYPE_ACMSECTIONSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMSECTIONSTANDARDDIN;
*DWG_TYPE_ACMSECTIONSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMSECTIONSTANDARDISO;
*DWG_TYPE_ACMSECTIONSTANDARDISO2001 = *LibreDWGc::DWG_TYPE_ACMSECTIONSTANDARDISO2001;
*DWG_TYPE_ACMSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMSTANDARDANSI;
*DWG_TYPE_ACMSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMSTANDARDCSN;
*DWG_TYPE_ACMSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMSTANDARDDIN;
*DWG_TYPE_ACMSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMSTANDARDISO;
*DWG_TYPE_ACMSURFSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMSURFSTANDARDANSI;
*DWG_TYPE_ACMSURFSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMSURFSTANDARDCSN;
*DWG_TYPE_ACMSURFSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMSURFSTANDARDDIN;
*DWG_TYPE_ACMSURFSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMSURFSTANDARDISO;
*DWG_TYPE_ACMSURFSTANDARDISO2002 = *LibreDWGc::DWG_TYPE_ACMSURFSTANDARDISO2002;
*DWG_TYPE_ACMSURFSYM = *LibreDWGc::DWG_TYPE_ACMSURFSYM;
*DWG_TYPE_ACMTAPERSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMTAPERSTANDARDANSI;
*DWG_TYPE_ACMTAPERSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMTAPERSTANDARDCSN;
*DWG_TYPE_ACMTAPERSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMTAPERSTANDARDDIN;
*DWG_TYPE_ACMTAPERSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMTAPERSTANDARDISO;
*DWG_TYPE_ACMTHREADLINESTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMTHREADLINESTANDARDANSI;
*DWG_TYPE_ACMTHREADLINESTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMTHREADLINESTANDARDCSN;
*DWG_TYPE_ACMTHREADLINESTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMTHREADLINESTANDARDDIN;
*DWG_TYPE_ACMTHREADLINESTANDARDISO = *LibreDWGc::DWG_TYPE_ACMTHREADLINESTANDARDISO;
*DWG_TYPE_ACMWELDSTANDARDANSI = *LibreDWGc::DWG_TYPE_ACMWELDSTANDARDANSI;
*DWG_TYPE_ACMWELDSTANDARDCSN = *LibreDWGc::DWG_TYPE_ACMWELDSTANDARDCSN;
*DWG_TYPE_ACMWELDSTANDARDDIN = *LibreDWGc::DWG_TYPE_ACMWELDSTANDARDDIN;
*DWG_TYPE_ACMWELDSTANDARDISO = *LibreDWGc::DWG_TYPE_ACMWELDSTANDARDISO;
*DWG_TYPE_ACMWELDSYM = *LibreDWGc::DWG_TYPE_ACMWELDSYM;
*DWG_TYPE_ACRFATTGENMGR = *LibreDWGc::DWG_TYPE_ACRFATTGENMGR;
*DWG_TYPE_ACRFINSADJ = *LibreDWGc::DWG_TYPE_ACRFINSADJ;
*DWG_TYPE_ACRFINSADJUSTERMGR = *LibreDWGc::DWG_TYPE_ACRFINSADJUSTERMGR;
*DWG_TYPE_ACRFMCADAPIATTHOLDER = *LibreDWGc::DWG_TYPE_ACRFMCADAPIATTHOLDER;
*DWG_TYPE_ACRFOBJATTMGR = *LibreDWGc::DWG_TYPE_ACRFOBJATTMGR;
*DWG_TYPE_ACSH_SUBENT_MATERIAL_CLASS = *LibreDWGc::DWG_TYPE_ACSH_SUBENT_MATERIAL_CLASS;
*DWG_TYPE_AC_AM_2D_XREF_MGR = *LibreDWGc::DWG_TYPE_AC_AM_2D_XREF_MGR;
*DWG_TYPE_AC_AM_BASIC_VIEW = *LibreDWGc::DWG_TYPE_AC_AM_BASIC_VIEW;
*DWG_TYPE_AC_AM_BASIC_VIEW_DEF = *LibreDWGc::DWG_TYPE_AC_AM_BASIC_VIEW_DEF;
*DWG_TYPE_AC_AM_COMPLEX_HIDE_SITUATION = *LibreDWGc::DWG_TYPE_AC_AM_COMPLEX_HIDE_SITUATION;
*DWG_TYPE_AC_AM_COMP_VIEW_DEF = *LibreDWGc::DWG_TYPE_AC_AM_COMP_VIEW_DEF;
*DWG_TYPE_AC_AM_COMP_VIEW_INST = *LibreDWGc::DWG_TYPE_AC_AM_COMP_VIEW_INST;
*DWG_TYPE_AC_AM_DIRTY_NODES = *LibreDWGc::DWG_TYPE_AC_AM_DIRTY_NODES;
*DWG_TYPE_AC_AM_HIDE_SITUATION = *LibreDWGc::DWG_TYPE_AC_AM_HIDE_SITUATION;
*DWG_TYPE_AC_AM_MAPPER_CACHE = *LibreDWGc::DWG_TYPE_AC_AM_MAPPER_CACHE;
*DWG_TYPE_AC_AM_MASTER_VIEW_DEF = *LibreDWGc::DWG_TYPE_AC_AM_MASTER_VIEW_DEF;
*DWG_TYPE_AC_AM_MVD_DEP_MGR = *LibreDWGc::DWG_TYPE_AC_AM_MVD_DEP_MGR;
*DWG_TYPE_AC_AM_OVERRIDE_FILTER = *LibreDWGc::DWG_TYPE_AC_AM_OVERRIDE_FILTER;
*DWG_TYPE_AC_AM_PROPS_OVERRIDE = *LibreDWGc::DWG_TYPE_AC_AM_PROPS_OVERRIDE;
*DWG_TYPE_AC_AM_SHAFT_HIDE_SITUATION = *LibreDWGc::DWG_TYPE_AC_AM_SHAFT_HIDE_SITUATION;
*DWG_TYPE_AC_AM_STDP_VIEW_DEF = *LibreDWGc::DWG_TYPE_AC_AM_STDP_VIEW_DEF;
*DWG_TYPE_AC_AM_TRANSFORM_GHOST = *LibreDWGc::DWG_TYPE_AC_AM_TRANSFORM_GHOST;
*DWG_TYPE_ADAPPL = *LibreDWGc::DWG_TYPE_ADAPPL;
*DWG_TYPE_AECC_ALIGNMENT_DESIGN_CHECK_SET = *LibreDWGc::DWG_TYPE_AECC_ALIGNMENT_DESIGN_CHECK_SET;
*DWG_TYPE_AECC_ALIGNMENT_LABEL_SET = *LibreDWGc::DWG_TYPE_AECC_ALIGNMENT_LABEL_SET;
*DWG_TYPE_AECC_ALIGNMENT_LABEL_SET_EXT = *LibreDWGc::DWG_TYPE_AECC_ALIGNMENT_LABEL_SET_EXT;
*DWG_TYPE_AECC_ALIGNMENT_PARCEL_NODE = *LibreDWGc::DWG_TYPE_AECC_ALIGNMENT_PARCEL_NODE;
*DWG_TYPE_AECC_ALIGNMENT_STYLE = *LibreDWGc::DWG_TYPE_AECC_ALIGNMENT_STYLE;
*DWG_TYPE_AECC_APPURTENANCE_STYLE = *LibreDWGc::DWG_TYPE_AECC_APPURTENANCE_STYLE;
*DWG_TYPE_AECC_ASSEMBLY_STYLE = *LibreDWGc::DWG_TYPE_AECC_ASSEMBLY_STYLE;
*DWG_TYPE_AECC_BUILDING_SITE_STYLE = *LibreDWGc::DWG_TYPE_AECC_BUILDING_SITE_STYLE;
*DWG_TYPE_AECC_CANT_DIAGRAM_VIEW_STYLE = *LibreDWGc::DWG_TYPE_AECC_CANT_DIAGRAM_VIEW_STYLE;
*DWG_TYPE_AECC_CATCHMENT_STYLE = *LibreDWGc::DWG_TYPE_AECC_CATCHMENT_STYLE;
*DWG_TYPE_AECC_CLASS_NODE = *LibreDWGc::DWG_TYPE_AECC_CLASS_NODE;
*DWG_TYPE_AECC_CONTOURVIEW = *LibreDWGc::DWG_TYPE_AECC_CONTOURVIEW;
*DWG_TYPE_AECC_CORRIDOR_STYLE = *LibreDWGc::DWG_TYPE_AECC_CORRIDOR_STYLE;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CANT_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CANT_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CSV;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CURVE_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_CURVE_LABEL;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_DESIGNSPEED_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_DESIGNSPEED_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_GEOMPT_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_GEOMPT_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_INDEXED_PI_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_INDEXED_PI_LABEL;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_MINOR_STATION_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_MINOR_STATION_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_PI_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_PI_LABEL;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SPIRAL_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SPIRAL_LABEL;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STAEQU_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STAEQU_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_OFFSET_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_STATION_OFFSET_LABEL;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SUPERELEVATION_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_SUPERELEVATION_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TABLE;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TANGENT_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_TANGENT_LABEL;
*DWG_TYPE_AECC_DISP_REP_ALIGNMENT_VERTICAL_GEOMPT_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ALIGNMENT_VERTICAL_GEOMPT_LABELING;
*DWG_TYPE_AECC_DISP_REP_APPURTENANCE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_APPURTENANCE;
*DWG_TYPE_AECC_DISP_REP_APPURTENANCE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_APPURTENANCE_CSV;
*DWG_TYPE_AECC_DISP_REP_APPURTENANCE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_APPURTENANCE_LABELING;
*DWG_TYPE_AECC_DISP_REP_APPURTENANCE_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_APPURTENANCE_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_ASSEMBLY = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_ASSEMBLY;
*DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE;
*DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_PROFILE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_PROFILE;
*DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_SECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_AUTO_CORRIDOR_FEATURE_LINE_SECTION;
*DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE;
*DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_CSV;
*DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_PROFILE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_PROFILE;
*DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_SECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_AUTO_FEATURE_LINE_SECTION;
*DWG_TYPE_AECC_DISP_REP_BUILDINGSITE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_BUILDINGSITE;
*DWG_TYPE_AECC_DISP_REP_BUILDINGUTIL_CONNECTOR = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_BUILDINGUTIL_CONNECTOR;
*DWG_TYPE_AECC_DISP_REP_CANT_DIAGRAM_VIEW = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_CANT_DIAGRAM_VIEW;
*DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA;
*DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_CATCHMENT_AREA_LABEL;
*DWG_TYPE_AECC_DISP_REP_CORRIDOR = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_CORRIDOR;
*DWG_TYPE_AECC_DISP_REP_CROSSING_PIPE_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_CROSSING_PIPE_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_CROSSING_PRESSURE_PIPE_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_CROSSING_PRESSURE_PIPE_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_CSVSTATIONSLIDER = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_CSVSTATIONSLIDER;
*DWG_TYPE_AECC_DISP_REP_FACE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FACE;
*DWG_TYPE_AECC_DISP_REP_FEATURE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FEATURE;
*DWG_TYPE_AECC_DISP_REP_FEATURE_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FEATURE_LABEL;
*DWG_TYPE_AECC_DISP_REP_FEATURE_LINE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FEATURE_LINE;
*DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_CSV;
*DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_PROFILE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_PROFILE;
*DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_SECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FEATURE_LINE_SECTION;
*DWG_TYPE_AECC_DISP_REP_FITTING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FITTING;
*DWG_TYPE_AECC_DISP_REP_FITTING_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FITTING_CSV;
*DWG_TYPE_AECC_DISP_REP_FITTING_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FITTING_LABELING;
*DWG_TYPE_AECC_DISP_REP_FITTING_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FITTING_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_FLOW_SEGMENT_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_FLOW_SEGMENT_LABEL;
*DWG_TYPE_AECC_DISP_REP_GENERAL_SEGMENT_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_GENERAL_SEGMENT_LABEL;
*DWG_TYPE_AECC_DISP_REP_GRADING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_GRADING;
*DWG_TYPE_AECC_DISP_REP_GRAPH = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_GRAPH;
*DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_NETWORKPART = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_NETWORKPART;
*DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_PRESSUREPART = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_GRAPHPROFILE_PRESSUREPART;
*DWG_TYPE_AECC_DISP_REP_GRID_SURFACE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_GRID_SURFACE;
*DWG_TYPE_AECC_DISP_REP_GRID_SURFACE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_GRID_SURFACE_CSV;
*DWG_TYPE_AECC_DISP_REP_HORGEOMETRY_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_HORGEOMETRY_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_HYDRO_REGION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_HYDRO_REGION;
*DWG_TYPE_AECC_DISP_REP_INTERFERENCE_CHECK = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_INTERFERENCE_CHECK;
*DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART;
*DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART_SECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_INTERFERENCE_PART_SECTION;
*DWG_TYPE_AECC_DISP_REP_INTERSECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_INTERSECTION;
*DWG_TYPE_AECC_DISP_REP_INTERSECTION_LOCATION_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_INTERSECTION_LOCATION_LABELING;
*DWG_TYPE_AECC_DISP_REP_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_LABELING;
*DWG_TYPE_AECC_DISP_REP_LEGEND_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_LEGEND_TABLE;
*DWG_TYPE_AECC_DISP_REP_LINE_BETWEEN_POINTS_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_LINE_BETWEEN_POINTS_LABEL;
*DWG_TYPE_AECC_DISP_REP_LOTLINE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_LOTLINE_CSV;
*DWG_TYPE_AECC_DISP_REP_MASSHAULLINE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_MASSHAULLINE;
*DWG_TYPE_AECC_DISP_REP_MASS_HAUL_VIEW = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_MASS_HAUL_VIEW;
*DWG_TYPE_AECC_DISP_REP_MATCHLINE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_MATCHLINE_LABELING;
*DWG_TYPE_AECC_DISP_REP_MATCH_LINE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_MATCH_LINE;
*DWG_TYPE_AECC_DISP_REP_MATERIAL_SECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_MATERIAL_SECTION;
*DWG_TYPE_AECC_DISP_REP_NETWORK = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_NETWORK;
*DWG_TYPE_AECC_DISP_REP_NOTE_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_NOTE_LABEL;
*DWG_TYPE_AECC_DISP_REP_OFFSET_ELEV_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_OFFSET_ELEV_LABEL;
*DWG_TYPE_AECC_DISP_REP_PARCEL_BOUNDARY = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PARCEL_BOUNDARY;
*DWG_TYPE_AECC_DISP_REP_PARCEL_FACE_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PARCEL_FACE_LABEL;
*DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT;
*DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_LABEL;
*DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PARCEL_SEGMENT_TABLE;
*DWG_TYPE_AECC_DISP_REP_PARCEL_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PARCEL_TABLE;
*DWG_TYPE_AECC_DISP_REP_PIPE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PIPE;
*DWG_TYPE_AECC_DISP_REP_PIPENETWORK_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PIPENETWORK_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_PIPE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PIPE_CSV;
*DWG_TYPE_AECC_DISP_REP_PIPE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PIPE_LABELING;
*DWG_TYPE_AECC_DISP_REP_PIPE_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PIPE_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_PIPE_SECTION_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PIPE_SECTION_LABELING;
*DWG_TYPE_AECC_DISP_REP_PIPE_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PIPE_TABLE;
*DWG_TYPE_AECC_DISP_REP_POINT_ENT = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_POINT_ENT;
*DWG_TYPE_AECC_DISP_REP_POINT_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_POINT_GROUP;
*DWG_TYPE_AECC_DISP_REP_POINT_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_POINT_TABLE;
*DWG_TYPE_AECC_DISP_REP_PRESSUREPIPENETWORK = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PRESSUREPIPENETWORK;
*DWG_TYPE_AECC_DISP_REP_PRESSURE_PART_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PRESSURE_PART_TABLE;
*DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE;
*DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_CSV;
*DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_LABELING;
*DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_SECTION_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PRESSURE_PIPE_SECTION_LABELING;
*DWG_TYPE_AECC_DISP_REP_PROFILE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PROFILE;
*DWG_TYPE_AECC_DISP_REP_PROFILEDATA_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PROFILEDATA_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION;
*DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PROFILE_PROJECTION_LABEL;
*DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW;
*DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW_DEPTH_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_PROFILE_VIEW_DEPTH_LABEL;
*DWG_TYPE_AECC_DISP_REP_QUANTITY_TAKEOFF_AGGREGATE_EARTHWORK_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_QUANTITY_TAKEOFF_AGGREGATE_EARTHWORK_TABLE;
*DWG_TYPE_AECC_DISP_REP_RIGHT_OF_WAY = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_RIGHT_OF_WAY;
*DWG_TYPE_AECC_DISP_REP_SAMPLELINE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SAMPLELINE_LABELING;
*DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE;
*DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SAMPLE_LINE_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION;
*DWG_TYPE_AECC_DISP_REP_SECTIONALDATA_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTIONALDATA_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTIONDATA_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTIONDATA_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTIONSEGMENT_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTIONSEGMENT_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR;
*DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR_POINT_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_CORRIDOR_POINT_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTION_GRADEBREAK_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_GRADEBREAK_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTION_MINOR_OFFSET_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_MINOR_OFFSET_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTION_OFFSET_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_OFFSET_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTION_PIPENETWORK = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_PIPENETWORK;
*DWG_TYPE_AECC_DISP_REP_SECTION_PRESSUREPIPENETWORK = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_PRESSUREPIPENETWORK;
*DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION;
*DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_PROJECTION_LABEL;
*DWG_TYPE_AECC_DISP_REP_SECTION_SEGMENT_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_SEGMENT_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SECTION_VIEW = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_VIEW;
*DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_DEPTH_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_DEPTH_LABEL;
*DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_QUANTITY_TAKEOFF_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SECTION_VIEW_QUANTITY_TAKEOFF_TABLE;
*DWG_TYPE_AECC_DISP_REP_SHEET = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SHEET;
*DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_LABELING;
*DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SPANNING_PIPE_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_STATION_ELEV_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_STATION_ELEV_LABEL;
*DWG_TYPE_AECC_DISP_REP_STRUCTURE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_STRUCTURE;
*DWG_TYPE_AECC_DISP_REP_STRUCTURE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_STRUCTURE_CSV;
*DWG_TYPE_AECC_DISP_REP_STRUCTURE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_STRUCTURE_LABELING;
*DWG_TYPE_AECC_DISP_REP_STRUCTURE_PROFILE_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_STRUCTURE_PROFILE_LABELING;
*DWG_TYPE_AECC_DISP_REP_STRUCTURE_SECTION_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_STRUCTURE_SECTION_LABELING;
*DWG_TYPE_AECC_DISP_REP_SUBASSEMBLY = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SUBASSEMBLY;
*DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_DIAGRAM_VIEW = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SUPERELEVATION_DIAGRAM_VIEW;
*DWG_TYPE_AECC_DISP_REP_SURFACE_CONTOUR_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SURFACE_CONTOUR_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SURFACE_ELEVATION_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SURFACE_ELEVATION_LABEL;
*DWG_TYPE_AECC_DISP_REP_SURFACE_SLOPE_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SURFACE_SLOPE_LABEL;
*DWG_TYPE_AECC_DISP_REP_SURVEY_FIGURE_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SURVEY_FIGURE_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_SVFIGURE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SVFIGURE;
*DWG_TYPE_AECC_DISP_REP_SVFIGURE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SVFIGURE_CSV;
*DWG_TYPE_AECC_DISP_REP_SVFIGURE_PROFILE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SVFIGURE_PROFILE;
*DWG_TYPE_AECC_DISP_REP_SVFIGURE_SECTION = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SVFIGURE_SECTION;
*DWG_TYPE_AECC_DISP_REP_SVFIGURE_SEGMENT_LABEL = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SVFIGURE_SEGMENT_LABEL;
*DWG_TYPE_AECC_DISP_REP_SVNETWORK = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_SVNETWORK;
*DWG_TYPE_AECC_DISP_REP_TANGENT_INTERSECTION_TABLE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_TANGENT_INTERSECTION_TABLE;
*DWG_TYPE_AECC_DISP_REP_TIN_SURFACE = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_TIN_SURFACE;
*DWG_TYPE_AECC_DISP_REP_TIN_SURFACE_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_TIN_SURFACE_CSV;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CRESTCURVE_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CRESTCURVE_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CSV = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_CSV;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_HAGEOMPT_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_HAGEOMPT_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_LINE_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_LINE_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_MINOR_STATION_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_MINOR_STATION_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_PVI_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_PVI_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_SAGCURVE_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_SAGCURVE_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VALIGNMENT_STATION_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VALIGNMENT_STATION_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VERTICALGEOMETRY_BAND_LABEL_GROUP = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VERTICALGEOMETRY_BAND_LABEL_GROUP;
*DWG_TYPE_AECC_DISP_REP_VIEWFRAME_LABELING = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VIEWFRAME_LABELING;
*DWG_TYPE_AECC_DISP_REP_VIEW_FRAME = *LibreDWGc::DWG_TYPE_AECC_DISP_REP_VIEW_FRAME;
*DWG_TYPE_AECC_FEATURELINE_STYLE = *LibreDWGc::DWG_TYPE_AECC_FEATURELINE_STYLE;
*DWG_TYPE_AECC_FEATURE_STYLE = *LibreDWGc::DWG_TYPE_AECC_FEATURE_STYLE;
*DWG_TYPE_AECC_FITTING_STYLE = *LibreDWGc::DWG_TYPE_AECC_FITTING_STYLE;
*DWG_TYPE_AECC_FORMAT_MANAGER_OBJECT = *LibreDWGc::DWG_TYPE_AECC_FORMAT_MANAGER_OBJECT;
*DWG_TYPE_AECC_GRADEVIEW = *LibreDWGc::DWG_TYPE_AECC_GRADEVIEW;
*DWG_TYPE_AECC_GRADING_CRITERIA = *LibreDWGc::DWG_TYPE_AECC_GRADING_CRITERIA;
*DWG_TYPE_AECC_GRADING_CRITERIA_SET = *LibreDWGc::DWG_TYPE_AECC_GRADING_CRITERIA_SET;
*DWG_TYPE_AECC_GRADING_GROUP = *LibreDWGc::DWG_TYPE_AECC_GRADING_GROUP;
*DWG_TYPE_AECC_GRADING_STYLE = *LibreDWGc::DWG_TYPE_AECC_GRADING_STYLE;
*DWG_TYPE_AECC_IMPORT_STORM_SEWER_DEFAULTS = *LibreDWGc::DWG_TYPE_AECC_IMPORT_STORM_SEWER_DEFAULTS;
*DWG_TYPE_AECC_INTERFERENCE_STYLE = *LibreDWGc::DWG_TYPE_AECC_INTERFERENCE_STYLE;
*DWG_TYPE_AECC_INTERSECTION_STYLE = *LibreDWGc::DWG_TYPE_AECC_INTERSECTION_STYLE;
*DWG_TYPE_AECC_LABEL_COLLECTOR_STYLE = *LibreDWGc::DWG_TYPE_AECC_LABEL_COLLECTOR_STYLE;
*DWG_TYPE_AECC_LABEL_NODE = *LibreDWGc::DWG_TYPE_AECC_LABEL_NODE;
*DWG_TYPE_AECC_LABEL_RADIAL_LINE_STYLE = *LibreDWGc::DWG_TYPE_AECC_LABEL_RADIAL_LINE_STYLE;
*DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_CURVE_OR_SPIRAL_STYLE = *LibreDWGc::DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_CURVE_OR_SPIRAL_STYLE;
*DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_STYLE = *LibreDWGc::DWG_TYPE_AECC_LABEL_TEXT_ITERATOR_STYLE;
*DWG_TYPE_AECC_LABEL_TEXT_STYLE = *LibreDWGc::DWG_TYPE_AECC_LABEL_TEXT_STYLE;
*DWG_TYPE_AECC_LABEL_VECTOR_ARROW_STYLE = *LibreDWGc::DWG_TYPE_AECC_LABEL_VECTOR_ARROW_STYLE;
*DWG_TYPE_AECC_LEGEND_TABLE_STYLE = *LibreDWGc::DWG_TYPE_AECC_LEGEND_TABLE_STYLE;
*DWG_TYPE_AECC_MASS_HAUL_LINE_STYLE = *LibreDWGc::DWG_TYPE_AECC_MASS_HAUL_LINE_STYLE;
*DWG_TYPE_AECC_MASS_HAUL_VIEW_STYLE = *LibreDWGc::DWG_TYPE_AECC_MASS_HAUL_VIEW_STYLE;
*DWG_TYPE_AECC_MATCHLINE_STYLE = *LibreDWGc::DWG_TYPE_AECC_MATCHLINE_STYLE;
*DWG_TYPE_AECC_MATERIAL_STYLE = *LibreDWGc::DWG_TYPE_AECC_MATERIAL_STYLE;
*DWG_TYPE_AECC_NETWORK_PART_CATALOG_DEF_NODE = *LibreDWGc::DWG_TYPE_AECC_NETWORK_PART_CATALOG_DEF_NODE;
*DWG_TYPE_AECC_NETWORK_PART_FAMILY_ITEM = *LibreDWGc::DWG_TYPE_AECC_NETWORK_PART_FAMILY_ITEM;
*DWG_TYPE_AECC_NETWORK_PART_LIST = *LibreDWGc::DWG_TYPE_AECC_NETWORK_PART_LIST;
*DWG_TYPE_AECC_NETWORK_RULE = *LibreDWGc::DWG_TYPE_AECC_NETWORK_RULE;
*DWG_TYPE_AECC_PARCEL_NODE = *LibreDWGc::DWG_TYPE_AECC_PARCEL_NODE;
*DWG_TYPE_AECC_PARCEL_STYLE = *LibreDWGc::DWG_TYPE_AECC_PARCEL_STYLE;
*DWG_TYPE_AECC_PART_SIZE_FILTER = *LibreDWGc::DWG_TYPE_AECC_PART_SIZE_FILTER;
*DWG_TYPE_AECC_PIPE_RULES = *LibreDWGc::DWG_TYPE_AECC_PIPE_RULES;
*DWG_TYPE_AECC_PIPE_STYLE = *LibreDWGc::DWG_TYPE_AECC_PIPE_STYLE;
*DWG_TYPE_AECC_PIPE_STYLE_EXTENSION = *LibreDWGc::DWG_TYPE_AECC_PIPE_STYLE_EXTENSION;
*DWG_TYPE_AECC_POINTCLOUD_STYLE = *LibreDWGc::DWG_TYPE_AECC_POINTCLOUD_STYLE;
*DWG_TYPE_AECC_POINTVIEW = *LibreDWGc::DWG_TYPE_AECC_POINTVIEW;
*DWG_TYPE_AECC_POINT_STYLE = *LibreDWGc::DWG_TYPE_AECC_POINT_STYLE;
*DWG_TYPE_AECC_PRESSURE_PART_LIST = *LibreDWGc::DWG_TYPE_AECC_PRESSURE_PART_LIST;
*DWG_TYPE_AECC_PRESSURE_PIPE_STYLE = *LibreDWGc::DWG_TYPE_AECC_PRESSURE_PIPE_STYLE;
*DWG_TYPE_AECC_PROFILESECTIONENTITY_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILESECTIONENTITY_STYLE;
*DWG_TYPE_AECC_PROFILE_DESIGN_CHECK_SET = *LibreDWGc::DWG_TYPE_AECC_PROFILE_DESIGN_CHECK_SET;
*DWG_TYPE_AECC_PROFILE_LABEL_SET = *LibreDWGc::DWG_TYPE_AECC_PROFILE_LABEL_SET;
*DWG_TYPE_AECC_PROFILE_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_STYLE;
*DWG_TYPE_AECC_PROFILE_VIEW_BAND_STYLE_SET = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_BAND_STYLE_SET;
*DWG_TYPE_AECC_PROFILE_VIEW_DATA_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_DATA_BAND_STYLE;
*DWG_TYPE_AECC_PROFILE_VIEW_HORIZONTAL_GEOMETRY_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_HORIZONTAL_GEOMETRY_BAND_STYLE;
*DWG_TYPE_AECC_PROFILE_VIEW_PIPE_NETWORK_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_PIPE_NETWORK_BAND_STYLE;
*DWG_TYPE_AECC_PROFILE_VIEW_SECTIONAL_DATA_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_SECTIONAL_DATA_BAND_STYLE;
*DWG_TYPE_AECC_PROFILE_VIEW_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_STYLE;
*DWG_TYPE_AECC_PROFILE_VIEW_SUPERELEVATION_DIAGRAM_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_SUPERELEVATION_DIAGRAM_BAND_STYLE;
*DWG_TYPE_AECC_PROFILE_VIEW_VERTICAL_GEOMETRY_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_PROFILE_VIEW_VERTICAL_GEOMETRY_BAND_STYLE;
*DWG_TYPE_AECC_QUANTITY_TAKEOFF_CRITERIA = *LibreDWGc::DWG_TYPE_AECC_QUANTITY_TAKEOFF_CRITERIA;
*DWG_TYPE_AECC_ROADWAYLINK_STYLE = *LibreDWGc::DWG_TYPE_AECC_ROADWAYLINK_STYLE;
*DWG_TYPE_AECC_ROADWAYMARKER_STYLE = *LibreDWGc::DWG_TYPE_AECC_ROADWAYMARKER_STYLE;
*DWG_TYPE_AECC_ROADWAYSHAPE_STYLE = *LibreDWGc::DWG_TYPE_AECC_ROADWAYSHAPE_STYLE;
*DWG_TYPE_AECC_ROADWAY_STYLE_SET = *LibreDWGc::DWG_TYPE_AECC_ROADWAY_STYLE_SET;
*DWG_TYPE_AECC_ROOT_SETTINGS_NODE = *LibreDWGc::DWG_TYPE_AECC_ROOT_SETTINGS_NODE;
*DWG_TYPE_AECC_SAMPLE_LINE_GROUP_STYLE = *LibreDWGc::DWG_TYPE_AECC_SAMPLE_LINE_GROUP_STYLE;
*DWG_TYPE_AECC_SAMPLE_LINE_STYLE = *LibreDWGc::DWG_TYPE_AECC_SAMPLE_LINE_STYLE;
*DWG_TYPE_AECC_SECTION_LABEL_SET = *LibreDWGc::DWG_TYPE_AECC_SECTION_LABEL_SET;
*DWG_TYPE_AECC_SECTION_STYLE = *LibreDWGc::DWG_TYPE_AECC_SECTION_STYLE;
*DWG_TYPE_AECC_SECTION_VIEW_BAND_STYLE_SET = *LibreDWGc::DWG_TYPE_AECC_SECTION_VIEW_BAND_STYLE_SET;
*DWG_TYPE_AECC_SECTION_VIEW_DATA_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_SECTION_VIEW_DATA_BAND_STYLE;
*DWG_TYPE_AECC_SECTION_VIEW_ROAD_SURFACE_BAND_STYLE = *LibreDWGc::DWG_TYPE_AECC_SECTION_VIEW_ROAD_SURFACE_BAND_STYLE;
*DWG_TYPE_AECC_SECTION_VIEW_STYLE = *LibreDWGc::DWG_TYPE_AECC_SECTION_VIEW_STYLE;
*DWG_TYPE_AECC_SETTINGS_NODE = *LibreDWGc::DWG_TYPE_AECC_SETTINGS_NODE;
*DWG_TYPE_AECC_SHEET_STYLE = *LibreDWGc::DWG_TYPE_AECC_SHEET_STYLE;
*DWG_TYPE_AECC_SLOPE_PATTERN_STYLE = *LibreDWGc::DWG_TYPE_AECC_SLOPE_PATTERN_STYLE;
*DWG_TYPE_AECC_STATION_FORMAT_STYLE = *LibreDWGc::DWG_TYPE_AECC_STATION_FORMAT_STYLE;
*DWG_TYPE_AECC_STRUCTURE_RULES = *LibreDWGc::DWG_TYPE_AECC_STRUCTURE_RULES;
*DWG_TYPE_AECC_STUCTURE_STYLE = *LibreDWGc::DWG_TYPE_AECC_STUCTURE_STYLE;
*DWG_TYPE_AECC_SUPERELEVATION_DIAGRAM_VIEW_STYLE = *LibreDWGc::DWG_TYPE_AECC_SUPERELEVATION_DIAGRAM_VIEW_STYLE;
*DWG_TYPE_AECC_SURFACE_STYLE = *LibreDWGc::DWG_TYPE_AECC_SURFACE_STYLE;
*DWG_TYPE_AECC_SVFIGURE_STYLE = *LibreDWGc::DWG_TYPE_AECC_SVFIGURE_STYLE;
*DWG_TYPE_AECC_SVNETWORK_STYLE = *LibreDWGc::DWG_TYPE_AECC_SVNETWORK_STYLE;
*DWG_TYPE_AECC_TABLE_STYLE = *LibreDWGc::DWG_TYPE_AECC_TABLE_STYLE;
*DWG_TYPE_AECC_TAG_MANAGER = *LibreDWGc::DWG_TYPE_AECC_TAG_MANAGER;
*DWG_TYPE_AECC_TREE_NODE = *LibreDWGc::DWG_TYPE_AECC_TREE_NODE;
*DWG_TYPE_AECC_USER_DEFINED_ATTRIBUTE_CLASSIFICATION = *LibreDWGc::DWG_TYPE_AECC_USER_DEFINED_ATTRIBUTE_CLASSIFICATION;
*DWG_TYPE_AECC_VALIGNMENT_STYLE_EXTENSION = *LibreDWGc::DWG_TYPE_AECC_VALIGNMENT_STYLE_EXTENSION;
*DWG_TYPE_AECC_VIEW_FRAME_STYLE = *LibreDWGc::DWG_TYPE_AECC_VIEW_FRAME_STYLE;
*DWG_TYPE_AECS_DISP_PROPS_MEMBER = *LibreDWGc::DWG_TYPE_AECS_DISP_PROPS_MEMBER;
*DWG_TYPE_AECS_DISP_PROPS_MEMBER_LOGICAL = *LibreDWGc::DWG_TYPE_AECS_DISP_PROPS_MEMBER_LOGICAL;
*DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN = *LibreDWGc::DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN;
*DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN_SKETCH = *LibreDWGc::DWG_TYPE_AECS_DISP_PROPS_MEMBER_PLAN_SKETCH;
*DWG_TYPE_AECS_DISP_PROPS_MEMBER_PROJECTED = *LibreDWGc::DWG_TYPE_AECS_DISP_PROPS_MEMBER_PROJECTED;
*DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DESIGN = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DESIGN;
*DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DETAIL = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_ELEVATION_DETAIL;
*DWG_TYPE_AECS_DISP_REP_MEMBER_LOGICAL = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_LOGICAL;
*DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DESIGN = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DESIGN;
*DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DETAIL = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_MODEL_DETAIL;
*DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DESIGN = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DESIGN;
*DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DETAIL = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_DETAIL;
*DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_SKETCH = *LibreDWGc::DWG_TYPE_AECS_DISP_REP_MEMBER_PLAN_SKETCH;
*DWG_TYPE_AECS_MEMBER_NODE_SHAPE = *LibreDWGc::DWG_TYPE_AECS_MEMBER_NODE_SHAPE;
*DWG_TYPE_AECS_MEMBER_STYLE = *LibreDWGc::DWG_TYPE_AECS_MEMBER_STYLE;
*DWG_TYPE_AEC_2DSECTION_STYLE = *LibreDWGc::DWG_TYPE_AEC_2DSECTION_STYLE;
*DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPBDGELEVLINEPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPBDGSECTIONLINEPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCEILINGGRIDPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCOLUMNGRIDPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLLAYOUTPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPCURTAINWALLUNITPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPMVBLOCKREFPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPROOFPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPROOFPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPROOFPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPROOFPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPROOFSLABPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPSLABPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPSLABPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPSLABPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPSLABPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPSPACEPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPWALLPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPWALLPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPWALLPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPWALLPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN100;
*DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPWINDOWASSEMBLYPLAN50;
*DWG_TYPE_AEC_AECDBDISPREPZONE100 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPZONE100;
*DWG_TYPE_AEC_AECDBDISPREPZONE50 = *LibreDWGc::DWG_TYPE_AEC_AECDBDISPREPZONE50;
*DWG_TYPE_AEC_AECDBZONEDEF = *LibreDWGc::DWG_TYPE_AEC_AECDBZONEDEF;
*DWG_TYPE_AEC_AECDBZONESTYLE = *LibreDWGc::DWG_TYPE_AEC_AECDBZONESTYLE;
*DWG_TYPE_AEC_ANCHOR_OPENINGBASE_TO_WALL = *LibreDWGc::DWG_TYPE_AEC_ANCHOR_OPENINGBASE_TO_WALL;
*DWG_TYPE_AEC_CLASSIFICATION_DEF = *LibreDWGc::DWG_TYPE_AEC_CLASSIFICATION_DEF;
*DWG_TYPE_AEC_CLASSIFICATION_SYSTEM_DEF = *LibreDWGc::DWG_TYPE_AEC_CLASSIFICATION_SYSTEM_DEF;
*DWG_TYPE_AEC_CLEANUP_GROUP_DEF = *LibreDWGc::DWG_TYPE_AEC_CLEANUP_GROUP_DEF;
*DWG_TYPE_AEC_CURTAIN_WALL_LAYOUT_STYLE = *LibreDWGc::DWG_TYPE_AEC_CURTAIN_WALL_LAYOUT_STYLE;
*DWG_TYPE_AEC_CURTAIN_WALL_UNIT_STYLE = *LibreDWGc::DWG_TYPE_AEC_CURTAIN_WALL_UNIT_STYLE;
*DWG_TYPE_AEC_CVSECTIONVIEW = *LibreDWGc::DWG_TYPE_AEC_CVSECTIONVIEW;
*DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN = *LibreDWGc::DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN;
*DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN100 = *LibreDWGc::DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN100;
*DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN50 = *LibreDWGc::DWG_TYPE_AEC_DB_DISP_REP_DIM_GROUP_PLAN50;
*DWG_TYPE_AEC_DIM_STYLE = *LibreDWGc::DWG_TYPE_AEC_DIM_STYLE;
*DWG_TYPE_AEC_DISPLAYTHEME_STYLE = *LibreDWGc::DWG_TYPE_AEC_DISPLAYTHEME_STYLE;
*DWG_TYPE_AEC_DISP_PROPS_2D_SECTION = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_2D_SECTION;
*DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME;
*DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME_RESULT = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_CLIP_VOLUME_RESULT;
*DWG_TYPE_AEC_DISP_PROPS_DIM = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DIM;
*DWG_TYPE_AEC_DISP_PROPS_DISPLAYTHEME = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DISPLAYTHEME;
*DWG_TYPE_AEC_DISP_PROPS_DOOR = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DOOR;
*DWG_TYPE_AEC_DISP_PROPS_DOOR_NOMINAL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DOOR_NOMINAL;
*DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_100 = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_100;
*DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DOOR_PLAN_50;
*DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_SYMBOL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_DOOR_THRESHOLD_SYMBOL_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_EDITINPLACEPROFILE_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_EDITINPLACEPROFILE_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_ENT = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_ENT;
*DWG_TYPE_AEC_DISP_PROPS_ENT_REF = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_ENT_REF;
*DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_GRID_ASSEMBLY_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_LAYOUT_CURVE = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_LAYOUT_CURVE;
*DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID2D = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID2D;
*DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID3D = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_LAYOUT_GRID3D;
*DWG_TYPE_AEC_DISP_PROPS_MASKBLOCK = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_MASKBLOCK;
*DWG_TYPE_AEC_DISP_PROPS_MASS_ELEM_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_MASS_ELEM_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_MASS_GROUP = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_MASS_GROUP;
*DWG_TYPE_AEC_DISP_PROPS_MATERIAL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_MATERIAL;
*DWG_TYPE_AEC_DISP_PROPS_OPENING = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_OPENING;
*DWG_TYPE_AEC_DISP_PROPS_POLYGON_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_POLYGON_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_POLYGON_TRUECOLOUR = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_POLYGON_TRUECOLOUR;
*DWG_TYPE_AEC_DISP_PROPS_RAILING_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_RAILING_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_RAILING_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_RAILING_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_ROOF = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_ROOF;
*DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB;
*DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_ROOFSLAB_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_SCHEDULE_TABLE = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_SCHEDULE_TABLE;
*DWG_TYPE_AEC_DISP_PROPS_SLAB = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_SLAB;
*DWG_TYPE_AEC_DISP_PROPS_SLAB_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_SLAB_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_SLICE = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_SLICE;
*DWG_TYPE_AEC_DISP_PROPS_SPACE_DECOMPOSED = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_SPACE_DECOMPOSED;
*DWG_TYPE_AEC_DISP_PROPS_SPACE_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_SPACE_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_SPACE_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_SPACE_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_STAIR_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_STAIR_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN_OVERLAPPING = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_STAIR_PLAN_OVERLAPPING;
*DWG_TYPE_AEC_DISP_PROPS_WALL_GRAPH = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WALL_GRAPH;
*DWG_TYPE_AEC_DISP_PROPS_WALL_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WALL_MODEL;
*DWG_TYPE_AEC_DISP_PROPS_WALL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WALL_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_WALL_SCHEM = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WALL_SCHEM;
*DWG_TYPE_AEC_DISP_PROPS_WINDOW = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WINDOW;
*DWG_TYPE_AEC_DISP_PROPS_WINDOW_ASSEMBLY_SILL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WINDOW_ASSEMBLY_SILL_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_WINDOW_NOMINAL = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WINDOW_NOMINAL;
*DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_100 = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_100;
*DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WINDOW_PLAN_50;
*DWG_TYPE_AEC_DISP_PROPS_WINDOW_SILL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_WINDOW_SILL_PLAN;
*DWG_TYPE_AEC_DISP_PROPS_ZONE = *LibreDWGc::DWG_TYPE_AEC_DISP_PROPS_ZONE;
*DWG_TYPE_AEC_DISP_REP_2D_SECTION = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_2D_SECTION;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN100 = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN100;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN50 = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSELEMPLAN50;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN100 = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN100;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN50 = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPMASSGROUPPLAN50;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN100 = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN100;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN50 = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLAN50;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLANREFLECTED = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGPLANREFLECTED;
*DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGSILLPLAN = *LibreDWGc::DWG_TYPE_AEC_DISPREPAECDBDISPREPOPENINGSILLPLAN;
*DWG_TYPE_AEC_DISP_REP_ANCHOR = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ANCHOR;
*DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID;
*DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_MODEL;
*DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ANCHOR_BUB_TO_GRID_RCP;
*DWG_TYPE_AEC_DISP_REP_ANCHOR_ENT_TO_NODE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ANCHOR_ENT_TO_NODE;
*DWG_TYPE_AEC_DISP_REP_ANCHOR_EXT_TAG_TO_ENT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ANCHOR_EXT_TAG_TO_ENT;
*DWG_TYPE_AEC_DISP_REP_ANCHOR_TAG_TO_ENT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ANCHOR_TAG_TO_ENT;
*DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_MODEL;
*DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_PLAN;
*DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_ELEVLINE_RCP;
*DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_MODEL;
*DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_PLAN;
*DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_SECTIONLINE_RCP;
*DWG_TYPE_AEC_DISP_REP_BDG_SECTION_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_SECTION_MODEL;
*DWG_TYPE_AEC_DISP_REP_BDG_SECTION_SUBDIV = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_BDG_SECTION_SUBDIV;
*DWG_TYPE_AEC_DISP_REP_CEILING_GRID = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CEILING_GRID;
*DWG_TYPE_AEC_DISP_REP_CEILING_GRID_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CEILING_GRID_MODEL;
*DWG_TYPE_AEC_DISP_REP_CEILING_GRID_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CEILING_GRID_RCP;
*DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_MODEL;
*DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_PLAN;
*DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT;
*DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT_SUBDIV = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CLIP_VOLUME_RESULT_SUBDIV;
*DWG_TYPE_AEC_DISP_REP_COL_BLOCK = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_BLOCK;
*DWG_TYPE_AEC_DISP_REP_COL_CIRCARC2D = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CIRCARC2D;
*DWG_TYPE_AEC_DISP_REP_COL_CONCOINCIDENT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONCOINCIDENT;
*DWG_TYPE_AEC_DISP_REP_COL_CONCONCENTRIC = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONCONCENTRIC;
*DWG_TYPE_AEC_DISP_REP_COL_CONEQUALDISTANCE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONEQUALDISTANCE;
*DWG_TYPE_AEC_DISP_REP_COL_CONMIDPOINT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONMIDPOINT;
*DWG_TYPE_AEC_DISP_REP_COL_CONNECTOR = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONNECTOR;
*DWG_TYPE_AEC_DISP_REP_COL_CONNORMAL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONNORMAL;
*DWG_TYPE_AEC_DISP_REP_COL_CONPARALLEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONPARALLEL;
*DWG_TYPE_AEC_DISP_REP_COL_CONPERPENDICULAR = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONPERPENDICULAR;
*DWG_TYPE_AEC_DISP_REP_COL_CONSYMMETRIC = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONSYMMETRIC;
*DWG_TYPE_AEC_DISP_REP_COL_CONTANGENT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_CONTANGENT;
*DWG_TYPE_AEC_DISP_REP_COL_DIMANGLE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_DIMANGLE;
*DWG_TYPE_AEC_DISP_REP_COL_DIMDIAMETER = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_DIMDIAMETER;
*DWG_TYPE_AEC_DISP_REP_COL_DIMDISTANCE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_DIMDISTANCE;
*DWG_TYPE_AEC_DISP_REP_COL_DIMLENGTH = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_DIMLENGTH;
*DWG_TYPE_AEC_DISP_REP_COL_DIMMAJORRADIUS = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_DIMMAJORRADIUS;
*DWG_TYPE_AEC_DISP_REP_COL_DIMMINORRADIUS = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_DIMMINORRADIUS;
*DWG_TYPE_AEC_DISP_REP_COL_ELLIPARC2D = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_ELLIPARC2D;
*DWG_TYPE_AEC_DISP_REP_COL_LAYOUTDATA = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_LAYOUTDATA;
*DWG_TYPE_AEC_DISP_REP_COL_LINE2D = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_LINE2D;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_ADD = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_ADD;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_CUTPLANE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_CUTPLANE;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_EXTRUSION = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_EXTRUSION;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_GROUP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_GROUP;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_LOFT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_LOFT;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_PATH = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_PATH;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_REVOLVE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_REVOLVE;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_SUBTRACT = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_SUBTRACT;
*DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_TRANSITION = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_MODIFIER_TRANSITION;
*DWG_TYPE_AEC_DISP_REP_COL_POINT2D = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_POINT2D;
*DWG_TYPE_AEC_DISP_REP_COL_PROFILE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_PROFILE;
*DWG_TYPE_AEC_DISP_REP_COLUMN_GRID = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COLUMN_GRID;
*DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_MODEL;
*DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COLUMN_GRID_RCP;
*DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE;
*DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE_REF = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_COL_WORKPLANE_REF;
*DWG_TYPE_AEC_DISP_REP_CONFIG = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CONFIG;
*DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_MODEL;
*DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_LAYOUT_PLAN;
*DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_MODEL;
*DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_CURTAIN_WALL_UNIT_PLAN;
*DWG_TYPE_AEC_DISP_REP_DCM_DIMRADIUS = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DCM_DIMRADIUS;
*DWG_TYPE_AEC_DISP_REP_DISPLAYTHEME = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DISPLAYTHEME;
*DWG_TYPE_AEC_DISP_REP_DOOR_ELEV = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_ELEV;
*DWG_TYPE_AEC_DISP_REP_DOOR_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_MODEL;
*DWG_TYPE_AEC_DISP_REP_DOOR_NOMINAL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_NOMINAL;
*DWG_TYPE_AEC_DISP_REP_DOOR_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_PLAN;
*DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_50;
*DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_HEKTO = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_PLAN_HEKTO;
*DWG_TYPE_AEC_DISP_REP_DOOR_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_RCP;
*DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_PLAN;
*DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_SYMBOL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_DOOR_THRESHOLD_SYMBOL_PLAN;
*DWG_TYPE_AEC_DISP_REP_EDITINPLACEPROFILE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_EDITINPLACEPROFILE;
*DWG_TYPE_AEC_DISP_REP_ENT_REF = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ENT_REF;
*DWG_TYPE_AEC_DISP_REP_LAYOUT_CURVE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_LAYOUT_CURVE;
*DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID2D = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID2D;
*DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID3D = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_LAYOUT_GRID3D;
*DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF;
*DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASKBLOCK_REF_RCP;
*DWG_TYPE_AEC_DISP_REP_MASS_ELEM_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASS_ELEM_MODEL;
*DWG_TYPE_AEC_DISP_REP_MASS_ELEM_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASS_ELEM_RCP;
*DWG_TYPE_AEC_DISP_REP_MASS_ELEM_SCHEM = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASS_ELEM_SCHEM;
*DWG_TYPE_AEC_DISP_REP_MASS_GROUP_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASS_GROUP_MODEL;
*DWG_TYPE_AEC_DISP_REP_MASS_GROUP_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASS_GROUP_PLAN;
*DWG_TYPE_AEC_DISP_REP_MASS_GROUP_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MASS_GROUP_RCP;
*DWG_TYPE_AEC_DISP_REP_MATERIAL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MATERIAL;
*DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF;
*DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_MODEL;
*DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_MVBLOCK_REF_RCP;
*DWG_TYPE_AEC_DISP_REP_OPENING = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_OPENING;
*DWG_TYPE_AEC_DISP_REP_OPENING_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_OPENING_MODEL;
*DWG_TYPE_AEC_DISP_REP_POLYGON_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_POLYGON_MODEL;
*DWG_TYPE_AEC_DISP_REP_POLYGON_TRUECOLOUR = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_POLYGON_TRUECOLOUR;
*DWG_TYPE_AEC_DISP_REP_RAILING_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_RAILING_MODEL;
*DWG_TYPE_AEC_DISP_REP_RAILING_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_RAILING_PLAN;
*DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_100 = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_100;
*DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_RAILING_PLAN_50;
*DWG_TYPE_AEC_DISP_REP_ROOF_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ROOF_MODEL;
*DWG_TYPE_AEC_DISP_REP_ROOF_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ROOF_PLAN;
*DWG_TYPE_AEC_DISP_REP_ROOF_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ROOF_RCP;
*DWG_TYPE_AEC_DISP_REP_ROOFSLAB_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ROOFSLAB_MODEL;
*DWG_TYPE_AEC_DISP_REP_ROOFSLAB_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ROOFSLAB_PLAN;
*DWG_TYPE_AEC_DISP_REP_SCHEDULE_TABLE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SCHEDULE_TABLE;
*DWG_TYPE_AEC_DISP_REP_SET = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SET;
*DWG_TYPE_AEC_DISP_REP_SLAB_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SLAB_MODEL;
*DWG_TYPE_AEC_DISP_REP_SLAB_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SLAB_PLAN;
*DWG_TYPE_AEC_DISP_REP_SLICE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SLICE;
*DWG_TYPE_AEC_DISP_REP_SPACE_DECOMPOSED = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SPACE_DECOMPOSED;
*DWG_TYPE_AEC_DISP_REP_SPACE_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SPACE_MODEL;
*DWG_TYPE_AEC_DISP_REP_SPACE_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SPACE_PLAN;
*DWG_TYPE_AEC_DISP_REP_SPACE_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SPACE_RCP;
*DWG_TYPE_AEC_DISP_REP_SPACE_VOLUME = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_SPACE_VOLUME;
*DWG_TYPE_AEC_DISP_REP_STAIR_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_STAIR_MODEL;
*DWG_TYPE_AEC_DISP_REP_STAIR_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_STAIR_PLAN;
*DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_100 = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_100;
*DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_50;
*DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_OVERLAPPING = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_STAIR_PLAN_OVERLAPPING;
*DWG_TYPE_AEC_DISP_REP_STAIR_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_STAIR_RCP;
*DWG_TYPE_AEC_DISP_REP_WALL_GRAPH = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WALL_GRAPH;
*DWG_TYPE_AEC_DISP_REP_WALL_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WALL_MODEL;
*DWG_TYPE_AEC_DISP_REP_WALL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WALL_PLAN;
*DWG_TYPE_AEC_DISP_REP_WALL_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WALL_RCP;
*DWG_TYPE_AEC_DISP_REP_WALL_SCHEM = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WALL_SCHEM;
*DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_MODEL;
*DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_ASSEMBLY_PLAN;
*DWG_TYPE_AEC_DISP_REP_WINDOWASSEMBLY_SILL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOWASSEMBLY_SILL_PLAN;
*DWG_TYPE_AEC_DISP_REP_WINDOW_ELEV = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_ELEV;
*DWG_TYPE_AEC_DISP_REP_WINDOW_MODEL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_MODEL;
*DWG_TYPE_AEC_DISP_REP_WINDOW_NOMINAL = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_NOMINAL;
*DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN;
*DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_100 = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_100;
*DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_PLAN_50;
*DWG_TYPE_AEC_DISP_REP_WINDOW_RCP = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_RCP;
*DWG_TYPE_AEC_DISP_REP_WINDOW_SILL_PLAN = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_WINDOW_SILL_PLAN;
*DWG_TYPE_AEC_DISP_REP_ZONE = *LibreDWGc::DWG_TYPE_AEC_DISP_REP_ZONE;
*DWG_TYPE_AEC_DISPROPSMASSELEMPLANCOMMON = *LibreDWGc::DWG_TYPE_AEC_DISPROPSMASSELEMPLANCOMMON;
*DWG_TYPE_AEC_DISPROPSMASSGROUPPLANCOMMON = *LibreDWGc::DWG_TYPE_AEC_DISPROPSMASSGROUPPLANCOMMON;
*DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMON = *LibreDWGc::DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMON;
*DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMONHATCHED = *LibreDWGc::DWG_TYPE_AEC_DISPROPSOPENINGPLANCOMMONHATCHED;
*DWG_TYPE_AEC_DISPROPSOPENINGSILLPLAN = *LibreDWGc::DWG_TYPE_AEC_DISPROPSOPENINGSILLPLAN;
*DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_100 = *LibreDWGc::DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_100;
*DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_ROPS_RAILING_PLAN_50;
*DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_100 = *LibreDWGc::DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_100;
*DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_50 = *LibreDWGc::DWG_TYPE_AEC_DISP_ROPS_STAIR_PLAN_50;
*DWG_TYPE_AEC_DOOR_STYLE = *LibreDWGc::DWG_TYPE_AEC_DOOR_STYLE;
*DWG_TYPE_AEC_ENDCAP_STYLE = *LibreDWGc::DWG_TYPE_AEC_ENDCAP_STYLE;
*DWG_TYPE_AEC_FRAME_DEF = *LibreDWGc::DWG_TYPE_AEC_FRAME_DEF;
*DWG_TYPE_AEC_LAYERKEY_STYLE = *LibreDWGc::DWG_TYPE_AEC_LAYERKEY_STYLE;
*DWG_TYPE_AEC_LIST_DEF = *LibreDWGc::DWG_TYPE_AEC_LIST_DEF;
*DWG_TYPE_AEC_MASKBLOCK_DEF = *LibreDWGc::DWG_TYPE_AEC_MASKBLOCK_DEF;
*DWG_TYPE_AEC_MASS_ELEM_STYLE = *LibreDWGc::DWG_TYPE_AEC_MASS_ELEM_STYLE;
*DWG_TYPE_AEC_MATERIAL_DEF = *LibreDWGc::DWG_TYPE_AEC_MATERIAL_DEF;
*DWG_TYPE_AEC_MVBLOCK_DEF = *LibreDWGc::DWG_TYPE_AEC_MVBLOCK_DEF;
*DWG_TYPE_AEC_MVBLOCK_REF = *LibreDWGc::DWG_TYPE_AEC_MVBLOCK_REF;
*DWG_TYPE_AEC_NOTIFICATION_TRACKER = *LibreDWGc::DWG_TYPE_AEC_NOTIFICATION_TRACKER;
*DWG_TYPE_AEC_POLYGON = *LibreDWGc::DWG_TYPE_AEC_POLYGON;
*DWG_TYPE_AEC_POLYGON_STYLE = *LibreDWGc::DWG_TYPE_AEC_POLYGON_STYLE;
*DWG_TYPE_AEC_PROPERTY_SET_DEF = *LibreDWGc::DWG_TYPE_AEC_PROPERTY_SET_DEF;
*DWG_TYPE_AEC_RAILING_STYLE = *LibreDWGc::DWG_TYPE_AEC_RAILING_STYLE;
*DWG_TYPE_AEC_REFEDIT_STATUS_TRACKER = *LibreDWGc::DWG_TYPE_AEC_REFEDIT_STATUS_TRACKER;
*DWG_TYPE_AEC_ROOFSLABEDGE_STYLE = *LibreDWGc::DWG_TYPE_AEC_ROOFSLABEDGE_STYLE;
*DWG_TYPE_AEC_ROOFSLAB_STYLE = *LibreDWGc::DWG_TYPE_AEC_ROOFSLAB_STYLE;
*DWG_TYPE_AEC_SCHEDULE_DATA_FORMAT = *LibreDWGc::DWG_TYPE_AEC_SCHEDULE_DATA_FORMAT;
*DWG_TYPE_AEC_SLABEDGE_STYLE = *LibreDWGc::DWG_TYPE_AEC_SLABEDGE_STYLE;
*DWG_TYPE_AEC_SLAB_STYLE = *LibreDWGc::DWG_TYPE_AEC_SLAB_STYLE;
*DWG_TYPE_AEC_SPACE_STYLES = *LibreDWGc::DWG_TYPE_AEC_SPACE_STYLES;
*DWG_TYPE_AEC_STAIR_STYLE = *LibreDWGc::DWG_TYPE_AEC_STAIR_STYLE;
*DWG_TYPE_AEC_STAIR_WINDER_STYLE = *LibreDWGc::DWG_TYPE_AEC_STAIR_WINDER_STYLE;
*DWG_TYPE_AEC_STAIR_WINDER_TYPE_BALANCED = *LibreDWGc::DWG_TYPE_AEC_STAIR_WINDER_TYPE_BALANCED;
*DWG_TYPE_AEC_STAIR_WINDER_TYPE_MANUAL = *LibreDWGc::DWG_TYPE_AEC_STAIR_WINDER_TYPE_MANUAL;
*DWG_TYPE_AEC_STAIR_WINDER_TYPE_SINGLE_POINT = *LibreDWGc::DWG_TYPE_AEC_STAIR_WINDER_TYPE_SINGLE_POINT;
*DWG_TYPE_AEC_VARS_AECBBLDSRV = *LibreDWGc::DWG_TYPE_AEC_VARS_AECBBLDSRV;
*DWG_TYPE_AEC_VARS_ARCHBASE = *LibreDWGc::DWG_TYPE_AEC_VARS_ARCHBASE;
*DWG_TYPE_AEC_VARS_DWG_SETUP = *LibreDWGc::DWG_TYPE_AEC_VARS_DWG_SETUP;
*DWG_TYPE_AEC_VARS_MUNICH = *LibreDWGc::DWG_TYPE_AEC_VARS_MUNICH;
*DWG_TYPE_AEC_VARS_STRUCTUREBASE = *LibreDWGc::DWG_TYPE_AEC_VARS_STRUCTUREBASE;
*DWG_TYPE_AEC_WALLMOD_STYLE = *LibreDWGc::DWG_TYPE_AEC_WALLMOD_STYLE;
*DWG_TYPE_AEC_WALL_STYLE = *LibreDWGc::DWG_TYPE_AEC_WALL_STYLE;
*DWG_TYPE_AEC_WINDOW_ASSEMBLY_STYLE = *LibreDWGc::DWG_TYPE_AEC_WINDOW_ASSEMBLY_STYLE;
*DWG_TYPE_AEC_WINDOW_STYLE = *LibreDWGc::DWG_TYPE_AEC_WINDOW_STYLE;
*DWG_TYPE_ALIGNMENTGRIPENTITY = *LibreDWGc::DWG_TYPE_ALIGNMENTGRIPENTITY;
*DWG_TYPE_AMCONTEXTMGR = *LibreDWGc::DWG_TYPE_AMCONTEXTMGR;
*DWG_TYPE_AMDTADMENUSTATE = *LibreDWGc::DWG_TYPE_AMDTADMENUSTATE;
*DWG_TYPE_AMDTAMMENUSTATE = *LibreDWGc::DWG_TYPE_AMDTAMMENUSTATE;
*DWG_TYPE_AMDTBROWSERDBTAB = *LibreDWGc::DWG_TYPE_AMDTBROWSERDBTAB;
*DWG_TYPE_AMDTDMMENUSTATE = *LibreDWGc::DWG_TYPE_AMDTDMMENUSTATE;
*DWG_TYPE_AMDTEDGESTANDARDDIN = *LibreDWGc::DWG_TYPE_AMDTEDGESTANDARDDIN;
*DWG_TYPE_AMDTEDGESTANDARDDIN13715 = *LibreDWGc::DWG_TYPE_AMDTEDGESTANDARDDIN13715;
*DWG_TYPE_AMDTEDGESTANDARDISO = *LibreDWGc::DWG_TYPE_AMDTEDGESTANDARDISO;
*DWG_TYPE_AMDTEDGESTANDARDISO13715 = *LibreDWGc::DWG_TYPE_AMDTEDGESTANDARDISO13715;
*DWG_TYPE_AMDTFORMULAUPDATEDISPATCHER = *LibreDWGc::DWG_TYPE_AMDTFORMULAUPDATEDISPATCHER;
*DWG_TYPE_AMDTINTERNALREACTOR = *LibreDWGc::DWG_TYPE_AMDTINTERNALREACTOR;
*DWG_TYPE_AMDTMCOMMENUSTATE = *LibreDWGc::DWG_TYPE_AMDTMCOMMENUSTATE;
*DWG_TYPE_AMDTMENUSTATEMGR = *LibreDWGc::DWG_TYPE_AMDTMENUSTATEMGR;
*DWG_TYPE_AMDTNOTE = *LibreDWGc::DWG_TYPE_AMDTNOTE;
*DWG_TYPE_AMDTNOTETEMPLATEDB = *LibreDWGc::DWG_TYPE_AMDTNOTETEMPLATEDB;
*DWG_TYPE_AMDTSECTIONSYM = *LibreDWGc::DWG_TYPE_AMDTSECTIONSYM;
*DWG_TYPE_AMDTSECTIONSYMLABEL = *LibreDWGc::DWG_TYPE_AMDTSECTIONSYMLABEL;
*DWG_TYPE_AMDTSYSATTR = *LibreDWGc::DWG_TYPE_AMDTSYSATTR;
*DWG_TYPE_AMGOBJPROPCFG = *LibreDWGc::DWG_TYPE_AMGOBJPROPCFG;
*DWG_TYPE_AMGSETTINGSOBJ = *LibreDWGc::DWG_TYPE_AMGSETTINGSOBJ;
*DWG_TYPE_AMIMASTER = *LibreDWGc::DWG_TYPE_AMIMASTER;
*DWG_TYPE_AM_DRAWING_MGR = *LibreDWGc::DWG_TYPE_AM_DRAWING_MGR;
*DWG_TYPE_AM_DWGMGR_NAME = *LibreDWGc::DWG_TYPE_AM_DWGMGR_NAME;
*DWG_TYPE_AM_DWG_DOCUMENT = *LibreDWGc::DWG_TYPE_AM_DWG_DOCUMENT;
*DWG_TYPE_AM_DWG_SHEET = *LibreDWGc::DWG_TYPE_AM_DWG_SHEET;
*DWG_TYPE_AM_VIEWDIMPARMAP = *LibreDWGc::DWG_TYPE_AM_VIEWDIMPARMAP;
*DWG_TYPE_BINRECORD = *LibreDWGc::DWG_TYPE_BINRECORD;
*DWG_TYPE_CAMSCATALOGAPPOBJECT = *LibreDWGc::DWG_TYPE_CAMSCATALOGAPPOBJECT;
*DWG_TYPE_CAMSSTRUCTBTNSTATE = *LibreDWGc::DWG_TYPE_CAMSSTRUCTBTNSTATE;
*DWG_TYPE_CATALOGSTATE = *LibreDWGc::DWG_TYPE_CATALOGSTATE;
*DWG_TYPE_CBROWSERAPPOBJECT = *LibreDWGc::DWG_TYPE_CBROWSERAPPOBJECT;
*DWG_TYPE_DEPMGR = *LibreDWGc::DWG_TYPE_DEPMGR;
*DWG_TYPE_DMBASEELEMENT = *LibreDWGc::DWG_TYPE_DMBASEELEMENT;
*DWG_TYPE_DMDEFAULTSTYLE = *LibreDWGc::DWG_TYPE_DMDEFAULTSTYLE;
*DWG_TYPE_DMLEGEND = *LibreDWGc::DWG_TYPE_DMLEGEND;
*DWG_TYPE_DMMAP = *LibreDWGc::DWG_TYPE_DMMAP;
*DWG_TYPE_DMMAPMANAGER = *LibreDWGc::DWG_TYPE_DMMAPMANAGER;
*DWG_TYPE_DMSTYLECATEGORY = *LibreDWGc::DWG_TYPE_DMSTYLECATEGORY;
*DWG_TYPE_DMSTYLELIBRARY = *LibreDWGc::DWG_TYPE_DMSTYLELIBRARY;
*DWG_TYPE_DMSTYLEREFERENCE = *LibreDWGc::DWG_TYPE_DMSTYLEREFERENCE;
*DWG_TYPE_DMSTYLIZEDENTITIESTABLE = *LibreDWGc::DWG_TYPE_DMSTYLIZEDENTITIESTABLE;
*DWG_TYPE_DMSURROGATESTYLESETS = *LibreDWGc::DWG_TYPE_DMSURROGATESTYLESETS;
*DWG_TYPE_DM_PLACEHOLDER = *LibreDWGc::DWG_TYPE_DM_PLACEHOLDER;
*DWG_TYPE_EXACTERMXREFMAP = *LibreDWGc::DWG_TYPE_EXACTERMXREFMAP;
*DWG_TYPE_EXACXREFPANELOBJECT = *LibreDWGc::DWG_TYPE_EXACXREFPANELOBJECT;
*DWG_TYPE_EXPO_NOTIFYBLOCK = *LibreDWGc::DWG_TYPE_EXPO_NOTIFYBLOCK;
*DWG_TYPE_EXPO_NOTIFYHALL = *LibreDWGc::DWG_TYPE_EXPO_NOTIFYHALL;
*DWG_TYPE_EXPO_NOTIFYPILLAR = *LibreDWGc::DWG_TYPE_EXPO_NOTIFYPILLAR;
*DWG_TYPE_EXPO_NOTIFYSTAND = *LibreDWGc::DWG_TYPE_EXPO_NOTIFYSTAND;
*DWG_TYPE_EXPO_NOTIFYSTANDNOPOLY = *LibreDWGc::DWG_TYPE_EXPO_NOTIFYSTANDNOPOLY;
*DWG_TYPE_GSMANAGER = *LibreDWGc::DWG_TYPE_GSMANAGER;
*DWG_TYPE_IRD_DSC_DICT = *LibreDWGc::DWG_TYPE_IRD_DSC_DICT;
*DWG_TYPE_IRD_DSC_RECORD = *LibreDWGc::DWG_TYPE_IRD_DSC_RECORD;
*DWG_TYPE_IRD_OBJ_RECORD = *LibreDWGc::DWG_TYPE_IRD_OBJ_RECORD;
*DWG_TYPE_MAPFSMRVOBJECT = *LibreDWGc::DWG_TYPE_MAPFSMRVOBJECT;
*DWG_TYPE_MAPGWSUNDOOBJECT = *LibreDWGc::DWG_TYPE_MAPGWSUNDOOBJECT;
*DWG_TYPE_MAPIAMMOUDLE = *LibreDWGc::DWG_TYPE_MAPIAMMOUDLE;
*DWG_TYPE_MAPMETADATAOBJECT = *LibreDWGc::DWG_TYPE_MAPMETADATAOBJECT;
*DWG_TYPE_MAPRESOURCEMANAGEROBJECT = *LibreDWGc::DWG_TYPE_MAPRESOURCEMANAGEROBJECT;
*DWG_TYPE_McDbContainer2 = *LibreDWGc::DWG_TYPE_McDbContainer2;
*DWG_TYPE_McDbMarker = *LibreDWGc::DWG_TYPE_McDbMarker;
*DWG_TYPE_NAMEDAPPL = *LibreDWGc::DWG_TYPE_NAMEDAPPL;
*DWG_TYPE_NEWSTDPARTPARLIST = *LibreDWGc::DWG_TYPE_NEWSTDPARTPARLIST;
*DWG_TYPE_NOTEPOSITION = *LibreDWGc::DWG_TYPE_NOTEPOSITION;
*DWG_TYPE_OBJCLONER = *LibreDWGc::DWG_TYPE_OBJCLONER;
*DWG_TYPE_PARAMMGR = *LibreDWGc::DWG_TYPE_PARAMMGR;
*DWG_TYPE_PARAMSCOPE = *LibreDWGc::DWG_TYPE_PARAMSCOPE;
*DWG_TYPE_PILLAR = *LibreDWGc::DWG_TYPE_PILLAR;
*DWG_TYPE_STDPART2D = *LibreDWGc::DWG_TYPE_STDPART2D;
*DWG_TYPE_TCH_ARROW = *LibreDWGc::DWG_TYPE_TCH_ARROW;
*DWG_TYPE_TCH_AXIS_LABEL = *LibreDWGc::DWG_TYPE_TCH_AXIS_LABEL;
*DWG_TYPE_TCH_BLOCK_INSERT = *LibreDWGc::DWG_TYPE_TCH_BLOCK_INSERT;
*DWG_TYPE_TCH_COLUMN = *LibreDWGc::DWG_TYPE_TCH_COLUMN;
*DWG_TYPE_TCH_DBCONFIG = *LibreDWGc::DWG_TYPE_TCH_DBCONFIG;
*DWG_TYPE_TCH_DIMENSION2 = *LibreDWGc::DWG_TYPE_TCH_DIMENSION2;
*DWG_TYPE_TCH_DRAWINGINDEX = *LibreDWGc::DWG_TYPE_TCH_DRAWINGINDEX;
*DWG_TYPE_TCH_HANDRAIL = *LibreDWGc::DWG_TYPE_TCH_HANDRAIL;
*DWG_TYPE_TCH_LINESTAIR = *LibreDWGc::DWG_TYPE_TCH_LINESTAIR;
*DWG_TYPE_TCH_OPENING = *LibreDWGc::DWG_TYPE_TCH_OPENING;
*DWG_TYPE_TCH_RECTSTAIR = *LibreDWGc::DWG_TYPE_TCH_RECTSTAIR;
*DWG_TYPE_TCH_SLAB = *LibreDWGc::DWG_TYPE_TCH_SLAB;
*DWG_TYPE_TCH_SPACE = *LibreDWGc::DWG_TYPE_TCH_SPACE;
*DWG_TYPE_TCH_TEXT = *LibreDWGc::DWG_TYPE_TCH_TEXT;
*DWG_TYPE_TCH_WALL = *LibreDWGc::DWG_TYPE_TCH_WALL;
*DWG_TYPE_TGrupoPuntos = *LibreDWGc::DWG_TYPE_TGrupoPuntos;
*DWG_TYPE_VAACIMAGEINVENTORY = *LibreDWGc::DWG_TYPE_VAACIMAGEINVENTORY;
*DWG_TYPE_VAACXREFPANELOBJECT = *LibreDWGc::DWG_TYPE_VAACXREFPANELOBJECT;
*DWG_TYPE_FREED = *LibreDWGc::DWG_TYPE_FREED;
*DWG_TYPE_UNKNOWN_ENT = *LibreDWGc::DWG_TYPE_UNKNOWN_ENT;
*DWG_TYPE_UNKNOWN_OBJ = *LibreDWGc::DWG_TYPE_UNKNOWN_OBJ;
*DWG_TYPE_UNUSED_r11 = *LibreDWGc::DWG_TYPE_UNUSED_r11;
*DWG_TYPE_LINE_r11 = *LibreDWGc::DWG_TYPE_LINE_r11;
*DWG_TYPE_POINT_r11 = *LibreDWGc::DWG_TYPE_POINT_r11;
*DWG_TYPE_CIRCLE_r11 = *LibreDWGc::DWG_TYPE_CIRCLE_r11;
*DWG_TYPE_SHAPE_r11 = *LibreDWGc::DWG_TYPE_SHAPE_r11;
*DWG_TYPE_REPEAT_r11 = *LibreDWGc::DWG_TYPE_REPEAT_r11;
*DWG_TYPE_ENDREP_r11 = *LibreDWGc::DWG_TYPE_ENDREP_r11;
*DWG_TYPE_TEXT_r11 = *LibreDWGc::DWG_TYPE_TEXT_r11;
*DWG_TYPE_ARC_r11 = *LibreDWGc::DWG_TYPE_ARC_r11;
*DWG_TYPE_TRACE_r11 = *LibreDWGc::DWG_TYPE_TRACE_r11;
*DWG_TYPE_LOAD_r11 = *LibreDWGc::DWG_TYPE_LOAD_r11;
*DWG_TYPE_SOLID_r11 = *LibreDWGc::DWG_TYPE_SOLID_r11;
*DWG_TYPE_BLOCK_r11 = *LibreDWGc::DWG_TYPE_BLOCK_r11;
*DWG_TYPE_ENDBLK_r11 = *LibreDWGc::DWG_TYPE_ENDBLK_r11;
*DWG_TYPE_INSERT_r11 = *LibreDWGc::DWG_TYPE_INSERT_r11;
*DWG_TYPE_ATTDEF_r11 = *LibreDWGc::DWG_TYPE_ATTDEF_r11;
*DWG_TYPE_ATTRIB_r11 = *LibreDWGc::DWG_TYPE_ATTRIB_r11;
*DWG_TYPE_SEQEND_r11 = *LibreDWGc::DWG_TYPE_SEQEND_r11;
*DWG_TYPE_JUMP_r11 = *LibreDWGc::DWG_TYPE_JUMP_r11;
*DWG_TYPE_POLYLINE_r11 = *LibreDWGc::DWG_TYPE_POLYLINE_r11;
*DWG_TYPE_VERTEX_r11 = *LibreDWGc::DWG_TYPE_VERTEX_r11;
*DWG_TYPE_3DLINE_r11 = *LibreDWGc::DWG_TYPE_3DLINE_r11;
*DWG_TYPE_3DFACE_r11 = *LibreDWGc::DWG_TYPE_3DFACE_r11;
*DWG_TYPE_DIMENSION_r11 = *LibreDWGc::DWG_TYPE_DIMENSION_r11;
*DWG_TYPE_VIEWPORT_r11 = *LibreDWGc::DWG_TYPE_VIEWPORT_r11;
*DWG_TYPE_UNKNOWN_r11 = *LibreDWGc::DWG_TYPE_UNKNOWN_r11;
*DWG_NOERR = *LibreDWGc::DWG_NOERR;
*DWG_ERR_WRONGCRC = *LibreDWGc::DWG_ERR_WRONGCRC;
*DWG_ERR_NOTYETSUPPORTED = *LibreDWGc::DWG_ERR_NOTYETSUPPORTED;
*DWG_ERR_UNHANDLEDCLASS = *LibreDWGc::DWG_ERR_UNHANDLEDCLASS;
*DWG_ERR_INVALIDTYPE = *LibreDWGc::DWG_ERR_INVALIDTYPE;
*DWG_ERR_INVALIDHANDLE = *LibreDWGc::DWG_ERR_INVALIDHANDLE;
*DWG_ERR_INVALIDEED = *LibreDWGc::DWG_ERR_INVALIDEED;
*DWG_ERR_VALUEOUTOFBOUNDS = *LibreDWGc::DWG_ERR_VALUEOUTOFBOUNDS;
*DWG_ERR_CLASSESNOTFOUND = *LibreDWGc::DWG_ERR_CLASSESNOTFOUND;
*DWG_ERR_SECTIONNOTFOUND = *LibreDWGc::DWG_ERR_SECTIONNOTFOUND;
*DWG_ERR_PAGENOTFOUND = *LibreDWGc::DWG_ERR_PAGENOTFOUND;
*DWG_ERR_INTERNALERROR = *LibreDWGc::DWG_ERR_INTERNALERROR;
*DWG_ERR_INVALIDDWG = *LibreDWGc::DWG_ERR_INVALIDDWG;
*DWG_ERR_IOERROR = *LibreDWGc::DWG_ERR_IOERROR;
*DWG_ERR_OUTOFMEM = *LibreDWGc::DWG_ERR_OUTOFMEM;
*DWG_HDL_OWNER = *LibreDWGc::DWG_HDL_OWNER;
*DWG_HDL_SOFTOWN = *LibreDWGc::DWG_HDL_SOFTOWN;
*DWG_HDL_HARDOWN = *LibreDWGc::DWG_HDL_HARDOWN;
*DWG_HDL_SOFTPTR = *LibreDWGc::DWG_HDL_SOFTPTR;
*DWG_HDL_HARDPTR = *LibreDWGc::DWG_HDL_HARDPTR;
*DWG_COLOR_METHOD_VOID = *LibreDWGc::DWG_COLOR_METHOD_VOID;
*DWG_COLOR_METHOD_BYLAYER = *LibreDWGc::DWG_COLOR_METHOD_BYLAYER;
*DWG_COLOR_METHOD_BYBLOCK = *LibreDWGc::DWG_COLOR_METHOD_BYBLOCK;
*DWG_COLOR_METHOD_ACI = *LibreDWGc::DWG_COLOR_METHOD_ACI;
*DWG_COLOR_METHOD_TRUECOLOR = *LibreDWGc::DWG_COLOR_METHOD_TRUECOLOR;
*DWG_COLOR_METHOD_FGCOLOR = *LibreDWGc::DWG_COLOR_METHOD_FGCOLOR;
*DWG_COLOR_METHOD_NONE = *LibreDWGc::DWG_COLOR_METHOD_NONE;
*SPLINETYPE_QUADRATIC = *LibreDWGc::SPLINETYPE_QUADRATIC;
*SPLINETYPE_CUBIC = *LibreDWGc::SPLINETYPE_CUBIC;
*SPLINE_SCENARIO_SPLINE = *LibreDWGc::SPLINE_SCENARIO_SPLINE;
*SPLINE_SCENARIO_BEZIER = *LibreDWGc::SPLINE_SCENARIO_BEZIER;
*SPLINE_SPLINEFLAGS_METHOD_FIT_POINTS = *LibreDWGc::SPLINE_SPLINEFLAGS_METHOD_FIT_POINTS;
*SPLINE_SPLINEFLAGS_CV_FRAME_SHOW = *LibreDWGc::SPLINE_SPLINEFLAGS_CV_FRAME_SHOW;
*SPLINE_SPLINEFLAGS_CLOSED = *LibreDWGc::SPLINE_SPLINEFLAGS_CLOSED;
*SPLINE_KNOTPARAM_CHORD = *LibreDWGc::SPLINE_KNOTPARAM_CHORD;
*SPLINE_KNOTPARAM_SQUARE_ROOT = *LibreDWGc::SPLINE_KNOTPARAM_SQUARE_ROOT;
*SPLINE_KNOTPARAM_UNIFORM = *LibreDWGc::SPLINE_KNOTPARAM_UNIFORM;
*SPLINE_KNOTPARAM_CUSTOM = *LibreDWGc::SPLINE_KNOTPARAM_CUSTOM;
*EXTRA_R11_UNKNOWN_1 = *LibreDWGc::EXTRA_R11_UNKNOWN_1;
*EXTRA_R11_HAS_EED = *LibreDWGc::EXTRA_R11_HAS_EED;
*EXTRA_R11_HAS_VIEWPORT = *LibreDWGc::EXTRA_R11_HAS_VIEWPORT;
*EXTRA_R11_UNKNOWN_8 = *LibreDWGc::EXTRA_R11_UNKNOWN_8;
*EXTRA_R11_UNKNOWN_16 = *LibreDWGc::EXTRA_R11_UNKNOWN_16;
*EXTRA_R11_UNKNOWN_32 = *LibreDWGc::EXTRA_R11_UNKNOWN_32;
*EXTRA_R11_UNKNOWN_64 = *LibreDWGc::EXTRA_R11_UNKNOWN_64;
*EXTRA_R11_UNKNOWN_128 = *LibreDWGc::EXTRA_R11_UNKNOWN_128;
*FLAG_R11_HAS_COLOR = *LibreDWGc::FLAG_R11_HAS_COLOR;
*FLAG_R11_HAS_LTYPE = *LibreDWGc::FLAG_R11_HAS_LTYPE;
*FLAG_R11_HAS_ELEVATION = *LibreDWGc::FLAG_R11_HAS_ELEVATION;
*FLAG_R11_HAS_THICKNESS = *LibreDWGc::FLAG_R11_HAS_THICKNESS;
*FLAG_R11_HAS_UNKNOWN_16 = *LibreDWGc::FLAG_R11_HAS_UNKNOWN_16;
*FLAG_R11_HAS_HANDLING = *LibreDWGc::FLAG_R11_HAS_HANDLING;
*FLAG_R11_HAS_PSPACE = *LibreDWGc::FLAG_R11_HAS_PSPACE;
*FLAG_R11_HAS_ATTRIBS = *LibreDWGc::FLAG_R11_HAS_ATTRIBS;
*FLAG_R11_DIMENSION_LINEAR = *LibreDWGc::FLAG_R11_DIMENSION_LINEAR;
*FLAG_R11_DIMENSION_ALIGNED = *LibreDWGc::FLAG_R11_DIMENSION_ALIGNED;
*FLAG_R11_DIMENSION_ANG2LN = *LibreDWGc::FLAG_R11_DIMENSION_ANG2LN;
*FLAG_R11_DIMENSION_DIAMETER = *LibreDWGc::FLAG_R11_DIMENSION_DIAMETER;
*FLAG_R11_DIMENSION_RADIUS = *LibreDWGc::FLAG_R11_DIMENSION_RADIUS;
*FLAG_R11_DIMENSION_ANG3PT = *LibreDWGc::FLAG_R11_DIMENSION_ANG3PT;
*FLAG_R11_DIMENSION_ORDINATE = *LibreDWGc::FLAG_R11_DIMENSION_ORDINATE;
*FLAG_R11_DIMENSION_ORDINATE_TYPE = *LibreDWGc::FLAG_R11_DIMENSION_ORDINATE_TYPE;
*FLAG_R11_DIMENSION_TEXT_POSITIONED = *LibreDWGc::FLAG_R11_DIMENSION_TEXT_POSITIONED;
*OPTS_R11_XREF_DEP = *LibreDWGc::OPTS_R11_XREF_DEP;
*OPTS_R11_XREF_REF = *LibreDWGc::OPTS_R11_XREF_REF;
*OPTS_R11_XREF_RESOLVED = *LibreDWGc::OPTS_R11_XREF_RESOLVED;
*OPTS_R11_XDATA = *LibreDWGc::OPTS_R11_XDATA;
*OPTS_R11_3DFACE_HAS_Z_FIRST = *LibreDWGc::OPTS_R11_3DFACE_HAS_Z_FIRST;
*OPTS_R11_3DFACE_HAS_Z_SECOND = *LibreDWGc::OPTS_R11_3DFACE_HAS_Z_SECOND;
*OPTS_R11_3DFACE_HAS_Z_THIRD = *LibreDWGc::OPTS_R11_3DFACE_HAS_Z_THIRD;
*OPTS_R11_3DFACE_HAS_Z_FOURTH = *LibreDWGc::OPTS_R11_3DFACE_HAS_Z_FOURTH;
*OPTS_R11_3DLINE_HAS_Z_FIRST = *LibreDWGc::OPTS_R11_3DLINE_HAS_Z_FIRST;
*OPTS_R11_3DLINE_HAS_Z_SECOND = *LibreDWGc::OPTS_R11_3DLINE_HAS_Z_SECOND;
*OPTS_R11_ARC_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_ARC_HAS_EXTRUSION;
*OPTS_R11_ATTDEF_UNKNOWN_1 = *LibreDWGc::OPTS_R11_ATTDEF_UNKNOWN_1;
*OPTS_R11_ATTDEF_HAS_ROTATION = *LibreDWGc::OPTS_R11_ATTDEF_HAS_ROTATION;
*OPTS_R11_ATTDEF_HAS_WIDTH_FACTOR = *LibreDWGc::OPTS_R11_ATTDEF_HAS_WIDTH_FACTOR;
*OPTS_R11_ATTDEF_HAS_OBLIQUE_ANGLE = *LibreDWGc::OPTS_R11_ATTDEF_HAS_OBLIQUE_ANGLE;
*OPTS_R11_ATTDEF_HAS_STYLE = *LibreDWGc::OPTS_R11_ATTDEF_HAS_STYLE;
*OPTS_R11_ATTDEF_HAS_GENERATION = *LibreDWGc::OPTS_R11_ATTDEF_HAS_GENERATION;
*OPTS_R11_ATTDEF_HAS_HORIZ_ALIGNMENT = *LibreDWGc::OPTS_R11_ATTDEF_HAS_HORIZ_ALIGNMENT;
*OPTS_R11_ATTDEF_HAS_ALIGNMENT_POINT = *LibreDWGc::OPTS_R11_ATTDEF_HAS_ALIGNMENT_POINT;
*OPTS_R11_ATTDEF_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_ATTDEF_HAS_EXTRUSION;
*OPTS_R11_ATTDEF_HAS_VERT_ALIGNMENT = *LibreDWGc::OPTS_R11_ATTDEF_HAS_VERT_ALIGNMENT;
*OPTS_R11_ATTRIB_UNKNOWN_1 = *LibreDWGc::OPTS_R11_ATTRIB_UNKNOWN_1;
*OPTS_R11_ATTRIB_HAS_ROTATION = *LibreDWGc::OPTS_R11_ATTRIB_HAS_ROTATION;
*OPTS_R11_ATTRIB_HAS_WIDTH_FACTOR = *LibreDWGc::OPTS_R11_ATTRIB_HAS_WIDTH_FACTOR;
*OPTS_R11_ATTRIB_HAS_OBLIQUE_ANGLE = *LibreDWGc::OPTS_R11_ATTRIB_HAS_OBLIQUE_ANGLE;
*OPTS_R11_ATTRIB_HAS_STYLE = *LibreDWGc::OPTS_R11_ATTRIB_HAS_STYLE;
*OPTS_R11_ATTRIB_HAS_GENERATION = *LibreDWGc::OPTS_R11_ATTRIB_HAS_GENERATION;
*OPTS_R11_ATTRIB_HAS_HORIZ_ALIGNMENT = *LibreDWGc::OPTS_R11_ATTRIB_HAS_HORIZ_ALIGNMENT;
*OPTS_R11_ATTRIB_HAS_ALIGNMENT_POINT = *LibreDWGc::OPTS_R11_ATTRIB_HAS_ALIGNMENT_POINT;
*OPTS_R11_ATTRIB_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_ATTRIB_HAS_EXTRUSION;
*OPTS_R11_ATTRIB_HAS_VERT_ALIGNMENT = *LibreDWGc::OPTS_R11_ATTRIB_HAS_VERT_ALIGNMENT;
*OPTS_R11_BLOCK_UNKNOWN_1 = *LibreDWGc::OPTS_R11_BLOCK_UNKNOWN_1;
*OPTS_R11_BLOCK_HAS_XREF_PNAME = *LibreDWGc::OPTS_R11_BLOCK_HAS_XREF_PNAME;
*OPTS_R11_BLOCK_HAS_BLOCK_NAME = *LibreDWGc::OPTS_R11_BLOCK_HAS_BLOCK_NAME;
*OPTS_R11_CIRCLE_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_CIRCLE_HAS_EXTRUSION;
*OPTS_R11_DIMENSION_HAS_DXF12 = *LibreDWGc::OPTS_R11_DIMENSION_HAS_DXF12;
*OPTS_R11_DIMENSION_HAS_FLAG = *LibreDWGc::OPTS_R11_DIMENSION_HAS_FLAG;
*OPTS_R11_DIMENSION_HAS_TEXT = *LibreDWGc::OPTS_R11_DIMENSION_HAS_TEXT;
*OPTS_R11_DIMENSION_HAS_DXF13 = *LibreDWGc::OPTS_R11_DIMENSION_HAS_DXF13;
*OPTS_R11_DIMENSION_HAS_DXF14 = *LibreDWGc::OPTS_R11_DIMENSION_HAS_DXF14;
*OPTS_R11_DIMENSION_HAS_DXF15 = *LibreDWGc::OPTS_R11_DIMENSION_HAS_DXF15;
*OPTS_R11_DIMENSION_HAS_ANGLES = *LibreDWGc::OPTS_R11_DIMENSION_HAS_ANGLES;
*OPTS_R11_DIMENSION_HAS_DXF40 = *LibreDWGc::OPTS_R11_DIMENSION_HAS_DXF40;
*OPTS_R11_DIMENSION_HAS_ROTATION = *LibreDWGc::OPTS_R11_DIMENSION_HAS_ROTATION;
*OPTS_R11_DIMENSION_UNKNOWN_512 = *LibreDWGc::OPTS_R11_DIMENSION_UNKNOWN_512;
*OPTS_R11_DIMENSION_HAS_DXF53 = *LibreDWGc::OPTS_R11_DIMENSION_HAS_DXF53;
*OPTS_R11_DIMENSION_UNKNOWN_2048 = *LibreDWGc::OPTS_R11_DIMENSION_UNKNOWN_2048;
*OPTS_R11_DIMENSION_UNKNOWN_4096 = *LibreDWGc::OPTS_R11_DIMENSION_UNKNOWN_4096;
*OPTS_R11_DIMENSION_UNKNOWN_8192 = *LibreDWGc::OPTS_R11_DIMENSION_UNKNOWN_8192;
*OPTS_R11_DIMENSION_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_DIMENSION_HAS_EXTRUSION;
*OPTS_R11_DIMENSION_HAS_DIMSTYLE = *LibreDWGc::OPTS_R11_DIMENSION_HAS_DIMSTYLE;
*OPTS_R11_INSERT_HAS_SCALE_X = *LibreDWGc::OPTS_R11_INSERT_HAS_SCALE_X;
*OPTS_R11_INSERT_HAS_SCALE_Y = *LibreDWGc::OPTS_R11_INSERT_HAS_SCALE_Y;
*OPTS_R11_INSERT_HAS_ROTATION = *LibreDWGc::OPTS_R11_INSERT_HAS_ROTATION;
*OPTS_R11_INSERT_HAS_SCALE_Z = *LibreDWGc::OPTS_R11_INSERT_HAS_SCALE_Z;
*OPTS_R11_INSERT_HAS_NUM_COLS = *LibreDWGc::OPTS_R11_INSERT_HAS_NUM_COLS;
*OPTS_R11_INSERT_HAS_NUM_ROWS = *LibreDWGc::OPTS_R11_INSERT_HAS_NUM_ROWS;
*OPTS_R11_INSERT_HAS_COL_SPACING = *LibreDWGc::OPTS_R11_INSERT_HAS_COL_SPACING;
*OPTS_R11_INSERT_HAS_ROW_SPACING = *LibreDWGc::OPTS_R11_INSERT_HAS_ROW_SPACING;
*OPTS_R11_INSERT_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_INSERT_HAS_EXTRUSION;
*OPTS_R11_LINE_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_LINE_HAS_EXTRUSION;
*OPTS_R11_POINT_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_POINT_HAS_EXTRUSION;
*OPTS_R11_POINT_HAS_X_ANG = *LibreDWGc::OPTS_R11_POINT_HAS_X_ANG;
*OPTS_R11_POLYLINE_HAS_FLAG = *LibreDWGc::OPTS_R11_POLYLINE_HAS_FLAG;
*OPTS_R11_POLYLINE_HAS_START_WIDTH = *LibreDWGc::OPTS_R11_POLYLINE_HAS_START_WIDTH;
*OPTS_R11_POLYLINE_HAS_END_WIDTH = *LibreDWGc::OPTS_R11_POLYLINE_HAS_END_WIDTH;
*OPTS_R11_POLYLINE_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_POLYLINE_HAS_EXTRUSION;
*OPTS_R11_POLYLINE_HAS_M_VERTS = *LibreDWGc::OPTS_R11_POLYLINE_HAS_M_VERTS;
*OPTS_R11_POLYLINE_HAS_N_VERTS = *LibreDWGc::OPTS_R11_POLYLINE_HAS_N_VERTS;
*OPTS_R11_POLYLINE_HAS_M_DENSITY = *LibreDWGc::OPTS_R11_POLYLINE_HAS_M_DENSITY;
*OPTS_R11_POLYLINE_HAS_N_DENSITY = *LibreDWGc::OPTS_R11_POLYLINE_HAS_N_DENSITY;
*OPTS_R11_POLYLINE_HAS_CURVETYPE = *LibreDWGc::OPTS_R11_POLYLINE_HAS_CURVETYPE;
*OPTS_R11_POLYLINE_IN_EXTRA = *LibreDWGc::OPTS_R11_POLYLINE_IN_EXTRA;
*OPTS_R11_SHAPE_HAS_ROTATION = *LibreDWGc::OPTS_R11_SHAPE_HAS_ROTATION;
*OPTS_R11_SHAPE_HAS_LOAD_NUM = *LibreDWGc::OPTS_R11_SHAPE_HAS_LOAD_NUM;
*OPTS_R11_SHAPE_HAS_WIDTH_FACTOR = *LibreDWGc::OPTS_R11_SHAPE_HAS_WIDTH_FACTOR;
*OPTS_R11_SHAPE_HAS_OBLIQUE_ANGLE = *LibreDWGc::OPTS_R11_SHAPE_HAS_OBLIQUE_ANGLE;
*OPTS_R11_SOLID_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_SOLID_HAS_EXTRUSION;
*OPTS_R11_TEXT_HAS_ROTATION = *LibreDWGc::OPTS_R11_TEXT_HAS_ROTATION;
*OPTS_R11_TEXT_HAS_WIDTH_FACTOR = *LibreDWGc::OPTS_R11_TEXT_HAS_WIDTH_FACTOR;
*OPTS_R11_TEXT_HAS_OBLIQUE_ANGLE = *LibreDWGc::OPTS_R11_TEXT_HAS_OBLIQUE_ANGLE;
*OPTS_R11_TEXT_HAS_STYLE = *LibreDWGc::OPTS_R11_TEXT_HAS_STYLE;
*OPTS_R11_TEXT_HAS_GENERATION = *LibreDWGc::OPTS_R11_TEXT_HAS_GENERATION;
*OPTS_R11_TEXT_HAS_HORIZ_ALIGNMENT = *LibreDWGc::OPTS_R11_TEXT_HAS_HORIZ_ALIGNMENT;
*OPTS_R11_TEXT_HAS_ALIGNMENT_POINT = *LibreDWGc::OPTS_R11_TEXT_HAS_ALIGNMENT_POINT;
*OPTS_R11_TEXT_HAS_EXTRUSION = *LibreDWGc::OPTS_R11_TEXT_HAS_EXTRUSION;
*OPTS_R11_TEXT_ALIGNED_VERT_TO = *LibreDWGc::OPTS_R11_TEXT_ALIGNED_VERT_TO;
*OPTS_R11_VERTEX_HAS_START_WIDTH = *LibreDWGc::OPTS_R11_VERTEX_HAS_START_WIDTH;
*OPTS_R11_VERTEX_HAS_END_WIDTH = *LibreDWGc::OPTS_R11_VERTEX_HAS_END_WIDTH;
*OPTS_R11_VERTEX_HAS_BULGE = *LibreDWGc::OPTS_R11_VERTEX_HAS_BULGE;
*OPTS_R11_VERTEX_HAS_FLAG = *LibreDWGc::OPTS_R11_VERTEX_HAS_FLAG;
*OPTS_R11_VERTEX_HAS_TANGENT_DIR = *LibreDWGc::OPTS_R11_VERTEX_HAS_TANGENT_DIR;
*OPTS_R11_VERTEX_HAS_INDEX1 = *LibreDWGc::OPTS_R11_VERTEX_HAS_INDEX1;
*OPTS_R11_VERTEX_HAS_INDEX2 = *LibreDWGc::OPTS_R11_VERTEX_HAS_INDEX2;
*OPTS_R11_VERTEX_HAS_INDEX3 = *LibreDWGc::OPTS_R11_VERTEX_HAS_INDEX3;
*OPTS_R11_VERTEX_HAS_INDEX4 = *LibreDWGc::OPTS_R11_VERTEX_HAS_INDEX4;
*OPTS_R11_VERTEX_UNKNOWN_512 = *LibreDWGc::OPTS_R11_VERTEX_UNKNOWN_512;
*OPTS_R11_VERTEX_UNKNOWN_1024 = *LibreDWGc::OPTS_R11_VERTEX_UNKNOWN_1024;
*OPTS_R11_VERTEX_UNKNOWN_2048 = *LibreDWGc::OPTS_R11_VERTEX_UNKNOWN_2048;
*OPTS_R11_VERTEX_UNKNOWN_4096 = *LibreDWGc::OPTS_R11_VERTEX_UNKNOWN_4096;
*OPTS_R11_VERTEX_UNKNOWN_8192 = *LibreDWGc::OPTS_R11_VERTEX_UNKNOWN_8192;
*OPTS_R11_VERTEX_HAS_NOT_X_Y = *LibreDWGc::OPTS_R11_VERTEX_HAS_NOT_X_Y;
*FLAG_TABLE_IS_XREF_REF = *LibreDWGc::FLAG_TABLE_IS_XREF_REF;
*FLAG_TABLE_IS_XREF_RESOLVED = *LibreDWGc::FLAG_TABLE_IS_XREF_RESOLVED;
*FLAG_TABLE_IS_XREF_DEP = *LibreDWGc::FLAG_TABLE_IS_XREF_DEP;
*FLAG_TABLE_IS_REMOVED = *LibreDWGc::FLAG_TABLE_IS_REMOVED;
*FLAG_BLOCK_ANONYMOUS = *LibreDWGc::FLAG_BLOCK_ANONYMOUS;
*FLAG_BLOCK_HAS_ATTRIBS = *LibreDWGc::FLAG_BLOCK_HAS_ATTRIBS;
*FLAG_BLOCK_IS_EXT_REF = *LibreDWGc::FLAG_BLOCK_IS_EXT_REF;
*FLAG_BLOCK_IS_XREF_OVERLAY = *LibreDWGc::FLAG_BLOCK_IS_XREF_OVERLAY;
*FLAG_ATTDEF_INVISIBLE = *LibreDWGc::FLAG_ATTDEF_INVISIBLE;
*FLAG_ATTDEF_CONSTANT = *LibreDWGc::FLAG_ATTDEF_CONSTANT;
*FLAG_ATTDEF_VERIFY = *LibreDWGc::FLAG_ATTDEF_VERIFY;
*FLAG_ATTDEF_PRESET = *LibreDWGc::FLAG_ATTDEF_PRESET;
*FLAG_ATTRIB_INVISIBLE = *LibreDWGc::FLAG_ATTRIB_INVISIBLE;
*FLAG_ATTRIB_CONSTANT = *LibreDWGc::FLAG_ATTRIB_CONSTANT;
*FLAG_ATTRIB_VERIFY = *LibreDWGc::FLAG_ATTRIB_VERIFY;
*FLAG_ATTRIB_PRESET = *LibreDWGc::FLAG_ATTRIB_PRESET;
*FLAG_POLYLINE_CLOSED = *LibreDWGc::FLAG_POLYLINE_CLOSED;
*FLAG_POLYLINE_CURVE_FIT = *LibreDWGc::FLAG_POLYLINE_CURVE_FIT;
*FLAG_POLYLINE_SPLINE_FIT = *LibreDWGc::FLAG_POLYLINE_SPLINE_FIT;
*FLAG_POLYLINE_3D = *LibreDWGc::FLAG_POLYLINE_3D;
*FLAG_POLYLINE_MESH = *LibreDWGc::FLAG_POLYLINE_MESH;
*FLAG_POLYLINE_MESH_CLOSED = *LibreDWGc::FLAG_POLYLINE_MESH_CLOSED;
*FLAG_POLYLINE_PFACE_MESH = *LibreDWGc::FLAG_POLYLINE_PFACE_MESH;
*FLAG_POLYLINE_LT_PATTERN_CONTINUES = *LibreDWGc::FLAG_POLYLINE_LT_PATTERN_CONTINUES;
*LTYPE_SHAPE_FLAG_ABS_ROTATION = *LibreDWGc::LTYPE_SHAPE_FLAG_ABS_ROTATION;
*LTYPE_SHAPE_FLAG_IS_TEXT = *LibreDWGc::LTYPE_SHAPE_FLAG_IS_TEXT;
*LTYPE_SHAPE_FLAG_IS_SHAPE = *LibreDWGc::LTYPE_SHAPE_FLAG_IS_SHAPE;
*FLAG_LAYER_FROZEN = *LibreDWGc::FLAG_LAYER_FROZEN;
*FLAG_LAYER_FROZEN_IN_NEW = *LibreDWGc::FLAG_LAYER_FROZEN_IN_NEW;
*FLAG_LAYER_LOCKED = *LibreDWGc::FLAG_LAYER_LOCKED;
*FLAG_LAYER_PLOTFLAG = *LibreDWGc::FLAG_LAYER_PLOTFLAG;
*FLAG_STYLE_SHAPE = *LibreDWGc::FLAG_STYLE_SHAPE;
*FLAG_STYLE_VERTICAL_TEXT = *LibreDWGc::FLAG_STYLE_VERTICAL_TEXT;
*FLAG_VIEW_PSPACE = *LibreDWGc::FLAG_VIEW_PSPACE;
*FLAG_VX_IS_ON = *LibreDWGc::FLAG_VX_IS_ON;
*FLAG_LWPOLYLINE_HAS_EXTRUSION = *LibreDWGc::FLAG_LWPOLYLINE_HAS_EXTRUSION;
*FLAG_LWPOLYLINE_HAS_THICKNESS = *LibreDWGc::FLAG_LWPOLYLINE_HAS_THICKNESS;
*FLAG_LWPOLYLINE_HAS_CONSTWIDTH = *LibreDWGc::FLAG_LWPOLYLINE_HAS_CONSTWIDTH;
*FLAG_LWPOLYLINE_HAS_ELEVATION = *LibreDWGc::FLAG_LWPOLYLINE_HAS_ELEVATION;
*FLAG_LWPOLYLINE_HAS_NUM_BULGES = *LibreDWGc::FLAG_LWPOLYLINE_HAS_NUM_BULGES;
*FLAG_LWPOLYLINE_HAS_NUM_WIDTHS = *LibreDWGc::FLAG_LWPOLYLINE_HAS_NUM_WIDTHS;
*FLAG_LWPOLYLINE_UNKNOWN_64 = *LibreDWGc::FLAG_LWPOLYLINE_UNKNOWN_64;
*FLAG_LWPOLYLINE_UNKNOWN_128 = *LibreDWGc::FLAG_LWPOLYLINE_UNKNOWN_128;
*FLAG_LWPOLYLINE_PLINEGEN = *LibreDWGc::FLAG_LWPOLYLINE_PLINEGEN;
*FLAG_LWPOLYLINE_CLOSED = *LibreDWGc::FLAG_LWPOLYLINE_CLOSED;
*FLAG_LWPOLYLINE_VERTEXIDCOUNT = *LibreDWGc::FLAG_LWPOLYLINE_VERTEXIDCOUNT;
*FLAG_VERTEX_EXTRA_VERTEX = *LibreDWGc::FLAG_VERTEX_EXTRA_VERTEX;
*FLAG_VERTEX_CURVE_FIT = *LibreDWGc::FLAG_VERTEX_CURVE_FIT;
*FLAG_VERTEX_UNUSED_4 = *LibreDWGc::FLAG_VERTEX_UNUSED_4;
*FLAG_VERTEX_SPLINE_FIT = *LibreDWGc::FLAG_VERTEX_SPLINE_FIT;
*FLAG_VERTEX_SPLINE_FRAME_CONTROL_POINT = *LibreDWGc::FLAG_VERTEX_SPLINE_FRAME_CONTROL_POINT;
*FLAG_VERTEX_3D = *LibreDWGc::FLAG_VERTEX_3D;
*FLAG_VERTEX_MESH = *LibreDWGc::FLAG_VERTEX_MESH;
*FLAG_VERTEX_PFACE_MESH = *LibreDWGc::FLAG_VERTEX_PFACE_MESH;
*TEXT_GENERATION_BACKWARDS = *LibreDWGc::TEXT_GENERATION_BACKWARDS;
*TEXT_GENERATION_UPSIDE_DOWN = *LibreDWGc::TEXT_GENERATION_UPSIDE_DOWN;
*HORIZ_ALIGNMENT_LEFT = *LibreDWGc::HORIZ_ALIGNMENT_LEFT;
*HORIZ_ALIGNMENT_CENTER = *LibreDWGc::HORIZ_ALIGNMENT_CENTER;
*HORIZ_ALIGNMENT_RIGHT = *LibreDWGc::HORIZ_ALIGNMENT_RIGHT;
*HORIZ_ALIGNMENT_ALIGNED = *LibreDWGc::HORIZ_ALIGNMENT_ALIGNED;
*HORIZ_ALIGNMENT_MIDDLE = *LibreDWGc::HORIZ_ALIGNMENT_MIDDLE;
*HORIZ_ALIGNMENT_FIT = *LibreDWGc::HORIZ_ALIGNMENT_FIT;
*VERT_ALIGNMENT_BASELINE = *LibreDWGc::VERT_ALIGNMENT_BASELINE;
*VERT_ALIGNMENT_BOTTOM = *LibreDWGc::VERT_ALIGNMENT_BOTTOM;
*VERT_ALIGNMENT_MIDDLE = *LibreDWGc::VERT_ALIGNMENT_MIDDLE;
*VERT_ALIGNMENT_TOP = *LibreDWGc::VERT_ALIGNMENT_TOP;
*MLINE_JUSTIFICATION_TOP = *LibreDWGc::MLINE_JUSTIFICATION_TOP;
*MLINE_JUSTIFICATION_MIDDLE = *LibreDWGc::MLINE_JUSTIFICATION_MIDDLE;
*MLINE_JUSTIFICATION_BOTTOM = *LibreDWGc::MLINE_JUSTIFICATION_BOTTOM;
*POLYLINE_CURVETYPE_DEFAULT = *LibreDWGc::POLYLINE_CURVETYPE_DEFAULT;
*POLYLINE_CURVETYPE_QUADR_BSPLINE = *LibreDWGc::POLYLINE_CURVETYPE_QUADR_BSPLINE;
*POLYLINE_CURVETYPE_CUBIC_BSPLINE = *LibreDWGc::POLYLINE_CURVETYPE_CUBIC_BSPLINE;
*POLYLINE_CURVETYPE_BEZIER_SURFACE = *LibreDWGc::POLYLINE_CURVETYPE_BEZIER_SURFACE;
*LEADER_PATHTYPE_STRAIGHT = *LibreDWGc::LEADER_PATHTYPE_STRAIGHT;
*LEADER_PATHTYPE_SPLINE = *LibreDWGc::LEADER_PATHTYPE_SPLINE;
*LEADER_ANNOTTYPE_MTEXT = *LibreDWGc::LEADER_ANNOTTYPE_MTEXT;
*LEADER_ANNOTTYPE_TOLERANCE = *LibreDWGc::LEADER_ANNOTTYPE_TOLERANCE;
*LEADER_ANNOTTYPE_INSERT = *LibreDWGc::LEADER_ANNOTTYPE_INSERT;
*LEADER_ANNOTTYPE_NO_ANNOT = *LibreDWGc::LEADER_ANNOTTYPE_NO_ANNOT;
*MLINE_FLAGS_HAS_VERTEX = *LibreDWGc::MLINE_FLAGS_HAS_VERTEX;
*MLINE_FLAGS_CLOSED = *LibreDWGc::MLINE_FLAGS_CLOSED;
*MLINE_FLAGS_SUPPRESS_START_CAPS = *LibreDWGc::MLINE_FLAGS_SUPPRESS_START_CAPS;
*MLINE_FLAGS_SUPPRESS_END_CAPS = *LibreDWGc::MLINE_FLAGS_SUPPRESS_END_CAPS;
*MLINESTYLE_FLAG_FILL = *LibreDWGc::MLINESTYLE_FLAG_FILL;
*MLINESTYLE_FLAG_MITERS = *LibreDWGc::MLINESTYLE_FLAG_MITERS;
*MLINESTYLE_FLAG_UNKNOWN_4 = *LibreDWGc::MLINESTYLE_FLAG_UNKNOWN_4;
*MLINESTYLE_FLAG_UNKNOWN_8 = *LibreDWGc::MLINESTYLE_FLAG_UNKNOWN_8;
*MLINESTYLE_FLAG_START_SQUARE_END = *LibreDWGc::MLINESTYLE_FLAG_START_SQUARE_END;
*MLINESTYLE_FLAG_START_INNER_ARC = *LibreDWGc::MLINESTYLE_FLAG_START_INNER_ARC;
*MLINESTYLE_FLAG_START_ROUND = *LibreDWGc::MLINESTYLE_FLAG_START_ROUND;
*MLINESTYLE_FLAG_UNKNOWN_128 = *LibreDWGc::MLINESTYLE_FLAG_UNKNOWN_128;
*MLINESTYLE_FLAG_END_SQUARE = *LibreDWGc::MLINESTYLE_FLAG_END_SQUARE;
*MLINESTYLE_FLAG_END_INNER_ARC = *LibreDWGc::MLINESTYLE_FLAG_END_INNER_ARC;
*MLINESTYLE_FLAG_END_ROUND = *LibreDWGc::MLINESTYLE_FLAG_END_ROUND;
*LIGHT_TYPE_DISTANT = *LibreDWGc::LIGHT_TYPE_DISTANT;
*LIGHT_TYPE_POINT = *LibreDWGc::LIGHT_TYPE_POINT;
*LIGHT_TYPE_SPOT = *LibreDWGc::LIGHT_TYPE_SPOT;
*LIGHT_ATTENUATION_TYPE_NONE = *LibreDWGc::LIGHT_ATTENUATION_TYPE_NONE;
*LIGHT_ATTENUATION_TYPE_INV_LINEAR = *LibreDWGc::LIGHT_ATTENUATION_TYPE_INV_LINEAR;
*LIGHT_ATTENUATION_TYPE_INV_SQUARE = *LibreDWGc::LIGHT_ATTENUATION_TYPE_INV_SQUARE;
*LIGHT_EXTLIGHT_SHAPE_LINEAR = *LibreDWGc::LIGHT_EXTLIGHT_SHAPE_LINEAR;
*LIGHT_EXTLIGHT_SHAPE_RECT = *LibreDWGc::LIGHT_EXTLIGHT_SHAPE_RECT;
*LIGHT_EXTLIGHT_SHAPE_DISK = *LibreDWGc::LIGHT_EXTLIGHT_SHAPE_DISK;
*LIGHT_EXTLIGHT_SHAPE_CYLINDER = *LibreDWGc::LIGHT_EXTLIGHT_SHAPE_CYLINDER;
*LIGHT_EXTLIGHT_SHAPE_SPHERE = *LibreDWGc::LIGHT_EXTLIGHT_SHAPE_SPHERE;
*_3DFACE_INVISIBLE_EDGE1 = *LibreDWGc::_3DFACE_INVISIBLE_EDGE1;
*_3DFACE_INVISIBLE_EDGE2 = *LibreDWGc::_3DFACE_INVISIBLE_EDGE2;
*_3DFACE_INVISIBLE_EDGE3 = *LibreDWGc::_3DFACE_INVISIBLE_EDGE3;
*_3DFACE_INVISIBLE_EDGE4 = *LibreDWGc::_3DFACE_INVISIBLE_EDGE4;
*SECTION_UNKNOWN = *LibreDWGc::SECTION_UNKNOWN;
*SECTION_HEADER = *LibreDWGc::SECTION_HEADER;
*SECTION_AUXHEADER = *LibreDWGc::SECTION_AUXHEADER;
*SECTION_CLASSES = *LibreDWGc::SECTION_CLASSES;
*SECTION_HANDLES = *LibreDWGc::SECTION_HANDLES;
*SECTION_TEMPLATE = *LibreDWGc::SECTION_TEMPLATE;
*SECTION_OBJFREESPACE = *LibreDWGc::SECTION_OBJFREESPACE;
*SECTION_OBJECTS = *LibreDWGc::SECTION_OBJECTS;
*SECTION_REVHISTORY = *LibreDWGc::SECTION_REVHISTORY;
*SECTION_SUMMARYINFO = *LibreDWGc::SECTION_SUMMARYINFO;
*SECTION_PREVIEW = *LibreDWGc::SECTION_PREVIEW;
*SECTION_APPINFO = *LibreDWGc::SECTION_APPINFO;
*SECTION_APPINFOHISTORY = *LibreDWGc::SECTION_APPINFOHISTORY;
*SECTION_FILEDEPLIST = *LibreDWGc::SECTION_FILEDEPLIST;
*SECTION_SECURITY = *LibreDWGc::SECTION_SECURITY;
*SECTION_VBAPROJECT = *LibreDWGc::SECTION_VBAPROJECT;
*SECTION_SIGNATURE = *LibreDWGc::SECTION_SIGNATURE;
*SECTION_ACDS = *LibreDWGc::SECTION_ACDS;
*SECTION_INFO = *LibreDWGc::SECTION_INFO;
*SECTION_SYSTEM_MAP = *LibreDWGc::SECTION_SYSTEM_MAP;
*SECTION_HEADER_R13 = *LibreDWGc::SECTION_HEADER_R13;
*SECTION_CLASSES_R13 = *LibreDWGc::SECTION_CLASSES_R13;
*SECTION_HANDLES_R13 = *LibreDWGc::SECTION_HANDLES_R13;
*SECTION_OBJFREESPACE_R13 = *LibreDWGc::SECTION_OBJFREESPACE_R13;
*SECTION_TEMPLATE_R13 = *LibreDWGc::SECTION_TEMPLATE_R13;
*SECTION_AUXHEADER_R2000 = *LibreDWGc::SECTION_AUXHEADER_R2000;
*SECTION_THUMBNAIL_R13 = *LibreDWGc::SECTION_THUMBNAIL_R13;
*SECTION_HEADER_R11 = *LibreDWGc::SECTION_HEADER_R11;
*SECTION_BLOCK = *LibreDWGc::SECTION_BLOCK;
*SECTION_LAYER = *LibreDWGc::SECTION_LAYER;
*SECTION_STYLE = *LibreDWGc::SECTION_STYLE;
*SECTION_LTYPE = *LibreDWGc::SECTION_LTYPE;
*SECTION_VIEW = *LibreDWGc::SECTION_VIEW;
*SECTION_UCS = *LibreDWGc::SECTION_UCS;
*SECTION_VPORT = *LibreDWGc::SECTION_VPORT;
*SECTION_APPID = *LibreDWGc::SECTION_APPID;
*SECTION_DIMSTYLE = *LibreDWGc::SECTION_DIMSTYLE;
*SECTION_VX = *LibreDWGc::SECTION_VX;
*DWG_OPTS_LOGLEVEL = *LibreDWGc::DWG_OPTS_LOGLEVEL;
*DWG_OPTS_MINIMAL = *LibreDWGc::DWG_OPTS_MINIMAL;
*DWG_OPTS_DXFB = *LibreDWGc::DWG_OPTS_DXFB;
*DWG_OPTS_JSONFIRST = *LibreDWGc::DWG_OPTS_JSONFIRST;
*DWG_OPTS_INDXF = *LibreDWGc::DWG_OPTS_INDXF;
*DWG_OPTS_INJSON = *LibreDWGc::DWG_OPTS_INJSON;
*DWG_OPTS_IN = *LibreDWGc::DWG_OPTS_IN;
*DWG_VT_INVALID = *LibreDWGc::DWG_VT_INVALID;
*DWG_VT_STRING = *LibreDWGc::DWG_VT_STRING;
*DWG_VT_POINT3D = *LibreDWGc::DWG_VT_POINT3D;
*DWG_VT_REAL = *LibreDWGc::DWG_VT_REAL;
*DWG_VT_INT16 = *LibreDWGc::DWG_VT_INT16;
*DWG_VT_INT32 = *LibreDWGc::DWG_VT_INT32;
*DWG_VT_INT8 = *LibreDWGc::DWG_VT_INT8;
*DWG_VT_BINARY = *LibreDWGc::DWG_VT_BINARY;
*DWG_VT_HANDLE = *LibreDWGc::DWG_VT_HANDLE;
*DWG_VT_OBJECTID = *LibreDWGc::DWG_VT_OBJECTID;
*DWG_VT_BOOL = *LibreDWGc::DWG_VT_BOOL;
*DWG_VT_INT64 = *LibreDWGc::DWG_VT_INT64;
1;
